
//********************************************
// GM_Angel.h: interface for the Angle class.
//********************************************
#ifndef GM_ANGLE_H_
#define GM_ANGLE_H_

#include <math.h>
#include <vector>
using namespace std;

namespace GM_lib {

#ifndef POS_TOLERANCE	
#define POS_TOLERANCE		1e-6
#endif
#ifndef ANGLE_TOLERANCE	
#define ANGLE_TOLERANCE		1e-3
#endif

#ifndef M_E	
#define M_E         2.71828182845904523536
#endif
#ifndef M_LOG2E
#define M_LOG2E     1.44269504088896340736
#endif
#ifndef M_LOG10E
#define M_LOG10E    0.434294481903251827651
#endif
#ifndef M_LN2
#define M_LN2       0.693147180559945309417
#endif
#ifndef M_LN10
#define M_LN10      2.30258509299404568402
#endif
#ifndef M_PI
#define M_PI        3.14159265358979323846
#endif
#ifndef M_2PI
#define M_2PI       6.28318530717958647692
#endif
#ifndef M_2PI_3
#define M_2PI_3     2.094395102393195492308
#endif
#ifndef M_PI_2
#define M_PI_2      1.57079632679489661923
#endif
#ifndef M_PI_4
#define M_PI_4      0.785398163397448309616
#endif
#ifndef M_PI_180
#define M_PI_180	0.0174532925199432957692
#endif
#ifndef M_1_PI
#define M_1_PI      0.318309886183790671538
#endif
#ifndef M_2_PI
#define M_2_PI      0.636619772367581343076
#endif
#ifndef M_180_PI
#define M_180_PI	57.29577951308232087679
#endif
#ifndef M_1_SQRTPI
#define M_1_SQRTPI  0.564189583547756286948
#endif
#ifndef M_2_SQRTPI
#define M_2_SQRTPI  1.12837916709551257390
#endif
#ifndef M_SQRT2
#define M_SQRT2     1.41421356237309504880
#endif
#ifndef M_SQRT_2
#define M_SQRT_2    0.707106781186547524401
#endif
#ifndef NULL
#define NULL    0
#endif

//template <class T>
//void swap(T& a, T& b) {T t=a; a=b; b=t;}


//*********************************************************
//****************  The Angle class  **********************
//*********************************************************

class Angle {

	double  	a;

public:
	Angle(double d = 0.0) { a=d;}		// Default constructor rad.

	Angle(int i) { a=i*M_PI_180;}		// Default constructor deg.

	Angle(const Angle& ang) { a=ang.a;}	// Copy  constructor.


	Angle& operator=(double d)		{a=d; return (*this);};         
	Angle& operator=(int i)			{a=i*M_PI_180; return (*this);} 
	Angle& operator=(Angle ang)		{a=ang.a; return (*this);}
   
	double rad() const { return a;}
	int    deg() const { return int(a*M_180_PI);}
   
	double translate(double d=0)		// Translate periodic default: 0-2Pi
	{
		double aa=a;
		while(aa<d) aa += M_2PI;
		d += M_2PI;
		while(aa>d) aa -= M_2PI;
		return aa;
	}


	operator double() { return rad();}
	operator int()    { return deg();}


			//***** Additions and substractions. ******

	Angle& operator+=(const Angle& pt)
		{  a += pt.a; return *this; }

	Angle& operator-=(const Angle& pt)
		{  a -= pt.a; return *this; }

	Angle operator-() const
		{ Angle p;  p.a = -a; return p;}

	Angle operator+(Angle pt) const
		{ pt += (*this); return pt;}

	Angle operator-(Angle pt) const
		{ pt -= (*this); return -pt;}

	Angle operator+(double d) const
		{ return (*this) + Angle(d);}

	Angle operator-(double d) const
		{ return (*this) - Angle(d);}

	Angle operator+(int i) const
		{ return (*this) + Angle(i);}

	Angle operator-(int i) const
		{ return (*this) - Angle(i);}


			//***** Scaling. *****

	Angle& operator*=(double d)
		{  a *= d; return *this; }

	Angle operator*(double d) const
		{  return Angle(a*d);}

	friend 
	Angle operator*(double d, Angle pt)
		{ pt*=d; return pt;}


  			//***** Invers scaling. *****

	Angle&    operator/=(double d)
		{ d = 1/d; (*this) *= d; return *this; }

	Angle operator/(double d) const
		{ d = 1/d; return (*this)*d;}


 			//***** Boolean on equality. *****

	bool  operator==(const Angle& p) const
                     { return fabs(a - p.a) < ANGLE_TOLERANCE; }

	bool  operator!=(const Angle& pt) const
                     { return !((*this) == pt); }



			//***** Boolean on sorting. *****

	bool operator<(const Angle& v) const { return a < v.a; }

	bool operator>(const Angle& v) const { return a > v.a; }

	bool operator<=(const Angle& v) const { return !((*this) > v);}

	bool operator>=(const Angle& v) const { return !((*this) < v);} 

public:
 	template <class T_Stream>
	friend T_Stream& operator<<(T_Stream& out, const Angle& v)
	{
		out << v.rad();
		return out;
	}

	template <class T_Stream>
	friend T_Stream& operator>>(T_Stream& in, Angle& v)
	{
		double rad;
		in >> rad; 
		v = rad;
		return in;
	}
};

} // end namespace GM_lib

#endif /* GM_ANGLE_H_ */
