#ifndef _GM_APPLE_H_
#define _GM_APPLE_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Apple : public PSurf<T,n>
{
	T _r; // radius of the apple
protected:
	void _eval(T u, T v, int d1, int d2);

	T _st_p_u()		{ return T(-0.5*M_PI); }
	T _end_p_u()	{ return T(0.5*M_PI); }
	T _st_p_v()		{ return T(-M_PI); }
	T _end_p_v()	{ return T(M_PI); }

public:

	Apple( T radius=T(1) );
	Apple( const Apple<T,n>& s );

	const char* identity() const { return " Apple "  ;}
};

} // end namespace GM_lib

using namespace GM_lib;


template <class T, int n>
inline
Apple<T,n>::Apple(T radius):PSurf<T,n>()
{
	_r = radius;
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),radius));
}

template <class T, int n>
inline
Apple<T,n>::Apple(const Apple<T,n>& s):PSurf<T,n>(s)
{
	_r = s._r;
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),radius));
}


template <class T, int n>
void Apple<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(d1+1,d2+1);

	T c_u = (T)cos(u);
	T c_v = (T)cos(v);
	T s_u = (T)sin(u);
	T s_v = (T)sin(v);

	__p[0][0][0] =	 2*_r*cos(u)*(1+cos(v))*sin(v);
	__p[0][0][1] =	 2*_r*sin(u)*(1+cos(v))*sin(v);
	__p[0][0][2] =	-2*_r*cos(v)*(1+cos(v));

	if(d1) //u		
	{
		__p[1][0][0] = -2*_r*sin(u)*(1+cos(v))*sin(v);
		__p[1][0][1] =  2*_r*cos(u)*(1+cos(v))*sin(v);
		__p[1][0][2] =  T(0);
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	-2*_r*cos(u)*(1+cos(v))*sin(v);
		__p[2][0][1] =	-2*_r*sin(u)*(1+cos(v))*sin(v);
		__p[2][0][2] =	 T(0);
	}
	if(d2) //v	
	{
		__p[0][1][0] =	 2*_r*cos(u)*(2*cos(v)*(cos(v)+1)-1);
		__p[0][1][1] =	 2*_r*sin(u)*(2*cos(v)*(cos(v)+1)-1);		
		__p[0][1][2] =	 2*_r*sin(v)*(1+2*cos(v));	
	}
	if(d2>1) //vv	
	{
		__p[0][2][0] =	-2*_r*cos(u)*sin(v)*(1+4*cos(v));
		__p[0][2][1] =	-2*_r*sin(u)*sin(v)*(1+4*cos(v));
		__p[0][2][2] =	 2*_r*(cos(v)+2*(cos(v)*cos(v)-sin(v)*sin(v)));
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =	-2*_r*sin(u)*(2*cos(v)*(cos(v)+1)-1);
		__p[1][1][1] =	 2*_r*cos(u)*(2*cos(v)*(cos(v)+1)-1);
		__p[1][1][2] =	 T(0);												  		
	}
	if(d1>1 && d2)//uuv	
	{
		__p[2][1][0] =	-2*_r*cos(u)*(2*cos(v)*(cos(v)+1)-1);
		__p[2][1][1] =	-2*_r*sin(u)*(2*cos(v)*(cos(v)+1)-1);
		__p[2][1][2] =	 T(0);
	}
	if(d1 && d2>1) //uvv		
	{
		__p[1][2][0] =	 2*_r*sin(u)*sin(v)*(4*cos(v)+2);			
		__p[1][2][1] =	-2*_r*cos(u)*sin(v)*(4*cos(v)+2);			
		__p[1][2][2] =	 T(0);			
	}
	if(d1>1 && d2>1) //uuvv
	{					
		__p[2][2][0] =	 2*_r*cos(u)*sin(v)*(4*cos(v)+2);
		__p[2][2][1] =	 2*_r*sin(u)*sin(v)*(4*cos(v)+2);			 
		__p[2][2][2] =	 T(0);			 
	}		

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}


 #endif //_GM_APPLE_H_
