#ifndef _GM_APPLE_PLAN_H_
#define _GM_APPLE_PLAN_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class GM_Apple_Plan : public PSurf<T,n>
{
	T	_r;	//radius

	void _eval(T u, T v, int d1, int d2);

protected:

	T _st_p_u()		{ return T(-0.5*M_PI); }
	T _end_p_u()	{ return T(0.5*M_PI); }
	T _st_p_v()		{ return T(-M_PI); }
	T _end_p_v()	{ return T(M_PI); }


public:

	GM_Apple_Plan(T r = 1.0);
	GM_Apple_Plan(const GM_Apple_Plan<T,n>& s);

	~GM_Apple_Plan() {}

	const char* identity() const { return " GM_Apple_Plan "  ;}
};
} // GM_lib


using namespace GM_lib;

template <class T, int n>
GM_Apple_Plan<T,n>::GM_Apple_Plan(T r):PSurf<T,n>()
{
	_r = r;
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),_r));
}

template <class T, int n>
GM_Apple_Plan<T,n>::GM_Apple_Plan(const GM_Apple_Plan<T,n>& s):PSurf<T,n>(s)
{
	_r = s._r;
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),_r));
}

template <class T, int n>
void GM_Apple_Plan<T,n>::_eval(T u, T v, int d1,int d2)
{
    __p.setDim(3,3);

	__p[0][0][0] =	T(2)*_r*cos(u)*(T(1)+cos(v))*sin(v)-u;
	__p[0][0][1] =	T(2)*_r*(T(1)+cos(v))*sin(u)*sin(v)-v;
	__p[0][0][2] = -T(2)*_r*cos(v)*(T(1)+cos(v));

	if(d1) //u		
	{
		__p[1][0][0] = -T(2)*_r*(T(1)+cos(v))*sin(u)*sin(v)-T(1);
		__p[1][0][1] =  T(2)*_r*cos(u)*(T(1)+cos(v))*sin(v);
		__p[1][0][2] =  T(0);
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	-2.0*_r*cos(u)*(1.0+cos(v))*sin(v);
		__p[2][0][1] =	-2.0*_r*(1.0+cos(v))*sin(u)*sin(v);
		__p[2][0][2] =	0.0;
	}
	if(d2) //v	
	{
		__p[0][1][0] = -T(2)*_r*cos(u)*sin(v)*sin(v)+T(2)*_r*cos(u)*(T(1)+cos(v))*cos(v);
		__p[0][1][1] = -T(2)*_r*sin(v)*sin(v)*sin(u)+T(2)*_r*(T(1)+cos(v))*sin(u)*cos(v)-T(1);		
		__p[0][1][2] =	T(2)*_r*sin(v)*(T(1)+cos(v))+T(2)*_r*cos(v)*sin(v);
	}
	if(d2>1) //vv	
	{
		__p[0][2][0] =	-2.0*_r*cos(u)*sin(v)*(4.0*cos(v)+1.0);
		__p[0][2][1] =	-2.0*_r*sin(u)*sin(v)*(4.0*cos(v)+1.0);
		__p[0][2][2] =	 2.0*_r*(cos(v)+4.0*pow(cos(v),T(2.0))-2.0);
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =	-2*_r*sin(u)*(2*cos(v)*(cos(v)+1)-1);
		__p[1][1][1] =	 2*_r*cos(u)*(2*cos(v)*(cos(v)+1)-1);
		__p[1][1][2] =	 T(0);												  		
	}
	if(d1>1 && d2)//uuv	
	{
		__p[2][1][0] =	-2*_r*cos(u)*(2*cos(v)*(cos(v)+1)-1);
		__p[2][1][1] =	-2*_r*sin(u)*(2*cos(v)*(cos(v)+1)-1);
		__p[2][1][2] =	 T(0);
	}
	if(d1 && d2>1) //uvv		
	{
		__p[1][2][0] =	 2*_r*sin(u)*sin(v)*(4*cos(v)+2);			
		__p[1][2][1] =	-2*_r*cos(u)*sin(v)*(4*cos(v)+2);			
		__p[1][2][2] =	 T(0);			
	}
	if(d1>1 && d2>1) //uuvv
	{					
		__p[2][2][0] =	-2.0*_r*sin(u)*(-1.0+2.0*pow(cos(v),T(2.0))+cos(v));
		__p[2][2][1] =	 2.0*_r*cos(u)*(-1.0+2.0*pow(cos(v),T(2.0))+cos(v));			 
		__p[2][2][2] =	 0.0;
	}		

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}

#endif //_GM_APPLE_PLAN_H_
