#include "GM_Array.h"

namespace GM_lib {

//********************************************************
//**************** SORTING FUNCTION **********************
//**************** CHANGING THE ARRAY ********************
//********************************************************

template<class T>
void	Array<T>::sort()
{
	sorted = true;
	int	i,j,k;
	for (i=0; i < no_elem - 1; i++)
	{
		for (k=i,j=i+1; j < no_elem; j++)
			if (array[j] < array[k])   k = j;
		if (i != k) swap(i, k);
	}
	return;
}



//********************************************************
//**************** SORTING FUNCTION **********************
//**************** RETURNING AN INDEX ARRAY **************
//********************************************************

template<class T>
Array<int>	Array<T>::sortArray() const
{
  int		i,j,k;
  if (no_elem == 0)			return Array<int>(0);

  Array<int>	arr;
  arr.setSize(no_elem);

  for (i=0; i < no_elem; i++)  arr[i] = i;

  for (i=0; i < no_elem - 1; i++)
  {
    for (k=i,j=i+1; j < no_elem; j++)
      if (array[arr[j]] < array[arr[k]])  k = j;
    arr.swap(i,k);
  }
  return arr;
}



//********************************************************
//**************** DESTRUCTOR ****************************
//********************************************************

template<class T>
Array<T>::~Array()
{
	if (max_elem > 6) delete [] array;
}


//********************************************************
//**************** OPERATOR= THE HOLE ARRAY NEW VALUES ***
//********************************************************

template<class T>
Array<T>& Array<T>::operator=(const	Array<T>&	other)
{
    if (max_elem < other.size() || (max_elem-other.size()) > 10)
    {
       no_elem = other.size();
       if (max_elem > 6)		delete [] array;
       max_elem = (no_elem > 6 ? no_elem : 6);
       if (max_elem > 6)		array = new T[max_elem];
       else				array = pt;
    }
    else
       no_elem = other.size();

	sorted	= other.sorted;
	numb	= other.numb;

	//memcpy(array,other.array,no_elem*sizeof(T));
    for (int i=0; i < no_elem; i++)
        array[i] = other.array[i];
    return *this;
}


//********************************************************
//**************** STRIPELEMENTS() - NO FREE ELEMENTS ****
//********************************************************

template<class T>
void	Array<T>::stripElements()
{
		// Remove surplus elements.
	for (int j,i=0; i<no_elem-1; i++)
		for (j=no_elem-1; j>i; j--)
			if(array[i] == array[j])	removeIndex(j);
}


//********************************************************
//**************** ALWAYSINSERT(OBJECT) - EVEN IF EXIST **
//**************** DEFAULT AT END OF ARRAY ***************
//********************************************************

template<class T>
void	Array<T>::alwaysInsert(const T& obj, bool first)
{
   int i;
		// Is the array full?
   if (no_elem == max_elem)
   {
		// Allocate new array.
//       max_elem += 10;
       max_elem *= 2;
       T*	tmp = new T[max_elem];

		// Copy old array.
//	   memcpy(tmp,array,no_elem*sizeof(T));
       for (int i=0; i<no_elem; i++)
		   tmp[i] = array[i];

		// Delete old array and use the new.
       if (array != pt)	delete [] array;
       array = tmp;
   }

   if(sorted)
   {
      for (i=no_elem; i> 0 && obj<array[i-1]; --i) array[i] = array[i-1];
       array[i] = obj;
   }
   else if (first)
   {
       for (i=no_elem; i> 0; --i) array[i] = array[i-1];
       array[0] = obj;
   }
   else
       array[no_elem] = obj;

   no_elem++;
}


//********************************************************
//**************** ALWAYSINSERT(ARRAY) - EVEN IF EXIST ***
//**************** DEFAULT AT END OF ARRAY ***************
//********************************************************

template<class T>
void	Array<T>::alwaysInsert(const Array<T>& arr, bool first)
{
	for (int i=0; i< arr.size(); i++) 
		alwaysInsert(arr.array[i],first);
}


//********************************************************
//****************   REMOVEINDEX(index) -  ***************
//****************  remove element with given index ******
//********************************************************

template<class T>
bool	Array<T>::removeIndex(int	index)
{
		// Check index.
	if (index < 0 || index >= no_elem)		return false;
		// Move array forward.
	no_elem--;
	if (sorted) for (int j=index; j<no_elem; j++) array[j] = array[j+1];
	else if(index != no_elem)	array[index] = array[no_elem];
	return true;
}


//********************************************************
//****************   SETSIZE(number) -  ******************
//*******  set size of active numbers not potential ******
//********************************************************

template<class T>
void	Array<T>::setSize(int s)
{
	int old_max = max_elem;
	int limit = no_elem < s ? no_elem : s;
	if (max_elem < s || (no_elem - s) > 10 || s == 0)
	{
		T* old_array = array;

		no_elem  = s;
		max_elem = (no_elem > 6 ? no_elem : 6);
		if (max_elem > 6)
		{
			array = new T[max_elem];
		}
		else
		{
			array = pt;
		}
		for (int i = 0; i < limit; i++)
			array[i] = old_array[i];
		if (old_max > 6)
			delete [] old_array;
	}
	else
	{
		no_elem = s;
	}
	for (int i = limit; i < no_elem; i++)
		array[i] = T();
}

//********************************************************
//****************   SETSIZE(number, type) -  ************
//*******  set size of active numbers not potential ******
//********************************************************

template<class T>
void	Array<T>::setSize(int s, T t)
{
	setSize(s);
	for(int i=0; i<s; i++) array[i] = t;

}

//********************************************************
//****************   SETMAX(number) -   ******************
//************  set size of potential numbers ************
//********************************************************

template<class T>
void	Array<T>::setMax(int s)
{
  if (s > max_elem && s > 6)
  {
    int old_max   = max_elem;
    T*  old_array = array;
    max_elem = s;
    array = new T[max_elem];

	//memcpy(array,old_array,no_elem*sizeof(T));
    for (int i = 0; i < no_elem; i++)
      array[i] = old_array[i];

    if (old_max > 6)
      delete [] old_array;
  }
}


//********************************************************
//******************   STRIP() -     *********************
//******************  only actual numbers  ***************
//********************************************************

template<class T>
void	Array<T>::strip()
{
	if (max_elem > no_elem && max_elem > 6)
	{
		T* old_array = array;
		if (no_elem > 6)
		{
			array = new T[no_elem];
			max_elem = no_elem;
		}
		else
		{
			array = pt;
			max_elem = 6;
		}
//		memcpy(array,old_array,no_elem*sizeof(T));
		for (int i=0; i = no_elem; i++)
		array[i] = old_array[i];

		if (old_array != pt) delete [] old_array;
	}
}


//********************************************************
//******************   INDEX(object) - *******************
//********************************************************

template<class T>
int	Array<T>::index(const T& obj) const
{
	for (int i=0; i<no_elem; i++)
		if (array[i] == obj)		return i;

	return -1;
}


//********************************************************
//******************   TURN() -     **********************
//******************  turn the array  ********************
//********************************************************

template<class T>
void	Array<T>::turn()
{
	for (int i=0, mid = no_elem/2; i<mid; i++)
		swap(i, no_elem - 1 - i);
}


//********************************************************
//******************   OPERATOR==   **********************
//******************  equality test for the hole array  **
//********************************************************

template<class T>
bool Array<T>::operator==(const Array<T>& a) const
{
	if (size() != a.size()) 		return false;
	for (int i=0; i < no_elem;  i++)
		if (array[i] != a.array[i]) return false;
	return true;
}

} // end namespace GM_lib

#include "GM_T_Array.h"

namespace GM_lib {

//********************************************************
//**************** OPERATOR= THE HOLE ARRAY NEW VALUES ***
//********************************************************

template<class T>
Array<T>& Array<T>::operator=(const	T_Array<T>&	a)
{
	clear();
	for(int i=0; i<a.size(); i++) alwaysInsert(a(i));
	return *this;
}


//********************************************************
//**************** COPY CONSTRUCTOR **********************
//********************************************************

template<class T>
Array<T>::Array(const T_Array<T>& a)
{ 
	no_elem  = 0; 
	max_elem = 6;
    array    = pt;
	*this    = a;
}

} // end namespace GM_lib



