
//********************************************************
// GM_Array.h: interface for the Array classes.
//********************************************************
#ifndef GM_ARRAY_H_
#define GM_ARRAY_H_

#include <memory.h>
#include <GM_Stream.h>

namespace GM_lib {


//*********************************************************
//************ The classes in the array system   **********
//*********************************************************
template <class T>
class Array;

template <class T>
class T_Array;

template <class T>
class LX_Array;


//*********************************************************
//****************  The Array class  **********************
//*********************************************************


template<class T> class Array {

   T  	pt[6];
   T* 	array;
   int	no_elem;
   int	max_elem;
   bool sorted;
   bool numb;			// true:  iostream writing and reading the number -- default!
						// false: iostream writing and reading only the
						//		  array, depending on no_elem.			
public:

   Array(int n = 6);						// Default constructor.
   Array(int n, T);							// Default constructor All n element = T
   Array(const Array<T>&);					// Copy  constructor.
   Array(const T_Array<T>&);				// Copy  constructor.
   Array(int n, const T* p);				// Standard constructor.

  ~Array();									// Destructor

   void     sort();							// Using operator < of the elements
											// and sorting this array
   Array<int>  sortArray() const;			// Using operator < of the elements
											// and returns a sorted index array

   void		setSorted(bool s=true)			// The sort status is true only if the
				{ sorted = true;}			// array is sorted here you can change it.

   T*       ptr() { return &array[0]; }

   void     clear();						// Remove all elements.

   T&       operator[](int);				// return given element.
   const T& operator()(int) const;			// return copy of given element.

   T&		last() {return (*this)[no_elem-1];} // return last element in array.

   Array<T>& operator=(const Array<T>&);	// Copy the other Array.
   Array<T>& operator=(const T_Array<T>&);	// Copy the other Array.

   void     stripElements();				// Make each element exist only once.

   bool     exist(const T& t) const			// True if present.
               { return bool(index(t) >= 0); }

   int      index(const T&) const;			// Index in array (-1 if not present).

   void     initialize(const T& t);			// Initiate array (all element equal t).

   void     alwaysInsert(const T&, bool first=false);
											// Inserting first or last in array.

   void     operator+=(const T& t)			// Insert last.
               { alwaysInsert(t); }

   bool     insert(const T&, bool first=false);// Insert only if not existing,
											   // True if inserted (False existing)
											
   void     alwaysInsert(const Array<T>&, bool first=false);// These tree functions
															// and operators works
   void     operator+=(const Array<T>& t)					// like the tree
               { alwaysInsert(t); }							// functions above,
															// but are inserting
   bool     insert(const Array<T>&, bool first=false);		// a whole array.
															
   bool     removeIndex(int);				// Remove object with specified index

   bool     removeFirst() {return removeIndex(0);}	// Remove first object in array

   bool     removeLast() {return removeIndex(no_elem-1);}// Remove last object in array

   bool     remove(const T& obj)			// True if removed (False not existing)
               { return removeIndex(index(obj)); } 

   int      size() const					// Number of elements.
               { return no_elem;}

   int      maxSize() const					// Number of max possible elements.
               { return max_elem;}

   void     setSize(int);					// Set number of elements in the array.
											// New elements are not initialized
											// surplus element removed from end.

   void     setSize(int,T);					// Set number of elements in the array.
											// New elements are initialized with T
											// surplus element removed from end.

   void     resetSize();					// Set number of elements in the array to 0
											// without doing anything else! FAST VERSON.

   void     setMax(int);					// Set number of possible elements
											// max_elem, no_elem is not changed.

   void     strip();						// Remove any surplus(max_elem=no_elem)

   void     resize(int n) {setSize(n);strip();}

   bool	isEmpty() const { return no_elem == 0; }

   void     turn();							// Turn the array
											// (set first element as last etc.)

   void     swap(int i, int j)				// Swap two elements.
               { T tmp = array[i]; array[i] = array[j]; array[j] = tmp; }

   void	push()	{ (*this) += last(); }
   void	pop()	{ no_elem--; }

   void	setStreamMode(bool s=false){numb=s;}// default is removing the number.
   bool	getStreamMode()const {return numb;}

   bool     operator==(const Array<T>&) const;	// EQUAL?
   bool     operator!=(const Array<T>& a) const	// NOT EQUAL?
               { return !((*this) == a); }


 public:
	template <class T_Stream>
	friend T_Stream& operator<<(T_Stream& out, const Array<T>& v)
	{ 
		if (v.getStreamMode())
			out << v.size() << element_separator;
		for (int i=0; i<v.size();i++) 
			out << v(i) << object_separator;
		return out;
	}

	template< class T_Stream>
	friend T_Stream& operator>>(T_Stream& in, Array<T>& v)
	{ 
		int i;
		static Separator es(element_separator);
		static Separator os(object_separator);
		static Separator gs(group_separator);
		if (v.getStreamMode())
		{
			in >> i >> es;
			v.setSize(i);
		}
		for (i=0; i<v.size();i++) 
			in >> v[i] >> os;
		return in;
	}
};




//********************************************************
//**************** DEFAULT CONSTRUCTOR *******************
//********************************************************

template<class T>
inline
Array<T>::Array(int n)
{
	sorted	= false;
	numb	= false;
	no_elem = 0;
	max_elem = (n < 6 ? 6 : n);
    if (max_elem > 6)	array = new T[max_elem];
    else				array = pt;
}

//********************************************************
//**************** STANDARD CONSTRUCTOR *******************
//********************************************************

/* // kills Array<int>
template<class T>
inline
Array<T>::Array(T t)
{
	sorted = false;
	no_elem = 1;
	max_elem = 6;
    array = pt;
	pt[0]=t;
}
*/


//********************************************************
//**************** STANDARD CONSTRUCTOR *******************
//********************************************************

template<class T>
inline
Array<T>::Array(int n, T t)
{
	sorted = false;
	no_elem = n;
	max_elem = (n < 6 ? 6 : n);
    if (max_elem > 6)	array = new T[max_elem];
    else				array = pt;
	for(int i=0; i<n, i++) array[i] = t;
}



//********************************************************
//**************** COPY CONSTRUCTOR **********************
//********************************************************

template<class T>
inline
Array<T>::Array(const Array<T>& da)
{ 
	no_elem  = 0; 
	max_elem = 6;
    array    = pt;
	*this    = da;
}


//********************************************************
//*************** STANDARD CONSTRUCTOR *******************
//********************************************************

template<class T>
inline
Array<T>::Array(int n, const T* p)
{
	sorted	= false;
	numb	= true;
	no_elem = n;
	max_elem = (n < 6 ? 6 : n);
    if (max_elem > 6)	array = new T[max_elem];
    else				array = pt;

	memcpy(array,p,n*sizeof(T)); //for(int i=0; i < n; i++)	array[i] = p[i];
}


//********************************************************
//**************** RESET THE ARRAY ***********************
//********************************************************

template<class T>
inline
void  Array<T>::clear()
{
	if (max_elem > 6) delete [] array; 
	no_elem  = 0; 
	max_elem = 6; 
	array    = pt;
	sorted	 = false;
	numb	 = true;

}


//********************************************************
//**************** OPERATOR[] -ORGINAL ELEMENT************
//********************************************************

template<class T>
inline
T&  Array<T>::operator[](int i)
{
#ifdef DEBUG
    if (i < 0 ||  i >= no_elem)
		printf("index %d is outside (0, %d)\n", i, no_elem);
#endif
    return array[i]; 
}


//********************************************************
//**************** OPERATOR() - COPY OF ELEMENT **********
//********************************************************

template<class T>
inline
const T&   Array<T>::operator()(int i) const
{
#ifdef DEBUG
    if (i < 0 ||  i >= no_elem)
		printf("index %d is outside (0, %d)\n", i, no_elem);
#endif
    return array[i];
}


//********************************************************
//**************** INSERT(OBJECT) - INSERT IN ARRAY  *****
//********************************************************

template<class T>
inline
bool	 Array<T>::insert(const T& obj, bool first)
{ 
	bool ret = true;
	if (exist(obj))  ret = false;
	else             alwaysInsert(obj,first);
	return ret;
}


//********************************************************
//************** INSERT(ARRAY) - INSERT HOLE ARRAY  ******
//********************************************************

template<class T>
inline
bool	 Array<T>::insert(const Array<T>& arr, bool first)
{ 
	bool ret = true;

	for(int i=0; i<arr.size(); i++)
	{
		if (exist(arr.array[i]))  ret = false;
		else			alwaysInsert(arr.array[i],first);
	}
	return ret;
}

 
//********************************************************
//************* Initiate array (all element equal t) *****
//********************************************************

template<class T>
inline
void Array<T>::initialize(const T& t)
{ 
	for(int i=0; i<size(); i++)
		(*this)[i] = t;
}


//********************************************************
//************ Reset array (0 elements fast version) *****
//********************************************************

template<class T>
inline
void Array<T>::resetSize()
{ 
	no_elem=0;
}


} // end namespace GM_lib

#include "GM_Array.c"

#endif /* GM_ARRAY_H_ */

