#ifndef _GM_ASTEROIDAL_SPHERE_H
#define _GM_ASTEROIDAL_SPHERE_H

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Asteroidal_Sphere : public PSurf<T,n>
{
	T _a;		//scalex
	T _b;		//scaley;
	T _c;		//scalez;

	void _eval(T u, T v, int d1, int d2);

protected:

	T _st_p_u()				{ return T(0.0); }
	T _end_p_u()			{ return T(M_2PI); }
	T _st_p_v()				{ return T(-M_PI*0.5); }
	T _end_p_v()			{ return T(M_PI*0.5); }

public:
	Asteroidal_Sphere(T scalex=T(1), T scaley=T(1), T scalez=T(1));
	Asteroidal_Sphere(const Asteroidal_Sphere<T,n>& s);

	virtual  
	~Asteroidal_Sphere() {}

	void setConstants(T scalex, T scaley, T scalez);

	const char* identity() const { return " GM_Asteroidal Sphere ";}
};

} // end namespace GM_lib

using namespace GM_lib;

template <class T, int n>
Asteroidal_Sphere<T,n>::Asteroidal_Sphere(T scalex, T scaley, T scalez):PSurf<T,n>()
{
	setConstants(scalex, scaley, scalez);
}

template <class T, int n>
Asteroidal_Sphere<T,n>::Asteroidal_Sphere(const Asteroidal_Sphere<T,n>& s):PSurf<T,n>(s)
{
	setConstants(s._a, s._b ,s._c);
}

template <class T, int n>
void Asteroidal_Sphere<T,n>::setConstants(T scalex, T scaley, T scalez)
{
	_a = scalex;
	_b = scaley;
	_c = scalez;

	T r = max(max(scalex,scaley),scalez);

	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),r));
}

template <class T, int n>
void Asteroidal_Sphere<T,n>::_eval(T u, T v, int d1,int d2)
{
 	__p.setDim(4,4);

	// Position
	__p[0][0][0] =	pow((_a*cos(u)*cos(v)),T(3));		
	__p[0][0][1] =	pow((_b*sin(u)*cos(v)),T(3));		
	__p[0][0][2] =	pow((_c*sin(v)),T(3));		

	if(d1) //u
	{
		__p[1][0][0] =	-T(3)*_a*_a*_a*cos(u)*cos(u)*cos(v)*cos(v)*cos(v)*sin(u);	
		__p[1][0][1] =	-T(3)*_b*_b*_b*(-1.0+cos(u)*cos(u))*cos(v)*cos(v)*cos(v)*cos(u);	
		__p[1][0][2] =	T(0);	
	}
	if(d1>1)//uu
	{
		__p[2][0][0] = -T(3)*_a*_a*_a*cos(u)*cos(v)*cos(v)*cos(v)*(-T(2)+T(3)*cos(u)*cos(u));		
		__p[2][0][1] =	T(3)*_b*_b*_b*sin(u)*cos(v)*cos(v)*cos(v)*(T(3)*cos(u)*cos(u)-1.0);	
		__p[2][0][2] =	T(0);	
	}
	if(d2) //v
	{
		__p[0][1][0] =	-T(3)*_a*_a*_a*cos(u)*cos(u)*cos(u)*cos(v)*cos(v)*sin(v);	
		__p[0][1][1] =	T(3)*_b*_b*_b*(-1.0+cos(u)*cos(u))*sin(u)*cos(v)*cos(v)*sin(v);	
		__p[0][1][2] =	-T(3)*_c*_c*_c*(-1.0+cos(v)*cos(v))*cos(v);	
	}
	if(d2>1) //vv
	{
		__p[0][2][0] =	-T(3)*_a*_a*_a*cos(u)*cos(u)*cos(u)*cos(v)*(-T(2)+T(3)*cos(v)*cos(v));	
		__p[0][2][1] =	T(3)*_b*_b*_b*(-1.0+cos(u)*cos(u))*sin(u)*cos(v)*(-T(2)+T(3)*cos(v)*cos(v));	
		__p[0][2][2] =	T(3)*_c*_c*_c*sin(v)*(T(3)*cos(v)*cos(v)-1.0);	
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =  T(9)*_a*_a*_a*cos(u)*cos(u)*cos(v)*cos(v)*sin(u)*sin(v);		
		__p[1][1][1] =	T(9)*_b*_b*_b*(-1.0+cos(u)*cos(u))*cos(v)*cos(v)*cos(u)*sin(v);	
		__p[1][1][2] =	T(0);	
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =	T(9)*_a*_a*_a*cos(u)*cos(v)*cos(v)*sin(v)*(-T(2)+T(3)*cos(u)*cos(u));	
		__p[2][1][1] =	-T(9)*_b*_b*_b*sin(u)*cos(v)*cos(v)*sin(v)*(T(3)*cos(u)*cos(u)-1.0);	
		__p[2][1][2] =	T(0);	
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =	T(9)*_a*_a*_a*cos(u)*cos(u)*cos(v)*sin(u)*(-T(2)+T(3)*cos(v)*cos(v));	
		__p[1][2][1] =	T(9)*_b*_b*_b*(-1.0+cos(u)*cos(u))*cos(v)*cos(u)*(-T(2)+T(3)*cos(v)*cos(v));	
		__p[1][2][2] =	T(0);	
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =	T(9)*_a*_a*_a*cos(u)*cos(v)*(T(4)-T(6)*cos(v)*cos(v)-T(6)*cos(u)*cos(u)+T(9)*cos(u)*cos(u)*cos(v)*cos(v));
		__p[2][2][1] =	-T(9)*_b*_b*_b*sin(u)*cos(v)*(-T(6)*cos(u)*cos(u)+T(9)*cos(u)*cos(u)*cos(v)*cos(v)+T(2)-T(3)*cos(v)*cos(v));	
		__p[2][2][2] =	T(0);
	}

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}

#endif // _GM_ASTEROIDAL_SPHERE_H

