
//*************************************************************************************
// GM_BSplineCurve.h: interface for the BSplineCurve classes.
//*************************************************************************************
#ifndef GM_BSPLINECURVE_H_
#define GM_BSPLINECURVE_H_

#include "GM_PCurve.h"

namespace GM_lib{

template <class T, int n>
class BSplineCurve: public PCurve<T,n>
{
	void init();

	void localSimulate(double dt){}

	bool __ctrl_poly;

protected:
	Array<Array<Point<T,n> > >	_c;
	Array<T>					_t;
	int							_d;

	void _eval(T t, int d);

	T _st_p() { return _t[0]; }
	T _end_p(){ return _t[_t.size()-1]; }

 public:
//	BSplineCurve(const Array<Point<T,n> >& c, const Array<T>& t, int d):PCurve<T,n>() 
//	{_c.setSize(d+1); _c[0]=c; _t=t; _d=d; init(); __ctrl_poly=false;}

// Changed for debuging
	BSplineCurve(const Array<Point<T,n> >& c, const Array<T>& t, int d):PCurve<T,n>() 
	{
		_c.setSize(d+1);
		_c[0]=c;
		_t=t;
		_d=d;
		init();
		__ctrl_poly=false;
	}
	BSplineCurve(const BSplineCurve<T,n>& c):PCurve<T,n>(c) { _c=c._c; _t=c._t; _d=c._d; __ctrl_poly=false;}

	virtual
	~BSplineCurve() {}

	void displayCtrlPolygon() { __ctrl_poly = !__ctrl_poly; }

	PCurve<T,n>* split(T t) { return NULL;}

	const char* identity() const { return "BSPLINE-CURVE";}
};


} // end namespace GM_lib


#include <GM_DMatrix.h>
using namespace GM_lib;

template <class T, int n>
void BSplineCurve<T,n>::init()
{
	int i,j,nn=_c[0].size();

	for(i=1; i<=_d; i++)
	{
		_c[i].setSize(nn-i);
		for(j=0;j<nn-i;j++) _c[i][j] = ((_d+1-i)/(_t[j+_d+3-i]-_t[j]))*(_c[i-1][j+1]-_c[i-1][j]);
	}

	__material = Brass;
	resetPlot(23);
}


template <class T, int n>
void BSplineCurve<T,n>::_eval(T t, int d)
{
	if(d<=__d && t==__t) return;
	__t=t;
	__d=d;

	static DMatrix<T> b(10,10);
	if(_d > b.dim1()) b.setDim(_d,_d);

	int i,j,s,s1,nn=_c[0].size();
	Array<T> w;
	w.setSize(_d);

	for(s=0; s<nn; s++) if(_t[s+1] > t) break;
//	for(s=_d; s<nn+_d; s++) if(_t[s+1] > t) break;
	if(s==nn) s--; 
	s1=s+1;

	b[0][0] = T(1.0);
	for(i=1; i<=_d; i++)
	{
		for(j=0;j<i;j++)w[j]=(t-_t[s1-i+j])/(_t[s1+j]-_t[s1-i+j]);

		b[i][0] = (1-w[0])*b[i-1][0];
		for(j=1;j<i;j++) b[i][j] = w[j-1]*b[i-1][j-1]+(1-w[j])*b[i-1][j];
		b[i][i] = w[i-1]*b[i-1][i-1];
	}

	__p.setDim(d+1);
//	__p.setSize(d+1);
	for(i=0;i<=d;i++)
	{
		__p[i] = Point<T,n>(0.0);
		for(j=0; j<=_d-i; j++) __p[i] += _c[i][s-_d+j]*b[_d-i][j];
	}
}

#endif   /* GM_BSPLINECURVE_H_ */