
//*************************************************************************************
// GM_BSplineCurve.h: interface for the BSplineCurve classes.
//*************************************************************************************
#ifndef GM_BSPLINESURF_H_
#define GM_BSPLINESURF_H_

#include <GM_PSurf.h>
#include <GM_SelectorGrid.h>

namespace GM_lib{

template <class T, int n>
class BSplineSurf: public PSurf<T,n>
{
	bool			_clu;				// Closed in u-direction?
	bool			_clv;				// Closed in v-direction?
	bool			_ctrl_pt_moved;		// Short time status if a ctrl-point is moved
	bool			_pre_eval;			// Pre evaluation are used!
	bool			_selectors_exist;	// The set of selectores exist or not

	// Private help fuctions
	void _init();
	void _findIndex(T u,T v, int& i, int& j){i=(__sam1-1)*(u-startPar_u())/(deltaPar_u())+0.1; 
											 j=(__sam2-1)*(v-startPar_v())/(deltaPar_v())+0.1;}
	void _updateDiffCoef(int i, int j);
	void _sjoenBerg(int i, int j, T& u, T& v);

	void strIn(GM_String& in);
	void strOut(GM_String& out);

protected:
	DMatrix<DMatrix<Point<T,n> > >	_c;			// Controll-polygon
	DVector<T>						_u;			// knot-vector in u-direction
	DVector<T>						_v;			// knot-vector in v-direction
	int								_d1;		// degree in u-direction
	int								_d2;		// degree in v-direction
	DMatrix<Selector<T,n>*>			_edit_pts;	// The selectors, one at each control point 
	SelectorGrid<T,n>*				_grid;		// A pointer to the selector grid

	// Inherited virtual functions
	void _eval(T u, T v, int d1, int d2);

	//**********************************************
	//********   Set the parameter domain   ********
	//**********************************************
	T _st_p_u()		{ return _u[0]; }
	T _end_p_u()	{ return _u[_u.dim()-1]; }
	T _st_p_v()		{ return _v[0]; }
	T _end_p_v()	{ return _v[_v.dim()-1]; }

 public:
	BSplineSurf(const DMatrix<Point<T,n> >& c, const DVector<T>& u, const DVector<T>& v, int d1, int d2);
	BSplineSurf(const DMatrix<Point<T,n> >& c, Selector<T,n>* sp,int d1, int d2);
	BSplineSurf(const BSplineSurf<T,n>& c);

	template <class T_Stream>
	BSplineSurf(T_Stream& in) {/* _strIn(in);*/ _selectors_exist=false; _init();}

	virtual
	~BSplineSurf() {}

	void estimateClpPar(const Point<T,n>& p, T& u, T& v);

	void toggleSelectorExistence();
	void toggleSelector();

	void updateCoeffs(const Vector<T,n>& d);
	void edit(int selector_id);

	const char* identity() const { return "BSPLINE-SURF";}
};



template <class T_Stream, class T, int n>
inline	
T_Stream &operator<<(T_Stream& out, const BSplineSurf<T,n>& b)
{
	GM_String str;
	friend class BSplineSurf<T,n>;
	b.strOut(str);
	return out << str;
}

template <class T_Stream, class T, int n>
inline	
T_Stream &operator>>(T_Stream& in, const BSplineSurf<T,n>& b)
{
friend class BSplineSurf<T,n>;
	int n1,n2;
	in >> n1;
	if(n1)
	{
		char file[100];
		kvil kv;
		in >> file;
		glGenTextures(1,&__texname);
		kv.LoadGLSphereMap(__texname, file);
	}
	in >> b._d1 >> b._d2 >> n1 >> n2;
	b._u.setStreamMode(false);
	b._v.setStreamMode(false);
	b._u.setSize(n1+_d1+1);
	b._v.setSize(n2+_d2+1);
	in >> b._u >> b._v;
	b._c.setDim(_d1+1,_d2+1);
	b._c[0][0].setDim(n1,n2);
	in >> b._c[0][0];
	return in;
}

} // end namespace GM_lib


#include <GM_DMatrix.h>
using namespace GM_lib;

template <class T, int n>
inline
BSplineSurf<T,n>::BSplineSurf(const DMatrix<Point<T,n> >& c, const DVector<T>& u, const DVector<T>& v, int d1,int d2):PSurf<T,n>() 
{
	_c.setDim(d1+1, d2+1);
	_c[0][0] = c; 
	_u=u; 
	_v=v; 
	_d1=d1; 
	_d2=d2;
	_selectors_exist=false;
	_init();
}
template <class T, int n>
inline
BSplineSurf<T,n>::BSplineSurf(const DMatrix<Point<T,n> >& c, Selector<T,n>* sp, int d1, int d2):PSurf<T,n>() 
{
	_c.setDim(d1+1,d2+1);
	_c[0][0]=c;
	errrrrrrrorrrr
	_u.setSize(d1+3);
	_v.setSize(d2+3);
	for (int i=0;i<_c.dim)
	{
	}
	errrrrrrrrrror
	_d1=d1; 
	_d2=d2;
	_selectors_exist=false;
	_init();
}


template <class T, int n>
inline	
BSplineSurf<T,n>::BSplineSurf(const BSplineSurf<T,n>& c):PSurf<T,n>(c) 
{ 
	_c=c._c; 
	_u=c._u; 
	_v=c._v; 
	_d1=c._d1; 
	_d2=c._d2;
	_selectors_exist=false;
}


template <class T, int n>
void BSplineSurf<T,n>::_init()
{
	int i,j,i1,j1,k;

	// Initialize coeffisient matrix (all partial derivatives) for speed up
	for(j=0; j<_c.dim2(); j++)
	{
		if(j!=0)
		{
			k = j*(_d1+1);
			_c[0][j].setDim(_c[0][0].dim1(), _c[0][0].dim2()-j);
			for(i1=0; i1<_c[0][j].dim1(); i1++)
				for(j1=0; j1<_c[0][j].dim2(); j1++) 
					_c[0][j][i1][j1] = ((_d2+1-j)/(_v[j1+_d2+1]-_v[j1+j]))*(_c[0][j-1][i1][j1+1]-_c[0][j-1][i1][j1]);
		}
		for(i=1; i<_c.dim1(); i++)
		{
			k = i+j*(_d1+1);
			_c[i][j].setDim(_c[0][0].dim1()-i, _c[0][0].dim2()-j);
			for(i1=0; i1<_c[i][j].dim1(); i1++)
				for(j1=0; j1<_c[i][j].dim2(); j1++)
					_c[i][j][i1][j1] = ((_d1+1-i)/(_u[i1+_d1+1]-_u[i1+i]))*(_c[i-1][j][i1+1][j1]-_c[i-1][j][i1][j1]);
		}
	}
}
	

// Initialize selectors for editing B-spline surface
template <class T, int n>
void BSplineSurf<T,n>::toggleSelectorExistence()
{
	int i,j;
	if(_selectors_exist)
	{
		for (i=0; i<_edit_pts.dim1(); i++)
			for (j=0; j<_edit_pts.dim2(); j++)
			{
				remove(_edit_pts[i][j]);
				delete _edit_pts[i][j];
			}
		remove(_grid);
		delete _grid;
		_selectors_exist = false;
	}
	else
	{
		int k;
		GL_Color grid_color(0.0,0.9,0.0);
		GL_Color selector_color(0.8,0.2,0.2);
		T		 selector_size = T(0.125);
		_edit_pts.setDim(_c[0][0].dim1(),_c[0][0].dim2());
		_grid  = new SelectorGrid<T,n>(_c[0][0][0][0],this,grid_color);	
		for (i=0, k=0; i<_c[0][0].dim1();i++)
			for (j=0; j<_c[0][0].dim2();j++)
			{
				_edit_pts[i][j] = new Selector<T,n>((_c[0][0][i][j]),k++,this,selector_size,selector_color);
				insert(_edit_pts[i][j]);
				if(j!=0) _grid->add(_c[0][0][i][j-1], _c[0][0][i][j]);  // Horisontal lines in grid
				if(i!=0) _grid->add(_c[0][0][i-1][j], _c[0][0][i][j]);  // Vertical ilines in grid
			}
		insert(_grid);	
		_selectors_exist = true;
	}
}


template <class T, int n>
void BSplineSurf<T,n>::_eval(T u, T v, int d1,int d2)
{
	static DMatrix<T> bu(10,10);
	static DMatrix<T> bv(10,10);
	if(_d1 > bu.dim1()) bu.setDim(_d1,_d1);
	if(_d2 > bv.dim1()) bv.setDim(_d2,_d2);

	int i,j,i1,j1,s,s1,s2,nn1=_c[0][0].dim1(),nn2=_c[0][0].dim2();
	Array<T> w;
	w.setSize((_d1>_d2?_d1:_d2)+1);

	for(s=_d1; s<nn1; s++) if(_u[s+1] > u) break;
	if(s==nn1) s--; 
	s1=s+1;
	for(s=_d2; s<nn2; s++) if(_v[s+1] > v) break;
	if(s==nn2) s--; 
	s2=s+1;

	bu[0][0] = T(1.0);
	for(i=1; i<=_d1; i++)
	{
		j1 = s1-i;
		for(j=0;j<i;j++) w[j]=(u-_u[j1+j])/(_u[s1+j]-_u[j1+j]);
		i1= i-1;
		bu[i][0] = (1-w[0])*bu[i-1][0];
		for(j=1;j<i;j++) bu[i][j] = w[j-1]*bu[i1][j-1]+(1-w[j])*bu[i1][j];
		bu[i][i] = w[i1]*bu[i1][i1];
	}
	bv[0][0] = T(1.0);
	for(i=1; i<=_d2; i++)
	{
		j1 = s2-i;
		for(j=0;j<i;j++) w[j]=(v-_v[j1+j])/(_v[s2+j]-_v[j1+j]);
		i1= i-1;
		bv[i][0] = (1-w[0])*bv[i1][0];
		for(j=1;j<i;j++) bv[i][j] = w[j-1]*bv[i1][j-1]+(1-w[j])*bv[i1][j];
		bv[i][i] = w[i1]*bv[i1][i1];
	}

	__p.setDim(d1+1,d2+1);
	
	for(i=0; i<=d1; i++)
		for(j=0; j<=d2; j++)
		{
	  		__p[i][j] = Point<T,n>(0.0);
			for(i1=0;i1<=_d1-i;i1++)
				for(j1=0; j1<=_d2-j; j1++) 
					__p[i][j] += _c[i][j][s1-_d1+i1-1][s2-_d2+j1-1]*(bu[_d1-i][i1]*bv[_d2-j][j1]);
		}

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}


template <class T, int n>
inline
void BSplineSurf<T,n>::_updateDiffCoef(int i2, int j2)
{
	int i,j,i1,j1;
	for(j=0; j<_c.dim2(); j++)
	{
		if(j!=0)
		{
			for(j1=max(j2-j,0); j1<min(j2+1,_c[0][j].dim2()); j1++) 
				_c[0][j][i2][j1] = ((_d2+1-j)/(_v[j1+_d2+1]-_v[j1+j]))*(_c[0][j-1][i2][j1+1]-_c[0][j-1][i2][j1]);
		}
		for(i=1; i<_c.dim1(); i++)
		{
			for(i1=max(i2-i,0); i1<min(i2+1,_c[i][j].dim1()); i1++)
				for(j1=max(j2-j,0); j1<min(j2+1,_c[i][j].dim2()); j1++) 
					_c[i][j][i1][j1] = ((_d1+1-i)/(_u[i1+_d1+1]-_u[i1+i]))*(_c[i-1][j][i1+1][j1]-_c[i-1][j][i1][j1]);
		}
	}
}


template <class T, int n>
inline
void BSplineSurf<T,n>::_sjoenBerg(int i, int j, T& u, T& v)
{
	int k1=i+_d1, k2=j+_d2;
	for(u=T(0),i++; i<=k1;i++) u += _u[i];
	for(v=T(0),j++; j<=k2;j++) v += _v[j];
	u /= _d1;
	v /= _d2;
}

template <class T, int n>
void BSplineSurf<T,n>::toggleSelector()
{
	for (int i=0; i<_edit_pts.dim1();i++)
		for (int j=0; j<_edit_pts.dim2();j++) 
			_edit_pts[i][j]->onOff();
	_grid->onOff();
//	DisplayObject::toggleSelector();
}	


template <class T, int n>
void BSplineSurf<T,n>::updateCoeffs(const Vector<T,n>& d)
{
	if(_ctrl_pt_moved)
	{
		Vector<T,n> diff = _matrix.inverse2()*d;
		for (int i=0; i<_c[0][0].dim1(); i++)
			for (int j=0; j<_c[0][0].dim2();j++) 
			{
				_c[0][0][i][j] += diff;
				_edit_pts[i][j]->translate(diff);
			}
		LA_DisplayObject::translate(-d);
		resetPlot(0,0,true);
	}
}


template <class T, int n>
void BSplineSurf<T,n>::edit(int selector_id)
{
	_ctrl_pt_moved = true;
	_updateDiffCoef(selector_id/_edit_pts.dim2(),selector_id%_edit_pts.dim2());
	resetPlot(0,0,true);
	if(__parent) __parent->edit(this);
	_ctrl_pt_moved = false;
}


template <class T, int n>
void BSplineSurf<T,n>::estimateClpPar(const Point<T,n>& p, T& u, T& v)
{
	int i,j,im=0,jm=0;
	T d = (p-_c[0][0][0][0]).length();
	for(i=0; i<_c[0][0].dim1(); i++)
		for(j=0; j<_c[0][0].dim2(); j++)
		{
			T dh =(p-_c[0][0][i][j]).length();
			if(dh < d)
			{
				d = dh;
				im = i;
				jm = j;
			}
		}
	_sjoenBerg(im,jm,u,v);
}

/*
template <class T_Stream, class T, int n>
T_Stream& BSplineSurf<T,n>::strIn(T_Stream& in)
{
	int n1,n2;
	in >> n1;
	if(n1)
	{
		char file[100];
		kvil kv;
		in >> file;
		glGenTextures(1,&__texname);
		kv.LoadGLSphereMap(__texname, file);
	}
	in >> __sam1 >> __sam2;
	in  >> __amb >> __dif >> __spc;
	in >> _d1 >> _d2 >> n1 >> n2;
	_u.setStreamMode(false);
	_v.setStreamMode(false);
	_u.setSize(n1+_d1+1);
	_v.setSize(n2+_d2+1);
	in >> _u >> _v;
	_c.setSize((_d1+1)*(_d2+1));
	_c[0][0].setdim(n1,n2);
	in >> _c[0][0];
	return in;
}

template <class T_Stream, class T, int n>
T_Stream& BSplineSurf<T,n>::strOut(T_Stream& out)
{
	return out;
}

*/

#endif   /* GM_BSPLINESURF_H_ */