
//***********************************************************************
//*******   GM_Bent_Horn.h: interface for the Bent_Horn classes.  *******
//***      This is a Surface made by R. L. Bagula and P. Bourke       ***
//***********************************************************************
#ifndef _GM_BENT_HORNS_H_
#define _GM_BENT_HORNS_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Bent_Horns : public PSurf<T,n>
{
	void _eval(T u, T v, int d1, int d2);
protected:

	T _st_p_u()		{ return T(-M_PI); }
	T _end_p_u()	{ return T(M_PI); }
	T _st_p_v()		{ return T(-2*M_PI); }
	T _end_p_v()	{ return T(2*M_PI); }

public:
	Bent_Horns();
	Bent_Horns(const Bent_Horns<T,n>& s);

	bool  closedU()  { return true; }

	const char* identity() const { return " Bent_Horns "  ;}
};




template <class T, int n>
inline
Bent_Horns<T,n>::Bent_Horns():PSurf<T,n>()
{
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),4.0));
}

template <class T, int n>
inline
Bent_Horns<T,n>::Bent_Horns(const Bent_Horns<T,n>& s):PSurf<T,n>(s)
{
}

template <class T, int n>
void Bent_Horns<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(d1+1,d2+1);

	T cu	 = cos(u);
	T cv	 = cos(v);
	T su	 = sin(u);
	T sv	 = sin(v);
	T cv1	 = cv - 1;
	T cu2pm3 = cos(u - M_2PI_3);
	T cu2pp3 = cos(u + M_2PI_3);
	T su2pm3 = sin(u - M_2PI_3);
	T su2pp3 = sin(u + M_2PI_3);
	T v3s	 = (v-3*sv)/3;
	T v3c	 = (3*cv-1)/3;

						//S
	__p[0][0][0] = (2+cu)*v3s;
	__p[0][0][1] = (2+cu2pm3)*cv1;
	__p[0][0][2] = (2+cu2pp3)*cv1;
	if(d1)				//Su
	{
		__p[1][0][0] = -su*v3s;
		__p[1][0][1] = -su2pm3*cv1;
		__p[1][0][2] = -su2pp3*cv1;
	}
	if(d1>1)			//Suu	
	{
		__p[2][0][0] = -cu*v3s;				 
		__p[2][0][1] = -cu2pm3*cv1;		
		__p[2][0][2] = -cu2pp3*cv1;			
	}
	if(d2)				//Sv			
	{
		__p[0][1][0] = -(2+cu)*v3c;				
		__p[0][1][1] = -(2+cu2pm3)*sv;				
		__p[0][1][2] = -(2+cu2pp3)*sv;				
	}
	if(d2>1)			//Svv				
	{
		__p[0][2][0] =	(2+cu)*sv;				 
		__p[0][2][1] = -(2+cu2pm3)*cv;
		__p[0][2][2] = -(2+cu2pp3)*cv;	
	}
	if(d1 && d2)		//Suv			
	{
		__p[1][1][0] =	su*v3c;				 
		__p[1][1][1] =	su2pm3*sv;				 
		__p[1][1][2] =	su2pp3*sv;				 
	}
	if(d1>1 && d2)		//Suuv	
	{
		__p[2][1][0] =	cu*v3c;
		__p[2][1][1] =	cu2pm3*sv;
		__p[2][1][2] =	cu2pp3*sv;
	}
	if(d1 && d2>1)		//Suvv	
	{
		__p[1][2][0] = -su*sv;				 
		__p[1][2][1] =	su2pm3*cv;				 
		__p[1][2][2] =	su2pp3*cv;				 
	}
	if(d1>1 && d2>1)	//Suuvv
	{				
		__p[2][2][0] = -cu*sv;	
		__p[2][2][1] =	cu2pm3*cv;				
		__p[2][2][2] =	cu2pp3*cv;				
	}			
	if(d1>2 && d2)		//Suuuv	
	{
		__p[3][1][0] = -su*v3c;
		__p[3][1][1] = -su2pm3*sv;
		__p[3][1][2] = -su2pp3*sv;
	}
	if(d1>2 && d2>1)	//Suuuvv	
	{
		__p[3][2][0] =  su*sv;
		__p[3][2][1] = -su2pm3*cv;
		__p[3][2][2] = -su2pp3*cv;
	}
	if(d1>2 && d2>2)	//Suuuvvv	
	{
		__p[3][3][0] =  su*cv;
		__p[3][3][1] =  su2pm3*sv;
		__p[3][3][2] =  su2pp3*sv;
	}
	if(d1 && d2>2)		//Suvvv	
	{
		__p[1][3][0] = -su*cv;				 
		__p[1][3][1] = -su2pm3*sv;				 
		__p[1][3][2] = -su2pp3*sv;	
	}
	if(d1>1 && d2>2)	//Suuvvv	
	{
		__p[2][3][0] = -cu*cv;				 
		__p[2][3][1] = -cu2pm3*sv;				 
		__p[2][3][2] = -cu2pp3*sv;				 
	}
	if(d1>2)			//Suuu	
	{
		__p[3][0][0] =  su*v3s;				 
		__p[3][0][1] =  su2pm3*cv1;		
		__p[3][0][2] =  su2pp3*cv1;			
	}
	if(d2>2)			//Svvv				
	{
		__p[0][3][0] =	(2+cu)*cv;				 
		__p[0][3][1] =  (2+cu2pm3)*sv;
		__p[0][3][2] =  (2+cu2pp3)*sv;	
	}
	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}

} // GM_lib

 #endif //_GM_BENT_HORNS_H_
