
//*************************************************************************************
// GM_BezierSurf.h: interface for the BezierSurface classes.
//*************************************************************************************

#ifndef GM_BEZIERSURF_H_
#define GM_BEZIERSURF_H_

#include "GM_PSurf.h"
#include "GM_SelectorGrid.h"

namespace GM_lib{

template <class T, int n>
class BezierSurf: public PSurf<T,n>
{
	bool			_clu;			// Closed in u-direction?
	bool			_clv;			// Closed in v-direction?
	bool			_ctrl_poly;		// Shall the control-polygon be displayed? 
	bool			_ctrl_pt_moved;	// Short time status if a ctrl-point is moved
	bool			_pre_eval;		// Pre evaluation are used!

	// Private help fuctions
	void		init();
	void		findIndex(T u,T v, int& i, int& j) {i= (__sam1-1)*(u-startPar_u())/(deltaPar_u())+0.1; 
													j= (__sam2-1)*(v-startPar_v())/(deltaPar_v())+0.1;}
	void		calcCtrlPts(const DMatrix<Vector<T,n> >& c,const Point<T,2>& p,const Point<T,2>& d);

	const DMatrix<T>& bernstainPoly(int k, T y, T dx=T(1));

protected:
	DMatrix<Vector<T,n> >		_c;				// The control points
	DMatrix<Selector<T,n>*>		_edit_pts;	// The selectors, one at each control point 
	SelectorGrid<T,n>*			_grid;			// A pointer to the selector grid
	DMatrix<DMatrix<T> >		_bpus;			// Preevaluated basis functions in u-direction
	DMatrix<DMatrix<T> >		_bpvs;			// Preevaluated basis functions in v-direction

	// Inherited virtual functions
	void	_eval(T u, T v, int d1, int d2);					   // Virtual from PSurf
	void	_makeSample( DMatrix<Arrow<T,n> >& a, int m1, int m2); // Virtual from PSurf
	void	localDisplay();										   // Virtual from DisplayObject

	T	_st_p_u()		{ return T(0); }
	T	_end_p_u()		{ return T(1); }
	T	_st_p_v()		{ return T(0); }
	T	_end_p_v()		{ return T(1); }

 public:
	BezierSurf(const DMatrix<Vector<T,n> >& c, DisplayObject* sp=NULL, bool interpolate=false, 
			   const Point<T,2>& p=Point<T,2>(), const Point<T,2>& d=Point<T,2>(1.0));
	BezierSurf(const BezierSurf<T,n>& c);

//	template <class T_Stream>
//	BezierSurf(T_Stream& in, DisplayObject* p=NULL):PSurf<T,n>(in,0) { _parent=p; prIn(in); }

	virtual
	~BezierSurf() {}

	void displayCtrlPolygon() { _ctrl_poly = !_ctrl_poly; }

	const char* identity() const { return "BEZIER-SURF";}

	bool closedU()  { return _clu; }
	bool closedV()  { return _clv; }

	void toggleSelector();

	DMatrix<Point<T,n> > getPolator(const Point<T,2>& p, int d1=0, int d2=0)
										{ return getPolator(p(0),p(1),d1,d2);}

	DMatrix<Point<T,n> > getPolator(T u, T v, int d1=0, int d2=0)
	{
		_eval(u,v,d1,d2);
		DMatrix<Point<T,n> > tmp;
		tmp.setDim(__p.dim1(),__p.dim2());

		for (int i=0; i<tmp.dim1();i++)
		{
			for (int j=0; j<tmp.dim2();j++)
			{
				if(i==0 && j==0 ) tmp[0][0]=_matrix*(Point<T,n>)__p[0][0];
				else tmp[i][j]=_matrix*__p[i][j];
			}
		}
		return tmp;
	}

	void setPreEval(T s, T ds)
	{
	}

	void updateCoeffs(const Vector<T,n>& d)
	{
		if(_ctrl_pt_moved)
		{
			Vector<T,n> diff = _matrix.inverse2()*d;
			for (int i=0; i<_c.dim1(); i++)
				for (int j=0; j<_c.dim2();j++) 
				{
					_c[i][j] += diff;
					_edit_pts[i][j]->translate(diff);
				}
			LA_DisplayObject::translate(-d);
			resetPlot(0,0,true);
		}
	}

	void edit(int selector_id)
	{
		_ctrl_pt_moved = true;
		 resetPlot(0,0,true);
		__parent->edit(this);
		_ctrl_pt_moved = false;
	}

/*
	template <class T_Stream>
	T_Stream& prOut(T_Stream& out)
	{	
		out << (int)_clu << element_separator << (int)_clv << group_separator;
		out <<  _c    << group_separator;
		return out;
	}
	
	template <class T_Stream>
	T_Stream& prIn(T_Stream& in)
	{
		static Separator es(element_separator);
		static Separator gs(group_separator);
		
		int us,vs;
		in  >> us >> es >> vs >> gs;
		_clu=us; _clv=vs;
		in >> _c    >> gs;
		_pre_eval=false;
		_ctrl_poly	= false;
		_ctrl_pt_moved = false;
		init();
		return in;
	}
*/
/*
#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
	template <class T_Stream>
		BezierSurf(T_Stream& in):PSurf<T,n>()
	{
		int n1,n2;
		in >> n1;
		if(n1)
		{
			char file[100];
			kvil kv;
			in >> file;
			glGenTextures(1,&__texname);
			kv.LoadGLSphereMap(__texname, file);
		}
		in >> __sam1 >> __sam2;
//		in  >> __amb >> __dif >> __spc;
		in >> _d1 >> _d2 >> n1 >> n2;
		_u.setStreamMode(false);
		_v.setStreamMode(false);
		_u.setSize(n1+_d1+1);
		_v.setSize(n2+_d2+1);
		in >> _u >> _v;
		_c.setSize((_d1+1)*(_d2+1));
		_c[0].setdim(n1,n2);
		in >> _c[0];
		__ctrl_poly=false;
		init();
	}
#endif*/
};

/*
template <class T_Stream, class T, int n>
T_Stream& operator<<(T_Stream& out, BezierSurf<T,n>& s)
{
	out << *((PSurf<T,n> *) &s);
	s.prOut(out);
	return out;
}

template <class T_Stream, class T, int n>
T_Stream& operator>>(T_Stream& in,  BezierSurf<T,n>& s)
{
	in >> *((PSurf<T,n> *) &s);
	s.prIn(in);
	return in;
}
*/
} // end namespace GM_lib



#include <GM_DMatrix.h>
using namespace GM_lib;

	
template <class T, int n>
inline
BezierSurf<T,n>::BezierSurf( const DMatrix<Vector<T,n> >& c, DisplayObject* sp, 
							 bool interpolate, const Point<T,2>& p, const Point<T,2>& d):PSurf<T,n>(20,20,sp) 
{
	_ctrl_poly		= false;
	_pre_eval		= false;
	_ctrl_pt_moved	= false;
	_clu = _clv		= false;
	if(interpolate)	{ calcCtrlPts(c,p,d); }
	else			{ _c = c; init(); }
}


template <class T, int n>
inline	
BezierSurf<T,n>::BezierSurf(const BezierSurf<T,n>& c):PSurf<T,n>(c) 
{ 
	_c				= c._c; 
	_clu			= c._clu;
	_clv			= c._clv;
	_ctrl_poly		= false;
	_ctrl_pt_moved	= false;
}


template <class T, int n>
void BezierSurf<T,n>::calcCtrlPts(const DMatrix<Vector<T,n> >& c,const Point<T,2>& p,const Point<T,2>& d)
{
	DMatrix<T> bu = bernstainPoly(c.dim1(), p(0), 1.0/d(0));
	DMatrix<T> bv = bernstainPoly(c.dim2(), p(1), 1.0/d(1));
	bu.invert();
	bv.invert();
	_c = bu*(c^bv.transpose());
	init();
	for(int i=0;i<_c.dim1();i++)
		for(int j=0;j<_c.dim2();j++)
		{ 
			_c[i][j] -= c(0)(0);
			_edit_pts[i][j]->translate(-c(0)(0));
		}
	LA_DisplayObject::translate(c(0)(0));
}


template <class T, int n>
void BezierSurf<T,n>::init()
{
	GL_Color grid_color(0.0,0.9,0.0);
	GL_Color selector_color(0.8,0.2,0.2);
	T		 selector_size = T(0.125);

	_grid  = new SelectorGrid<T,n>(_c[0][0],this,grid_color);	
	_edit_pts.setDim(_c.dim1(),_c.dim2());
	for (int r=0, k=0; r<_c.dim1();r++)
		for (int s=0; s<_c.dim2();s++)
		{
			_edit_pts[r][s]=new Selector<T,n>((_c[r][s]),k++,this,selector_size,selector_color);
			_edit_pts[r][s]->onOff();					// set default off
			insert(_edit_pts[r][s]);
			if(s!=0) _grid->add(_c[r][s-1], _c[r][s]);  // Horisontal lines in grid
			if(r!=0) _grid->add(_c[r-1][s], _c[r][s]);  // Vertical ilines in grid
		}
	_grid->onOff(); // set default off
	insert(_grid);

	setVisible(true);
}

template <class T, int n>
void  BezierSurf<T,n>::_makeSample(DMatrix<Arrow<T,n> >& a, int m1, int m2)
{
	if(true)//(m1!=__sam1 || m2!=__sam2) || _Bu.dim1()==0) 
	{
		T u, v;
		T du = 1.0/(m1-1);
		T dv = 1.0/(m2-1);
		_pre_eval = false;
		_bpus.setDim(m1,m2);
		_bpvs.setDim(m1,m2);

        for(int i=0; i<m1; i++)	// precalc
		{
			u = i*du;
			for(int j=0;j<m2; j++)
			{
			    v = j*dv;
				_bpus[i][j] = bernstainPoly(_c.dim1(), u);
				_bpvs[i][j] = bernstainPoly(_c.dim2(), v);
			}
		}
	} // end if precalc
	PSurf<T,n>::_makeSample(a,m1,m2);
}


template <class T, int n> 
inline
const DMatrix<T>& BezierSurf<T,n>::bernstainPoly(int k, T y, T dx )
{
	static DMatrix<T> bv;		// Matrix storing result
	int i, j, d = k-1;
	T	x = T(1) - y;
	bv.setDim(k,k);

	// Ordinary Cox/deBoor like algorithm
	bv[d-1][0] = x; bv[d-1][1] = y;
	for(i=d-2; i>=0; i--)	
	{
		bv[i][0] = x*bv[i+1][0];
		for( j=1; j<d-i; j++) bv[i][j] = y*bv[i+1][j-1]+x*bv[i+1][j];
		bv[i][j] = y*bv[i+1][j-1];
	}

	// Computing all derivatives for all basisfunctions
	bv[d][0] = -dx; bv[d][1] = dx;
	for(int d1=2; d1<k; d1++)
	{
		T dy = dx * d1;
		for( i=d; i>d-d1; i--)
		{
			bv[i][d1] = dy*bv[i][d1-1];
			for( j=d1-1; j>0; j--) bv[i][j] =  dy*(bv[i][j-1] - bv[i][j]);
			bv[i][0] = -dy*bv[i][0];
		}
	}
    return bv;
}


template <class T, int n>
void BezierSurf<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(d1+1,d2+1);
	int dim1 = min(__p.dim1(),_c.dim1());
	int dim2 = min(__p.dim2(),_c.dim2());
	if(_pre_eval) 
	{
		int i=0, j=0;
		findIndex(u,v,i,j); 
		DMatrix<T>& pu = _bpus[i][j];
		DMatrix<T>& pv = _bpvs[i][j];
		for( i=0; i < dim1; i++)
			for( j=0; j< dim2; j++)
				__p[j][i]=pu[i]*(_c^pv[j]);
	}
	 else
	{
		DMatrix<T> pu = bernstainPoly(_c.dim1(), u);	
		DMatrix<T> pv = bernstainPoly(_c.dim2(), v);
		for( int i=0; i < dim1; i++)
			for( int j=0; j < dim2; j++)
				__p[j][i]=pu[i]*(_c^pv[j]);
	}
	for( int i=dim1; i<__p.dim1(); i++)
		for( int j=dim2; j<__p.dim2(); j++)
			__p[j][i]=Vector<T,3>(T(0));

	if(d1>0 && d2>0 ) __n = Vector3D<T>(__p[0][1])^__p[1][0];
}


template <class T, int n>
void BezierSurf<T,n>::localDisplay()
{
//	if(_ctrl_poly) glCallList(__display_list);
	if(_visible) PSurf<T,n>::localDisplay();
}


template <class T, int n>
void BezierSurf<T,n>::toggleSelector()
{
	for (int r=0; r<_c.dim1();r++)
		for (int s=0; s<_c.dim2();s++) _edit_pts[r][s]->onOff();
	_grid->onOff();
	DisplayObject::toggleSelector();
}	

#endif   /* GM_BEZIERSURF_H_ */