
#ifndef _GM_BOHEMIAN_DOME_H_
#define _GM_BOHEMIAN_DOME_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Bohemian_Dome : public PSurf<T,n>
{
	T _a;		//radius
	T _b;		//domewidth
	T _c;		//height

	void _eval(T u, T v, int d1, int d2);
	// void localSimulate(double dt);
protected:

	T _st_p_u()				{ return T(0); }
	T _end_p_u()			{ return T(M_2PI); }
	T _st_p_v()				{ return T(0); }
	T _end_p_v()			{ return T(M_2PI); }

public:
	Bohemian_Dome(T rad=T(5), T domewidth=T(2), T height=T(2));
	Bohemian_Dome(const Bohemian_Dome<T,n>& s);

	virtual  
	~Bohemian_Dome() {}

	void setConstants(T rad, T domewidth, T height);

	const char* identity() const { return " Bohemian_Dome ";}
};

} // GM_lib


using namespace GM_lib;

template <class T, int n>
Bohemian_Dome<T,n>::Bohemian_Dome(T rad, T domewidth, T height):PSurf<T,n>()
{
	setConstants(rad, domewidth, height);
}

template <class T, int n>
Bohemian_Dome<T,n>::Bohemian_Dome(const Bohemian_Dome<T,n>& b):PSurf<T,n>(b)
{
	setConstants(b._a, b._b, b._c);
}

template <class T, int n>
inline
void Bohemian_Dome<T,n>::setConstants(T rad, T domewidth, T height)
{
	_a = rad;
	_b = domewidth;
	_c = height;
}


template <class T, int n>
void Bohemian_Dome<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(4,4);
	__p[0][0][0] =	_a*cos(u);		
	__p[0][0][1] =	_a*sin(u)+_b*cos(v);		
	__p[0][0][2] =	_c*sin(v);		

	if(d1) //u
	{
		__p[1][0][0] =	-_a*sin(u);	
		__p[1][0][1] =	_a*cos(u);	
		__p[1][0][2] =	T(0);	
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	-_a*cos(u);	
		__p[2][0][1] =	-_a*sin(u);	
		__p[2][0][2] =	T(0);
	}
	if(d2) //v
	{
		__p[0][1][0] =	T(0);	
		__p[0][1][1] =	-_b*sin(v);	
		__p[0][1][2] =	 _c*cos(v);	
	}
	if(d2>1) //vv
	{
		__p[0][2][0] =	T(0);	
		__p[0][2][1] =	-_b*cos(v);	
		__p[0][2][2] =	-_c*sin(v);	
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =	T(0);	
		__p[1][1][1] =	T(0);	
		__p[1][1][2] =	T(0);	
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =	T(0);	
		__p[2][1][1] =	T(0);	
		__p[2][1][2] =	T(0);	
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =	T(0);	
		__p[1][2][1] =	T(0);	
		__p[1][2][2] =	T(0);	
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =	T(0);	
		__p[2][2][1] =	T(0);	
		__p[2][2][2] =	T(0);	
	}

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];

}

#endif //_GM_BOHEMIAN_DOME_H_

