
#ifndef _GM_BOTTLE8_H_
#define _GM_BOTTLE8_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Bottle8 : public PSurf<T,n>
{
	T _a;		//radius

	void _eval(T u, T v, int d1, int d2);
	// void localSimulate(double dt);
protected:

	T _st_p_u()				{ return T(0.0); }
	T _end_p_u()			{ return T(M_2PI); }
	T _st_p_v()				{ return T(0.0); }
	T _end_p_v()			{ return T(M_2PI); }

public:
	Bottle8(T radius=T(4));
	Bottle8(const Bottle8<T,n>& s);

	virtual  
	~Bottle8() {}

	void setRadius(T radius);

	const char* identity() const { return " Bottle8 "  ;}
};

} // GM_lib

using namespace GM_lib;

template <class T, int n>
Bottle8<T,n>::Bottle8(T radius):PSurf<T,n>()
{
	_a = radius;
}

template <class T, int n>
Bottle8<T,n>::Bottle8(const Bottle8<T,n>& b):PSurf<T,n>(b)
{
	_a = b._a;
}

template <class T, int n>
void Bottle8<T,n>::setRadius(T radius)
{
	_a = radius;
}

template <class T, int n>
void Bottle8<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(4,4);

	__p[0][0][0] =	(_a+cos(T(.5)*u)*sin(v)-sin(T(.5)*u)*sin(T(2)*v))*cos(u);		
	__p[0][0][1] =	(_a+cos(T(.5)*u)*sin(v)-sin(T(.5)*u)*sin(T(2)*v))*sin(u);		
	__p[0][0][2] =	sin(T(.5)*u)*sin(v)+cos(T(.5)*u)*sin(T(2)*v);		

	if(d1) //u
	{
		__p[1][0][0] =	-cos(u)*sin(T(.5)*u)*T(.5)*sin(v)-cos(u)*cos(T(.5)*u)*T(.5)*sin(T(2)*v)
						-sin(u)*_a-sin(u)*cos(T(.5)*u)*sin(v)+sin(u)*sin(T(.5)*u)*sin(T(2)*v);
		__p[1][0][1] =	-sin(u)*sin(T(.5)*u)*T(.5)*sin(v)-sin(u)*cos(T(.5)*u)*T(.5)*sin(T(2)*v)
						+cos(u)*_a+cos(u)*cos(T(.5)*u)*sin(v)-cos(u)*sin(T(.5)*u)*sin(T(2)*v);
		__p[1][0][2] =	-T(.5)*(-cos(T(.5)*u)*sin(v)+sin(T(.5)*u)*sin(T(2)*v));	
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	-cos(u)*cos(T(.5)*u)*T(.25)*sin(v)+cos(u)*sin(T(.5)*u)*T(.25)*sin(T(2)*v)
						+2.0*sin(u)*sin(T(.5)*u)*T(.5)*sin(v)+2.0*sin(u)*cos(T(.5)*u)*T(.5)*sin(T(2)*v)
						-cos(u)*_a-cos(u)*cos(T(.5)*u)*sin(v)+cos(u)*sin(T(.5)*u)*sin(T(2)*v);
		__p[2][0][1] =	-sin(u)*cos(T(.5)*u)*T(.25)*sin(v)+sin(u)*sin(T(.5)*u)*T(.25)*sin(T(2)*v)
						-2.0*cos(u)*sin(T(.5)*u)*T(.5)*sin(v)-2.0*cos(u)*cos(T(.5)*u)*T(.5)*sin(T(2)*v)
						-sin(u)*_a-sin(u)*cos(T(.5)*u)*sin(v)+sin(u)*sin(T(.5)*u)*sin(T(2)*v);
		__p[2][0][2] =	-T(.25)*(sin(T(.5)*u)*sin(v)+cos(T(.5)*u)*sin(T(2)*v));	
	}
	if(d2) //v
	{
		__p[0][1][0] =	-(-cos(T(.5)*u)*cos(v)+sin(T(.5)*u)*cos(T(2)*v)*T(2))*cos(u);	
		__p[0][1][1] =	-(-cos(T(.5)*u)*cos(v)+sin(T(.5)*u)*cos(T(2)*v)*T(2))*sin(u);	
		__p[0][1][2] =	sin(T(.5)*u)*cos(v)+cos(T(.5)*u)*cos(T(2)*v)*T(2);	
	}
	if(d2>1) //vv
	{
		__p[0][2][0] =	(-cos(T(.5)*u)*sin(v)+sin(T(.5)*u)*sin(T(2)*v)*T(4))*cos(u);
		__p[0][2][1] =	(-cos(T(.5)*u)*sin(v)+sin(T(.5)*u)*sin(T(2)*v)*T(4))*sin(u);
		__p[0][2][2] =	-sin(T(.5)*u)*sin(v)-cos(T(.5)*u)*sin(T(2)*v)*T(4);	
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =  -cos(u)*sin(T(.5)*u)*T(.5)*cos(v)-cos(u)*cos(T(.5)*u)*T(.5)*cos(T(2)*v)*T(2)
						-sin(u)*cos(T(.5)*u)*cos(v)+sin(u)*sin(T(.5)*u)*cos(T(2)*v)*T(2);
		__p[1][1][1] =	-sin(u)*sin(T(.5)*u)*T(.5)*cos(v)-sin(u)*cos(T(.5)*u)*T(.5)*cos(T(2)*v)*T(2)
						+cos(u)*cos(T(.5)*u)*cos(v)-cos(u)*sin(T(.5)*u)*cos(T(2)*v)*T(2);
		__p[1][1][2] =	-T(.5)*(-cos(T(.5)*u)*cos(v)+sin(T(.5)*u)*cos(T(2)*v)*T(2));	
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =	-cos(u)*cos(T(.5)*u)*T(.25)*cos(v)+cos(u)*sin(T(.5)*u)*T(.25)*cos(T(2)*v)*T(2)
						+2.0*sin(u)*sin(T(.5)*u)*T(.5)*cos(v)+2.0*sin(u)*cos(T(.5)*u)*T(.5)*cos(T(2)*v)*T(2)
						-cos(u)*cos(T(.5)*u)*cos(v)+cos(u)*sin(T(.5)*u)*cos(T(2)*v)*T(2);
		__p[2][1][1] =	-sin(u)*cos(T(.5)*u)*T(.25)*cos(v)+sin(u)*sin(T(.5)*u)*T(.25)*cos(T(2)*v)*T(2)
						-2.0*cos(u)*sin(T(.5)*u)*T(.5)*cos(v)-2.0*cos(u)*cos(T(.5)*u)*T(.5)*cos(T(2)*v)*T(2)
						-sin(u)*cos(T(.5)*u)*cos(v)+sin(u)*sin(T(.5)*u)*cos(T(2)*v)*T(2);
		__p[2][1][2] =	-T(.25)*(sin(T(.5)*u)*cos(v)+cos(T(.5)*u)*cos(T(2)*v)*T(2));	
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =	cos(u)*sin(T(.5)*u)*T(.5)*sin(v)+cos(u)*cos(T(.5)*u)*T(.5)*sin(T(2)*v)*T(4)
						+sin(u)*cos(T(.5)*u)*sin(v)-sin(u)*sin(T(.5)*u)*sin(T(2)*v)*T(4);
		__p[1][2][1] =	sin(u)*sin(T(.5)*u)*T(.5)*sin(v)+sin(u)*cos(T(.5)*u)*T(.5)*sin(T(2)*v)*T(4)
						-cos(u)*cos(T(.5)*u)*sin(v)+cos(u)*sin(T(.5)*u)*sin(T(2)*v)*T(4);
		__p[1][2][2] =	T(.5)*(-cos(T(.5)*u)*sin(v)+sin(T(.5)*u)*sin(T(2)*v)*T(4));	
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =	cos(u)*cos(T(.5)*u)*T(.25)*sin(v)-cos(u)*sin(T(.5)*u)*T(.25)*sin(T(2)*v)*T(4)
						-2.0*sin(u)*sin(T(.5)*u)*T(.5)*sin(v)-2.0*sin(u)*cos(T(.5)*u)*T(.5)*sin(T(2)*v)*T(4)
						+cos(u)*cos(T(.5)*u)*sin(v)-cos(u)*sin(T(.5)*u)*sin(T(2)*v)*T(4);
		__p[2][2][1] =	sin(u)*cos(T(.5)*u)*T(.25)*sin(v)-sin(u)*sin(T(.5)*u)*T(.25)*sin(T(2)*v)*T(4)
						+2.0*cos(u)*sin(T(.5)*u)*T(.5)*sin(v)+2.0*cos(u)*cos(T(.5)*u)*T(.5)*sin(T(2)*v)*T(4)
						+sin(u)*cos(T(.5)*u)*sin(v)-sin(u)*sin(T(.5)*u)*sin(T(2)*v)*T(4);
		__p[2][2][2] =	T(.25)*(sin(T(.5)*u)*sin(v)+cos(T(.5)*u)*sin(T(2)*v)*T(4));	
	}

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1]; 
}

#endif 

