
#ifndef _GM_BOURS_SURFACE_H_
#define _GM_BOURS_SURFACE_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Bours_Surface : public PSurf<T,n>
{
	T _sx; //scale x
	T _sy; //scale y
	T _sz; //scale z

	void _eval(T u, T v, int d1, int d2);
	// void localSimulate(double dt);
protected:

	T _st_p_u()				{ return T(-M_PI); }
	T _end_p_u()			{ return T(M_PI); }
	T _st_p_v()				{ return T(-M_PI); }
	T _end_p_v()			{ return T(M_PI); }

public:
	Bours_Surface(T scalex=T(1), T scaley=T(1), T scalez=T(1));
	Bours_Surface(const Bours_Surface<T,n>& s);

	virtual  
	~Bours_Surface() {}

	void setConstants(T scalex, T scaley, T scalez);

	const char* identity() const { return " Bours_Surface "  ;}
};
} // GM_lib

using namespace GM_lib;

template <class T, int n>
Bours_Surface<T,n>::Bours_Surface(T scalex, T scaley, T scalez):PSurf<T,n>()
{
	setConstants(scalex, scaley, scalez);
}

template <class T, int n>
Bours_Surface<T,n>::Bours_Surface(const Bours_Surface<T,n>& b):PSurf<T,n>(b)
{
	setConstants(b._sx, b._sy, b._sz);
}

template <class T, int n>
void Bours_Surface<T,n>::setConstants(T scalex, T scaley, T scalez)
{
	_sx = scalex;
	_sy = scaley;
	_sz = scalez;
}

template <class T, int n>
void Bours_Surface<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(4,4);

	__p[0][0][0] =	_sx*((T(2)*u*u - u*u*u*u - T(2)*v*v + T(6)*v*v*u*u - v*v*v*v)*0.25);
	__p[0][0][1] =	_sy*(u*v*(-T(1) - u*u + v*v ));
	__p[0][0][2] =	_sz*(T(2)/T(3)*(u*u*u - T(3)*u*v*v));

	if(d1) //u
	{
		__p[1][0][0] =	_sx*(u-u*u*u+T(3)*u*v*v);
		__p[1][0][1] =	_sy*(v*(-T(1)-u*u+v*v)-T(2)*u*u*v);
		__p[1][0][2] =	_sz*(T(2)*u*u-T(2)*v*v);
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	_sx*(T(1)-T(3)*u*u+T(3)*v*v);	 
		__p[2][0][1] =	_sy*(-T(6)*u*v);	 
		__p[2][0][2] =	_sz*(T(4)*u);	 
	}
	if(d2) //v
	{
		__p[0][1][0] =	_sx*(-v+T(3)*u*u*v-v*v*v);			 
		__p[0][1][1] =	_sy*(u*(-T(1)-u*u+v*v)+T(2)*u*v*v);	 
		__p[0][1][2] =	_sz*(-T(4)*u*v);						
	}
	if(d2>1) //vv
	{
		__p[0][2][0] =	_sx*(-T(1)+T(3)*u*u-T(3)*v*v);	 
		__p[0][2][1] =	_sy*(T(6)*u*v);	 
		__p[0][2][2] =	_sz*(-T(4)*u);	 
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =	_sx*( T(6)*u*v);				
		__p[1][1][1] =	_sy*(-T(1)-T(3)*u*u+T(3)*v*v);	
		__p[1][1][2] =	_sz*(-T(4)*v);					
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =	T(0);
		__p[2][1][1] =	T(0);
		__p[2][1][2] =	T(0);
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =	_sx*6.0*v;		
		__p[1][2][1] =	_sy*(-6.0*u);		
		__p[1][2][2] =	T(0);	
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =	_sx*T(6);	
		__p[2][2][1] =	T(0);	  
		__p[2][2][2] =	T(0);	  
	} 

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}

#endif //_GM_BOURS_SURFACE_H_

