
#ifndef _GM_BOY_SURFACE_H_
#define _GM_BOY_SURFACE_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class GM_Boy_Surface : public PSurf<T,n>
{

	PSurf<T,n>			*surf;
	T u;
	T v;
	T _a, _b, _c; //local parameters for the function
	
	void _eval(T u, T v, int d1, int d2);
	// void localSimulate(double dt);
protected:

	T _st_p_u()		{ return T(0); }
	T _end_p_u()	{ return T(M_PI); }
	T _st_p_v()		{ return T(0); }
	T _end_p_v()	{ return T(M_PI); }


public:

	GM_Boy_Surface();
	GM_Boy_Surface(const GM_Boy_Surface<T,n>& s);

	~GM_Boy_Surface() {}

	const char* identity() const { return " GM_Boy_Surface "  ;}
};
} // GM_lib

using namespace GM_lib;

template <class T, int n>
GM_Boy_Surface<T,n>::GM_Boy_Surface():PSurf<T,n>()
{
	_a = _b = _c = T(2);
}

template <class T, int n>
GM_Boy_Surface<T,n>::GM_Boy_Surface(const GM_Boy_Surface<T,n>& s):PSurf<T,n>(s)
{
	_a = s._a;
	_b = s._b;
	_c = s._c;
}

template <class T, int n>
void GM_Boy_Surface<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(4,4);

	__p[0][0][0] = cos(u)*(T(M_SQRT2)/T(3)*cos(u)*cos(T(2)*v)+T(2)/T(3)*sin(u)*cos(v))/(T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v));
	__p[0][0][1] = cos(u)*(T(M_SQRT2)/T(3)*cos(u)*sin(T(2)*v)-T(2)/T(3)*sin(u)*sin(v))/(T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v));
	__p[0][0][2] = cos(u)/(T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v))*cos(u)-T(1);

	if(d1)
	{
		__p[1][0][0] = -sin(u)*(T(1)/T(3)*T(M_SQRT2)*cos(u)*cos(T(2)*v)+T(2)/T(3)*sin(u)*cos(v))
					/ (T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v)) 
					+ cos(u)*(-T(1)/T(3)*T(M_SQRT2)*sin(u)*cos(T(2)*v)+ T(2)/T(3)*cos(u)*cos(v))
					/(T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v))
					- cos(u)*(T(1)/T(3)*T(M_SQRT2)*cos(u)*cos(T(2)*v) + T(2)/T(3)*sin(u)*cos(v))
					* (-T(M_SQRT2)*cos(u)*cos(u)*sin(T(3)*v)+T(M_SQRT2)*sin(u)*sin(u)*sin(T(3)*v))
					/ pow((T)(T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v)),T(2));
		__p[1][0][1] = -sin(u)*((T(1)/T(3)*cos(u)*sin(T(2)*v)*T(M_SQRT2) - T(2)/T(3)*sin(u)*sin(v)))
					/ (T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v))
					+ cos(u)*(-(T(1)/T(3)*sin(u)*sin(T(2)*v)*T(M_SQRT2) - T(2)/T(3)*cos(u)*sin(v)))
					/(T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v))
					- cos(u)*(T(1)/T(3)*cos(u)*sin(T(2)*v)*T(M_SQRT2) - T(2)/T(3)*sin(u)*sin(v))
					* (-T(M_SQRT2)*cos(u)*cos(u)*sin(T(3)*v) + T(M_SQRT2)*sin(u)*sin(u)*sin(T(3)*v))
					/ pow((T)(T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v)),T(2));
		__p[1][0][2] = -T(2)*cos(u)*sin(u)/(T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v))
					- (cos(u)*cos(u)*(-T(M_SQRT2)*cos(u)*cos(u)*sin(T(3)*v)	+ T(M_SQRT2)*sin(u)*sin(u)*sin(T(3)*v)))
					/ pow((T)(T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v)),T(2));
	}
	if(d2)
	{
		__p[0][1][0] = cos(u)*(-T(2)/T(3)*cos(u)*sin(T(2)*v)*T(M_SQRT2)-T(2)/T(3)*sin(u)*sin(v))
					/ (T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v))
					+ T(3)*cos(u)*cos(u)*(T(1)/T(3)*T(M_SQRT2)*cos(u)*cos(T(2)*v)
					+ T(2)/T(3)*sin(u)*cos(v))*T(M_SQRT2)*sin(u)*cos(T(3)*v)
					/ pow((T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v)),T(2));
		__p[0][1][1] = cos(u)*(T(2)/T(3)*T(M_SQRT2)*cos(u)*cos(T(2)*v)-T(2)/T(3)*sin(u)*cos(v))
					/ (T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(3*v))
					+ T(3)*cos(u)*cos(u)*(T(1)/T(3)*cos(u)*sin(T(2)*v)*T(M_SQRT2)
					- T(2)/T(3)*sin(u)*sin(v))*T(M_SQRT2)*sin(u)*cos(T(3)*v)
					/ pow((T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v)),T(2));
		__p[0][1][2] = T(3)*cos(u)*cos(u)*cos(u)*T(M_SQRT2)*sin(u)*cos(T(3)*v)
					/ pow((T(1)-T(M_SQRT2)*sin(u)*cos(u)*sin(T(3)*v)),T(2));
	}
	if(d1>1)
	{
		__p[2][0][0] = -cos(u)*(sqrt(T(2.0))*cos(u)*cos(T(2.0)*v)/T(3.0)+T(2.0)/T(3.0)*sin(u)*cos(v))/(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))-T(2.0)*sin(u)*(-sqrt(T(2.0))*sin(u)*cos(T(2.0)*v)/
			T(3.0)+T(2.0)/T(3.0)*cos(u)*cos(v))/(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))+T(2.0)*sin(u)*
			(sqrt(T(2.0))*cos(u)*cos(T(2.0)*v)/T(3.0)+T(2.0)/T(3.0)*sin(u)*cos(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u
			)*cos(u)*sin(T(3.0)*v),T(2.0))*(-sqrt(T(2.0))*pow(cos(u),T(2.0))*sin(T(3.0)*v)+sqrt(T(2.0))*pow(
			sin(u),T(2.0))*sin(T(3.0)*v))+cos(u)*(-sqrt(T(2.0))*cos(u)*cos(T(2.0)*v)/T(3.0)-T(2.0)/T(3.0)*sin(u)*cos(v))/(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))-T(2.0)*cos(u)*(-sqrt(T(2.0))*sin(u)*cos(T(2.0)*v
			)/T(3.0)+T(2.0)/T(3.0)*cos(u)*cos(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),T(2.0))*(-
			sqrt(T(2.0))*pow(cos(u),T(2.0))*sin(T(3.0)*v)+sqrt(T(2.0))*pow(sin(u),T(2.0))*sin(T(3.0)*v))+T(2.0)*
			cos(u)*(sqrt(T(2.0))*cos(u)*cos(T(2.0)*v)/T(3.0)+T(2.0)/T(3.0)*sin(u)*cos(v))/pow(T(1.0)-sqrt(T(2.0)
			)*sin(u)*cos(u)*sin(T(3.0)*v),T(3.0))*pow(-sqrt(T(2.0))*pow(cos(u),T(2.0))*sin(T(3.0)*v)+sqrt(
			T(2.0))*pow(sin(u),T(2.0))*sin(T(3.0)*v),T(2.0))-T(4.0)*pow(cos(u),T(2.0))*(sqrt(T(2.0))*cos(u)*cos(
			T(2.0)*v)/T(3.0)+T(2.0)/T(3.0)*sin(u)*cos(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),
			T(2.0))*sqrt(T(2.0))*sin(u)*sin(T(3.0)*v);
		__p[2][0][1] = -cos(u)*(sqrt(T(2.0))*cos(u)*sin(T(2.0)*v)/T(3.0)-T(2.0)/T(3.0)*sin(u)*sin(v))/(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))-T(2.0)*sin(u)*(-sqrt(T(2.0))*sin(u)*sin(T(2.0)*v)/
			T(3.0)-T(2.0)/T(3.0)*cos(u)*sin(v))/(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))+T(2.0)*sin(u)*
			(sqrt(T(2.0))*cos(u)*sin(T(2.0)*v)/T(3.0)-T(2.0)/T(3.0)*sin(u)*sin(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u
			)*cos(u)*sin(T(3.0)*v),T(2.0))*(-sqrt(T(2.0))*pow(cos(u),T(2.0))*sin(T(3.0)*v)+sqrt(T(2.0))*pow(
			sin(u),T(2.0))*sin(T(3.0)*v))+cos(u)*(-sqrt(T(2.0))*cos(u)*sin(T(2.0)*v)/T(3.0)+T(2.0)/T(3.0)*sin(u)*sin(v))/(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))-T(2.0)*cos(u)*(-sqrt(T(2.0))*sin(u)*sin(T(2.0)*v
			)/T(3.0)-T(2.0)/T(3.0)*cos(u)*sin(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),T(2.0))*(-
			sqrt(T(2.0))*pow(cos(u),T(2.0))*sin(T(3.0)*v)+sqrt(T(2.0))*pow(sin(u),T(2.0))*sin(T(3.0)*v))+T(2.0)*
			cos(u)*(sqrt(T(2.0))*cos(u)*sin(T(2.0)*v)/T(3.0)-T(2.0)/T(3.0)*sin(u)*sin(v))/pow(T(1.0)-sqrt(T(2.0)
			)*sin(u)*cos(u)*sin(T(3.0)*v),T(3.0))*pow(-sqrt(T(2.0))*pow(cos(u),T(2.0))*sin(T(3.0)*v)+sqrt(
			T(2.0))*pow(sin(u),T(2.0))*sin(T(3.0)*v),T(2.0))-T(4.0)*pow(cos(u),T(2.0))*(sqrt(T(2.0))*cos(u)*sin(
			T(2.0)*v)/T(3.0)-T(2.0)/T(3.0)*sin(u)*sin(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),
			T(2.0))*sqrt(T(2.0))*sin(u)*sin(T(3.0)*v);
		__p[2][0][2] = T(2.0)*pow(sin(u),T(2.0))/(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))+T(4.0)*cos(u)/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),T(2.0))*sin(u)*(-sqrt(T(2.0))*pow(cos(
			u),T(2.0))*sin(T(3.0)*v)+sqrt(T(2.0))*pow(sin(u),T(2.0))*sin(T(3.0)*v))-T(2.0)*pow(cos(u),T(2.0))/(
			T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))+T(2.0)*pow(cos(u),T(2.0))/pow(T(1.0)-sqrt(T(2.0))*
			sin(u)*cos(u)*sin(T(3.0)*v),T(3.0))*pow(-sqrt(T(2.0))*pow(cos(u),T(2.0))*sin(T(3.0)*v)+sqrt(
			T(2.0))*pow(sin(u),T(2.0))*sin(T(3.0)*v),T(2.0))-T(4.0)*pow(cos(u),T(3.0))/pow(T(1.0)-sqrt(T(2.0))*sin(
			u)*cos(u)*sin(T(3.0)*v),T(2.0))*sqrt(T(2.0))*sin(u)*sin(T(3.0)*v);
	}
/*	if(d1>1 && d2>1)
	{
		__p[4][0] =  -sin(u)*(-T(2.0)/T(3.0)*sqrt(T(2.0))*cos(u)*sin(T(2.0)*v)-T(2.0)/T(3.0)*sin(u)*sin(v))/(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))-T(3.0)*pow(sin(u),T(2.0))*(sqrt(T(2.0))*cos(u)
			*cos(T(2.0)*v)/T(3.0)+T(2.0)/T(3.0)*sin(u)*cos(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*
			v),T(2.0))*sqrt(T(2.0))*cos(u)*cos(T(3.0)*v)+cos(u)*(T(2.0)/T(3.0)*sqrt(T(2.0))*sin(u)*sin(T(2.0)*v)
			-T(2.0)/T(3.0)*cos(u)*sin(v))/(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))+T(3.0)*pow(cos(u),T(2.0))*(-sqrt(T(2.0))*sin(u)*cos(T(2.0)*v)/T(3.0)+T(2.0)/T(3.0)*cos(u)*cos(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),T(2.0))*sqrt(T(2.0))*sin(u)*
			cos(T(3.0)*v)-cos(u)*(-T(2.0)/T(3.0)*sqrt(T(2.0))*cos(u)*sin(T(2.0)*v)-T(2.0)/T(3.0)*sin(u)*sin(v))/
			pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),T(2.0))*(-sqrt(T(2.0))*pow(cos(u),T(2.0))*sin
			(T(3.0)*v)+sqrt(T(2.0))*pow(sin(u),T(2.0))*sin(T(3.0)*v))-T(6.0)*pow(cos(u),T(2.0))*(sqrt(T(2.0))*
			cos(u)*cos(T(2.0)*v)/T(3.0)+T(2.0)/T(3.0)*sin(u)*cos(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*
			sin(T(3.0)*v),T(3.0))*(-sqrt(T(2.0))*pow(cos(u),T(2.0))*sin(T(3.0)*v)+sqrt(T(2.0))*pow(sin(u),T(2.0)
			)*sin(T(3.0)*v))*sqrt(T(2.0))*sin(u)*cos(T(3.0)*v)-cos(u)*(sqrt(T(2.0))*cos(u)*cos(T(2.0)*v)/
			T(3.0)+T(2.0)/T(3.0)*sin(u)*cos(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),T(2.0))*(
			-T(3.0)*sqrt(T(2.0))*pow(cos(u),T(2.0))*cos(T(3.0)*v)+T(3.0)*sqrt(T(2.0))*pow(sin(u),T(2.0))*cos(T(3.0)
			*v));
		__p[4][1] = -sin(u)*(T(2.0)/T(3.0)*sqrt(T(2.0))*cos(u)*cos(T(2.0)*v)-T(2.0)/T(3.0)*sin(u)*cos(v))/(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))-T(3.0)*pow(sin(u),T(2.0))*(sqrt(T(2.0))*cos(u)*
			sin(T(2.0)*v)/T(3.0)-T(2.0)/T(3.0)*sin(u)*sin(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v
			),T(2.0))*sqrt(T(2.0))*cos(u)*cos(T(3.0)*v)+cos(u)*(-T(2.0)/T(3.0)*sqrt(T(2.0))*sin(u)*cos(T(2.0)*v)
			-T(2.0)/T(3.0)*cos(u)*cos(v))/(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))+T(3.0)*pow(cos(u),T(2.0))*(-sqrt(T(2.0))*sin(u)*sin(T(2.0)*v)/T(3.0)-T(2.0)/T(3.0)*cos(u)*sin(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),T(2.0))*sqrt(T(2.0))*sin(u)*
			cos(T(3.0)*v)-cos(u)*(T(2.0)/T(3.0)*sqrt(T(2.0))*cos(u)*cos(T(2.0)*v)-T(2.0)/T(3.0)*sin(u)*cos(v))/
			pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),T(2.0))*(-sqrt(T(2.0))*pow(cos(u),T(2.0))*sin
			(T(3.0)*v)+sqrt(T(2.0))*pow(sin(u),T(2.0))*sin(T(3.0)*v))-T(6.0)*pow(cos(u),T(2.0))*(sqrt(T(2.0))*
			cos(u)*sin(T(2.0)*v)/T(3.0)-T(2.0)/T(3.0)*sin(u)*sin(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*
			sin(T(3.0)*v),T(3.0))*(-sqrt(T(2.0))*pow(cos(u),T(2.0))*sin(T(3.0)*v)+sqrt(T(2.0))*pow(sin(u),T(2.0)
			)*sin(T(3.0)*v))*sqrt(T(2.0))*sin(u)*cos(T(3.0)*v)-cos(u)*(sqrt(T(2.0))*cos(u)*sin(T(2.0)*v)/
			T(3.0)-T(2.0)/T(3.0)*sin(u)*sin(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),T(2.0))*(
			-T(3.0)*sqrt(T(2.0))*pow(cos(u),T(2.0))*cos(T(3.0)*v)+T(3.0)*sqrt(T(2.0))*pow(sin(u),T(2.0))*cos(T(3.0)
			*v));
		__p[4][2] =  -T(6.0)*pow(cos(u),T(2.0))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),T(2.0))*pow(sin(u),T(2.0))*sqrt(T(2.0))*cos(T(3.0)*v)-T(6.0)*pow(cos(u),T(3.0))/pow(T(1.0)-sqrt(T(2.0))*sin
			(u)*cos(u)*sin(T(3.0)*v),T(3.0))*(-sqrt(T(2.0))*pow(cos(u),T(2.0))*sin(T(3.0)*v)+sqrt(T(2.0))*pow
			(sin(u),T(2.0))*sin(T(3.0)*v))*sqrt(T(2.0))*sin(u)*cos(T(3.0)*v)-pow(cos(u),T(2.0))/pow(T(1.0)-
			sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),T(2.0))*(-T(3.0)*sqrt(T(2.0))*pow(cos(u),T(2.0))*cos(T(3.0)
			*v)+T(3.0)*sqrt(T(2.0))*pow(sin(u),T(2.0))*cos(T(3.0)*v));
	}
	if(d2>1)
	{
		__p[5][0] = cos(u)*(-T(4.0)/T(3.0)*sqrt(T(2.0))*cos(u)*cos(T(2.0)*v)-T(2.0)/T(3.0)*sin(u)*cos(v))/(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))+T(6.0)*pow(cos(u),T(2.0))*(-T(2.0)/T(3.0)*sqrt(T(2.0)
			)*cos(u)*sin(T(2.0)*v)-T(2.0)/T(3.0)*sin(u)*sin(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(
			T(3.0)*v),T(2.0))*sqrt(T(2.0))*sin(u)*cos(T(3.0)*v)+T(36.0)*pow(cos(u),T(3.0))*(sqrt(T(2.0))*cos(u)*
			cos(T(2.0)*v)/T(3.0)+T(2.0)/T(3.0)*sin(u)*cos(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v
			),T(3.0))*pow(sin(u),T(2.0))*pow(cos(T(3.0)*v),T(2.0))-T(9.0)*pow(cos(u),T(2.0))*(sqrt(T(2.0))*cos(u
			)*cos(T(2.0)*v)/T(3.0)+T(2.0)/T(3.0)*sin(u)*cos(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)
			*v),T(2.0))*sqrt(T(2.0))*sin(u)*sin(T(3.0)*v);
		__p[5][1] = cos(u)*(-T(4.0)/T(3.0)*sqrt(T(2.0))*cos(u)*sin(T(2.0)*v)+T(2.0)/T(3.0)*sin(u)*sin(v))/(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v))+T(6.0)*pow(cos(u),T(2.0))*(T(2.0)/T(3.0)*sqrt(T(2.0))
			*cos(u)*cos(T(2.0)*v)-T(2.0)/T(3.0)*sin(u)*cos(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(
			T(3.0)*v),T(2.0))*sqrt(T(2.0))*sin(u)*cos(T(3.0)*v)+T(36.0)*pow(cos(u),T(3.0))*(sqrt(T(2.0))*cos(u)*
			sin(T(2.0)*v)/T(3.0)-T(2.0)/T(3.0)*sin(u)*sin(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v
			),T(3.0))*pow(sin(u),T(2.0))*pow(cos(T(3.0)*v),T(2.0))-T(9.0)*pow(cos(u),T(2.0))*(sqrt(T(2.0))*cos(u
			)*sin(T(2.0)*v)/T(3.0)-T(2.0)/T(3.0)*sin(u)*sin(v))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)
			*v),T(2.0))*sqrt(T(2.0))*sin(u)*sin(T(3.0)*v);
		__p[5][2] = T(36.0)*pow(cos(u),T(4.0))/pow(T(1.0)-sqrt(T(2.0))*sin(u)*cos(u)*sin(T(3.0)*v),T(3.0))*pow(sin(u),T(2.0))*pow(cos(T(3.0)*v),T(2.0))-T(9.0)*pow(cos(u),T(3.0))/pow(T(1.0)-sqrt(T(2.0))*sin(
			u)*cos(u)*sin(T(3.0)*v),T(2.0))*sqrt(T(2.0))*sin(u)*sin(T(3.0)*v);
	}
	if(d1>2) //duuu
	{
		__p[6][0] =  000.0;
		__p[6][1] =  000.0;
		__p[6][2] =  000.0;
	}
	if(d1>2 && d2>2) //duuv
	{
		__p[7][0] =  0.0;
		__p[7][1] =  0.0;
		__p[7][2] =  000.0;

	}
	if(d1>2 && d2>2) //duvv
	{
		__p[8][0] = 0.0;
		__p[8][1] = 0.0;
		__p[8][2] = 000.0;

	}
	if(d2>2)	//dvvv
	{
		__p[9][0] =  000.0;
		__p[9][1] =  000.0;
		__p[9][2] =  000.0;
	}
	if(d1>3)	//duuuu
	{
		__p[10][0] =  000.0;
		__p[10][1] =  000.0;
		__p[10][2] =  000.0;

	}
	if(d1>3 && d2>3)	//duuuv
	{
		__p[11][0] =  000.0;
		__p[11][1] =  000.0;
		__p[11][2] =  000.0;

	}
	if(d1>3 && d2>3)	//duuvv
	{
		__p[12][0] = 0.0;
		__p[12][1] =  0.0;
		__p[12][2] =  000.0;

	}
*/

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}


#endif //_GM_BOY_SURFACE_H_
