//////////////////////////////////////////////////////////////////////
// Camera.cpp: implementation of the Camera class.
//////////////////////////////////////////////////////////////////////

#include <GM_Camera.h>
#include <GM_Material.h>
using namespace GM_lib;

// #include <GL/glut.h>

Scene	Camera::def;
GLuint	Camera::display_list = 0;


GL_Matrix& Camera::matrix()
{ 
	return _matrix.inverse2();
}


void Camera::_reset(GLfloat z)
{
	basisChange(_side, _up, _dir, _pos);	// Must repete because the virtual functions do not work proper jet.
	_ratio		= 1.0;
	_near_plane = 1.0;
	_far_plane	= 1000.0;
	_eye_dist	= 0.08;
	setFocalDist(z*50);
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0,0,0),1.0));
	if (!display_list) makeGraphic();
	coordSys();
	frustrumOn();
	_type_id  = 2;
}


void Camera::_setPerspective()
{
	GLfloat	hh = _near_plane*_angle_tan;
	GLfloat	rr = _ratio*hh;
	_frustum = Frustum(_mat,_pos,_dir,_up,_side,_angle_tan,_ratio,_near_plane,_far_plane);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum(-rr, rr, -hh, hh, _near_plane, _far_plane);
	glMatrixMode(GL_MODELVIEW);
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


Camera::Camera(Scene& s) : LA_DisplayObject(),_scene(&s)
{
	_reset();
}
Camera::Camera(Scene* s) : LA_DisplayObject(),_scene(s)
{
	_reset();
}

Camera::Camera(const Point<GLfloat,3>&  p, const Point<GLfloat,3>&  lp): LA_DisplayObject(lp,p),_scene(&def)
{
	_reset();
}


Camera::Camera(const Point<GLfloat,3>&  p, const Vector<GLfloat,3>& d,
		 const Vector<GLfloat,3>& u, GLfloat z) : LA_DisplayObject(p,d,u),_scene(&def)
{
	_reset(z);
}


Camera::~Camera()
{
}


DisplayObject* Camera::findSelectObject(int x, int y, int type_id)
{	   
	glPushAttrib(GL_ALL_ATTRIB_BITS ); // Save complete state	
    glDisable(GL_COLOR_MATERIAL);glDisable(GL_LIGHTING);glDisable(GL_BLEND); 
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	glShadeModel(GL_FLAT);
	  GL_Color c;
	  _select(type_id);	
	  glReadPixels(x,y,1,1,GL_RGB,GL_UNSIGNED_BYTE,(GLubyte*)(&c));
	glPopAttrib(); // reset state
	return _find(c.get());
}


double Camera::distanceToObject(int x, int y)
{	  
	DisplayObject* 	obj = findSelectObject(x, y);
	return distanceToObject(obj);
}


double Camera::distanceToObject(DisplayObject* obj)
{
	if(obj)	return (_matrix*_mat_inv*obj->centerPos()).length();
	else	return 0.0;
}


void Camera::reshape(int w1, int h1, int w2, int h2)
{
	_x = w1;
	_y = h1;
	_w = w2-w1;
	_h = h2-h1;
//	glViewport(w1,h1,w2,h2);
	_ratio = GLfloat(_w)/GLfloat(_h);
//	_setPerspective();
}


DisplayObject* Camera::lockAtTargetAtPixel(int x, int y)
{ 

	DisplayObject* 	obj = findSelectObject(x, y);
	if(obj)
	{
		lockAtTarget(obj);
		return obj;
	}
	return 0;
}


double Camera::deltaTranslate(LA_DisplayObject * ob)
{
	if(ob)
		return 26*(getDir()*(ob->getPos()-getPos()))*getNearPlane()/(getFocalLength()*viewPortH());
	else
		return 0.0;
}



void Camera::_select(int type_id)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	_setPerspective();
	glViewport(_x,_y,_w,_h);
	glPushMatrix();
		_matrix.mult();
		_mat_inv.mult();
		_scene->culling(_frustum);
		_scene->select(type_id);
	glPopMatrix();
}


void Camera::go(bool stereo)
{
	_aktiv = true;
	if (stereo)
	{
		Point3D<GLfloat>		tmp_pos  = _pos  - _eye_dist*_side;
		UnitVector3D<GLfloat>	tmp_dir  = _dir  + _ed_fd*_side; //tmp_dir  = _pos + _focus_dist*_dir - tmp_pos;
		UnitVector3D<GLfloat>	tmp_side = _side - _ed_fd*_dir;  //tmp_side = _up^tmp_dir;
		basisChange(tmp_side, _up, tmp_dir, tmp_pos);			// Change to right eye
		_display();
		basisChange(_side, _up, _dir, _pos);						// Back to left eye
	}
	else
	{
		if(_keep_lock_at)
		{
			_updateOrientation(_keepLockAt());
			basisChange(_side, _up, _dir, _pos);
		}
		_display();
	}
	_aktiv = false;
}


void Camera::zoom(GLfloat z)
{
	setFocalDist(z*_focal_length);
}



void Camera::localDisplay()
{
	static Material lins(GL_Color(0.19125, 0.0735, 0.0225, 1.0),
						 GL_Color(0.5038, 0.27048, 0.0828, 1.0),
						 GL_Color(0.256777, 0.137622, 0.086014, 1.0), 12.8);
	static Material rubb(GL_Color(0.105882, 0.058824, 0.103725, 1.0),
						 GL_Color(0.427451, 0.470588, 0.501176, 1.0),
						 GL_Color(0.333333, 0.333333, 0.501569, 1.0), 9.84615 );	   
	PolishedSilver.glSet();
	glCallList(display_list);
	glCallList(display_list+2);
	rubb.glSet();
	glCallList(display_list+1);
	glCallList(display_list+3);
	lins.glSet();
	glCallList(display_list+4);
	if(_frustrum_on)
	{
		GL_Color		col(1.0,1.0,1.0,1.0); // hvit
		Point3D<GLfloat> p1(0,0,-_near_plane);
		Point3D<GLfloat> p2(0,0,-_far_plane);
		Vector3D<GLfloat> v1(0,_angle_tan*_near_plane,0);
		Vector3D<GLfloat> v2(-_ratio*_angle_tan*_near_plane,0,0);
		Vector3D<GLfloat> v3(0,_angle_tan*_far_plane,0);
		Vector3D<GLfloat> v4(-_ratio*_angle_tan*_far_plane,0,0);
		Point3D<GLfloat> p1m(p1-v1);
		Point3D<GLfloat> p2m(p2-v3);
		p1 += v1;
		p2 += v3;
		GLboolean lg;
		glGetBooleanv(GL_LIGHTING,&lg);
		if(lg) glDisable(GL_LIGHTING);
		col.glSet();
		glBegin(GL_LINE_LOOP);
			glPoint(p1+v2);	glPoint(p1-v2);	glPoint(p1m-v2);	glPoint(p1m+v2);
		glEnd();
		glBegin(GL_LINE_LOOP);
			glPoint(p2+v4);	glPoint(p2-v4);	glPoint(p2m-v4);	glPoint(p2m+v4);
		glEnd();
		glBegin(GL_LINES);
			glPoint(p1 +v2);	glPoint(p2 +v4);	glPoint(p1 -v2);	glPoint(p2 -v4);
			glPoint(p1m-v2);	glPoint(p2m-v4);	glPoint(p1m+v2);	glPoint(p2m+v4);
		glEnd();
		if(lg) glEnable(GL_LIGHTING);
	}
	if(_keep_lock_at && ! _lock_at_object)
	{
		GLboolean lg;
		glGetBooleanv(GL_LIGHTING,&lg);
		if(lg) glDisable(GL_LIGHTING);
		glPushMatrix();
		glTranslate(_present.inverse2()*_lock_at_pos);
		Green.glSet();
		glCallList(display_list+8);
		glPopMatrix();
		if(lg) glEnable(GL_LIGHTING);
	}
}


void Camera::localSelect()
{
	if(!_aktiv)
	{
		glCallList(display_list+9);
	}
}

void Camera::_drawActivCam()
{
	GLfloat hh = -1.5*_near_plane*_angle_tan;
	Point3D<GLfloat> cp(_ratio*hh, hh, -_near_plane-1.0);
	cp*=_mat;
	cp*=matrix();
	GLboolean lg;
	glGetBooleanv(GL_LIGHTING,&lg);
	if(lg) glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
		Red.glSet();	glPoint(cp); glPoint(cp+Vector3D<GLfloat>(0.1,0,0));
		Green.glSet();	glPoint(cp); glPoint(cp+Vector3D<GLfloat>(0,0.1,0));
		Blue.glSet();	glPoint(cp); glPoint(cp+Vector3D<GLfloat>(0,0,0.1));
	glEnd();
	if(_keep_lock_at && ! _lock_at_object)
	{
		glPushMatrix();
		glTranslate(_lock_at_pos);
		glCallList(display_list+8);
		glPopMatrix();
	}
	if (lg) glEnable(GL_LIGHTING);
}

void Camera::makeGraphic()
{	
	int i;
	Point<GLfloat,3>		p[18];
	UnitVector<GLfloat,3>	v[18];

	p[0] = Point3D<GLfloat>(0.07,0.1,0.3);		p[4] = Point3D<GLfloat>(-0.07,0.1,0.3);
	p[1] = Point3D<GLfloat>(0.07,0.1,-0.3);		p[5] = Point3D<GLfloat>(-0.07,0.1,-0.3);
	p[2] = Point3D<GLfloat>(0.07,-0.1,-0.3);	p[6] = Point3D<GLfloat>(-0.07,-0.1,-0.3);
	p[3] = Point3D<GLfloat>(0.07,-0.1,0.3);		p[7] = Point3D<GLfloat>(-0.07,-0.1,0.3);
	
	v[0] = Vector3D<GLfloat>(1,0,0);
	v[1] = Vector3D<GLfloat>(0,1,0);
	v[2] = Vector3D<GLfloat>(-1,0,0);
	v[3] = Vector3D<GLfloat>(0,-1,0);
	v[4] = Vector3D<GLfloat>(0,0,1);
	v[5] = Vector3D<GLfloat>(0,0,-1);

	display_list = glGenLists(10);

	// Lager displayliste for display
	glNewList(display_list, GL_COMPILE);
		glBegin(GL_QUADS);
			glNormal(v[2]); glPoint(p[0]); glPoint(p[1]); glPoint(p[2]); glPoint(p[3]);	// right
			glNormal(v[1]); glPoint(p[0]); glPoint(p[4]); glPoint(p[5]); glPoint(p[1]); // top
			glNormal(v[0]); glPoint(p[7]); glPoint(p[6]); glPoint(p[5]); glPoint(p[4]); // left
			glNormal(v[3]); glPoint(p[3]); glPoint(p[2]); glPoint(p[6]); glPoint(p[7]); // botton
			glNormal(v[4]); glPoint(p[0]); glPoint(p[3]); glPoint(p[7]); glPoint(p[4]); // front
			glNormal(v[5]); glPoint(p[2]); glPoint(p[1]); glPoint(p[5]); glPoint(p[6]); // behind
		glEnd();
	glEndList();

	glNewList(display_list+9, GL_COMPILE);
		glBegin(GL_QUADS);
			glPoint(p[0]); glPoint(p[1]); glPoint(p[2]); glPoint(p[3]);	// right
			glPoint(p[0]); glPoint(p[4]); glPoint(p[5]); glPoint(p[1]); // top
			glPoint(p[7]); glPoint(p[6]); glPoint(p[5]); glPoint(p[4]); // left
			glPoint(p[3]); glPoint(p[2]); glPoint(p[6]); glPoint(p[7]); // botton
			glPoint(p[0]); glPoint(p[3]); glPoint(p[7]); glPoint(p[4]); // front
			glPoint(p[2]); glPoint(p[1]); glPoint(p[5]); glPoint(p[6]); // behind
		glEnd();
	glEndList();

	p[0] = Point3D<GLfloat>(0.1,0.1,0.1);	p[4] = Point3D<GLfloat>(-0.1,0.1,0.1);
	p[1] = Point3D<GLfloat>(0.1,0.1,-0.1);	p[5] = Point3D<GLfloat>(-0.1,0.1,-0.1);
	p[2] = Point3D<GLfloat>(0.1,-0.1,-0.1);	p[6] = Point3D<GLfloat>(-0.1,-0.1,-0.1);
	p[3] = Point3D<GLfloat>(0.1,-0.1,0.1);	p[7] = Point3D<GLfloat>(-0.1,-0.1,0.1);
	
	glNewList(display_list+8, GL_COMPILE);	//	For lock at Point.
		glBegin(GL_QUADS);
			glNormal(v[2]); glPoint(p[0]); glPoint(p[1]); glPoint(p[2]); glPoint(p[3]);	// right
			glNormal(v[1]); glPoint(p[0]); glPoint(p[4]); glPoint(p[5]); glPoint(p[1]); // top
			glNormal(v[0]); glPoint(p[7]); glPoint(p[6]); glPoint(p[5]); glPoint(p[4]); // left
			glNormal(v[3]); glPoint(p[3]); glPoint(p[2]); glPoint(p[6]); glPoint(p[7]); // botton
			glNormal(v[4]); glPoint(p[0]); glPoint(p[3]); glPoint(p[7]); glPoint(p[4]); // front
			glNormal(v[5]); glPoint(p[2]); glPoint(p[1]); glPoint(p[5]); glPoint(p[6]); // behind
		glEnd();
	glEndList();

	p[0] = Point3D<GLfloat>(0.02,-0.1,-0.02);	p[4] = Point3D<GLfloat>(-0.02,-0.1,-0.02);
	p[1] = Point3D<GLfloat>(0.02,-0.1,-0.08);	p[5] = Point3D<GLfloat>(-0.02,-0.1,-0.08);
	p[2] = Point3D<GLfloat>(0.02,-0.25,-0.06);	p[6] = Point3D<GLfloat>(-0.02,-0.25,-0.06);
	p[3] = Point3D<GLfloat>(0.02,-0.25,0.0);	p[7] = Point3D<GLfloat>(-0.02,-0.25,0.0);
	
	glNewList(display_list+1, GL_COMPILE);	//	For handtak.
		glBegin(GL_QUADS);
			glNormal(v[2]); glPoint(p[0]); glPoint(p[1]); glPoint(p[2]); glPoint(p[3]);	// right
			glNormal(v[1]); glPoint(p[0]); glPoint(p[4]); glPoint(p[5]); glPoint(p[1]); // top
			glNormal(v[0]); glPoint(p[7]); glPoint(p[6]); glPoint(p[5]); glPoint(p[4]); // left
			glNormal(v[3]); glPoint(p[3]); glPoint(p[2]); glPoint(p[6]); glPoint(p[7]); // botton
			glNormal(v[4]); glPoint(p[0]); glPoint(p[3]); glPoint(p[7]); glPoint(p[4]); // front
			glNormal(v[5]); glPoint(p[2]); glPoint(p[1]); glPoint(p[5]); glPoint(p[6]); // behind
		glEnd();
	glEndList();

	p[0] = Point3D<GLfloat>(-0.016,0.12,0.09);	p[4] = Point3D<GLfloat>(-0.054,0.12,0.09);
	p[1] = Point3D<GLfloat>(-0.016,0.12,0.05);	p[5] = Point3D<GLfloat>(-0.054,0.12,0.05);
	p[2] = Point3D<GLfloat>(-0.016,0.1,0.05);	p[6] = Point3D<GLfloat>(-0.054,0.1,0.05);
	p[3] = Point3D<GLfloat>(-0.016,0.1,0.09);	p[7] = Point3D<GLfloat>(-0.054,0.1,0.09);
	
	glNewList(display_list+2, GL_COMPILE);	
		glBegin(GL_QUADS);						//	For sker del 1
			glNormal(v[2]); glPoint(p[0]); glPoint(p[1]); glPoint(p[2]); glPoint(p[3]);	// right
			glNormal(v[0]); glPoint(p[7]); glPoint(p[6]); glPoint(p[5]); glPoint(p[4]); // left
			glNormal(v[4]); glPoint(p[0]); glPoint(p[3]); glPoint(p[7]); glPoint(p[4]); // front
			glNormal(v[5]); glPoint(p[2]); glPoint(p[1]); glPoint(p[5]); glPoint(p[6]); // behind
		glEnd();

		p[0] = Point3D<GLfloat>(-0.015,0.16,0.35);
		p[1] = Point3D<GLfloat>(-0.015,0.16,0.0);
		p[2] = Point3D<GLfloat>(-0.015,0.12,0.0);
		p[3] = Point3D<GLfloat>(-0.015,0.12,0.35);
		p[4] = Point3D<GLfloat>(-0.055,0.16,0.35);
		p[5] = Point3D<GLfloat>(-0.055,0.16,0.0);
		p[6] = Point3D<GLfloat>(-0.055,0.12,0.0);
		p[7] = Point3D<GLfloat>(-0.055,0.12,0.35);

		glBegin(GL_QUADS);		//	For sker del 2
			glNormal(v[2]); glPoint(p[0]); glPoint(p[1]); glPoint(p[2]); glPoint(p[3]);	// right
			glNormal(v[1]); glPoint(p[0]); glPoint(p[4]); glPoint(p[5]); glPoint(p[1]); // top
			glNormal(v[0]); glPoint(p[7]); glPoint(p[6]); glPoint(p[5]); glPoint(p[4]); // left
			glNormal(v[3]); glPoint(p[3]); glPoint(p[2]); glPoint(p[6]); glPoint(p[7]); // botton
			glNormal(v[4]); glPoint(p[0]); glPoint(p[3]); glPoint(p[7]); glPoint(p[4]); // front
			glNormal(v[5]); glPoint(p[2]); glPoint(p[1]); glPoint(p[5]); glPoint(p[6]); // behind
		glEnd();
	glEndList();

	GL_Matrix mat(Angle(M_PI_4),Point3D<GLfloat>(-0.035,0.14,0.0),UnitVector3D<GLfloat>(0,0,-1));
	p[1]  = Point3D<GLfloat>(-0.055,0.16,0.35);		v[1]  = Vector3D<GLfloat>(-1,1,0);
	p[5]  = Point3D<GLfloat>(-0.015,0.16,0.35);		v[5]  = Vector3D<GLfloat>(1,1,0);
	p[9]  = Point3D<GLfloat>(-0.015,0.12,0.35);		v[9]  = Vector3D<GLfloat>(1,-1,0);
	p[13] = Point3D<GLfloat>(-0.055,0.12,0.35);		v[13] = Vector3D<GLfloat>(-1,-1,0);
	p[17] = Point3D<GLfloat>(-0.055,0.16,0.35);		v[17] = Vector3D<GLfloat>(-1,1,0);

	p[3]  = Point3D<GLfloat>(-0.035,0.16,0.35);		v[3]  = Vector3D<GLfloat>(0,1,0);
	p[7]  = Point3D<GLfloat>(-0.015,0.14,0.35);		v[7]  = Vector3D<GLfloat>(1,0,0);
	p[11] = Point3D<GLfloat>(-0.035,0.12,0.35);		v[11] = Vector3D<GLfloat>(0,-1,0);
	p[15] = Point3D<GLfloat>(-0.055,0.14,0.35);		v[15] = Vector3D<GLfloat>(-1,0,0);

	p[0] = p[1] + Vector3D<GLfloat>(-0.01,0.01,0.05); v[0] = v[1] + Vector3D<GLfloat>(0.0,0.0,0.02);

	for(i=2; i<18;i+=2)
	{
		p[i] = mat*p[i-2];
		v[i] = mat*v[i-2];
	}

	glNewList(display_list+3, GL_COMPILE);		
		glBegin(GL_TRIANGLE_STRIP);		//	For sker del 3
			for(i=0; i<18;i++)
			{
				glNormal(v[i]); glPoint(p[i]);
			}
		glEnd();
	glEndList();

	// The lence
	GL_Matrix mat2(Angle(M_PI_4),UnitVector3D<GLfloat>(0,0,1));
	Point3D<GLfloat> cp(0,0,-0.3);
	p[0] = cp + Vector3D<GLfloat>(0.0,0.05,0.0); 
	p[1] = cp + Vector3D<GLfloat>(0.0,0.08,-0.2); 
	v[0] = Vector3D<GLfloat>(0.0,1.0,0.15);

	for(i=2; i<18;i+=2)
	{
		p[i]   = mat2*p[i-2];
		p[i+1] = mat2*p[i-1];
		v[i]   = mat2*v[i-2];
	}

	glNewList(display_list+4, GL_COMPILE);		
		glBegin(GL_TRIANGLE_STRIP);		//	For Linse
			for(i=0; i<18;i+=2)
			{
				glNormal(v[i]); glPoint(p[i]); glPoint(p[i+1]);
			}
		glEnd();
	glEndList();

}

