//////////////////////////////////////////////////////////////////////
// CameraIso.cpp: implementation of the CameraIso class.
//////////////////////////////////////////////////////////////////////

#include <GM_CameraIso.h>
using namespace GM_lib;



void CameraIso::_reset(GLfloat z)
{
	_horisontal	= z;
	_gw  = _gh  = 0.1;
}


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CameraIso::CameraIso(Scene* s) : Camera(s)
{
	_reset();
}

CameraIso::CameraIso(const Point<GLfloat,3>&  p, const Point<GLfloat,3>&  lp): Camera(p,lp)
{
	_reset();
}


CameraIso::CameraIso(const Point<GLfloat,3>&  p, const Vector<GLfloat,3>& d,
		 const Vector<GLfloat,3>& u, GLfloat z) : Camera(p,d,u,z)
{
	_reset(z);
}


CameraIso::~CameraIso()
{
}


void CameraIso::_setPerspective()
{
	GLfloat	hh = _horisontal;
	GLfloat	rr = _ratio*_horisontal;
	_frustum = Frustum(_mat,_pos,_horisontal,_ratio,_dir,_up,_side,_near_plane,_far_plane);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-rr, rr, -hh, hh, _near_plane, _far_plane);
	glMatrixMode(GL_MODELVIEW);
}


void CameraIso::zoom(GLfloat z)
{
	_horisontal *= z;
}

void CameraIso::lockAtTarget(DisplayObject* obj) {}
void CameraIso::lockAtTarget(const Point<GLfloat,3>& p) {}
void CameraIso::lockAtTarget(double d) {}
void CameraIso::localDisplay() {}
void CameraIso::localSelect() {}

DisplayObject* CameraIso::lockAtTargetAtPixel(int x, int y)
{ 
	return 0;
}

double CameraIso::deltaTranslate(LA_DisplayObject * ob)
{
	if(ob) 
		return 2*_horisontal/viewPortH();
	else
		return 0.0;
}

void CameraIso::go(bool)
{
	_aktiv = true;
	_display();
	_aktiv = false;
}

void CameraIso::_drawActivCam()
{
	int i,ih,iw;
	iw = _horisontal/_gw;
	ih = _ratio*_horisontal/_gh;
	Point3D<GLfloat> cp=_pos+(0.99*_near_plane+0.01*_far_plane)*_dir;
	GLboolean lg;
	glGetBooleanv(GL_LIGHTING,&lg);
	if(lg) glDisable(GL_LIGHTING);
	GL_Color gr(120,120,120);
	gr.glSet();
	glBegin(GL_LINES);
		Vector<GLfloat,3> dside = _gw*_side;
		Vector<GLfloat,3> dup   = _gh*_up;
		Vector<GLfloat,3> side  = ih*dside;
		Vector<GLfloat,3> up    = iw*dup;
		for(i=-iw; i<=iw; i++)
		{
			glPoint(cp+double(i)*dup-side); 
			glPoint(cp+double(i)*dup+side);
		}
		for(i=-ih; i<=ih; i++)
		{
			glPoint(cp-up+double(i)*dside); 
			glPoint(cp+up+double(i)*dside);
		}
	glEnd();
	if (lg) glEnable(GL_LIGHTING);
}




