
#ifndef _GM_CONE_H_
#define _GM_CONE_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Cone : public PSurf<T,n>
{
	T _h;
	T _r;
	
	void _eval(T u, T v, int d1, int d2);
	// void localSimulate(double dt);
protected:

	T _st_p_u()		{ return -_h; }
	T _end_p_u()	{ return _h; }
	T _st_p_v()		{ return T(-0.5*M_PI); }
	T _end_p_v()	{ return T(1.5*M_PI); }

	

public:
	Cone(T height=T(3), T radius=T(5));
	Cone(const Cone<T,n>& s);

	~Cone() {}

	void setConstants(T height, T radius);

	const char* identity() const { return " Cone "  ;}
};

} // GM_lib

using namespace GM_lib;

template <class T, int n>
Cone<T,n>::Cone(T height, T radius):PSurf<T,n>()
{
	setConstants(height, radius);
}

template <class T, int n>
Cone<T,n>::Cone(const Cone<T,n>& s):PSurf<T,n>(s)
{
	setConstants(s._h), s._r);
}

template <class T, int n>
void Cone<T,n>::setConstants(T height, T radius)
{
	_h = height;
	_r = radius;
}

template <class T, int n>
void Cone<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(4,4);

	__p[0][0][0] =	(_h-u)*_r*cos(v)/_h;
	__p[0][0][1] =	(_h-u)*_r*sin(v)/_h;
	__p[0][0][2] =	u;

	if(d1) //u
	{
		__p[1][0][0] =	-_r*cos(v)/_h;	
		__p[1][0][1] =	-_r*sin(v)/_h;	
		__p[1][0][2] =	T(1);	
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	T(0);	
		__p[2][0][1] =	T(0);	
		__p[2][0][2] =	T(0);	
	}
	if(d2) //v
	{
		__p[0][1][0] =	-(_h-u)*_r*sin(v)/_h;	
		__p[0][1][1] =	(_h-u)*_r*cos(v)/_h;	
		__p[0][1][2] =	T(0);	
	}
	if(d2>1) //vv
	{
		__p[0][2][0] =	-(_h-u)*_r*cos(v)/_h;	
		__p[0][2][1] =	-(_h-u)*_r*sin(v)/_h;	
		__p[0][2][2] =	T(0);	
	}
	if(d1>1 && d2>1) //uv
	{
		__p[1][1][0] =	_r*sin(v)/_h;	
		__p[1][1][1] =	-_r*cos(v)/_h;	
		__p[1][1][2] =	T(0);	
	}
	if(d1>2)//uuv
	{
		__p[2][1][0] =	T(0);	
		__p[2][1][1] =	T(0);	
		__p[2][1][2] =	T(0);	
	}
	if(d2>2) //uvv
	{
		__p[1][2][0] =	_r*cos(v)/_h;	
		__p[1][2][1] =	_r*sin(v)/_h;	
		__p[1][2][2] =	T(0);	
	}
	if(d1>2 && d2>2) //uuvv
	{
		__p[2][2][0] =	T(0);	
		__p[2][2][1] =	T(0);	
		__p[2][2][2] =	T(0);	
	}

	if(d1>0 && d2>0 && n==3) 
	{
		if(abs(u-_h)<1e-6)	__n = Vector3D<T>(-cos(v), -sin(v), -_r/_h);
		else				__n = Vector3D<T>(__p[1][0])^__p[0][1];
	}
}

#endif //_GM_CONE_H_

