#ifndef _GM_CROSSCAP_H_
#define _GM_CROSSCAP_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Crosscap : public PSurf<T,n>
{

	PSurf<T,n>			*surf;
	T _r1; //radius

	void _eval(T u, T v, int d1, int d2);
	void localDisplay();
	// void localSimulate(double dt);
protected:

	T _st_p_u()		{ return T(0); }
	T _end_p_u()		{ return T(2*M_PI); }
	T _st_p_v()		{ return T(0); }
	T _end_p_v()		{ return T(2*M_PI); }


public:

	Crosscap();
	Crosscap(T radius);
	// Crosscap(Crosscap<T,n>& s);

	~Crosscap() {}

	void setConstants(T radius);

	const char* identity() const { return " Crosscap ";}
};

} // GM_lib

using namespace GM_lib;

template <class T, int n>
Crosscap<T,n>::Crosscap():PSurf<T,n>()
{
	setConstants(T(2));
}

template <class T, int n>
Crosscap<T,n>::Crosscap(T radius):PSurf<T,n>()
{
	setConstants(radius)
}

template <class T, int n>
void Crosscap<T,n>::setConstants(T radius)
{
	_r1 = radius;

	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),4.0));
}

template <class T, int n>
void Crosscap<T,n>::_eval(T u, T v, int d1,int d2)
{

	__p.setDim(4,4);
	__p[0][0][0] =	_r1*_r1*cos(v)*sin(u)*sin(v);
	__p[0][0][1] =	_r1*_r1*cos(v)*cos(v)*sin(2*u);
	__p[0][0][2] =	_r1*_r1*cos(2*u)*cos(v)*cos(v);

	if(d1) //u
	{
		__p[1][0][0] =	_r1*_r1*cos(v)*cos(u)*sin(v);
		__p[1][0][1] =	2.0*_r1*_r1*cos(v)*cos(v)*cos(2*u);
		__p[1][0][2] =	-2.0*_r1*_r1*cos(v)*cos(v)*sin(2*u);
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	 -_r1*_r1*cos(v)*sin(u)*sin(v);			
		__p[2][0][1] =	 -4.0*_r1*_r1*cos(v)*cos(v)*sin(2.0*u);	
		__p[2][0][2] =	-4.0*_r1*_r1*cos(2.0*u)*cos(v)*cos(v);	
	}
	if(d2) //v
	{
		__p[0][1][0] =	-_r1*_r1*sin(v)*sin(v)*sin(u)+_r1*_r1*cos(v)*cos(v)*sin(u);			
		__p[0][1][1] =	-2.0*_r1*_r1*cos(v)*sin(2*u)*sin(v);			
		__p[0][1][2] =	-2.0*_r1*_r1*cos(v)*cos(2*u)*sin(v);			
	}
	if(d2>1) //vv	
	{
		__p[0][2][0] =	 -4.0*_r1*_r1*cos(v)*sin(u)*sin(v);												 
		__p[0][2][1] =	2.0*_r1*_r1*sin(v)*sin(v)*sin(2.0*u)-2.0*_r1*_r1*cos(v)*cos(v)*sin(2.0*u);		 
		__p[0][2][2] =	2.0*_r1*_r1*cos(2.0*u)*sin(v)*sin(v)-2.0*_r1*_r1*cos(2.0*u)*cos(v)*cos(v);		 
	}
	if(d1 && d2) //uv			
	{
		__p[1][1][0] =	-_r1*_r1*sin(v)*sin(v)*cos(u)+_r1*_r1*cos(v)*cos(v)*cos(u);			
		__p[1][1][1] =	 -4.0*_r1*_r1*cos(2.0*u)*cos(v)*sin(v);			
		__p[1][1][2] =	 4.0*_r1*_r1*cos(v)*sin(2.0*u)*sin(v);			
	}
	if(d1>1 && d2)//uuv	
	{
		__p[2][1][0] =	 _r1*_r1*sin(v)*sin(v)*sin(u)-_r1*_r1*cos(v)*cos(v)*sin(u);
		__p[2][1][1] =	  8.0*_r1*_r1*cos(v)*sin(2.0*u)*sin(v);
		__p[2][1][2] =	 8.0*_r1*_r1*cos(2.0*u)*cos(v)*sin(v);
	}
	if(d1 && d2>1) //uvv	
	{
		__p[1][2][0] =	-4.0*_r1*_r1*cos(v)*cos(u)*sin(v);			
		__p[1][2][1] =	 4.0*_r1*_r1*cos(2.0*u)*sin(v)*sin(v)-4.0*_r1*_r1*cos(2.0*u)*cos(v)*cos(v);			
		__p[1][2][2] =	-4.0*_r1*_r1*sin(v)*sin(v)*sin(2.0*u)+4.0*_r1*_r1*cos(v)*cos(v)*sin(2.0*u);			
	}
	if(d1>1 && d2>1) //uuvv
	{					
		__p[2][2][0] =	 4.0*_r1*_r1*cos(v)*sin(u)*sin(v);	
		__p[2][2][1] =	 -8.0*_r1*_r1*sin(v)*sin(v)*sin(2.0*u)+8.0*_r1*_r1*cos(v)*cos(v)*sin(2.0*u);			
		__p[2][2][2] =	 -8.0*_r1*_r1*cos(2.0*u)*sin(v)*sin(v)+8.0*_r1*_r1*cos(2.0*u)*cos(v)*cos(v);			
	}								

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];

}
template <class T, int n>
void Crosscap<T,n>::localDisplay()
{
	// r_mat.mult();
	PSurf<T,n>::localDisplay();
}
 #endif //_GM_CROSSCAP_H_
