
#ifndef _GM_CYLINDER_H_
#define _GM_CYLINDER_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Cylinder : public PSurf<T,n>
{
	T _rx;			//Radius in x direction
	T _ry;			//Radius in y direction
	T _h;			//Height

	void _eval(T u, T v, int d1, int d2);
	void localDisplay();
	// void localSimulate(double dt);
protected:

	T _st_p_u()				{ return T(-0.5); }
	T _end_p_u()			{ return T(0.5); }
	T _st_p_v()				{ return T(-0.5*M_PI); }
	T _end_p_v()			{ return T(1.5*M_PI); }

public:
	Cylinder();
	Cylinder(T radx, T rady, T height);
	//Cylinder(Cylinder<T,n>& s);

	~Cylinder() {}

	void setConstants(T radx, T rady, T height);

	const char* identity() const { return " Cylinder ";}
};

} // GM_lib

using namespace GM_lib;
template <class T, int n>
Cylinder<T,n>::Cylinder():PSurf<T,n>()
{
	setConstants(T(4), T(4), T(2));
}

template <class T, int n>
Cylinder<T,n>::Cylinder(T radx, T rady, T height):PSurf<T,n>()
{
	setConstants(radx, rady, height);
}

template <class T, int n>
void Cylinder<T,n>::setConstants(T radx, T rady, T height)
{
	_rx = radx;
	_ry = rady;
	_h = height;

	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),10.0));
}

template <class T, int n>
void Cylinder<T,n>::_eval(T u, T v, int d1,int d2)
{

    __p.setDim(4,4);
	__p[0][0][0] =		_rx*cos(v);
	__p[0][0][1] =		_ry*sin(v);
	__p[0][0][2] =		_h*u;

	if(d1) //u
	{
		__p[1][0][0] =	T(0);	
		__p[1][0][1] =	T(0);
		__p[1][0][2] =	_h;	
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	T(0);	
		__p[2][0][1] =	T(0);	
		__p[2][0][2] =	T(0);	
	}
	if(d2) //v
	{
		__p[0][1][0] = -_rx*sin(v);		
		__p[0][1][1] = _ry*cos(v);		
		__p[0][1][2] =	T(0);	
	}
	if(d2>1) //vv
	{
		__p[0][2][0] = -_rx*cos(v);	
		__p[0][2][1] =	-_ry*sin(v);	
		__p[0][2][2] =	T(0);	
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =	T(0);	
		__p[1][1][1] =	T(0);	
		__p[1][1][2] =	T(0);	
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] = T(0);		
		__p[2][1][1] = T(0);		
		__p[2][1][2] = T(0);		
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =	T(0);	
		__p[1][2][1] =	T(0);	
		__p[1][2][2] =	T(0);	
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =	T(0);	
		__p[2][2][1] =	T(0);	
		__p[2][2][2] =	T(0);	
	}

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];

}
template <class T, int n>
void Cylinder<T,n>::localDisplay()
{
	// r_mat.mult();
	PSurf<T,n>::localDisplay();
}
 #endif //_GM_CYLINDER_H_

