#include "GM_DArrayN.h"

namespace GM_lib
{

/**************************************************************************
************** DEFAULT CONSTRUCTOR ****************************************
**************************************************************************/
template<class T, int n>
DArrayN<T,n>::DArrayN<T,n>()
{
	array = NULL;
	no_elem = 0;
}

//Stopping Criteria
template<class T>
DArrayN<T,1>::DArrayN<T,1>()
{
	array = NULL;
	no_elem = 0;

}


/**************************************************************************
************** STANDARD CONSTRUCTOR ***************************************
**************************************************************************/
template<class T, int n>
DArrayN<T,n>::DArrayN<T,n>(const DVector<int> size)
{
	array = NULL;
	no_elem = 0;
	setSize(size);
}

//Stopping Criteria
template<class T>
DArrayN<T,1>::DArrayN<T,1>(const DVector<int> size)
{
	array = NULL;
	no_elem = 0;
	setSize(size);
}


/**************************************************************************
************** STANDARD CONSTRUCTOR ***************************************
**************************************************************************/
template<class T, int n>
DArrayN<T,n>::DArrayN<T,n>(const DVector<int> size, T val)
{
	array = NULL;
	no_elem = 0;
	setSize(size);
	setElements(val);
}

//Stopping Criteria
template<class T>
DArrayN<T,1>::DArrayN<T,1>(const DVector<int> size, T val)
{
	array = NULL;
	no_elem = 0;
	setSize(size);
	setElements(val);
}


/**************************************************************************
************** COPY CONSTRUCTOR *******************************************
**************************************************************************/
template<class T, int n>
DArrayN<T,n>::DArrayN<T,n>(const DArrayN<T,n> &arr)
{
	array = NULL;
	copy(arr);
}

//Stopping Criteria
template<class T>
DArrayN<T,1>::DArrayN<T,1>(const DArrayN<T,1> &arr)
{
	array = NULL;
	copy(arr);
}


/**************************************************************************
************** DESTRUCTOR		   ****************************************
**************************************************************************/
template<class T, int n>
DArrayN<T,n>::~DArrayN<T,n>()
{
	if(array) delete [] array;
	array = NULL;
}

//Stopping Criteria
template<class T>
DArrayN<T,1>::~DArrayN<T,1>()
{
	if(array) delete [] array;
	array = NULL;
}


/***************************************************************************
****																	****
****				 PRIVATE FUNCTIONS									****
****																	****
***************************************************************************/

/**************************************************************************
********************** copy(const DArrayN<T,n> &arr) ****************************
***************** make a copy of the DArrayN ******************************
**************************************************************************/
template<class T, int n>
void DArrayN<T,n>::copy(const DArrayN<T,n> &arr)
{
	if(array) delete [] array;
	if(arr.no_elem>0) 
	{ 
		array = new DArrayN<T,n-1> [arr.no_elem];
		for(int i=0; i<arr.no_elem; i++) array[i] = arr.array[i];
	}
	else
		array = NULL;
	no_elem = arr.no_elem;
}

//Stopping criteria
template<class T>
void DArrayN<T,1>::copy(const DArrayN<T,1> &arr)
{
	if(array) delete [] array;
	if(arr.no_elem>0) 
	{ 
		array = new T[arr.no_elem];
		for(int i=0; i<arr.no_elem; i++) array[i] = arr.array[i];
	}
	else
		array = NULL;
	no_elem = arr.no_elem;
}




/***************************************************************************
****																	****
****				 PUBLIC FUNCTIONS									****
****																	****
***************************************************************************/

/**************************************************************************
********************** SETSIZE(DVector<int> size) *************************
***************** setting the size in every direction   ******************
**************************************************************************/
template<class T, int n>
void DArrayN<T,n>::setSize(DVector<int> size)
{
	if(array) delete [] array;
	no_elem = size[0];
	if(no_elem>0)	array = new DArrayN<T,n-1> [no_elem];
	else			array = NULL;
	
	DVector<int> newsize;
	newsize = size.getSubVector(1,n);
	for(int s=0; s<no_elem; s++) array[s].setSize(newsize);
}

//Stopping Criteria
template<class T>
void DArrayN<T,1>::setSize(DVector<int> size)
{
	if(array) delete [] array;
	no_elem = size[0];
	if(no_elem>0)	array = new T[no_elem];
	else			array = NULL;
}



/**************************************************************************
********************** SETSIZE(DVector<int> size, T val) ******************
** setting the size in every dir and setting the value for all elements  **
**************************************************************************/
template<class T, int n>
void DArrayN<T,n>::setSize(DVector<int> size, T val)
{
	array = NULL;
	no_elem = 0;
	setSize(size);
	setElements(val);
}

//Stopping Criteria
template<class T>
void DArrayN<T,1>::setSize(DVector<int> size, T val)
{
	array = NULL;
	no_elem = 0;
	setSize(size);
	setElements(val);
}


/**************************************************************************
********************** GETSIZE(DVector<int> &size) ************************
***************** returning the size in every direction  ******************
**************************************************************************/
template<class T, int n>
void DArrayN<T,n>::getSize(DVector<int> &size) const
{
	//size.append(1,no_elem);
	size.append(no_elem,1); //With the new DVector class
	array[0].getSize(size);
}

//Stopping Criteria
template<class T>
void DArrayN<T,1>::getSize(DVector<int> &size) const
{
	//size.append(1,no_elem);
	size.append(no_elem,1); //With the new DVector class
}


/**************************************************************************
********************** SETELEMENTS(T val=0) *******************************
***************** set all the elements equal val **************************
**************************************************************************/
template<class T, int n>
void DArrayN<T,n>::setElements(T val)
{
	for(int i=0; i<no_elem; i++) array[i].setElements(val);
}

//Stopping Criteria
template<class T>
void DArrayN<T,1>::setElements(T val)
{
	for(int i=0; i<no_elem; i++) array[i]=val;
}


/**************************************************************************
********************** GETMINMAX(T &min, T &max) **************************
***************** returning the min and max value in the DArrayN **********
**************************************************************************/
template<class T, int n>
void DArrayN<T,n>::getMinMax(T &_min, T &_max) const
{
	_min = 999999999;
	_max = -999999999;

	for(int i=0; i<no_elem; i++)
	{
		T tmpmin;
		T tmpmax;
		array[i].getMinMax(tmpmin,tmpmax);

		_min = min(tmpmin, _min);
		_max = max(tmpmax, _max);
	}
}

//Stopping Criteria
template<class T>
void DArrayN<T,1>::getMinMax(T &_min, T &_max) const
{
	_min = 999999999;
	_max = -999999999;

	for(int i=0; i<no_elem; i++)
	{
		_min = min(array[i], _min);
		_max = max(array[i], _max);
	}

}

/**************************************************************************
********************** SETELEMENT(T val, DVector<int> pos) ****************
***************** Used to set the element inside the class ****************
**************************************************************************/
template<class T, int n>
inline
void DArrayN<T,n>::setElement(T val, DVector<int> pos)
{
	DVector<int> newpos;
	newpos = pos.getSubVector(1,n);
	this->array[pos[0]].setElement(val,newpos);
}

//Stopping Criteria
template<class T>
inline
void DArrayN<T,1>::setElement(T val, DVector<int> pos)
{
	this->array[pos[0]] = val;
}


/**************************************************************************
********************** GETELEMENT(DVector<int> pos) ***********************
***************** Used to get the element value inside the class **********
**************************************************************************/
template<class T, int n>
inline
T& DArrayN<T,n>::getElement(DVector<int> pos)
{
	static T ret;
	DVector<int> newpos;
	newpos = pos.getSubVector(1,n);
	ret = this->array[pos[0]].getElement(newpos);
	return ret;
}

//Stopping Criteria
template<class T>
inline
T& DArrayN<T,1>::getElement(DVector<int> pos)
{
	static T ret;
	ret = this->array[pos[0]];
	return ret;
}


/**************************************************************************
********************** GETINDEXES(Array<DVector<int>*> &indexes) **********
***************** Returning an array of all posbile combinations **********
***************** of indexes.									 **********
**************************************************************************/
template<class T, int n>
void DArrayN<T,n>::getIndexes(Array<DVector<int>* > &indexes)
{
	DVector<int> mysize;
	this->getSize(mysize);

	int size1 = 1;	//Total size of the array
	int size2 = 1;	//Size og the next level

	for(int i=0; i<n; i++) size1 = size1*mysize[i];
	for(int j=1; j<n; j++) size2 = size2*mysize[j];

	//geting the indexes for next level
	Array<DVector<int>* >	tmpindexes;
	DArrayN<T,n-1>			tmparr;
	DVector<int>			tmpsize;
	tmpsize = mysize.getSubVector(1,n);
	tmparr.setSize(tmpsize);
	tmparr.getIndexes(tmpindexes);

	//seting size of the returning array
	indexes.setSize(size1, NULL);

	int mysize0 = mysize[0];

	for(i=0; i<mysize0; i++) {
		for( j=0; j<size2; j++) {
			DVector<int> tmpvec;
			tmpvec.setDim(1);
			tmpvec[0] = i;
			tmpvec.append(*tmpindexes[j]);

			if(indexes[j+size2*i]) delete indexes[j+size2*i];
			indexes[j+size2*i] = new DVector<int> (tmpvec);
		}
	}
}

//Stopping Criteria
template<class T>
void DArrayN<T,1>::getIndexes(Array<DVector<int>* > &indexes)
{
	indexes.setSize(no_elem, NULL);

	for(int i=0; i<no_elem; i++) {
		DVector<int> tmpvec;
		tmpvec.setDim(1);
		tmpvec[0] = i;
		
		if(indexes[i]) delete indexes[i];
		indexes[i] = new DVector<int> (tmpvec);
	}
}

/**************************************************************************
******************* GETFLATENARRAY(Array<ArrayElem<T,n> > & ret) **********
**** Return a flaten n-dim array. Each element have their respective    ***
**** indexes, sum of the indexes and their value (ArrayElem)			***
**************************************************************************/
template<class T, int n>
void DArrayN<T,n>::getFlatenArray(Array<ArrayElem<T,n> > & ret)
{
	DVector<int> mysize;
	this->getSize(mysize);

	int totsize = 1;
	for(int i=0; i<n; i++)	totsize = totsize*mysize[i];
	ret.setSize(totsize);

	Array<DVector<int>*> indexes;
	this->getIndexes(indexes);

	for(i=0; i<totsize; i++)
	{
		DVector<int> tmpvec = (*indexes[i]);
		int tmpsum = 0;
		for(int j=0; j<n; j++)
		{
			ret[i].index[j] = tmpvec[j];
			tmpsum += tmpvec[j];
		}
		ret[i].sumindex = tmpsum;
		ret[i].val = this->getElement(tmpvec);
	}
}

//Stopping Criteria (Not needed here, but is faster)
template<class T>
void DArrayN<T,1>::getFlatenArray(Array<ArrayElem<T,1> > & ret)
{
	ret.setSize(no_elem);

	for(int i=0; i<no_elem; i++)
	{
		ret[i].index[0] = i;
		ret[i].sumindex = i;
		ret[i].val = this->array[i];
	}

}

/**************************************************************************
*** EXTRUDEARRAY<Array<ArrayElem<T,1> >* flatarr, DVector<int> orgsize) ***
*** Build an DArrayN of size orgsize from an array of arrayelements	    ***
**************************************************************************/
template<class T, int n>
void DArrayN<T,n>::extrudeArray(Array<ArrayElem<T,n> >* flatarr, DVector<int> orgsize)
{
	this->setSize(orgsize);
	DVector<int> tmppos;
	for(int i=0; i<flatarr->size(); i++)
	{
		tmppos = (*flatarr)[i].getIndexes();
		this->setElement((*flatarr)[i].val, tmppos);
	}
}

//Stopping Criteria (Not needed here, but is faster)
template<class T>
void DArrayN<T,1>::extrudeArray(Array<ArrayElem<T,1> >* flatarr, DVector<int> orgsize)
{
	setSize(orgsize);
	for(int i=0; i<flatarr->size(); i++)
	{
		this->array[i] = (*flatarr)[i].val;
	}
}

/**************************************************************************
************* GETSUBARRAY(DVector<int> start, DVector<int> end) ***********
************ Returns the subArray from index start to index end, **********
************ including both index start and index end.			 **********
**************************************************************************/
template<class T, int n>
DArrayN<T,n>& DArrayN<T,n>::getSubArray(DVector<int> start, DVector<int> end)
{
	static DArrayN<T,n> ret;
	DVector<int> retsize;
	retsize.setDim(n);

	if(start[0]<0) start[0] = 0;
	if(end[0]>no_elem) end[0] = no_elem-1;
	for(int i=0; i<n; i++) retsize[i] = (end[i]+1)-start[i];//including end
	ret.setSize(retsize);

	DVector<int> newstart;
	DVector<int> newend;
	newstart = start.getSubVector(1,n);
	newend = end.getSubVector(1,n);

	int start0 = start[0];
	int stopp = end[0]+1;

	DArrayN<T,n-1> tmparr;
	for(i=start0; i<stopp; i++) 
	{
		tmparr = this->array[i];
		ret.array[i-start0] = tmparr.getSubArray(newstart, newend);
	}
	return ret;
}

//Stopping Criteria
template<class T>
DArrayN<T,1>& DArrayN<T,1>::getSubArray(DVector<int> start, DVector<int> end)
{
	static DArrayN<T,1> ret;
	
	if(start[0]<0) start[0] = 0;
	if(end[0]>no_elem) end[0] = no_elem;
	else	end[0] = end[0]+1;

	ret.setSize(end-start);

	int start0 = start[0];
	int stopp = end[0];
		
	for(int i=start0; i<stopp && i<no_elem; i++) 
	{
		ret.array[i-start0] = array[i];
	}

	return ret;
}


/**************************************************************************
********** INSERTSUBARRAY(DArrayN<T,n> &arr,DVector<int> start) ***********
********* Insert an array into a already existing array, if the ***********
********* size of the insering array will exceed the size of the **********
********* existing array, the overshooting elements will be removed *******
**************************************************************************/
template<class T, int n>
void DArrayN<T,n>::insertSubArray(DArrayN<T,n> &arr,DVector<int> start)
{
	DVector<int> newstart;
	newstart = start.getSubVector(1,n);

	int start0 = start[0];
	DArrayN<T,n-1> tmparr;
	for(int i=start0; i<no_elem && arr.no_elem>i-start0; i++)
	{
		tmparr = arr.array[i-start0];
		array[i].insertSubArray(tmparr, newstart);
	}
}

//Stopping Criteria
template<class T>
void DArrayN<T,1>::insertSubArray(DArrayN<T,1> &arr,DVector<int> start)
{
	int start0 = start[0];
	for(int i=start0; i<no_elem && arr.no_elem>i-start0; i++)
	{
		array[i] = arr[i-start0];
	}
}


/**************************************************************************
********** DYADDOWN(int dir, DVector<int> shift, bool ODDindx=false) ******
********* Dyadic downsamling in the given direction dir. Shift the  *******
********* downsamling to the left with the given number in DVector ********
********* shift. If ODDindx is false, the EVEN numbers are removed, *******
********* if true; ODD indexes are removed. Indexation starting with 0 ****
**************************************************************************/
template<class T, int n>
DArrayN<T,n>& DArrayN<T,n>::dyaddown(int dir, DVector<int> shift, bool ODDindx=false)
{
	static DArrayN<T,n> ret;

	DVector<int> mysize;
	this->getSize(mysize);

	int t1=0;
	if(dir == n)
		{
			if(ODDindx) mysize[0] = (int) ceil((double) mysize[0]/2);
			else		mysize[0] = (int) floor((double)mysize[0]/2);
			ret.setSize(mysize);

			int mysize0 = mysize[0];
			int shift0 = shift[0];
			for(int i=0; i<mysize0; i++)
			{
				if(ODDindx) t1 = 2*i-shift0;
				else		t1 = 2*i+1-shift0;
				ret.array[i] = this->array[t1];
			}//end for
		}//end if
		else
		{
			ret.setSize(mysize);
			DVector<int> newshift;
			newshift = shift.getSubVector(1,n);
			DArrayN<T,n-1> tmparr;

			int mysize0 = mysize[0];
			for(int i=0; i<mysize0; i++)
			{
				tmparr = this->array[i];
				ret.array[i] = tmparr.dyaddown(dir,newshift,ODDindx);
			}//end for

		}//end else

		return ret;
}

//Stopping Criteria
template<class T>
DArrayN<T,1>& DArrayN<T,1>::dyaddown(int dir, DVector<int> shift, bool ODDindx=false)
{
	static DArrayN<T,1> ret;

	DVector<int> mysize;
	this->getSize(mysize);
	
	if(ODDindx) mysize[0] = (int) ceil((double) mysize[0]/2);
	else		mysize[0] = (int) floor((double)mysize[0]/2);

	ret.setSize(mysize);

	int mysize0 = mysize[0];
	int shift0 = shift[0];
	int t1=0;
	for(int i=0; i<mysize0; i++)
	{
		if(ODDindx) t1 = 2*i-shift0;
		else		t1 = 2*i+1-shift0;
		ret.array[i] = this->array[t1];
	}//end for

	return ret;
}


/**************************************************************************
********** DYADUP(int dir, DVector<int> shift, bool ODDindx=false) ********
********* Dyadic upsamling in the given direction dir. Shift the  *********
********* upsamling to the left with the given number in DVector **********
********* shift. If ODDindx is false, inserting 0 on all the ODD **********
********* indexes, if true; inserting 0 on all the EVEN indexes.	*******
********* Indexation starting with 0.								*******
**************************************************************************/
template<class T, int n>
DArrayN<T,n>& DArrayN<T,n>::dyadup(int dir, DVector<int> shift, bool ODDindx=false)
{
	static DArrayN<T,n> ret;

	DVector<int> mysize;
	this->getSize(mysize);

	int s=0;
	if(ODDindx) s = 1;
	else		s = -1;

	int t1=0;
	if(dir == n)
		{
			mysize[0] = mysize[0]*2+s;
			ret.setSize(mysize);
			ret.setElements();
			int shift0 = shift[0];
			for(int i=0; i<no_elem; i++)
			{
				if(ODDindx) t1 = 2*i+1-shift0;
				else		t1 = 2*i-shift0;
				ret.array[t1] = this->array[i];
			}
		}
		else
		{
			ret.setSize(mysize);
			DVector<int> newshift;
			newshift = shift.getSubVector(1,n);
			DArrayN<T,n-1> tmparr;
			int mysize0 = mysize[0];
			for(int i=0; i<mysize0; i++)
			{
				tmparr = this->array[i];
				ret.array[i] = tmparr.dyadup(dir, newshift, ODDindx);
			}
		}
	
		return ret;
}

//Stopping Criteria
template<class T>
DArrayN<T,1>& DArrayN<T,1>::dyadup(int dir, DVector<int> shift, bool ODDindx=false)
{
	static DArrayN<T,1> ret;

	DVector<int> mysize;
	this->getSize(mysize);

	int s=0;
	if(ODDindx) s = 1;
	else		s = -1;

	int t1=0;
	mysize[0] = mysize[0]*2+s;
	ret.setSize(mysize);
	ret.setElements();
	int shift0 = shift[0];
	for(int i=0; i<no_elem; i++)
	{
		if(ODDindx) t1 = 2*i+1-shift0;
		else		t1 = 2*i-shift0;
		ret.array[t1] = this->array[i];
	}

	return ret;
}


/**************************************************************************
********************** CONV(DVector<T> F, int dir) ************************
****** Convolv the array with a filter F in the given direction dir. ******
**************************************************************************/
template<class T, int n>
DArrayN<T,n>& DArrayN<T,n>::conv(DVector<T> F, int dir)
{
	static DArrayN<T,n> ret;
	DVector<int> mysize;
	this->getSize(mysize);

	if(dir == n)
	{
		//seting new size of ret
		int Fdim = F.dim();
		DVector<int> newsize;
		newsize = mysize;
		newsize[0] = mysize[0]+Fdim-1;
		ret.setSize(newsize);

		int newsize0 = newsize[0];
		int mysize0 = mysize[0];

		//generating the indexes
		DArrayN<T,n-1>	tmparr;
		DVector<int>	tmpsize;
		tmpsize = mysize.getSubVector(1,n);
		tmparr.setSize(tmpsize);

		Array<DVector<int>*> tmpindex;
		tmparr.getIndexes(tmpindex);
		int nrvec = tmpindex.size();

		

		//Doing the conv for all indexes
		DVector<int>	start;
		DVector<int>	end;
		DVector<int>	epos1(n);
		//epos1.setDim(n);
		epos1.clear();
		
		DVector<int>	epos2;
		T				tmpval;

		DVector<int>	size1d(1);
		size1d[0] = mysize[0];
		DArrayN<T,1>		tmp1Darray(size1d);

		for(int i=0; i<nrvec; i++)
		{
		//	DArrayN<T,n> tmparray;
			start.setDim(1);
			start[0] = 0;
			start.append(*tmpindex[i]);

		//	end.setDim(1);
		//	end[0] = mysize[0]-1; //-1 because getSubArray includes end
		//	end.append(*tmpindex[i]);

		//	tmparray = this->getSubArray(start,end);
			for(int p=0; p<mysize0; p++)
			{
			//	epos1[0] = p;
				start[0] = p;
				//tmp1Darray[p] = tmparray.getElement(epos1);
				tmp1Darray[p] = this->getElement(start);
			}

	//		epos2.setDim(1);
	//		epos2[0] = 0;
	//		epos2.append(*tmpindex[i]);

			start[0] = 0;

           // T s; 
			int t;
			for(int k=0; k<newsize0; k++) {
				tmpval = T(0);
					for(int j=0; j<mysize0; j++)
					{
						//epos1[0] = j;
						//s = T(0);
						t = k-j;
						if(t>=0 && t<Fdim) {tmpval+=tmp1Darray[j]*F[t];}//s = F[t];}
						if(t<0) break;

						//tmpval+=tmparray.getElement(epos1)*s;
						//if(s!=T(0))	tmpval+=tmp1Darray[j]*s;
                    }//end for j
					//epos2[0] = k;
					start[0] = k;
					//ret.setElement(tmpval, epos2);
					ret.setElement(tmpval, start);
				}//end for k
		}//end for i
	}//end if
	else
	{
		ret.setSize(mysize);
		DArrayN<T,n-1> tmparr;
		for(int i=0; i<mysize[0]; i++)
		{
			tmparr = this->array[i];
			ret.array[i] = tmparr.conv(F,dir);
		}
	}//end else

	return ret;
}

//Stopping Criteria
template<class T>
DArrayN<T,1>& DArrayN<T,1>::conv(DVector<T> F, int dir)
{
	static DArrayN<T,1> ret;
	DVector<int> mysize;
	this->getSize(mysize);

	//seting new size of ret
	int Fdim = F.dim();
	DVector<int> newsize;
	newsize = mysize;
	newsize[0] = mysize[0]+Fdim-1;
	ret.setSize(newsize);
	
	T tmpval;
	T s; int t;
	int newsize0 = newsize[0];
	int mysize0 = mysize[0];
	for(int k=0; k<newsize0; k++) {
		tmpval = T(0);
		for(int j=0; j<mysize0; j++)
		{
			s = T(0);
			t = k-j;
			if(t>=0 && t<Fdim) {tmpval+=this->array[j]*F[t];}//s=F[t];}
			if(t<0) break;

			//if(s!=T(0)) tmpval+=this->array[j]*s;
		}//end for j
		ret.array[k] = tmpval;
	}//end for k

	return ret;
}

/***************************************************************************
****																	****
****					 OPERATORS										****
****																	****
***************************************************************************/

/**************************************************************************
********************** OPERATOR[](int i) **********************************
***************** returning given element *********************************
**************************************************************************/

template<class T, int n>
DArrayN<T,n-1>& DArrayN<T,n>::operator [](int i)
{
	return array[i];
}

//Stopping Criteria
template<class T>
T& DArrayN<T,1>::operator [](int i)
{
	return array[i];
}


/**************************************************************************
********************** OPERATOR()(int i) **********************************
***************** returning a copy of given element ***********************
**************************************************************************/
template<class T, int n>
DArrayN<T,n-1> DArrayN<T,n>::operator ()(int i) const
{
	return array[i];
}

//Stopping Criteria
template<class T>
T DArrayN<T,1>::operator ()(int i) const
{
	return array[i];
}


/**************************************************************************
********************** OPERATOR= (const DArrayN<T,n> & arr*****************
***************** copying the hole DArrayN ********************************
**************************************************************************/
template<class T,int n>
DArrayN<T,n>& DArrayN<T,n>::operator=(const DArrayN<T,n> &arr)
{
	copy(arr);
	return (*this);
}

//Stopping Criteria
template<class T>
DArrayN<T,1>& DArrayN<T,1>::operator=(const DArrayN<T,1> &arr)
{
	copy(arr);
	return (*this);
}


/**************************************************************************
********************** OPERATOR+ (const DArrayN<T,n> & arr*****************
***************** Summing two DArrayN element by element ******************
**************************************************************************/
template<class T,int n>
DArrayN<T,n>& DArrayN<T,n>::operator +(const DArrayN<T,n> &arr)
{
	static DArrayN<T,n> ret;
	DVector<int> retsize;
	arr.getSize(retsize);
	ret.setSize(retsize);
	int retsize0 = retsize[0];
	for(int i=0; i<retsize0; i++)
	{
		ret.array[i] = array[i]+arr.array[i];
	}
	return ret;
}

//Stopping Criteria
template<class T>
DArrayN<T,1>& DArrayN<T,1>::operator +(const DArrayN<T,1> &arr)
{
	static DArrayN<T,1> ret;
	DVector<int> retsize;
	arr.getSize(retsize);
	ret.setSize(retsize);
	int retsize0 = retsize[0];
	for(int i=0; i<retsize0; i++)
	{
		ret.array[i] = array[i]+arr.array[i];
	}
	return ret;
}


/**************************************************************************
********************** OPERATOR- (const DArrayN<T,n> & arr*****************
***************** Subtracting two DArrayN element by element ******************
**************************************************************************/
template<class T,int n>
DArrayN<T,n>& DArrayN<T,n>::operator -(const DArrayN<T,n> &arr)
{
	static DArrayN<T,n> ret;
	DVector<int> retsize;
	arr.getSize(retsize);
	ret.setSize(retsize);
	int retsize0 = retsize[0];
	for(int i=0; i<retsize0; i++)
	{
		ret.array[i] = array[i]-arr.array[i];
	}
	return ret;
}

//Stopping Criteria
template<class T>
DArrayN<T,1>& DArrayN<T,1>::operator -(const DArrayN<T,1> &arr)
{
	static DArrayN<T,1> ret;
	DVector<int> retsize;
	arr.getSize(retsize);
	ret.setSize(retsize);
	int retsize0 = retsize[0];
	for(int i=0; i<retsize0; i++)
	{
		ret.array[i] = array[i]-arr.array[i];
	}
	return ret;
}


/**************************************************************************
********************** OPERATOR* (const double val) ****************************
***************** Multiplying each element with a number val **************
**************************************************************************/
template<class T,int n>
DArrayN<T,n>& DArrayN<T,n>::operator *(const double val)
{
	static DArrayN<T,n> ret;
	DVector<int> retsize;
	this->getSize(retsize);
	ret.setSize(retsize);
	int retsize0 = retsize[0];
	for(int i=0; i<retsize0; i++)
	{
		ret.array[i] = array[i]*val;
	}
	return ret;
}

//Stopping Criteria
template<class T>
DArrayN<T,1>& DArrayN<T,1>::operator *(const double val)
{
	static DArrayN<T,1> ret;
	DVector<int> retsize;
	this->getSize(retsize);
	ret.setSize(retsize);
	int retsize0 = retsize[0];
	for(int i=0; i<retsize0; i++)
	{
		ret.array[i] = array[i]*val;
	}
	return ret;
}



/***************************************************************************
****																	****
****					PRIVATE IOSTREAM FUNCTIONS						****
****																	****
***************************************************************************/

#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
template<class T, int n>
inline
ostream& DArrayN<T,n>::_prOut(ostream& out) const
{
	for(int i=0; i<no_elem; i++)
	{
		out<<array[i];
		out<<"\n";
	}
	return out;
}

//Stopping Criteria;
template<class T>
inline
ostream& DArrayN<T,1>::_prOut(ostream& out) const
{
	for(int i=0; i<no_elem; i++)
	{
		out<<array[i]<<" ";
	}
	return out;
}
#endif

/**************************************************************************
********************** IOSTREAM for QT ************************************
**************************************************************************/
#if defined(QTEXTSTREAM_H)
template<class T, int n>
inline
QTextStream& DArrayN<T,n>::_prOut(QTextStream& out) const
{
	for(int i=0; i<no_elem; i++)
	{
		out<<array[i];
		out<<"\n";
	}
	return out;
}

//Stopping Criteria;
template<class T>
inline
QTextStream& DArrayN<T,1>::_prOut(QTextStream& out) const
{
	for(int i=0; i<no_elem; i++)
	{
		out<<array[i]<<" ";
	}
	return out;
}
#endif


}//end namespace