/***************************************************************************
OBS!!! GM_DArrayN.c includeres p slutten av denne fila -> exclude build 
properties for WaveDWT.c m derfor settes til yes!!
***************************************************************************/

#ifndef _GM_DARRAY_H_
#define _GM_DARRAY_H_

#include <math.h>
#include <qtextstream.h>
#include <GM_DVector.h>
#include <GM_Array.h>


namespace GM_lib
{

template<class T, int n>
class ArrayElem
{
public:
	ArrayElem() {}
	~ArrayElem() {}

	int index[n];
	int sumindex;
	T   val;

	DVector<int> getIndexes()
	{
		DVector<int> ret(n);
		for(int i=0; i<n; i++) ret[i] = index[i];
		return ret;
	}

	bool operator <  (const ArrayElem<T,n>& le) const	{ return (sumindex < le.sumindex); }
	bool operator >  (const ArrayElem<T,n>& le) const	{ return (sumindex > le.sumindex); }

/*****IOSTREAM overloaded functions for QT ******************************/
#if defined(QTEXTSTREAM_H)
private:
	QTextStream& _prOut(QTextStream& out) const
	{
		out<<"\n(";
		for(int i=0; i<n; i++) out<<index[i]<<",";
		out<<") "<<sumindex;
		out<<" "<<val;
		return out;
	}

public:
	friend
		QTextStream& operator<<(QTextStream& out, const ArrayElem<T,n>& le)
											{ return le._prOut(out); }
	friend
		QTextStream& operator<<(QTextStream& out, const ArrayElem<T,n>* le)
											{ return le->_prOut(out); }
#endif;
};

template<class T, int n>
class DArrayN
{
private:
	DArrayN<T,n-1>* array;
	int no_elem;

	void copy(const DArrayN<T,n>& arr);

public:
	DArrayN();									//Default Constructor
	DArrayN(const DVector<int> size);			//Standard Constructor
	DArrayN(const DVector<int> size, T val);	//Standard Constructor
	DArrayN(const DArrayN<T,n> &arr);			//Copy Constructor
	~DArrayN();

	void setSize(DVector<int> size);			//seting the size in every direction
	void setSize(DVector<int> size, T val);		//seting the size in every direction and giving all element value val
	void getSize(DVector<int> &size) const;		//geting the size in every direction

	void setElements(T val = 0);				//Setting all the elements equal val
	void getMinMax(T &_min, T &_max) const;		//returning min and max value for the array			

	void getIndexes(Array<DVector<int>* > &indexes); //Retuning an array with all posible combination of indexes
	void setElement(T val, DVector<int> pos);		 //Used for seting a element internal inside the DArrayN
	T&	 getElement(DVector<int> pos);				 //Used to get element value internal inside the DArrayN
	void getFlatenArray(Array<ArrayElem<T,n> > & ret);	//flaten out the n-dim array. include the index and the value
	void extrudeArray(Array<ArrayElem<T,n> >* flatarr, DVector<int> orgsize); //build a DArrayN from an array

	DArrayN<T,n>& getSubArray(DVector<int> start, DVector<int> end); //Returning a subArray
	void insertSubArray(DArrayN<T,n> &arr, DVector<int> start); //insering a sub array into already existing array
	DArrayN<T,n>& dyaddown(int dir, DVector<int> shift, bool ODDindx=false); //dyadic downsampling in direction dir
	DArrayN<T,n>& dyadup(int dir, DVector<int> shift, bool ODDindx=false); //dyadic upsampling in direction dir
	DArrayN<T,n>& conv(DVector<T> F, int dir);				//Convolv the array with a filter F in given direction


	/****OPERATORER*********************************************************/
	DArrayN<T,n-1>& operator[](int i);					//returns given element
	DArrayN<T,n-1>  operator()(int i) const;			//returns a copy of given element
	DArrayN<T,n>& operator=(const DArrayN<T,n>& arr);	//Copy the other array
	DArrayN<T,n>& operator+(const DArrayN<T,n>& arr);	//Summing two DArrayN element by element
	DArrayN<T,n>& operator-(const DArrayN<T,n>& arr);	//Subtracting two DArrayN element by element
	DArrayN<T,n>& operator*(const double val);			//Multiplying each element with a number val

	/*****IOSTREAM overloaded functions************************************/
#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
private:
	ostream& _prOut(ostream& out) const;

public:
	friend
		ostream& operator<<(ostream& out, const DArrayN<T,n>& arr)
											{ return arr._prOut(out); }
	friend
		ostream& operator<<(ostream& out, const DArrayN<T,n>* arr)
											{ return arr->_prOut(out); }
#endif;

	/*****IOSTREAM overloaded functions for QT ******************************/
#if defined(QTEXTSTREAM_H)
private:
	QTextStream& _prOut(QTextStream& out) const;

public:
	friend
		QTextStream& operator<<(QTextStream& out, const DArrayN<T,n>& arr)
											{ return arr._prOut(out); }
	friend
		QTextStream& operator<<(QTextStream& out, const DArrayN<T,n>* arr)
											{ return arr->_prOut(out); }
#endif;
};

/**************************************************************************
************** STOPPING CRITERIA ******************************************
**************************************************************************/
template<class T>
class DArrayN<T,1>
{
private:
	T* array;
	int no_elem;

	void copy(const DArrayN<T,1>& arr);


public:
	DArrayN();									//Default Constructor
	DArrayN(const DVector<int> size);			//Standard Constructor
	DArrayN(const DVector<int> size, T val);	//Standard Constructor
	DArrayN(const DArrayN<T,1> &arr);			//Copy Constructor
	~DArrayN();

	void setSize(DVector<int> size);			//seting the size in every direction
	void setSize(DVector<int> size, T val);		//seting the size in every direction and giving all element value val
	void getSize(DVector<int> &size) const;		//geting the size in every direction

	void setElements(T val = 0);				//Setting all the elements equal val
	void getMinMax(T &_min, T &_max) const;		//returning min and max value for the array

	void getIndexes(Array<DVector<int>* > &indexes); //Retuning an array with all posible combination of indexes
	void setElement(T val, DVector<int> pos);		 //Used for seting a element internal inside the DArrayN
	T&	 getElement(DVector<int> pos);				 //Used to get element value internal inside the DArrayN
	void getFlatenArray(Array<ArrayElem<T,1> > & ret);	//flaten out the n-dim array. include the index and the value
	void extrudeArray(Array<ArrayElem<T,1> >* flatarr, DVector<int> orgsize); //build a DArrayN from an array

	DArrayN<T,1>& getSubArray(DVector<int> start, DVector<int> end); //Returning a subArray
	void insertSubArray(DArrayN<T,1> &arr, DVector<int> start); //insering a sub array into already existing array
	DArrayN<T,1>& dyaddown(int dir, DVector<int> shift, bool ODDindx=false); //dyadic downsampling in direction dir
	DArrayN<T,1>& dyadup(int dir, DVector<int> shift, bool ODDindx=false); //dyadic upsampling in direction dir
	DArrayN<T,1>& conv(DVector<T> F, int dir);				//Convolv the array with a filter F in given direction

	/****OPERATORER*********************************************************/
	T& operator[](int i);								//returns given element
	T  operator()(int i) const;							//returns a copy of given element
	DArrayN<T,1>& operator=(const DArrayN<T,1>& arr);	//Copy the other array
	DArrayN<T,1>& operator+(const DArrayN<T,1>& arr);	//Summing two DArrayN element by element
	DArrayN<T,1>& operator-(const DArrayN<T,1>& arr);	//Subtracting two DArrayN element by element
	DArrayN<T,1>& operator*(const double val);			//Multiplying each element with a number val

	/*****IOSTREAM overloaded functions************************************/
#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
private:
	ostream& _prOut(ostream& out) const;

public:
	friend
		ostream& operator<<(ostream& out, const DArrayN<T,1>& arr)
											{ return arr._prOut(out); }
	friend
		ostream& operator<<(ostream& out, const DArrayN<T,1>* arr)
											{ return arr->_prOut(out); }
#endif;

	/*****IOSTREAM overloaded functions for QT ******************************/
#if defined(QTEXTSTREAM_H)
private:
	QTextStream& _prOut(QTextStream& out) const;

public:
	friend
		QTextStream& operator<<(QTextStream& out, const DArrayN<T,1>& arr)
											{ return arr._prOut(out); }
	friend
		QTextStream& operator<<(QTextStream& out, const DArrayN<T,1>* arr)
											{ return arr->_prOut(out); }
#endif;

};


}//end namespace GM_lib

#include "GM_DArrayN.c"

#endif //_GM_DARRAY_H_