//********************************************************
// GM_DVectorN.h: interface for the DVectorN class.
//********************************************************
#ifndef GM_DVECTORN_H_
#define GM_DVECTORN_H_

#include <GM_DVector.h>
#include <GM_Point.h>

namespace GM_lib {


template <class T, int n>
class DVectorN
{
	DVectorN<T,n-1>* pt;
	int   _n, lvl;
	void copy(const DVectorN<T,n>& v);
	void copy(const T p[]);
	DVector<T>* _flatVec;

	void indeksOK(int i) const {
#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(__STD_FSTREAM__)
		if (i<0 || i>=_n) cout <<"indeksfeil vektor!";
#endif        
	}

	//void min(int a, int b) { return a<b?a:b;}

public:

	DVectorN(int i=0)								{ pt = (i ? new DVectorN<T,n-1>[i]:0); _n=i; lvl=0;}
	DVectorN(const DVectorN<T,n>& v)				{ pt=NULL; copy(v); lvl=0;}
	DVectorN(const T p[])							{ pt=NULL; copy(p); lvl=0;}

	~DVectorN()										{ if (pt) delete []pt; }

	DVectorN<T,n>& operator=(const DVectorN<T,n>& v){ copy(v); return(*this);}
	DVectorN<T,n>& operator=(T p[])					{ copy(p); return(*this);}

	void			setDim(const Vector<int,n>& i);
	//void			setDim(int* p[]){cout << "sieze of p: " << sizeOf(p) << endl; setDim(*(new Vector<int,sizeOf(p)>(p))); } // Added by Einar Ramussen 23.02.2006
	Vector<int,n>&	dim() const {static Vector<int,n> r; if(_n) for(int j=0; j<(n-1); j++) r[j]=pt[0].dim()(j); r[n-1] = _n; return r;}
	// getDim() returns the dimensions in reverse order, for use with setDim.
	Vector<int,n>&	getDim() const { static Vector<int,n> res1,res2; res1=dim(); for(int i=0; i<n; i++) res2[(n-1)-i]=res1[i]; return res2;}
	DVector<T>*	flat(); //Returns a one-dimensional DVector of the entire n-dimensional datastructure. By Einar Rasmussen 13.03.2006

	DVectorN<T,n>&	operator+=(const DVectorN<T,n>&); //(+) On all elements.
	DVectorN<T,n>&	operator-=(const DVectorN<T,n>&); //(-) On all elements.
	DVectorN<T,n>&  operator%=(const DVectorN<T,n>&);
	DVectorN<T,n>&  operator*=(double d);
	DVectorN<T,n>&  operator/=(double d);
	DVectorN<T,n-1>&	operator*(DVectorN<T,1>& v);  //Multiply with a vector returning a DVectorN of one lower dimension. By Einar Rasmussen 24.03.2006

	DVectorN<T,n>	operator+(const DVectorN<T,n>& a)	const
									{ DVectorN<T,n> na=(*this); na += a; return na; }
	DVectorN<T,n>	operator-(const DVectorN<T,n>& a)	const
									{ DVectorN<T,n> na=(*this); na -= a; return na; }
	DVectorN<T,n>	operator%(const DVectorN<T,n>& v)	const
									{ DVectorN<T,n> na=(*this); na %= a; return na; }
	DVectorN<T,n>	operator*(double d) const
									{ DVectorN<T,n> na=(*this); na *= d; return na; }
	DVectorN<T,n>	operator/(double d) const
									{ DVectorN<T,n> na=(*this); na /= d; return na; }

	//By Einar Rasmussen 22.02.06
	DVectorN<T,n-1>& operator[] (int i); // return given element.
	const DVectorN<T,n-1>&		operator()(int i) const; // return const of given element.

	T operator*(const DVectorN<T,n>& v);

	// ********** PRINT ***********
public:
	template<class T_Stream>
		friend T_Stream& operator<<(T_Stream& out, const DVectorN<T,n>& v)
	{ 
		for (int i=0; i<v.dim(); i++) 
			out << v(i) << object_separator;
		return out; 
	}

	template<class T_Stream>
		friend 	T_Stream& operator>>(T_Stream& in, DVectorN<T,n>& v)
	{
		static Separator os(object_separator);
		for (int i=0; i<v.dim(); i++) 
			in  >> v[i] >> os;
		return in; 
	}
};

template<class T>
class DVectorN<T,1>: public DVector<T>
{
	//DVector<T>* _flatVec;
public:
	DVectorN(int i=0):DVector<T>(i) {}
	DVectorN(const DVector<T>& v):DVector<T>(v) {}
	DVectorN(const T p[]):DVector<T>(p) {}

	void			setDim(const Vector<int,1>& i) { DVector<T>::setDim(i(0));}
	Vector<int,1>	dim() const { Vector<int,1> r(DVector<T>::dim()); return r;}
	Vector<int,1>&	getDim() const { return dim();}

	DVector<T>*	flat()
	{
		static DVector<T>* _flatVec = new DVector<T>(DVector<T>::dim(),1);

		for(int i=DVector<T>::dim()-1; i>=0; i--) (*_flatVec)[i]= (*this)(i); 

		return _flatVec;
	}

	void copy(const DVectorN<T,1>& v);
};


template<class T, int n>
void DVectorN<T,n>::copy(const DVectorN<T,n>& v)
{
	int i;
	if (_n == v._n)
		for(i=0;i<v._n;i++) pt[i]=v.pt[i];
	else
	{
		if (pt) delete pt; 
		if (v._n > 0)
		{cout << "Copy<T,n> :" << endl;
			pt = new DVectorN<T,n-1>[v._n];
			for(int i=0;i<v._n;i++) pt[i]=v.pt[i];
		}
		else   pt = NULL;
		_n = v._n;
	}
}

template<class T>
void DVectorN<T,1>::copy(const DVectorN<T,1>& v)
{
	int i;
	if (_n == v._n)
		for(i=0;i<v._n;i++) pt[i]=v.pt[i];
	else
	{
		if (pt) delete pt; 
		if (v._n > 0)
		{cout << "Copy<T,1> :" << endl;
			pt = new DVectorN<T,n-1>[v._n];
			for(int i=0;i<v._n;i++) pt[i]=v.pt[i];
		}
		else   pt = NULL;
		_n = v._n;
	}
}

template<class T, int n>
void DVectorN<T,n>::copy(const T p[])
{
	for(int i=0;i<_n;i++) pt[i]=p[i];
}

template<class T, int n>
void DVectorN<T,n>::setDim(const Vector<int,n>& i)
{
	if (i(n-1) != _n) 
	{
		if (pt) delete pt;
		_n  = i(n-1);
		if(_n>0)	pt = new DVectorN<T,n-1>[_n];
		else	pt = NULL;
	}
	Vector<int,n-1> ii;
	for(int j=(n-2); j>=0; j--) ii[j]= i(j);
	for(int j=(_n-1); j>=0; j--) pt[j].setDim(ii);
}

template<class T, int n>
DVector<T>* DVectorN<T,n>::flat()
{
	lvl++;
	static Vector<int,n> vec = dim();
	int dim = 1;
	for(int i=0; i<n; i++) dim*=vec[i];
	static DVector<T>* _flatVec = new DVector<T>(dim,1);
	//static T* ar = new T[dim];
	for(int i=vec[n-1]-1; i>=0; i--) _flatVec->putFirst((DVector<T>) *(pt[i].flat())); 
    
	return _flatVec;
}

/**********************************************************************
*						Operators
***********************************************************************/

template<class T, int n>
DVectorN<T,n-1>& DVectorN<T,n>::operator*(DVectorN<T,1>& v)
{
	static Vector<int,n> dims = getDim();
	static Vector<int,n-1> dim(n-1);
	for(int i=0;i<(n-1);i++) dim[i]=dims[i];
	static DVectorN<T,n-1> result;
	result.setDim(dim);

	for(int i=0; i<dim[n-1]; i++)
	{
		result[i] = pt[i]*v;
	}

	return result;
}

template<class T, int n>
DVectorN<T,n-1>& DVectorN<T,n>::operator[](int i)
{
	indeksOK(i);
	return pt[i];
}//By Einar Rasmussen 22.02.06

template<class T, int n>
const DVectorN<T,n-1>& DVectorN<T,n>::operator()(int i) const
{
	indeksOK(i);
	return pt[i];
}//By Einar Rasmussen 22.02.06

template<class T, int n>
DVectorN<T,n>& DVectorN<T,n>::operator+=(const DVectorN<T,n>& v)
{
   int j = min (_n,v._n);
   for(int i=0; i<j; i++) pt[i] += v.pt[i];
   return *this;
}

template<class T, int n>
DVectorN<T,n>& DVectorN<T,n>::operator-=(const DVectorN<T,n>& v)
{
   int j = min (_n,v._n);
   for(int i=0; i<j; i++) pt[i] -= v.pt[i];
   return *this;
}

template<class T, int n>
DVectorN<T,n>& DVectorN<T,n>::operator%=(const DVectorN<T,n>& v)
{
   int j = min (_n,v._n);
   for(int i=0; i<j; i++) pt[i] %= v.pt[i];
   return *this;
}

template<class T, int n>
DVectorN<T,n>& DVectorN<T,n>::operator*=(double d)
{
	for(int i=0;i<_n;i++) pt[i] *= d;
	return *this;
}

template<class T, int n>
DVectorN<T,n>& DVectorN<T,n>::operator/=(double d)
{
	for(int i=0;i<_n;i++) pt[i] /= d;
	return *this;
}

template<class T, int n>
T operator*(const DVectorN<T,n>& v)
{
	int j = min (_n,v._n);
	T r(0);
	for(int i=0;i<j;i++) r += pt[i]*v.pt[i];
	return r;
}

} // end namespace GM_lib

#endif /*GM_DVectorNN_H_*/
