#ifndef _GM_DINI_SURF_H_
#define _GM_DINI_SURF_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Dini_Surf : public PSurf<T,n>
{
	T _r1;	//radius
	T _r2; //dist between turns

	void _eval(T u, T v, int d1, int d2);
	//void localDisplay();
	// void localSimulate(double dt);
protected:

	T _st_p_u()		{ return T(0); }
	T _end_p_u()		{ return T(4*M_PI); }
	T _st_p_v()		{ return T(0.001); }
	T _end_p_v()		{ return T(2); }


public:

	Dini_Surf();
	Dini_Surf(T radius, T dist_turn);
	// Dini_Surf(Dini_Surf<T,n>& s);

	~Dini_Surf() {}

	void setConstants(T radius, T dist_turn);

	const char* identity() const { return " Dini_Surf "  ;}
};
} // GM_lib

using namespace GM_lib;

template <class T, int n>
Dini_Surf<T,n>::Dini_Surf():PSurf<T,n>()
{
	setConstants(T(1.0), T(0.1));
}

template <class T, int n>
Dini_Surf<T,n>::Dini_Surf(T radius, T dist_turn):PSurf<T,n>()
{
	setConstants(radius, dist_turn);
}

template <class T, int n>
void Dini_Surf<T,n>::setConstants(T radius, T dist_turn)
{
	_r1 = radius;  
	_r2 = dist_turn;
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,-4.0),5.0));
}

template <class T, int n>
void Dini_Surf<T,n>::_eval(T u, T v, int d1,int d2)
{

	__p.setDim(4,4);
	__p[0][0][0] =  	_r1*cos(u)*sin(v);
	__p[0][0][1] =  	_r1*sin(u)*sin(v);
	__p[0][0][2] =  	_r1*(cos(v)+log(tan((T(0.5)*v))))+_r2*u;

	if(d1) //u
	{
		__p[1][0][0] =	-_r1*sin(u)*sin(v);
		__p[1][0][1] =	_r1*cos(u)*sin(v);
		__p[1][0][2] =	_r2;
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	-_r1*cos(u)*sin(v);	
		__p[2][0][1] =	-_r1*sin(u)*sin(v);	
		__p[2][0][2] =	T(0);				
	}
	if(d2) //v	
	{
		__p[0][1][0] =	_r1*cos(u)*cos(v);	 
		__p[0][1][1] =	_r1*sin(u)*cos(v);	 
		__p[0][1][2] =	_r1*(-sin(v)+(1+tan(T(.5)*v)*tan(T(.5)*v))*T(.5)/tan(T(.5)*v));	 
	}
	if(d2>1) //vv	
	{
		__p[0][2][0] =	-_r1*cos(u)*sin(v);
		__p[0][2][1] =	-_r1*sin(u)*sin(v);
		__p[0][2][2] =	-0.25*_r1*(-4.0*cos(v)*pow(cos(T(0.5)*v),T(2.0))+4.0*cos(v)*pow(cos(T(0.5)*v),T(4.0))+1.0
						-2.0*pow(cos(T(0.5)*v),T(2.0)))/pow(cos(T(0.5)*v),T(2.0))/(-1.0+pow(cos(T(0.5)*v),T(2.0)));
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =	-_r1*sin(u)*cos(v);	 
		__p[1][1][1] =	_r1*cos(u)*cos(v);		 
		__p[1][1][2] =	T(0);	 
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =	-_r1*cos(u)*cos(v);
		__p[2][1][1] =	-_r1*sin(u)*cos(v);
		__p[2][1][2] =	T(0);
	}
	if(d1 && d2>1) //uvv			//duuv 
	{
		__p[1][2][0] =	_r1*sin(u)*sin(v);	 
		__p[1][2][1] =	-_r1*cos(u)*sin(v);	 
		__p[1][2][2] =	T(0);	 
	}
	if(d1>1 && d2>1) //uuvv
	{			
		__p[2][2][0] =	_r1*cos(u)*sin(v);	
		__p[2][2][1] =	_r1*sin(u)*sin(v);	 
		__p[2][2][2] =	T(0);	 
	} 

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];

}

 #endif //_GM_DINI_SURF_H_
