
#ifndef _GM_Eight_Surface_H_
#define _GM_Eight_Surface_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Eight_Surface : public PSurf<T,n>
{
	void _eval(T u, T v, int d1, int d2);

//	void localSimulate(double dt);

protected:
	T _st_p_u()			{ return T(0.0); }
	T _end_p_u()			{ return T(2*M_PI); }
	T _st_p_v()			{ return T(0.0); }
	T _end_p_v()			{ return T(2*M_PI); }

public:
	Eight_Surface();
	Eight_Surface(const Eight_Surface<T,n>& s);

	const char* identity() const { return " Eight_Surface "  ;}
};
} // GM_lib

using namespace GM_lib;

template <class T, int n>
Eight_Surface<T,n>::Eight_Surface():PSurf<T,n>()
{
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),10.0));
}

template <class T, int n>
Eight_Surface<T,n>::Eight_Surface(const Eight_Surface<T,n>& s):PSurf<T,n>(s)
{
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),10.0));
}

template <class T, int n>
void Eight_Surface<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(4,4);
	__p[0][0][0] =	cos(u)*sin(v)*cos(v);		
	__p[0][0][1] =	sin(u)*sin(v)*cos(v);		
	__p[0][0][2] =	sin(v);		

	if(d1) //u
	{
		__p[1][0][0] =	-sin(u)*sin(v)*cos(v);	
		__p[1][0][1] =	cos(u)*sin(v)*cos(v);	
		__p[1][0][2] =	T(0);	
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	-cos(u)*sin(v)*cos(v);	
		__p[2][0][1] =	-sin(u)*sin(v)*cos(v);	
		__p[2][0][2] =	T(0);		
	}
	if(d2) //v
	{
		__p[0][1][0] =	cos(u)*(T(2)*cos(v)*cos(v)-T(1));	
		__p[0][1][1] =	sin(u)*(T(2)*cos(v)*cos(v)-T(1));
		__p[0][1][2] =	cos(v);	
	}
	if(d2>1) //vv
	{
		__p[0][2][0] =	-T(4)*cos(u)*sin(v)*cos(v);	
		__p[0][2][1] =	-T(4)*sin(u)*sin(v)*cos(v);	
		__p[0][2][2] =	-sin(v);	
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =	-sin(u)*(T(2)*cos(v)*cos(v)-T(1));	
		__p[1][1][1] =	cos(u)*(T(2)*cos(v)*cos(v)-T(1));
		__p[1][1][2] =	T(0);	
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =	-cos(u)*(T(2)*cos(v)*cos(v)-T(1));	
		__p[2][1][1] =	-sin(u)*(T(2)*cos(v)*cos(v)-T(1));
		__p[2][1][2] =	T(0);	
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =	T(4)*sin(u)*sin(v)*cos(v);	
		__p[1][2][1] =	-T(4)*cos(u)*sin(v)*cos(v);	
		__p[1][2][2] =	T(0);	
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =	T(4)*cos(u)*sin(v)*cos(v);	
		__p[2][2][1] =	T(4)*sin(u)*sin(v)*cos(v);	
		__p[2][2][2] =	T(0);	
	}

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}


#endif 

