
#ifndef _GM_ENNEPERS_SURFACE_H_
#define _GM_ENNEPERS_SURFACE_H_

#include "GM_PSurf.h"

namespace GM_lib{

	template <class T, int n>
	class Ennepers_Surface : public PSurf<T,n>
	{
		void _eval(T u, T v, int d1, int d2);
		//void localDisplay();
		// void localSimulate(double dt);
	protected:

		T _st_p_u()			{ return T(-1.5); }
		T _end_p_u()			{ return T(1.5); }
		T _st_p_v()			{ return T(-1.5); }
		T _end_p_v()			{ return T(1.5); }

	public:
		Ennepers_Surface();
		Ennepers_Surface(Ennepers_Surface<T,n>& s);
 
		~Ennepers_Surface() {}
		const char* identity() const { return " Ennepers_Surface "  ;}
	};
} // GM_lib

using namespace GM_lib;
template <class T, int n>
Ennepers_Surface<T,n>::Ennepers_Surface():PSurf<T,n>()
{
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),10.0));
}
template <class T, int n>
void Ennepers_Surface<T,n>::_eval(T u, T v, int d1,int d2)
{
	T a = T(3.3333333e-01);

	__p.setDim(4,4);
	__p[0][0][0] =  -a*(u*u*u)+(v*v)*u+u;
	__p[0][0][1] =  -v-v*(u*u)+a*(v*v*v);
	__p[0][0][2] =  (u*u)-(v*v);

	if(d1) //u
	{
		__p[1][0][0] =	-T(3)*a*u*u+v*v+T(1);	
		__p[1][0][1] =	-T(2)*v*u;	
		__p[1][0][2] =	T(2)*u;	
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	-T(6)*a*u;	
		__p[2][0][1] =	-T(2)*v;	
		__p[2][0][2] =	T(2);	
	}
	if(d2) //v
	{
		__p[0][1][0] =	T(2)*v*u;	
		__p[0][1][1] =	-T(1)-u*u+T(3)*a*v*v;	
		__p[0][1][2] =	 -T(2)*v;	
	}
	if(d2>1) //vv
	{
		__p[0][2][0] =	T(2)*u;	
		__p[0][2][1] =	T(6)*a*v;	
		__p[0][2][2] =	-T(2);	
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =	T(2)*v;	
		__p[1][1][1] =	-T(2)*u;	
		__p[1][1][2] =	T(0);	
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =	T(0);	
		__p[2][1][1] =	-T(2);	
		__p[2][1][2] =	T(0);	
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =	T(2);
		__p[1][2][1] =	T(0);	
		__p[1][2][2] =	T(0);	
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =	T(0);	
		__p[2][2][1] =	T(0);	
		__p[2][2][2] =	T(0);	
	}


	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];

}

#endif //_GM_ENNEPERS_SURFACE_H_

