#pragma once
#include <gm_pcurve.h>

using namespace GM_lib;
template <class T, int n>
class ExpoRationalBasis :public PCurve<T,n>
{
	void init();
	bool _poly;
	double alfa;
	double beta;
	double gamma;
	double sigma;
	double alfa2;
	double beta2;
	double gamma2;
	double sigma2;
	double lambda;
	int s,nn;

	T phi(T t);
	T integral(T a,T b, int stp=7);
	double trapez(T a,T b,int n);

	Array<T>					_t;

	void localSimulate(double dt){}
	void localDisplay()
	{ 
		GLboolean lg;
		glGetBooleanv(GL_LIGHTING,&lg);
		if(lg) glDisable(GL_LIGHTING);

		glPointSize(3.80);
		Red.glSet();
		glBegin(GL_POINTS);
		for(int q=0;q<31;q++) 
		{ 
			_eval(startPar()+q*deltaPar()/30.0,1);
			glPoint(__p[0]);
		}
		glEnd();

		Blue.glSet();
		glLineWidth(3);
		glBegin(GL_LINE_STRIP);
		for(int q=0;q<64;q++) 
		{ 
			_eval(startPar()+q*deltaPar()/63.0,1);
			glPoint(__p[1]);
		}
		glEnd();
		glLineWidth(1.0);
/*
		(Green+Red).glSet();
		float t;
		glBegin(GL_LINE_STRIP);
		for(int q=0;q<31;q++) 
		{ 
			 t=startPar()+q*deltaPar()/30.0;
			_eval(t,1);
			Point3D<float> tmp(__p[0]);
			tmp[1]*=t;
			glPoint(tmp);
		}
		glEnd();

		(Green+Red+Blue).glSet();
		glBegin(GL_LINE_STRIP);
		for(int q=0;q<31;q++) 
		{ 
			t=startPar()+q*deltaPar()/30.0;
			_eval(t,1);
			Point3D<float> tmp(__p[0]);
			tmp[1]*=t*t*0.5;
			glPoint(tmp);
		}
		glEnd();
		(0.3*Green+0.3*Red+Blue).glSet();
		glBegin(GL_LINE_STRIP);
		for(int q=0;q<31;q++) 
		{ 
			t=startPar()+q*deltaPar()/30.0;
			_eval(t,1);
			Point3D<float> tmp(__p[0]);
			tmp[1]*=t*t*t/6.0;
			glPoint(tmp);
		}
		glEnd();
		(0.7*Green+0.3*Red+0.6*Blue).glSet();
		glBegin(GL_LINE_STRIP);
		for(int q=0;q<31;q++) 
		{ 
			t=startPar()+q*deltaPar()/30.0;
			_eval(t,1);
			Point3D<float> tmp(__p[0]);
			tmp[1]*=t*t*t*t/24.0;
			glPoint(tmp);
		}
		glEnd();
		//*/

	    if(lg) glEnable(GL_LIGHTING);
		PCurve<T,n>::localDisplay(); 			
	}

protected:
	void _eval(T t, int d);

	T	 _st_p() { return T(_t[0]); }
	T	 _end_p(){ return T(_t[_t.size()-1]); }
public:

	PCurve<T,n>* split(T t) { return this;}
	void setAlfa(T a) { if (a>0.0) alfa=a; }
	void setBeta(T b) { if (b>0.0) beta=b; }
	void setGamma(T g){ if (g>0.0) gamma=g;}
	void setSigma(T s){ if (s>0.0) sigma=s;}
	void setAlfa2(T a) { if (a>0.0) alfa2=a; }
	void setBeta2(T b) { if (b>0.0) beta2=b; }
	void setGamma2(T g){ if (g>0.0) gamma2=g;}
	void setSigma2(T s){ if (s>0.0) sigma2=s;}
	void setLambda(T l){ if(l>=0.0 && l<=1.0) lambda=l; }
	
	void resetPars()
	{
		alfa=1.0;
		beta=1.0;
		gamma=1.0;
		sigma=1.0;
		alfa2=1.0;
		beta2=1.0;
		gamma2=1.0;
		sigma2=1.0;
		lambda=0.5;
	}
	

	ExpoRationalBasis(void)
	{
		alfa=1.0;
		beta=1.0;
		gamma=1.0;
		sigma=1.0;
		alfa2=1.0;
		beta2=1.0;
		gamma2=1.0;
		sigma2=1.0;
		lambda=0.5;
		nn =3;
		_t.setSize(nn);
		_t[0]=-1.0;
		_t[1]= 0.0;
		_t[2]= 1.0;
		__material=Ruby;
		_type_id=2001;
		resetPlot(12,true);
	}

	~ExpoRationalBasis(void)
	{
	}
};

template <class T, int n>
inline
T ExpoRationalBasis<T,n>::phi(T t)
{
	int sp=s+1; 
	T tmi = t-((1.0-lambda)*_t[s]+lambda*_t[sp]);//t-0.5*(_t[sp]+_t[s]);
	T dem1 = (t-_t[s]);
	T dem2 =pow((T)(_t[sp]-t),(T)gamma);
	if(t<0)
	{
		T dem2 =pow((T)(_t[sp]-t),(T)gamma);
		if (t>_t[s] && t<_t[sp]) return exp(- beta*pow((T)(tmi*tmi),(T)sigma)/pow((T)(dem1*dem2),(T)alfa)); // ksi1=1.0;
		else					 return T(0);
	} else
	{
		T dem2 =pow((T)(_t[sp]-t),(T)gamma2);
		if (t>_t[s] && t<_t[sp]) return exp(- beta2*pow((T)(tmi*tmi),(T)sigma2)/pow((T)(dem1*dem2),(T)alfa2)); // ksi1=1.0;
		else					 return T(0);
	}
}

template <class T, int n>
inline
double ExpoRationalBasis<T,n>::trapez(T a, T b,int nn)  // I(0,n)
{	
	int		k = pow(2,nn);
	double	h = (b-a)/k;
	double	sum = 0.0;
	for (int i=1; i<k; i++) sum += phi(a+h*i);
	sum *= h;
	sum += (phi(a)+phi(b))*0.5*h;
	return sum;
}

template <class T, int n>
T ExpoRationalBasis<T,n>::integral(T a, T b, int stp) //  I(v,n) almost like p. 23 (5.6 and 5.7)
{	
	//int stp=7;
	static DMatrix<T> In(11,11);

	In[0][0]=trapez(a,b,0);
	In[0][1]=trapez(a,b,1);

	In[1][0]=(pow(2,2*1)*In[0][1]-In[0][0])/(pow(2,2*1)-1.0);

	for(int v=1;v<stp+1;v++) 
	{
		In[0][v]=trapez(a,b,v);
		for(int k=1;k<=v;k++) In[k][v-k]=(pow(2,2*k)*In[k-1][v-k+1]-In[k-1][v-k])/(pow(2,2*k)-1.0);
	}
	return In[stp][0];
}

template <class T, int n>
void ExpoRationalBasis<T,n>::_eval(T t, int d)
{
	//if(d<=__d && t==__t) return;
	//__t=t;
	//__d=d;

	// First, find the interval
	int tn=nn;
	for(s=0; s<tn-2; s++) if(_t[s+1] > t) break;

	int i;
	Array<T> B;
	Array<T> ph;
	B.setSize(2); ph.setSize(2);// Basis Bk(t)
	T sc=integral(_t[s],_t[s+1],10);
	B[1]=integral(_t[s],t,9)/sc;
	B[0]=1.0-B[1]; 
	ph[1]=phi(t)/sc;
	ph[0]=-ph[1];

	__p.setSize(d+1);
	for(i=0;i<=d;i++) __p[i] = Point<T,n>(0.0);
	if(t<_t[1])
	{
		__p[0][0]=t;
		__p[0][1]=B[1];
		__p[1][0]=t;
		__p[1][1]=ph[1];
	}
	else 
	{
		__p[0][0]=t;
		__p[0][1]=B[0];
		__p[1][0]=t;
		__p[1][1]=ph[0];
	}
}

