
//*************************************************************************************
// ExpoRationalCurve.h: interface for the ExpoRationalCurve classes.
//*************************************************************************************
#ifndef GM_ExpoRationalCurve_H_
#define GM_ExpoRationalCurve_H_

#include "GM_PCurve.h"
#include "GM_Selector.h"
#include <GM_DMatrix.h>
#include <GM_ExpoRationalBasis.h>


namespace GM_lib{

	template <class T, int n>
	class ExpoRationalCurve: public PCurve<T,n>
	{
		void init();
		void calcCtrlPts();
		int findVertex(T t) { return (__vertex.size()-1)*(t-startPar())/(deltaPar())+0.1; }
		// int nn;  // number of ctrl/interpolating points
		int s; // contains current interval in evaluation
		bool _hermite;
		bool _closed;
		bool _poly;		// drawing control polygon 
		bool _blending; // drawing blending functions
		double alfa;
		double beta;
		double gamma;
		double lambda;
		double sigma;
		double time;
		ExpoRationalBasis<float,3> *basis;

		double trapez(T a,T b,int n);
		T integral(T a,T b);

		Array<T>					_t;
		Array<DVector<Point<T,n> > >	_c;
		Array<DVector<Point<T,n> > > bts;
		Array<DVector<DMatrix<T> > > bp ;

		T phi(T t);
		void currentInterval(T& dx, T& y, T t, int r);
		DMatrix<T> BernstainPolynomials(T dx, T y, int order);


		void localSimulate(double dt)
		{
			time+=dt*0.1;
			if(time>M_2PI)time=0.0;
			// rotate(dt,surroundingSphere().pos(),UnitVector3D<float>(0.0,0.0,1.0));
			setAlfa((sin(2*time)+1.0001)*2);
			setBeta((sin(3*time)+1.0001)*2);
			setGamma((sin(5*time)+1.0001)*2);
			setLambda(0.5*(sin(time)+1.0001)); // 0<l<1
			setSigma(sin(7*time)+1.0001);
			if(basis)
			{
				basis->setAlfa((sin(2*time)+1.0001)*2);
				basis->setBeta((sin(3*time)+1.0001)*2);
				basis->setGamma((sin(5*time)+1.0001)*2);
				// basis->setLambda(0.5*(sin(time)+1.0)); // 0<l<1
				basis->setSigma(sin(7*time)+1.0001);
				basis->setAlfa2((sin(2*time)+1.0001)*2);
				basis->setBeta2((sin(3*time)+1.0001)*2);
				basis->setGamma2((sin(5*time)+1.0001)*2);
				basis->setLambda(0.5*(sin(time)+1.0001)); // 0<l<1
				basis->setSigma2(sin(7*time)+1.0001);
				basis->resetPlot(64,true);
				
			}
			resetPlot(64,true);
		}
		void localDisplay()
		{ 			
			if(_poly)
			{
				__material.glSetInverse();
				for(int i=0; i<_c.size(); i++) 
				{ 
					glBegin(GL_LINE_STRIP);
					for(int q=0;q<bts[i].dim();q++) glPoint(bts[i][q]);
					glEnd();
				}
			}
			if(_blending)
			{
				Emerald.glSet();
				int j,start1,end1,start2,end2;
				for(int i=0; i<_c.size(); i++) 
				{ 
					if(_closed)
					{
						if (i==0) 
						{ 
							start1=0; end1=findVertex(_t[1])-1;
							start2=findVertex(_t[_c.size()-1])+1; end2=__vertex.size()-1;
						} 
						else 
						{
							start1=findVertex(_t[i-1]); end1=findVertex(_t[i]);
							start2=end1+1; end2=findVertex(_t[i+1]);
						}
					} 
					else
					{
						if(i==0)
						{
							start1= findVertex(_t[0]); end1 = findVertex(_t[1]);
							start2=end1+1; end2=findVertex(_t[2]);
						} 
						else if(i==_c.size()-1)
						{
							start1= findVertex(_t[_c.size()-2]); end1 = findVertex(_t[_c.size()-1]);
							start2=end1+1; end2=__vertex.size()-1;
						} 
						else
						{
							start1=findVertex(_t[i-1]); end1=findVertex(_t[i]);
							start2=end1+1; end2=findVertex(_t[i+1]);
						}
					}
					glLineWidth(3.0);
					glBegin(GL_LINE_STRIP);

					for(j=start1+1; j<end1; j++) 
					{
						Point3D<float> tmp(0.0);
						for(int q=0;q<bts[i].dim();q++) tmp+=bp[j][1][0][q]*bts[i][q];
						glPoint(tmp);
					}

					for(j=start2; j<end2-1; j++) 
					{
						Point3D<float> tmp(0.0);
						for(int q=0;q<bts[i].dim();q++) tmp+=bp[j][0][0][q]*bts[i][q];
						glPoint(tmp);
					}

					glEnd();
				}
			}

			PCurve<T,n>::localDisplay(); 
		}

	protected:
		void _eval(T t, int d);

		T _st_p() { return T(_t[0]); }
		T _end_p(){ return T(_t[_t.size()-1]); }
		void _makeSample(Array<Point<GLfloat,3> >& a, int m1);

	public:
		ExpoRationalCurve(const Array<DVector<Point<T,n> > >& c,const Array<T>& t, bool cl=false, bool h=false):PCurve<T,n>(),_c(c),_t(t) 
		{
			_hermite	= h;
			_closed		= cl;
			_poly		= true;
			_blending	= true;
			
			alfa		= 1.0;
			beta		= 1.0;
			gamma		= 1.0;
			lambda		= 0.5;
			sigma		= 1.0;

			init();
		}

		ExpoRationalCurve(const Array<DVector<Point<T,n> > >& c, T startpar=-M_PI, T endpar=M_PI, bool cl=false):PCurve<T,n>(),_c(c)
		{
		_hermite=true;
		_closed	= cl;
		_poly	= true;
		_blending=true;
		int tn	= _c.size();
		if(_closed)tn++;
		_t.setSize(tn);
		for(int i=0;i<tn;i++) _t[i]=startpar+i*(endpar-startpar)/(tn-1);
		alfa		= 1.0;
		beta		= 1.0;
		gamma		= 1.0;
		lambda		= 0.5;
		sigma		= 1.0;
		basis = new ExpoRationalBasis<float,3>();
		basis->translateGlobal(Vector3D<float>(-2,0,0));
		insert(basis);
		init();
		}

		ExpoRationalCurve(PCurve<T,n>* c, int nr):PCurve<T,n>()
		{
			_closed	= c->closed();
			_poly	= true;
			_blending=true;
			int i, tn=nr;
			if(_closed)tn++;
			T dp	= c->deltaPar()/(tn-1);
			_t.setSize(tn);
			for(i=0;i<tn;i++) _t[i] = c->startPar() + i*dp;
			_c.setSize(nr);
			for(i=0;i<nr;i++) _c[i] = c->evaluate(_t[i)];
			alfa		= 1.0;
			beta		= 1.0;
			gamma		= 1.0;
			lambda		= 0.5;
			sigma		= 1.0;
			init();
		}

		ExpoRationalCurve(const ExpoRationalCurve<T,n>& c):PCurve<T,n>(c) 
		{ 
			alfa		= c.alfa;
			beta		= c.beta;
			gamma		= c.gamma;	
			lambda		= c.lambda;	
			sigma		= c.sigma;	
			_closed=c._closed; _poly=c._poly; _c=c._c; _t=c._t;}
		Array<T> getTs() { return _t; }
		virtual ~ExpoRationalCurve() {}

		Point<float,3> getPoint(int i) { return _c[0][i]; }
		PCurve<T,n>* split(T t) { return NULL;}
		void edit(int selector);
		void editT(int idx, T v) 
		{
			 _t[idx]=_t[idx-1]+v*(_t[idx+1]-_t[idx-1]); 
			resetPlot(0,true); 
		}
		void setAlfa(T b)	{ if (b>0.0)			alfa=b;		if(basis) { basis->setAlfa(b);    basis->setAlfa2(b); basis->resetPlot(64,true);     } }
		void setBeta(T b)	{ if (b>0.0)			beta=b;		if(basis) { basis->setBeta(b);    basis->setBeta2(b); basis->resetPlot(64,true);      } }
		void setGamma(T b)	{ if (b>0.0)			gamma=b;	if(basis) { basis->setGamma(b);	  basis->setGamma2(b);basis->resetPlot(64,true);		  } }
		void setLambda(T b) { if (b>=0.0 && b<=1.0) lambda=b;	if(basis) { basis->setLambda(b); basis->resetPlot(64,true); 	  } }
		void setSigma(T b)	{ if (b>0.0)			sigma=b;	if(basis) { basis->setSigma(b);	  basis->setSigma2(b); basis->resetPlot(64,true);		  } }
		void setTime(T t)	{ time=t; localSimulate(0.0);}

		double getAlfa  () { return alfa	; }
		double getBeta  () { return beta	; }
		double getGamma () { return gamma	; }
		double getLambda() { return lambda	; }
		double getSigma () { return sigma	; }


		const char* identity() const { return "GM_ExpoRationalCurve";}	
		bool closed() { return _closed; }

		void toggleSelector() 
		{ 
			for(int i=0; i<_object.size(); i++) 
			{
				Selector<T,n> *s=dynamic_cast<Selector<T,n> *>(_object[i]);
				if(s) s->onOff();
			}
			_poly=!_poly;
			_blending=_poly;
		}
		void toggleBlend() 
		{ 
			_blending=!_blending;
		}
	};

} // end namespace GM_lib

using namespace GM_lib;

template <class T, int n>
void ExpoRationalCurve<T,n>::init()
{
	int i,j;
	if(!_hermite) calcCtrlPts(); 
	else 
	{
		bts.setSize(_c.size());
		for(int k=0;k<_c.size();k++) { bts[k].setDim(_c[k].dim()); bts[k]=_c[k]; }
	}
			
	__material=Ruby;
	resetPlot(16); // 0.0005 Setter surroundingsphere
	// Pick points
	float radius=(curveLength()/_c.size())*0.125; // curvelength / nn /10 ?
	if(radius<0.0125) radius=0.0125;
	for(i=0; i<_c.size(); i++) for(j=0;j<bts[i].dim();j++) insert(new Selector<float,3>((bts[i][j]),i,this,radius,GL_Color(0.4,0.4,0.9)));
}



template <class T, int n>
void ExpoRationalCurve<T,n>::edit(int s)
{
	T dlt=deltaPar()/(__vertex.size()-1);
	int sb,se;
	if(_closed)
	{
		if(s==0)
		{
			sb= 0;
		    se= findVertex(_t[1]);
			for(int i=sb;i<=se;i++) __vertex[i]=(*this)(startPar()+i*dlt);
			sb= findVertex(_t[_c.size()-1]);
		    se= __vertex.size()-1;
			for(int i=sb;i<=se;i++) __vertex[i]=(*this)(startPar()+i*dlt);
			
		}else
		{
			sb=findVertex(_t[s-1]);
			se=findVertex(_t[s+1]);
			for(int i=sb;i<=se;i++) __vertex[i]=(*this)(startPar()+i*dlt);
		}
	} else
	{
		if(s==0)
		{
			sb= findVertex(_t[0]);
			se= findVertex(_t[1]);
		} else
		if(s==_c.size()-1)
		{
			sb= findVertex(_t[_c.size()-2]);
			se= findVertex(_t[_c.size()-1]);
		} else
		{
			sb= findVertex(_t[s-1]);
			se= findVertex(_t[s+1]);
		}
		for(int i=sb;i<=se;i++) __vertex[i]=(*this)(startPar()+i*dlt);
	}
}
template <class T, int n>
inline
T ExpoRationalCurve<T,n>::phi(T u)
{
	int sp=s+1; 
	double t=u;
	double tmi = pow(pow(t-((1.0-lambda)*_t[s]+lambda*_t[sp]),2.0),sigma);   //t-0.5*(_t[sp]+_t[s]);
	double dem = pow((t-_t[s])*pow((_t[sp]-t),gamma),alfa);              //(t-_t[s])*(_t[sp]-t);
	if (t>_t[s] && t<_t[sp]) return exp(- beta*(tmi/dem)); // ksi1=1.0;
	else					 return T(0);
}

template <class T, int n>
inline
double ExpoRationalCurve<T,n>::trapez(T a, T b,int nn)  // I(0,n)
{	
	int		k = pow(2,nn);
	double	h = (b-a)/k;
	double	sum = 0.0;
	for (int i=1; i<k; i++) sum += phi(a+h*i);
	sum *= h;
	sum += (phi(a)+phi(b))*0.5*h;
	return sum;
}

template <class T, int n>
T ExpoRationalCurve<T,n>::integral(T a, T b) //  I(v,n) almost like p. 23 (5.6 and 5.7)
{	
	int stp=7;
	GM_lib::DMatrix<T> In(10,10);

	In[0][0]=trapez(a,b,0);
	In[0][1]=trapez(a,b,1);

	In[1][0]=(pow(2,2*1)*In[0][1]-In[0][0])/(pow(2,2*1)-1.0);

	for(int v=1;v<stp+1;v++) 
	{
		In[0][v]=trapez(a,b,v);
		for(int k=1;k<=v;k++) In[k][v-k]=(pow(2,2*k)*In[k-1][v-k+1]-In[k-1][v-k])/(pow(2,2*k)-1.0);
	}
	return In[stp][0];
}
template <class T, int n> 
void ExpoRationalCurve<T,n>::currentInterval(T& dx, T& y, T t, int r)
{
	int rm=r-1, rp=r+1, max =_c.size();

	if(_closed)
	{
		if (rm<0) 
		{
			dx=1.0/(_t[1]-_t[0]+_t[max]-_t[max-1]); 
			y=(t-_t[0]+_t[max]-_t[max-1])*dx;	
		} else if(rp>max)
		{
			dx=1.0/(_t[1]-_t[0]+_t[max]-_t[max-1]);
			y=(t-_t[max-1])*dx;	
		} else
		{ 	
			dx=1.0/(_t[rp]-_t[rm]);
			y=(t-_t[rm])*dx;					
		}
	} 
	else// ok, open
	{   
		if (rm<0) rm=0;
		if (rp>max-1) rp=max-1;
		dx=1.0/(_t[rp]-_t[rm]);
		y=(t-_t[rm])*dx;				
	}
}


template <class T, int n> 
DMatrix<T> ExpoRationalCurve<T,n>::BernstainPolynomials(T dx, T y, int order)
{
	DMatrix<T> bpv(order,order); // Local storage for Berntstein polynomials
	bpv[0][0] = T(1.0);
	T x=T(1)-y;	
	for(int i=1; i<order; i++)
	{
		bpv[i][0] = x*bpv[i-1][0];
		for(int j=1;j<i;j++) bpv[i][j] = y*bpv[i-1][j-1]+x*bpv[i-1][j];
		bpv[i][i] = y*bpv[i-1][i-1];
	}
	dx*=-1.0;
	T myc=1.0;
	DMatrix<T> bv(order,order);
	for(int k=order;k>0;k--) // deriverte (nk-k)
	{
		for(int i=k;i<order;i++) // nedover
		{
			bpv[i][0] = bpv[i-1][0];
			for(int j=1;j<i;j++) bpv[i][j] = bpv[i-1][j]-bpv[i-1][j-1]; //bortover
			bpv[i][i] = -bpv[i-1][i-1];
		}
		for(int j=0; j<order; j++) bv[order-k][j]=bpv[order-1][j]*myc; 
		myc*=dx*(k-1);
	}
	return bv;
}


template <class T, int n>
void ExpoRationalCurve<T,n>::calcCtrlPts()
{
	T  dx, y;
	bts.setSize(_c.size());
	for(s=0; s<_c.size(); s++)
	{
		currentInterval( dx, y, _t[s],  s);
		DMatrix<T> bu=BernstainPolynomials(dx, y, _c[s].dim());
		bu.invert();
		bts[s] =  bu*_c[s]; 		
	} 
}

template <class T, int n>
void  ExpoRationalCurve<T,n>::_makeSample(Array<Point<GLfloat,3> >& a, int m1)
{
	int i;
	T u;
	T du = deltaPar()/(m1-1);
	bp.clear();
	bp.setSize(m1);
    T dx,y;
	for(i=0; i<m1; i++)
	{
		bp[i].setDim(2);
		u = startPar() + i*du;		
		for(s=0; s<_t.size()-2; s++) if(_t[s+1] > u) break;	
		
		for(int ss=s; ss<s+2; ss++)
		{
			int s1=ss;
			if (s1>=bts.size() ) s1-=bts.size();
			 currentInterval( dx, y, u,  ss);
			bp[i][ss-s]=BernstainPolynomials(dx, y, bts[s1].dim());
		}
	}
	PCurve<T,n>::_makeSample(a,m1);
}


template <class T, int n>
void ExpoRationalCurve<T,n>::_eval(T t, int d)
{
	// First, find the intervals
	for(s=0; s<_t.size()-2; s++) if(_t[s+1] > t) break;	

	Vector<T,2> B;
	Vector<T,2> ph;
	T sc  = integral(_t[s],_t[s+1]);
	if(sc==T(0)) sc=1.0; // For knot insertion
	B[1]  = integral(_t[s],t)/sc;
	B[0]  = 1.0-B[1]; 
	ph[1] = phi(t)/sc;
	ph[0] = -ph[1];

	__p.setSize(d+1);
	for(int i=0;i<=d;i++) __p[i] = Vector<T,n>(0.0);

	T dx, y;
	for(int ss=s; ss<s+2; ss++)
	{
		int s1=ss;
		if (s1>=bts.size() ) s1-=bts.size();
		currentInterval( dx, y, t, ss);
		DMatrix<T> bpu=bp[findVertex(t)][ss-s];//BernstainPolynomials(dx, y, bts[s1].dim());		
		__p[0] += (bts[s1]*bpu[0])*B[ss-s]; 
		if(d>0) __p[1] += (bts[s1]*bpu[1])*B[ss-s]+(bts[s1]*bpu[0])*ph[ss-s]; 
	}
}



#endif   /* GM_ExpoRationalCurve_H_ */

	