#include <GM_ExpoRational_Surf.h>

using namespace GM_lib;

template <class T, int n>
GM_ExpoRational_Surf<T,n>::GM_ExpoRational_Surf():PSurf<T,n>() 
{
	alfa=1.0;
	beta=1.0;
	_ctrl_poly= false;
	__material = Ruby;	
	time=0.0;
	_usePreEval=false;
	_fromFile=false;
	setIdentity("GM_ExpoRational_Surf");
}

template <class T, int n>
GM_ExpoRational_Surf<T,n>::GM_ExpoRational_Surf(GM_ExpoRational_Surf<T,n>& s):PSurf<T,n>() 
{
	alfa		=	s.alfa;
	beta		=	s.beta;
	_clu		=   s._clu;
	_clv		=   s._clv;
	_ctrl_poly	=	s._ctrl_poly;
	__material	=	s.__material;
	__vertex	=	s.__vertex;
	time		=	s.time;
	_usePreEval	=	s._usePreEval;
	_c			=	s._c;			
	_ipos		=	s._ipos;		
	_editPoints	=	s._editPoints;
	_iPoints	=	s._iPoints;	
	_u			=	s._u;
	_v			=	s._v;
	_Bv			=	s._Bv;
	_Bu			=	s._Bu;
	_Phiu		=	s._Phiu;
	_Phiv		=	s._Phiv;
	_Phiuk		=	s._Phiuk;
	_Phivk		=	s._Phivk;
	_fromFile	=	false;
	setIdentity(s._ident);
}

template <class T, int n>
GM_ExpoRational_Surf<T,n>::GM_ExpoRational_Surf(const DMatrix<DMatrix<Vector<T,n> > >& c,const Array<T>& u,const Array<T>& v, bool clu, bool clv):PSurf<T,n>(),_c(c),_u(u),_v(v) 
{
	alfa=1.0;
	beta=1.0;
	_clu=clu;
	_clv=clv;
	_ctrl_poly= false;
	__material = Ruby;	
	time=0.0;
	_usePreEval=true;
	_fromFile=false;
	setIdentity("GM_ExpoRational_Surf");
	init();
	// setDisplayMode(2);
}

template <class T, int n>
GM_ExpoRational_Surf<T,n>::GM_ExpoRational_Surf(PSurf<T,n>* c,int m1, int m2, int d1, int d2):PSurf<T,n>(*c) 
{
	alfa=1.0;
	beta=1.0;

	if(c->typeId()==1010) ((GM_ExpoRational_Surf<float,3> *)c)->setPreEval(false);
	_clu = c->closedU();
	_clv = c->closedV();
	_ctrl_poly= false;
	time=0.0;
	_usePreEval=true;
	_fromFile=false;
	_u.setSize(m1+2);
	_v.setSize(m2+2);
	_c.setDim(m1,m2);
	T u=c->startPar_u();
	T v=c->startPar_v();
	T du=c->deltaPar_u()/(T)(_u.size()-3+(int)_clu);
	T dv=c->deltaPar_v()/(T)(_v.size()-3+(int)_clv);

	_u[0]=u;  if(_clu) _u[0]-=du;
	_v[0]=v;  if(_clv) _v[0]-=dv;
	for(int r=1; r<_u.size()-1+(int)_clu; r++) _u[r]=u+(r-1)*du;
	for(int s=1; s<_v.size()-1+(int)_clv; s++) _v[s]=v+(s-1)*dv;
	if(!_clu) _u[_u.size()-1]=_u[_u.size()-2];
	if(!_clv) _v[_v.size()-1]=_v[_v.size()-2];

	DMatrix<Vector<T,3> > p;
	for(int r=0; r<m1; r++)
		for(int s=0; s<m2; s++) _c[r][s]=c->evaluate(_u[r+1],_v[s+1],d1,d2);

	if(c->typeId()==1010) ((GM_ExpoRational_Surf<float,3> *)c)->setPreEval(true);
	setIdentity("GM_ExpoRational_Surf");
	init();
	// setDisplayMode(2);
}

template <class T, int n>
void GM_ExpoRational_Surf<T,n>::init()
{
	_initOK=false;
	_type_id = 1010;

	if(!_fromFile) 
	{
	
	// for(int r=0; r<_c.dim1(); r++)
	//	for(int s=0; s<_c.dim2(); s++)
	//		_ipos[r][s]=Arrow<T,n>(_c[r][s][0][0],Vector<T,n>(_c[r][s][0][0]));

		_localPatch.setDim(_c.dim1()+(int)_clu,_c.dim2()+(int)_clv);
		for(int r=0; r<_c.dim1(); r++)
			for(int s=0; s<_c.dim2(); s++)
			{
				_localPatch[r][s]=new BezierSurf<T,n>(_c[r][s],this,true,mapToLocal(r+1,s+1,_u[r+1],_v[s+1]),Point2D<T>(_u[r+2]-_u[r],_v[s+2]-_v[s]));
				_localPatch[r][s]->resetPlot(__sam1,__sam2,true);
				_localPatch[r][s]->setMaterial(Pewter);
				_localPatch[r][s]->setCollapsed(true);
				_localPatch[r][s]->setSelected(false);	
				insert(_localPatch[r][s]);	
			}
			if(_clu) for(int s=0; s<_c.dim2(); s++) _localPatch[_c.dim1()][s]=_localPatch[0][s];
			if(_clv) for(int r=0; r<_c.dim1(); r++) _localPatch[r][_c.dim2()]=_localPatch[r][0];
			if(_clv && _clu) _localPatch[_c.dim1()][_c.dim2()]=_localPatch[0][0];
	}
	
	_ipos.setDim(_localPatch.dim1(),_localPatch.dim2());
	for (int r=0; r<_localPatch.dim1();r++) 
	    for(int s=0; s<_localPatch.dim2();s++)
	    {
			_ipos[r][s].setPos((_localPatch[r][s]->getPolator(mapToLocal(r+1,s+1,_u[r+1],_v[s+1]),0,0))[0][0]);
			_ipos[r][s].setDir(_ipos[r][s].pos());
	    }
	
	resetPlot(__sam1,__sam2,true);
	_initOK=true;
}

template <class T, int n>
void GM_ExpoRational_Surf<T,n>::edit(DisplayObject* lp)
{
	if(_initOK)
	{
		int r,s;
		for(r=0; r<_localPatch.dim1(); r++) 
			for(s=0;s<_localPatch.dim2();s++)
			{ 
				if(lp==_localPatch[r][s]) goto out;
			}
out:		// r,s found

			_ipos[r][s].setPos((_localPatch[r][s]->getPolator(mapToLocal(r+1,s+1,_u[r+1],_v[s+1]),0,0))[0][0]);
			Vector<T,n> diff=_ipos[r][s].pos()-_ipos[r][s].dir();
			_ipos[r][s].setDir(_ipos[r][s].pos());
			_localPatch[r][s]->updateCoeffs(-diff);

			edit(r+1,s+1);	// update patch  __vertex
	}
}

template <class T, int n>
void GM_ExpoRational_Surf<T,n>::edit(int r0, int s0)
{
    int r[2],s[2];
	r[0]=r0-1; r[1]=r0;
	s[0]=s0-1; s[1]=s0;

	if(r[0]<1)									r[0] = (_clu ? _localPatch.dim1():0) - 1;
	else if(!_clu && (r[1] == (_localPatch.dim1())))     r[1] = -1;
	if(s[0]<1)									s[0] = (_clv ? _localPatch.dim2():0) - 1;
	else if(!_clv && (s[1] == (_localPatch.dim2())))     s[1] = -1;

	T		du = deltaPar_u()/__vertex.dim1();
	T		dv = deltaPar_v()/(__vertex.dim2()/2-1);
	double	dr =  __vertex.dim1()/double(_localPatch.dim1()-1);
	double	ds = (__vertex.dim2()/2-1)/double(_localPatch.dim2()-1);

	for(int ir=0; ir<2; ir++)
		if(r[ir] >= 0)
		{
			int rr1 = (r[ir]-1)*dr+0.999;
			int rr2 = (r[ir])*dr+1.001;
			for(int is=0; is<2; is++)
				if(s[is] >= 0)
				{
					int ss1 = (s[is]-1)*ds+0.999;
					int ss2 = (s[is])*ds+1.001;
					for(int i=rr1; i<rr2; i++)
						for(int j=ss1; j<ss2; j++)
						{
							evaluate(startPar_u()+i*du, startPar_v()+j*dv,1,1);
							Point<GLfloat,n>		ps = __p[0][0].toFloat();
							UnitVector<GLfloat,n>	ns = __n.toFloat();
							Arrow<GLfloat,n>		as = Arrow<GLfloat,n>(ps,ns);
							if(i==0)					__vertex[i][2*j]     = as;
							else if(i==__vertex.dim1()) __vertex[i-1][2*j+1] = as;
							else	 __vertex[i][2*j] = __vertex[i-1][2*j+1] = as;
							// updateSurroundingSphere(ps);
						}
				}
		}
		
}

template <class T, int n>
void  GM_ExpoRational_Surf<T,n>::_makeSample(DMatrix<Arrow<T,n> >& a, int m1, int m2)
{
	if(true)//(m1!=__sam1 || m2!=__sam2) || _Bu.dim1()==0) 
	{
		_Bu.setDim(m1,m2);
		_Phiu.setDim(m1,m2);
		_Phiuk.setDim(m1,m2);
		_Bv.setDim(m1,m2);
		_Phiv.setDim(m1,m2);
		_Phivk.setDim(m1,m2);

		// _eval
		T u,v;
		T du = deltaPar_u()/(m1-1);
		T dv = deltaPar_v()/(m2-1);

		for(int si=0; si<m1; si++)
		{
			u = startPar_u() + si*du; 
			for(int sj=0;sj<m2; sj++)
			{			    
				v = startPar_v() + sj*dv;			
				for(r1=1; r1<_u.size()-3+(int)_clu ;r1++) if(_u[r1+1] > u) break;
				for(s1=1; s1<_v.size()-3+(int)_clv; s1++) if(_v[s1+1] > v) break;	
				Vector<T,2> Bu;
				Vector<T,2> Bv;
				Vector<T,2> phu;
				Vector<T,2> phv;
				Vector<T,2> phuk;
				Vector<T,2> phvk;

				// Basis Bk(t)
				ud=true; // Working in U direction
				T sc=integral(_u[r1],_u[r1+1]);
				Bu[1]=integral(_u[r1],u)/sc;
				Bu[0]=1.0-Bu[1]; 
				phu[1]=phi_u(u)/sc;
				phu[0]=-phu[1];
				phuk[1]=phi_kernel_der_u(u)/sc;
				phuk[0]=phuk[1];

				ud=false; // Working in V direction
				sc=integral(_v[s1],_v[s1+1]);
				Bv[1]=integral(_v[s1],v)/sc;
				Bv[0]=1.0-Bv[1]; 
				phv[1]=phi_v(v)/sc;
				phv[0]=-phv[1];
				phvk[1]=phi_kernel_der_v(v)/sc;
				phvk[0]=phvk[1];

				_Bu[si][sj]=Bu[1];
				_Bv[si][sj]=Bv[1];
				_Phiu[si][sj]=phu[1];
				_Phiv[si][sj]=phv[1];
				_Phiuk[si][sj]=phuk[1];
				_Phivk[si][sj]=phvk[1];
			}
		}
	} // end if precalc
	_usePreEval=true;
	PSurf<T,n>::_makeSample(a,m1,m2);
}

template <class T, int n>
inline
void GM_ExpoRational_Surf<T,n>::_eval(T u, T v, int d1, int d2)
{   
	// First, find the intervals, which patch should we use, note differences in indices in knot vectors and _localPatch
	for(r1=1; r1<_u.size()-3+(int)_clu; r1++) if(_u[r1+1] > u) break;
	for(s1=1; s1<_v.size()-3+(int)_clv; s1++) if(_v[s1+1] > v) break;	

	Vector<T,2> Bu;   // Expo-Rational Basis
	Vector<T,2> Bv;
	Vector<T,2> phu;  // it's derivative
	Vector<T,2> phv;
	Vector<T,2> phuk; // and the derivative of the kernel
	Vector<T,2> phvk;

	__p.setDim(d1+1,d2+1);
	for(int i=0;i<__p.dim1();i++)
		for(int j=0;j<__p.dim2();j++) __p[i][j] = Vector<T,n>(0.0);

	if(_usePreEval) 
	{
		int us=0, vs=0;
		findIndex(u,v,us,vs);		  			// in pre computed grid
		Bu[1]=_Bu[us][vs];            			Bv[1]=_Bv[us][vs];
		Bu[0]=1.0-Bu[1];			  			Bv[0]=1.0-Bv[1]; 
		phu[1]=_Phiu[us][vs];		  			phv[1]=_Phiv[us][vs];
		phu[0]=-phu[1];				  			phv[0]=-phv[1];
		phuk[1]=_Phiuk[us][vs];		  			phvk[1]=_Phivk[us][vs];
		phuk[0]=phuk[1];			  			phvk[0]=phvk[1];
	} 
	else
	{
		// Basis Bk(t)
		ud=true;						        
		T sc=integral(_u[r1],_u[r1+1]);		    
		Bu[1]=integral(_u[r1],u)/sc;		    
		Bu[0]=1.0-Bu[1];					    
		phu[1]=phi_u(u)/sc;					    
		phu[0]=-phu[1];						    
		phuk[1]=phi_kernel_der_u(u)/sc;		    
		phuk[0]=phuk[1];	
		ud=false; // Working direction , must be set because of integral() and trapez()
		sc=integral(_v[s1],_v[s1+1]);
		Bv[1]=integral(_v[s1],v)/sc;
		Bv[0]=1.0-Bv[1]; 
		phv[1]=phi_v(v)/sc;
		phv[0]=-phv[1];
		phvk[1]=phi_kernel_der_v(v)/sc;
		phvk[0]=phvk[1];

	} 

	for(int r=r1; r<r1+2; r++)
	{
		for(int s=s1; s<s1+2; s++)
		{
			DMatrix<Point<T,n> > f = _localPatch[r-1][s-1]->getPolator(mapToLocal(r,s,u,v), d1,d2);
			Array<DVector<T> > pre, post;
			pre.setSize(d2+1); post.setSize(d1+1);
			for(int i=0; i<(d2+1);i++) { pre[i].setDim(f.dim2());  pre[i].clear(); }
			for(int i=0; i<(d1+1);i++) { post[i].setDim(f.dim1()); post[i].clear(); }
			pre[0][0]=Bv[s-s1];	
			if(d2>0) 
			{
				pre[1][1]=pre[0][0];						
				pre[1][0]=(phv[s-s1]);  
			}
			if(d2>1) 
			{
				pre[2][2]=pre[1][1];						
				pre[2][1]=pre[1][0]*2;						
				pre[2][0]=pre[1][0]*phvk[s-s1];	
			}
			post[0][0]=Bu[r-r1];
			if(d1>0) 
			{
				post[1][1]=post[0][0];
				post[1][0]=(phu[r-r1]); 
			}
			if(d1>1) 
			{
				post[2][2]=post[1][1];
				post[2][1]=post[1][0]*2;
				post[2][0]=post[1][0]*phuk[r-r1]; 
			}
			for(int o=0; o<(d1+1); o++)
				for(int p=0; p<(d2+1); p++)
				{
					__p[o][p]+=pre[p]*(f*post[o]);
				}
		}
	}
	if(d1>0 && d2>0 ) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}


template <class T, int n>
inline 
T GM_ExpoRational_Surf<T,n>::phi_kernel_der_u(T t)
{
	int sp=r1+1, s=r1; 
	T tmi = t-0.5*(_u[sp]+_u[s]);
	T dem = (t-_u[s])*(_u[sp]-t);
	T x=tmi*tmi;
	T y=pow((T)dem,(T)alfa);
	T x_der=2*tmi;
	T y_der=pow((T)(_u[sp]-t),(T)alfa)-alfa*pow((T)(_u[sp]-t),(T)(alfa-1.0))*(t-_u[s]);
	T der=x_der*y-x*y_der/(y*y);
	if (t>_u[s] && t<_u[sp]) return -beta*der; 
	else					 return T(0);
}
template <class T, int n>
inline 
T GM_ExpoRational_Surf<T,n>::phi_kernel_der_v(T t)
{
	int sp=s1+1, s=s1; 
	T tmi = t-0.5*(_v[sp]+_v[s]);
	T dem = (t-_v[s])*(_v[sp]-t);
	T x=tmi*tmi;
	T y=pow((T)dem,(T)alfa);
	T x_der=2*tmi;
	T y_der=pow((T)(_v[sp]-t),(T)alfa)-alfa*pow((T)(_v[sp]-t),(T)(alfa-1.0))*(t-_v[s]);
	T der=x_der*y-x*y_der/(y*y);
	if (t>_v[s] && t<_v[sp]) return -beta*der; 
	else					 return T(0);
}

template <class T, int n>
inline
T GM_ExpoRational_Surf<T,n>::phi_u(T t)     
{
	int rp=r1+1, r=r1; 
	T tmi = t-0.5*(_u[rp]+_u[r]);
	T dem = (t-_u[r])*(_u[rp]-t);
	if (t>_u[r] && t<_u[rp]) return exp(- beta*((tmi*tmi/pow((T)dem,(T)alfa)))); // ksi1=1.0;
	else					 return T(0);
}
template <class T, int n>
inline
T GM_ExpoRational_Surf<T,n>::phi_v(T t)     
{
	int sp=s1+1, s=s1; 
	T tmi = t-0.5*(_v[sp]+_v[s]);
	T dem = (t-_v[s])*(_v[sp]-t);
	if (t>_v[s] && t<_v[sp]) return exp(- beta*((tmi*tmi/pow((T)dem,(T)alfa)))); // ksi1=1.0;
	else					 return T(0);
}

// The bottom level functions are split in two directions at this level to take advantage of 
// hyper-threading and speculative pre-fetch and processing
// This should be tested for cache hits/miss
// later on with larger cache, the pre-evaluation should be sized according to available caches.
// also the size of onboard memory on the graphic card should be considered

template <class T, int n>
double GM_ExpoRational_Surf<T,n>::trapez(T a, T b,int nn)  // I(0,n)
{	
	int		k = pow(2,nn);
	double	h = (b-a)/k;
	double	sum = 0.0;
	if(ud)
	{	
		for (int i=1; i<k; i++) sum += phi_u(a+h*i);
		sum *= h;
		sum += (phi_u(a)+phi_u(b))*0.5*h;
	}
	else
	{
		for (int i=1; i<k; i++) sum += phi_v(a+h*i);
		sum *= h;
		sum += (phi_v(a)+phi_v(b))*0.5*h;
	}
	return sum;
}

template <class T, int n>
T GM_ExpoRational_Surf<T,n>::integral(T a, T b) //  I(v,n) almost like p. 23 (5.6 and 5.7)
{	
	int stp=7;
	static GM_lib::DMatrix<T> In(10,10);

	In[0][0]=trapez(a,b,0);
	In[0][1]=trapez(a,b,1);

	In[1][0]=(pow(2,2*1)*In[0][1]-In[0][0])/(pow(2,2*1)-1.0);

	for(int v=1;v<stp+1;v++) 
	{
		In[0][v]=trapez(a,b,v);
		for(int k=1;k<=v;k++) In[k][v-k]=(pow(2,2*k)*In[k-1][v-k+1]-In[k-1][v-k])/(pow(2,2*k)-1.0);
	}
	return In[stp][0];
}


template <class T, int n>
void GM_ExpoRational_Surf<T,n>::localDisplay()
{
	// Keep this for testing purposes, in finished version it should be taken care of PSurf
	PSurf<T,n>::localDisplay(); 
}

template <class T, int n>
void GM_ExpoRational_Surf<T,n>:: scale(const Point<GLfloat,3>& scale_factor)
{
	for (int i=0; i<_c.dim1(); i++)
		for(int j=0;j<_c.dim2(); j++)
		{
			_c[i][j][0][0]%=scale_factor;
			_ipos[i][j].setPos(_ipos[i][j].pos()%scale_factor);
			_ipos[i][j].setDir(_ipos[i][j].dir()%scale_factor);
		}
    for(int i=0; i<_object.size(); i++) _object[i]->scale(scale_factor);
	// DisplayObject::scale(scale_factor);
	resetPlot(0,0,true);
}