
//*****************************************************************
// GM_ExpoRational_Surf.h: interface for the ExpoRational surface.
//*****************************************************************
#ifndef GM_ExpoRational_Surf_H
#define GM_ExpoRational_Surf_H


#include "GM_PSurf.h"
#include "GM_Selector.h"
#include <GM_SelectorGrid.h>
#include <GM_BezierSurf.h>

namespace GM_lib{

template <class T, int n>
class GM_ExpoRational_Surf: public PSurf<T,n>
{
	char							_ident[255];
	bool							_usePreEval;
	bool							_fromFile;
	bool							_ctrl_poly;
	bool							_clu;
	bool							_clv;
	bool							_initOK;
	DMatrix<T>			            _Bv,_Bu,_Phiu, _Phiv, _Phiuk, _Phivk;

	void init();
	void _eval(T u, T v, int d1, int d2);

	void findIndex(T u,T v, int& i, int& j) {  i= (__sam1-1)*(u-startPar_u())/(deltaPar_u())+0.1; 
											   j= (__sam2-1)*(v-startPar_v())/(deltaPar_v())+0.1;}

	T phi_u(T t);
	T phi_v(T t);
	T phi_kernel_der_u(T t);
	T phi_kernel_der_v(T t);

	T integral(T a, T b);
	double trapez(T a, T b,int nn);  // I(0,n)
	int s1,r1;  // current knot
	bool ud;    // ud is true, when calculating integrals in u-direction, false in v-direction

	void _makeSample(DMatrix<Arrow<T,n> >& a, int m1, int m2); //needed for preEvaluation

protected:
	DMatrix<DMatrix<Vector<T,n> > >			_c;				// Lagrange interpolating points
	DMatrix<Arrow<T,n> >					_ipos;			// work store for interpolating points
	DMatrix<BezierSurf<T,n>* >				_localPatch;    // The blending patches which stores the interpolant ? 
	Array<T>								_u;				// Knot vector - multiple knots at end
	Array<T>								_v;             // Knot vector - check size problem regarding open/closed

	double time;

	double alfa;		// may also have left and right hand values
	double beta;
	// double gamma;  
	// double sigma;
	// double lambda;

	T _st_p_u()		{ return _u[1]; }
	T _end_p_u()	{ return _u.last(); }
	T _st_p_v()		{ return _v[1]; }
	T _end_p_v()	{ return _v.last(); }
	bool closedU()  { return _clu; }
	bool closedV()  { return _clv; }

    void localDisplay();
	void localSimulate(double dt)
	{		
		time+=dt;
		if(time>M_2PI)time=0.0;
		 // rotate(dt,surroundingSphere().pos(),UnitVector3D<float>(0.0,0.0,1.0));
	}

public:
	GM_ExpoRational_Surf();
	GM_ExpoRational_Surf(GM_ExpoRational_Surf<T,n>& s);
	GM_ExpoRational_Surf(const DMatrix<DMatrix<Vector<T,n> > >& c,const Array<T>& u,const Array<T>& v, bool clu=false, bool clv=false);
	GM_ExpoRational_Surf(PSurf<T,n>* c,int m1, int m2, int d1=2, int d2=2);


template <class T_Stream>
	GM_ExpoRational_Surf(T_Stream& in, int st):PSurf<T,n>(in,st) 
	{
		alfa=1.0;
		beta=1.0;
		_ctrl_poly= false;
		time=0.0;
		_usePreEval=false;
		_fromFile=false;
		prIn(in);
	}


virtual	~GM_ExpoRational_Surf() { }
const char* identity() const { return _ident;}
void setIdentity(const char *id,int s=0) { if (s>=254) s=254; if(s==0) s=22; for(int i=0;i<s;i++) _ident[i]=*(id+i); _ident[s+1]=0; }
void togglePoly() { _ctrl_poly=!ctrl_poly; }
void togglePreEval() { _usePreEval=!_usePreEval; }
void setPreEval(bool b) { _usePreEval=b; }
bool getPreEval() { return _usePreEval; }
void toggleSelector() 
{ 
	for(int i=0; i<_object.size(); i++) 
	{
		Selector<T,n> *s=dynamic_cast<Selector<T,n> *>(_object[i]);
		if(s) 
		{ 
			if(s->isOn()) s->allOff();
			else s->on();
		}
	}
	//_poly=!_poly;
}


Point<T,2> mapToLocal(int r, int s, T u, T v)
{
	if(r==_u.size()-1) { u-=deltaPar_u(); r-=_u.size()-2;}
	if(s==_v.size()-1) { v-=deltaPar_v(); s-=_v.size()-2;}
	T sku=(u-_u[r-1])/(_u[r+1]-_u[r-1]);
	T skv=(v-_v[s-1])/(_v[s+1]-_v[s-1]);
	 
	return Point2D<T>(sku,skv);
}

void edit(int r,int s);
void edit(DisplayObject* lp);
void editU(int idx, T u) {  _u[idx]=_u[idx-1]+u*(_u[idx+1]-_u[idx-1]);  resetPlot(0,0,true);  setPreEval(false); resetPlot(0,0,true); }
void editV(int idx, T v) {  _v[idx]=_v[idx-1]+v*(_v[idx+1]-_v[idx-1]);  resetPlot(0,0,true);  setPreEval(false); resetPlot(0,0,true); }
void setAlfa(T a) { if (a>0.0) alfa=a; }
void setBeta(T b) { if (b>0.0) beta=b; }

void scale(const Point<GLfloat,3>& scale_factor);

T gaussCurvature(T u, T v){ bool p=_usePreEval; setPreEval(false); T r=PSurf<T,n>::gaussCurvature(u,v); setPreEval(p); return r; }
T meanCurvature(T u, T v) { bool p=_usePreEval; setPreEval(false); T r=PSurf<T,n>::meanCurvature(u,v);  setPreEval(p); return r; }

template <class T_Stream>
T_Stream& prOut(T_Stream& out)
{
    
	out << (int)_clu << element_separator << (int)_clv << group_separator;
	_u.setStreamMode(true); _v.setStreamMode(true);
	out << _u << group_separator << _v << group_separator;
    // out <<  _c  << group_separator;
	out << _localPatch.dim1() << element_separator << _localPatch.dim2() << group_separator;
	for(int i=0; i<_localPatch.dim1(); i++)
		for(int j=0;j<_localPatch.dim2();j++)
			out <<  *(_localPatch[i][j])    << group_separator;
	// out <<  _ipos << group_separator;
	out << alfa << element_separator << beta << group_separator;
	// out << __material << group_separator;
	return out;
}

template <class T_Stream>
T_Stream& prIn(T_Stream& in)
{
	static Separator es(element_separator);
	static Separator gs(group_separator);
	
	int us,vs;
	in  >> us >> es >> vs >> gs;
	_clu=us; _clv=vs;
	_u.setStreamMode(true); 
	_v.setStreamMode(true);
	in >> _u >> gs;
	in >> _v >> gs;
    // in >> _c  >> gs;
	in >> us >> es >> vs >> gs;
	_localPatch.setDim(us,vs);
	for (int i=0; i<_localPatch.dim1();i++) 
	    for(int j=0; j<_localPatch.dim2()  ;j++)
	    {
			BezierSurf<GLfloat,3>* tmp=new BezierSurf<GLfloat,3>(in, this);
			tmp->resetPlot(__sam1,__sam2,true);
			tmp->setCollapsed(true);
			tmp->setSelected(false);	
			insert(tmp);
	        _localPatch[i][j]=tmp;
	    }
	
	// in >> _ipos >> gs;
	in >> alfa >> es >> beta >>gs;
	// in >> __material >> gs;
	_fromFile=true;
	init();
	setVisible(true);
	return in;
}

};

template <class T_Stream, class T, int n>
T_Stream& operator<<(T_Stream& out, GM_ExpoRational_Surf<T,n>& s)
{
	out << *((PSurf<T,n> *) &s);
	s.prOut(out);
	return out;
}

template <class T_Stream, class T, int n>
T_Stream& operator>>(T_Stream& in,  GM_ExpoRational_Surf<T,n>& s)
{
	in >> *((PSurf<T,n> *) &s);
	s.prIn(in);
	return in;
}

}// end GM_lib

#include <GM_ExpoRational_Surf.c>

#endif