#ifndef _GM_HEART_H_
#define _GM_HEART_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Heart : public PSurf<T,n>
{
	void _eval(T u, T v, int d1, int d2);
	void localDisplay();
	// void localSimulate(double dt);
protected:

	T _st_p_u()		{ return T(-0.98); }
	T _end_p_u()		{ return T(0.98); }
	T _st_p_v()		{ return T(-M_PI+0.0001); }
	T _end_p_v()		{ return T(M_PI-0.0001); }


public:

	Heart();
	// Heart(Heart<T,n>& s);

	virtual  
	~Heart() {}
	const char* identity() const { return " Heart "  ;}
};
} // GM_lib

using namespace GM_lib;

template <class T, int n>
Heart<T,n>::Heart():PSurf<T,n>()
{
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),4.0));
}

template <class T, int n>
void Heart<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(4,4);
	__p[0][0][0] =	(T(4)*sqrt((T)(T(1) - u*u))*pow( (T)sin((T)abs(v)) , (T)abs(v)))*sin(v);	
	__p[0][0][1] =	(T(4)*sqrt((T)(T(1) - u*u))*pow( (T)sin((T)abs(v)) , (T)abs(v)))*cos(v);	
	__p[0][0][2] =	u;	

	if(d1) //u
	{
		__p[1][0][0] =	-T(4)*pow(sin(abs(v)),abs(v))*sin(v)*u/(sqrt(T(1)-u*u));
		__p[1][0][1] =	 -T(4)*pow(sin(abs(v)),abs(v))*cos(v)*u/(sqrt(T(1)-u*u));
		__p[1][0][2] =	T(1);
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	T(0);
		__p[2][0][1] =	T(0);
		__p[2][0][2] =	T(0);
	}
	if(d2) //v
	{
		__p[0][1][0] =	T(4)*sqrt(T(1)-u*u)*pow(sin(abs(v)),abs(v))*(/*abs(1,v)*/log(sin(abs(v)))+abs(v)*cos(abs(v))
						/*abs(1,v)*//sin(abs(v)))*sin(v)+T(4)*sqrt(T(1)-u*u)*pow(sin(abs(v)),abs(v))*cos(v);
		__p[0][1][1] =	T(4)*sqrt(T(1)-u*u)*pow(sin(abs(v)),abs(v))*(/*abs(1,v)*/log(sin(abs(v)))+abs(v)*cos(abs(v))
						/*abs(1,v)*//(abs(v)))*cos(v)-T(4)*sqrt(T(1)-u*u)*pow(sin(abs(v)),abs(v))*sin(v);
		__p[0][1][2] =	T(0);
	}					
	if(d2>1) //vv
	{
		__p[0][2][0] =	T(0);	
		__p[0][2][1] =	T(0);	
		__p[0][2][2] =	T(0);	
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =	T(0);	
		__p[1][1][1] =	T(0);	
		__p[1][1][2] =	T(0);	
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =	T(0);	
		__p[2][1][1] =	T(0);	
		__p[2][1][2] =	T(0);	
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =	T(0);	
		__p[1][2][1] =	T(0);	
		__p[1][2][2] =	T(0);	
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =	T(0);	
		__p[2][2][1] =	T(0);	
		__p[2][2][2] =	T(0);	
	}
	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}

template <class T, int n>
void Heart<T,n>::localDisplay()
{
	// r_mat.mult();
	PSurf<T,n>::localDisplay();
}
 #endif //_GM_HEART_H_
