
#ifndef _GM_HELICOID_H_
#define _GM_HELICOID_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Helicoid : public PSurf<T,n>
{
	T _a; //scaleX
	T _b; //scaleY
	T _c; //scaleZ

	void _eval(T u, T v, int d1, int d2);
	//void localDisplay();
	// void localSimulate(double dt);
protected:

	T _st_p_u()				{ return T(0); }
	T _end_p_u()			{ return T(15.0); }
	T _st_p_v()				{ return T(-1.0); }
	T _end_p_v()			{ return T(1.0); }

public:
	Helicoid();
	Helicoid(T scalex, T scaley, T scalez);
	//Helicoid(Helicoid<T,n>& s);
 
	~Helicoid() {}

	void setConstants(T scalex, T scaley, T scalez);

	const char* identity() const { return " Helicoid ";}
};
} // GM_lib

using namespace GM_lib;
template <class T, int n>
Helicoid<T,n>::Helicoid():PSurf<T,n>()
{
	setConstants(T(1),T(1),T(.32));
}

template <class T, int n>
Helicoid<T,n>::Helicoid(T scalex, T scaley, T scalez):PSurf<T,n>()
{
	setConstants(scalex, scaley, scalez);
}

template <class T, int n>
void Helicoid<T,n>::setConstants(T scalex, T scaley, T scalez)
{
	_a = scalex; 
	_b = scaley;
	_c = scalez;
	
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),15.0));
}
template <class T, int n>
void Helicoid<T,n>::_eval(T u, T v, int d1,int d2)
{

	__p.setDim(4,4);
	__p[0][0][0] =	_a*sin(u)*v;
	__p[0][0][1] =	_b*v*cos(u);
	__p[0][0][2] =	_c*u;

	if(d1) //u
	{
		__p[1][0][0] =	_a*v*cos(u);
		__p[1][0][1] =	-_b*sin(u)*v;
		__p[1][0][2] =	_c;
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	-_a*sin(u)*v;
		__p[2][0][1] =	-_b*v*cos(u);
		__p[2][0][2] =	T(0);
	}
	if(d2) //v	
	{
		__p[0][1][0] =	_a*sin(u);	
		__p[0][1][1] =	_b*cos(u);	
		__p[0][1][2] =	T(0);	  	
	}
	if(d2>1) //vv
	{
		__p[0][2][0] =	T(0);	
		__p[0][2][1] =	T(0);	
		__p[0][2][2] =	T(0);	
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =	_a*cos(u);
		__p[1][1][1] =	-_b*sin(u);
		__p[1][1][2] =	T(0);
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =	-_a*sin(u);
		__p[2][1][1] =	-_b*cos(u);
		__p[2][1][2] =	T(0);
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =	T(0);	
		__p[1][2][1] =	T(0);	
		__p[1][2][2] =	T(0);	
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =	T(0);	
		__p[2][2][1] =	T(0);	
		__p[2][2][2] =	T(0);	
	}

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];

}

#endif //_GM_HELICOID_H_

