
//*************************************************************************************
// GM_HermiteCurve.h: interface for the HermiteCurve classes.
//*************************************************************************************
#ifndef GM_HERMITECURVE_H_
#define GM_HERMITECURVE_H_

#include "GM_PCurve.h"

namespace GM_lib{

template <class T, int n>
class HermiteCurve: public PCurve<T,n>
{
	void init();

	void localSimulate(double dt){}

 protected:
	Arrow<T,n> _p1;
	Arrow<T,n> _p2;

	void _eval(T t, int d);

	T _st_p() { return T(0); }
	T _end_p(){ return T(1); }

 public:
	HermiteCurve(const Arrow<T,n>& p1,const Arrow<T,n>& p2):PCurve<T,n>(),_p1(p1),_p2(p2) { init();}
	HermiteCurve(const Point<T,n>& p1,const Vector<T,n>& v1,const Point<T,n>& p2,const Vector<T,n>& v2)
		:PCurve<T,n>(),_p1(p1,v1),_p2(p2,v2) { init();}
	HermiteCurve(const HermiteCurve<T,n>& c):PCurve<T,n>(c) { _p1=c._p1; _p2=c._p2;}

	~HermiteCurve() {}

	PCurve<T,n>* split(T t) { return NULL;}

	const char* identity() const { return "HERMITE-CURVE";}
};


} // end namespace GM_lib


#include <GM_DMatrix.h>
using namespace GM_lib;

template <class T, int n>
void HermiteCurve<T,n>::init()
{
	__amb.set(0.7,0.5,0.0,1.0); // rd
	__dif.set(0.0,1.0,0.0,1.0); // rd
	__spc.set(1.0,1.0,1.0,1.0); // hvit
	 resetPlot(0.0005);
}


template <class T, int n>
void HermiteCurve<T,n>::_eval(T t, int d)
{
	if(d<=__d && t==__t) return;
	__t=t;
	__d=d;

	static GM_lib::DMatrix<T> b(4,4);
	static bool _init=false;
	if(!_init)
	{
		b[3][0] = 12;
		b[3][1] = -12;
		b[3][2] = 6;
		b[3][3] = 6;
		_init = true;
	}

	int i;
	T t2 = t*t;
	T t3 = t2*t;

	b[0][0] = 2*t3-3*t2+1;
	b[0][1] = 3*t2-2*t3;
	b[0][2] = t3-2*t2+t;
	b[0][3] = t3-t2;

	if(d > 0)
	{
		b[1][0] = 6*t2-6*t;
		b[1][1] = 6*t-6*t2;
		b[1][2] = 3*t2-4*t+1;
		b[1][3] = 3*t2-2*t;
	}
	if(d > 1)
	{
		b[2][0] = 12*t-6;
		b[2][1] = 6-12*t;
		b[2][2] = 6*t-4;
		b[2][3] = 6*t-2;
	}


	__p.setSize(d+1);

	int dd = d<4?d:3;
	for(i=0;i<=dd;i++)
		__p[i] = _p1.pos()*b[i][0]+_p2.pos()*b[i][1]+_p1.dir()*b[i][2]+_p2.dir()*b[i][3];
	for(;i<=d;i++)
		__p[i] =  Point<T,n>(T(0.0));
}


#endif   /* GM_HERMITECURVE_H_ */