
#ifndef _GM_INSIDE_OUT_TORUS_H_
#define _GM_INSIDE_OUT_TORUS_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Inside_Out_Torus : public PSurf<T,n>
{
	T _a;		//Wheelradius
	T _b;		//Tuberadius1
	T _c;		//Tuberadius2

	void _eval(T u, T v, int d1, int d2);
	void localDisplay();
	// void localSimulate(double dt);
protected:

	T _st_p_u()			{ return T(0.0); }
	T _end_p_u()			{ return T(2*M_PI); }
	T _st_p_v()			{ return T(0.0); }
	T _end_p_v()			{ return T(2*M_PI); }

public:
	Inside_Out_Torus();
	Inside_Out_Torus(T wheelrad, T tuberad1, T tyberad2);
//	Inside_Out_Torus(Inside_Out_Torus<T,n>& s);

	~Inside_Out_Torus() {}

	void setConstants(T wheelrad, T tuberad1, T tuberad2);

	const char* identity() const { return " Inside_Out_Torus "  ;}
};
} // GM_lib

using namespace GM_lib;

template <class T, int n>
Inside_Out_Torus<T,n>::Inside_Out_Torus():PSurf<T,n>()
{
	setConstants(T(1), T(3), T(3));
}

template <class T, int n>
Inside_Out_Torus<T,n>::Inside_Out_Torus(T wheelrad, T tuberad1, T tuberad2):PSurf<T,n>()
{
	setConstants(wheelrad, tuberad1, tuberad2);
}

template<class T, int n>
void Inside_Out_Torus<T,n>::setConstants(T wheelrad, T tuberad1, T tuberad2)
{
	_a = wheelrad;
	_b = tuberad1;
	_c = tuberad2;

	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),_a+max(_b,_c)));
}

template <class T, int n>
void Inside_Out_Torus<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(4,4);
	__p[0][0][0] =			cos(u)*(_b*cos(v)+_a);
	__p[0][0][1] =			sin(u)*(_b*cos(v)+_a);
	__p[0][0][2] =			_c*sin(v);

	if(d1) //u
	{
		__p[1][0][0] =		-sin(u)*(_b*cos(v)+_a);
		__p[1][0][1] =		cos(u)*(_b*cos(v)+_a);
		__p[1][0][2] =		0.0;
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =		-cos(u)*(_b*cos(v)+_a);
		__p[2][0][1] =		-sin(u)*(_b*cos(v)+_a);
		__p[2][0][2] =		0.0;
	}
	if(d2) //v
	{
		__p[0][1][0] =		-_b*cos(u)*sin(v);
		__p[0][1][1] =		-_b*sin(u)*sin(v);
		__p[0][1][2] =		_c*cos(v);
	}
	if(d2>1) //vv
	{
		__p[0][2][0] =		-_b*cos(u)*cos(v);
		__p[0][2][1] =		-_b*sin(u)*cos(v);
		__p[0][2][2] =		-_c*sin(v);
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =		_b*sin(u)*sin(v);
		__p[1][1][1] =		-_b*cos(u)*sin(v);
		__p[1][1][2] =		0.0;
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =		_b*cos(u)*sin(v);
		__p[2][1][1] =		_b*sin(u)*sin(v);
		__p[2][1][2] =		0.0;
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =		_b*sin(u)*cos(v);
		__p[1][2][1] =		-_b*cos(u)*cos(v);
		__p[1][2][2] =		0.0;
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =		_b*cos(u)*cos(v);
		__p[2][2][1] =		_b*sin(u)*cos(v);
		__p[2][2][2] =		0.0;
	}

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}
template <class T, int n>
void Inside_Out_Torus<T,n>::localDisplay()
{
	// r_mat.mult();
	PSurf<T,n>::localDisplay();
}
 #endif //_GM_INSIDE_OUT_TORUS_H_

