
#ifndef _GM_KLEINS_BOTTLE_H_
#define _GM_KLEINS_BOTTLE_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Kleins_Bottle : public PSurf<T,n>
{
	T _sr;		//scale radius
	T _sx;		//scale x
	T _sy;		//scale y

	void _eval(T u, T v, int d1, int d2);
	void localDisplay();
	// void localSimulate(double dt);
protected:

	T _st_p_u()			{ return T(0.0); }
	T _end_p_u()		{ return T(2.0*M_PI); }
	T _st_p_v()			{ return T(0.0); }
	T _end_p_v()		{ return T(2*M_PI); }

public:
	Kleins_Bottle();
	Kleins_Bottle(T scalerad, T scalex, T scaley);
	//Kleins_Bottle(Kleins_Bottle<T,n>& s);

	virtual  
	~Kleins_Bottle() {}
	
	void setConstants(T scalerad, T scalex, T scaley);

	const char* identity() const { return " Kleins_Bottle "  ;}
};
} // GM_lib

using namespace GM_lib;

template <class T, int n>
Kleins_Bottle<T,n>::Kleins_Bottle():PSurf<T,n>()
{
	setConstants(T(1),T(1.5),T(4));
}

template <class T, int n>
Kleins_Bottle<T,n>::Kleins_Bottle(T scalerad, T scalex, T scaley):PSurf<T,n>()
{
	setConstants(scalerad, scalex, scaley)
}

template <class T, int n>
void Kleins_Bottle<T,n>::setConstants(T scalerad, T scalex, T scaley)
{
	_sr = scalerad;
	_sx = scalex;
	_sy = scaley;

	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),10.0));
}
template <class T, int n>
void Kleins_Bottle<T,n>::_eval(T u, T v, int d1,int d2)
{

    __p.setDim(4,4);

	if(u>=0.0 && u<=M_PI) __p[0][0][0] = _sx*cos(u)*(T(1)+sin(u))+_sr*(T(1)-cos(u)*T(.5))*cos(u)*cos(v);
	else				  __p[0][0][0] = _sx*cos(u)*(T(1)+sin(u))+_sr*(T(1)-cos(u)*T(.5))*cos(v+M_PI);
	if(u>=0.0 && u<=M_PI) __p[0][0][1] = _sy*sin(u)+_sr*(T(1)-cos(u)*T(.5))*sin(u)*cos(v);
	else				  __p[0][0][1] = _sy*sin(u);
						  __p[0][0][2] = _sr*(T(1)-cos(u)*T(.5))*sin(v);

	if(d1)	//u
	{
		if(u>=0.0 && u<=M_PI) __p[1][0][0] = -_sx*sin(u)*T(1)-_sx+T(2)*_sx*cos(u)*cos(u)
											 +T(2)*_sr*sin(u)*T(.5)*cos(u)*cos(v)-_sr*sin(u)*cos(v)*T(1);
		else				  __p[1][0][0] = -_sx*sin(u)*T(1)-_sx+T(2)*_sx*cos(u)*cos(u)
											 +_sr*sin(u)*T(.5)*cos(v+M_PI);
		if(u>=0.0 && u<=M_PI) __p[1][0][1] = _sy*cos(u)+_sr*T(.5)*cos(v)-T(2)*_sr*cos(u)*cos(u)*cos(v)*T(.5)
											 +_sr*cos(u)*cos(v)*T(1);
		else				  __p[1][0][1] = _sy*cos(u);
							  __p[1][0][2] = _sr*sin(u)*T(.5)*sin(v);
	}

	if(d1>1)//uu
	{
		if(u>=0.0 && u<=M_PI) __p[2][0][0] = -_sx*cos(u)*T(1)-T(4)*_sx*sin(u)*cos(u)+T(4)*_sr*cos(u)*cos(u)*T(.5)*cos(v)
											 -T(2)*_sr*T(.5)*cos(v)-_sr*cos(u)*cos(v)*T(1);
		else				  __p[2][0][0] = -cos(u)*(_sx*T(1)+T(4)*_sx*sin(u)-_sr*T(.5)*cos(v+M_PI));
		if(u>=0.0 && u<=M_PI) __p[2][0][1] = -sin(u)*(_sy-T(4)*_sr*cos(v)*cos(u)*T(.5)+_sr*cos(v)*T(1));
		else				  __p[2][0][1] = -_sy*sin(u);
							  __p[2][0][2] = _sr*cos(u)*T(.5)*sin(v);
	}

	if(d2)	//v
	{
		if(u>=0.0 && u<=M_PI) __p[0][1][0] = -_sr*(T(1)-cos(u)*T(.5))*cos(u)*sin(v);
		else				  __p[0][1][0] = -_sr*(T(1)-cos(u)*T(.5))*sin(v+M_PI);
		if(u>=0.0 && u<=M_PI) __p[0][1][1] = -_sr*(T(1)-cos(u)*T(.5))*sin(u)*sin(v);
		else				  __p[0][1][1] = T(0);
							  __p[0][1][2] = _sr*(T(1)-cos(u)*T(.5))*cos(v);
		
	}

	if(d2>1) //vv
	{
		if(u>=0.0 && u<=M_PI) __p[0][2][0] = -_sr*(T(1)-cos(u)*T(.5))*cos(u)*cos(v);
		else				  __p[0][2][0] = _sr*(-T(1)+cos(u)*T(.5))*cos(v+M_PI);
		if(u>=0.0 && u<=M_PI) __p[0][2][1] = -_sr*(T(1)-cos(u)*T(.5))*sin(u)*cos(v);
		else				  __p[0][2][1] = T(0);
							  __p[0][2][2] = _sr*(-T(1)+cos(u)*T(.5))*sin(v);
	}

	if(d1 && d2) //uv
	{
		if(u>=0.0 && u<=M_PI) __p[1][1][0] = -_sr*sin(u)*sin(v)*(T(2)*cos(u)*T(.5)-T(1));
		else				  __p[1][1][0] = -_sr*sin(u)*T(.5)*sin(v+M_PI);
		if(u>=0.0 && u<=M_PI) __p[1][1][1] = _sr*sin(v)*(-T(.5)+cos(u)*cos(u)-cos(u));
		else				  __p[1][1][1] = T(0);
							  __p[1][1][2] = _sr*sin(u)*T(.5)*cos(v);
	} 

	if(d1>1 & d2)//uuv
	{
		if(u>=0.0 && u<=M_PI) __p[2][1][0] = -_sr*sin(v)*(T(2)*cos(u)*cos(u)-2.0*T(.5)-cos(u));
		else				  __p[2][1][0] = -_sr*cos(u)*T(.5)*sin(v+M_PI);
		if(u>=0.0 && u<=M_PI) __p[2][1][1] = -_sr*sin(u)*sin(v)*(T(2)*cos(u)-T(1));
		else				  __p[2][1][1] = T(0);
							  __p[2][1][2] =  _sr*cos(u)*T(.5)*cos(v);
	}

	if(d1 && d2>1) //uvv
	{
		if(u>=0.0 && u<=M_PI) __p[1][2][0] = -_sr*sin(u)*cos(v)*(cos(u)-T(1));
		else				  __p[1][2][0] = -_sr*sin(u)*T(.5)*cos(v+M_PI);
		if(u>=0.0 && u<=M_PI) __p[1][2][1] = _sr*cos(v)*(-T(.5)+cos(u)*cos(u)-cos(u));
		else				  __p[1][2][1] = T(0);
							  __p[1][2][2] = -_sr*sin(u)*T(.5)*sin(v);
	}

	if(d1>1 && d2>1) //uuvv
	{
		if(u>=0.0 && u<=M_PI) __p[2][2][0] = -_sr*cos(v)*(T(2)*cos(u)*cos(u)-T(1)-cos(u));
		else				  __p[2][2][0] = -_sr*cos(u)*T(.5)*cos(v+M_PI);
		if(u>=0.0 && u<=M_PI) __p[2][2][1] = -_sr*sin(u)*cos(v)*(T(2)*cos(u)-T(1));
		else				  __p[2][2][1] = T(0);
							  __p[2][2][2] = -_sr*cos(u)*T(.5)*sin(v);

	}

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}
template <class T, int n>
void Kleins_Bottle<T,n>::localDisplay()
{
	// r_mat.mult();
	PSurf<T,n>::localDisplay();
}
 #endif //_Kleins_Bottle_H_

