#include "GM_LX_Array.h"

namespace GM_lib {


//********************************************************
//**************** newRow     ****************************
//********************************************************

template<class T>
void LX_Array<T>::newRow(int n, T* pt)
{
	if(n<=0) return;

	no_p++;
	LX_pt<T>*  	np = new LX_pt<T>[no_p];
  int i;
 	for(i=0; i<no_p-1; i++)
	{
		np[i]  = p[i];
		p[i].p = NULL;
		p[i].size = 0;
	}
	if(pt)	np[i].p    = pt;
	else	np[i].p = new T[n];
		

	np[i].size = n;

	if(no_p > 1) delete [] p;
	p = np;

	max_elem += n;
}




//********************************************************
//**************** DESTRUCTOR ****************************
//********************************************************

template<class T>
LX_Array<T>::~LX_Array()
{
 	for(int i=0; i<no_p; i++)
		if(p[i].size > 0) {delete [] p[i].p; p[i].p=NULL; }
	if(no_p > 0) delete [] p;
}


//********************************************************
//**********  OPERATOR= THE HOLE LX_Array NEW VALUES ***
//********************************************************

template<class T>
LX_Array<T>& LX_Array<T>::operator=(const	LX_Array<T>&	other)
{
	// BBang 27.06.00 - to avoid slicing, check for assignment to self
    if(this == &other) return *this;
	clear();
	if( other.size() > 0)
	{
		newRow(other.size());
		for(int i=0; i<other.size(); i++) p[0].p[i] = other(i);
		no1 = 1;
		no2 = 0;
		no_elem = other.size();
	}

	return *this;
}


//********************************************************
//**************** STRIPELEMENTS() - NO FREE ELEMENTS ****
//********************************************************

template<class T>
void	LX_Array<T>::stripElements()
{
		// Remove surplus elements.
	for (int j,i=0; i<no_elem-1; i++)
		for (j=no_elem-1; j>i; j--)
			if((*this)[i] == (*this)[j])	removeIndex(j++);
}


//********************************************************
//**************** ALWAYSINSERT(OBJECT) - EVEN IF EXIST **
//**************** DEFAULT AT END OF LX_Array ************
//********************************************************

template<class T>
void	LX_Array<T>::alwaysInsert(const T& obj, bool first)
{
	if (no_elem == max_elem) newRow(incr);	 // Is the LX_Array full?
	
	if(first)
	{
		for(int i=no_elem; i>0; i--)	(*this)[i] = (*this)[i-1];
		p[0].p[0] = obj;
	}
	else
		p[no1].p[no2] = obj;

	no1_2pluss();
}


//********************************************************
//**************** ALWAYSINSERT(LX_Array) - EVEN IF EXIST ***
//**************** DEFAULT AT END OF LX_Array ***************
//********************************************************

template<class T>
void	LX_Array<T>::alwaysInsert(const LX_Array<T>& arr, bool first)
{
	for (int i=0; i< arr.size(); i++) 
		alwaysInsert(arr(i),first);
}


//********************************************************
//****************   REMOVEINDEX(index) -  ***************
//****************  remove element with given index ******
//********************************************************

template<class T>
bool	LX_Array<T>::removeIndex(int	index)
{
		// Check index.
	if (index < 0 || index >= no_elem)		return false;
		// Move LX_Array forward.
	no1_2minus();

	(*this)[index] = p[no1].p[no2];
	return true;
}


//********************************************************
//****************   SETSIZE(number) -  ******************
//*******  set size of active numbers not potential ******
//********************************************************

template<class T>
void	LX_Array<T>::setSize(int s)
{
	if (s <= 0)		clear();
	else if (s > max_elem)
	{
		int n = s - max_elem;
		if (n<incr) n = incr;
		newRow(n);
	}

	if (s < no_elem)
	{
		int n = no_elem - s;
		while(n>0)
			if(no2 == 0)
			{
				no1--;
				no2 = p[no1].size-1;
				n--;
			}
			else if(no2 <= n)
			{
				n -= no2;
				no2 = 0;
			}
			else
			{
				no2 -= n;
				n = 0;
			}

		no_elem = s;
	}
	else if (s > no_elem)
	{
		int n = s - no_elem;
		while(n>0)
			if(p[no1].size - no2 <= n)
			{
				n -= p[no1].size - no2;
				no1++;
				no2 = 0;
			}
			else
			{
				no2 += n;
				n = 0;
			}

		for (int i = no_elem; i < s; i++) (*this)[i] = T();
		no_elem = s;
	}
}


//********************************************************
//****************   SETMAX(number) -   ******************
//************  set size of potential numbers ************
//********************************************************

template<class T>
void	LX_Array<T>::setMax(int s, int inc)
{
 	if (s == 0)		clear();
	else if (max_elem < s)
	{
		int n = s - max_elem;

		no_p++;
		LX_pt<T>*  	np = new LX_pt<T>[no_p];
    int i;
 		for(i=0; i<no_p-1; i++)
		{
			np[i]  = p[i];
			p[i].p = NULL;
			p[i].size = 0;
		}
		np[i].p = new T[n];		
		np[i].size = n;

		if(no_p > 1) delete [] p;
		p = np;

		max_elem += n;
	}
	if(inc>0)	incr = inc;
}


//********************************************************
//******************   STRIP() -     *********************
//******************  only actual numbers  ***************
//********************************************************

template<class T>
void	LX_Array<T>::strip()
{
	if (max_elem > no_elem)
	{
//		GA_pt<T>*  	np = new GA_pt<T>[1];
// Can't find any GA_pt<> BBang 03.06.2002
// changed it to LX_pt<T>
		LX_pt<T>*  	np = new LX_pt<T>[1];
		np[0].p        = new T[no_elem];
		np[0].size     = no_elem;
		for(int i=0; i<no_elem; i++)
			np[0].p[i] = (*this)[i];

		clear();
		max_elem = no_elem;
		no_p = 1;
		p = np;
		no1=1;
		no2=0;
	}
}


//********************************************************
//******************   INDEX(object) - *******************
//********************************************************

template<class T>
int	LX_Array<T>::index(const T& obj) const
{
	for (int i=0; i<no_elem; i++)
		if ((*this)(i) == obj)		return i;

	return -1;
}


//********************************************************
//******************   TURN() -     **********************
//******************  turn the LX_Array  ********************
//********************************************************

template<class T>
void	LX_Array<T>::turn()
{
	for (int i=0, mid = no_elem/2; i<mid; i++)
		swap(i, no_elem - 1 - i);
}


//********************************************************
//******************   OPERATOR==   **********************
//******************  equality test for the hole LX_Array  **
//********************************************************

template<class T>
bool LX_Array<T>::operator==(const LX_Array<T>& a) const
{
	if (size() != a.size()) 		return false;
	for (int i=0; i < no_elem;  i++)
		if ((*this)(i) != a(i)) return false;
	return true;
}

} // end namespace GM_lib

