
//********************************************************
// Light.h: interface for the Light class.
//********************************************************
#ifndef _LIGHT_H__
#define _LIGHT_H__

#include <GM_OpenGL.h>

namespace GM_lib {

class Light  
{	
	static GLuint		_next_light;	// For automatisk navnegenerering.
	static Array<GLuint> _free_light;

	GLenum				_light_name;

	GL_Color			_ambient;
	GL_Color			_diffuse;
	GL_Color			_specular;
	bool				_disableculling; //disable culling for this light.

protected:
	static GLfloat		_min_light_contribution;

	void glLight(GLenum pn, const GL_Color& co);
	void glLight(GLenum pn, GLfloat f);
	void glLight(GLenum pn, int f);

	void glLightSun(const Vector<GLfloat,3>& pos);
	void glLightPos(const Point<GLfloat,3>& pos);
	void glLightDir(const Vector<GLfloat,3>& dir);


	const GL_Color& _getAmbient()	{ return _ambient; }
	const GL_Color& _getDiffuse()	{ return _diffuse; }
	const GL_Color& _getSpecular()	{ return _specular; }

	GLuint	_getLightName()			{ return _light_name; }

public:
	Light();	
	Light(const GL_Color& amb , const GL_Color& dif, const GL_Color& spe);

	Light(const Light&);

	virtual ~Light();

	void setColor(	const GL_Color& ambient = GL_Color(0.2,0.2,0.2),
					const GL_Color& diffuse = GL_Color(1.0,1.0,1.0), 
					const GL_Color& specular= GL_Color(1.0,1.0,1.0));

	void setIntensity(double d,int i=0);

	void on()		{ glEnable(_light_name); }
	void off()		{ glDisable(_light_name); }
	bool isOn()		{ return glIsEnabled(_light_name)? true:false; }

	void cullingOff() {_disableculling = true;};
	void cullingOn() {_disableculling = true;};
	bool cullable() {return !_disableculling;};

	virtual void lighting() {}
	virtual void culling(const Frustum&) {};

};



inline
void Light::glLight(GLenum pn, const GL_Color& co)
{
	GM_lib::glLight(_light_name, pn, co);
}

inline
void Light::glLight(GLenum pn, GLfloat f)
{
	GM_lib::glLight(_light_name, pn, f);
}

inline
void Light::glLight(GLenum pn, int f)
{
	GM_lib::glLight(_light_name, pn, f);
}

inline
void Light::glLightSun(const Vector<GLfloat,3>& dir)
{
	GM_lib::glLightSun(_light_name, dir);
}

inline
void Light::glLightPos(const Point<GLfloat,3>& pos)
{
	GM_lib::glLightPos(_light_name, pos);
}

inline
void Light::glLightDir(const Vector<GLfloat,3>& dir)
{
	GM_lib::glLightDir(_light_name, dir);
}

inline
void Light::setColor(const GL_Color& a, const GL_Color& d, const GL_Color& s)
{
	_ambient		= a;
	_diffuse		= d;
	_specular		= s;
	
	setIntensity(1.0);
}

inline
void Light::setIntensity(double d, int i)
{
	if(i==0 || i == 1)
		glLight(GL_AMBIENT,d*_ambient);
	if(i==0 || i == 2)
		glLight(GL_DIFFUSE,d*_diffuse);
	if(i==0 || i == 3)
		glLight(GL_SPECULAR,d*_specular);
}


} // end namespace GM_lib


#endif /* _LIGHT_H__ */
