
#ifndef _GM_MOEBIUS_H_
#define _GM_MOEBIUS_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Moebius : public PSurf<T,n>
{
	T _w;	//Half width
	T _r;	//Radius

	void _eval(T u, T v, int d1, int d2);
	void localDisplay();
	// void localSimulate(double dt);
protected:

	T _st_p_u()				{ return T(-1.0); }
	T _end_p_u()			{ return T(1.0); }
	T _st_p_v()				{ return T(-0.5*M_PI); }
	T _end_p_v()			{ return T(1.5*M_PI); }

public:
	Moebius();
	Moebius(T width, T radius);
	//Moebius(Moebius<T,n>& s);

	~Moebius() {}

	void setConstants(T width, T radius);

	const char* identity() const { return " Moebius "  ;}
};
} // GM_lib

using namespace GM_lib;

template <class T, int n>
Moebius<T,n>::Moebius():PSurf<T,n>()
{
	setConstants(T(1), T(2));
}

template <class T, int n>
Moebius<T,n>::Moebius(T width, T radius):PSurf<T,n>()
{
	setConstants(width, radius);
}

template <class T, int n>
void Moebius<T,n>::setConstants(T width, T radius)
{
	_w = width;
	_r = radius;
	
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),_r));
}

template <class T, int n>
void Moebius<T,n>::_eval(T u, T v, int d1,int d2)
{

	__p.setDim(4,4);
	__p[0][0][0] =	(_r+_w*u*cos(T(.5)*v))*cos(v);
	__p[0][0][1] =	(_r+_w*u*cos(T(.5)*v))*sin(v);
	__p[0][0][2] =	_w*u*sin(T(.5)*v);

	if(d1) //u
	{
		__p[1][0][0] =		_w*cos(T(.5)*v)*cos(v);
		__p[1][0][1] =		_w*cos(T(.5)*v)*sin(v);
		__p[1][0][2] =		_w*sin(T(.5)*v);
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	T(0);
		__p[2][0][1] =	T(0);
		__p[2][0][2] =	T(0);
	}
	if(d2) //v
	{
		__p[0][1][0] =	-_w*u*sin(T(.5)*v)*T(.5)*cos(v)-sin(v)*_r-sin(v)*_w*u*cos(T(.5)*v);
		__p[0][1][1] =	-_w*u*sin(T(.5)*v)*T(.5)*sin(v)+cos(v)*_r+cos(v)*_w*u*cos(T(.5)*v);
		__p[0][1][2] =	_w*u*cos(T(.5)*v)*T(.5);
	}
	if(d2>1) //vv
	{
		__p[0][2][0] =	-_w*u*cos(T(.5)*v)*T(.25)*cos(v)
						+2.0*_w*u*sin(T(.5)*v)*T(.5)*sin(v)-cos(v)*_r-cos(v)*_w*u*cos(T(.5)*v);	
		__p[0][2][1] =	-_w*u*cos(T(.5)*v)*T(.25)*sin(v)
						-2.0*_w*u*sin(T(.5)*v)*T(.5)*cos(v)-sin(v)*_r-sin(v)*_w*u*cos(T(.5)*v);
		__p[0][2][2] =	-_w*u*sin(T(.5)*v)*T(.25);	
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =	-_w*(sin(T(.5)*v)*T(.5)*cos(v)+cos(T(.5)*v)*sin(v));	
		__p[1][1][1] =	-_w*(sin(T(.5)*v)*T(.5)*sin(v)-cos(T(.5)*v)*cos(v));	
		__p[1][1][2] =	_w*cos(T(.5)*v)*T(.5);	
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =		T(0);
		__p[2][1][1] =		T(0);
		__p[2][1][2] =		T(0);
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =	-_w*(cos(T(.5)*v)*T(.25)*cos(v)-2.0*sin(T(.5)*v)*T(.5)*sin(v)+cos(T(.5)*v)*cos(v));
		__p[1][2][1] =	-_w*(cos(T(.5)*v)*T(.25)*sin(v)+2.0*sin(T(.5)*v)*T(.5)*cos(v)+cos(T(.5)*v)*sin(v));
		__p[1][2][2] =	-_w*sin(T(.5)*v)*T(.25);
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =		T(0);
		__p[2][2][1] =		T(0);
		__p[2][2][2] =		T(0);
	}

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];

}
template <class T, int n>
void Moebius<T,n>::localDisplay()
{
	// r_mat.mult();
	PSurf<T,n>::localDisplay();
}
 #endif //_GM_MOEBIUS_H_

