
//********************************************************
// GM_OpenGL.h: interface for the OpenGL in GM_lib.
//********************************************************
#ifndef GM_OPENGL_H_
#define GM_OPENGL_H_

#if defined(_WIN32)
//  To avoid including <windows.h> to avoid name space pollution, 
//	but Win32's <GL/gl.h> needs APIENTRY and WINGDIAPI defined properly.
# if 1
#  define  WIN32_LEAN_AND_MEAN
#  include <windows.h>
# else
   // XXX This is from Win32's <windef.h>
#  ifndef APIENTRY
#   define GLUT_APIENTRY_DEFINED
#   if (_MSC_VER >= 800) || defined(_STDCALL_SUPPORTED)
#    define APIENTRY    __stdcall
#   else
#    define APIENTRY
#   endif
#  endif
   // XXX This is from Win32's <winnt.h>
#  ifndef CALLBACK
#   if (defined(_M_MRX000) || defined(_M_IX86) || defined(_M_ALPHA) || defined(_M_PPC)) && !defined(MIDL_PASS)
#    define CALLBACK __stdcall
#   else
#    define CALLBACK
#   endif
#  endif
   // XXX This is from Win32's <wingdi.h> and <winnt.h>
#  ifndef WINGDIAPI
#   define GLUT_WINGDIAPI_DEFINED
#   define WINGDIAPI __declspec(dllimport)
#  endif
   // XXX This is from Win32's <ctype.h>
#  ifndef _WCHAR_T_DEFINED
typedef unsigned short wchar_t;
#   define _WCHAR_T_DEFINED
#  endif
# endif
#endif

#include <iostream>
using namespace std;
#include <GM_Stream.h>

#include <GL/gl.h>


#include "GM_Point3D.h"
#include "GM_Point2D.h"

namespace GM_lib {

//**************************************
//********** class GL_Color  ***********
//**************************************
class GL_Color
{
	union{
		GLuint name;
		GLubyte rgba[4];
	}c;

public:
	GL_Color(GLuint n=0)
	{ c.name = n; }

	GL_Color(GLubyte r, GLubyte g, GLubyte b, GLubyte a=255)
	{ c.rgba[0] = r; c.rgba[1] = g; c.rgba[2] = b; c.rgba[3] = a; }

	GL_Color(int r, int g, int b, int a=255)
	{ c.rgba[0] = GLubyte(r); c.rgba[1] = GLubyte(g); 
	  c.rgba[2] = GLubyte(b); c.rgba[3] = GLubyte(a); }
/**/
	GL_Color(GLclampd r, GLclampd g, GLclampd b, GLclampd a=1.0)
	{ c.rgba[0] = GLubyte(r*255); c.rgba[1] = GLubyte(g*255); 
	  c.rgba[2] = GLubyte(b*255); c.rgba[3] = GLubyte(a*255); }

	GL_Color(const GL_Color& k)
	{ c = k.c; }


	GLuint  get()const			{ return c.name; }
	GLubyte get(int i)const		{ return c.rgba[i]; }
	GLubyte getR()const			{ return c.rgba[0]; }
	GLubyte getG()const			{ return c.rgba[1]; }
	GLubyte getB()const			{ return c.rgba[2]; }
	GLubyte getA()const			{ return c.rgba[3]; }
	GLclampd getd(int i)const	{ return c.rgba[i]/255.0; }
	GLclampd getRd()const		{ return c.rgba[0]/255.0; }
	GLclampd getGd()const		{ return c.rgba[1]/255.0; }
	GLclampd getBd()const		{ return c.rgba[2]/255.0; }
	GLclampd getAd()const		{ return c.rgba[3]/255.0; }

	void set(GLuint n)				{ c.name = n; }
	void set(GLubyte  rgba, int i)	{ c.rgba[i] = rgba; }
	void set(GLclampd rgba, int i)	{ c.rgba[i] = GLubyte(rgba*255); }
	void setR(GLubyte r)			{ c.rgba[0] = r; }
	void setG(GLubyte g)			{ c.rgba[1] = g; }
	void setB(GLubyte b)			{ c.rgba[2] = b; }
	void setA(GLubyte a)			{ c.rgba[3] = a; }
	void setR(GLclampd r)			{ c.rgba[0] = GLubyte(r*255); }
	void setG(GLclampd g)			{ c.rgba[1] = GLubyte(g*255); }
	void setB(GLclampd b)			{ c.rgba[2] = GLubyte(b*255); }
	void setA(GLclampd a)			{ c.rgba[3] = GLubyte(a*255); }
	void set(GLubyte r, GLubyte g, GLubyte b, GLubyte a=0)	
	{ c.rgba[0] = r; c.rgba[1] = g; c.rgba[2] = b; c.rgba[3] = a; }
	void set(GLclampd r, GLclampd g, GLclampd b, GLclampd a=0.0)
	{ c.rgba[0] = GLubyte(r*255); c.rgba[1] = GLubyte(g*255); 
	  c.rgba[2] = GLubyte(b*255); c.rgba[3] = GLubyte(a*255); }

	GL_Color& operator*=(double d)
	{
		double k;
		k = c.rgba[0]*d; if(k > 255.0) c.rgba[0] = 255; else c.rgba[0] = GLubyte(k);
		k = c.rgba[1]*d; if(k > 255.0) c.rgba[1] = 255; else c.rgba[1] = GLubyte(k);
		k = c.rgba[2]*d; if(k > 255.0) c.rgba[2] = 255; else c.rgba[2] = GLubyte(k);
		k = c.rgba[3]*d; if(k > 255.0) c.rgba[3] = 255; else c.rgba[3] = GLubyte(k);
		return * this;
	}

	GL_Color inverse() const
	{
		return GL_Color(255-c.rgba[0],255-c.rgba[1],255-c.rgba[3]);
	}


	void toHSV()
	{
		double r=getRd(),g=getGd(),b=getBd();
		double h,s,v,mx,mn;
		mx = max(max(r,g),b);
		mn = min(min(r,g),b);
		v=mx;
		if(mx > 1e-6) s = (mx-mn)/mx;
		else		  s = 0.0;
		if(s == 0.0)  h = 0.0;
		else
		{
			double rc = (mx-r)/(mx-mn);
			double gc = (mx-g)/(mx-mn);
			double bc = (mx-b)/(mx-mn);
			if(r == mx)			h = bc-gc;
			else if(g == mx)	h = rc-bc + 2.0;
			else if(b == mx)	h = gc-rc + 4.0;
			h *= 60.0;
			if(h<0.0) h += 360.0;
			h /= 360.0;			// 360 degree is skaled to 1.
		}
		setR(h); setG(s); setB(v);
	}

	void toRGB()
	{
		double h=getRd(),s=getGd(),v=getBd();
		double r,g,b;
		h *= 360.0;
		if(s < 0.000001)
			r = g = b = v;
		else
		{
			if(h > 359.999999) h = 0.0;
			h /= 60.0;
			int    i = h;
			double f = h - double(i);
			double p = v*(1-s);
			double q = v*(1-s*f);
			double t = v*(1-s*(1-f));
			switch(i)
			{
			case 0: r=v; g=t; b=p; break; 
			case 1: r=q; g=v; b=p; break; 
			case 2: r=p; g=v; b=t; break; 
			case 3: r=p; g=q; b=v; break; 
			case 4: r=t; g=p; b=v; break; 
			case 5: r=v; g=p; b=q; break;
			default: ;
			}
		}
		setR(r); setG(g); setB(b);
	}


	GL_Color interpolateHSV(double d, GL_Color mx) const
	{
		GL_Color mn = *this;
		mx.toHSV();
		mn.toHSV();
		mx = (1-d)*mn+d*mx;
		mx.toRGB();
		return mx;
	}


	friend
	GL_Color operator*(double d, GL_Color o)
	{ return o *= d; }

	GL_Color operator*(double d)const
	{  return d * (*this); }

	GL_Color& operator/=(double d)
	{ d=1/d; return (*this) *= d; }

	GL_Color operator/(double d) const
	{ d=1/d; return d * (*this); }

	GL_Color& operator+=(const GL_Color& o)
	{
		int k;
		k = c.rgba[0]+o.c.rgba[0]; if(k > 255) c.rgba[0] = 255; else c.rgba[0] = k;
		k = c.rgba[1]+o.c.rgba[1]; if(k > 255) c.rgba[1] = 255; else c.rgba[1] = k;
		k = c.rgba[2]+o.c.rgba[2]; if(k > 255) c.rgba[2] = 255; else c.rgba[2] = k;
		k = c.rgba[3]+o.c.rgba[3]; if(k > 255) c.rgba[3] = 255; else c.rgba[3] = k;
		return * this;
	}	
	GL_Color operator+(const GL_Color& o) const
	{  GL_Color k= (*this); return k += o; }

	GL_Color& operator-=(const GL_Color& o)
	{
		int k;
		k = c.rgba[0]-o.c.rgba[0]; if(k < 0) c.rgba[0] = 0; else c.rgba[0] = k;
		k = c.rgba[1]-o.c.rgba[1]; if(k < 0) c.rgba[1] = 0; else c.rgba[1] = k;
		k = c.rgba[2]-o.c.rgba[2]; if(k < 0) c.rgba[2] = 0; else c.rgba[2] = k;
		k = c.rgba[3]-o.c.rgba[3]; if(k < 0) c.rgba[3] = 0; else c.rgba[3] = k;
		return * this;
	}	
	GL_Color operator-(const GL_Color& o) const
	{  GL_Color k= (*this); return k -= o; }

	GL_Color& operator+=(int o)
	{
		int k;
		k = c.rgba[0]+o; if(k > 255) c.rgba[0] = 255; else c.rgba[0] = k;
		k = c.rgba[1]+o; if(k > 255) c.rgba[1] = 255; else c.rgba[1] = k;
		k = c.rgba[2]+o; if(k > 255) c.rgba[2] = 255; else c.rgba[2] = k;
		k = c.rgba[3]+o; if(k > 255) c.rgba[3] = 255; else c.rgba[3] = k;
		return *this;
	}	
	GL_Color operator+(int o) const
	{  GL_Color k= (*this); return k += o; }

	GL_Color& operator-=(int o)
	{
		int k;
		k = c.rgba[0]-o; if(k < 0) c.rgba[0] = 0; else c.rgba[0] = k;
		k = c.rgba[1]-o; if(k < 0) c.rgba[1] = 0; else c.rgba[1] = k;
		k = c.rgba[2]-o; if(k < 0) c.rgba[2] = 0; else c.rgba[2] = k;
		k = c.rgba[3]-o; if(k < 0) c.rgba[3] = 0; else c.rgba[3] = k;
		return * this;
	}	
	GL_Color operator-(int o) const
	{  GL_Color k= (*this); return k -= o; }

	GL_Color& operator+=(double o)
	{
		int k = int(o*255);
		return (*this) += k;
	}	
	GL_Color operator+(double o)
	{  GL_Color k= (*this); return k += o; }

	GL_Color& operator-=(double o)
	{
		int k = int(o*255);
		return (*this) -= k;
	}

	GL_Color operator-(double o) const
	{  GL_Color k= (*this); return k -= o; }


	bool operator==(const GL_Color& co) const
	{ return c.name == co.c.name; }


	void glSet() const
	{ glColor4ubv(c.rgba); }

	template<class T_Stream>
	friend T_Stream& operator>>(T_Stream& in, GL_Color& v)
	{
		static Separator es(element_separator);
//		GLubyte r,g,b,a;
		GLclampd r,g,b,a;
		in >> r >> es >> g >> es >> b >> es >> a;
		v.set(r,g,b,a);
		return in;
	}

	template<class T_Stream>
	friend T_Stream& operator<<(T_Stream& out, const GL_Color& v)
	{
		out << v.getRd() << element_separator << v.getGd() << element_separator << v.getBd() << element_separator << v.getAd();
		return out;
	}
/*
#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
private:
   ostream& _prOut(ostream& out)const
		{ out << getRd() << "  " << getGd() << "  " << getBd() << "  " << getAd() ; return out;}
   istream& _prIn(istream& in)
		{ GLclampd r,g,b,a; in >> r >> g >> b >> a; set(r,g,b,a); return in;}
   friend 
   ostream& operator<<(ostream& out, const GL_Color& v)
	   { return v._prOut(out); }
   friend 
   istream& operator>>(istream& in, GL_Color& v)
	   { return v._prIn(in); }
#endif
*/
};

inline
void glColor(const GL_Color& c)
{
	c.glSet();
}

//*************************************************
//********** overloaded Light functions ***********
//*************************************************

inline
void glLight(GLenum li, GLenum pn, const GL_Color& co)
{
	GLfloat f[4];
	f[0] = GLfloat(co.getRd());
	f[1] = GLfloat(co.getGd());
	f[2] = GLfloat(co.getBd());
	f[3] = GLfloat(co.getAd());
	glLightfv(li,pn,f);
}

inline
void glLightSun(GLenum li, const Point<GLfloat,3>& pos)
{
	GLfloat f[4];
	f[0] = pos(0);
	f[1] = pos(1);
	f[2] = pos(2);
	f[3] = GLfloat(0.0);
	glLightfv(li,GL_POSITION,f);
}

inline
void glLightPos(GLenum li, const Point<GLfloat,3>& pos)
{
	GLfloat f[4];
	f[0] = pos(0);
	f[1] = pos(1);
	f[2] = pos(2);
	f[3] = GLfloat(1.0);
	glLightfv(li,GL_POSITION,f);
}


inline
void glLightDir(GLenum li, const Vector<GLfloat,3>& dir)
{
	GLfloat f[3];
	f[0] = dir(0);
	f[1] = dir(1);
	f[2] = dir(2);
	glLightfv(li,GL_SPOT_DIRECTION,f);
}


inline
void glLight(GLenum li,GLenum pn, GLfloat f)
{
	glLightf(li,pn,f);
}

inline
void glLight(GLenum li,GLenum pn, int f)
{
	glLighti(li,pn,f);
}


//***************************************************
//********** overloaded glMaterial functions ********
//***************************************************

inline
void glMaterial(GLenum li,GLenum pn, const GL_Color& co)
{
	GLfloat f[4];
	f[0] = GLfloat(co.getRd());
	f[1] = GLfloat(co.getGd());
	f[2] = GLfloat(co.getBd());
	f[3] = GLfloat(co.getAd());
	glMaterialfv(li,pn,f);
}

inline
void glMaterial(GLenum li,GLenum pn, const Point<GLfloat,3>& co)
{
	glMaterialfv(li,pn,(GLfloat*)(&co));
}

inline
void glMaterial(GLenum li,GLenum pn, GLfloat f)
{
	glMaterialf(li,pn,f);
}

inline
void glMaterial(GLenum li,GLenum pn, int f)
{
	glMateriali(li,pn,f);
}

//*************************************************
//********** overloaded Transformation functions **
//*************************************************

//*************GLfloat******************
inline
void glScale(const Point<GLfloat,3>& p)
{
	glScalef(p(0),p(1),p(2));
}

inline
void glRotate(Angle a, const Vector<GLfloat,3>& p)
{
	glRotatef(GLfloat(180*a.rad()/M_PI),p(0),p(1),p(2));
}

inline
void glTranslate(const Point<GLfloat,3>& p)
{
	glTranslatef(p(0),p(1),p(2));
}


//**************double******************
inline
void glScale(const Point<double,3>& p)
{
	glScaled(p(0),p(1),p(2));
}

inline
void glRotate(Angle a, const Vector<double,3>& p)
{
	glRotated(GLfloat(180*a.rad()/M_PI),p(0),p(1),p(2));
}

inline
void glTranslate(const Point<double,3>& p)
{
	glTranslated(p(0),p(1),p(2));
}

//**************************************
//********** class GL_Matrix ***********
//**************************************

static GLfloat _I[16] = {1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1};
	
class GL_Matrix 	
{
	GLfloat matrix[16];

	void cpy()
	{
		memcpy(matrix,_I,64);//for (i=0; i<16; i++) matrix[i] = _I[i];
	}

	void cpy(const GLfloat* m)
	{
		memcpy(matrix,m,64);//for (i=0; i<16; i++) matrix[i] = m[i];
	}

	void makeOrtho(const UnitVector<GLfloat,3>& d, Vector3D<GLfloat>& x, Vector3D<GLfloat>& y, Vector3D<GLfloat>& z)
	{
		double d0=fabs(d(0)),d1=fabs(d(1)),d2=fabs(d(2));
		if (d0 > d1)
		{
			if(d2 > d1)		// d(1) er minst
			{ z[0] = d(2); z[2] = -d(0); z[1] = 0.0f; }
			else			// d(2) er minst
			{ z[0] = d(1); z[1] = -d(0); z[2] = 0.0f; }
		}
		else // d(1) > d(0)
		{
			if(d2 > d0)		// d(0) er minst
			{ z[1] = d(2); z[2] = -d(1); z[0] = 0.0f; }
			else			// d(2) er minst
			{ z[0] = d(1); z[1] = -d(0); z[2] = 0.0f; }
		}
		z/=z.length();
		y = z^d;
		x[0] = d(0); x[1] = y(0); x[2] = z(0);
		y[0] = d(1); z[0] = d(2);
		swap(z[1],y[2]);
	}

	void _mult(const GL_Matrix& m1,const GL_Matrix& m2)
	{
		int i,j,k;
		for(i=0;i<4;i++)
			for(j=0;j<4;j++)
				for((*this)(i,j)=0,k=0;k<4;k++)
					(*this)(i,j) += m1.get(i,k)*m2.get(k,j);
	}
	void _mult2(GL_Matrix& m1,const GL_Matrix& m2) const
	{
		int i,j,k;
		for(i=0;i<4;i++)
			for(j=0;j<4;j++)
				for(m1(i,j)=0,k=0;k<4;k++)
					m1(i,j) += this->get(i,k)*m2.get(k,j);
	}



public:
	GL_Matrix()										// Default constructor
	{ cpy();}

	GL_Matrix(GLfloat* m)							// Standard constructor
	{ 	if(m)	cpy(m);
		else	cpy();
	}

	GL_Matrix(const GL_Matrix& m)					// Copy constructor
	{ cpy(m.pt());}

	GL_Matrix(Angle a, const UnitVector<GLfloat,3>& r)	// Rotation constructor 1
	{ 
		Vector3D<GLfloat> x,y,z,kx,ky,kz;
		GLfloat sina = (GLfloat)sin(a), cosa=(GLfloat)cos(a);
		makeOrtho(r,x,y,z);

		kx[0] = x(0);	kx[1] = x(1)*cosa-x(2)*sina;	kx[2] = x(1)*sina+x(2)*cosa;
		ky[0] = y(0);	ky[1] = y(1)*cosa-y(2)*sina;	ky[2] = y(1)*sina+y(2)*cosa;
		kz[0] = z(0);	kz[1] = z(1)*cosa-z(2)*sina;	kz[2] = z(1)*sina+z(2)*cosa;

		matrix[0] = x*kx;	matrix[4] = x*ky;	matrix[8] = x*kz;	matrix[12] = 0.0;
		matrix[1] = y*kx;	matrix[5] = y*ky;	matrix[9] = y*kz;	matrix[13] = 0.0;
		matrix[2] = z*kx;	matrix[6] = z*ky;	matrix[10]= z*kz;	matrix[14] = 0.0;
		matrix[3] = 0.0;	matrix[7] = 0.0;	matrix[11]= 0.0;	matrix[15] = 1.0;
	}

	GL_Matrix(Angle a, const Point<GLfloat,3>& p,const UnitVector<GLfloat,3>& d)	// Rotation constructor 2
	{
		GL_Matrix mm;	
		Vector3D<GLfloat> x,y,z,kx,ky,kz,r;
		GLfloat sina = (GLfloat)sin(a), cosa=(GLfloat)cos(a);
		makeOrtho(d,x,y,z);
		GLfloat sp = x(1)*p(0)+y(1)*p(1)+z(1)*p(2), up = x(2)*p(0)+y(2)*p(1)+z(2)*p(2);

		kx[0] = x(0);	kx[1] = x(1)*cosa-x(2)*sina;	kx[2] = x(1)*sina+x(2)*cosa;
		ky[0] = y(0);	ky[1] = y(1)*cosa-y(2)*sina;	ky[2] = y(1)*sina+y(2)*cosa;
		kz[0] = z(0);	kz[1] = z(1)*cosa-z(2)*sina;	kz[2] = z(1)*sina+z(2)*cosa;
		r[0]  = -(d*p);	r[1]  = -sp*cosa+up*sina;		r[2]  = -sp*sina-up*cosa;

		matrix[0] = x*kx;	matrix[4] = x*ky;	matrix[8] = x*kz;	matrix[12] = p(0)+x*r;
		matrix[1] = y*kx;	matrix[5] = y*ky;	matrix[9] = y*kz;	matrix[13] = p(1)+y*r;
		matrix[2] = z*kx;	matrix[6] = z*ky;	matrix[10]= z*kz;	matrix[14] = p(2)+z*r;
		matrix[3] = 0.0;	matrix[7] = 0.0;	matrix[11]= 0.0;	matrix[15] = 1.0;
}

	GL_Matrix(const Vector<GLfloat,3>& tr)			// Translate constructor
	{ cpy(); memcpy((matrix+12), tr.ptr(), 12);}
			//matrix[12] = s(0);	matrix[13] = s(1);	matrix[14] = s(2); }

	GL_Matrix& operator=(GLfloat* m)
	{ cpy(m); return *this; }

	GL_Matrix& operator=(const GL_Matrix& m)
	{ cpy(m.pt()); return *this; }

	GLfloat* pt() const { return (GLfloat*)(matrix); }

	void reset() { cpy(); }

	GLfloat&  operator()(int i,int j)
	{ return matrix[(i<<2)+j]; }//matrix[i*4+j]; }

	const GLfloat&  get(int i,int j) const
	{ return matrix[(i<<2)+j]; }

	void get(GLenum p)
	{	glGetFloatv(p,matrix); }

	void put() const
	{ 	glLoadMatrixf(matrix); }

	void mult() const
	{ 	glMultMatrixf(matrix); }

	GL_Matrix& operator *=(const GL_Matrix& m)
	{	GL_Matrix mat= *this; _mult(m,mat); return *this; }

	GL_Matrix operator *(GL_Matrix& m)
	{	GL_Matrix mat; mat._mult(m,*this); return mat; }

	GL_Matrix& reverseMult(const GL_Matrix& m)
	{	GL_Matrix mat= *this; _mult(mat,m); return *this; }

	Point<GLfloat,3> operator *(const Point<GLfloat,3>& p) const
	{
		Point<GLfloat,3> lp;
//		double  dd;
		lp[0] = matrix[0]*p(0) + matrix[4]*p(1) + matrix[8]*p(2)  + matrix[12];
		lp[1] = matrix[1]*p(0) + matrix[5]*p(1) + matrix[9]*p(2)  + matrix[13];
		lp[2] = matrix[2]*p(0) + matrix[6]*p(1) + matrix[10]*p(2) + matrix[14];
//		dd    = matrix[3]*p(0) + matrix[7]*p(1) + matrix[11]*p(2) + matrix[15];
//		lp/=dd;
		return lp;
	}

	Vector<GLfloat,3> operator *(const Vector<GLfloat,3>& p) const
	{
		Point<GLfloat,3> lp;
		lp[0] = matrix[0]*p(0) + matrix[4]*p(1) + matrix[8]*p(2);
		lp[1] = matrix[1]*p(0) + matrix[5]*p(1) + matrix[9]*p(2);
		lp[2] = matrix[2]*p(0) + matrix[6]*p(1) + matrix[10]*p(2);
		return lp;
	}

	Arrow<GLfloat,3> operator *(const Arrow<GLfloat,3>& b) const
	{
		Arrow<GLfloat,3> lp;
		lp.setPos((*this)*b.pos());
		lp.setDir((*this)*b.dir());
		return lp;
	}

	Box<GLfloat,3> operator *(const Box<GLfloat,3>& b) const
	{
		Box<GLfloat,3> lp;
		Point<GLfloat,3> p = b.minPoint();
		p *= (*this);
		lp.reset(p);
		p = b.maxPoint();
		p *= (*this);
		lp += p;
		return lp;
	}


	Sphere<GLfloat,3> operator *(const Sphere<GLfloat,3>& b) const
	{
		Sphere<GLfloat,3> lp;
		if(b.valid())
		{
			Point<GLfloat,3> p = b.pos();
			p *= (*this);
			lp.resetPos(p);
			Vector<GLfloat,3> v;
			v[0]= b.radius(); v[1] = v[2] = 0.0;
			v *= (*this);
			lp.resetRadius(v.length());
		}
		return lp;
	}

	friend
	Point<GLfloat,3>& operator*=(Point<GLfloat,3>& p, const GL_Matrix& m)
	{	p = m*p; return p; }

	friend
	Vector<GLfloat,3>& operator*=(Vector<GLfloat,3>& p, const GL_Matrix& m)
	{	p = m*p; return p; }

	friend
	UnitVector<GLfloat,3>& operator*=(UnitVector<GLfloat,3>& p, const GL_Matrix& m)
	{	p = m*p; return p; }

	friend
	Arrow<GLfloat,3>& operator*=(Arrow<GLfloat,3>& p, const GL_Matrix& m)
	{	p = m*p; return p; }

	friend
	Box<GLfloat,3>& operator*=(Box<GLfloat,3>& p, const GL_Matrix& m)
	{	p = m*p; return p; }

	friend
	Sphere<GLfloat,3>& operator*=(Sphere<GLfloat,3>& p, const GL_Matrix& m)
	{	p = m*p; return p; }


	void translate(const Point<GLfloat,3>& s)
	{
		GL_Matrix m(s);
		(*this) *= m;
	}

	void translateGlobal(const Point<GLfloat,3>& s)
	{
		GL_Matrix m(s);
		reverseMult(m);
	}

	void scale(const Point<GLfloat,3>& s)
	{
		GL_Matrix m;
		m.matrix[0] = s(0);	m.matrix[5] = s(1);	m.matrix[10] = s(2);
		(*this) *= m;
	}	

	void scaleGlobal(const Point<GLfloat,3>& s)
	{
		GL_Matrix m;
		m.matrix[0] = s(0);	m.matrix[5] = s(1);	m.matrix[10] = s(2);
		reverseMult(m);
	}	

	void rotate(Angle a, const UnitVector<GLfloat,3>& d)
	{	
		GL_Matrix m(a,d);
		(*this) *= m;
	}

	void rotateGlobal(Angle a, const Vector<GLfloat,3>& s)
	{	
		GL_Matrix m(a,s);  
		reverseMult(m);
	}

	void rotate(Angle a, const Point<GLfloat,3>& p,const UnitVector<GLfloat,3>& d)
	{
		GL_Matrix m(a,p,d);	
		(*this) *= m;
	}

	void rotateGlobal(Angle a, const Point<GLfloat,3>& p,const UnitVector<GLfloat,3>& d)
	{
		GL_Matrix m(a,p,d);	
		reverseMult(m);
	}

	void basisChangeCam(const Vector<GLfloat,3>& x, const Vector<GLfloat,3>& y,
					    const Vector<GLfloat,3>& z, const Vector<GLfloat,3>& p)
	{
		matrix[0] = -x(0);	matrix[4] = -x(1);	matrix[8] = -x(2);	matrix[12] =  x*p;
		matrix[1] =  y(0);	matrix[5] =  y(1);	matrix[9] =  y(2);	matrix[13] =-(y*p);
		matrix[2] = -z(0);	matrix[6] = -z(1);	matrix[10]= -z(2);	matrix[14] =  z*p;
		matrix[3] =   0.0;	matrix[7] =   0.0;	matrix[11]=   0.0;	matrix[15] =  1.0;
	}

	void basisChange(const Vector<GLfloat,3>& x, const Vector<GLfloat,3>& y,
					 const Vector<GLfloat,3>& z, const Vector<GLfloat,3>& p)
	{
		memcpy(matrix,x.ptr(),12);
		memcpy(matrix+4,y.ptr(),12);
		memcpy(matrix+8,z.ptr(),12);
		memcpy(matrix+12,p.ptr(),12);
		matrix[3] =  0.0;	matrix[7] =  0.0;	matrix[11]=  0.0;	matrix[15] =  1.0;
	}

	void basisChangeInv(const Vector<GLfloat,3>& x, const Vector<GLfloat,3>& y,
					    const Vector<GLfloat,3>& z, const Vector<GLfloat,3>& p)
	{
		matrix[0] = x(0);	matrix[4] = x(1);	matrix[8] = x(2);	matrix[12] = -(x*p);
		matrix[1] = y(0);	matrix[5] = y(1);	matrix[9] = y(2);	matrix[13] = -(y*p);
		matrix[2] = z(0);	matrix[6] = z(1);	matrix[10]= z(2);	matrix[14] = -(z*p);
		matrix[3] =  0.0;	matrix[7] =  0.0;	matrix[11]=  0.0;	matrix[15] =    1.0;
	}


	void invert2()
	{
		GLfloat m12 = matrix[12];
		GLfloat m13 = matrix[13];
		GLfloat m14 = matrix[14];
		swap(matrix[1],matrix[4]);
		swap(matrix[2],matrix[8]);
		swap(matrix[6],matrix[9]);
		matrix[12] =  -matrix[0]*m12-matrix[4]*m13-matrix[8]*m14;
		matrix[13] =  -matrix[1]*m12-matrix[5]*m13-matrix[9]*m14;
		matrix[14] =  -matrix[2]*m12-matrix[6]*m13-matrix[10]*m14;
	}


	void invert()
	{
		double m[16];
		double a[16];
		int i;
		for (i=0;i<16;i++) m[i] = (double)matrix[i];
		m[1]  /= m[0];		m[2] /= m[0];		m[3] /= m[0];		a[0] = 1.0/m[0];
		m[5]  -= m[4]*m[1]; m[6]  -= m[4]*m[2]; m[7]  -= m[4]*m[3]; a[4] = -m[4]*a[0];
		m[6]  /= m[5];		m[7] /= m[5];		a[4] /= m[5];		a[5] = 1.0/m[5];
		m[9]  -= m[8]*m[1]; m[10] -= m[8]*m[2]; m[11] -= m[8]*m[3]; a[8] = -m[8]*a[0];
		m[10] -= m[9]*m[6]; m[11] -= m[9]*m[7]; a[8]  -= m[9]*a[4]; a[9] = -m[9]*a[5];
		m[11] /= m[10];		a[8] /= m[10];		a[9] /= m[10];		a[10] = 1.0/m[10];
		m[13] -= m[12]*m[1];m[14] -= m[12]*m[2];m[15] -= m[12]*m[3];a[12] = -m[12]*a[0];
		m[14] -= m[13]*m[6];m[15] -= m[13]*m[7];a[12] -= m[13]*a[4];a[13] = -m[13]*a[5];
		m[15] -=m[14]*m[11];a[12] -= m[14]*a[8];a[13] -= m[14]*a[9];a[14] = -m[14]*a[10];
		a[12] /= m[15];		a[13] /= m[15];		a[14] /= m[15];		a[15] = 1.0/m[15];
		a[8] -= m[11]*a[12];a[9] -= m[11]*a[13];a[10]-= m[11]*a[14];a[11]= -m[11]*a[15];
		a[4] -= m[7] *a[12];a[5] -= m[7] *a[13];a[6] = -m[7]*a[14]; a[7] = -m[7]*a[15];
		a[0] -= m[3] *a[12];a[1] = -m[3] *a[13];a[2] = -m[3]*a[14]; a[3] = -m[3]*a[15];
		a[4] -= m[6] *a[8]; a[5] -= m[6] *a[9]; a[6] -= m[6]*a[10]; a[7] -= m[6]*a[11];
		a[0] -= m[2] *a[8]; a[1] -= m[2] *a[9]; a[2] -= m[2]*a[10]; a[3] -= m[2]*a[11];
		a[0] -= m[1] *a[4]; a[1] -= m[1] *a[5]; a[2] -= m[1]*a[6];	a[3] -= m[1]*a[7];
		for (i=0;i<16;i++) matrix[i] = (GLfloat)a[i];
	}

	GL_Matrix& inverse2() const
	{
		static GL_Matrix m;
		m = *this;
		m.invert2();
		return m;
	}
	GL_Matrix& inverse() const
	{
		static GL_Matrix m;
		m = *this;
		m.invert();
		return m;
	}

	void roundoff(int i=1000)
	{
		matrix[0]+=i;  matrix[0]-=i;
		matrix[1]+=i;  matrix[1]-=i;
		matrix[2]+=i;  matrix[2]-=i;
		matrix[3]+=i;  matrix[3]-=i;
		matrix[4]+=i;  matrix[4]-=i;
		matrix[5]+=i;  matrix[5]-=i;
		matrix[6]+=i;  matrix[6]-=i;
		matrix[7]+=i;  matrix[7]-=i;
		matrix[8]+=i;  matrix[8]-=i;
		matrix[9]+=i;  matrix[9]-=i;
		matrix[10]+=i;  matrix[10]-=i;
		matrix[11]+=i;  matrix[11]-=i;
		matrix[12]+=i;  matrix[12]-=i;
		matrix[13]+=i;  matrix[13]-=i;
		matrix[14]+=i;  matrix[14]-=i;
		matrix[15]+=i;  matrix[15]-=i;
	}


	bool operator<(const GL_Matrix& m) const		// DUMMY
	{ return true; }


public:

	template<class T_Stream>
	friend T_Stream& operator<<(T_Stream& out, const GL_Matrix& mm)
	{
		GLfloat* m = mm.pt();
		out << m[0] << element_separator << m[4] << element_separator << m[8] << element_separator << m[12] << group_separator;
		out << m[1] << element_separator << m[5] << element_separator << m[9] << element_separator << m[13] << group_separator;
		out << m[2] << element_separator << m[6] << element_separator << m[10]<< element_separator << m[14] << group_separator;
		out << m[3] << element_separator << m[7] << element_separator << m[11]<< element_separator << m[15] << group_separator;
		return out;
	}


	template<class T_Stream>
	friend T_Stream& operator>>(T_Stream& in, GL_Matrix& mm)
	{
		GLfloat* m = mm.pt();
		static Separator es(element_separator);
		static Separator gs(group_separator);
		
		in >> m[0] >> es >> m[4] >> es >> m[8] >> es >> m[12] >> gs;
		in >> m[1] >> es >> m[5] >> es >> m[9] >> es >> m[13] >> gs;
		in >> m[2] >> es >> m[6] >> es >> m[10]>> es >> m[14] >> gs;
		in >> m[3] >> es >> m[7] >> es >> m[11]>> es >> m[15] >> gs;
		return in;		
	}

		 	// ********** IOSTREAM overloaded operators ***********
/*
#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
private:
	ostream& _prOut(ostream& out)const
	{
		cout << matrix[0] << "  " << matrix[4] << "  " << matrix[8] << "  " << matrix[12] << "\n";
		cout << matrix[1] << "  " << matrix[5] << "  " << matrix[9] << "  " << matrix[13] << "\n";
		cout << matrix[2] << "  " << matrix[6] << "  " << matrix[10]<< "  " << matrix[14] << "\n";
		cout << matrix[3] << "  " << matrix[7] << "  " << matrix[11]<< "  " << matrix[15] << "\n";
		return out;
	}
   istream& _prIn(istream& in)
	{
		in >> matrix[0] >> matrix[4] >> matrix[8] >> matrix[12]
		   >> matrix[1] >> matrix[5] >> matrix[9] >> matrix[13]
		   >> matrix[2] >> matrix[6] >> matrix[10]>> matrix[14]
		   >> matrix[3] >> matrix[7] >> matrix[11]>> matrix[15];
		return in;
	}
public:
  friend
	ostream& operator<<(ostream& out, const GL_Matrix& v)
	   { return v._prOut(out); }
  friend
	ostream& operator<<(ostream& out, const GL_Matrix* v)
	   { return v->_prOut(out); }
  friend
	istream& operator>>(istream& in, GL_Matrix& v)
	   { return v._prIn(in); }
  friend
	istream& operator>>(istream& in, GL_Matrix* v)
	   { return v->_prIn(in); }

	void print(char prompt[]="GL_Matrix", ostream & out = cout) const
	   { out << prompt << ": " << (*this) << "\n"; }
#endif */
};


//*************************************************
//********** overloaded Matrix functions **********
//*************************************************

inline
void glLoadMatrix(const GL_Matrix& m)
{
	glLoadMatrixf(m.pt());
}

inline
void glMultMatrix(const GL_Matrix& m)
{
	glMultMatrixf(m.pt());
}



//********************************
//********  DIMENSION  2 *********
//********************************

//*********************************************************
//********** Point<double,2>:based gl-functions ***********
//*********************************************************

inline
void glPoint(const Point<double,2>& p)
{
	glVertex2dv((double *)&p);
}

inline
void glPoint(Point<double,2>* p)
{
	glVertex2dv((double *)p);
}


//*********************************************************
//********** Point<float,2>:based gl-functions ************
//*********************************************************

inline
void glPoint(const Point<float,2>& p)
{
	glVertex2fv((float *)&p);
}

inline
void glPoint(Point<float,2>* p)
{
	glVertex2fv((float *)p);
}

inline
void glTexCoord(const Point<float,2>& p)
{
	glTexCoord2fv((float *)&p);
}

//*********************************************************
//********** Point<int,2>:based gl-functions **************
//*********************************************************

inline
void glPoint(const Point<int,2>& p)
{
	glVertex2iv((int *)&p);
}

inline
void glPoint(Point<int,2>* p)
{
	glVertex2iv((int *)p);
}


//*********************************************************
//********** Point<short,2>:based gl-functions ************
//*********************************************************

inline
void glPoint(const Point<short,2>& p)
{
	glVertex2sv((short *)&p);
}

inline
void glPoint(Point<short,2>* p)
{
	glVertex2sv((short *)p);
}




//********************************
//********  DIMENSION  3 *********
//********************************

//*********************************************************
//********** Point<double,3>:based gl-functions ***********
//*********************************************************

inline
void glPoint(const Point<double,3>& p)
{
	glVertex3dv((double *)&p);
}

inline
void glVector(const Vector<double,3>& p)
{
	double a[4];
	memcpy(a,p.ptr(),3);
	a[3] = 0;
	glVertex4dv(a);
}

inline
void glNormal(const Vector<double,3>& v)
{
	glNormal3dv((double *)&v);
}

inline
void glVertex(const Arrow<double,3>& p)
{
	glNormal3dv((double *)(p.dir().ptr()));
	glVertex3dv((double *)&p);
}


inline
void glPoint(Point<double,3>* p)
{
	glVertex3dv((double *)p);
}

inline
void glVector(Vector<double,3>* p)
{
	double a[4];
	memcpy(a,p,3);
	a[3] = 0;
	glVertex4dv(a);
}

inline
void glNormal(Vector<double,3>* v)
{
	glNormal3dv((double *)v);
}

inline
void glVertex(Arrow<double,3>* p)
{
	glNormal3dv((double *)(p->dir().ptr()));
	glVertex3dv((double *)p);

}

//*********************************************************
//********** Point<float,3>:based gl-functions ************
//*********************************************************

inline
void glPoint(const Point<float,3>& p)
{
	glVertex3fv((float *)&p);
}

inline
void glVector(const Vector<float,3>& p)
{
	float a[4];
	memcpy(a,p.ptr(),3);
	a[3] = 0;
	glVertex4fv(a);
}

inline
void glNormal(const Vector<float,3>& v)
{
	glNormal3fv((float *)&v);
}

inline
void glVertex(const Arrow<float,3>& p)
{
	glNormal3fv((float *)(p.dir().ptr()));
	glVertex3fv((float *)&p);
}


inline
void glPoint(Point<float,3>* p)
{
	glVertex3fv((float *)p);
}

inline
void glVector(Vector<float,3>* p)
{
	float a[4];
	memcpy(a,p,3);
	a[3] = 0;
	glVertex4fv(a);
}

inline
void glNormal(Vector<float,3>* v)
{
	glNormal3fv((float *)v);
}

inline
void glVertex(Arrow<float,3>* p)
{
	glNormal3fv((float *)(p->dir().ptr()));
	glVertex3fv((float *)p);
}


//*********************************************************
//********** Point<int,3>:based gl-functions **************
//*********************************************************

inline
void glPoint(const Point<int,3>& p)
{
	glVertex3iv((int *)&p);
}

inline
void glNormal(const Vector<int,3>& v)
{
	glNormal3iv((int *)&v);
}

inline
void glVertex(const Arrow<int,3>& p)
{
	glNormal3iv((int *)(p.dir().ptr()));
	glVertex3iv((int *)&p);
}


inline
void glPoint(Point<int,3>* p)
{
	glVertex3iv((int *)p);
}

inline
void glNormal(Vector<int,3>* v)
{
	glNormal3iv((int *)v);
}

inline
void glVertex(Arrow<int,3>* p)
{
	glNormal3iv((int *)(p->dir().ptr()));
	glVertex3iv((int *)p);
}


//*********************************************************
//********** Point<short,3>:based gl-functions ************
//*********************************************************

inline
void glPoint(const Point<short,3>& p)
{
	glVertex3sv((short *)&p);
}

inline
void glNormal(const Vector<short,3>& v)
{
	glNormal3sv((short *)&v);
}

inline
void glVertex(const Arrow<short,3>& p)
{
	glNormal3sv((short *)(p.dir().ptr()));
	glVertex3sv((short *)&p);
}


inline
void glPoint(Point<short,3>* p)
{
	glVertex3sv((short *)p);
}

inline
void glNormal(Vector<short,3>* v)
{
	glNormal3sv((short *)v);
}

inline
void glVertex(Arrow<short,3>* p)
{
	glNormal3sv((short *)(p->dir().ptr()));
	glVertex3sv((short *)p);
}



//********************************
//********  DIMENSION  4 *********
//********************************

//*********************************************************
//********** Point<double,4>:based gl-functions ***********
//*********************************************************

inline
void glPoint(const Point<double,4>& p)
{
	glVertex4dv((double *)&p);
}

inline
void glPoint(Point<double,4>* p)
{
	glVertex4dv((double *)p);
}


//*********************************************************
//********** Point<float,4>:based gl-functions ************
//*********************************************************

inline
void glPoint(const Point<float,4>& p)
{
	glVertex4fv((float *)&p);
}

inline
void glPoint(Point<float,4>* p)
{
	glVertex4fv((float *)p);
}


//*********************************************************
//********** Point<int,4>:based gl-functions **************
//*********************************************************

inline
void glPoint(const Point<int,4>& p)
{
	glVertex4iv((int *)&p);
}

inline
void glPoint(Point<int,4>* p)
{
	glVertex4iv((int *)p);
}


//*********************************************************
//********** Point<short,4>:based gl-functions ************
//*********************************************************

inline
void glPoint(const Point<short,4>& p)
{
	glVertex4sv((short *)&p);
}

inline
void glPoint(Point<short,4>* p)
{
	glVertex4sv((short *)p);
}

void swapBuffers(void);


//*******************************************************
// A class to describe a frutrum to a perspectiv view
//*******************************************************

class Frustum  
{
	Point<float,3>	 p[2];	// p[0]: hyre/opp/bak-hjrne  p[1]: venstre/ned/foran-hjrne
	Vector<float,3>  v[6];	// normal: venstre, hyre, opp, ned, bak, fram.

public:											
	Frustum(){}													// Default constructor
	Frustum(    const GL_Matrix& m,								// Standar perspective constructor
				const Point<float,3>& p, 
				const Vector<float,3>& d, 
				const Vector<float,3>& u, 
				const Vector<float,3>& s, 
				float angle_tan,
				float ratio,
				float nearplane,
				float farplane)     {set(m,p,d,u,s,angle_tan,ratio,nearplane,farplane); }

	Frustum(    const GL_Matrix& m,								// Iso constructor constructor
				const Point<float,3>& p, 
				float width,
				float ratio,
				const Vector<float,3>& d, 
				const Vector<float,3>& u, 
				const Vector<float,3>& s, 
				float nearplane,
				float farplane)     {set(m,p,width,ratio,d,u,s,nearplane,farplane); }


	Frustum(const Frustum&  v) {memcpy(this,&v,sizeof(Frustum));}// Copy constructor

	void set(   const GL_Matrix& m,			// Perspective
				const Point<float,3>& p, 
				const Vector<float,3>& d, 
				const Vector<float,3>& u, 
				const Vector<float,3>& s, 
				float angle_tan,
				float ratio,
				float nearplane,
				float farplane);

	void set(   const GL_Matrix& m,			// Iso
				const Point<float,3>& p, 
				float width,
				float ratio,
				const Vector<float,3>& d, 
				const Vector<float,3>& u, 
				const Vector<float,3>& s, 
				float nearplane,
				float farplane);

	Point<float,3>  pos(int i) const;
	Vector<float,3> normal(int i) const;

	int isInterfering(const Sphere<float,3>&) const;	//  1 : Totally inside
														//  0 : Intersecting sides
														// -1 : Totally outside
};


inline
Point<float,3> Frustum::pos(int i) const 
{ 
	if(i<4) return p[0]; 
	else	return p[i-3];
}


inline
Vector<float,3> Frustum::normal(int i) const
{ 
	return v[i];
}


inline
void Frustum::set(  const GL_Matrix& m,
					const Point<float,3>& po, 
					const Vector<float,3>& d, 
					const Vector<float,3>& u, 
					const Vector<float,3>& s, 
					float at,
					float r,
					float nearplane,
					float farplane)
{
	r *= at;
	double rr = sqrt(1+r*r);
	double tt = sqrt(1+at*at);
	Vector<float,3> f  = m*d;
	Vector<float,3> oe = m*u;
	Vector<float,3> ve = m*s;
	Vector<float,3> pp = r*ve-at*oe;
	p[0] = m*po;			// Venstre, hyre, opp, ned (posisjon)			  
	p[1] = p[0]+farplane*(f+pp);   
	p[0] += nearplane*(f-pp);
	v[0] = ve-r*f;			// Venstre	(normal)
	v[1] = (v[0]-2*ve)/rr;	// Hyre	(normal)
	v[0] /= rr;
	v[2] = oe-at*f;			// Opp		(normal)
	v[3] = (v[2]-2*oe)/tt;	// ned		(normal)
	v[2] /= tt;
	v[4] = -f;				// Bak		(normal)
	v[5] = f;				// Fram		(normal)
}
	

inline
void Frustum::set(  const GL_Matrix& m,
					const Point<float,3>& po, 
					float w,
					float r,
					const Vector<float,3>& d, 
					const Vector<float,3>& u, 
					const Vector<float,3>& s, 
					float nearplane,
					float farplane)
{
	Vector<float,3> f  = m*d;
	Vector<float,3> oe = m*u;
	Vector<float,3> ve = m*s;
	p[0] = m*po;
	p[1] = p[0]+farplane*f+w*(r*ve-oe);   
	p[0] += nearplane*f-w*(r*ve-oe);
	v[0] = ve;				// Venstre	(normal)
	v[1] = -ve;				// Hyre	(normal)
	v[2] = oe;				// Opp		(normal)
	v[3] = -oe;				// ned		(normal)
	v[4] = -f;				// Bak		(normal)
	v[5] = f;				// Fram		(normal)
}
	


inline
int Frustum::isInterfering(const Sphere<float,3>& s) const
{								// For  optimalisere br rekkeflgen av			-1: Totaly outside
	if(!s.valid())	return -1;	// behandling av sidene studeres nrmere,			 0: Intersecting
	int ret = 1;				// i snitt br antallet if else vre under 3		 1: Totaly inside
								// hvis objektet er utenfor frustrumet.
	Vector<float,3> d = s.pos()-p[0];	
	float dv = d*v[1];							// Hyre	
	if(dv >= s.radius())		return -1;
	else if(dv > -s.radius())	ret = 0;	
	dv = d*v[2];								// Opp
	if(dv >= s.radius())		return -1;	
	else if(dv > -s.radius())	ret = 0;	
	dv = d*v[4];								// Bak
	if(dv >= s.radius())		return -1;	
	else if(dv > -s.radius())	ret = 0;	

	d=s.pos()-p[1];
	dv = d*v[0];								// Venstre
	if(dv >= s.radius())		return -1;	
	else if(dv > -s.radius())	ret = 0;	
	dv = d*v[3];								// Ned
	if(dv >= s.radius())		return -1;	
	else if(dv > -s.radius())	ret = 0;	
	dv = d*v[5];								// Fram
	if(dv >= s.radius())		return -1;	
	else if(dv > -s.radius())	ret = 0;	
	return ret;
}



class ScaleObj
{
	bool			scaled;
	Point<float,3>	s;
	float			max;

	void updateMax() { max = (s[0]>s[1] ? (s[0]>s[2] ? s[0]:s[2]):(s[1]>s[2] ? s[1]:s[2]));}

public:
	ScaleObj()					{ reset(); }
	ScaleObj(float	s)			{ reset(Point<float,3>(s)); }
	ScaleObj(Point<float,3>	sc) { reset(sc); }

	void scale(const Point<float,3>& sc)
	{
		scaled=true; s %= sc; updateMax();
	}

	void reset()
	{ 
		scaled=false; s=Point<float,3>(1);
	}

	void reset(const Point<float,3>& sc) 
	{ 
		if(sc(0)!= 1 || sc(1)!= 1 || sc(2)!= 1) { s=sc; updateMax(); scaled=true; }
		else { scaled=false; s=sc; }
	}

	void glScaling() 
	{
		if(scaled) glScale(s); 
	}

	bool  isActiv()  const { return scaled; }
	float maxScale() const { return max; }

	Point<float,3>const& getScale() { return s; }

	Sphere<float,3> sphere(const Sphere<float,3>& sp) const
	{ 
		if(scaled)
		{
			if(sp.valid()) return Sphere<float,3>(max*sp.pos(),max*sp.radius());
			else return sp;
		}
		else return sp;
	}

	template<class T_Stream>
	friend T_Stream& operator>>(T_Stream& in, ScaleObj& v)
	{
		static Separator es(element_separator);
		//		GLubyte r,g,b,a;
		int scaled;
		in >> scaled >> es >> v.s >> es >> v.max;
		v.scaled = (bool)scaled;
		return in;
	}

	template<class T_Stream>
	friend T_Stream& operator<<(T_Stream& out, const ScaleObj& v)
	{
		out << (int)v.scaled << element_separator << v.s << element_separator << v.max ;
		return out;
	}
};


class Sphere3D : public Sphere<float,3>
{
	int _m1, _m2;
	Arrow<float,3> _pt[2];
	Array<Array<Arrow<float,3> > > _p;
public:
	Sphere3D(float r=1.0, int m1=7, int m2=5) : Sphere<float,3>(Point<float,3>(float(0)),r)
	{
		makeSample(m1,m2);
	}

	Sphere3D(const Sphere<float,3>& s, int m1=7, int m2=5) : Sphere<float,3>(s)
	{
		makeSample(m1,m2);
	}

	void makeSample(int m1, int m2)
	{
		double u,v,du,dv,sv,cv,su,cu,ru,r=_value;
		_m1 = m1-1; _m2 = m2+1;
		_pt[0] = Arrow<float,3>(_pos+Point3D<float>(0,0,r), Vector3D<float>(0,0,1));
		_pt[1] = Arrow<float,3>(_pos+Point3D<float>(0,0,-r),Vector3D<float>(0,0,-1));
		du = M_PI/m1;
		dv = M_2PI/m2;
		_p.setSize(_m1);
		u = M_PI_2-du;
		for(int i=0; i<_m1; i++,u-=du)
		{
			su = sin(u);
			cu = cos(u);
			ru = r*cu;
			_p[i].setSize(_m2);
			v = 0;
			for(int j=0; j<_m2; j++, v+=dv)
			{
				sv = sin(v);
				cv = cos(v);
				_p[i][j] = Arrow<float,3>(_pos+Point3D<float>(ru*cv,ru*sv,r*su),Vector3D<float>(cu*cv,cu*sv,su));
			}
		}
	}

	void display()
	{
		int j,i;
		glBegin(GL_TRIANGLE_FAN);
			glVertex(_pt[0]);
			for(j=0;j<_m2;j++)
				glVertex(_p[0][j]);
		glEnd();
		for(i=0; i<_m1-1; i++)
		{
			glBegin(GL_TRIANGLE_STRIP);
			for(j=0;j<_m2;j++)
			{
				glVertex(_p[i][j]); 
				glVertex(_p[i+1][j]);
			}
			glEnd();
		}
		glBegin(GL_TRIANGLE_FAN);
			glVertex(_pt[1]);
			for(j=_m2-1;j>=0;j--)
				glVertex(_p[_m1-1][j]);
		glEnd();
	}
};

}  // end namespace GM_lib
#endif   /* GM_OPENGL_H_ */

