
//********************************************
// GM_Ball.h: interface for the Ball classes.
//********************************************
#ifndef GM_PPLANE_H_
#define GM_PPLANE_H_

#include "GM_PSurf.h"


namespace GM_lib{


template <class T, int n>
class Collision;


template <class T, int n>
class PPlane: public PSurf<T,n>
{
	PSurf<T,n>		*surf;
	Collision<T,n>	*collision;

	void init();

	void _eval(T u, T v, int d1, int d2);

	void localSimulate(double dt);

protected:
	Point<T,n>		_p;
	Vector<T,n>		_u;
	Vector<T,n>		_v;
	UnitVector<T,n>	_n;

	T _st_p_u()		{ return T(0); }
	T _end_p_u()	{ return T(1); }
	T _st_p_v()		{ return T(0); }
	T _end_p_v()	{ return T(1); }

friend class Collision<T,n>;

 public:
	PPlane(const Point<T,n>& p, const Vector<T,n>& u, const Vector<T,n>& v, PSurf<T,n>* s);
	PPlane(const PPlane<T,n>& pl);
#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
	PPlane(istream& in, PSurf<T,n>* s);
#endif

	virtual
	~PPlane() {}

	void setCollision(Collision<T,n>* co) { collision = co; }

//	PSurf<T,n>* split(T t) { return NULL;}

	Point<T,n> position()	{ return _present*_p; }
	UnitVector<T,n> normal(){ return _present*_n; }
	Point<T,n> u()			{ return _present*_u; }
	Point<T,n> v()			{ return _present*_v; }

	// Vector<T,n> speed() { return _hv; }

	const char* identity() const { return "P-PLANE-SURF";}
};


} // end namespace GM_lib



using namespace GM_lib;

#include "GM_Collision.h"

// template <class T, int n>
// const Vector3D<T> PPlane<T,n>::g(0,0,9.81);

template <class T, int n>
inline
PPlane<T,n>::PPlane(const Point<T,n>& p, const Vector<T,n>& u, const Vector<T,n>& v, PSurf<T,n>* s):PSurf<T,n>() 
{
	_p = p;
	_u = u;
	_v = v;
	if(n==3) _n = Vector3D<T>(u)^v;
	Sphere<GLfloat,3> sp(p);
	sp += p+u+v; sp += p+u; sp += p+v;
	setSurroundingSphere(sp);
	__amb.set(0.3,0.3,0.3,1.0);
	__dif.set(0.3,0.3,0.3,1.0);
	__spc.set(0.1,0.1,0.1,1.0);
	surf		= s;
	collision	= NULL;
	init();
}


template <class T, int n>
inline	
PPlane<T,n>::PPlane(const PPlane<T,n>& c):PSurf<T,n>(c) 
{ 
	_p = c._p;
	_u = c._u;
	_v = c._v;
	_n = c._n;
	surf		= c.surf;
	collision	= c.collision;
}


#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
template <class T, int n>
inline	
PPlane<T,n>::PPlane(istream& in, PSurf<T,n>* s):PSurf<T,n>() 
{
	int n1;
	in >> n1;
	if(n1)
	{
		kvil kv;
		char file[100];
		in >> file;
		glGenTextures(1,&__texname);
		kv.LoadGLSphereMap(__texname, file);
	}
	in >> __sam1 >> __sam2;
	in  >> __amb >> __dif >> __spc;
	in >> _p >> _u >> _v;
	if(n==3) _n = Vector3D<T>(_u)^_v;
	Sphere<GLfloat,3> sp(_p);
	sp += _p+_u+_v; sp += _p+_u; sp += _p+_v;
	setSurroundingSphere(sp);
	surf = s;
	collision = NULL;
	init();
}
#endif


template <class T, int n>
inline
void PPlane<T,n>::init()
{
	resetPlot(__sam1,__sam2);
}


template <class T, int n>
void PPlane<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setSize(10);

	__p[0] = _p + u*_u+v*_v ;
	if(d1)
	{
		__p[1] = _u;	// S_u
	}
	if(d2)
	{
		__p[3] = _v;	// S_v
	}
	if(d1>1)
	{
		__p[2][0] = T(0);	// S_uu
		__p[2][1] = T(0);
		__p[2][2] = T(0);
	}
	if(d1>1 && d2>1)
	{
		__p[4][0] = T(0);	// S_uv
		__p[4][1] = T(0);
		__p[4][2] = T(0);
	}
	if(d2>1)
	{
		__p[5][0] = T(0);	// S_vv
		__p[5][1] = T(0);
		__p[5][2] = T(0);
	}
	if(d1>2)
	{
		__p[6][0] = T(0);	// S_uuu
		__p[6][1] = T(0);
		__p[6][2] = T(0);
	}
	if(d1>2 && d2>2)
	{
		__p[7][0] = T(0);	// S_uuv
		__p[7][1] = T(0);
		__p[7][2] = T(0);

	}
	if(d1>2 && d2>2)
	{
		__p[8][0] = T(0);	// S_uvv
		__p[8][1] = T(0);
		__p[8][2] = T(0);

	}
	if(d2>2)
	{
		__p[9][0] = T(0);	// S_vvv
		__p[9][1] = T(0);
		__p[9][2] = T(0);
	}


	if(d1>0 && d2>0 && n==3)
		__n = _n;
}


template <class T, int n>
void PPlane<T,n>::localSimulate(double dt)
{		
}


#endif   /* GM_PPLANE_H_ */