
//////////////////////////////////////////////////////////////////////
// GM_PSurf.c: implementation of the PSurf class.
//////////////////////////////////////////////////////////////////////
#include "GM_PSurf.h"

using namespace GM_lib;


//////////////////////////////////////////////////////////////////////
// Closest point evaluation
//////////////////////////////////////////////////////////////////////

template <class T, int n>
bool PSurf<T,n>::closestPoint(const Point<T,n>& q, T& u, T& v)
{
	T a11, a12, a21, a22, b1, b2;
	T du, dv, det;
	Point<T,n> p = _present.inverse2()*q;

	for(int i=0; i<20;i++)
	{
		_eval(u, v, 2, 2);
		Vector<T,n> d = p-__p[0][0];

		a11 = d*__p[2][0]-__p[1][0]*__p[1][0];
		a12 = d*__p[1][1]-__p[1][0]*__p[0][1];
		a21 = a12;
		a22 = d*__p[0][2]-__p[0][1]*__p[0][1];

		b1  = -(d*__p[1][0]);
		b2  = -(d*__p[0][1]);

		det = a11*a22-a12*a21;
		du  = (b1*a22-a12*b2)/det;
		dv  = (a11*b2-b1*a21)/det;
		u += du; v += dv;
		if(fabs(du)< 1e-6 && fabs(dv) < 1e-6) return true;
	}
	return false;
}


//////////////////////////////////////////////////////////////////////
// Curvature evaluation
//////////////////////////////////////////////////////////////////////

template <class T, int n>
T PSurf<T,n>::gaussCurvature(T u, T v)
{
	_eval_(u,v,2,2);
	UnitVector<T,3> N   = normal(u,v);	//__p[0][1]^__p[1][0];
	Vector<T,3>		du  = der_u(u,v);	//__p[0][1];
	Vector<T,3>		dv  = der_v(u,v);	//__p[1][0];
	Vector<T,3>		duu = der_uu(u,v);	//__p[0][2];
	Vector<T,3>		duv = der_uv(u,v);	//__p[1][1];
	Vector<T,3>		dvv = der_vv(u,v);	//__p[2][0];
	T E=du*du;
	T F=du*dv;
	T G=dv*dv;
	T e=N*duu;
	T f=N*duv;
	T g=N*dvv;
	return (e*g - f*f)/(E*G-F*F);
}

template <class T, int n>
T PSurf<T,n>::meanCurvature(T u, T v)
{
	_eval_(u,v,2,2);
	UnitVector<T,3> N   = normal(u,v);
	Vector<T,3>		du  = der_u(u,v);
	Vector<T,3>		dv  = der_v(u,v);
	Vector<T,3>		duu = der_uu(u,v);
	Vector<T,3>		duv = der_uv(u,v);
	Vector<T,3>		dvv = der_vv(u,v);
	T E=du*du;
	T F=du*dv;
	T G=dv*dv;
	T e=N*duu;
	T f=N*duv;
	T g=N*dvv;
    return 0.5*(e*G-2*(f*F)+g*E)/(E*G-F*F);
}

