//********************************************
// GM_Psurf.h: interface for the PSurf class.
//********************************************
#ifndef GM_PSURF_H_
#define GM_PSURF_H_

#include <GM_DMatrix.h>
#include <GM_Material.h>
#include <GM_LA_DisplayObject.h>

namespace GM_lib {

template <class T, int n>
class PSurf: public LA_DisplayObject
{
	bool						__dynamic;		// true -> Vertex-arrays, fals -> display-list
	GLuint						__display_list;	// For OpenGl display-lists
	unsigned int				__disp_mode;	// Displaying SurroundingSphere or normals/derivatives/...

	// Private help functions 
	void _eval_(T u, T v, int d1, int d2) {	if(!(d1 <=__d1 && d2 <=__d2 &&  u ==__u && v ==__v))
							{__u=u;__v=v;__d1=d1;__d2=d2;_eval(_shift_u(u),_shift_v(v),d1,d2);}}
	int  _sum(int i, int j) { int r=0; for(j+=i+1; j>1; j--) r+=j; return r-=i; }

protected:
	Point2D<T>					__st_sam;		// Frst sampling value usually (startPar_u(),startPar_v())
	Point2D<T>					__end_sam;		// Last sampling value usually (endtPar_u(),endPar_v())
	int							__sam1;			// Samplings-number u directions
	int							__sam2;			// Samplings-number v directions
	int							__defalt_d;		// used by operator() for number of derivative to evaluate. 
	Material					__material;		// The material of this Surface
	DMatrix<Arrow<GLfloat,3> >	__vertex;		// Vertex-arrays for dynamic display
	DMatrix<Point<GLfloat,2> >	__texcoords;	// The correspoding arrays for textur coordinates

	DisplayObject*				__parent;		// The mother if present!


	//************************************************************
	// xxxxx     The result of the previous evaluation!!!   xxxxxx
	//************************************************************
	DMatrix<Vector<T,n> >	__p;		// Position and belonging partial derivatives
	Vector<T,3>				__n;		// Surface normal, for display in 3D
	T						__u;		// The parameter value in u-direction used for last evaluation
	T						__v;		// The parameter value in v-direction used for last evaluation
	int						__d1;		// Number of derivatives in u-direction computed last time
	int						__d2;		// Number of derivatives in v-direction computed last time
	bool					__diagonal; // True if only upper left half of matrix is evaluated.


	// Shift of parameter.
	T	__tr_u;		// translate u-parametre
	T	__sk_u;		// skale
	T	_shift_u(T u) {return _st_p_u()+__sk_u*(u - startPar_u()); }

	T	__tr_v;		// translate v-parametre
	T	__sk_v;		// skale
	T	_shift_v(T v) {return _st_p_v()+__sk_v*(v - startPar_v()); }

	//*********************************************************************
	//********   Virtual functions nessesary for inheritance !!!   ********
	//* This functions must be implemented in the real parametric surface *
	//*********************************************************************
	virtual
	void _eval(T u, T v, int d1, int d2) = 0;
	virtual
	T _st_p_u()  = 0;
	virtual
	T _end_p_u() = 0;
	virtual
	T _st_p_v()  = 0;
	virtual
	T _end_p_v() = 0;

	
	//**************************************************************
	//********   Virtual functions already implemented !!!   *******
	//**  Must only be implemented if you want something special  **
	//**************************************************************
	virtual
	void _makeSample(DMatrix<Arrow<T,n> >& a,  int m1, int m2);


	//**************************************************************************
	//*************     Functions inherited from DisplayObject.    *************
	//* You do not need to make these functions in the real parametric surface *
	//********      exept if you want to do something special.         *********
	//**************************************************************************
	void localDisplay();
	void localSelect();

 public:
	PSurf(int s1=20, int s2=20, DisplayObject* parent=NULL);
	PSurf(const PSurf<T,n>& c); 

	virtual
	~PSurf() { if( __display_list) glDeleteLists(__display_list,12); }

	void setUDomain(T start, T end) {__sk_u=(end-start)/(_end_p_u()-_st_p_u()); __tr_u=start-_st_p_u(); }
	void setUScaleDomain(T sc) {__sk_u = sc;}
	void setUTransDomain(T tr) {__tr_u = tr;}

	void setVDomain(T start, T end) {__sk_v=(end-start)/(_end_p_v()-_st_p_v()); __tr_v=start-_st_p_v(); }
	void setVScaleDomain(T sc) {__sk_v = sc;}
	void setVTransDomain(T tr) {__tr_v = tr;}

	T deltaPar_u()	{ return __sk_u*(_end_p_u()-_st_p_u()); }
	T startPar_u()	{ return _st_p_u()+__tr_u; }
	T endPar_u()	{ return startPar_u()+deltaPar_u(); }

	T deltaPar_v()	{ return __sk_v*(_end_p_v()-_st_p_v()); }
	T startPar_v()	{ return _st_p_v()+__tr_v; }
	T endPar_v()	{ return startPar_v()+deltaPar_v(); }


	void resetPlot( int m1=0, int m2=0, bool dynamic=false);

	void setEval(int d)					{ __defalt_d  = d; }
	void setDisplayMode(int i)			{ __disp_mode = i;}

	void setMaterial(const Material& m) { __material = m; }

	Point<T,n>  operator()(T u, T v);
	Vector<T,n> der_u(T u, T v);
	Vector<T,n> der_v(T u, T v);
	Vector<T,n> der_uu(T u, T v);
	Vector<T,n> der_vv(T u, T v);
	Vector<T,n> der_uv(T u, T v);
 	Vector<T,n> norm(T u, T v);
 	UnitVector<T,n> normal(T u, T v)	{ return norm(u,v); }

	Array<Vector<T,n> >		evaluateD(T u, T v, int d1, int d2);  // in local coords
	DMatrix<Vector<T,n> >&	evaluate(T u, T v, int d1, int d2);
	DMatrix<Vector<T,n> >&	evaluateLocal(T u, T v, int d1, int d2);
	DMatrix<Vector<T,n> >&	evaluateMother(T u, T v, int d1, int d2);

	Array<Vector<T,n> >		evaluateD(Point<T,2> p, int d1, int d2) { return evaluateD(p[0],p[1],d1,d2);}
	DMatrix<Vector<T,n> >&	evaluate(Point<T,2> p, int d1, int d2) { return evaluate(p[0],p[1],d1,d2);}
	DMatrix<Vector<T,n> >&	evaluateLocal(Point<T,2> p, int d1, int d2) { return evaluateLocal(p[0],p[1],d1,d2);}
	DMatrix<Vector<T,n> >&	evaluateMother(Point<T,2> p, int d1, int d2) { return evaluateMother(p[0],p[1],d1,d2);}

	virtual
	T gaussCurvature(T u, T v);
	virtual
	T meanCurvature(T u, T v);

	virtual
	bool closestPoint(const Point<T,n>& p, T& u, T& v);
	virtual
	void estimateClpPar(const Point<T,n>& p, T& u, T& v) {}

	virtual
	PSurf<T,n>* split(T t, int uv) {return 0;}

	virtual
	bool closedU() { return false; }
	virtual
	bool closedV() { return false; }

	const char* identity() const { return "PSURF";}
	
	template <class T_Stream>
	PSurf(T_Stream& in, int st) : LA_DisplayObject(in,st)
	{
		__d1 = -1;
		__d2 = -1; 
		__disp_mode=0;
		__display_list=0; 
		setEval(0);
		prIn(in);
	}
	
	template <class T_Stream>
	T_Stream& prOut(T_Stream& out)
	{
		out << __tr_u << element_separator << __sk_u << element_separator;
		out << __tr_v << element_separator << __sk_v << element_separator;
		out << __sam1 << element_separator << __sam2 << element_separator;
		out << __material << group_separator;
		return out;
	}
	
	template <class T_Stream>
	friend T_Stream& operator<<(T_Stream& out, PSurf<T,n>& s)
	{
		out << *((LA_DisplayObject *) &s);
		s.prOut(out);
		return out;
	}

	template <class T_Stream>
	T_Stream& prIn(T_Stream& in)
	{
		static Separator es(element_separator);
		static Separator gs(group_separator);
		in >> __tr_u >> es >> __sk_u >> es;
		in >> __tr_v >> es >> __sk_v >> es;
		in >> __sam1 >> es >> __sam2 >> es;
		in >> __material >> gs;
		return in;
	}
	
	template <class T_Stream>
	friend T_Stream& operator>>(T_Stream& in,  PSurf<T,n>& s)
	{
	    in >> *((LA_DisplayObject *) &s);
		s.prIn(in);
	}

	//**************************************************************
	//	PCurve version og affin transformation
	//	Necessary because if there is a parent 
	//	then give this parent a message if the surface is edited
	//**************************************************************

	// In local coordinates.
	void rotate(Angle a, const Vector<GLfloat,3>& rot_axel);
	void rotate(Angle a, const Point<GLfloat,3>& p,const UnitVector<GLfloat,3>& d);
	void translate(const Vector<GLfloat,3>& trans_vector);

	// In the global coordinates of the scene.
	void rotateGlobal(Angle a, const Vector<GLfloat,3>& rot_axel);
	void rotateGlobal(Angle a, const Point<GLfloat,3>& p,const UnitVector<GLfloat,3>& d);
	void translateGlobal(const Vector<GLfloat,3>& trans_vector);
};

} // end namespace GM_lib



using namespace GM_lib;


template <class T, int n>
inline
PSurf<T,n>::PSurf(int s1, int s2, DisplayObject* parent) : LA_DisplayObject(),__material()
{
	__d1 = -1;
	__d2 = -1; 
	__tr_u = T(0); 
	__sk_u = T(1);
	__tr_v = T(0); 
	__sk_v = T(1); 
	__sam1 = s1;
	__sam2 = s2;
	__parent = parent;
	__disp_mode    = 0;
	__display_list = 0; 
	__defalt_d     = 0;
	__dynamic      = false;

	_type_id = 1000;
}

template <class T, int n>
inline
PSurf<T,n>::PSurf(const PSurf<T,n>& c):LA_DisplayObject(c),__material(c.__material)
{
	__p  = c.__p;
	__n  = c.__n; 
	__u  = c.__u; 
	__v  = c.__v; 
	__d1 = c.__d1; 
	__d2 = c.__d2; 
	__tr_u = c.__tr_u; 
	__sk_u = c.__sk_u; 
	__tr_v = c.__tr_v; 
	__sk_v = c.__sk_v; 
	__sam1 = c.__sam1;
	__sam2 = c.__sam2;
	__parent = c.__parent;
	__disp_mode    = c.__disp_mode;
	__display_list = c.__display_list;
	__defalt_d     = c.__defalt_d;
	__dynamic      = c.__dynamic;

	_type_id = 1000;
}



template <class T, int n>
void PSurf<T,n>::localDisplay()
{
	__material.glSet();
	if(__dynamic)
	{
		for(int i=0;i<__vertex.dim1();i++)
		{
			glNormalPointer(GL_FLOAT,2*3*sizeof(GLfloat),(__vertex[i][0].dir()).ptr());
			glVertexPointer(3,GL_FLOAT,2*3*sizeof(GLfloat),(__vertex[i][0].pos()).ptr());
			if(__material.texture() && __texcoords.dim1()>2) 
				glTexCoordPointer(2,GL_FLOAT,2*sizeof(GLfloat),(__texcoords[i][0]).ptr());
			glDrawArrays(GL_TRIANGLE_STRIP,0,__vertex[i].dim());
		}
	}
	else
	{
		glCallList(__display_list);
		if(__disp_mode == 1)			glCallList(__display_list+2);
		for(int i=3; i<12; i++)
			if(__disp_mode == (i-1))	glCallList(__display_list+i);
	}
	__material.glUnSet();
}


template <class T, int n>
void PSurf<T,n>::localSelect()
{
	if(__dynamic)
	{
		for(int i=0;i<__vertex.dim1();i++)
		{
			glVertexPointer(3,GL_FLOAT,2*3*sizeof(GLfloat),(__vertex[i][0].pos()).ptr());
			glDrawArrays(GL_TRIANGLE_STRIP,0,__vertex[i].dim());
		}
	}
	else
		glCallList(__display_list+1);
}


template <class T, int n>
void  PSurf<T,n>::resetPlot( int m1, int m2, bool dynamic)
{
	if(m1<2) m1 =__sam1; else __sam1 = m1;
	if(m2<2) m2 =__sam2; else __sam2 = m2;

	__dynamic   = dynamic;

	if(__display_list)
	{
		glDeleteLists(__display_list, 12);
		__display_list = 0;
	}

    DMatrix<Arrow<T,n> > p;
	_makeSample(p,m1,m2);

	// Forenklet laging av suroundingSphere
	Sphere<GLfloat,3>  s(p[0][0].pos());
	s += p[p.dim1()-1][p.dim2()-1].pos();	s += p[p.dim1()/2][p.dim2()/2].pos();
	s += p[p.dim1()-1][0].pos();			s += p[0][p.dim2()-1].pos();
	s += p[p.dim1()-1][p.dim2()/2].pos();	s += p[p.dim1()/2][p.dim2()-1].pos();
	s += p[0][p.dim2()/2].pos();			s += p[p.dim1()/2][0].pos();
	setSurroundingSphere(s);

	if(dynamic)		// Using Vertex arrays
	{
		glEnableClientState(GL_NORMAL_ARRAY);				// The main vertex array (point and normal)
		glEnableClientState(GL_VERTEX_ARRAY);
		__vertex.setDim(p.dim1()-1,p.dim2()*2);
		for(int i=0;i<p.dim1()-1;i++)
			for(int j=0; j<p.dim2();j++)
			{
				__vertex[i][2*j]   = p[i][j].toFloat();
				__vertex[i][2*j+1] = p[i+1][j].toFloat();
			}

		if(__material.texture())							// The texture coordinate array
		{
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
			__texcoords.setDim(p.dim1()-1,p.dim2()*2);
			for(int i=0;i<p.dim1()-1;i++)
				for(int j=0; j<p.dim2();j++)
				{
					__texcoords[i][2*j]   = Point2D<float>(i/float(p.dim1()-1),j/float(p.dim2()-1));
					__texcoords[i][2*j+1] = Point2D<float>((i+1)/float(p.dim1()-1),j/float(p.dim2()-1));
				}
		}
	}
	else			// Using Display lists
	{
		int i,j;	
		T u,v;
		T du = deltaPar_u()/(m1-1);
		T dv = deltaPar_v()/(m2-1);

		__display_list = glGenLists(12);

		// Lager displayliste for display				nr. 0 main list
		glNewList(__display_list, GL_COMPILE);
			for(i=0; i<p.dim1()-1; i++)
			{
				glBegin(GL_TRIANGLE_STRIP);
				for(j=0; j<p.dim2(); j++) 
				{ 
					glTexCoord(Point2D<float>(i/float(p.dim1()-1),j/float(p.dim2()-1))); 
					glVertex(p[i][j]);
					glTexCoord(Point2D<float>((i+1)/float(p.dim1()-1),j/float(p.dim2()-1))); 
					glVertex(p[i+1][j]);
				}
				glEnd();
			}
		glEndList();

		// Lager displayliste for select				nr. 1
		glNewList(__display_list+1, GL_COMPILE);
			for(i=0; i<p.dim1()-1; i++)
			{
				glBegin(GL_TRIANGLE_STRIP);
				for(j=0; j<p.dim2(); j++) 
				{
					glPoint(p[i][j].pos());
					glPoint(p[i+1][j].pos());
				}
				glEnd();
			}
		glEndList();

		// Lager displayliste for SurroundingSphere		nr. 2
		Sphere3D sp(s,10,10);
		glNewList(__display_list+2, GL_COMPILE);
			sp.display();
		glEndList();

		// Lager displayliste for display av normaler	nr. 3	
		glNewList(__display_list + 3, GL_COMPILE);
			__material.glSetInverse();
			for(i=0; i<p.dim1(); i++)	
				for(j=0; j<p.dim2(); j++) 
				{ 
					glBegin(GL_LINES);
						glPoint(p[i][j].pos());
						glPoint(p[i][j].pos()+p[i][j].dir());
					glEnd();
				}
		glEndList();

		// Lager displayliste for display av deriverte opp til duuvv som vektorer		nr. 4,5,6,...
		int ii, jj, dd=4;
		for(ii=0; ii<3; ii++)
			for(jj=0; jj<3; jj++)
			{
				if(ii == 0 && jj == 0) continue;
				glNewList(__display_list+(dd++), GL_COMPILE);
					__material.glSetInverse();
					for(i=0; i<m1; i++)
					{
						u = startPar_u() + i*du;
						for(j=0;j<m2;j++)
						{	
							v = startPar_v() + j*dv;
							_eval(u,v,2,2);
							glBegin(GL_LINES);
							glPoint(__p[0][0]);
							glPoint(__p[0][0]+__p[ii][jj]);
							glEnd();
						}
					}
				glEndList();
			}
	}
}


template <class T, int n>
void  PSurf<T,n>::_makeSample(DMatrix<Arrow<T,n> >& a, int m1, int m2)
{
	int i,j;
	T u,v;
	T du = deltaPar_u()/(m1-1);
	T dv = deltaPar_v()/(m2-1);
	a.setDim(m1, m2);
	for(i=0; i<m1; i++)
	{
		u = startPar_u() + i*du;
		for(j=0;j<m2;j++)
		{	
			v = startPar_v() + j*dv;
			_eval(u,v,1,1);
			a[i][j] = Arrow<T,n>(__p[0][0],UnitVector<T,n>(__n));
		}
	}
}


template <class T, int n>
inline
Point<T,n> PSurf<T,n>::operator()(T u, T v)	
{ 
	_eval_(u,v,__defalt_d,__defalt_d);
	return _present*(Point<T,n>)__p[0][0];//  
}

template <class T, int n>
inline
Vector<T,n> PSurf<T,n>::der_u(T u, T v)	
{ 
	_eval_(u,v,1,0);
	return _present*__p[1][0];
}

template <class T, int n>
inline
Vector<T,n> PSurf<T,n>::der_v(T u, T v)	
{ 
	_eval_(u,v,0,1);
	return _present*__p[0][1];
}

template <class T, int n>
inline
Vector<T,n> PSurf<T,n>::der_uu(T u, T v)	
{ 
	_eval_(u,v,2,0);
	return _present*__p[2][0];
}

template <class T, int n>
inline
Vector<T,n> PSurf<T,n>::der_uv(T u, T v)	
{ 
	_eval_(u,v,2,2);
	return _present*__p[1][1];
}

template <class T, int n>
inline
Vector<T,n> PSurf<T,n>::der_vv(T u, T v)	
{ 
	_eval_(u,v,0,2);
	return _present*__p[0][2];
}

template <class T, int n>
inline
Vector<T,n> PSurf<T,n>::norm(T u, T v)	
{ 
	_eval_(u,v,1,1);
	return _present*__n;
}

template <class T, int n>
inline
Array<Vector<T,n> > PSurf<T,n>::evaluateD(T u, T v, int d1, int d2)
{
	_eval_(u,v,d1,d2);
	Array<Vector<T,n> > p(_sum(d1,d2));
	int i,j;
	for(i=0;i<=d1;i++)
		for(j=0;j<=min(i,d2);j++)	 p+= __p[i-j][j];
	for(;i<=max(d1,d2);i++)
		for(j=i-d1;j<=min(i,d2);j++) p+= __p[i-j][j];
	return p;
}

template <class T, int n>
inline
DMatrix<Vector<T,n> >& PSurf<T,n>::evaluate(T u, T v, int d1, int d2)
{
	static DMatrix<Vector<T,n> > tmp;
	_eval(u,v,d1,d2);
	tmp.setDim(__p.dim1(),__p.dim2());
	for (int i=0; i<tmp.dim1();i++)
	{
		for (int j=0; j<tmp.dim2();j++)
		{
			if(i==0 && j==0 ) tmp[0][0]=_present*(Point<T,n>)__p[0][0];
			else tmp[i][j]=_present*__p[i][j];
		}
	}
	return tmp;
}


template <class T, int n>
inline
DMatrix<Vector<T,n> >& PSurf<T,n>::evaluateLocal(T u, T v, int d1, int d2)
{
	_eval_(u, v, d1, d2);
	return __p;
}


template <class T, int n>
inline
DMatrix<Vector<T,n> >& PSurf<T,n>::evaluateMother(T u, T v, int d1, int d2)
{
	static DMatrix<Vector<T,n> > tmp;
	_eval(u,v,d1,d2);
	tmp.setDim(__p.dim1(),__p.dim2());
	for (int i=0; i<tmp.dim1();i++)
	{
		for (int j=0; j<tmp.dim2();j++)
		{
			if(i==0 && j==0 ) tmp[0][0]=_matrix*(Point<T,n>)__p[0][0];
			else tmp[i][j]=_matrix*__p[i][j];
		}
	}
	return tmp;
}


//****************************************************
//	PSurf version og affin transformation
//	Necessary because if there is a parent then
//	give the parent a message if the surface is edited
//****************************************************

// In local coordinates.
template <class T, int n>
void PSurf<T,n>::rotate(Angle a, const Vector<GLfloat,3>& rot_axel) 
{
	LA_DisplayObject::rotate(a,rot_axel); 
	if(__parent) __parent->edit(this); 
}
template <class T, int n>
void PSurf<T,n>::rotate(Angle a, const Point<GLfloat,3>& p,const UnitVector<GLfloat,3>& d)
{
	LA_DisplayObject::rotate(a,p,d);
	if(__parent) __parent->edit(this);
}
template <class T, int n>
void PSurf<T,n>::translate(const Vector<GLfloat,3>& trans_vector)
{
	LA_DisplayObject::translate(trans_vector);
	if(__parent)__parent->edit(this);
}

// In the global coordinates of the scene.
template <class T, int n>
void PSurf<T,n>::rotateGlobal(Angle a, const Vector<GLfloat,3>& rot_axel)
{
	LA_DisplayObject::rotateGlobal(a,rot_axel);
	if(__parent) __parent->edit(this);
}
template <class T, int n>
void PSurf<T,n>::rotateGlobal(Angle a, const Point<GLfloat,3>& p,const UnitVector<GLfloat,3>& d)
{
	LA_DisplayObject::rotateGlobal(a,p,d);
	if(__parent) __parent->edit(this);
}
template <class T, int n>
void PSurf<T,n>::translateGlobal(const Vector<GLfloat,3>& trans_vector)
{
	LA_DisplayObject::translateGlobal(trans_vector);
	if(__parent) __parent->edit(this);
}

#include "GM_PSurf.c"

#endif   /* GM_PSURF_H_ */
