
//********************************************
// GM_Psurf.h: interface for the PSurf class.
//********************************************
#ifndef GM_PSURF_H_
#define GM_PSURF_H_
 
#include <iostream>
#include <fstream>
using namespace std;

#include <GM_Point.h>
#include <GM_Array.h>
#include <GM_DMatrix.h>
#include <DisplayObject.h>
// #include "kvil.h"
#include <GM_Texture.h>

namespace GM_lib {

template <class T, int n>
class PSurf: public DisplayObject
{
	bool				__dynamic;
	GLuint				__display_list;

	int					__defalt_d;

	unsigned int		__disp_mode;

	virtual
	void _makeSample(DMatrix<Arrow<T,n> >& a,  int m1, int m2);

 protected:
	// Material data for the surf.
	GL_Color			__amb;
	GL_Color			__dif;
	GL_Color			__spc;
	float				__shininess;
		
	GLuint				__texname;

	int					__sam1;
	int					__sam2;

	// Existing evaluation.
	Array<Vector<T,n> >	__p;
	Vector<T,n>			__n;
	T					__u;
	T					__v;
	int					__d1;
	int					__d2;

	// Aproximated pwl-curve, exist if __c!= NULL
	// PSurf<T,n>*			__c;
	T					__eps;

	// Shift of parameter.
	T					__tr_u;	// translate
	T					__sk_u;	// skale
	T	_shift_u(T u) {return _st_p_u()+__sk_u*(u - startPar_u()); }

	T					__tr_v;	// translate
	T					__sk_v;	// skale
	T	_shift_v(T v) {return _st_p_v()+__sk_v*(v - startPar_v()); }

	// Virtual functions nessesary for inheritance.
	virtual
		void _eval(T u, T v, int d1, int d2) = 0;
	virtual
		T _st_p_u() = 0;
	virtual
		T _end_p_u() = 0;
	virtual
		T _st_p_v() = 0;
	virtual
		T _end_p_v() = 0;

	// Functions inherited from DisplayObject.
	void localDisplay();
	void localSelect();

 public:
	PSurf();
	PSurf(const PSurf<T,n>& c); 

	virtual
	~PSurf() { //if(__c) delete __c; 	
		if(__display_list) glDeleteLists(__display_list,12); }

	void setUDomain(T start, T end) {__sk_u=(end-start)/(_end_p_u()-_st_p_u()); __tr_u=start-_st_p_u(); }
	void setUScaleDomain(T sc) {__sk_u = sc;}
	void setUTransDomain(T tr) {__tr_u = tr;}

	void setVDomain(T start, T end) {__sk_v=(end-start)/(_end_p_v()-_st_p_v()); __tr_v=start-_st_p_v(); }
	void setVScaleDomain(T sc) {__sk_v = sc;}
	void setVTransDomain(T tr) {__tr_v = tr;}

	T deltaPar_u()	{ return __sk_u*(_end_p_u()-_st_p_u()); }
	T startPar_u()	{ return _st_p_u()+__tr_u; }
	T endPar_u()	{ return startPar_u()+deltaPar_u(); }

	T deltaPar_v()	{ return __sk_v*(_end_p_v()-_st_p_v()); }
	T startPar_v()	{ return _st_p_v()+__tr_v; }
	T endPar_v()	{ return startPar_v()+deltaPar_v(); }


	void resetPlot( int m1, int m2, bool dynamic=false);

	void setEval(int d)			{ __defalt_d  = d; }
	void setDisplayMode(int i)	{ __disp_mode = 1<<(i-1);}

	Point<T,n>& operator()(T u, T v);
	Vector<T,n>& der_u(T u, T v);
	Vector<T,n>& der_v(T u, T v);
	Vector<T,n>& der_uu(T u, T v);
	Vector<T,n>& der_vv(T u, T v);
	Vector<T,n>& der_uv(T u, T v);
 	Vector<T,n>& norm(T u, T v);
 	UnitVector<T,n> normal(T u, T v)	{ return norm(u,v); }

//	Array<Vector<T,n> >& evaluate(T u, T v, int d1, int d2);

	virtual
	bool closestPoint(const Point<T,n>& p, T& u, T& v);

	virtual
	void estimateClpPar(const Point<T,n>& p, T& u, T& v) {}

	virtual
	PSurf<T,n>* split(T t, int uv) {return NULL;}

	const char* identity() const { return "PSURF";}
};

} // end namespace GM_lib



using namespace GM_lib;


template <class T, int n>
inline
PSurf<T,n>::PSurf():DisplayObject()
{
	__d1 = -1;
	__d2 = -1; 
//	__c = NULL; 
	__tr_u=T(0); 
	__sk_u=T(1);
	__tr_v=T(0); 
	__sk_v=T(1); 

	__texname=0;
	__disp_mode=0;
	__display_list=0; 
	__sam1 = __sam2=20;
	__shininess = 2.0;
	setEval(0);
}

template <class T, int n>
inline
PSurf<T,n>::PSurf(const PSurf<T,n>& c):DisplayObject(c)
{
	__p=c.__p;
	__n=c.__n; 
	__u=c.__u; 
	__v=c.__v; 
	__d1=c.__d1; 
	__d2=c.__d2; 
//	__c=c.__c; 
	__eps=c.__eps;
	__tr_u=c.__tr_u; 
	__sk_u=c.__sk_u; 
	__tr_v=c.__tr_v; 
	__sk_v=c.__sk_v; 
	__sam1=c.__sam1;
	__sam2=c.__sam2;
	__amb = c.__amb;
	__dif = c.__dif ; 
	__spc = c.__spc ;
	__shininess = c.__shininess;
	__disp_mode=c.__disp_mode;
	__display_list=c.__display_list;
	__defalt_d=c.__defalt_d;
	__texname=c.__texname;
	__dynamic=c.__dynamic;
	setEval(0); 
}



template <class T, int n>
void PSurf<T,n>::localDisplay()
{
	if(__dynamic)
	{
		glCallList(__display_list);
	}
	else
	{
		glMaterial(GL_FRONT,GL_AMBIENT,__amb);
		glMaterial(GL_FRONT,GL_DIFFUSE,__dif);
		glMaterial(GL_FRONT,GL_SPECULAR,__spc);
		glMaterial(GL_FRONT,GL_SHININESS,__shininess);
		if(__texname)
		{
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D, __texname);
		}
		glCallList(__display_list);
		if(__texname) glDisable(GL_TEXTURE_2D);
		for(int i=2; i<12; i++)
			if((1<<(i-2)) & __disp_mode)glCallList(__display_list+i);
	}
}


template <class T, int n>
void PSurf<T,n>::localSelect()
{
	if(__dynamic)
	{
	}
	else
	{
		glCallList(__display_list+1);
	}
}


template <class T, int n>
void  PSurf<T,n>::resetPlot( int m1, int m2, bool dynamic)
{
	__sam1 = m1;
	__sam2 = m2;

	if(__display_list)
	{
		glDeleteLists(__display_list,12);
		__display_list=0;
	}
	__dynamic = dynamic;

	if(dynamic)
	{
	}
	else
	{
		int i,j,ii;
		DMatrix<Arrow<T,n> >  p;
		T u,v;
		T du = deltaPar_u()/(m1-1);
		T dv = deltaPar_v()/(m2-1);

		_makeSample(p,__sam1,__sam2);

		Sphere<GLfloat,3>  s(p[0][0].pos());
		s += p[p.dim1()-1][p.dim2()-1].pos();
		s += p[p.dim1()-1][0].pos();
		s += p[0][p.dim2()-1].pos();
		setSurroundingSphere(s);

		__display_list = glGenLists(12);

		// Lager displayliste for display
		glNewList(__display_list, GL_COMPILE);
		glEnable(GL_COLOR_MATERIAL);
		    glMaterial(GL_FRONT_AND_BACK,GL_AMBIENT,__amb);
			glMaterial(GL_FRONT_AND_BACK,GL_DIFFUSE,__dif);
			glMaterial(GL_FRONT_AND_BACK,GL_SPECULAR,__spc);
			glMaterial(GL_FRONT_AND_BACK,GL_SHININESS,2);
			
			for(i=0;i<p.dim1()-1;i++)
			{
				glBegin(GL_TRIANGLE_STRIP);
				for(j=0;j<p.dim2();j++) 
				{ 
					if(__texname)
						glTexCoord(Point2D<float>(i/float(p.dim1()-1),j/float(p.dim2()-1))); 
					glVertex(p[i][j]);
					if(__texname)
						glTexCoord(Point2D<float>((i+1)/float(p.dim1()-1),j/float(p.dim2()-1))); 
					glVertex(p[i+1][j]);
				}
				glEnd();
			}
			glDisable(GL_COLOR_MATERIAL);
		glEndList();

		// Lager displayliste for select
		glNewList(__display_list+1, GL_COMPILE);
			for(i=0;i<p.dim1()-1;i++)
			{
				glBegin(GL_TRIANGLE_STRIP);
				for(j=0;j<p.dim2();j++) 
				{ 

					glPoint(p[i][j].pos());
					glPoint(p[i+1][j].pos());
				}
				glEnd();
			}
		glEndList();

		
		glNewList(__display_list + 2, GL_COMPILE);
			glMaterial(GL_FRONT,GL_AMBIENT,__amb.inverse());
			glMaterial(GL_FRONT,GL_DIFFUSE,__dif.inverse());
			glMaterial(GL_FRONT,GL_SPECULAR,__spc.inverse());
			glMaterial(GL_FRONT,GL_SHININESS,2);

			for(i=0;i<p.dim1();i++)	
				for(j=0;j<p.dim2();j++) 
				{ 
					glBegin(GL_LINES);
						glPoint(p[i][j].pos());
						glPoint(p[i][j].pos()+p[i][j].dir());
					glEnd();
				}
		glEndList();

		for(ii=3; ii < 12; ii++)
		{
			glNewList(__display_list+ii, GL_COMPILE);
				glMaterial(GL_FRONT,GL_AMBIENT,__amb.inverse());
				glMaterial(GL_FRONT,GL_DIFFUSE,__dif.inverse());
				glMaterial(GL_FRONT,GL_SPECULAR,__spc.inverse());
				glMaterial(GL_FRONT,GL_SHININESS,2);

				for(i=0; i<m1; i++)
				{
					u = startPar_u() + i*du;
					for(j=0;j<m2;j++)
					{	
						v = startPar_v() + j*dv;
						_eval(u,v,3,3);
						glBegin(GL_LINES);
							glPoint(__p[0]);
							glPoint(__p[0]+__p[ii-2]);
						glEnd();
					}
				}
			glEndList();
		}

	}
}


template <class T, int n>
void  PSurf<T,n>::_makeSample(DMatrix<Arrow<T,n> >& a, int m1, int m2)
{
	int i,j;
	T u,v;
	T du = deltaPar_u()/(m1-1);
	T dv = deltaPar_v()/(m2-1);
	a.setdim(m1, m2);
	for(i=0; i<m1; i++)
	{
		u = startPar_u() + i*du;
		for(j=0;j<m2;j++)
		{	
			v = startPar_v() + j*dv;
			_eval(u,v,1,1);
			a[i][j]=Arrow<T,n>(__p[0],UnitVector<T,n>(__n));
		}
	}
}

template <class T, int n>
inline
Point<T,n>& PSurf<T,n>::operator()(T u, T v)	
{ 
	if(!(0 <=__d1 && 0 <=__d2 && u ==__u && v ==__v))
		_eval(_shift_u(u),_shift_v(v),__defalt_d,__defalt_d);
	return _present*__p[0]; //  
}

template <class T, int n>
inline
Vector<T,n>& PSurf<T,n>::der_u(T u, T v)	
{ 
	if(!(1 <=__d1 && 0 <=__d2 &&  u ==__u && v ==__v))
		_eval(_shift_u(u),_shift_v(v),1,0);
	return _present*Vector<T,n>(__p[1]); //
}

template <class T, int n>
inline
Vector<T,n>& PSurf<T,n>::der_v(T u, T v)	
{ 
	if(!(0 <=__d1 && 1 <=__d2 &&  u ==__u && v ==__v))
		_eval(_shift_u(u),_shift_v(v),0,1);
	return _present*Vector<T,n>(__p[2]);//
}

template <class T, int n>
inline
Vector<T,n>& PSurf<T,n>::der_uu(T u, T v)	
{ 
	if(!(2 <=__d1 && 0 <=__d2 && u ==__u && v ==__v))
		_eval(_shift_u(u),_shift_v(v),2,0);
	return _present*Vector<T,n>(__p[3]);//
}

template <class T, int n>
inline
Vector<T,n>& PSurf<T,n>::der_uv(T u, T v)	
{ 
	if(!(2 <=__d1 && 2 <=__d2 && u ==__u && v ==__v))
		_eval(_shift_u(u),_shift_v(v),2,2);
	return _present*Vector<T,n>(__p[4]);//
}

template <class T, int n>
inline
Vector<T,n>& PSurf<T,n>::der_vv(T u, T v)	
{ 
	if(!(0 <=__d1 && 2 <=__d2 && u ==__u && && v ==__v))
		_eval(_shift_u(u),_shift_v(v),0,2);
	return _present*Vector<T,n>(__p[5]); //
}

template <class T, int n>
inline
Vector<T,n>& PSurf<T,n>::norm(T u, T v)	
{ 
	if(!(1 <=__d1 && 1 <=__d2 && u ==__u && v ==__v))
		_eval(_shift_u(u),_shift_v(v),1,1);
	return _present*Vector<T,n>(__n);//_present*__n;   //
}

/*
template <class T, int n>
inline
Array<Vector<T,n> >& PSurf<T,n>::evaluate(T u, T v, int d1, int d2)
{
	if(!(d <=__d1 && d <=__d2 && u ==__u && v ==__v))
		_eval(_shift_u(u),_shift_v(v),d1,d2);
	return __p;//_present*__p;
}
*/


#include "GM_PSurf.c"

#endif   /* GM_PSURF_H_ */
