
//*************************************************************************************
// GM_PWLCurve.h: interface for the PWLCurve classes.
//*************************************************************************************
#ifndef GM_PWLCURVE_H_
#define GM_PWLCURVE_H_

#include "GM_PCurve.h"

namespace GM_lib{

template <class T, int n>
class PWLCurve: public PCurve<T,n>
{
	void init(bool make_t=false);

	void localSimulate(double dt){}

	void _makeSample(Array<Point<T,n> >& a, T eps);

protected:
	Array<Point<T,n> >	_c;
	Array<T> 			_t;


	void _eval(T t, int d);

	T _st_p() { return _t[0]; }
	T _end_p(){ return _t[_t.size()-1]; }

 public:
	PWLCurve(const Array<Point<T,n> >& c):PCurve<T,n>() {_c=c; init();}
	PWLCurve(const Array<Point<T,n> >& c, const Array<T>& t):PCurve<T,n>() {_c=c; _t=t; init();}
	PWLCurve(const PWLCurve<T,n>& c):PCurve<T,n>(c) { _c=c._c; _t=c._t; }

	virtual
	~PWLCurve() {}

	PCurve<T,n>* split(T t) { return NULL;}

	const char* identity() const { return "PWL-CURVE";}
};


} // end namespace GM_lib


#include <GM_DMatrix.h>
using namespace GM_lib;

template <class T, int n>
void PWLCurve<T,n>::init(bool make_t)
{
	if(make_t)
	{
	}
	__amb.set(0.1,0.5,0.4,1.0); // rd
	__dif.set(0.0,1.0,1.0,1.0); // rd
	__spc.set(1.0,1.0,1.0,1.0); // hvit
		
	resetPlot(0.001);
}


template <class T, int n>
void PWLCurve<T,n>::_eval(T t, int d)
{
	if(d<=__d && t==__t) return;
	__t=t;
	__d=d;
/*
	int i,j,nn = _c[0].size();
	T t1=T(1)-t;
	static GM_lib::DMatrix<T> b(10,10);
	if(nn > b.dim1()) b.setdim(nn,nn);

	b[0][0] = T(1.0);
	for(i=1; i<nn; i++)
	{
		b[i][0] = t1*b[i-1][0];
		for(j=1;j<i;j++) b[i][j] = t*b[i-1][j-1]+t1*b[i-1][j];
		b[i][i] = t*b[i-1][i-1];
	}

	__p.setSize(d+1);
	for(i=0;i<=d;i++)
	{
		__p[i] = Point<T,n>(0.0);
		for(j=0; j<nn-i; j++) __p[i] += _c[i][j]*b[nn-i-1][j];
	}*/
}



template <class T, int n>
void  PWLCurve<T,n>::_makeSample(Array<Point<T,n> >& a, T eps)
{
	for(int i=0; i<_c.size();i++)
		a += _c[i];
}


#endif   /* GM_PWLCURVE_H_ */