#include "GM_Point.h"

namespace GM_lib {


template <class T, int n>
Arrow<T,n>* Point<T,n>::_a = new Arrow<T,n>();

template <class T, int n>
int        Point<T,n>::_type = 1;



//********************************************************
//******************   Angle computing.    - *************
//********************************************************

template <class T, int n>
Angle  Point<T,n>::angle(Point<T,n> v) const
{
   Point<T,n> u = (*this);
   u /= u.length();
   v /= v.length();
   double ang = u*v;
   if (ang > 1.0) ang = 1.0;
   else if (ang < -1.0) ang = -1.0;
   ang = acos(ang);
   if (n ==2)
   {
      double orient = u[0]*v[1]-u[1]*v[0];
      if (orient >= 0.0) 	return Angle(ang);
      else 		            return Angle(2*M_PI - ang);
   }
   else                     return Angle(ang);
}


//********************************************************
//******************   Boolean on sorting. - *************
//********************************************************

template <class T, int n>
bool Point<T,n>::operator < (const Point<T,n>& v) const
{
   switch(_type){
   case 1: return bool((_a->pos()-(*this)).length() < (_a->pos()-v).length());
   case 2: return bool((_a->dir()*(*this)) <  (_a->dir()*v));
   case 3: return bool(_a->dir().angle((*this)-_a->pos()) < _a->dir().angle(v-_a->pos()));
   default: return false;// Use a debug error-message
   }
}

template <class T, int n>
bool Point<T,n>::operator > (const Point<T,n>& v) const
{
   switch(_type){
   case 1: return bool((_a->pos()-(*this)).length() > (_a->pos()-v).length());
   case 2: return bool((_a->dir()*(*this)) >  (_a->dir()*v));
   case 3: return bool(_a->dir().angle((*this)-_a->pos()) > _a->dir().angle(v-_a->pos()));
   default: return false;// Use a debug error-message
   }
}


//********************************************************
//******************  init type of test.   - *************
//********************************************************

template <class T, int n>
void Point<T,n>::setTestType(int t, const Point<T,n>& p, const Vector<T,n>& d)
{
   _type = t;
   if (t==1)      _a->setPos(p);
   else if (t==2) _a->setDir(p);
   else if (t==3){_a->setPos(p); _a->setDir(d);}
   else 
   {
    // Use a debug error message 
   }
}


//********************************************************
//*****************  insert Point in Box.  - *************
//********************************************************
template <class T, int n>
void Box<T,n>::insert(const Point<T,n>& p)
{
	for(int i=0;i<n;i++)
	{
		if(p(i) < _min[i]) _min[i] = p(i);
		if(p(i) > _max[i]) _max[i] = p(i);
	}
}


//********************************************************
//*****************  insert Box in Box.    - *************
//********************************************************
template <class T, int n>
void Box<T,n>::insert(const Box<T,n>& p)
{
	for(int i=0;i<n;i++)
	{
		if(p._min(i) < _min[i]) _min[i] = p._min(i);
		if(p._max(i) > _max[i]) _max[i] = p._max(i);
	}
}


//********************************************************
//*************  Point is inside the Box?  - *************
//********************************************************
template <class T, int n>
bool  Box<T,n>::surround(const Point<T,n>& p) const
{
	for(int i=0;i<n;i++)
	{
		if(p(i) < _min(i)) return false;
		if(p(i) > _max(i)) return false;
	}
	return true;
}

//********************************************************
//*************  Box is inside the Box?  -   *************
//********************************************************
// BBang 15.07.00
template <class T, int n>
bool  Box<T,n>::surround(const Box<T,n>& p) const
{
	for(int i=0;i<n;i++)
	{
		if(p._min(i) < _min(i)) return false;
		if(p._max(i) > _max(i)) return false;
	}
	return true;
}


//********************************************************
//********* This box intersect an other Box?  - **********
//********************************************************
template <class T, int n>
bool  Box<T,n>::intersect(const Box<T,n>& p) const
{
	for(int i=0;i<n;i++)
	{
		if(p._min(i) > _max(i)) return false;
		if(p._max(i) < _min(i)) return false;
	}
	return true;
}



//********************************************************
//********* Expanding the sphere with a point - **********
//********************************************************
template <class T, int n>
Sphere<T,n>& Sphere<T,n>::operator+=(const Point<T,n>& p)
{
	if(_valid)
	{
		Vector<T,n> v = p - _pos;
		T r, d = v.length();

		if (d > _value)
		{
			r		= (d+_value)/2;
			_pos   += ((r-_value)/d)*v;
			_value	= r;
		}
	}
	else
	{
		_pos   = p;
		_value = T(0);
		_valid = true;
	}

	return *this;
}



//********************************************************
//********* -  Union of two speres  - ********************
//********************************************************
template <class T, int n>
Sphere<T,n>& Sphere<T,n>::operator+=(const Sphere<T,n>& p)
{
	if(p._valid)
	{
		if(_valid)
		{
			Vector<T,n> v =  p._pos - _pos;
			T r, d = v.length();
			if(_value > p._value)
			{
				if (d > _value - p._value)
				{
					r		= (d+_value+p._value)/2;
					_pos   += ((r-_value)/d)*v;
					_value	= r;
				}
			}
			else
			{
				if (d > p._value - _value)
				{
					r		= (d+_value+p._value)/2;
					_pos   += ((r-_value)/d)*v;
					_value	= r;
				}
				else
					*this = p;
			}
		}
		else
			*this  = p;
	}
	return *this;
}



//********************************************************
//***** - This Sphere intersect an other Sphere?  - ******
//********************************************************
template <class T, int n>
bool Sphere<T,n>::intersect(const Sphere<T,n>& p) const
{
	if(_valid && p._valid)
	{
		T d = (_pos - p._pos).length();
		return (_value+p._value) > d;
	}
	return false;
}

} // end namespace GM_lib

