
//****************************************************************************************
// GM_Point.h: interface for the Point/Vector/UnitVector/Arrow/Scalarfield/Sphere classes.
//****************************************************************************************
#ifndef GM_POINT_H_
#define GM_POINT_H_

#include <GM_Angle.h>
#include <GM_Stream.h>

#include <memory.h>

namespace GM_lib {


//*********************************************************
//********* The four classes in the affine system   *******
//*********************************************************

template <class T, int n>
class Point;

template <class T, int n>
class Vector;

template <class T, int n>
class UnitVector;

template <class T, int n>
class Arrow;

template <class T, int n>
class Line;

template <class T, int n>
class Box;

template <class T, int n>
class ScalarPoint;

template <class T, int n>
class Sphere;

template <class T, int n, int m>
class SubSpace;

//*********************************************************
//****************  The Point/Vector class  ***************
//*********************************************************


//#define Vector Point

template <class T, int n>
class Point
{
 protected:

   T pt[n];

   void cpy(const Point<T,n>& v)	{ memcpy(pt,v.pt,n*sizeof(T));}//for(int i=0;i<n;i++) pt[i] = v.pt[i];}
   void cpy(const T p[n])           { memcpy(pt,p,n*sizeof(T));}//for(int i=0;i<n;i++) pt[i] = p[i];}
   void cpy(const T& d)             { for(int i=0;i<n;i++) pt[i] = d;}

 private:

   static Arrow<T,n>* _a;
   static int _type;

 public:
   Point()							{ cpy(T(0.0));}
   Point(T d)						{ cpy(d);}
   Point(const T p[n])              { cpy(p);}
   Point(const Point<T,n>& v)		{ cpy(v);}

   Point<T,n>& operator=(const T d)    { cpy(d); return(*this);}
   Point<T,n>& operator=(const T p[n]) { cpy(p); return(*this);}
   Point<T,n>& operator=(const Point<T,n>& v) { cpy(v); return(*this);}

			//***** Get element. ******

   T* ptr() const						{ return (T*)this;}

   T& operator[](int i)					{ return pt[i];}  //return given element.   
   const T&  operator()(int i) const	{ return pt[i];}  //return copy.

                               
			//***** Additions and substractions. ******

   Point<T,n>& operator+=(const Point<T,n>& p)
      { for(int i=0;i<n;i++) pt[i] += p.pt[i]; return *this; }

   Point<T,n>& operator-=(const Point<T,n>& p)
      { for(int i=0;i<n;i++) pt[i] -= p.pt[i]; return *this; }

   Point<T,n> operator-() const
      { Point<T,n> p; for(int i=0;i<n;i++) p[i] = -pt[i]; return p;}

   Point<T,n> operator+(const Point<T,n>& p) const
      { Point<T,n> r = (*this); return r += p;}

   Point<T,n> operator-(const Point<T,n>& p) const
      { Point<T,n> r = (*this); return r -= p;}


			//***** Inner product. ******

   T  operator*(const Point<T,n>& v) const             
    				      {T r(0); for(int i=0;i<n;i++) r+=pt[i]*v.pt[i]; return r;}

   			//***** The angle between this and another point *****

   Angle angle(Point<T,n> v) const;

     		//***** Length (distanse from origo) *****

   T length() const
      { return T(sqrt(double((*this)*(*this))));}


       		//***** Casting *****

//   operator T() const					// Microsoft visual C++ 6.0 do not take this
//      { return length();}				// so comment this out using MSV-C++ 6.0 

	template <class G, int m>
	operator Point<G,m>() const
	{
		Point<G,m> v;
		const int j = min(m,n);
		for(int i=0; i<j; i++)	v[i] = G(pt[i]);	
		return v;
	}

	Point<float,n> toFloat() const
	{
		Point<float,n> v;
		for(int i=0; i<n; i++)	v[i] = (float)(pt[i]);	
		return v;
	}

	Point<double,n> toDouble() const
	{
		Point<double,n> v;
		for(int i=0; i<n; i++)	v[i] = (double)(pt[i]);	
		return v;
	}


			//***** Scaling. *****

   Point<T,n>& operator*=(double d)
      { for(int i=0;i<n;i++) pt[i] *= T(d); return *this; }

   Point<T,n> operator*(double d) const
      { Point<T,n> p=*this; for(int i=0;i<n;i++) p.pt[i]*=(T)d; return p;}


   Point<T,n>& operator%=(const Point<T,n>& p)
      { for(int i=0;i<n;i++) pt[i] *= p.pt[i]; return *this; }

   Point<T,n> operator%(const Point<T,n>& pp) const
      { Point<T,n> p=*this; p%=pp; return p;}

// *** Linux klarer ikke denne?!, flyttet den p�utsiden av klassen!***
// friend Point<T,n> operator*(const double d, Point<T,n> p)
//					{ p*=d; return p; }

  		   //***** Invers scaling. *****

   Point<T,n>&    operator/=(double d)
      { d = 1/d; (*this) *= d; return *this; }

   Point<T,n> operator/(double d) const
      { d = 1/d; return (*this)*d;}


			//***** Boolean on equality. *****

   bool  operator==(const Point<T,n>& p) const
                     { Point<T,n> d=(*this)-p;return bool((d*d)<POS_TOLERANCE);}

   bool  operator!=(const Point<T,n>& pt) const
                     { return !((*this) == pt); }


		   //***** Boolean on sorting. *****

   bool operator <  (const Point<T,n>& v) const;
   bool operator >  (const Point<T,n>& v) const;
   bool operator <= (const Point<T,n>& v) const { return !((*this) > v);}
   bool operator >= (const Point<T,n>& v) const { return !((*this) < v);}

   void setTestType(int, const Point<T,n>&, const Vector<T,n>& d=T(0));
};


template <class T, int n>
Point<T,n> operator*(const double d, Point<T,n> p) { return p*=d; }


		   //***** Stream operators for std-stream, QT-stream, etc. *****

template <class T_Stream, class T, int n>
inline
T_Stream &operator<<(T_Stream& out, const Point<T,n>& v)
{
	for(int i=0;i<n;i++) 
	{
		out << v(i);
		out << element_separator; 
	}
	return out; 
}


template <class T_Stream, class T, int n>
T_Stream& operator<<(T_Stream& out, const Point<T,n>* v)
{ 
	for(int i=0;i<n;i++) out << (*v)(i) << element_separator; 
	return out; 
}

template <class T_Stream, class T, int n>
T_Stream& operator>>(T_Stream& in, Point<T,n>& v)
{
	Separator es(element_separator);
	for(int i=0;i<n;i++) 
		in >> v[i] >> es;
	return in;
}

template <class T_Stream, class T, int n>
T_Stream& operator>>(T_Stream& in, Point<T,n>* v)
{
	Separator es(element_separator);
	for(int i=0;i<n;i++) 
		in >> (*v)[i] >> es;
	return in;
}


//*****************************************************
//****************  The Vector class  *****************
//*****************************************************

template <class T, int n>
class Vector : public Point<T,n>
{
public:
  
	Vector(T d=1) : Point<T,n>(d)				{}
	Vector(const T p[n]) : Point<T,n>(p)		{}
	Vector(const Point<T,n>& v) : Point<T,n>(v)	{}

	void normalize()	{ (*this)/=length(); }
	void setLength(T d)	{ d /= length(); (*this)*= d; }

	Vector<T,n> getLinIndVec() const
	{
		if(n==1|| length()==0) return Vector<T,n>(T(0));
		else
		{
			int i,j=0;
			for(i=1; i < n;i++) if((pt[i]*pt[i])>(pt[j]*pt[j])) j=i;
			if(j==0) i = 1;
			else	 i = j-1;

			Vector<T,n> r = *this;
			T tmp = -r[j];
			r[j]  = r[i];
			r[i]  = tmp;
			r    -= ((r*(*this))/((*this)*(*this)))*(*this);
			return r;
		}
	}
};


template <class T, int n>
Vector<T,n>& toVector(Point<T,n>& p) { return *((Vector<T,n>*)(&p)); }



//*********************************************************
//****************  The UnitVector class  *****************
//*********************************************************


template <class T, int n>
class UnitVector : public Vector<T,n>
{
public:
  
	UnitVector(T d=1) : Vector<T,n>(d)				{ Point<T,n>::operator/=(length());}
	UnitVector(const T p[n]) : Vector<T,n>(p)		{ Point<T,n>::operator/=(length());}
	UnitVector(const Point<T,n>& v) : Vector<T,n>(v){ Point<T,n>::operator/=(length());}

	Point<T,n>& operator=(const T d)   
		{ cpy(d);return Point<T,n>::operator/=(length());}
	Point<T,n>& operator=(const T p[n])
		{ cpy(p);return Point<T,n>::operator/=(length());}
	Point<T,n>& operator=(const Point<T,n>& v)
		{ cpy(v);return Point<T,n>::operator/=(length());}

 			//***** Get element. ******

	T& operator[](int i)         { return pt[i]; normalize();}

			//***** Additions. ******

	Point<T,n>& operator+=(const Point<T,n>& p)
		{ Point<T,n>::operator+=(p); return Point<T,n>::operator/=(length());}

			//***** Substractions. ******

	Point<T,n>& operator-=(const Point<T,n>& p)
		{ Point<T,n>::operator-=(p); return Point<T,n>::operator/=(length());}

 
			//***** Impossible Scaling. *****

	Point<T,n>& operator*=(const double d)
		{ return *this; }

  			//***** Impossible Invers scaling. *****

	Point<T,n>&    operator/=(double d)
		{ return *this; }

};


//*********************************************************
//****************  The Arrow class  **********************
//*********************************************************

template <class T, int n>
class Arrow : public Point<T,n>
{
protected:
	Vector<T,n> _dir;

public:
	Arrow() : Point<T,n>()								{ _dir = Vector<T,n>(1); }
	Arrow(const Point<T,n>& p) : Point<T,n>(p)			{ _dir = Vector<T,n>(1); }
	Arrow(const Point<T,n>& p ,const Vector<T,n>& v) : Point<T,n>(p) { _dir = v; }
	Arrow(const Arrow<T,n>& a) : Point<T,n>(a)			{ _dir = a._dir; }

	void setPos(const Point<T,n>& v) { cpy(v); }
	void setDir(const Vector<T,n>& v) { _dir = v; }

	const Point<T,n>&  pos() const { return (*this); }
	const Vector<T,n>& dir() const { return _dir; }

	Arrow<T,n>& operator=(const Arrow<T,n>& v)
	{ setPos(v.pos()); setDir(v.dir()); return *this;}

	Arrow<T,n>& operator+=(const Point<T,n>& p)
	{ for(int i=0;i<n;i++) pt[i] += p(i); return *this; }

	Arrow<T,n>& operator-=(const Point<T,n>& p)
	{ for(int i=0;i<n;i++) pt[i] -= p(i); return *this; }


	Arrow<T,n> operator+(const Point<T,n>& p) const
	{ Point<T,n> r = (*this); return r += p;}

	Arrow<T,n> operator-(const Point<T,n>& p) const
	{ Point<T,n> r = (*this); return r -= p;}

	Arrow<T,n>& operator+=(const Vector<T,n>& p)
	{ for(int i=0;i<n;i++) _dir[i] += p(i); return *this; }

	Arrow<T,n>& operator-=(const Vector<T,n>& p)
	{ for(int i=0;i<n;i++) _dir[i] -= p(i); return *this; }

	Arrow<T,n> operator+(const Vector<T,n>& p) const
	{ Point<T,n> r = (*this); return r += p;}

	Arrow<T,n> operator-(const Vector<T,n>& p) const
	{ Point<T,n> r = (*this); return r -= p;}

	Arrow<T,n> operator-() const
	{ Point<T,n> p; for(int i=0;i<n;i++) p[i] = -pt[i]; return p;}

	template <class G>
		operator Arrow<G,n>() const
	{
		Arrow<G,n> v;
		v.setPos((G)pos());
		v.setDir((G)dir());	
		return v;
	}

	Arrow<float,n> toFloat() const
	{
		Arrow<float,n> v;
		v.setPos(pos().toFloat());
		v.setDir(dir().toFloat());	
		return v;
	}

	Arrow<double,n> toDouble() const
	{
		Arrow<double,n> v;
		v.setPos(pos().toDouble());
		v.setDir(dir().toDouble());
		return v;
	}

	 	// ********** IOSTREAM overloaded operators ***********

public:
	
};
template <class T_Stream,class T, int n>
 T_Stream &operator<<(T_Stream& out, const Arrow<T,n>& v)
{ 
	out << v.pos() << element_separator << v.dir();
	return out;
}

template <class T_Stream,class T, int n>
 T_Stream& operator<<(T_Stream& out, const Arrow<T,n>* v)
{ 
	out << v->pos() << element_separator << v->dir();
	return out;
}

template <class T_Stream,class T, int n>
 T_Stream& operator>>(T_Stream& in, Arrow<T,n>& v)
{
	Separator es(element_separator);
	//for (int i=0;i<n;i++) //NB!!!!! template n undefined here ?????
	//{
	//	in >> v.pt[i] >> element_separator;
	// }
	// in >> v._dir;
	Point<T,n>  pt;
	Vector<T,n> dir;
	in >> pt >> es >> dir;
	v.setPos(pt);
	v.setDir(dir);
	return in;
}

template <class T_Stream,class T, int n>
 T_Stream& operator>>(T_Stream& in, Arrow<T,n>* v)
{
	Separator es(element_separator);
	Point<T,n>  pt;
	Vector<T,n> dir;
	in >> pt >> es >> dir;
	v->setPos(pt);
	v->setDir(dir);
	return in;
}

//*********************************************************
//****************  The Line class  **********************
//*********************************************************

template <class T, int n>
class Line : public Arrow<T,n>
{
public:
	Line() : Arrow<T,n>()								{}
	Line(const Point<T,n>& p) : Arrow<T,n>(p)			{}
	Line(const Point<T,n>& p ,const Vector<T,n>& v) : Arrow<T,n>(p,v) {}
	Line(const Arrow<T,n>& a) : Arrow<T,n>(a)			{}

	Point<T,n> closestPoint(const Point<T,n>& p) const
	{
		Vector<T,n> d = p-pos();
		return pos()+(d*dir())*dir();
	}

	Vector<T,n> distanceVector(const Point<T,n>& p) const
	{
		Vector<T,n> d = p-pos();
		return d-(d*dir())*dir();
	}

	T distanceAlong(const Point<T,n>& p) const
	{
		return (p-pos())*dir();
	}

	T distanceATo(const Point<T,n>& p) const
	{
		return distanceVector(p).length();
	}
};

//*********************************************************
//****************  The Box class  ************************
//*********************************************************

template <class T, int n>
class Box
{
private:
	Point<T,n> _min;
	Point<T,n> _max;
  
public:
  Box() { _min=_max=(T) 0; }
	Box(const Point<T,n>& p)	{ _min = _max = p; }
	Box(const Box<T,n>& b)		{ _min = b._min; _max = b._max; }
	Box(const Point<T,n>& p1, const Point<T,n>& p2)	{ _min = _max = p1; insert(p2); }
	Box(const Point<T,n>& p1, const Point<T,n>& p2, const Point<T,n>& p3)
								{ _min = _max = p1; insert(p2); insert(p3); }

	void reset(const Point<T,n>& p ) { _min = _max = p; }
  void reset() { _min = _max = (T) 0; }

	void insert(const Point<T,n>&);
	void insert(const Box<T,n>&);

	Box<T,n>& operator+=(const Point<T,n>& p) { insert(p); return *this; }
	Box<T,n>  operator+(const Point<T,n>& p) { Box<T,n> a=(*this); a+=p; return a; }

	Box<T,n>& operator+=(const Box<T,n>& p) { insert(p); return *this; }
	Box<T,n>  operator+(const Box<T,n>& p) { Box<T,n> a=(*this); a+=p; return a; }

	bool surround(const Point<T,n>& p) const;
	bool surround(const Box<T,n>& p) const;   // 15.07.00 BBang  is p inside this box
	bool intersect(const Box<T,n>& p) const;

	Point<T,n> minPoint() const	{ return _min; }
	Point<T,n> maxPoint() const	{ return _max; }
	Point<T,n> centerPoint() const	{ return (_max + _min)*0.5; }
	Vector<T,n> delta() const   { return _max - _min; }

	T minValue(int i) const		{ return _min(i); }
	T maxValue(int i) const		{ return _max(i); }
	T centerValue(int i) const	{ return 0.5*(_max(i) + _min(i)); }
	T delta(int i) const	{ return _max(i) - _min(i); }

	T& p(int i, int j)		{ return i == 0 ? _min[j] : _max[j]; }


};
	 	


	// ********** IOSTREAM overloaded operators ***********

	template<class T_Stream, class T, int n>
	T_Stream &operator<<(T_Stream& os, const Box<T,n>& b)
	{
		os << b.minPoint() << element_separator << b.maxPoint();		
		return os;
	}

	template<class T_Stream, class T, int n>
    T_Stream &operator<<(T_Stream& os, const Box<T,n>* b)
	{
		os << b->minPoint() << element_separator << b->maxPoint();		
		return os;
	}

	template<class T_Stream, class T, int n>
    T_Stream &operator>>(T_Stream& is, Box<T,n>& b)
	{ 
		Separator es(element_separator);
		Point<T,n> p1,p2;
		is >> p1 >> es >> p2;
		b.reset(p1);
		b.insert(p2);
		return is; 
	}

	template<class T_Stream, class T, int n>
    T_Stream &operator>>(T_Stream& is, Box<T,n>* b)
	{ 
		Separator es(element_separator);
		Point<T,n> p1,p2;
		is >> p1 >> es >> p2;
		b->reset(p1);
		b->insert(p2);
		return is; 
	} 


//*********************************************************
//****************  The ScalarPoint class  ****************
//*********************************************************

template <class T, int n>
class ScalarPoint
{
protected:
	Point<T,n>	_pos;
	T			_value;
public:
	ScalarPoint():_pos(),_value(0) {}
	ScalarPoint(const Point<T,n>& p, T v = T(0))	{ _pos = p; _value = v;}
	
	ScalarPoint(const ScalarPoint<T,n>& s)			{ _pos = s._pos; _value = s._value; }

	void reset(const Point<T,n>& p , T v = T(0))	{ _pos = p; _value = v;}
	void reset()									{_pos=(T)0; _value=(T) 0;}
	void resetValue(T t)							{ _value = t; }
	void resetPos(const Point<T,n>& p)				{ _pos = p; }

	ScalarPoint<T,n>& operator+=(const Point<T,n>& p){ _pos += p; return *this; }
	ScalarPoint<T,n>  operator+(const Point<T,n>& p) const { ScalarPoint<T,n> a=(*this); a+=p; return a;}

	ScalarPoint<T,n>& operator+=(T p)	{ _value += p; return *this; }
	ScalarPoint<T,n>  operator+(T p) const	{ ScalarPoint<T,n> a=(*this); a+=p; return a; }

	ScalarPoint<T,n>& operator+=(const ScalarPoint<T,n>& p) 
					{ _pos += p._pos;_pos /= 2.0; _value += p._value;_value /= 2.0; return *this;}
	ScalarPoint<T,n>  operator+(const ScalarPoint<T,n>& p) const 
					{ ScalarPoint<T,n> a=(*this); a+=p; return a; }

					// Scaling !! NB! only scalar value.
	ScalarPoint<T,n>& operator*=(double d) 
					{  _value *= d; return *this;}
	ScalarPoint<T,n>  operator*(double d) const 
					{ ScalarPoint<T,n> a=(*this); a*=d; return a; }
	friend
	ScalarPoint<T,n>  operator*(double d, ScalarPoint<T,n> p) 
					{ p*=d; return p; }
	ScalarPoint<T,n>& operator/=(double d) 
					{  _value /= d; return *this;}
	ScalarPoint<T,n>  operator/(double d) const 
					{ ScalarPoint<T,n> a=(*this); a/=d; return a; }

	ScalarPoint<T,n>& operator%=(const Point<T,n>& p) 
					{  _pos %= p; return *this;}
	ScalarPoint<T,n>  operator%(const Point<T,n>& p) const 
					{ ScalarPoint<T,n> a=(*this); a%=p; return a; }

	Point<T,n>	pos()   const	{ return _pos; }
	T			value() const	{ return _value; }

	 	// ********** IOSTREAM overloaded operators ***********
public:
	template<class T_Stream, class T, int n>
	friend T_Stream &operator<<(T_Stream& os, const ScalarPoint<T,n>& s)
	{
		os << b.pos() << element_separator << b.value();		
		return os;
	}
	template<class T_Stream, class T, int n>
	friend T_Stream &operator<<(T_Stream& os, const ScalarPoint<T,n>* s)
	{
		os << b->pos() << element_separator << b->value();		
		return os;
	}
	template<class T_Stream, class T, int n>
	friend T_Stream &operator>>(T_Stream& is, ScalarPoint<T,n>& s)
	{ 
		Separator es(element_separator);
		Point<T,n> p1;
		T v;
		is >> p1 >> es >> v;
		s.reset(p1,v);
		return is; 
	}
	template<class T_Stream, class T, int n>
	friend T_Stream &operator>>(T_Stream& is, ScalarPoint<T,n>* s)
	{ 
		Separator es(element_separator);
		Point<T,n> p1;
		T v;
		is >> p1 >> es >> v;
		s->reset(p1,v);
		return is; 
	}
};


//*********************************************************
//****************  The Sphere class  *********************
//*********************************************************

template <class T, int n>
class Sphere : public ScalarPoint<T,n>
{
	bool _valid;

public:
	Sphere():ScalarPoint<T,n>()										{_valid = false;}
	Sphere(const Point<T,n>& p, T v = T(0)) : ScalarPoint<T,n>(p,v)	{_valid = true;}	
	Sphere(const ScalarPoint<T,n>& s) : ScalarPoint<T,n>(s)			{_valid = true;}
	Sphere(const Sphere<T,n>& s) : ScalarPoint<T,n>(s)				{_valid = s._valid;}

	void resetPos(const Point<T,n>& p)	{_valid = true;  ScalarPoint<T,n>::resetPos(p);}
	void resetRadius(T t)				{ resetValue(t);}
	void reset()						{_valid = false; ScalarPoint<T,n>::reset();}

	Sphere<T,n>& operator+=(const Point<T,n>& p);
	Sphere<T,n>  operator+(const Point<T,n>& p) const { Sphere<T,n> a=(*this); a+=p; return a;}



	Sphere<T,n>& operator+=(const Sphere<T,n>& p);
	Sphere<T,n>  operator+(const Sphere<T,n>& p) const { Sphere<T,n> a=(*this); a+=p; return a;}

	T	 radius() const	{ return _value; }
	bool valid()  const { return _valid; }

	bool intersect(const Sphere<T,n>& p) const;

	template<class T_Stream, class T, int n>
	friend T_Stream &operator<<(T_Stream& os, const Sphere<T,n>& s)
	{
		if(s.valid())
			os << ScalarPoint<T,n>::s;
		else
			os << ScalarPoint<T,n>(Point<T,n>(),T(-1));
		return os;
	}
	template<class T_Stream, class T, int n>
	friend T_Stream &operator<<(T_Stream& os, const Sphere<T,n>* s)
	{
		if(s.valid())
			os << ScalarPoint<T,n>::(*s);		
		else
			os << ScalarPoint<T,n>(Point<T,n>(),T(-1));
		return os;
	}
// /*
	template<class T_Stream,class T,int n>
	friend T_Stream &operator>>(T_Stream& is, Sphere<T,n>& s)
	{ 
		ScalarPoint<T,n> ss;
		is >> ss;
		if(ss.value()<0)
			s.reset();
		else
			s = Sphere<T,n>(ss);
		return is; 	
	}
//	*/
	template<class T_Stream, class T, int n>
	friend T_Stream &operator>>(T_Stream& is, Sphere<T,n>* s)
	{ 
		ScalarPoint<T,n> ss;
		is >> ss;
		if(ss.value()<0)
			(*s).reset();
		else
			(*s) = Sphere<T,n>(ss);
		return is; 
	}
};


} // end namespace GM_lib

#include "GM_Point.c"

#endif   /* GM_POINT_H_ */

