#include "GM_Point2D.h"
#include "GM_Point3D.h"


namespace GM_lib {


//****************************************************************
//***** See if this point is inside a closed polygon		******
//***** If it is inside the return value is 1.				******
//***** If it is on the edges the return value is -n,		******
//***** where n is the nuber on the actual edge.			******
//***** If it is outside the polygon the retun value is 0.	******
//****************************************************************

template <class T>
int Point2D<T>::isInside(const Array<Point<T,2> >& a) const
{
	Array<int> v;
	double r;

	for (int i=0,j=1; i<a.size(); i++,j++)
	{
		if (j == a.size())	j = 0;
		UnitVector2D<T> b = a(j) - a(i);
		r = b^((*this) - a(i));
		if (r < - POS_TOLERANCE)		return 0;
		if (fabs(r) < POS_TOLERANCE)	v += -i-1;
	}

	if (v.size() == 0)	return 1;
	else				return v[0];
}


//****************************************************************
//***** See if this point is inside a circle defined		******
//***** by tree points on the circle.						******
//***** If it is inside the circle the return value is true	******
//***** else it is false.									******
//************************************************************

template <class T>
int  Point2D<T>::isInsideCircle(const Point<T,2>& p1,const Point<T,2>& p2,
													  const Point<T,2>& p3) const
{
	T b1 = p1*p1;
	T b2 = p2*p2;
	T b3 = p3*p3;

	Point2D<T> b(b2-b1,b3-b2);
	Point2D<T> a1 = p2 - p1;
	Point2D<T> a2 = p3 - p2;
		
	Point2D<T> c = (0.5/(a1^a2))*
					Point2D<T>(Point2D<T>(a2[1],-a1[1])*b,Point2D<T>(-a2[0],a1[0])*b);
	a1 = (*this) - c;
	a2 = p1 - c;

	T r = a1*a1;
	T s = a2*a2;

	if (fabs(r-s) < POS_TOLERANCE)	return -1;
	else if (r < s)					return  1;
	else							return  0;
}


//****************************************************************
//***** This function compute the convex hull of a point	******
//***** set stored in the array a.							******
//***** The convex hull is also stored in the array a,		******
//***** and it is a counter clockwise oriented polygon.		******
//***** If the convex hull consist off all original points	******
//***** from a then false is retuned else true is returned.	******
//****************************************************************

template <class T>
bool convexHullFrom(Array<Point<T,2> >& a, const Vector2D<T>& v)
{
	bool removed = false;
	if (a.size()<4)	return removed;

	int i,j,k;
	Point2D<T> p;

	for(i=0; i<a.size(); i++) p += a[i];

	p.setTestType(3,p/a.size(),v);
	a.sort();

	for(i=0; i<a.size(); i++) 
	{
		if(i<a.size()-2)		{ j=i+1; k=i+2; }
		else if(i<a.size()-1)	{ j=i+1; k=0;   }
		else					{ j=0;   k=1;   }
		p = a[j]-a[i];
		if((p^(a[k]-a[j])) < 0)
		{
			a.removeIndex(j);
			removed = true;
			if(i==a.size()) i-=3;
			else if(i>0)	i-=2;
			else			i--;
		}
	}
	return removed;
}


}// end namespace GM_lib
