
//***********************************************************
// GM_Point2D.h: interface for the Point2D/Vector2D classes.
//***********************************************************
#ifndef GM_POINT2D_H_
#define GM_POINT2D_H_

#include <GM_Point.h>
#include <GM_Array.h>

namespace GM_lib {

//*********************************************************
//************** The Point2D/Vector2D class ***************
//*********************************************************

//#define Vector2D Point2D

template <class T>
class Point2D: public Point<T,2> 
{

 public:
	Point2D(T d=0) : Point<T,2>(d)						{}  
	Point2D(const T p[2]) : Point<T,2>(p)				{}
	Point2D(const Point<T,2>& v) : Point<T,2>(v)		{}
	Point2D(const Point<T,3>& v)		{pt[0]=v(0); pt[1]=v(1);}
	Point2D(const T& x,const T& y)		{ pt[0]=x; pt[1]=y; }  

	T operator^(const Point<T,2>& v) const		// Semi vector product.
    		{ return pt[0]*v(1)-pt[1]*v(0);}

	Point<T,2> normal()							// Return a vector 90 deg. to this.
			{ return Point2D<T>(-pt[1],pt[0]); }

	int isInside(const Array<Point<T,2> >&) const;

	int isInside(const Point<T,2>& v1,const Point<T,2>& v2,const Point<T,2>& v3) const
			{ Array<Point<T,2> > arr; arr+=v1;arr+=v2;arr+=v3; return isInside(arr);}

	int isInsideCircle(const Point<T,2>&,const Point<T,2>&,const Point<T,2>&) const;
};



//*****************************************************
//************* The Vector2D class  *******************
//*****************************************************

template <class T>
class Vector2D : public Vector<T,2>
{
public:
	Vector2D(T d=1) : Vector<T,2>(d)				{}
	Vector2D(const T p[2]) : Vector<T,2>(p)			{}
	Vector2D(const Point<T,2>& v) : Vector<T,2>(v)	{}
	Vector2D(const T& x,const T& y) : Vector<T,2>(Point2D<T>(x,y)){}


	T operator^(const Point<T,2>& v) const        // Semi vector product.
    					{ return pt[0]*v(1)-pt[1]*v(0);}

	Point<T,2> normal()							// Return a vector 90 deg. to this.
			{ return Vector2D<T>(-pt[1],pt[0]); }
};


//*********************************************************
//************* The UnitVector2D class  *******************
//*********************************************************

template <class T>
class UnitVector2D : public UnitVector<T,2>
{
public:
	UnitVector2D(T d=1) : UnitVector<T,2>(d)				{}
	UnitVector2D(const T p[2]) : UnitVector<T,2>(p)			{}
	UnitVector2D(const Point<T,2>& v) : UnitVector<T,2>(v)	{}
	UnitVector2D(const T& x,const T& y) : UnitVector<T,2>(Point2D<T>(x,y)){}


	T operator^(const Point<T,2>& v) const        // Semi vector product.
    					{ return pt[0]*v(1)-pt[1]*v(0);}

	Point<T,2> normal()							// Return a vector 90 deg. to this.
			{ return Vector2D<T>(-pt[1],pt[0]); }
};


template <class T>
bool convexHullFrom(Array<Point<T,2> >&, const Vector2D<T> v = Vector2D<T>(1,0));


} // end namespace GM_lib

#include "GM_Point2D.c"

#endif   /* GM_POINT2D_H_ */

