//***********************************************************
// GM_Point3D.h: interface for the Point3D/Vector3D classes.
//***********************************************************
#ifndef GM_POINT3D_H_
#define GM_POINT3D_H_

#include <GM_Point.h>

namespace GM_lib {

//*********************************************************
//********* The Point3D/Vector3D class ********************
//*********************************************************

//#define Vector3D Point3D

template <class T>
class Point3D: public Point<T,3> 
{

 public:
   Point3D(T d=0) : Point<T,3>(d)							{}  
   Point3D(const T p[3]) : Point<T,3>(p)					{}
   Point3D(const Point<T,3>& v) : Point<T,3>(v)				{}
   Point3D(const T& x,const T& y,const T& z)  { pt[0]=x; pt[1]=y; pt[2]=z; }  

   Point<T,3> operator^(const Point<T,3>& v)  const     // Vector product.
			{ return Point3D<T>(pt[1]*v(2)-pt[2]*v(1),
								pt[2]*v(0)-pt[0]*v(2),
								pt[0]*v(1)-pt[1]*v(0));}
};


//*****************************************************
//************** The Vector3D class *******************
//*****************************************************

template <class T>
class Vector3D : public Vector<T,3>
{
public:
	Vector3D(T d=1) : Vector<T,3>(d)				{}
	Vector3D(const T p[3]) : Vector<T,2>(p)			{}
	Vector3D(const Point<T,3>& v) : Vector<T,3>(v)	{}
	Vector3D(const T& x,const T& y,const T& z) : Vector<T,3>(Point3D<T>(x,y,z)){}

	Point<T,3> operator^(const Point<T,3>& v) const     // Vector product.
			{ return Point3D<T>(pt[1]*v(2)-pt[2]*v(1),
								pt[2]*v(0)-pt[0]*v(2),
								pt[0]*v(1)-pt[1]*v(0));}
};

//*********************************************************
//************** The UnitVector3D class *******************
//*********************************************************

template <class T>
class UnitVector3D : public UnitVector<T,3>
{
public:
	UnitVector3D(T d=1) : UnitVector<T,3>(d)				{}
	UnitVector3D(const T p[3]) : UnitVector<T,2>(p)			{}
	UnitVector3D(const Point<T,3>& v) : UnitVector<T,3>(v)	{}
	UnitVector3D(const T& x,const T& y,const T& z) : UnitVector<T,3>(Point3D<T>(x,y,z)){}

	Point<T,3> operator^(const Point<T,3>& v) const     // Vector product.
			{ return Point3D<T>(pt[1]*v(2)-pt[2]*v(1),
								pt[2]*v(0)-pt[0]*v(2),
								pt[0]*v(1)-pt[1]*v(0));}
};

//*********************************************************
//****************  The Plane class  **********************
//*********************************************************


template <class T, int n>
class Plane : public Arrow<T,3>
{
public:
	Plane() : Arrow<T,3>()								{}
	Plane(const Point<T,3>& p) : Arrow<T,3>(p)			{}
	Plane(const Point<T,3>& p ,const Vector<T,n>& v) : Arrow<T,3>(p,v) {}
	Plane(const Arrow<T,3>& a) : Arrow<T,3>(a)			{}

	void setNormal(const Vector<T,3>& v)	{ setDir(v); }

	const Vector<T,3>&	normal() const		{ return dir(); }

	Point<T,3> closestPoint(const Point<T,3>& p) const
	{
		Vector<T,3> d = p-pos();
		return pos()+d-(d*normal())*normal();
	}

	Vector<T,3> distanceVector(const Point<T,3>& p) const
	{
		Vector<T,3> d = p-pos();
		return (d*normal())*normal();
	}

	T distanceTo(const Point<T,3>& p) const
	{
		return (p-pos())*normal();
	}
};



} // end namespace GM_lib

#endif   /* GM_POINT3D_H_ */

