//////////////////////////////////////////////////////////////////////
// PointLight.cpp: implementation of the PointLight class.
//////////////////////////////////////////////////////////////////////

#include <GM_PointLight.h>
using namespace GM_lib;
#include "float.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PointLight::PointLight():Light(),LA_DisplayObject(),_pos(0.0f),_light_sphere(Point3D<GLfloat>(0,0,0),FLT_MAX)
{
	_type_id  = 3;
}


PointLight::PointLight(	const Point<GLfloat,3>& pos):Light(),_pos(0.0f)
{
	translate(pos);
	_type_id  = 3;
}

PointLight::PointLight(	const GL_Color& amb,
						const GL_Color& dif,
						const GL_Color& spe,
						const Point<GLfloat,3>& pos):Light(amb,dif,spe),LA_DisplayObject(),_pos(0.0f)
{
	translate(pos);
	_type_id  = 3;
}

PointLight::PointLight(	const PointLight& pl):Light(pl),LA_DisplayObject(pl),_pos(pl._pos)
{
	_type_id  = 3;
}

PointLight::~PointLight()
{
}


//////////////////////////////////////////////////////////////////////
// Virtuelle lightning funksjoner
//////////////////////////////////////////////////////////////////////

void PointLight::lighting()
{
	glPushMatrix();
	_present.inverse2().mult();
	glLightPos(_pos);
	glPopMatrix();
}


void PointLight::_calculateRadius(GLfloat constant, GLfloat linear, GLfloat quadratic)
{
	_attenuation[0] = constant;
	_attenuation[1] = linear;
	_attenuation[1] = quadratic;
	double aa, solution;
	if ((linear == 0) && (quadratic == 0)) 
	{ 
		cullingOff(); 
		solution = FLT_MAX;
	}
	else if (quadratic == 0)
	{
		if(constant <= _min_light_contribution)
			solution = (_min_light_contribution-constant) / linear;
		else
			solution = 0;
	}
	else
	{
		if(constant <= _min_light_contribution)
			aa= sqrt(linear*linear + 4 * (_min_light_contribution-constant) * quadratic);
		else 
			aa = linear;
		solution = (aa -linear)/(2 * quadratic);
	}
	_light_sphere.resetValue(solution);
}

//////////////////////////////////////////////////////////////////////
// Public funksjoner
//////////////////////////////////////////////////////////////////////

void PointLight::setAttenuation(GLfloat constant, GLfloat linear, GLfloat quadratic)
{
	_calculateRadius(constant, linear, quadratic);

	glLight(GL_CONSTANT_ATTENUATION,constant);
	glLight(GL_LINEAR_ATTENUATION,linear);
	glLight(GL_QUADRATIC_ATTENUATION,quadratic);
}

void PointLight::setAttenuation(const GLfloat att[])
{
	setAttenuation(att[0],att[1],att[2]);
}

void PointLight::culling(const Frustum& frustum)
{
	if (!cullable()) {on(); return;}
	int k = frustum.isInterfering(_light_sphere);
	if (k < 0) off();
	else on();
}

