#pragma once

#include <qgl.h>
#include <qmessagebox.h>

#include <GM_Window.h>
using namespace GM_lib;

class GM_QtWindow : public QGLWidget, public GM_Window
{
	Q_OBJECT
signals:	
	virtual void	TextOut(const QString&);
	void			isActive();	//Telling Qt that this GM_QtWindow is active. In use with several windows 
	void			dblClicked();
	virtual void	FrameRate(int);
	virtual void	ActivObject(LA_DisplayObject *);
	
public slots:
	void	reDraw() 				{ updateGL(); }
	void	setMouseActive(bool& m)	{ _mouseactive = m; }

private:
	void	_swapBuffers();
	void	_screenDump();
	bool	_toggleRun();

	bool	_mouseactive;
	int		_timerID;

	Tid		frametimer;
	int		framecounter;
	double	framerate;

protected:
	/*** QT functions ******************/
	void	initializeGL();
	void	paintGL();
	void	glDraw();
	void	resizeGL(int w, int h);
//	bool	eventFilter( QObject *obj, QEvent *ev );
	void	mouseMoveEvent(QMouseEvent *e); 
	void	mouseReleaseEvent(QMouseEvent *e);
	void	mousePressEvent(QMouseEvent *e);
	void	timerEvent(QTimerEvent *qte);
	void	keyPressEvent( QKeyEvent *k );
	void	keyReleaseEvent( QKeyEvent *k );
	void	mouseDoubleClickEvent ( QMouseEvent * e );
	
	void	_message(const std::string& str) { TextOut( (QString)str.c_str() ); }
	void	_message(LA_DisplayObject* d) { ActivObject(d); }
	void	_message(DisplayObject* d) { ActivObject((LA_DisplayObject*)d); }
	void    _message(double& f) { QString str; str.number(f); TextOut(str); }
	void	_message(Point<float,3>& p) {	QString str;
											QTextStream ts( &str, IO_WriteOnly ); element_separator=' '; 
											ts.precision(3);
											ts << p << "\n"; TextOut(str); }
	void _keyDef_b(int x, int y);
	void _keyDef_B(int x, int y);
	void _keyDef_c(int x, int y);


public:
	GM_QtWindow( QWidget* parent, const char* name );
   ~GM_QtWindow(void);

	void clear();

	GM_Window& operator=(const GM_Window& gw){ return GM_Window::operator =(gw); } 
};

