#include <QString>
#include <qimage>
#include <qfiledialog>
#include <GM_QtWindow.h>


//***************************************************************
//*******        Constructors and Destructor       **************
//***************************************************************
void GM_QtWindow::_swapBuffers()
{
	swapBuffers();
}

GM_QtWindow::GM_QtWindow(QWidget* parent, const char* name )
: QGLWidget( parent )
{
	setFocusPolicy(Qt::StrongFocus); //need for keyPressEvent

	_w = parent->width();
	_h = parent->height();
	_mouseactive = true;
	framecounter=0;
	framerate=50;
	frametimer.set();
}

GM_QtWindow::~GM_QtWindow(void)
{
}

//***************************************************************
//***********    Overloaded Qt functions    *********************
//***************************************************************
void GM_QtWindow::initializeGL()
{
	_init();
	qglClearColor(QColor(127,127,127));
	TextOut("GL initialized!");
}

void GM_QtWindow::glDraw()
{
	 _display();
	 if((++framecounter)==50) 
	 { 
		 framerate=50.0/frametimer.secAndSet(); 
		 FrameRate((int)(framerate+0.5)); 
		 framecounter=0; 
	 }
}

void GM_QtWindow::paintGL()
{
	 glDraw();
}

void GM_QtWindow::resizeGL(int w, int h)
{
	paintGL();
	_reshape(w,h);
	paintGL();
}


void GM_QtWindow::mouseDoubleClickEvent(QMouseEvent *e)
{
	paintGL();
	emit isActive();
   
	if(e->button() & LeftButton) 
	{
	    if(e->button() & AltModifier) emit dblClicked();
		else _mouseDoubleClick(GM_LEFT_BUTTON,e->x(),e->y());
	}
	else if(e->button() & MidButton)	_mouseDoubleClick(GM_MIDDLE_BUTTON,e->x(),e->y());
	else if(e->button() & RightButton)	_mouseDoubleClick(GM_RIGHT_BUTTON,e->x(),e->y());
	paintGL();
}

void GM_QtWindow::mousePressEvent(QMouseEvent *e)
{
	paintGL();
	emit isActive();
	if(e->button() & LeftButton)		_mouseButtonDown(GM_LEFT_BUTTON,e->x(),e->y());
	else if(e->button() & MidButton)	_mouseButtonDown(GM_MIDDLE_BUTTON,e->x(),e->y());
	else if(e->button() & RightButton)	_mouseButtonDown(GM_RIGHT_BUTTON,e->x(),e->y());
	paintGL();
}

void GM_QtWindow::mouseReleaseEvent(QMouseEvent *e)
{
	paintGL();
	if(e->button() & LeftButton)		_mouseButtonUp(GM_LEFT_BUTTON,e->x(),e->y());
	else if(e->button() & MidButton)	_mouseButtonUp(GM_MIDDLE_BUTTON,e->x(),e->y());
	else if(e->button() & RightButton)	_mouseButtonUp(GM_RIGHT_BUTTON,e->x(),e->y());
	paintGL();
}


void GM_QtWindow::mouseMoveEvent(QMouseEvent *e)
{
	paintGL();
	_mouseMotion(e->x(),e->y());
	paintGL();
}


void GM_QtWindow::keyPressEvent( QKeyEvent *k )
{
	paintGL();
	QString tmp = k->text();
	unsigned char key = ((tmp.size()>0)? (tmp.at(0).toAscii()):(0));

	if(key !=0)
	{
		_keyboard(key,0,0);
	}
	else
	{
		int speckey = k->key();
		
		switch(speckey)
		{
		case Key_A:
			_keyboard('a',0,0);
			break;
		case Key_Up:
			_specKeyboard(GM_KEY_UP,0,0);
			break;
		case Key_Down:
			_specKeyboard(GM_KEY_DOWN,0,0);
			break;
		case Key_Left:
			_specKeyboard(GM_KEY_LEFT,0,0);
			break;
		case Key_Right:
			_specKeyboard(GM_KEY_RIGHT,0,0);
			break;
		case Key_PageUp:
			_specKeyboard(GM_KEY_PAGE_UP,0,0);
			break;
		case Key_PageDown:
			_specKeyboard(GM_KEY_PAGE_DOWN,0,0);
			break;
		case Key_Home:
			_specKeyboard(GM_KEY_HOME,0,0);
			break;
		case Key_End:
			_specKeyboard(GM_KEY_END,0,0);
			break;
		case Key_Insert:
			_specKeyboard(GM_KEY_INSERT,0,0);
			break;

		case Key_Shift:
			_setModKey(GM_SHIFT_BUTTON);	_message("Shift-button down");
			break;
		case Key_Control:
			_setModKey(GM_CONTROL_BUTTON);	_message("Ctrl-button down");
			break;
		case Key_Alt:
			_setModKey(GM_ALT_BUTTON);		_message("Alt-button down");
			break;
		default:
			break;		
		}//end switch
	}	
	paintGL();
}

void GM_QtWindow::keyReleaseEvent( QKeyEvent *k )
{
	paintGL();
	QByteArray tmp=k->text().toAscii();
	unsigned char kk = tmp.toUShort();
	if(kk==0)
	{
		switch(k->key())
		{
		case Key_Shift:
			_setModKey(GM_NO_BUTTON);_message("Shift-button Up");
			break;
		case Key_Control:
			_setModKey(GM_NO_BUTTON);_message("Ctrl-button Up");
			break;
		case Key_Alt:
			_setModKey(GM_NO_BUTTON);_message("Alt-button Up");
			break;
		default:
			break;
		}//end switch
	}
	paintGL();
}

void GM_QtWindow::timerEvent(QTimerEvent *qte)
{
	glDraw();
}


bool GM_QtWindow::_toggleRun()
{	
	if(GM_Window::_toggleRun())
       _timerID = startTimer(0);	// 0 gives idle(), otherwise in millisec.
	else
		killTimer(_timerID);
	return true;
}

void GM_QtWindow::_screenDump()
{ 
	QImage mypic;
	mypic=grabFrameBuffer(true);
//	QString picname = QFileDialog::getSaveFileName(".","Images (*.bmp)",this,"save file dialog"
//		"Choose a filename to save under" );
	QString picname = QFileDialog::getSaveFileName(
		this,
		"Choose a filename to save under",
		".",
		"Images (*.bmp)");
	picname.append(".bmp");
	if (mypic.save(picname,"BMP",-1) ) TextOut(picname);
	else TextOut("Save failed"); 
}

void GM_QtWindow::clear()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}
