
//*************************************************************************************
// GM_RoseCurve.h: interface for the HermiteCurve classes.
//*************************************************************************************
#ifndef GM_ROSECURVE_H_
#define GM_ROSECURVE_H_

#include "GM_PCurve.h"

namespace GM_lib{

template <class T, int n>
class RoseCurve: public PCurve<T,n>
{
	void init();

 protected:

	void _eval(T t, int d);

	T _st_p() { return T(0); }
	T _end_p(){ return T(8*M_PI); }

 public:
	RoseCurve():PCurve<T,n>() { init();}
	RoseCurve(const RoseCurve<T,n>& c):PCurve<T,n>(c) { _p1=c._p1; _p2=c._p2;}

	virtual
	~RoseCurve() {}

	PCurve<T,n>* split(T t) { return NULL;}

	const char* identity() const { return "ROSE-CURVE";}
};


template <class T, int n>
void RoseCurve<T,n>::init()
{
	setMaterial(Bronze);
	resetPlot(0.0005);
}


template <class T, int n>
void RoseCurve<T,n>::_eval(T t, int d)
{
	__p.setDim(d+1);

	__p[0][0] = cos(1.75*t)*cos(t);
	__p[0][1] = sin(t)*cos(1.75*t);
	__p[0][2] = 0.0;

	if(d > 0)
	{
		__p[1][0] =-1.75*sin(1.75*t)*cos(t)-sin(t)*cos(1.75*t);
		__p[1][1] =-1.75*sin(t)*sin(1.75*t)+cos(1.75*t)*cos(t);
		__p[1][2] =0.0;
	}
	if(d > 1)
	{
		__p[2][0]=3.5*sin(t)*sin(1.75*t)-4.0625*cos(1.75*t)*cos(t);
		__p[2][1]=-3.5*sin(1.75*t)*cos(t)-4.0625*sin(t)*cos(1.75*t);
		__p[2][2]=0.0;
	}
}

} // end namespace GM_lib


#endif   /* GM_ROSECURVE_H_ */