
//**********************************************************
// GM_Matrix.h: interface for the Static matrices classes.
//**********************************************************

#ifndef GM_MATRIX_H_
#define GM_MATRIX_H_

#include <GM_Point.h>

namespace GM_lib {

//*********************************************************
//****************  The Matrix classes  ******************
//*********************************************************

template <class T, int m, int n>
class Matrix
{
protected:
	Point<T,n> pt[m];


	Point<T,m>  col(int j)const {Point<T,m> p; for(int i=0;i<n;i++) p[i]=pt[i](j); return p;}
	const Point<T,n>& row(int i)const { return pt[i];}
	Point<T,n>  dia(int i)const {Point<T,n> p; int j= m<n?m:n; for(int i=0;i<j;i++) p[i]=pt[i](i); return p;}

	void cpy()			{ for(int i=0;i<m;i++)for(int j=0;j<n;j++) pt[i][j] = i==j?T(1):T(0);}
	void cpy(T t)		{ for(int i=0;i<m;i++)for(int j=0;j<n;j++) pt[i][j] = t;}
	void cpy(const Matrix<T,m,n>& v)		    { for(int i=0;i<m;i++) pt[i] = v.pt[i];}
	void cpy_r(const Point<T,n>& p,int i)   {pt[i] = p;}
	void cpy_c(const Point<T,m>& p,int j)   { for(int i=0;i<m;i++) pt[i][j] = p(i);}
	void cpy_d(const Point<T,n>& p)		{ int j= m<n?m:n; for(int i=0;i<j;i++) pt[i][i] = p(i);}
	void swap(int i,int j)				{ T tmp = pt[i][j]; pt[i][j] = pt[j][i]; pt[j][i] = tmp;}
	void swap(int i,int j,int a,int b)	{ T tmp = pt[i][j]; pt[i][j] = pt[a][b]; pt[a][b] = tmp;}

 public:
	Matrix(bool i=false)							{ if(i) cpy();}
	Matrix(T t)										{ cpy(t);}
	Matrix(const Point<T,m>& u,const Point<T,n>& v)	{ for(int i=0;i<m;i++) pt[i] = u(i)*v;}
	Matrix(const Matrix<T,m,n>& v)					{ cpy(v);}


	Matrix<T,m,n>& operator=(const Matrix<T,m,n>& v) { cpy(v); return(*this);}

	Matrix<T,n,m>& transpose() { Matrix<T,n,m> x; for(int i=0;i<m;i++) x.cpy_K(pt[i],i); return x;}

			//***** Get element. ******

	Point<T,n>&			operator[](int i)		{ return pt[i];}  //return given element.   
	const Point<T,n>&	operator()(int i) const	{ return pt[i];}  //return copy.

	void setCol(const Point<T,m>& c, int i)	{ cpy_c(c,i);}
	void setRow(const Point<T,n>& r, int j)	{ cpy_r(r,j);}
	void setDia(const Point<T,n>& r)		{ cpy_d(r);}
 
	const Point<T,m>  getCol(int i) const { return col(i);}
	const Point<T,n>& getRow(int j) const { return row(j);}
	const Point<T,n>  getDia(int i) const { return dia(i);}

                               
			//***** Additions and substractions. ******

	Matrix<T,m,n>& operator+=(const Matrix<T,m,n>& p)
      { for(int i=0;i<n;i++) pt[i] += p.pt[i]; return *this; }

	Matrix<T,m,n>& operator-=(const Matrix<T,m,n>& p)
      { for(int i=0;i<n;i++) pt[i] -= p.pt[i]; return *this; }

	Matrix<T,m,n> operator-() const
      { Matrix<T,m,n> p; for(int i=0;i<n;i++) p[i] = -pt[i]; return p;}

	Matrix<T,m,n> operator+(const Matrix<T,m,n>& p) const
      { Matrix<T,n,m> r = (*this); return r += p;}

	Matrix<T,m,n> operator-(const Matrix<T,m,n>& p) const
      { Matrix<T,m,n> r = (*this); return r -= p;}


			// *******  product. ******

	Point<T,m>   operator*(const Point<T,n>& v) const 
    		{Point<T,m> r; for(int i=0;i<m;i++)  r[i]=pt[i]*v; return r;}

			//***** Scaling. *****

	Matrix<T,m,n>& operator*=(const double d)
      { for(int i=0;i<n;i++) pt[i] *= T(d); return *this; }

	Matrix<T,m,n> operator*(const double d) const
      { Matrix<T,m,n> p=*this; p*=d; return p;}

	friend Matrix<T,m,n> operator*(const double d, Matrix<T,m,n> p)
      { p*=d; return p;}


  		   //***** Invers scaling. *****

	Matrix<T,m,n>&    operator/=(double d)
      { d = 1/d; (*this) *= d; return *this; }

	Matrix<T,m,n> operator/(double d) const
      { d = 1/d; return (*this)*d;}



	 	// ********** IOSTREAM overloaded operators ***********

#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
private:
	ostream& _prOut(ostream& out)const
					{for(int i=0;i<m;i++) out << pt[i] << "\n";return out; }
	istream& _prIn(istream& in)
					{for(int i=0;i<m;i++) in >> pt[i];return in;}
public:
  friend
	ostream& operator<<(ostream& out, const Matrix<T,m,n>& v)
	   { return v._prOut(out); }
  friend
	ostream& operator<<(ostream& out, const Matrix<T,m,n>* v)
	   { return v->_prOut(out); }
  friend
	istream& operator>>(istream& in, Matrix<T,m,n>& v)
	   { return v._prIn(in); }
 friend
	istream& operator>>(istream& in, Matrix<T,m,n>* v)
	   { return v->_prIn(in); }

	void print(char prompt[]="Matrix<T,m,n>", ostream & out = cout) const
	   { out << prompt << ": " << (*this) << "\n"; }
#endif  
   
};


template <class T, int n, int m, int k>
Matrix<T,m,k>  operator*(const Matrix<T,m,n>& a, const Matrix<T,n,k>& b)             
{
	int i,j;
	Matrix<T,m,k> r; 
	for(i=0;i<m;i++)
		for(j=0;j<k;j++) 
			r[i][j]=a.getRow(i)*b.getCol(j); 
	return r;
}


//*********************************************************
//****************  The SqMatrix classes  *****************
//*********************************************************


template <class T, int n>
class SqMatrix: public Matrix<T,n,n>
{
 protected:
	void makeOrtho(const Vector<T,n>& u, const Vector<T,n>& v, UnitVector<T,n>* x);
 public:
	SqMatrix(bool i=false): Matrix<T,n,n>(i)		{}
	SqMatrix(T t): Matrix<T,n,n>(t)					{}
	SqMatrix(const Point<T,n>& u, const Point<T,n>& v): Matrix<T,n,n>(u,v){}
	SqMatrix(const Matrix<T,n,n>& v): Matrix<T,n,n>(v)	{}
	SqMatrix(Angle a, const Vector<T,n>& u, const Vector<T,n>& v) { rotation(a,v);}
	SqMatrix(Angle a, int x = 0, int y=1);

	SqMatrix<T,n>& transposed() { for(int i=0;i<n;i++)for(int j=i+1;j<n;j++) swap(i,j); return *this;}

	void rotateXY(Angle a, int x=0, int y=1);
	void rotation(Angle a, const Vector<T,n>& u, const Vector<T,n>& v);
		
	void basisChange(const Vector<T,n>* x)
	{ SqMatrix<T,n> m; for(int i=0; i<n; i++) m.setCol(x[i], i); (*this) = m*(*this); }

	void basisChangeInv(const Vector<T,n>* x)
	{ SqMatrix<T,n> m; for(int i=0; i<n; i++) m.setRow(x[i], i); (*this) = m*(*this); }


 };

//*********************************************************
//****************  The HqMatrix classes  ****************
//*********************************************************


template <class T, int n>
class HqMatrix: public SqMatrix<T,n>
{
	Point<T,n>	pt[2];
	T			w;

 public:
	HqMatrix(bool i=false):SqMatrix<T,n>(i)			{if(i){pt[0]=pt[1]=T(0); w=T(1);}}
	HqMatrix(T t):SqMatrix<T,n>(t)						{pt[0]=pt[1]=w=t;}
	HqMatrix(const Point<T,n>& u,const Point<T,n>& v):SqMatrix<T,n>(u,v){}
	HqMatrix(const HqMatrix<T,n>& v):SqMatrix<T,n>(v)	{pt[0]=v.pt[0];pt[1]=v.pt[1];w=v.w; }
	HqMatrix(Angle a, const Vector<T,n>& v):SqMatrix<T,n>(a,v) {pt[0]=pt[1]=T(0); w=T(1);}

	HqMatrix<T,n>& transposed() { Point<T,n> p=pt[0]; pt[0]=pt[1]; pt[1]=p; Matrix::transposed();}

	void rotation(Angle a, const Vector<T,n> u, const Vector<T,n> v, const Point<T,n> p);
 };


//********************************************************
//********* -   Make a rotation matrix   - ***************
//********************************************************

template <class T, int n>
SqMatrix<T,n>::SqMatrix(Angle a, int x, int y)
{
	T sina = T(sin(a)), cosa = T(cos(a));
	(*this)[x][x]=cosa; (*this)[y][x]=-sina; (*this)[x][y]=sina; (*this)[y][y]=cosa; 
}

template <class T, int n>
void SqMatrix<T,n>::rotateXY(Angle a, int x, int y)
{
	T sina = T(sin(a)), cosa = T(cos(a));
	SqMatrix<T,n> m(true); 
	m[x][x]=cosa; m[y][x]=-sina; m[x][y]=sina; m[y][y]=cosa; 
	(*this) = m*(*this); 
}


template <class T, int n>
void SqMatrix<T,n>::rotation(Angle a, const Vector<T,n>& u, const Vector<T,n>& v)
{
	UnitVector<T,n> x[n];
	makeOrtho(u,v,x);
	basisChangeInv(x);
	rotateXY(a);
	basisChange(x);
}


template <class T, int n>
void SqMatrix<T,n>::makeOrtho(const Vector<T,n>& u, const Vector<T,n>& v, UnitVector<T,n>& x)
{
	int i,j,k=2;
	x[0] = u;
	x[1] = v - (v*x[0])*x[0];
	for(i=2;i<n;i++)
	{
		do{
			Vector<T,n> dum(T(0));
			dum[i-k]=T(1);
			for(j=0; j<i; j++) dum -= (dum*x[j])*x[j];
			if(dum.length()> 0.1)
			{	
				x[i] = dum;
				break;
			}
			k--;
		}while(true);
	}
}


} // end namespace GM_lib

#endif /*GM_MATRIX_H_*/
