
//////////////////////////////////////////////////////////////////////
// Scene.cpp: implementation of the Scene class.
//////////////////////////////////////////////////////////////////////
#include <GM_Scene.h>
#include <GM_DisplayObject.h>
#include <GM_Light.h>
using namespace GM_lib;


Scene::Scene(DisplayObject*  d): _scene(),_obj(128),_mat(32)
{
	_scene += d;
	time_scale = 1;
	time_logg  = 0;
	is_stopped = false;
	_mat += GL_Matrix();
}


Scene::Scene(): _scene(),_obj(128),_mat(32)
{
	time_scale = 1;
	time_logg  = 0;
	is_stopped = false;
	_mat += GL_Matrix();
}


Scene::Scene(const Scene&  d): _scene(d._scene),_obj(d._obj),_mat(d._mat)
{
	time_scale = 1;
	time_logg  = 0;
	is_stopped = false;
}


Scene::~Scene()
{
	for(int i=0; i<_scene.size();i++) delete _scene[i];
}


void Scene::prepare()
{
	int nr=0;
	_light.resetSize();
	for(int i=0; i < _scene.size(); i++)
		nr += _scene[i]->prepare(_light,_mat,this);
	if(_obj.maxSize() < nr) _obj.setMax(nr);
}

void Scene::lighting()
{
	for(int i=0; i < _light.size(); i++)
		_light[i]->lighting();
}

void Scene::display()
{
	for(int i=0; i < _obj.size(); i++)
		_obj[i]->display();
}

void Scene::select(int type_id)
{
	for(int i=0; i < _obj.size(); i++)
		_obj[i]->select(type_id);
}

void Scene::culling(const Frustum& f)
{
	int i;
	_obj.resetSize();
	for(i=0; i< _scene.size(); i++)
		_scene[i]->culling(_obj,f);
	// Culling of light is now possible.
}

void Scene::simulate()
{
	if(is_stopped) return;

	if(time_logg==0)	prepare();

	double dt = time_scale * tiden.secAndSet();

	if(dt)
	{
		time_logg  += dt;
		for(int i=0; i< _scene.size(); i++)
			_scene[i]->simulate(dt);
	}
}

DisplayObject* Scene::find(GLuint name)
{
	DisplayObject* d;

	for(int i=0; i < _scene.size(); i++)
		if(d = _scene[i]->find(name)) return d;
	return 0;
}

void Scene::insert(DisplayObject* obj)
{
	if(obj)  _scene.insert(obj);
}


void Scene::remove(DisplayObject* obj)
{
	if(obj)  _scene.remove(obj);
}


DisplayObject* Scene::operator[](int i)
{
	return _scene[i];
}

int Scene::size()
{
	return _scene.size();
}

DisplayObject* Scene::operator()(int i)
{
	return _obj[i];
}

int Scene::totalSize()
{
	return _obj.size();
}


Sphere<GLfloat,3> Scene::sphere(void)
{
	Sphere<GLfloat,3> sp;
	for(int i=0; i< _scene.size(); i++)
		sp += _scene[i]->surroundingSphere();
	return sp;
}

Sphere<GLfloat,3>	Scene::cleanSphere() const
{
	Sphere<GLfloat,3> sp;
	for(int i=0; i< _scene.size(); i++)
		sp += _scene(i)->cleanSurroundingSphere();
	return sp;
}

Scene& Scene::operator=(const Scene& sc)
{
	is_stopped	= true;
	time_logg	= sc.time_logg;
	time_scale	= sc.time_scale;

	_scene		= sc._scene;

	return *this;
}
	
bool Scene::visible(DisplayObject* obj)
{
	return _obj.exist(obj);
}

