
//********************************************************
// Scene.h: interface for the Scene class.
//********************************************************
#ifndef _SCENE_H__
#define _SCENE_H__

#include <GM_Tid.h>
#include <GM_Array.h>
#include <GM_OpenGL.h>

namespace GM_lib {

class DisplayObject;
class Camera;
class Light;

//*********************************************************************
//	Inneholder: Scenen er startnoden i ojecttreet
//
//		privat;
//				En array av pekere til frstenode objektene i ojecttreet
//		public:
//				Konstuktr med mulige default verdier og sletter
//				En display funksjon for displaying av objektet.
//				En select funksjon for plukking(select) av objektet med mus.
//				En simulate funksjon for at objektet kan bevege seg eller subobjekter.
//				En find funksjon for  finne en funksjon med et gitt navn.
//				En insert funksjon for  legge inn et nytt frstenode objekt
//				En remove funksjon for  fjerne et frstenode objekt
//				En operator [] som returnerer et gitt fstenode-element.
//				En funksjon size() som gir antallet fstenode-element.
//				En funksjon getMatrix(obj,mat,type) som gir transformasjonsmatrisen 
//				   fra scenen til og med obj hvis type er 1 og til obj hvis type er 2.
//*********************************************************************


class Scene  
{
	Array<DisplayObject*>	_scene;

	Array<DisplayObject*>	_obj;
	Array<Light*>			_light;
	Array<GL_Matrix>		_mat;

	Tid			tiden;
	double		time_logg;
	double		time_scale;
	bool		is_stopped;

	friend class Camera;			// four functions only used by a Camera
	void lighting();
	void display();
	void select(int type_id);
	virtual void culling(const Frustum& f);

protected:
	void simulate();				// Brukes bare av GM_Window	

	bool visible(DisplayObject* obj);

public:
	Scene();						// Default constructor
	Scene(DisplayObject* obj);		// Standar constructor
	Scene(const Scene&  d);			// Copy constructor

	virtual ~Scene();

	void prepare();
	void insert(DisplayObject* obj);
	void remove(DisplayObject* obj);

	DisplayObject* find(GLuint name);

	DisplayObject* operator[](int i);
	int size();
	DisplayObject* operator()(int i);
	int totalSize();

	void   setTimeScale(double s)	{ time_scale = s;}
	double getTimeScale()			{ return time_scale;}
	bool   isStopped()				{ return is_stopped;}
	void   stopTime()				{ is_stopped = true;}
	void   startTime()				{ is_stopped = false; tiden.set();}
	double timeLogg()				{ return time_logg; }
	void   resetTimeLogg()			{ time_logg = 0; }

	Sphere<GLfloat,3>	sphere(void);
	Sphere<GLfloat,3>	cleanSphere() const;

	Scene& operator=(const Scene& sc);
};

} // end namespace GM_lib

#endif // _SCENE_H__
