
#ifndef _GM_SEASHELL_H_
#define _GM_SEASHELL_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class GM_Seashell : public PSurf<T,n>
{
	void _eval(T u, T v, int d1, int d2);

protected:
	T _st_p_u()			{ return T(0.0); }
	T _end_p_u()			{ return T(2*M_PI); }
	T _st_p_v()			{ return T(M_PI*0.25); }
	T _end_p_v()			{ return T(5*M_PI); }

public:
	GM_Seashell();
	GM_Seashell(const GM_Seashell<T,n>& s);

	bool  closedU()  { return true; }

	const char* identity() const { return " GM_Seashell ";}
};



template <class T, int n>
inline
GM_Seashell<T,n>::GM_Seashell():PSurf<T,n>()
{
	setMaterial(Ruby);
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),10.0));
	resetPlot(32,32);
}

template <class T, int n>
inline
GM_Seashell<T,n>::GM_Seashell(const GM_Seashell<T,n>& s):PSurf<T,n>(s)
{
}

template <class T, int n>
void GM_Seashell<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(d1+1,d2+1);

	T a = 0.51449576;
	T b = 0.085749293;
	T v1 = 0.1*v;
	T su = sin(u);
	T cu = cos(u);
	T sv = sin(v);
	T cv = cos(v);
	T cucv = cu*cv;
	T cusv = cu*sv;
	T sucv = su*cv;
	T susv = su*sv;

						//S
	__p[0][0][0] =	    cv+v1*(cucv+a*susv);
	__p[0][0][1] =      sv+v1*(cusv-a*sucv);
	__p[0][0][2] =      (b*su+0.6)*v;
	if(d1)				//Su
	{
		__p[1][0][0] =  v1*(-sucv+a*cusv);
		__p[1][0][1] =  v1*(-susv-a*cucv);
		__p[1][0][2] =  b*v*cu;
	}
	if(d2)				//Sv
	{
		__p[0][1][0] =  v1*(a*sucv-cusv)+0.1*(cucv+a*susv)-sv;
		__p[0][1][1] =  v1*(a*susv+cucv)+0.1*(cusv-a*sucv)+cv;
		__p[0][1][2] =  b*su+0.6;
	}
	if(d1>1)			//Suu
	{
		__p[2][0][0] =  v1*(-cucv-a*susv);
		__p[2][0][1] =  v1*(-cusv+a*sucv);
		__p[2][0][2] = -b*v*su;
	}
	if(d1 && d2)		 //Suv
	{
		__p[1][1][0] =  v1*(a*cucv+susv)-0.1*(sucv+a*cusv);
		__p[1][1][1] =  v1*(a*cusv-sucv)-0.1*(susv+a*cucv);
		__p[1][1][2] =  b*cu;
	}
	if(d2>1)			//Svv
	{
		__p[0][2][0] =  0.2*(a*sucv-cusv)+v1*(-cucv-a*susv)-cv;
		__p[0][2][1] =  0.2*(a*susv+cucv)+v1*(a*sucv-cusv)-sv;
		__p[0][2][2] =  0.0;
	}
	if(d1>2)			//Suuu
	{
		__p[3][0][0] =  v1*(sucv-a*cusv);
		__p[3][0][1] =  v1*(susv+a*cucv);
		__p[3][0][2] = -b*v*cu;
	}
	if(d1>1 && d2)		//Suuv
	{
		__p[2][1][0] =  v1*(cusv-a*sucv)-0.1*(cucv-a*susv);
		__p[2][1][1] =  v1*(-cucv-a*susv)-0.1*(cusv+a*sucv);
		__p[2][1][2] = -b*su;
	}
	if(d1 && d2>1)		//Suvv
	{
		__p[1][2][0] =  0.2*(a*cucv+susv)+v1*(sucv-a*cusv);
		__p[1][2][1] =  0.2*(a*cusv-sucv)+v1*(susv+a*cucv);
		__p[1][2][2] =  0;
	}
	if(d2>2)			//Svvv
	{
		__p[0][3][0] =  0.3*(-a*susv-cucv)+v1*(cusv-a*sucv)+sv;
		__p[0][3][1] =  0.3*(a*sucv-cusv)+v1*(-a*susv-cucv)-cv;
		__p[0][3][2] =  0;
	}
	if(d1>2 && d2)		//Suuuv	
	{
		__p[3][1][0] =  v1*(-susv-a*cucv)-0.1*(-sucv-a*cusv);
		__p[3][1][1] =  v1*(sucv-a*cusv)-0.1*(-susv+a*cucv);
		__p[3][1][2] =  -b*cu;
	}
	if(d1>1 && d2>1)	//Suuvv
	{
		__p[2][2][0] =  0.2*(cusv-a*sucv)+v1*(cucv+a*susv);
		__p[2][2][1] =  0.2*(-a*susv-cucv)+v1*(-a*sucv+cusv);
		__p[2][2][2] =  0;
	}
	if(d1>2 && d2>1)	//Suuuvv
	{
		__p[3][2][0] =  0.2*(-susv-a*cucv)+v1*(-sucv+a*cusv);
		__p[3][2][1] =  0.2*(-a*cusv+sucv)+v1*(-a*cucv-susv);
		__p[3][2][2] =  0;
	}
	if(d1 && d2>2)		//Suvvv	
	{
		__p[1][3][0] =  0.3*(-a*cusv+sucv)+v1*(-susv-a*cucv);
		__p[1][3][1] =  0.3*(a*cucv+susv)+v1*(-a*cusv+sucv);
		__p[1][3][2] =  0;
	}
	if(d1>1 && d2>2)	//Suuvvv	
	{
		__p[2][3][0] =  0.3*(a*susv+cucv)+v1*(-cusv+a*sucv);
		__p[2][3][1] =  0.3*(-a*sucv+cusv)+v1*(a*susv+cucv);
		__p[2][3][2] =  0;
	}
	if(d1>2 && d2>2)	//Suuuvvv	
	{
		__p[3][3][0] =  0.3*(a*cusv-sucv)+v1*(susv+a*cucv);
		__p[3][3][1] =  0.3*(-a*cucv-susv)+v1*(a*cusv-sucv);
		__p[3][3][2] =  0;
	}

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}

} // GM_lib

 #endif //_GM_SEASHELL_H_

