#include "GM_Selector.h"
using namespace GM_lib;



template <class T, int n>
int Selector<T,n>::_display_list=0;


template <class T, int n>
void Selector<T,n>::_allOn()  // all children on
{
	for(int i=0; i<_object.size(); i++) 
	{
		Selector<T,n> *s=dynamic_cast<Selector<T,n> *>(_object[i]);
		if(s)  s->allOn(); 
	} 
}

template <class T, int n>
void Selector<T,n>::_allOff()  // all children off
{
	for(int i=0; i<_object.size(); i++) 
	{
		Selector<T,n> *s=dynamic_cast<Selector<T,n> *>(_object[i]);
		if(s)  s->allOff(); 
	} 
}

template <class T, int n>
Selector<T,n>::Selector(const Selector<T,n>& s) : LA_DisplayObject(s),_position(s._position)
{
	_type_id	= 10;
	_id			= s._id;
	_parent		= s._parent;
	_on			= s._on;
	_default	= s._default;
	_marked		= s._marked;
	_root		= s._root;
	_selected	= false;
	if(!_display_list) _makeDisplayList();
}

template <class T, int n>
Selector<T,n>::Selector(Point<T,n>& mp, int id, DisplayObject* parent, T r, const GL_Color& c, Selector<T,n>* root):_position(mp)
{ 
	Sphere<GLfloat,3> ts(Point<float,3>(float(0)),0.866); 
	setSurroundingSphere(ts); 
	_type_id	= 10;
	_id			= id;
	_parent		= parent;
	_on			= true;
	_default	= c; 
	_marked		= c.inverse();
	_selected	= false;
	_root		= root;
	translate(_position.toFloat());
	if(r != 1.0) scale(Vector3D<float>(r,r,r));
	if(!_display_list) _makeDisplayList();
}

template <class T, int n>
void Selector<T,n>::localDisplay()
{
	if(_on)
	{
		GLboolean lg;
		glGetBooleanv(GL_LIGHTING,&lg);
		if(lg) glDisable(GL_LIGHTING);
		if(_selected)	_marked.glSet(); 
		else			_default.glSet();
		// glCallList(_display_list);
		glBegin(GL_QUAD_STRIP);
		glVertex3f( 0.5,-0.5,-0.5 ); glVertex3f( 0.5, 0.5,-0.5 );
		glVertex3f( 0.5,-0.5, 0.5 ); glVertex3f( 0.5, 0.5, 0.5 );
		glVertex3f(-0.5,-0.5, 0.5 ); glVertex3f(-0.5, 0.5, 0.5 );
		glVertex3f(-0.5,-0.5,-0.5 ); glVertex3f(-0.5, 0.5,-0.5 );
		glVertex3f( 0.5,-0.5,-0.5 ); glVertex3f( 0.5, 0.5,-0.5 );
		glEnd();
		glBegin(GL_QUADS);
		glVertex3f(-0.5,-0.5,-0.5 ); glVertex3f( 0.5,-0.5,-0.5 );
		glVertex3f( 0.5,-0.5, 0.5 ); glVertex3f(-0.5,-0.5, 0.5 );
		glVertex3f(-0.5, 0.5,-0.5 ); glVertex3f(-0.5, 0.5, 0.5 );
		glVertex3f( 0.5, 0.5, 0.5 ); glVertex3f( 0.5, 0.5,-0.5 );
		glEnd();
		if(lg) glEnable(GL_LIGHTING);
	}
}

template <class T, int n>
void Selector<T,n>::localSelect()
{
	if(_on)
	{
//		GL_Color name(getName());
//		name.glSet();
		// glCallList(_display_list);	
		glBegin(GL_QUAD_STRIP);
		glVertex3f( 0.5,-0.5,-0.5 ); glVertex3f( 0.5, 0.5,-0.5 );
		glVertex3f( 0.5,-0.5, 0.5 ); glVertex3f( 0.5, 0.5, 0.5 );
		glVertex3f(-0.5,-0.5, 0.5 ); glVertex3f(-0.5, 0.5, 0.5 );
		glVertex3f(-0.5,-0.5,-0.5 ); glVertex3f(-0.5, 0.5,-0.5 );
		glVertex3f( 0.5,-0.5,-0.5 ); glVertex3f( 0.5, 0.5,-0.5 );
		glEnd();
		glBegin(GL_QUADS);
		glVertex3f(-0.5,-0.5,-0.5 ); glVertex3f( 0.5,-0.5,-0.5 );
		glVertex3f( 0.5,-0.5, 0.5 ); glVertex3f(-0.5,-0.5, 0.5 );
		glVertex3f(-0.5, 0.5,-0.5 ); glVertex3f(-0.5, 0.5, 0.5 );
		glVertex3f( 0.5, 0.5, 0.5 ); glVertex3f( 0.5, 0.5,-0.5 );
		glEnd();
	}
}

template <class T, int n>
bool Selector<T,n>::isOn()
{
	for(int i=0; i<_object.size(); i++) 
	{
		Selector<T,n> *s=dynamic_cast<Selector<T,n> *>(_object[i]);
		if(s) { if(s->isOn()) return true; } 
	} 
	return _on;
}


template <class T, int n>
void Selector<T,n>::_makeDisplayList()
{
	_display_list = glGenLists(1);
	
	glNewList(_display_list, GL_COMPILE);	// Lager displayliste for en kube 
		glBegin(GL_QUAD_STRIP);
			glVertex3f( 0.5,-0.5,-0.5 ); glVertex3f( 0.5, 0.5,-0.5 );
			glVertex3f( 0.5,-0.5, 0.5 ); glVertex3f( 0.5, 0.5, 0.5 );
			glVertex3f(-0.5,-0.5, 0.5 ); glVertex3f(-0.5, 0.5, 0.5 );
			glVertex3f(-0.5,-0.5,-0.5 ); glVertex3f(-0.5, 0.5,-0.5 );
			glVertex3f( 0.5,-0.5,-0.5 ); glVertex3f( 0.5, 0.5,-0.5 );
		glEnd();
		glBegin(GL_QUADS);
			glVertex3f(-0.5,-0.5,-0.5 ); glVertex3f( 0.5,-0.5,-0.5 );
			glVertex3f( 0.5,-0.5, 0.5 ); glVertex3f(-0.5,-0.5, 0.5 );
			glVertex3f(-0.5, 0.5,-0.5 ); glVertex3f(-0.5, 0.5, 0.5 );
			glVertex3f( 0.5, 0.5, 0.5 ); glVertex3f( 0.5, 0.5,-0.5 );
		glEnd();
	glEndList();
    
}

