//************************************************
// Selector.h: interface for the Selector classes.
//************************************************
#ifndef GM_SEL
#define GM_SEL

#include <GM_LA_DisplayObject.h>

namespace GM_lib{

template <class T, int n>
class Selector: public LA_DisplayObject
{
	static int		_display_list;

	int				_id; 
	DisplayObject*	_parent;
	Point<T,n>&		_position;

	void _makeDisplayList();
	
protected:
	bool			_on;
	Selector<T,n>*  _root;
	GL_Color		_default;
	GL_Color		_marked;

	void localDisplay();
	void localSelect();
	void _allOn();				// all children on
	void _allOff();				// all children off

 public:

	Selector(const Selector<T,n>& s);
	Selector(Point<T,n>& mp, int id, DisplayObject* parent, T r=1, const GL_Color& c=GL_Color(0.0,0.0,0.7), Selector<T,n>* root=NULL );
	virtual ~Selector() {}
	
	void	editPos(Vector<float,3> dp)	{ _position+=Point<T,n>(_present.inverse2()*dp); translate(_present.inverse2()*dp); _parent->edit(_id); }
	bool	onOff()						{ _on=!_on;   return _on; } // visibility
	void	on()						{ _on=true;   if(_root)  _root->off(); _allOff();} // 
	void	off()						{ _on=false;  _allOn(); if(_root) _root->on();  }
	void	allOff()					{ _on=false;  _allOff();} 
	void	allOn()						{ _on=true;   _allOn();} 
	bool	isOn();						// Is the children or my self on?
	char*	identity()					{ return " Selector"; }
	void	myUpdate(const Point<T,n>& p);

	// Virtual inherited from displayObject
	virtual void	edit()				{ _parent->selectEvent(_id); }
	const   char*	identity() const	{ return "Selector";}
};




template <class T, int n>
inline
void Selector<T,n>::myUpdate(const Point<T,n>& p) 
{
	Vector<T,n> d=p-_position;
	_position=p;
	translate(d.toFloat());
	for(int i=0; i<_object.size(); i++) 
	{
		Selector<T,n> *s=dynamic_cast<Selector<T,n> *>(_object[i]);
		if(s)  s->translate(-d.toFloat());  // don't bring children
	} 
}
} // end namespace GM_lib
#include "GM_Selector.c"
#endif