#ifndef _GM_SHERK_SURF_H_
#define _GM_SHERK_SURF_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Sherk_Surf : public PSurf<T,n>
{
	T _r1; //radius

	void _eval(T u, T v, int d1, int d2);
	void localDisplay();
	// void localSimulate(double dt);
protected:

	T _st_p_u()		{ return T(-M_PI); }
	T _end_p_u()		{ return T(M_PI); }
	T _st_p_v()		{ return T(-M_PI); }
	T _end_p_v()		{ return T(M_PI); }


public:

	Sherk_Surf();
	Sherk_Surf(T radius);
	// Sherk_Surf(Sherk_Surf<T,n>& s);

	virtual  
	~Sherk_Surf() {}

	void setConstants(T radius);

	const char* identity() const { return " Sherk_Surf ";}
};
} // GM_lib

using namespace GM_lib;

template <class T, int n>
Sherk_Surf<T,n>::Sherk_Surf():PSurf<T,n>()
{
	setConstants(T(1));
}

template <class T, int n>
Sherk_Surf<T,n>::Sherk_Surf(T radius):PSurf<T,n>()
{
	setConstants(radius);
}

template <class T, int n>
void Sherk_Surf<T,n>::setConstants(T radius)
{
	_r1 = radius;

	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),4.0));
}

template <class T, int n>
void Sherk_Surf<T,n>::_eval(T u, T v, int d1,int d2)
{

	__p.setDim(4,4);
	__p[0][0][0] =	u;
	__p[0][0][1] =	v;
	__p[0][0][2] =	(log(abs((T)cos(_r1*u)/(T)cos(_r1*v))))/(_r1);

	if(d1) //u
	{
		__p[1][0][0] =	T(1);
		__p[1][0][1] =	T(0);
		__p[1][0][2] =	-sin(_r1*u)/cos(_r1*u);
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	T(0);	
		__p[2][0][1] =	T(0);	
		__p[2][0][2] =	T(0);	
	}
	if(d2) //v
	{
		__p[0][1][0] =	T(0);	
		__p[0][1][1] =	T(1);	
		__p[0][1][2] =	sin(_r1*v)/cos(_r1*v);	
	}
	if(d2>1) //vv
	{
		__p[0][2][0] =	T(0);	
		__p[0][2][1] =	T(0);	
		__p[0][2][2] =	T(0);	
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =	T(0);	
		__p[1][1][1] =	T(0);	
		__p[1][1][2] =	T(0);	
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =	T(0);	
		__p[2][1][1] =	T(0);	
		__p[2][1][2] =	T(0);	
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =	T(0);	
		__p[1][2][1] =	T(0);	
		__p[1][2][2] =	T(0);	
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =	T(0);	
		__p[2][2][1] =	T(0);	
		__p[2][2][2] =	T(0);	
	}

if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];

}
template <class T, int n>
void Sherk_Surf<T,n>::localDisplay()
{
	// r_mat.mult();
	PSurf<T,n>::localDisplay();
}
 #endif //_GM_SHERK_SURF_H_
