#ifndef _GM_SIN_SURFACE_H_
#define _GM_SIN_SURFACE_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T>
class Sin_Surface : public PSurf<T,3>
{
	T _r;			// radius
	void _eval(T u, T v, int d1, int d2);

protected:
	T _st_p_u()			{ return T(0.0); }
	T _end_p_u()		{ return T(2*M_PI); }
	T _st_p_v()			{ return T(0.0); }
	T _end_p_v()		{ return T(2*M_PI); }

public:
	Sin_Surface(T radius = T(1));
	Sin_Surface(const Sin_Surface<T>& s);
 
	~Sin_Surface() {}

	const char* identity() const { return " Sin_Surface "  ;}
};

} // GM_lib

using namespace GM_lib;

template <class T>
Sin_Surface<T>::Sin_Surface(T radius):PSurf<T,3>()
{
	_r = radius;
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),_r));
}

template <class T>
Sin_Surface<T>::Sin_Surface(const Sin_Surface<T>& s):PSurf<T,3>(s)
{
	_r = s._r;
}


template <class T>
void Sin_Surface<T>::_eval(T u, T v, int d1,int d2)
{
    __p.setDim(d1+1,d2+1);

	//	S(u,v)
	__p[0][0][0] =	_r*sin(u);
	__p[0][0][1] =	_r*sin(v);
	__p[0][0][2] =	_r*sin(v+u);
	
	if(d1) //u
	{
		__p[1][0][0] =	_r*cos(u);
		__p[1][0][1] =	0.0;
		__p[1][0][2] =	_r*cos(v+u);
	}
	if(d1>1) //uu
	{
		__p[2][0][0] =	-_r*sin(u);
		__p[2][0][1] =	0.0;
		__p[2][0][2] =	-_r*sin(v+u);
	}					
	if(d2) //v			
	{			
		__p[0][1][0] =	0.0;
		__p[0][1][1] =	_r*cos(v);
		__p[0][1][2] =	_r*cos(v+u);
	}
	if(d2>1) //vv
	{		
		__p[0][2][0] =	0.0;			
		__p[0][2][1] =	-_r*sin(v);		
		__p[0][2][2] =	-_r*sin(v+u);	
	}
	if(d1 && d2) //uv
	{		
		__p[1][1][0] =	0.0;			
		__p[1][1][1] =	0.0;			
		__p[1][1][2] =	-_r*sin(v+u);			
	}
	if(d1>1 && d2) //uuv
	{
		__p[2][1][0] =	0.0;
		__p[2][1][1] =	0.0;
		__p[2][1][2] =	-_r*cos(v+u);
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =	0.0;			
		__p[1][2][1] =	0.0;			
		__p[1][2][2] =	-_r*cos(v+u);			
	}
	if(d1>1 && d2>1) //uuvv
	{		
		__p[2][2][0] =	0.0;	
		__p[2][2][1] =	0.0;			
		__p[2][2][2] =	_r*sin(v+u);			
	}								

	if(d1>0 && d2>0) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}

 #endif //_GM_SIN_SURFACE_H_

