#ifndef _GM_SLIPPERS_SURF_H_
#define _GM_SLIPPERS_SURF_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Slippers_Surf : public PSurf<T,n>
{
	void _eval(T u, T v, int d1, int d2);
	void localDisplay();
	// void localSimulate(double dt);
protected:

	T _st_p_u()		{ return T(0); }
	T _end_p_u()		{ return T(2*M_PI); }
	T _st_p_v()		{ return T(0); }
	T _end_p_v()		{ return T(2*M_PI); }


public:

	Slippers_Surf();
	// Slippers_Surf(Slippers_Surf<T,n>& s);

	~Slippers_Surf() {}

	const char* identity() const { return " Slippers_Surf ";}
};
} // GM_lib

using namespace GM_lib;

template <class T, int n>
Slippers_Surf<T,n>::Slippers_Surf():PSurf<T,n>()
{
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),4.0));
}

template <class T, int n>
void Slippers_Surf<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(4,4);
	__p[0][0][0] =  (T(2) + cos(u))*pow((T)cos(v),T(3))* sin(v);
	__p[0][0][1] =	(T(2) + cos(u+T(2)*M_PI/T(3)))*pow((T)cos(T(2)*M_PI/T(3)+v),T(2))*pow((T)sin(T(2)*M_PI/T(3)+v),T(2));
	__p[0][0][2] =	-(T(2) + cos(u-T(2)*M_PI/T(3)))*pow((T)cos(T(2)*M_PI/T(3)-v),T(2))*pow((T)sin(T(2)*M_PI/T(3)-v),T(3));

	if(d1) //u
	{
		__p[1][0][0] =	-sin(u)*pow((T)cos(v),T(3))*sin(v);
		__p[1][0][1] =	-sin(u+T(2)*M_PI/T(3))*pow((T)cos(v+T(2)*M_PI/T(3)),T(2))*pow((T)sin(v+T(2)*M_PI/T(3)),T(2));
		__p[1][0][2] =	-sin(-u+T(2)*M_PI/T(3))*pow((T)cos(T(2)*M_PI/T(3)-v),T(2))*pow((T)sin(T(2)*M_PI/T(3)-v),T(3));
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	-cos(u)*pow(cos(v),T(3.0))*sin(v);				 
		__p[2][0][1] =	cos(u+T(2.0)/T(3.0)*T(M_PI))*pow(cos(T(2.0)/T(3.0)*T(M_PI)+v),T(2.0))
						*(-T(1.0)+pow(cos(T(2.0)/T(3.0)*T(M_PI)+v),T(2.0)));				 
		__p[2][0][2] =	cos(u-T(2.0)/T(3.0)*T(M_PI))*pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(2.0))
						*(-T(1.0)+pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(2.0)))*sin(-T(2.0)/T(3.0)*T(M_PI)+v);				 
	}

	if(d2) //v	
	{
		__p[0][1][0] =	-T(3)*(T(2)+cos(u))*cos(v)*cos(v)*sin(v)*sin(v)+(T(2)+cos(u))*pow((T)cos(v),T(4));				 
		__p[0][1][1] =	-T(2)*(T(2)+cos(u+T(2)/T(3)*M_PI))*cos(v+T(2)/T(3)*M_PI)*pow((T)sin(v+T(2)/T(3)*M_PI),T(3))
						+T(2)*(T(2)+cos(u+T(2)/T(3)*M_PI))*pow((T)cos(v+T(2)/T(3)*M_PI),T(3))*sin(v+T(2)/T(3)*M_PI);				 
		__p[0][1][2] =	-T(2)*(T(2)+cos(-u+T(2)/T(3)*M_PI))*cos(T(2)/T(3)*M_PI-v)*pow((T)sin(T(2)/T(3)*M_PI-v),T(4))
						+T(3)*(T(2)+cos(-u+T(2)/T(3)*M_PI))*pow((T)cos(T(2)/T(3)*M_PI-v),T(3))*pow((T)sin(T(2)
						/T(3)*M_PI-v),T(2));				 
	}
	if(d2>1) //vv	
	{
		__p[0][2][0] =	-T(2.0)*cos(v)*sin(v)*(-6.0+16.0*pow(cos(v),T(2.0))-T(3.0)*cos(u)+8.0*pow(cos(v),T(2.0))*cos(u));				
		__p[0][2][1] =	T(4.0)-32.0*pow(cos(T(2.0)/T(3.0)*T(M_PI)+v),T(2.0))+32.0*pow(cos(T(2.0)/T(3.0)*T(M_PI)+v),T(4.0))
						+T(2.0)*cos(u+T(2.0)/T(3.0)*T(M_PI))-16.0*cos(u+T(2.0)/T(3.0)*T(M_PI))
						*pow(cos(T(2.0)/T(3.0)*T(M_PI)+v),T(2.0))+16.0*pow(cos(T(2.0)/T(3.0)*T(M_PI)+v),T(4.0))*cos(u+T(2.0)/T(3.0)*T(M_PI));				
		__p[0][2][2] =	sin(-T(2.0)/T(3.0)*T(M_PI)+v)*(T(4.0)-42.0*pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(2.0))+50.0
						*pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(4.0))+T(2.0)*cos(u-T(2.0)/T(3.0)*T(M_PI))-21.0*cos(u-T(2.0)/T(3.0)*T(M_PI))
						*pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(2.0))+25.0*pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(4.0))*cos(u-T(2.0)/T(3.0)*T(M_PI)));				
	}

	if(d1 && d2) //uv	
	{
		__p[1][1][0] =	-sin(u)*pow(cos(v),T(2.0))*(-T(3.0)+T(4.0)*pow(cos(v),T(2.0)));						
		__p[1][1][1] =	-T(2.0)*sin(u+T(2.0)/T(3.0)*T(M_PI))*cos(T(2.0)/T(3.0)*T(M_PI)+v)*sin(T(2.0)/T(3.0)*T(M_PI)+v)*(-T(1.0)+T(2.0)
						*pow(cos(T(2.0)/T(3.0)*T(M_PI)+v),T(2.0)));						
		__p[1][1][2] =	sin(u-T(2.0)/T(3.0)*T(M_PI))*cos(-T(2.0)/T(3.0)*T(M_PI)+v)*(-T(1.0)+pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(2.0)))
						*(-T(2.0)+T(5.0)*pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(2.0)));						
	}					
	if(d1>1 && d2)//uuv	
	{					
		__p[2][1][0] =	-cos(u)*pow(cos(v),T(2.0))*(-T(3.0)+T(4.0)*pow(cos(v),T(2.0)));
		__p[2][1][1] =	-T(2.0)*cos(u+T(2.0)/T(3.0)*T(M_PI))*cos(T(2.0)/T(3.0)*T(M_PI)+v)*sin(T(2.0)/T(3.0)*T(M_PI)+v)
						*(-T(1.0)+T(2.0)*pow(cos(T(2.0)/T(3.0)*T(M_PI)+v),T(2.0)));
		__p[2][1][2] =	cos(-T(2.0)/T(3.0)*T(M_PI)+v)*(-T(1.0)+pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(2.0)))*cos(u-T(2.0)/T(3.0)*T(M_PI))
						*(-T(2.0)+T(5.0)*pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(2.0)));
	}				  
	if(d1 && d2>1) //uvv		
	{
		__p[1][2][0] =	T(2.0)*sin(u)*cos(v)*sin(v)*(-T(3.0)+8.0*pow(cos(v),T(2.0)));				  
		__p[1][2][1] =	-T(2.0)*sin(u+T(2.0)/T(3.0)*T(M_PI))*(T(1.0)-8.0
						*pow(cos(T(2.0)/T(3.0)*T(M_PI)+v),T(2.0))+8.0*pow(cos(T(2.0)/T(3.0)*T(M_PI)+v),T(4.0)));				  
		__p[1][2][2] =	-sin(u-T(2.0)/T(3.0)*T(M_PI))*sin(-T(2.0)/T(3.0)*T(M_PI)+v)*(T(2.0)-21.0
						*pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(2.0))+25.0*pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(4.0)));				  
	}
	if(d1>1 && d2>1) //uuvv
	{					
		__p[2][2][0] =	T(2.0)*cos(u)*cos(v)*sin(v)*(-T(3.0)+8.0*pow(cos(v),T(2.0)));	
		__p[2][2][1] =	-T(2.0)*cos(u+T(2.0)/T(3.0)*T(M_PI))*(T(1.0)-8.0*pow(cos(T(2.0)/T(3.0)*T(M_PI)+v),T(2.0))
						+8.0*pow(cos(T(2.0)/T(3.0)*T(M_PI)+v),T(4.0)));				 
		__p[2][2][2] =	-cos(u-T(2.0)/T(3.0)*T(M_PI))*sin(-T(2.0)/T(3.0)*T(M_PI)+v)*(T(2.0)-21.0*pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(2.0))
						+25.0*pow(cos(-T(2.0)/T(3.0)*T(M_PI)+v),T(4.0)));				 
	}					

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];

}
template <class T, int n>
void Slippers_Surf<T,n>::localDisplay()
{
	// r_mat.mult();
	PSurf<T,n>::localDisplay();
}
 #endif //_GM_SLIPPERS_SURF_H_
