
//***********************************************************************
//********   GM_PSphere.h: interface for the PSphere classes.  **********
// This is a general n-dimentional Sphere with standard paramertisation *
//***********************************************************************

#ifndef _GM_SPHERE_H
#define _GM_SPHERE_H

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class PSphere: public PSurf<T,n>
{
	float _radius;
	void init();
	
	void _eval(T u, T v, int d1, int d2);

protected:
	T _st_p_u()		{ return T(0); }
	T _end_p_u()	{ return T(M_2PI); }
	T _st_p_v()		{ return -T(M_PI_2); }
	T _end_p_v()	{ return T(M_PI_2); }

 public:
	PSphere(float radius = T(1));
	PSphere(float radius, int s1, int s2);
	PSphere(const PSphere<T,n>& s);

	T radius()	{ return _radius;}

	bool  closedU()				 { return true; }
	const char* identity() const { return " PSphere";}
};

} // end namespace GM_lib



using namespace GM_lib;

template <class T, int n>
inline
PSphere<T,n>::PSphere(float radius):PSurf<T,3>()
{
	_radius = radius;
	init();
}

template <class T, int n>
inline
PSphere<T,n>::PSphere(float radius,int s1, int s2):PSurf<T,3>(s1,s2)
{
	_radius = radius;
	init();
}

template <class T, int n>
inline	
PSphere<T,n>::PSphere(const PSphere<T,n>& c):PSurf<T,3>(c) 
{ 
	_radius = c._radius;
	init();
}

template <class T, int n>
inline
void PSphere<T,n>::init()
{
	resetPlot(__sam1,__sam2);
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0,0,0),_radius));
}

template <class T, int n>
void PSphere<T,n>::_eval(T u, T v, int d1,int d2)
{
	T r=_radius;
	T cos_u = cos(u);
	T cos_v = r*cos(v);
	T sin_u = sin(u);
	T sin_v = r*sin(v);

	T cu_cv = cos_u*cos_v; 
	T cu_sv = cos_u*sin_v; 
	T su_cv = sin_u*cos_v; 
	T su_sv = sin_u*sin_v; 

	__p.setDim(d1+1,d2+1);

	__p[0][0][0] =  cu_cv;	// S
	__p[0][0][1] =  su_cv;
	__p[0][0][2] =  sin_v;

	if(d1)
	{
		__p[1][0][0] = -su_cv;	// S_u
		__p[1][0][1] =  cu_cv;
		__p[1][0][2] =  T(0);
	}
	if(d2)
	{
		__p[0][1][0] = -cu_sv;	// S_v
		__p[0][1][1] = -su_sv;
		__p[0][1][2] =  cos_v;
	}
	if(d1 && d2)
	{
		__p[1][1][0] =  su_sv;	// S_uv
		__p[1][1][1] = -cu_sv;
		__p[1][1][2] =  T(0);
	}
	if(d1>1)
	{
		__p[2][0][0] = -cu_cv;	// S_uu
		__p[2][0][1] = -su_cv;
		__p[2][0][2] =  T(0);
	}
	if(d2>1)
	{
		__p[0][2][0] = -cu_cv;	// S_vv
		__p[0][2][1] = -su_cv;
		__p[0][2][2] = -sin_v;
	}
	if(d1>1 && d2)
	{
		__p[2][1][0] =  cu_sv;	// S_uuv
		__p[2][1][1] =  su_sv;
		__p[2][1][2] =  T(0);

	}
	if(d1 && d2>1)
	{
		__p[1][2][0] =  su_cv;	// S_uvv
		__p[1][2][1] = -cu_cv;
		__p[1][2][2] =  T(0);

	}
	if(d1>1 && d2>1)
	{
		__p[2][2][0] =  cu_cv;	// S_uuvv
		__p[2][2][1] =  su_cv;
		__p[2][2][2] =  T(0);

	}
	if(d1>2)
	{
		__p[3][0][0] =  su_cv;	// S_uuu
		__p[3][0][1] = -cu_cv;
		__p[3][0][2] =  T(0);
	}
	if(d2>2)
	{
		__p[0][3][0] =  cu_sv;	// S_vvv
		__p[0][3][1] =  su_sv;
		__p[0][3][2] = -cos_v;
	}
	if(d1>2 && d2)
	{
		__p[3][1][0] = -su_sv;	// S_uuuv
		__p[3][1][1] =  cu_sv;
		__p[3][1][2] =  T(0);
	}
	if(d1>2 && d2>1)
	{
		__p[3][2][0] = -su_cv;	// S_uuuvv
		__p[3][2][1] =  cu_cv;
		__p[3][2][2] =  T(0);
	}
	if(d1 && d2>2)
	{
		__p[1][3][0] = -su_sv;	// S_uvvv
		__p[1][3][1] =  cu_sv;
		__p[1][3][2] =  T(0);
	}
	if(d1>1 && d2>2)
	{
		__p[2][3][0] = -cu_sv;	// S_uuvvv
		__p[2][3][1] = -su_sv;
		__p[2][3][2] =  T(0);
	}
	if(d1>2 && d2>2)
	{
		__p[3][3][0] =  su_sv;	// S_uuuvvv
		__p[3][3][1] = -cu_sv;
		__p[3][3][2] =  T(0);
	}

	if(d1>0 && d2>0)
	{
		if(v == -M_PI)
			__n = Vector3D<T>(-T(1),T(0),T(0));
		else if(v == M_PI)
			__n = Vector3D<T>(T(1),T(0),T(0));
		else
			__n = Vector3D<T>(__p[1][0])^__p[0][1];
	}
}

 #endif //_GM_SPHERE_H

