//////////////////////////////////////////////////////////////////////
// SpotLight.cpp: implementation of the SpotLight class.
//////////////////////////////////////////////////////////////////////

#include <GM_SpotLight.h>
using namespace GM_lib;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
SpotLight::SpotLight():PointLight()
{
	_dir = Vector3D<GLfloat>(0,0,1);

}


SpotLight::SpotLight(	const Point<GLfloat,3>& pos,
						const Vector<GLfloat,3>& dir,
						Angle cut_off) : PointLight(pos)
{
	_dir = Vector3D<GLfloat>(0,0,1);
	setCuttoff(cut_off);
	rotate(_dir.angle(dir),_dir^dir);
}

SpotLight::SpotLight(	const GL_Color& amb,
						const GL_Color& dif,
						const GL_Color& spe,
						const Point<GLfloat,3>& pos,
						const Vector<GLfloat,3>& dir,
						Angle cut_off)	:	PointLight(amb,dif,spe,pos)
{
	_dir = Vector3D<GLfloat>(0,0,1);
	setCuttoff(cut_off);
	rotate(_dir.angle(dir),_dir^dir);
}

SpotLight::SpotLight(	const SpotLight& pl):PointLight(pl)
{
	_dir = pl._dir;
	setCuttoff(pl._cutoff);
	rotate(_dir.angle(pl._dir),_dir^pl._dir);
}


SpotLight::~SpotLight()
{
}


//////////////////////////////////////////////////////////////////////
// Virtuelle lightning funksjoner
//////////////////////////////////////////////////////////////////////

void SpotLight::lighting()
{	glPushMatrix();
	_present.mult();
	glLightPos(_pos);
	glLightDir(_dir);
	glPopMatrix();
}

//////////////////////////////////////////////////////////////////////
// Public funksjoner
//////////////////////////////////////////////////////////////////////
	
void SpotLight::setCuttoff(const Angle cut_off)
{
	_cutoff = cut_off;
	glLight(GL_SPOT_CUTOFF,_cutoff.deg());
}


void SpotLight::setExponent(double exp)
{
	_exp = exp;
	glLight(GL_SPOT_EXPONENT,(GLfloat)_exp);
}

