//////////////////////////////////////////////////////////////////////
// SpotLight_G.cpp: implementation of the SpotLight_G class.
//////////////////////////////////////////////////////////////////////
#include <GM_SpotLight_G.h>
using namespace GM_lib;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
SpotLight_G::SpotLight_G():SpotLight()
{
	_material = Bronze;
	_init(_getAmbient());
}


SpotLight_G::SpotLight_G(	const Point<GLfloat,3>& pos,
							const Vector<GLfloat,3>& dir,
							Angle cut_off) : SpotLight(pos,dir,cut_off)
{
	_material = Bronze;
	_init(_getAmbient());
}

SpotLight_G::SpotLight_G(	const GL_Color& amb,
							const GL_Color& dif,
							const GL_Color& spe,
							const Point<GLfloat,3>& pos,
							const Vector<GLfloat,3>& dir,
							Angle cut_off):SpotLight(amb,dif,spe,pos,dir,cut_off)
{
	_material = Bronze;
	_init(_getAmbient());
}



SpotLight_G::SpotLight_G(	const SpotLight& pl):SpotLight(pl)
{
	_material = Bronze;
	_init(_getAmbient());
}


SpotLight_G::~SpotLight_G()
{
	glDeleteLists(_display_list,2);
}


//////////////////////////////////////////////////////////////////////
// Virtuelle display funksjoner
//////////////////////////////////////////////////////////////////////

void SpotLight_G::localDisplay()
{
	_material.glSet();
	glCallList(_display_list);
	_material.glUnSet();
}


void SpotLight_G::localSelect()
{
	glCallList(_display_list+1);
}


void SpotLight_G::_init(const GL_Color& col)
{
		// Lager boksen
	Sphere<GLfloat,3>	sb(Point3D<GLfloat>( 0.0,0.0,0.0),0.3);
	setSurroundingSphere(sb);

	int i;
	double k = 3.3;
	double b = k/sqrt(1+pow(tan(_cutoff.rad()),2));
	double a = sqrt(k*k-b*b);
	if(_cutoff.deg()>90) b = -b;

	Vector3D<GLfloat> d = _dir;
	Vector3D<GLfloat> n = d^d.getLinIndVec();
	d.setLength(b);
	n.setLength(a);
	UnitVector3D<GLfloat> n2 = -d;
	if(_cutoff.deg()!=90) n2 = Vector3D<GLfloat>((b/a)*n-(a/b)*d);
	GL_Matrix m1(Angle(30),_pos,_dir);
	GL_Matrix m2(Angle(-30),_pos,_dir);

	Array<Point<GLfloat,3> > p1;
	p1 += _pos;
	p1 += _pos+d+n;
	for(i=1; i<12; i++) p1 += m1*p1.last();

	Vector3D<GLfloat> bb=0.6*d;
	if(b<0.0) bb.setLength(0.0);
	Array<Point<GLfloat,3> > p2;
	p2 += _pos+bb+0.1*_dir;
	p2 += _pos+0.6*d+0.6*n;
	for(i=1; i<12; i++) p2 += m2*p2.last();

	Array<Vector<GLfloat,3> > v1;
	v1 += UnitVector<GLfloat,3>(-d);
	v1 += n2;
	for(i=1; i<12; i++) v1 += m1*v1.last();

	Array<Vector<GLfloat,3> > v2;
	v2 += UnitVector<GLfloat,3>(d);
	v2 += UnitVector<GLfloat,3>(d+n);
	for(i=1; i<12; i++) v2 += m2*v2.last();

	_display_list = glGenLists(2);

	glNewList(_display_list, GL_COMPILE); //Displayliste for display
		glBegin(GL_TRIANGLE_FAN);
			for(i=0; i<p1.size(); i++) { glNormal(v1[i]); glPoint(p1[i]); }
			glNormal(v1[1]); glPoint(p1[1]);
		glEnd();
		GLboolean lg;
		glGetBooleanv(GL_LIGHTING,&lg);
		if(lg) glDisable(GL_LIGHTING);
		col.glSet();
		glBegin(GL_TRIANGLE_FAN);
			for(i=0; i<p2.size(); i++) { glNormal(v2[i]); glPoint(p2[i]); }
			glNormal(v2[1]); glPoint(p2[1]);
		glEnd();
		if(lg) glEnable(GL_LIGHTING);
	glEndList();

	glNewList(_display_list+1, GL_COMPILE);	//Displayliste for select
		glBegin(GL_TRIANGLE_FAN);
			for(i=0; i<p1.size(); i++) glPoint(p1[i]);
			glPoint(p1[1]);
		glEnd();
	glEndList();
}
