
#ifndef GM_STEINERS_ROMAN_SURFACE_H_
#define GM_STEINERS_ROMAN_SURFACE_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Steiners_Roman_Surface : public PSurf<T,n>
{
	T _r;	//radius
	void _eval(T u, T v, int d1, int d2);
	void localDisplay();
	// void localSimulate(double dt);
protected:

	T _st_p_u()			{ return T(-M_PI*0.5); }
	T _end_p_u()			{ return T(M_PI*0.5); }
	T _st_p_v()			{ return T(0.0); }
	T _end_p_v()			{ return T(2*M_PI); }

public:
	Steiners_Roman_Surface();
	Steiners_Roman_Surface(T radius);
//	Steiners_Roman_Surface(Steiners_Roman_Surface<T,n>& s);

	virtual  
	~Steiners_Roman_Surface() {}
	char* identity() { return " Steiners_Roman_Surface "  ;}

	void setConstants(T radius);
};
} // GM_lib

using namespace GM_lib;
template <class T, int n>
Steiners_Roman_Surface<T,n>::Steiners_Roman_Surface():PSurf<T,n>()
{
	setConstants(T(1));
}

template <class T, int n>
Steiners_Roman_Surface<T,n>::Steiners_Roman_Surface(T radius):PSurf<T,n>()
{
	setConstants(radius);
}

template <class T, int n>
void Steiners_Roman_Surface<T,n>::setConstants(T radius)
{
	_r = radius;
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),_r));

}
template <class T, int n>
void Steiners_Roman_Surface<T,n>::_eval(T u, T v, int d1,int d2)
{

    __p.setDim(4,4);
	__p[0][0][0] =	T(.5)*_r*_r*sin(T(2)*u)*cos(v)*cos(v);
	__p[0][0][1] =	T(.5)*_r*_r*sin(u)*sin(T(2)*v);
	__p[0][0][2] =	T(.5)*_r*_r*cos(u)*sin(T(2)*v);
	
	if(d1) //u
	{
		__p[1][0][0] =	_r*_r*cos(T(2)*u)*cos(v)*cos(v);
		__p[1][0][1] =	T(.5)*_r*_r*cos(u)*sin(T(2)*v);
		__p[1][0][2] =	-T(.5)*_r*_r*sin(u)*sin(T(2)*v);
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	-T(2)*_r*_r*sin(T(2)*u)*pow(cos(v),T(2));
		__p[2][0][1] =	-_r*_r*sin(u)*sin(T(2)*v)/T(2);
		__p[2][0][2] =	-_r*_r*cos(u)*sin(T(2)*v)/T(2);
	}
	if(d2) //v		
	{
		__p[0][1][0] =	-_r*_r*sin(T(2)*u)*cos(v)*sin(v);
		__p[0][1][1] =	_r*_r*sin(u)*cos(T(2)*v);		 
		__p[0][1][2] =	_r*_r*cos(u)*cos(T(2)*v);		 
	}
	if(d2>1) //vv	
	{
		__p[0][2][0] =	_r*_r*sin(T(2)*u)*pow(sin(v),T(2))-_r*_r*sin(T(2)*u)*pow(cos(v),T(2));
		__p[0][2][1] =	-T(2)*_r*_r*sin(u)*sin(T(2)*v);
		__p[0][2][2] =	-T(2)*_r*_r*cos(u)*sin(T(2)*v);
	}
	if(d1 && d2) //uv		
	{
		__p[1][1][0] =	-T(2)*_r*_r*cos(T(2)*u)*cos(v)*sin(v);
		__p[1][1][1] =	_r*_r*cos(u)*cos(T(2)*v);			 
		__p[1][1][2] =	_r*_r*sin(u)*cos(T(2)*v);			 
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =	T(4)*_r*_r*sin(T(2)*u)*cos(v)*sin(v);
		__p[2][1][1] =	-_r*_r*sin(u)*cos(T(2)*v);
		__p[2][1][2] =	-_r*_r*cos(u)*cos(T(2)*v);
	}
	if(d1 && d2>1) //uvv		
	{
		__p[1][2][0] =	T(2)*_r*_r*cos(T(2)*u)*pow(sin(v),T(2))-T(2)*_r*_r*cos(T(2)*u)*pow(cos(v),T(2));			
		__p[1][2][1] =	-T(2)*_r*_r*cos(u)*sin(T(2)*v);			
		__p[1][2][2] =	T(2)*_r*_r*sin(u)*sin(T(2)*v);			
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =	-T(4)*_r*_r*sin(T(2)*u)*pow(sin(v),T(2))+T(4)*_r*_r*sin(T(2)*u)*pow(cos(v),T(2));	
		__p[2][2][1] =	T(2)*_r*_r*sin(u)*sin(T(2)*v);			
		__p[2][2][2] =	T(2)*_r*_r*cos(u)*sin(T(2)*v);			
	}								

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];

}
template <class T, int n>
void Steiners_Roman_Surface<T,n>::localDisplay()
{
	// r_mat.mult();
	PSurf<T,n>::localDisplay();
}
 #endif //GM_STEINERS_ROMAN_SURFACE_H_

