#include "GM_StlObject.h"
#ifdef QT
#include <qdatastream.h>
#include <qfile.h>
#endif

StlObject::~StlObject()
{
 glDeleteLists(list_name,2);
 delete [] fname;
}

StlObject::StlObject(char* filename, GL_Color c,int flip)
{
   CurrentColor=c;
   fname=new char [strlen(filename)+1];
   strcpy(fname,filename);

#ifdef QT
readSTLbinaryQT(filename, flip);
#else
readSTLbinary(filename);
#endif
   // update bounding box
   bbox.reset(vertices[0]);
   for(int i=0; i<vertices.size()-1; i++) bbox+=vertices[i];
   // make display lists
   list_name=glGenLists(2);
   glNewList(list_name,GL_COMPILE);
	glBegin(GL_TRIANGLES);
	for(i=0; i<normals.size()-1; i++)
	{	
	  glNormal(normals[i]);           // STL file only carries one normal
	  for(int j=0;j<3;j++)            // for each triangle, makes a facet shading,
	        glPoint(vertices[3*i+j]); // they must be averaged for true smooth surfaces.
	}
	glEnd();
   glEndList();

  // Build a new list for selection 
   glNewList(list_name+1,GL_COMPILE);
		glBegin(GL_TRIANGLES);
	for( i=0; i<(normals.size()-1); i++)
	{
	  glNormal(normals[i]);
	  for(int j=0;j<3;j++)
	        glPoint(vertices[3*i+j]);
	}
	glEnd();
   glEndList();

   // update bounding sphere
   Point3D<GLfloat> pos(bbox.centerPoint());          // which can be far from origo
   Sphere<GLfloat,3> s(pos,bbox.delta().length()*0.5);

   setSurroundingSphere(s);

}

StlObject::StlObject(float r)
{
    fname=new char [17];
	strcpy(fname,"STL place holder");
	kule=new PSphere<float,3>(r);   
	insert(kule);

  // Build a new list for selection  ??
  

   // update bounding sphere
   Point3D<GLfloat> pos(0,0,0);
   Sphere<GLfloat,3> s(pos,r);

   setSurroundingSphere(s);
   
}
#ifdef QT
int StlObject::readSTLbinaryQT(char* filename, int flip)
{
	QFile f( filename );
	bool ok=f.open( IO_ReadOnly );                      // open file for reading
    if (!ok) return -1;
	int filesize=f.size();	
    QDataStream s( &f ); // serialize using f
    char *buffer=new char[filesize+1];
	if(!buffer) return -1;
	char *p=buffer;
	s.readRawBytes(buffer,filesize);
    p+=80; // skip comments
	unsigned long int n_triangles;
	
	memcpy(&n_triangles,p,sizeof(unsigned long int)); p+=sizeof(unsigned long int);
	
	unsigned short int attribs;
	normals.setMax(n_triangles+1);
	vertices.setMax((n_triangles*3+1));
	UnitVector3D<float> norm;
	Point3D<float> pt;
	int stride=3*sizeof(float);
	
	for(unsigned long int t=0; t<n_triangles; t++ ) // Read all the triangles 
	{
		memcpy(norm.adr(),p,stride); p+=stride;
		normals.alwaysInsert(flip*norm);
		
		memcpy(pt.adr(),p,stride); p+=stride;
		vertices.alwaysInsert(pt);

		memcpy(pt.adr(),p,stride); p+=stride;
		vertices.alwaysInsert(pt);

		memcpy(pt.adr(),p,stride); p+=stride;
		vertices.alwaysInsert(pt);
		
		// don't need it, should alway be zero
		memcpy(&attribs,p,sizeof(unsigned short int));p+=sizeof(unsigned short int); 
	}
	delete [] buffer;
	return 1;

}

#endif

int StlObject::readSTLbinary(char* filename)
{
   	if( (stl_file = fopen( filename,"rb" )) == NULL)
		return(-1);
    unsigned long int t;
	// Skipping comment line
	for( t=0; t<80 ; t++ )
		fgetc( stl_file );

	// Get number for triangles
	unsigned long int n_triangles = getUli();
	float x,y,z;
	normals.setMax(n_triangles+1);
	vertices.setMax((n_triangles*3)+1);
	for( t=0; t<n_triangles; t++ ) // Read all the triangles 
	{
		// and put in array
		x=getFloat(); y=getFloat(); z=getFloat();
		normals.alwaysInsert(UnitVector3D<float>(x,y,z));
		

		x=getFloat(); y=getFloat(); z=getFloat();		
		vertices.alwaysInsert(Point3D<float> (x,y,z));

		x=getFloat(); y=getFloat(); z=getFloat();
		vertices.alwaysInsert(Point3D<float> (x,y,z));

		x=getFloat(); y=getFloat(); z=getFloat();
		vertices.alwaysInsert(Point3D<float> (x,y,z));
		
		// Get no. of attribute bytes (is always zero and can be ignored) 
	    getUint();
    }

	fclose( stl_file );
	return 1;
}

unsigned long StlObject::getUli()
{
	unsigned char byte1,byte2,byte3,byte4;
	unsigned long int number;
	byte1=(unsigned char)fgetc( stl_file );
	byte2=(unsigned char)fgetc( stl_file );
	byte3=(unsigned char)fgetc( stl_file );
	byte4=(unsigned char)fgetc( stl_file );
	number = (unsigned long int)byte1 + ((unsigned long int)byte2<<8)
      + ((unsigned long int)byte3<<16) + ((unsigned long int)byte4<<24);
	return( number );
}

float StlObject::getFloat()
{
	float number;
	unsigned char stream[5];
	// used to be 3,2,1,0
	stream[0]=(unsigned char)fgetc( stl_file );
	stream[1]=(unsigned char)fgetc( stl_file );
	stream[2]=(unsigned char)fgetc( stl_file );
	stream[3]=(unsigned char)fgetc( stl_file );

	stream[4] = '\0';

	memcpy( &number,&stream,4 ); 

	return( number );
}

unsigned int StlObject::getUint()
{
	unsigned char byte1,byte2;
	unsigned int number;
	byte1=(unsigned char)fgetc( stl_file );
	byte2=(unsigned char)fgetc( stl_file );
	number = (unsigned int)byte1 + ((unsigned int)byte2<<8);
	return( number );
}
