#ifndef GM_SEPARATOR_H_
#define GM_SEPARATOR_H_

namespace GM_lib{


//**********************************************************
//****  GM_Stream.h: interface for the Separator class. ****
//	  -------------------------------------------------
//	Separator is a class containing a caracter defining
//	the read/write separation between elements/ogjects etc.  
//	----------------------------------------------------
//				FOR FUTHER DEVELOPMENT !!
//	The invariants (stats) for Separator is:
//		_binary is default false.
//*********************************************************


class Separator
{
	unsigned int	_separator;
	bool			_binary;	// if true, the don't send the separator to the stream.
public:
	Separator(char s = ' ', bool bin = false) : _separator(s),_binary(bin)		{}
	Separator(const Separator& s) : _separator(s._separator),_binary(s._binary) {}

	bool getBinary() const		{ return _binary; }
	char getSeparator() const	{ return _separator; }

	void setBinary(bool bin)	{ _binary = bin; }
	void setSeparator(char sep) { _separator = sep; }

};


//**********************************************************
//******  Template iostream operators for Separators  ******
//**********************************************************

//If Separators flag for a binary stream has been set then nothing will be sendt to the stream. The 
//stream will also be left untouched if the separator is '' <- nothing. 
template <class T_Stream>
T_Stream& operator<<(T_Stream& out, const Separator& s)
{ 
	if (!s.getBinary())
	{
		out << s.getSeparator();
	}
	return out;
}


//If Separators flag for a binary stream has been set then nothing will be retrieved from the stream. 
template< class T_Stream >
T_Stream& operator>>(T_Stream& in, Separator& s)
{ 
	if (s.getBinary()) return in;

	if (s.getSeparator() != ' ' && s.getSeparator() != '\t' && s.getSeparator() != '\n')
	{
		unsigned int sep;
		in >> sep;
		s.setSeparator(sep);
	}
	return in;
}

//*******************************************************************
//* Predefinitions of the basic separators used throughout the code *
//*******************************************************************

// This separator is used between elements in a class. 
// For example Point where each dimention is separated.
extern Separator element_separator;
// This separator is used between objects in a class. 
// For example Array where each object in the array is separated.
extern Separator object_separator;
// For example used in matrix where each row is grouped.
extern Separator group_separator; 

} // end namespace GM_lib

#endif	//GM_SEPARATOR_H_