#ifndef _GM_SWALLOWTAIL_H_
#define _GM_SWALLOWTAIL_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Swallowtail : public PSurf<T,n>
{
	void _eval(T u, T v, int d1, int d2);
	void localDisplay();
	// void localSimulate(double dt);
protected:

	T _st_p_u()			{ return T(-2); }
	T _end_p_u()		{ return T(2); }
	T _st_p_v()			{ return T(-0.8); }
	T _end_p_v()		{ return T(0.8); }


public:

	Swallowtail();
	// Swallowtail(Swallowtail<T,n>& s);
  
	~Swallowtail() {}

	const char* identity() const { return " Swallowtail ";}
};
} // GM_lib

using namespace GM_lib;

template <class T, int n>
Swallowtail<T,n>::Swallowtail():PSurf<T,n>()
{
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),4.0));
}

template <class T, int n>
void Swallowtail<T,n>::_eval(T u, T v, int d1,int d2)
{

 	__p.setDim(4,4);
	__p[0][0][0] =	u*v*v + T(3)*v*v*v*v;
	__p[0][0][1] =	-T(2)*u*v - T(4)*v*v*v;
	__p[0][0][2] =	u;
 
 	if(d1) //u
 	{
		__p[1][0][0] =	v*v;
		__p[1][0][1] =	-T(2)*v;
		__p[1][0][2] =	T(1);
 	}
 	if(d1>1)//uu				 
 	{
		__p[2][0][0] =	T(0);			  
		__p[2][0][1] =	T(0);			  
		__p[2][0][2] =	T(0);			  
 	}
	if(d2) //v
 	{
		__p[0][1][0] =	2*u*v+12*v*v*v;
		__p[0][1][1] =	-2*u-12*v*v;
		__p[0][1][2] =	T(0);
 	}
	if(d2>1) //vv	
 	{
		__p[0][2][0] =	T(2)*u+T(36)*v*v;		
		__p[0][2][1] =	-T(24)*v;			
		__p[0][2][2] =	T(0);				
 	}
	if(d1 && d2) //uv
 	{
		__p[1][1][0] =	T(2)*v;			 
		__p[1][1][1] =	-T(2);			 
		__p[1][1][2] =	T(0);			 
 	}
	if(d1>1 && d2)//uuv				
 	{
		__p[2][1][0] =	T(0);	
		__p[2][1][1] =	T(0);	
		__p[2][1][2] =	T(0);	
 	}
	if(d1 && d2>1) //uvv			 
 	{
		__p[1][2][0] =	T(2);
		__p[1][2][1] =	T(0);
		__p[1][2][2] =	T(0);
 	}
 	if(d1>1 && d2>1) //uuvv
	{			 
 		__p[2][2][0] =	T(0);	
		__p[2][2][1] =	T(0);	
		__p[2][2][2] =	T(0);	
	}			


	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];

}
template <class T, int n>
void Swallowtail<T,n>::localDisplay()
{
	// r_mat.mult();
	PSurf<T,n>::localDisplay();
}
 #endif //_GM_SWALLOWTAIL_H_
