#include "GM_T_Array.h"



namespace GM_lib {


template<class T>
int T_Array<T>::incr = 4;

//********************************************************
//**************** DESTRUCTOR ****************************
//********************************************************

template<class T>
T_Array<T>::~T_Array()
{
	clear();
}


//********************************************************
//**********  OPERATOR= THE HOLE T_Array NEW VALUES ***
//********************************************************

template<class T>
T_Array<T>& T_Array<T>::operator=(const	T_Array<T>&	other)
{
	clear();
	if( other.size() > 0)
	{
		p = new T[other.size()];
		for(int i=0; i<other.size(); i++) p[i] = other.p[i];
		no_elem = max_elem = other.no_elem;
	}

	return *this;
}


//********************************************************
//**************** STRIPELEMENTS() - NO FREE ELEMENTS ****
//********************************************************

template<class T>
void	T_Array<T>::stripElements()
{
		// Remove surplus elements.
	for (int j,i=0; i<no_elem-1; i++)
		for (j=no_elem-1; j>i; j--)
			if((*this)[i] == (*this)[j])	removeIndex(j++);
}


//********************************************************
//**************** ALWAYSINSERT(OBJECT) - EVEN IF EXIST **
//**************** DEFAULT AT END OF T_Array ************
//********************************************************

template<class T>
void	T_Array<T>::alwaysInsert(const T& obj, bool first)
{
	if (no_elem == max_elem)
	{
		max_elem += incr;
		T* np = new T[max_elem];
		for(int i=0; i < no_elem; i++)	np[i] = p[i];
		delete [] p;
		p = np;
	}
	
	if(first)
	{
		for(int i=no_elem; i>0; i--)	p[i] = p[i-1];
		p[0] = obj;
	}
	else
		p[no_elem] = obj;

	no_elem++;
}


//********************************************************
//**************** ALWAYSINSERT(T_Array) - EVEN IF EXIST ***
//**************** DEFAULT AT END OF T_Array ***************
//********************************************************

template<class T>
void	T_Array<T>::alwaysInsert(const T_Array<T>& arr, bool first)
{
	for (int i=0; i< arr.size(); i++) 
		alwaysInsert(arr(i),first);
}


//********************************************************
//****************   REMOVEINDEX(index) -  ***************
//****************  remove element with given index ******
//********************************************************

template<class T>
bool	T_Array<T>::removeIndex(int	index)
{
		// Check index.
	if (index < 0 || index >= no_elem)		return false;
		// Move T_Array forward.
	no_elem--;
	for(int i=index; i< no_elem; i++)
		p[i] = p[i+1];
	return true;
}


//********************************************************
//****************   SETSIZE(number) -  ******************
//*******  set size of active numbers not potential ******
//********************************************************

template<class T>
void	T_Array<T>::setSize(int s)
{
	if (s <= 0)		clear();
	else if (s > max_elem)
	{
		max_elem = s;
		T* np = new T[max_elem];
		for(int i=0; i < no_elem; i++)	np[i] = p[i];
		delete [] p;
		p = np;
	}

	if (s > no_elem)
		for (int i = no_elem; i < s; i++) p[i] = T();

	no_elem = s;
}


//********************************************************
//****************   SETMAX(number) -   ******************
//************  set size of potential numbers ************
//********************************************************

template<class T>
void	T_Array<T>::setMax(int s)
{
 	if (s <= 0)		clear();
	else if (s > max_elem)
	{
		max_elem = s;
		T* np = new T[max_elem];
		for(int i=0; i < no_elem; i++)	np[i] = p[i];
		delete [] p;
		p = np;
	}
	else if (s < max_elem)
	{

	}

}


//********************************************************
//******************   STRIP() -     *********************
//******************  only actual numbers  ***************
//********************************************************

template<class T>
void	T_Array<T>::strip()
{
	if (max_elem > no_elem)
	{
		max_elem = no_elem;
		T* np = new T[max_elem];
		for(int i=0; i < no_elem; i++)	np[i] = p[i];
		delete [] p;
		p = np;
	}
}


//********************************************************
//******************   INDEX(object) - *******************
//********************************************************

template<class T>
int	T_Array<T>::index(const T& obj) const
{
	for (int i=0; i<no_elem; i++)
		if (p[i] == obj)		return i;

	return -1;
}


//********************************************************
//******************   TURN() -     **********************
//******************  turn the T_Array  ********************
//********************************************************

template<class T>
void	T_Array<T>::turn()
{
	for (int i=0, mid = no_elem/2; i<mid; i++)
		swap(i, no_elem - 1 - i);
}


//********************************************************
//******************   OPERATOR==   **********************
//******************  equality test for the hole T_Array  **
//********************************************************

template<class T>
bool T_Array<T>::operator==(const T_Array<T>& a) const
{
	if (size() != a.size()) 		return false;
	for (int i=0; i < size();  i++)
		if (p[i] != a.p[i]) return false;
	return true;
}

} // end namespace GM_lib

