
#include "GM_Texture.h"
extern "C" {
#include <jpeglib.h>
}
#include <stdarg.h> 
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <qgl.h>
#include <qimage.h>

#include "png.h"

//using namespace GM_lib;

namespace GM_lib {

map<char*, unsigned int>	Texture::_filenameMap;


// #define __DEBUG

/**[isExp]***********************************************
 * sjekker om 'answer' kan utredes fra en ren eksponent *
 * av 'fact' (f.eks. om 4 er et svar p en eksponent til*
 * 2 (2^2).												*
 ********************************************************/

bool is_exp(int fact, int answer) 
{
	int ans = fact;
	while (ans < answer) ans *= fact;
	if (ans == answer)	return true;
	else				return false;
}


/*
	konstruktr for Texture, kan bli brukt i framtiden
*/

Texture::Texture(bool store)
:data(NULL)
{
	_store = store;
}

Texture::Texture(int res_width, int res_hight)
:data(NULL)
{
	_store = false;
	height = res_hight;
	width  = res_width;
	data   = new unsigned char[4 * width * height];
}

/*
	dekonstruktr for selve Texture, fjerner bildedata.
*/

Texture::~Texture()
{
	if (this->data != NULL)
		delete this->data;
}


GL_Color& Texture::operator()(int i, int j)
{
	GL_Color* ct = (GL_Color*)data;
	return ct[j*width+i];
}


/*
	laster inn et BMP-bilde.
*/

void Texture::LoadBMP(FILE *fp, int num) 
{
	Texture_bmp_fileheader fh;
	Texture_bmp_infoheader ih;

	fread(&fh, 1, 14, fp);
	fread(&ih, 1, 40, fp);
#ifdef __DEBUG
	printf("(bmp) filesize: %d, width: %d, height: %d, bpp: %d\n", fh.file_size, ih.width, ih.height, ih.bpp);
#endif

	if ((ih.bpp == 24) && (ih.compression == 0))
	{
		unsigned char buf[3];
		width		= ih.width;
		height		= ih.height;
		bpp			= ih.bpp;
		dataformat	= GL_RGB; // IF CONVERSION REMOVED SET THIS TO GL_BGR_EXT
		data		= new unsigned char[width*height*(bpp/8)];	
		for (unsigned int i = 0; i < width*height; i++)
		{
			fread(buf, 1, ih.bpp/8, fp);			
			swap(buf[0], buf[2]);
			memcpy(data+i*3,buf, 3);
		}
	} 
	else 
	{
#ifdef __DEBUG
		printf("(bmp) format not supported, bpp: %d, compression: %d\n", ih.bpp, ih.compression);
#endif
	}
}

/*
	laster inn et JPEG-bilde (via libjpeg).
*/

void Texture::LoadJPEG(FILE *fp, int num) 
{
	struct jpeg_decompress_struct cinfo;
	struct jpeg_error_mgr jerr;

	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_decompress(&cinfo);
	jpeg_stdio_src(&cinfo, fp);
	jpeg_read_header(&cinfo, TRUE);

	width		= cinfo.image_width;
	height		= cinfo.image_height;
	bpp			= 24;
	dataformat	= GL_RGB;

#ifdef __DEBUG
	printf ("(jpeg) width: %d, height: %d\n", this->width, this->height);
#endif

	jpeg_start_decompress(&cinfo);
	data = new unsigned char[width*height*(bpp/8)];
	unsigned char * buf = new unsigned char[width*(bpp/8)];

	for(int i=0; cinfo.output_scanline < cinfo.output_height; i++)
	{
		jpeg_read_scanlines(&cinfo, (JSAMPARRAY) &buf, 1);
		// leser nedenfra og oppover, s vi setter verst i minnet frst..
		memcpy(data + 3*width*(cinfo.output_height - cinfo.output_scanline), buf, 3*width);
	}

	jpeg_finish_decompress(&cinfo);
	jpeg_destroy_decompress(&cinfo);
}

/*
	laster inn et TGA-bilde.
*/

void Texture::LoadTGA(FILE *fp, int num) 
{
	Texture_tga_fileheader fh;
	fread(&fh, 1, 18, fp);

	width		= fh.image_desc.width;
	height		= fh.image_desc.height;
	bpp			= fh.image_desc.bpp;
	dataformat	= bpp==24 ? GL_RGB :(bpp==32 ? GL_RGBA : GL_R);
#ifdef __DEBUG
	printf("(tga) width: %d, height: %d, bpp: %d\n", this->width, this->height, this->bpp);
#endif
	if (bpp<24) printf("(tga) WARNING: tga-loading for BPP < 24 not implemented!\n");

	fseek(fp, fh.idlength, SEEK_CUR);
	fseek(fp, fh.color_map.num_color_maps*(fh.color_map.bit_pr_map/8), SEEK_CUR);

	data = new unsigned char[width*height*(bpp/8)];
	fread(data, 1, width*height*(bpp/8), fp);
	for (unsigned int i=0; i<(width*height*(bpp/8)); i+=(bpp/8)) swap(data[i],data[i+2]);
}

/*
	laster inn et PNG-bilde (via libpng).
*/

void Texture::LoadPNG(FILE *fp, int num)
{
	png_uint_32 width, height, scanline;
	int bit_depth, color_type, interlace_type, row;

	png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	png_infop info_ptr  = png_create_info_struct(png_ptr);
	png_infop end_info  = png_create_info_struct(png_ptr);
	png_set_sig_bytes(png_ptr, 0);

	if (setjmp(png_jmpbuf(png_ptr)))
	{
		png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);	
		fclose(fp);
		return;
	}

	png_init_io(png_ptr, fp);
	png_read_info(png_ptr, info_ptr);
	png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, &interlace_type, NULL, NULL);
	
	height		= height;
	width		= width;
	bpp			= info_ptr->pixel_depth;
	dataformat	= GL_RGB;
#ifdef __DEBUG
	printf("(png) width: %d, height: %d, bpp: %d\n", this->width, this->height, this->bpp);
#endif
	if (color_type == PNG_COLOR_TYPE_PALETTE)	png_set_palette_to_rgb(png_ptr);
	if (bit_depth == 16)						png_set_strip_16(png_ptr);

	scanline = png_get_rowbytes(png_ptr,info_ptr);
	data     = new unsigned char[height*width*(bpp/8)];
	
	for(row = height-1; row >= 0; row--)
		png_read_row(png_ptr,(this->data+(row*scanline)),NULL);
	png_read_end(png_ptr,info_ptr);
	png_destroy_read_struct(&png_ptr,&info_ptr,(png_infopp)NULL);
}


unsigned int Texture::_findMapping(const char * filename)
{
	if(_store)
	{
		map<char*, unsigned int> ::iterator mapiterator;
		for ( mapiterator = _filenameMap.begin( ); mapiterator != _filenameMap.end( ); mapiterator++ )
		{
			//		printf("** comparing [%s], to [%s]\n",mapiterator->first,filename);
			if (stricmp(mapiterator->first, filename) == 0) 
			{
				return mapiterator->second;
			}
		}
	}
	return 0;
}
char* Texture::findFileName(int txname)
{
		map<char*, unsigned int> ::iterator mapiterator;
		for ( mapiterator = _filenameMap.begin( ); mapiterator != _filenameMap.end( ); mapiterator++ )
		{
			//		printf("** comparing [%s], to [%s]\n",mapiterator->first,filename);
			if (mapiterator->second== txname) 
			{
				return mapiterator->first;
			}
		}
		return 0;
}
void Texture::_addMapping(const char * filename, unsigned int texid)
{
//	if (_findMapping(filename) == 0)
	if(_store)
		_filenameMap.insert(Str_Uint_Pair(strcpy(new char[strlen(filename)+1],filename),texid));
}

/*
	return 
	1 if image was loaded.
	0 if image has been loaded earlier.
	-1 unable to load the image.
*/
int Texture::_load_openGL(const char* filename, int num, unsigned int & textureID)
{
	int result = -1;
//	textureID  = _findMapping(filename); // filename loaded earlier?
	textureID  = 0; // filename loaded earlier?

	if (textureID == 0) // image has not been loaded before.
	{
		glGenTextures(1,&textureID);
#ifdef __DEBUG
		printf ("[Texture] loading file: %s into texture %d\n", filename, textureID);
#endif
		if (!this->Load(filename, num))
		{
			glDeleteTextures(1,&textureID);
			textureID = 0;
			result = -1;
#ifdef __DEBUG
			printf ("[Texture] Load error on file: %s\n", filename);
#endif
		}
		if (textureID != 0) 
			_addMapping(filename, textureID);
		result = 1;
	}
	else
		result = 0;

	return result;
}


/*
	srger for  laste inn et bilde til minnet fr det
	sendes videre inn til opengl og frigjres fra vanlig
	systemminne.
*/
unsigned int Texture::LoadPatternGL(const char * filename, int num) 
{
	unsigned int textureID;
	int loadresult = this->_load_openGL(filename, num, textureID);

	if (loadresult == 1) 
	{
		glBindTexture(GL_TEXTURE_2D, textureID);
		if (this->bpp == 24) 
		{
			unsigned char  buf[4];
			unsigned char* newdata = new unsigned char[4*width*height];
			for(unsigned int j = 0; j < height; j++)
				for(unsigned int i = 0; i < width; i++)
				{
					memcpy(buf,data+i*3+j*3*width,3);
					buf[3] = (buf[0]==255 && buf[1]==255 && buf[2]== 255) ? 255:0;
					if ( i != (width -1) && i != 0) memcpy(newdata+i*4+j*(width-2)*4,buf,4);
				}
				gluBuild2DMipmaps(GL_TEXTURE_2D, 4, width-1, height, GL_RGBA, GL_UNSIGNED_BYTE, newdata);
				delete newdata;
		}
		else if(this->bpp == 32)
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data);

		delete this->data;
		this->data = NULL;

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	}
	else if (loadresult == 0)
	{
#ifdef __DEBUG
		printf ("[Texture] file: %s already loaded into texture %d\nCANNOT GUARANTEE THAT THE PARAMETERS FOR THIS IMAGE IS CORRECT.\n", filename, textureID);
#endif
	}
	return textureID;
}



unsigned int Texture::LoadGL() 
{
	unsigned int textureID;
	glGenTextures(1,&textureID);
	glBindTexture(GL_TEXTURE_2D, textureID);

	glTexImage2D(GL_TEXTURE_2D, 3, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGBA, GL_UNSIGNED_BYTE,data);

	delete data;
	data = NULL;

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	return textureID;
}


unsigned int Texture::LoadGL(const char * filename, int num) 
{
	unsigned int textureID;
	int loadresult = _load_openGL(filename, num, textureID);

	if (loadresult == 1) 
	{
		glBindTexture(GL_TEXTURE_2D, textureID);

		if (bpp == 24) 
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGB, GL_UNSIGNED_BYTE, data);
		else if(bpp == 32)
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data);

		delete data;
		data = NULL;

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	}
	else if (loadresult == 0)
	{
		#ifdef __DEBUG
				printf ("[Texture] file: %s already loaded into texture %d\nCANNOT GUARANTEE THAT THE PARAMETERS FOR THIS IMAGE IS CORRECT.\n", filename, textureID);
		#endif
	}
	return textureID;
}

unsigned int Texture::LoadGL( const QImage* image ) 
{
	QImage converted_image = QGLWidget::convertToGLFormat( *image );

	unsigned int textureID;

	glGenTextures(1,&textureID);
	width = converted_image.width();
	height = converted_image.height();
	bpp = converted_image.depth();
	dataformat	= GL_RGB;
	data = converted_image.bits();

	glBindTexture(GL_TEXTURE_2D, textureID);

	if (bpp == 24) 
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGB, GL_UNSIGNED_BYTE, data);
	else if(bpp == 32)
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data);

	data = NULL;

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	return textureID;
}

unsigned int Texture::LoadGLSphereMap(const char * filename, int num) 
{
	unsigned int textureID;
	int loadresult = _load_openGL(filename, num, textureID);

	if (loadresult == 1) 
	{
		glBindTexture(GL_TEXTURE_2D, textureID);

		if (bpp == 24)
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGB, GL_UNSIGNED_BYTE, data);
		else if(bpp == 32)
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data);

		delete data;
		data = NULL;

		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);	
	}
	else if (loadresult == 0)
	{
#ifdef __DEBUG
		printf ("[Texture] file: %s already loaded into texture %d\nCANNOT GUARANTEE THAT THE PARAMETERS FOR THIS IMAGE IS CORRECT.\n", filename, textureID);
#endif
	}
	return textureID;
}

unsigned int Texture::LoadGLObjLinearMap(const char * filename, int num) 
{
	unsigned int textureID;
	int loadresult = _load_openGL(filename, num, textureID);

	if (loadresult == 1) 
	{
		glBindTexture(GL_TEXTURE_2D, textureID);

		if (bpp == 24)
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGB, GL_UNSIGNED_BYTE, data);
		else if(bpp == 32)
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data);

		delete data;
		data = NULL;

		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);	
	}
	else if(loadresult == 0)
	{
		#ifdef __DEBUG
				printf ("[Texture] file: %s already loaded into texture %d\nCANNOT GUARANTEE THAT THE PARAMETERS FOR THIS IMAGE IS CORRECT.\n", filename, textureID);
		#endif
	}
	return textureID;
}

void Texture::splitToGL(int idx, int width, int height, int startx, int starty, GLuint filtermode) 
{
	unsigned char * buffer = new unsigned char[width*height*(this->bpp/8)];
	int x, y, i;

	for (y = 0; y < height; y++) 
		for (x = 0; x < width; x++)
			for (i = 0; i < bpp/8; i++)
				buffer[i+x*(bpp/8)+y*width*(bpp/8)] = data[i+(startx+x)*(bpp/8)+(starty+y)*(bpp/8)*width];

	glBindTexture(GL_TEXTURE_2D, idx);

	if (this->bpp == 24) 
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, buffer);
	else if(this->bpp == 32)
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, buffer);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filtermode);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filtermode);

	delete buffer;
}

bool Texture::Load(const char * filename, int num)
{
	FILE *fp = fopen(filename, "rb");

	bool loaded = false;

	if (fp) 
	{
		char buf[20];
		fread(buf, 1, 10, fp);
		rewind(fp);

		if ((buf[0] == 'B') && (buf[1] == 'M')) 
		{
			LoadBMP(fp, num);
			loaded = true;
		} 
		else if ((buf[6] == 'J') && (buf[7] == 'F') && (buf[8] == 'I') && (buf[9] == 'F')) 
		{
			LoadJPEG(fp, num);
			loaded = true;
		} 
		else if (png_sig_cmp((unsigned char *)&buf[0], 0, 8) == 0) 
		{
			LoadPNG(fp, num);
			loaded = true;
		} 
		else 
		{
			fseek(fp, -18, SEEK_END);
			fread(buf, 1, 18, fp);
			if (strcmp(buf, "TRUEVISION-XFILE.") == 0) 
			{
				rewind(fp);
				LoadTGA(fp, num);
				loaded = true;
			}
		}
		fclose(fp);
	} 
	else 
	{
#ifdef __DEBUG
		printf ("(Texture) sorry, couldnt open %s!\n", filename);
#endif
	
	}
	return loaded;
}

void Texture::ScreenDumpBMP(char * filename)
{
	GLint viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);		

	unsigned int* pixels = new unsigned int[viewport[2]*viewport[3]];
	memset(pixels, 0, viewport[2]*viewport[3]*sizeof(int));
	glReadBuffer(GL_FRONT);
	glReadPixels(0,0, viewport[2],viewport[3], GL_RGB, GL_BYTE, pixels);

	WriteBMP(filename, viewport[2],viewport[3], (char *)pixels);
	delete pixels;
}

void Texture::ScreenDumpBMPSequence(char * filename)
{
	static int frame = 0;
	char * fname = new char[strlen(filename)+10];
	sprintf(fname, "%s_%03d.bmp", filename, frame);

	ScreenDumpBMP(fname);
	delete fname;
	frame++;
}

void Texture::WriteBMP(char * filename, int w, int h, char * data)
{
	Texture_bmp_fileheader fh;
	Texture_bmp_infoheader ih;

	FILE * fp = fopen (filename, "wb");

	if (!fp)
	{
		char * message = new char[256];
		if (strlen(filename) > 200) filename[200] = '\0';
		sprintf(message, "Could not write file: %s", filename);
//		MessageBox(NULL, message, "(Texture) WriteBMP", 0);
		delete message;
	}
	else
	{
		memset(&fh, 0, 14);
		memset(&ih, 0, 40);

		fh.file_size	= w*h*3;
		/* 0x4D42 = BM */
		fh.signature	= 0x4D42;
		/* 0x36 = 54 */
		fh.dataoffset	= 0x36;

		ih.bpp				= 24;
		ih.compression		= 0;
		ih.width			= w;
		ih.height			= h;
		ih.size_of_header	= 40;

		fwrite(&fh, 1, 14, fp);
		fwrite(&ih, 1, 40, fp);
		for (int i=0; i < w*h*3; i+=3)	
		{
			swap(data[i],data[i+2]);
			fwrite(&data[i], 1, 3, fp);
		}
		fclose(fp);
	}
}

} // namespace GM_lib
