#pragma once

#include <stdio.h>
#include "GM_OpenGL.h"
#include <GL/glu.h>
//using namespace GM_lib;

#include <map>
#include <string.h>
using namespace std;

#include <qimage.h>
namespace GM_lib {

#define Texture_PROGRESS_CALLBACK
#define Texture_VERSION 006

#pragma pack(1)

typedef struct {
	unsigned int size_of_header;
	unsigned int width;
	unsigned int height;
	unsigned short planes;
	unsigned short bpp;
	unsigned int compression;
	unsigned int imagesize;
	unsigned int xpixelsprm;
	unsigned int ypixelsprm;
	unsigned int colorsused;
	unsigned int colorsimportant;
} Texture_bmp_infoheader;

typedef struct {
	short signature;
	// filesize er skrot.
	unsigned int file_size;
	unsigned int reserved;
	unsigned int dataoffset;
} Texture_bmp_fileheader;

typedef struct {
	unsigned short first_idx;
	unsigned short num_color_maps;
	char bit_pr_map;
} Texture_tga_colormap;

typedef struct {
	unsigned short x_origin;
	unsigned short y_origin;
	unsigned short width;
	unsigned short height;
	char bpp;
	char image_descriptor;
} Texture_tga_imagedesc;

typedef struct {
	char idlength;
	char colormap_type;
	char image_type;
	Texture_tga_colormap color_map;
	Texture_tga_imagedesc image_desc;
} Texture_tga_fileheader;

#pragma pack()

class Texture  
{
private:
	typedef pair<char*, unsigned int> Str_Uint_Pair;
	static map<char*, unsigned int>		_filenameMap;
	bool								_store;

protected:
	unsigned int	_findMapping(const char * filename);
	void			_addMapping(const char * filename, unsigned int texid);
	int				_load_openGL(const char* filename, int num, unsigned int & textureID);

public:
	Texture(bool store=true);
	Texture(int res_width, int res_hight);
	virtual ~Texture();

	GL_Color& operator()(int i, int j);

	unsigned int LoadGL();
	unsigned int LoadGL(const char * filename, int num = 0);
	unsigned int LoadGL( const QImage* image );
	unsigned int LoadGLSphereMap(const char * filename, int num = 0);
	unsigned int LoadGLObjLinearMap(const char * filename, int num=0);
	unsigned int LoadPatternGL(const char * filename, int num = 0);

	void LoadBMP(FILE *fp, int num);
	void LoadJPEG(FILE *fp, int num);
	void LoadTGA(FILE *fp, int num);
	void LoadPNG(FILE *fp, int num);
	
	void ScreenDumpBMPSequence(char * filename);
	void ScreenDumpBMP(char * filename);
	void WriteBMP(char * filename, int w, int h, char * data);
//	void LoadRAWGL(unsigned int texidx, char * filename, unsigned int x, unsigned int y, int num = 0);
/*	void LoadGL(GLuint * indexes, unsigned int files, ...);

	void LoadGL640x480Split(GLuint * indexes, char * filename, int num = 0);
	void LoadMultipleGL(GLuint * indexes, char * filename, unsigned int sizex, unsigned int sizey, int num = 0);
*/

	bool Load(const char * filename, int num = 0);
    static
	char* findFileName(int txname);

private:
	unsigned char*	data;
	unsigned int	height;
	unsigned int	width;
	unsigned short	bpp;
	unsigned int	dataformat;

	void splitToGL(int idx, int width, int height, int startx, int starty, GLuint filtermode = GL_NEAREST);
};

} // namespace GM_lib
