#ifndef TORUS_H
#define  TORUS_H

#include "GM_PSurf.h"

namespace GM_lib{

	template <class T, int n>
	class Torus : public PSurf<T,n>
	{
		T _a;		//Wheelradius
		T _b;		//Tuberadius1
		T _c;		//Tuberadius2

		void _eval(T u, T v, int d1, int d2);

	protected:
		T _st_p_u()			{ return T(0); }
		T _end_p_u()		{ return T(2*M_PI); }
		T _st_p_v()			{ return T(0); }
		T _end_p_v()		{ return T(2*M_PI); }

	public:

		Torus(T wheelrad=T(3), T tuberad1=T(1), T tyberad2=T(1));
		Torus(const Torus<T,n>& s);

		virtual  
		~Torus() {}

		bool closedU()   { return true; }
		bool closedV()   { return true; }

		const char* identity() const { return " Torus ";}
	};
} // GM_lib

using namespace GM_lib;

template <class T, int n>
Torus<T,n>::Torus(T wheelrad, T tuberad1, T tuberad2):PSurf<T,n>()
{
	_a = wheelrad;
	_b = tuberad1;
	_c = tuberad2;

	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),_a+max(_b,_c)));
}

template <class T, int n>
Torus<T,n>::Torus(const Torus<T,n>& s):PSurf<T,n>(s)
{
	_a = s._a;
	_b = s._b;
	_c = s._b;
}


template <class T, int n>
void Torus<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(d1+1,d2+1);

	T su = sin(u);
	T sv = sin(v);
	T cu = cos(u);
	T cv = cos(v);
	T bcva = _b*cos(v)+_a;
	T cucv = _b*cu*cv;
	T cusv = _b*cu*sv;
	T sucv = _b*su*cv;
	T susv = _b*su*sv;
	sv *= _c;
	cv *= _c;
	cu *= bcva;
	su *= bcva;
						//S
	__p[0][0][0] =	     cu;
	__p[0][0][1] =       su;
	__p[0][0][2] =       sv;
	if(d1)				//Su
	{
		__p[1][0][0] =	-su;
		__p[1][0][1] =	 cu;
		__p[1][0][2] =	 T(0);
	}
	if(d1>1)			//Suu
	{
		__p[2][0][0] =  -cu;
		__p[2][0][1] =  -su;
		__p[2][0][2] =   T(0);
	}
	if(d2)				//Sv
	{
		__p[0][1][0] =  -cusv;
		__p[0][1][1] =  -susv;
		__p[0][1][2] =   cv;
	}
	if(d2>1)			//Svv
	{
		__p[0][2][0] =  -cucv;
		__p[0][2][1] =  -sucv;
		__p[0][2][2] =  -sv;
	}
	if(d1 && d2)		//Suv
	{
		__p[1][1][0] =   susv;
		__p[1][1][1] =  -cusv;
		__p[1][1][2] =   T(0);
	}
	if(d1>1 && d2)		//Suuv
	{
		__p[2][1][0] =   cusv;
		__p[2][1][1] =   susv;
		__p[2][1][2] =   T(0);
	}
	if(d1 && d2>1)		//Suvv
	{
		__p[1][2][0] =   sucv;
		__p[1][2][1] =  -cucv;
		__p[1][2][2] =   T(0);
	}
	if(d1>1 && d2>1)	//Suuvv
	{
		__p[2][2][0] =   cucv;
		__p[2][2][1] =   sucv;
		__p[2][2][2] =   T(0);
	}
	if(d1>2 && d2)		//Suuuv
	{
		__p[3][1][0] =  -susv;
		__p[3][1][1] =   cusv;
		__p[3][1][2] =   T(0);
	}
	if(d1>2 && d2>1)	//Suuuvv
	{
		__p[3][2][0] =  -sucv;
		__p[3][2][1] =   cucv;
		__p[3][2][2] =   T(0);
	}
	if(d1>2 && d2>1)	//Suuuvvv
	{
		__p[3][3][0] =   susv;
		__p[3][3][1] =  -cusv;
		__p[3][3][2] =   T(0);
	}
	if(d1 && d2>2)		//Suvvv
	{
		__p[1][3][0] =  -susv;
		__p[1][3][1] =   cusv;
		__p[1][3][2] =   T(0);
	}
	if(d1>1 && d2>2)	//Suuvvv
	{
		__p[2][3][0] =  -cusv;
		__p[2][3][1] =  -susv;
		__p[2][3][2] =   T(0);
	}
	if(d1>2)			//Suuu
	{
		__p[3][0][0] =   su;
		__p[3][0][1] =  -cu;
		__p[3][0][2] =   T(0);
	}
	if(d2>2)			//Svvv
	{
		__p[0][3][0] =   cusv;
		__p[0][3][1] =   susv;
		__p[0][3][2] =  -cv;
	}

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}

#endif 
