// $Id:

#ifndef GM_TRIANGLE_H_
#define GM_TRIANGLE_H_


#include "GM_Point3D.h"
#include "GM_Point2D.h"
#include "GM_Array.h"
#include "GM_T_Array.h"
#include "GM_LX_Array.h"
#include "GM_DMatrix.h"

namespace GM_lib {


//*********************************************************
//********* The six classes in the triangle system  *******
//*********************************************************

							// This is the main class. Here you can make, adjust
template <class T>			// and handle a triangle based surface. The class is also
class TriangleFacets;		// the storage class for the vertices, edges and triangles

template <class T>
class TriangleSystem;		// The base class for vertices, edges and triangles

template <class T>
class Vertex;				// The vertex class storing 3D position and a normal

template <class T>
class Edge;					// The edge class defined by 2 vertices, can be fixed

template <class T>
class Triangle;				// The triangle class defined by 3 edges


template <class T>			// This is a picewice linear curve
class PWVLine;				// based on vertices. You can also use the normal


//*********************************************************
//*********** The storage class  TriangleFacets	 **********
//*********************************************************

template <class T>
class  TriangleFacets : public LX_Array<Vertex<T> >
{
	LX_Array<Edge<T>* >		_edge;
	LX_Array<Triangle<T>* > _triangle;

	int _d;
	DMatrix<T_Array<Triangle<T>*> > _tri_order;
	T_Array<T> _u;
	T_Array<T> _v;

	Box<T,3> _box;

	Vertex<T>	__v;		// dummy because of MS-VC++ compiler
	Edge<T>		__e;		// dummy because of MS-VC++ compiler
	Triangle<T> __t;		// dummy because of MS-VC++ compiler

	int  surroundingTriangle(Triangle<T>*&, const Vertex<T>&);// const;
	bool removeLastVertex();
	void set(int i);
	bool fillPolygon(Array<Edge<T>*>&);

  friend class TriangleSystem<T>; 

	void insertTriangle(Triangle<T>*);							// Used by  TriangleSystem<T>
	void removeTriangle(Triangle<T>*);							// Used by  TriangleSystem<T>
	void corrigateTriangle(Triangle<T>*, bool wider = false);	// Used by  TriangleSystem<T>

	Vertex<T>* find(const Point<T,3>&) const;
	Edge<T>*   find(const Point<T,3>&, const Point<T,3>&) const;

	LX_Array<Edge<T>* >&		edge()		{ return _edge; }
	LX_Array<Triangle<T>* >&	triangle()	{ return _triangle; }


 public:

	TriangleFacets(int d=0): LX_Array<Vertex<T> >(d>0?d+3:0),_edge(),_triangle()
															{ setStreamMode(); list_name=0;}
	TriangleFacets(const LX_Array<Vertex<T> >& v): LX_Array<Vertex<T> >(v.size()+3),
								_edge(),_triangle()	{ (*this) = v; setStreamMode();  list_name=0;}

	~TriangleFacets();
							
	Box<T,3>					box() const	{ return _box; }

	int numVertices()	const { return size(); }
	int numEdges()		const { return _edge.size(); }
	int numTriangles()	const { return _triangle.size(); }

	Vertex<T>*		vertex(int i)	const		{ return &elem(i); }
	Edge<T>*		edge(int i)		const		{ return _edge(i); }
	Triangle<T>*	triangle(int i)	const		{ return _triangle(i); }

	/* NY KODE FOR  GJRE DEN NYE VERTEXEN CONST 12/9 -2001 (MARTIN)*/
	bool insertVertex(Vertex<T>&, bool c=false);
	bool removeVertex(Vertex<T>&);
	bool removeVertexNew(Vertex<T>& v);

	/* NY KODE FOR  GJRE EN EDGE CONST 12/9 -2001 (MARTIN)*/
	bool setConstEdge(Vertex<T> v1, Vertex<T> v2);

	void computeNormals();

	void delaunayTriangulate();

	void insertPwl(PWVLine<T>&);

	/* TAGIT VEK CONST P BDE evalZ-FUNKSJONERNE 13/9 (MARTIN */
	T evalZ(const Point2D<T>&, int deg=1); //const
	T evalZ(T x, T y, int deg=1)  		{ return evalZ(Point2D<T>(x,y),deg); }

	Point<T,3> eval(T x, T y, int deg=1) const { return Point3D<T>(x,y,evalZ(x,y,deg)); }
	Point<T,3> eval(const Point2D<T>& p, int deg=1) const { return eval(p(0),p(1),deg); }

	void clear(int d=-1);

	// ********** GL-render functions ***********

 private:
	int	 list_name;
 public:
	int  initRender(); //const;
	void render(); // const;
		
	// ********** IOSTREAM overloaded operators ***********

 private:
	int	 stream_mode;
 public:
	void setStreamMode(int i=1) { stream_mode = i; }
	int  getStreamMode() { return stream_mode; }

#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
 private:
    ostream& _prOut(ostream& out) const;
    istream& _prIn(istream& in);
 public:
  friend
    ostream& operator<<(ostream& out, const  TriangleFacets<T>& v) { return v._prOut(out);}
  friend
    ostream& operator<<(ostream& out, const  TriangleFacets<T>* v) { return v->_prOut(out);}
  friend
	  istream& operator>>(istream& in,  TriangleFacets<T>& v) { return istream >> v; /*return v._prIn(in);*/ }
  friend
    istream& operator>>(istream& in,  TriangleFacets<T>* v) { return v->_prIn(in); }

    void print(char prompt[]=" TriangleFacets<T>", ostream & out = cout) const
	   { out << prompt << ": " << (*this) << "\n"; }
#endif
};


//*********************************************************
//*********** The base class TriangleSystem ***************
//*********************************************************

template <class T>
class TriangleSystem
{
	static  TriangleFacets<T>* tv;

 protected:

	void insert(Edge<T> *e)			{ (tv->edge()) += e; }
	void insert(Triangle<T> *t)		{ tv->insertTriangle(t); }

	void corrigate(Triangle<T> *t, bool wider = false)	{ tv->corrigateTriangle(t,wider); }

	void remove(Edge<T> *e)			{ (tv->edge()).remove(e); }
	void remove(Triangle<T> *t)		{  tv->removeTriangle(t); }


	Vertex<T>* find(const Point<T,3>& p) const			      { return tv->find(p); }
	Edge<T>*   find(const Point<T,3>& p1,const Point<T,3>& p2){ return tv->find(p1,p2); }

    int  getStreamMode()            { return tv->getStreamMode(); }

 public:

	void set( TriangleFacets<T>& ts)	{ tv = &ts;}
};


//*********************************************************
//**************** The Vertex class  **********************
//*********************************************************

template <class T>
class Vertex: private Arrow<T,3>, public TriangleSystem<T>
{


	T_Array<Edge<T>*> edg;

	bool _const;

	void setValues(const Point<T,3>& p, const Vector<T,3>& n) { setPos(p); setDir(n); }

  friend class Edge<T>;
  friend class TriangleFacets<T>;
	
	void setValues(const Vertex<T>& v) 							// Used by  TriangleFacets<T>
								{ setValues(v.position(),v.normal()); _const=v._const;}

	bool insertEdge(Edge<T>* e)	{ return edg.insert(e); }		// Used by Edge<T>
	bool removeEdge(Edge<T>* e)	{ return edg.remove(e); }		// Used by Edge<T>

	void setNormal();											// Used by  TriangleFacets<T>
	void deleteEdges();											// Used by  TriangleFacets<T>

 public:

	Vertex() : Arrow<T,3>(), edg()									{ _const = false; }  
	Vertex(const Point<T,2>& v) : Arrow<T,3>(Point3D<T>(v(0),v(1),0)){ _const = false; }
	Vertex(const Point<T,3>& p) : Arrow<T,3>(p), edg()				{ _const = false; }
	Vertex(const Point<T,3>& p, const Vector<T,3>& n) : Arrow<T,3>(p,n), edg()
																	{ _const = false; }
	Vertex(const T& x,const T& y,const T& z=T()) : Arrow<T,3>(Point3D<T>(x,y,z)), edg() 
																	{ _const = false; }  

   ~Vertex();


	void setConst(bool c=true)		{ _const = c; }
	bool getConst()	const			{ return _const; }
	
	void setZ(T z);	// NY KODE FOR  ENDRE Z-VERDI 1/9 -2001 (MARTIN)
	
	Point<T,2>			parameter() const	{ return Point2D<T>(pos()); }
	Point<T,3>			position() const	{ return pos(); }
	Point<T,3>			normal() const		{ return dir(); }
	T_Array<Edge<T>*>&	edges() 			{ return edg; }

	Array<Edge<T>*>		outerEdges() const;
	Array<Triangle<T>*> triangles() const;

	int  isInside(Triangle<T>*) const;

	Vertex<T>& operator=(const Vertex<T>& t);	// NY KODE  20/9 -2001 (MARTIN)
	bool operator<(const Vertex<T> &t) const { return true; }

	bool operator==(const Vertex<T>& t)	const { return parameter() == t.parameter();}

	// ********** IOSTREAM overloaded operators ***********

#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
 private:
    ostream& _prOut(ostream& out)const;
    istream& _prIn(istream& in);
 public:
  friend
    ostream& operator<<(ostream& out, const Vertex<T>& v) { return v._prOut(out); }
  friend
    ostream& operator<<(ostream& out, const Vertex<T>* v) { return v->_prOut(out); }
  friend
    istream& operator>>(istream& in, Vertex<T>& v)	{ return v._prIn(in); }
  friend
    istream& operator>>(istream& in, Vertex<T>* v)	{ return v->_prIn(in); }

    void print(char prompt[]="Vertex<T>", ostream & out = cout) const
		{ out << prompt << ": " << (*this) << "\n"; }
#endif
};


//*********************************************************
//**************** The Edge class  ************************
//*********************************************************

template <class T>
class Edge: public TriangleSystem<T>
{
	Vertex<T>*   _vertex[2];
	Triangle<T>* _triang[2];

	bool	_const;

	void upv()
		{ _vertex[0]->insertEdge(this); _vertex[1]->insertEdge(this);
		 if (_vertex[0]->getConst() && _vertex[1]->getConst()) _const = true; }

	bool swap();

  friend class Triangle<T>;
  friend class TriangleFacets<T>;

	void okDelaunay();										//Used by Triangle<T>

	void setTriangle(Triangle<T>* t1,Triangle<T>* t2)		//Used by Triangle<T> and TriangleFacets<T>
							{_triang[0]=t1;_triang[1]=t2;}

	bool swapTriangle(Triangle<T>* ot, Triangle<T>* nt);	//Used by TriangleFacets<T>

	bool removeTriangle(Triangle<T>*);						//Used by Triangle<T>

	Vertex<T>*	 otherVertex(const Vertex<T>&) const;		//Used by TriangleFacets<T>

	Triangle<T>* other(Triangle<T>*);						//Used by Triangle<T> and TriangleFacets<T>

	bool swapVertex(Vertex<T>& is_v,Vertex<T>& new_v);		//Used by TriangleFacets<T>

	bool isFirst(Vertex<T>* v) { return v == _vertex[0]; }	//Used by TriangleFacets<T>
	bool isLast(Vertex<T>* v)  { return v == _vertex[1]; }	//Used by TriangleFacets<T>
	
	void turn() {Vertex<T>* v=_vertex[0];_vertex[0]=_vertex[1];_vertex[1]=v;}//Used by TriangleFacets<T>

	bool boundary() const { return _triang[0] == NULL || _triang[1] == NULL;}//Used by TriangleFacets<T>

	bool splitt(Vertex<T>& p);								//Used by TriangleFacets<T>

	bool hasPoints(const Point<T,3>& , const Point<T,3>&) const;//Used by TriangleFacets<T>

	Edge<T>* next();											//Used by TriangleFacets<T>


 public:
	Edge() { _vertex[0]=_vertex[1]=NULL;_triang[0]=_triang[1]=NULL; _const=false; }
	Edge(Vertex<T>& s, Vertex<T>& e): _triang()
		{ _vertex[0]=&s; _vertex[1]=&e; _const=false; upv(); } 
	Edge(const Edge<T>& e): _vertex(e._vertex),_triang(e._triangle)
		{ _const = e._const; upv(); }

   ~Edge();

    void setConst(bool c=true)		{ _const = c; }

	Array<Triangle<T>*> getTriangle();

	Vertex<T>*	 firstVertex() const { return _vertex[0]; }

	Vertex<T>*	 lastVertex() const { return _vertex[1]; }

	Vertex<T>*	 commonVertex(const Edge<T>&) const;

	Vector<T,2>	 vector2D(){ return _vertex[1]->parameter() - _vertex[0]->parameter(); }
	Vector<T,3>	 vector()  { return _vertex[1]->position() - _vertex[0]->position(); }

	T length2D()	{ return vector2D().length(); }
	T length()		{ return vector().length(); }

 	// ********** IOSTREAM overloaded operators ***********

#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
 private:
    ostream& _prOut(ostream& out)const;
    istream& _prIn(istream& in);
 public:

  friend
    ostream& operator<<(ostream& out, const Edge<T>& v) { return v._prOut(out); }
  friend
    ostream& operator<<(ostream& out, const Edge<T>* v) { return v->_prOut(out); }
  friend
    istream& operator>>(istream& in, Edge<T>& v)	{ return v._prIn(in); }
  friend
    istream& operator>>(istream& in, Edge<T>* v)	{ return v->_prIn(in); }

    void print(char prompt[]="Edge<T>", ostream & out = cout) const
	   { out << prompt << ": " << (*this) << "\n"; }
#endif
};



//*********************************************************
//**************** The Triangle class  ********************
//*********************************************************

template <class T>
class Triangle: public TriangleSystem<T>
{
private:
	Edge<T>* _edge[3];

	Box<unsigned char,2> _box;

  friend class Edge<T>;
  friend class TriangleFacets<T>;

	void setEdges(Edge<T>* e1, Edge<T>* e2, Edge<T>* e3)//Used by Edge<T>
				{ _edge[0]=e1; _edge[1]=e2; _edge[2]=e3; }

	bool turn(Edge<T>*);								//Used by Edge<T> and TriangleFacets<T>

	bool splitt(Vertex<T>& p);							//Used by TriangleFacets<T>

	T evalZ(const Point2D<T>& p, int deg=1) const;		//Used by TriangleFacets<T>

	Box<unsigned char,2>& box() { return _box; }		//Used by TriangleFacets<T>

	void updateBox(T_Array<T>& ,T_Array<T>& , int);		//Used by TriangleFacets<T>

	void render()  const;								//Used by TriangleFacets<T>

 public:

	Triangle() { _edge[0]=NULL; _edge[1]=NULL; _edge[2]=NULL; }
	Triangle(Edge<T>* e1, Edge<T>* e2, Edge<T>* e3)  
		{ _edge[0]=e1; _edge[1]=e2; _edge[2]=e3; }
    
	Triangle(const Triangle<T>& t): _edge(t._edge),_box(t._box) {}

   ~Triangle();


	Array<Edge<T>*>		edges() const	{ return Array<Edge<T>*>(3,_edge);}
	Array<Vertex<T>*>	vertices() const;
    UnitVector<T,3>		normal() const;
	
	int	 isAround(const Vertex<T>& v) const { return v.isInside(*this); }

	// ********** IOSTREAM overloaded operators ***********

#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
 private:
    ostream& _prOut(ostream& out)const;
    istream& _prIn(istream& in);
 public:
  friend
    ostream& operator<<(ostream& out, const Triangle<T>& v)
	   { return v._prOut(out); }
  friend
    ostream& operator<<(ostream& out, const Triangle<T>* v)
	   { return v->_prOut(out); }
  friend
    istream& operator>>(istream& in, Triangle<T>& v)
	   { return v._prIn(in); }
  friend
    istream& operator>>(istream& in, Triangle<T>* v)
	   { return v->_prIn(in); }

    void print(char prompt[]="Triangle<T>", ostream & out = cout) const
	   { out << prompt << ": " << (*this) << "\n"; }
#endif
};


//*********************************************************
//*********** The storage class  TriangleFacets **********
//*********************************************************

template <class T>
class PWVLine : public Array<Vertex<T> >
{

 public:

	PWVLine(int d=0) : Array<Vertex<T> >(d)						{}  
	PWVLine(const Array<Vertex<T> >& v) : Array<Vertex<T> >(v)	{}
							
	Vertex<T> interpolate(int, double) const;
};




//********************************************************
//**************** PRIVATE OSTREAM FUNCTIONS *************
//********************************************************


#if defined(IOSTREAMH) || defined(__IOSTREAM_H) || defined(_IOSTREAM_)
template <class T>
inline
ostream&  TriangleFacets<T>::_prOut(ostream& out) const
{
	const LX_Array<Vertex<T> >& vertex = *this;
	if(stream_mode==1 || stream_mode==4 || stream_mode==5 || stream_mode==7)
		out << vertex << endl << endl;
	if(stream_mode==2 || stream_mode==4 || stream_mode==6 || stream_mode==7)
		out << _edge << endl;
	if(stream_mode==3 || stream_mode==5 || stream_mode==6 || stream_mode==7)
		out << _triangle << endl;
	return out;
}

template <class T>
inline
istream&  TriangleFacets<T>::_prIn(istream& in)
{  
	__e.set(*this);
	
	LX_Array<Vertex<T> >& vertex = *this;

	if(stream_mode==1 || stream_mode==4 || stream_mode==7)
	{
		int n ,mark = in.tellg();
		in >> n;
		in.seekg(mark);
		vertex.setMax(n+3);
		in >> vertex;
	}
	if(stream_mode==4 || stream_mode==7)
		in >> _edge;
	if(stream_mode==7)
		in >> _triangle;
	if(stream_mode==2 || stream_mode==3 || stream_mode==5 || stream_mode==6)
	{
		// Error message
	}
	return in;
}



template <class T>
inline
ostream& Vertex<T>::_prOut(ostream& out)const 
{
	out << pos() << " ";
//	if(  ) out << dir() << "\n ";
	
	return out;
}
 
template <class T>
inline
istream& Vertex<T>::_prIn(istream& in)
{
	Point<T,3> p;
	in >> p; setPos(p);
//	if(  ) in >> p; setDir(p);
	return in;
}

template <class T>
inline
ostream& Edge<T>::_prOut(ostream& out)const 
{
	out << (*_vertex[0]) << " " << (*_vertex[1]) 
		<< " " << _const << "\n ";
	
	return out;
}
 
template <class T>
inline
istream& Edge<T>::_prIn(istream& in)
{
	Point<T,3> p1, p2;
	in >> p1 >> p2 >> _const;
	_vertex[0] = find(p1);
	_vertex[1] = find(p2);

	return in;
}


template <class T>
inline
ostream& Triangle<T>::_prOut(ostream& out)const 
{
	Array<Vertex<T>*> v=vertices();
	out << (*v[0]) << " " << (*v[1]) << " " << (*v[2]) << " ";
	
	return out;
}


template <class T>
inline
istream& Triangle<T>::_prIn(istream& in)
{ 
	Point<T,3> p1, p2, p3;
	in >> p1 >> p2 >> p3;
	_edge[0] = find(p1,p2);
	_edge[1] = find(p2,p3);

	_edge[2] = find(p3,p1);

	return in;
}

#endif
		

//********************************************************
//**************** Public OpenGL functions   *************
//********************************************************

#if defined(GM_OPENGL_H_)

template <class T>
inline
/* TOG VEK CONST 1/9 -2001 (MARTIN)*/
int  TriangleFacets<T>::initRender() //const
{
  list_name = glGenLists(1);
  glNewList(list_name,GL_COMPILE);
	glBegin(GL_TRIANGLES);
		for(int i=0;i<_triangle.size();i++)
			_triangle(i)->render();
	glEnd();
  glEndList();
  return list_name;
}

template <class T>
inline
/* TOG VEK CONST 1/9 -2001 (MARTIN)*/
void  TriangleFacets<T>::render() //const
{
	if(!list_name) initRender();
	glCallList(list_name);
}

template <class T>
inline
void Triangle<T>::render() const
{
	Array<Vertex<T>*> v = vertices();
	double a = (v[0]->position()[2]+v[1]->position()[2]+v[2]->position()[2])/3/-5 + 0.0;

	glColor4f(0.0, 0.3, a, 1.0);
/* NY KODE FOR TEXTUREBRUK 1/9 -2001 (MARTIN)*/
	glTexCoord2f((v[0]->position()[0])/100, (v[0]->position()[1])/100);	
	glVertex((Arrow<T,3>*)v[0]);
	glTexCoord2f((v[1]->position()[0])/100, (v[1]->position()[1])/100);
	glVertex((Arrow<T,3>*)v[1]);
	glTexCoord2f((v[2]->position()[0])/100, (v[2]->position()[1])/100);
	glVertex((Arrow<T,3>*)v[2]);
}

#endif /* GM_OPENGL_H_ */


} // end namespace GM_lib

#include "GM_Triangle.c"

#endif   /* GM_TRIANGLE_H_ */

