#ifndef _GM_TRANGULOID_TREFOIL_H_
#define _GM_TRANGULOID_TREFOIL_H_

#include <GM_PSurf.h>

namespace GM_lib{

template <class T, int n>
class Tranguloid_Trefoil : public PSurf<T,n>
{
	void _eval(T u, T v, int d1, int d2);

protected:
	T _st_p_u()			{ return T(-M_PI); }
	T _end_p_u()		{ return T(M_PI); }
	T _st_p_v()			{ return T(-M_PI); }
	T _end_p_v()		{ return T(M_PI); }

public:
	Tranguloid_Trefoil();
	Tranguloid_Trefoil(const Tranguloid_Trefoil<T,n>& s);

	bool closedU()   { return true; }
	bool closedV()   { return true; }

	const char* identity() const { return " Tranguloid_Trefoil "  ;}
};




template <class T, int n>
inline
Tranguloid_Trefoil<T,n>::Tranguloid_Trefoil():PSurf<T,n>()
{
	//_r1 = (T) 2.0;
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),4.0));
}

template <class T, int n>
inline
Tranguloid_Trefoil<T,n>::Tranguloid_Trefoil(const Tranguloid_Trefoil<T,n>& s):PSurf<T,n>(s)
{
}

template <class T, int n>
void Tranguloid_Trefoil<T,n>::_eval(T u, T v, int d1,int d2)
{
	__p.setDim(d1+1,d2+1);

	T su = sin(u);
	T cu = cos(u);
	T s2u = sin(2*u);
	T s3u = sin(3*u);
	T c2u = cos(2*u);
	T c3u = cos(3*u);

	T sv = sin(v);
	T cv = cos(v);
	T cvp2 = cv+2;
	T sv23 = sin(v+M_2PI_3);
	T cv23 = cos(v+M_2PI_3);
	T cv23p2 = cv23+2;

	Point<T,3> fu[4];
	Point<T,3> fv[4];

	switch(d1)
	{
	case 3:  fu[3] = Point3D<T>(-54*c3u,-2*(cu+16*c2u), su-16*s2u);
	case 2:  fu[2] = Point3D<T>(-18*s3u,-2*(su+8*s2u), -cu+8*c2u);
	case 1:  fu[1] = Point3D<T>( 6*c3u,  2*(cu+4*c2u), -su+4*s2u);
	default: fu[0] = Point3D<T>( 2*s3u,  2*(su+2*s2u),  cu-2*c2u);
	}

	switch(d2)
	{
	case 3:  fv[3] = Point3D<T>(6*(sv*cv/pow(cvp2,3)+pow(sv,3)/pow(cvp2,4))+sv/(cvp2*cvp2),
								6*(sv23*cv23/pow(cv23p2,3)+pow(sv23,3)/pow(cv23p2,4))+sv23/(cv23p2*cv23p2), 
								(sv*cv23p2+3*cv*sv23+3*sv*cv23+sv23*cvp2)/4);
	case 2:  fv[2] = Point3D<T>(2*sv*sv/pow(cvp2,3)+cv/(cvp2*cvp2),
								2*(sv23*sv23/pow(cv23p2,3))+cv23/(cv23p2*cv23p2),
								-(cv*cv23p2-2*sv*sv23+cvp2*cv23)/4);
	case 1:  fv[1] = Point3D<T>( sv/(cvp2*cvp2),  sv23/(cv23p2*cv23p2), -(sv*cv23p2+cvp2*sv23)/4);
	default: fv[0] = Point3D<T>( 1/cvp2,  1/cv23p2,  cvp2*cv23p2/4);
	}

	for(int i=0; i<=d1;i++)
		for(int j=0; j<=d2; j++)
			__p[i][j] =	   fu[i]%fv[j];

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];
}
} // GM_lib

 #endif //_GM_TRANGULOID_TREFOIL_H_
