
#ifndef GM_WHITNEY_UMBRELLA_H_
#define GM_WHITNEY_UMBRELLA_H_

#include "GM_PSurf.h"

namespace GM_lib{

template <class T, int n>
class Whitney_Umbrella : public PSurf<T,n>
{
	void _eval(T u, T v, int d1, int d2);
	void localDisplay();
	// void localSimulate(double dt);
protected:

	T _st_p_u()			{ return T(-M_PI); }
	T _end_p_u()			{ return T(M_PI); }
	T _st_p_v()			{ return T(-M_PI); }
	T _end_p_v()			{ return T(M_PI); }

public:
	Whitney_Umbrella();
	//Whitney_Umbrella(Whitney_Umbrella<T,n>& s);

	virtual  
	~Whitney_Umbrella() {}
	char* identity() { return " Whitney_Umbrella "  ;}
};
} // GM_lib

using namespace GM_lib;
template <class T, int n>
Whitney_Umbrella<T,n>::Whitney_Umbrella():PSurf<T,n>()
{
	setSurroundingSphere(Sphere<GLfloat,3>(Point3D<GLfloat>(0.0,0.0,0.0),10.0));
	resetPlot(32,32);
}
template <class T, int n>
void Whitney_Umbrella<T,n>::_eval(T u, T v, int d1,int d2)
{

    __p.setDim(4,4);
	__p[0][0][0] =	v*u;
	__p[0][0][1] =	u;
	__p[0][0][2] =	v*v;
	
	if(d1) //u
	{
		__p[1][0][0] =	v;
		__p[1][0][1] =  T(1);
		__p[1][0][2] =	0.0;
	}
	if(d1>1)//uu
	{
		__p[2][0][0] =	0.0;
		__p[2][0][1] =	0.0;
		__p[2][0][2] =	0.0;
	}
	if(d2) //v
	{
		__p[0][1][0] =	u;			
		__p[0][1][1] =	0.0;		
		__p[0][1][2] =	T(2)*v;		
	}
	if(d2>1) //vv
	{
		__p[0][2][0] =	0.0;
		__p[0][2][1] =	0.0;
		__p[0][2][2] =	T(2);
	}
	if(d1 && d2) //uv
	{
		__p[1][1][0] =	T(1);
		__p[1][1][1] =	0.0;
		__p[1][1][2] =	0.0;
	}
	if(d1>1 && d2)//uuv
	{
		__p[2][1][0] =	0.0;
		__p[2][1][1] =	0.0;
		__p[2][1][2] =	0.0;
	}
	if(d1 && d2>1) //uvv
	{
		__p[1][2][0] =	0.0;
		__p[1][2][1] =	0.0;
		__p[1][2][2] =	0.0;
	}
	if(d1>1 && d2>1) //uuvv
	{
		__p[2][2][0] =	0.0;	
		__p[2][2][1] =	0.0;
		__p[2][2][2] =	0.0;
	}		

	if(d1>0 && d2>0 && n==3) __n = Vector3D<T>(__p[1][0])^__p[0][1];

}
template <class T, int n>
void Whitney_Umbrella<T,n>::localDisplay()
{
	// r_mat.mult();
	PSurf<T,n>::localDisplay();
}
 #endif //GM_WHITNEY_UMBRELLA_H_

