#include <GM_Window.h>

using namespace GM_lib;


//////////////////////////////////////////////////////////////////
// Implementation of the GM_Window class private inline functions.
//////////////////////////////////////////////////////////////////

inline
bool GM_Window::_findWindow(int x, int y, int& index)
{
	if(x<0 || x>_w || y<0 || y>_h)		// Outside window
	{
		index = 0;
		_activ.last().reset();
		return false;
	}

	Camera* c;
	if(_activ.last().find(x,y,c))		// Camera found
	{
		index = _camera.index(c);
		return true; 
	}
	else								// border found
	{
		index = -1;
		return false;
	}
}

inline
void GM_Window::_moveBorder(int x, int y)
{
	if(x>0 && x <_w && y>0 && y<_h) 
		(_activ.last()).moveBorder(x,y);
}


/////////////////////////////////////////////////////////////////////////
// Implementation of the WinBorder class private inline member functions.
/////////////////////////////////////////////////////////////////////////

inline
void WinBorder::move(int x, int y)

{ 
	int l_size = l.size(vertical);
	int r_size = r.size(vertical);
	if(vertical)
	{
		l_size = x1-l_size;
		r_size += x2;
		if((x > l_size && x < x1)||(x > x2 && x < r_size))
			d = (double)(x-l_size)/(r_size-l_size);
	}
	else
	{
		l_size = y1-l_size;
		r_size += y2;
		if((y > l_size && y < y1)||(y > y2 && y < r_size))
			d = (double)(y-l_size)/(r_size-l_size);
	}
} 

inline
void WinBorder::prepare(int x_1, int y_1, int x_2, int y_2, Array<WinBorder*>& wb)
{
	wb.insert(this);
	if(vertical)
	{
		x1 = x_1 + int(d*(x_2-x_1)) - 1;
		x2 = x1 + 2;
		y1 = y_1-2;
		y2 = y_2+2;
		l.prepare(x_1,y_1,x1,y_2,wb);
		r.prepare(x2,y_1,x_2,y_2,wb);
	}
	else
	{
		y1 = y_1 + int(d*(y_2-y_1)) - 1;
		y2 = y1 + 2;
		x1 = x_1-2;
		x2 = x_2+2;
		l.prepare(x_1,y_1,x_2,y1,wb);
		r.prepare(x_1,y2,x_2,y_2,wb);
	}
}

inline
ActivTree* WinBorder::hasCamera(Camera* c)
{
	if(l.isCamera(c)) return &r;
	else if(r.isCamera(c)) return &l;
	else return 0;
}

inline
void WinBorder::splitCamera(Camera* cam_to_split, Camera* new_cam, bool vertical, double d)
{
	l.splitCamera(cam_to_split,new_cam,vertical,d);
	r.splitCamera(cam_to_split,new_cam,vertical,d);
}

inline
void WinBorder::removeCamera(Camera* c)
{
	l.removeCamera(c);
	r.removeCamera(c);
}

inline
void WinBorder::find(int x, int y, Camera*& wc, Array<WinBorder*>& wb)
{
	if(x>(x1-5) && x<(x2+5) && y>(y1-5) && y<(y2+5)) wb += this;	
	l.find(x,y,wc,wb);
	r.find(x,y,wc,wb);
}

inline
int WinBorder::size(bool _vertical)
{
	if(_vertical == vertical)	return l.size(_vertical) + r.size(_vertical);
	else						return min(l.size(_vertical),r.size(_vertical));
}

inline
void WinBorder::display()
{
	glRecti(x1,y1,x2,y2);
}

inline
void WinBorder::corr(Array<WinBorder*>& nw, const Array<WinBorder*>& ow)
{
	l.corr(nw,ow);
	r.corr(nw,ow);
}


/////////////////////////////////////////////////////////////////////////
// Implementation of the ActivTree class private inline member functions.
/////////////////////////////////////////////////////////////////////////

inline
void ActivTree::split(Camera* new_cam, bool vertical, double d)
{
	if(c)
	{
		b = new WinBorder(c,new_cam,vertical,d);
		c = 0;
	}
	else
	{
		b = 0;
		c = new_cam;
	}
}

inline
int ActivTree::size(bool vertical)
{
	if(b) return b->size(vertical);
	else
	{	
		int w1,w2,h1,h2;
		c->viewPort(w1,w2,h1,h2);
		if(vertical)	return w2-w1;
		else			return h2-h1;
	}

}

inline
void ActivTree::corr(Array<WinBorder*>& nw, const Array<WinBorder*>& ow)
{
	if(b) b = nw[ow.index(b)];
}


/////////////////////////////////////////////////////////////////////////
// Implementation of the ActivSet class private inline member functions.
/////////////////////////////////////////////////////////////////////////

inline
void ActivSet::prepare(int wi, int hi)
{
	w = wi; h = hi;
	b.resetSize();
	root.prepare(0,0,w,h,b);
}

inline
void ActivSet::drawBorder()
{
	if(b.size()>0)
	{
		glDisable(GL_LIGHTING);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0,w,0,h,-1,1);
		glMatrixMode(GL_MODELVIEW);
		glViewport(0,0,w,h);
		glColor(border_color);
		for(int i=0; i< b.size(); i++) b[i]->display();
		glEnable(GL_LIGHTING);
	}
	
}

//inline
void ActivSet::drawCamera(bool stereo)
{
	drawBorder();
	for(int i=0; i< c.size(); i++) c[i]->go(stereo);
}

inline
bool ActivSet::insertCamera(Camera* cam_to_split, Camera* new_cam, bool vertical, double d)
{
	if(c.exist(new_cam))
		return false;
	else if(!c.exist(cam_to_split)) 
		return false;
	else
	{
		c += new_cam;
		if(c.size()>2)
			root.splitCamera(cam_to_split,new_cam,vertical,d);
		else
			root.split(new_cam,vertical,d);
		b.resetSize();
		root.prepare(0,0,w,h,b);
		return true;
	}
}

inline
void ActivSet::removeCamera(Camera* ca)
{
	if(c.size()>1)
	{
		int i = c.index(ca);
		if(i>=0) removeCamera(i);	
	}
}

inline
void ActivSet::removeCamera(int i)
{
	if(c.size()>1)
	{
		root.removeCamera(c[i]);
		c.removeIndex(i);
		prepare(w,h);
	}
}

inline
void ActivSet::moveBorder(int x, int y)
{
	for(int i=0; i<_selected.size(); i++)
		_selected[i]->move(x,y);
	prepare(w,h);
}

inline
bool ActivSet::find(int x, int y, Camera*& wc)
{
	wc=0;
	_selected.resetSize();
	root.find(x,y,wc,_selected);
	if(_selected.size() >0)		return false;
	else						return true;
}


///////////////////////////////////////////////////////////
// Implementation of the GM_Window virtual _init functions.
///////////////////////////////////////////////////////////

void GM_Window::_init()
{
//	glClearColor(0.0,0.0,0.0,1.0);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,GL_TRUE);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_NORMALIZE);
	// glEnable(GL_ALPHA_TEST);
	// glAlphaFunc(GL_GREATER,0.1);
	// glEnable(GL_BLEND);
	// glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	if(_sun)
		_sun->scaleDayLight(1.0);
	insertCamera(new Camera(Point3D<GLfloat>(10,10,5),Point3D<GLfloat>(-10,-10,-5),Vector3D<GLfloat>(0,0,-1)));
	_activ += ActivSet(_camera[0]);
	_activ.last().prepare(_w,_h);
	_default_only = true;
	int numberoflights;
	glGetIntegerv(GL_MAX_LIGHTS,&numberoflights);
	char s[32];
	sprintf(s,"Max number of lights %d\n", numberoflights);
	_message(s);
	char *extensions = NULL;
    extensions = (char *)glGetString(GL_EXTENSIONS);
	_message(extensions);
}

////////////////////////////////////////////////////////////////////////
// Default implementation of the GM_Window virtual _toggleRun functions.
////////////////////////////////////////////////////////////////////////


bool GM_Window::_toggleRun()
{
	return _running = !_running;
}

/////////////////////////////////////////////////////////////////////////////////////
// Implementation of the GM_Window standard user interface mouse/keyboard functions.
/////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////////////
// All mouse double click actions are reserved camera set operations, and are therfore not overloadable.
////////////////////////////////////////////////////////////////////////////////////////////////////////

void GM_Window::_mouseDoubleClick(int button, int x, int y) 
{
	if(_findWindow(x,_h-y-1,_i) && _activ.size()>1)	// Window is found and there is not only default camera
	{
		if(_modkey == GM_SHIFT_BUTTON && button == GM_RIGHT_BUTTON)
		{			// Insert a new Camera in current window, vertical
			Camera* cam;
			Point3D<GLfloat>  pos = _camera[_i]->getPos();
			Point3D<GLfloat>  tar = cleanSphere().pos();
			Vector3D<GLfloat> up  = _camera[_i]->getUp();
			pos += ((pos-tar).length()*2)*_camera[_i]->getDir();
			insertCamera(cam=new Camera(pos, tar-pos, up), true);
			cam->setCutPlane(1,8000);
			setActiv(_camera.index(cam),_i,true);
			_message("Insert new Camera");
		}		// Insert a new Camera in current window, horisontal
		else if(_modkey == GM_SHIFT_BUTTON && button == GM_LEFT_BUTTON)
		{
			Camera* cam;
			Point3D<GLfloat>  pos = _camera[_i]->getPos();
			Point3D<GLfloat>  tar = cleanSphere().pos();
			Vector3D<GLfloat> up  = _camera[_i]->getUp();
			pos += ((pos-tar).length()*2)*_camera[_i]->getDir();
			insertCamera(cam=new Camera(pos, tar-pos, up), true);
			cam->setCutPlane(1,8000);
			setActiv(_camera.index(cam),_i,false);
			_message("Insert new Camera");
		}
		else if(_modkey == GM_CONTROL_BUTTON && 
				(button == GM_MIDDLE_BUTTON || (button == GM_LEFT_BUTTON && _isbig)) &&
				_activ.size()>2)	// Pop last Activ Set
		{
			_activ.pop();
			_activ.last().prepare(_w,_h);
			_isbig = false;
		}
		else if(_modkey == GM_CONTROL_BUTTON && button == GM_LEFT_BUTTON && _activ.last().size()>1)
		{						// Push new Activ Set camera with found camer as only member
			_activ += ActivSet(_camera[_i]);
			_activ.last().prepare(_w,_h);
			_isbig = true;
		}
		else if(_modkey == GM_CONTROL_BUTTON && button == GM_RIGHT_BUTTON && _activ.last().size()>1)
		{						// push new Activ Set and remove found camera from the activ set
			_activ.push();
			_activ.last().removeCamera(_camera[_i]); // removeCamera do also prepare
		}
	}
}


void GM_Window::_mouseButtonDown(int button, int x, int y)
{
	if(_mousebutton==GM_BUTTON_UP)
	{
		y = _h-y-1;									// Coordinate transformation.
		if(_running = !isStopped())	stopTime();		// Threads?? or userfriendly????
		_mousebutton = button;						// Necessary in mousemotion

		if(_findWindow(x,y,_i))						// Window connected to camera[i] is found
		{
			switch(_mousebutton)
			{
			case GM_LEFT_BUTTON:	
				switch(_modkey )
				{
				case GM_CONTROL_BUTTON:				// ctrl + left
					{
						DisplayObject*sel=_camera[_i]->findSelectObject(x,y,10); // type_id == 10, Only selectors
						if (sel) 
						{
							if (sel->flipSelected()) _SelArr.insert(sel); else _SelArr.remove(sel);
							_message(sel->identity());
						}
					}
					break;
				case GM_SHIFT_BUTTON:				// shift + left
					{
						DisplayObject*sel=_camera[_i]->findSelectObject(x,y,-10);	// Selecting all objects except selectors
						if (sel) 
						{
							if (sel->flipSelected()) _SelArr.insert(sel); else _SelArr.remove(sel);
							_message(sel->identity());
							_message(sel);
						}
					}
					break;
				case GM_ALT_BUTTON:					// alt + left
					{
						DisplayObject*sel=_camera[_i]->findSelectObject(x,y,_sel_id); // type_id == _sel_id, Only specified objects
						if (sel) 
						{
							if (sel->flipSelected()) _SelArr.insert(sel); else _SelArr.remove(sel);
							_message(sel->identity());
						}
					}
					break;
				default:							// only left
					break;
				} // End switch _modkey
				break;
			case GM_MIDDLE_BUTTON:
				switch(_modkey )
				{
				case GM_CONTROL_BUTTON:				// ctrl + middle
					_rot_obj = _camera[_i]->findSelectObject(x,y,-10);
					if(_rot_obj)	_message("Locked for rotating!");
					break;
				case GM_SHIFT_BUTTON:				// shift + middle
					break;
				case GM_ALT_BUTTON:					// alt + middle
					break;
				default:							// only middle
					break;
				} // End switch _modkey
				break;
			case GM_RIGHT_BUTTON:
				switch(_modkey )
				{
				case GM_CONTROL_BUTTON:				// ctrl + right
					break;
				case GM_SHIFT_BUTTON:				// shift + right
					break;
				case GM_ALT_BUTTON:					// alt + right
					break;
				default:							// only right
					_target = _camera[_i]->lockAtTargetAtPixel(x,y);
					if(_target) 
						_message("Locked at target!");
					else 
					{
						if(_camera[_i]->isLocked())
						{
							_camera[_i]->unLockTarget();
							_message("Unlock target");
						}
						else
						{
							_camera[_i]->lockAtTarget((cleanSphere().pos()-_camera[_i]->getPos())*_camera[_i]->getDir());
							_message("Locked at Point");
						}
					}
					break;
				} // End switch _modkey
				break;
			default:
				break;
			}//end switch _mousebutton
		}
		else if(_i<0)		// Borders are found
		{
			if(button == GM_LEFT_BUTTON) _border = true;				// Start translalting border
			_i=0;
		}
		_prevx=x; _prevy=y;		// Store previous position
	}
}


void GM_Window::_mouseMotion(int x, int y)
{
	y = _h-y-1;				// Coordinate transformation.
	int i;

	switch(_mousebutton)
	{
	case GM_LEFT_BUTTON:
		switch(_modkey )
		{
		case GM_CONTROL_BUTTON:				// ctrl + left
			for(i=0;i<_SelArr.size();i++)
			{
				LA_DisplayObject * tmp=dynamic_cast<LA_DisplayObject *>(_SelArr[i]);
				if (tmp && tmp->typeId()==10) 
				{
					double dh = _camera[_i]->deltaTranslate(tmp);	
					Vector3D<float> deltav(((_prevx-x)*dh)*_camera[_i]->getSide() + ((y-_prevy)*dh)*_camera[_i]->getUp());
						if(deltav.length()>0.01 && deltav.length()<1000.0)
						{		
							tmp->editPos(deltav);
							// send ut koordinater
							_message(tmp->getPos());
						}
				}
				// else _message(_SelArr[i]->identity());
			}
			break;
		case GM_SHIFT_BUTTON:				// shift + left
			for(i=0;i<_SelArr.size();i++)
			{
				LA_DisplayObject * tmp=dynamic_cast<LA_DisplayObject *>(_SelArr[i]);
				if (tmp && tmp->typeId()!=10) 
				{
					double dh = _camera[_i]->deltaTranslate(tmp);
					Vector3D<float> deltav(((_prevx-x)*dh)*_camera[_i]->getSide() + ((y-_prevy)*dh)*_camera[_i]->getUp());
					if(deltav.length()>0.01 && deltav.length()<1000.0)
					{
					 tmp->translateGlobal(deltav);
					 _message(tmp->getPos());
					}
				}
				// else _message(_SelArr[i]->identity());
			}
			break;
		case GM_ALT_BUTTON:					// alt + left
			break;
		default:							// only left
			if(_border)		_moveBorder(x,y);
			else							// Move camera (if locked, rotating)
			{
				GLfloat scale;				// Only to unify the speed of the camera
				if(_camera[_i]->isLocked()) scale = M_2PI*_camera[_i]->getLockDist();
				else						scale = Scene::sphere().radius();
				_camera[_i]->move(Vector2D<GLfloat>((x-_prevx)*scale/_camera[_i]->viewPortW(),(_prevy-y)*scale/_camera[_i]->viewPortH()));
			}
			break;
		}  // End switch _modkey
		break;
	case GM_MIDDLE_BUTTON:
		switch(_modkey )
		{
		case GM_CONTROL_BUTTON:				// ctrl + middle
			if(_rot_obj)
			{
				UnitVector3D<float> rot_v = float(x-_prevx)*_camera[_i]->getUp()-float(_prevy - y)*_camera[_i]->getSide();
				Angle ang(M_2PI*sqrt(pow(double(x-_prevx)/_camera[_i]->viewPortW(),2)+pow(double(_prevy - y)/_camera[_i]->viewPortH(),2)));
				_rot_obj->rotate(ang,rot_v);
			}
			break;
		case GM_SHIFT_BUTTON:				// shift + middle
			for(i=0;i<_SelArr.size();i++)
			{
				LA_DisplayObject * tmp=dynamic_cast<LA_DisplayObject *>(_SelArr[i]);
				if (tmp) tmp->translateGlobal((double(_prevy - y)/_camera[_i]->viewPortH())*(tmp->getPos()-_camera[_i]->getPos()));
			}
			break;
		case GM_ALT_BUTTON:					// alt + middle
			break;
		default:							// only middle
			{
				double scale;
				if(_camera[_i]->isLocked()) scale = _camera[_i]->getLockDist();
				else						scale = sphere().radius();
				_camera[_i]->move((_prevy - y)*scale/_camera[_i]->viewPortH());
			}
			break;
		} // End switch _modkey
		break;  
	case GM_RIGHT_BUTTON:
		switch(_modkey )
		{
		case GM_CONTROL_BUTTON:				// ctrl + right
			break;
		case GM_SHIFT_BUTTON:				// shift + right
			break;
		case GM_ALT_BUTTON:					// alt + right
			break;
		default:							// only right
			break;
		}
		break;
	default:
		break;
	}	//end switch _mousebutton

	_prevx=x; _prevy=y;
	_display();
}


void GM_Window::_mouseButtonUp(int button, int x, int y)
{
	y = _h-y-1;				// Coordinate transformation.

	switch(button)
	{
	case GM_LEFT_BUTTON:
		_border = false;
		break;
	case GM_MIDDLE_BUTTON:
		if(_rot_obj)	_rot_obj = 0;
		break;
	case GM_RIGHT_BUTTON:
		break;
	default:
		break;
	}	//end switch button

	_mousebutton=GM_BUTTON_UP;
	if(_running) startTime(); //Starting the time if simulate
	_display();
}


bool GM_Window::_keyboard (unsigned char key, int x, int y)
{
	y = _h-y-1;		// Coordinate transformation.

	if(_running = !isStopped())	stopTime();

	switch (key) {
	   case '+':	_keyDef_PLUSS(x, y);
		   break; 
	   case '-':	_keyDef_MINUS(x, y);
		   break; 
	   case '0':	_keyDef_NULL(x, y);
		   break;
	   case 'a':	_keyDef_a(x, y);
			break;
	   case 'A':	_keyDef_A(x, y);
			break;
	   case 'b':	_keyDef_b(x, y);
			break;
	   case 'B':	_keyDef_B(x, y);
			break;
	   case 'c':	_keyDef_c(x, y);
			break;
	   case 'C':	_keyDef_C(x, y);
			break;
	   case 'd':	_keyDef_d(x, y);
			break;
	   case 'D':	_keyDef_D(x, y);
			break;
	   case 'e':	_keyDef_e(x, y);
			break;
	   case 'E':	_keyDef_E(x, y);
			break;
	   case 'f':	_keyDef_f(x, y);
		   break;
	   case 'F':	_keyDef_F(x, y);
		   break;
	   case 'g':	_keyDef_g(x, y);
			break;
	   case 'G':	_keyDef_G(x, y);
			break;
	   case 'h':	_keyDef_h(x, y);
			break;
	   case 'H':	_keyDef_H(x, y);
			break;
	   case 'i':	_keyDef_i(x, y);
			break;
	   case 'I':	_keyDef_I(x, y);
			break;
	   case 'j':	_keyDef_j(x, y);
			break;
	   case 'J':	_keyDef_J(x, y);
			break;
	   case 'k':	_keyDef_k(x, y);
			break;
	   case 'K':	_keyDef_K(x, y);
			break;
	   case 'l':	_keyDef_l(x, y);
		   break;
	   case 'L':	_keyDef_L(x, y);
		   break;
	   case 'm':	_keyDef_m(x, y);
		   break;
	   case 'M':	_keyDef_M(x, y);
		   break;
	   case 'n':	_keyDef_n(x, y);
		   break;
	   case 'N':	_keyDef_N(x, y);
		   break;
	   case 'o':	_keyDef_o(x, y);
		   break;
	   case 'O':	_keyDef_O(x, y);
		   break;
	   case 'p':	_keyDef_p(x, y);
		   break;
	   case 'P':	_keyDef_P(x, y);
		   break;
	   case 'q':	_keyDef_q(x, y);
			break;
	   case 'Q':	_keyDef_Q(x, y);
			break;
	   case 'r':	_keyDef_r(x, y);
		   break;
	   case 'R':	_keyDef_R(x, y);
		   break;
	   case 's':	_keyDef_s(x, y);
		   break;
	   case 'S':	_keyDef_S(x, y);
		   break;
	   case 't':	_keyDef_t(x, y);
		   break;
	   case 'T':	_keyDef_T(x, y);
		   break;
	   case 'u':	_keyDef_u(x, y);
			break;
	   case 'U':	_keyDef_U(x, y);
			 break;
	   case 'v':	_keyDef_v(x, y);
			break;
	   case 'V':	_keyDef_V(x, y);
			 break;
	   case 'w':	_keyDef_w(x, y);
		   break;
	   case 'W':	_keyDef_W(x, y);
		   break;
	   case 'x':	_keyDef_x(x, y);
		   break;
	   case 'X':	_keyDef_X(x, y);
		   break;
	   case 'y':	_keyDef_y(x, y);
		   break;
	   case 'Y':	_keyDef_Y(x, y);
		   break;
	   case 'z':	_keyDef_z(x, y);
		   break;
	   case 'Z':	_keyDef_Z(x, y);
		   break;
	   default:
		   return false;
	}

	if(_running) startTime();
	return true;
}


bool GM_Window::_specKeyboard(int speckey, int x, int y)
{
	y = _h-y-1;		// Coordinate transformation.

	if(_running = !isStopped())	stopTime();

	switch (speckey) {
		case GM_KEY_F1:		_keyDef_F1(x, y);
			break;
		case GM_KEY_F2:		_keyDef_F2(x, y);
			break;
		case GM_KEY_F3:		_keyDef_F3(x, y);
			break;
		case GM_KEY_F4:		_keyDef_F4(x, y);
			break;
		case GM_KEY_F5:		_keyDef_F5(x, y);
			break;
		case GM_KEY_F6:		_keyDef_F6(x, y);
			break;
		case GM_KEY_F7:		_keyDef_F7(x, y);
			break;
		case GM_KEY_F8:		_keyDef_F8(x, y);
			break;
		case GM_KEY_F9:		_keyDef_F9(x, y);
			break;
		case GM_KEY_F10:	_keyDef_F10(x, y);
			break;
		case GM_KEY_F11:	_keyDef_F11(x, y);
			break;
		case GM_KEY_F12:	_keyDef_F12(x, y);
			break;
		case GM_KEY_LEFT:	_keyDef_LEFT(x, y);
			break;
		case GM_KEY_UP:		_keyDef_UP(x, y);
			break;
		case GM_KEY_RIGHT:	_keyDef_RIGHT(x, y);
			break;
		case GM_KEY_DOWN:	_keyDef_DOWN(x, y);
			break;
		case GM_KEY_PAGE_UP:	_keyDef_PAGE_UP(x, y);
			break;
		case GM_KEY_PAGE_DOWN:	_keyDef_PAGE_DOWN(x, y);
			break;
		case GM_KEY_HOME:	_keyDef_HOME(x, y);
			break;
		case GM_KEY_END:	_keyDef_END(x, y);
			break;
		case GM_KEY_INSERT:	_keyDef_INSERT(x, y);
			break;
		default:
			return false;
   }	// End switch speckey

   if(_running) startTime();
   return true;
}
	
//////////////////////////////////////////////////////////////////////
// Implementation of the GM_Window class public member functions.
//////////////////////////////////////////////////////////////////////

GM_Window::GM_Window()
{
	_shaded	 = true;
	_stereo	= false;
	_border	= false;
	_running = false;
	_isbig	 = false;
	_prevx = _prevy = _i = 0;		// old x and old y
	_rot_obj = NULL;				// rotating object
	_target  = NULL;
	_sun	 = NULL;
	_move	 = 0.0;
	_viewmode = GM_EXAMINE;	
	stopTime();
}

GM_Window::GM_Window(const GM_Window& gw)
{
	(*this) = gw;
	_mousebutton=GM_BUTTON_UP;
}

GM_Window::~GM_Window()
{
	int i;
	if(_sun) delete _sun;
	for(i=0; i<_camera.size(); i++) 
	{
		remove(_camera[i]);
		delete _camera[i];
	}
	Light * light;
	for(i=_light.size()-1; i>=0; i=_light.size()-1)
	{
		light = _light.last();
		removeLight(light);
		delete light;
	}
}

GM_Window& GM_Window::operator=(const GM_Window& gw)
{
	Scene::operator=(gw);
	_camera	= gw._camera;
	_light	= gw._light;
	_sun	= gw._sun;
	for(int i=0; i<_camera.size(); i++) _camera[i]->changeScene(this);
	_shaded	 = true;
	_stereo	= false;
	_border	= false;
	_running = false;
	_isbig	 = false;
	_SelArr.clear();
	_i = _prevx = _prevy = 0;		// Index of activ camara, old x and y
	_rot_obj = 0;					// rotating object
	_activ = gw._activ;				// Active camera set
	_reshape(_w,_h);
	return *this;
}

void GM_Window::insertCamera(Camera * cam, bool insert_in_scene )
{
	cam->changeScene(this);
	_camera += cam;
	if(insert_in_scene)	insert(cam);
}

bool GM_Window::removeCamera(Camera * cam)
{
	if(_camera.size() < 2) return false;

	int i = _camera.index(cam);
	if(i>0)
	{
		for(int j=1; j<_activ.size(); j++)	_activ[j].removeCamera(_camera[_i]);
		remove(cam);
		_camera.removeIndex(i);
		return true;
	}
	return false;
}

void GM_Window::insertLight(Light * light, bool insert_in_scene )
{
	_light += light;
	light->on();
	DisplayObject* obj = dynamic_cast<DisplayObject*>(light);
	if(insert_in_scene && obj)	insert(obj);
}

bool GM_Window::removeLight(Light * light)
{
	int i = _light.index(light);
	if(i>=0)
	{
		DisplayObject* obj = dynamic_cast<DisplayObject*>(light);
		if(obj)	remove(obj);
		_light.removeIndex(i);
		light->off();
		return true;
	}
	return false;
}

void GM_Window::setActiv(int new_c, int old_c, bool vertical, double d)
{
	if(new_c<1 || new_c>=_camera.size() || old_c <1 || old_c>=_camera.size() || d<=0 || d>=1)
	{
		cout << "Error in camera index!!!\n";
		return;
	}
	if(_default_only)	
	{
		_activ += ActivSet(_camera[new_c]);
		_default_only = false;
	}
	else
		_activ.last().insertCamera(_camera[old_c],_camera[new_c],vertical,d);
	_activ.last().prepare(_w,_h);
}

void GM_Window::removeActiv(int i)
{
	_activ.last().removeCamera(i);
}

void GM_Window::scaleDayLight(double d)
{
	_sun->scaleDayLight(d);
}

bool GM_Window::stereoToggle()
{
	_stereo = !_stereo;
	return _stereo;
}

//////////////////////////////////////////////////////////////////////
// Implementation of the ActivSet class public member functions.
//////////////////////////////////////////////////////////////////////

ActivSet::ActivSet(Camera* cam):root(cam)
{
	if(cam)	c += cam;
	border_color = White;
}

ActivSet::ActivSet(const ActivSet& a)
{
	int i;
	c = a.c;
	w = a.w;
	h = a.h;
	border_color = a.border_color;
	root = a.root;
	for(i=0; i<a.b.size(); i++)
		b += new WinBorder(*(a.b(i)));
	root.corr(b,a.b);
	for(i=0; i<b.size(); i++)
		b[i]->corr(b,a.b);
	prepare(w,h);
}

ActivSet::~ActivSet()
{
	for(int i=0; i<b.size(); i++)	delete b[i];
} 

ActivSet& ActivSet::operator=(const ActivSet& a)
{
	int i;
	for(i=0; i<b.size(); i++) delete b[i];
	b.resetSize();
	c = a.c;
	w = a.w;
	h = a.h;
	border_color = a.border_color;
	root = a.root;
	for(i=0; i<a.b.size(); i++)
		b += new WinBorder(*(a.b(i)));
	for(i=0; i<b.size(); i++)
		b[i]->corr(b,a.b);
	root.corr(b,a.b);
	prepare(w,h);
	return *this;
}

//////////////////////////////////////////////////////////////////////
// Implementation of the ActivTree class recurcive member functions.
//////////////////////////////////////////////////////////////////////

void ActivTree::prepare(int x1, int y1, int x2, int y2, Array<WinBorder*>& wb)
{ 
	if(c) 
		c->reshape(x1,y1,x2,y2);
	else
		b->prepare(x1,y1,x2,y2,wb); 
}

void ActivTree::splitCamera(Camera* cam_to_split, Camera* new_cam, bool vertical, double d)
{
	if(c == cam_to_split)
		split(new_cam,vertical,d);
	else if(b)
		b->splitCamera(cam_to_split,new_cam,vertical,d);
}

void ActivTree::removeCamera(Camera* ca)
{
	if(b)
	{
		ActivTree* a = b->hasCamera(ca);
		if(a)
		{
			WinBorder* wb=b;
			(*this) = (*a);
			delete wb;
		}
		else	b->removeCamera(ca);
	}
}

void ActivTree::find(int x, int y, Camera*& wc, Array<WinBorder*>& wb)
{
	if(b) b->find(x,y,wc,wb);
	else
	{	
		int w1,w2,h1,h2;
		c->viewPort(w1,w2,h1,h2);
		if(x>=w1 && x<=w2 && y>=h1 && y<=h2) wc=c;
	}
}



