//********************************************************
// GM_Window.h: interface for the GM_Window class.
//********************************************************
#pragma once

#include <GM_Camera.h>
#include <GM_Sun.h>
#include <GM_Material.h>

namespace GM_lib {

class GM_Window;
class ActivSet;
class ActivTree;
class WinBorder;

enum {	GM_LEFT_BUTTON, GM_MIDDLE_BUTTON, GM_RIGHT_BUTTON,GM_BUTTON_UP};
enum {	GM_NO_BUTTON, GM_SHIFT_BUTTON, GM_CONTROL_BUTTON, GM_ALT_BUTTON};
enum {	GM_KEY, GM_KEY_F1, GM_KEY_F2, GM_KEY_F3, GM_KEY_F4, GM_KEY_F5, GM_KEY_F6,
		GM_KEY_F7, GM_KEY_F8, GM_KEY_F9, GM_KEY_F10, GM_KEY_F11, GM_KEY_F12,
		GM_KEY_LEFT = 100, GM_KEY_UP, GM_KEY_RIGHT, GM_KEY_DOWN,
		GM_KEY_PAGE_UP, GM_KEY_PAGE_DOWN, GM_KEY_HOME, GM_KEY_END, 
		GM_KEY_INSERT, GM_KEY_DELETE};
enum {	GM_EXAMINE, GM_FLYING};

//
// The Data structure for Window splitting and organization.
//

class ActivTree
{
	Camera*		c;
	WinBorder*	b;

	friend class ActivSet;
	void split(Camera* new_cam, bool vertical, double d);								// Splitting this in two

	friend class WinBorder;
	void prepare(int x1, int y1, int x2, int y2, Array<WinBorder*>& wb);
	void corr(Array<WinBorder*>& nw, const Array<WinBorder*>& ow);
	void find(int x, int y, Camera*& c, Array<WinBorder*>& b);
	bool isCamera(Camera* ca)	{ return c == ca; }
	void removeCamera(Camera* ca);
	int  size(bool vertical=true);
	void splitCamera(Camera* cam_to_split, Camera* new_cam, bool vertical, double d);	// Recurciv seek to find object to split

public:
	ActivTree(Camera* ca=0)			{ c=ca;  b=0; }
	ActivTree(WinBorder* ba)		{ c=0;   b=ba; }
	ActivTree(const ActivTree& a)	{ c=a.c; b=a.b; }
};

class WinBorder
{
	bool			vertical;		// True if border is vertical
	double			d;
	int				x1, x2, y1, y2;	// Pixel-index of the border
	ActivTree		l;
	ActivTree		r;

	friend class ActivSet;
	void corr(Array<WinBorder*>& nw, const Array<WinBorder*>& ow);
	bool isVertical() { return vertical; }
	void move(int x, int y);
	void display();

	friend class ActivTree;
	void prepare(int x_1, int y_1, int x_2, int y_2, Array<WinBorder*>& wb);
	ActivTree* hasCamera(Camera* c);
	void splitCamera(Camera* cam_to_split, Camera* new_cam, bool vertical, double d);
	void removeCamera(Camera* c);
	void find(int x, int y, Camera*& c, Array<WinBorder*>& b);
	int  size(bool vertical=true);

public:
	WinBorder(Camera* cam_to_split, Camera* new_cam, bool v, double r):vertical(v),d(r),l(cam_to_split),r(new_cam) {}
	WinBorder(const WinBorder& w):vertical(w.vertical),d(w.d),l(w.l),r(w.r) {}
};

class ActivSet
{
	int					w,h;	// Viewport size
	Array<Camera*>		c;
	Array<WinBorder*>	b;
	ActivTree			root;
	GL_Color			border_color;

	Array<WinBorder*>	_selected;

	friend class GM_Window;

	void setBorderColor(const GL_Color& bc) { border_color = bc;}
	void prepare(int w, int h);
	void drawBorder();
	void drawCamera(bool stereo=false);
	bool insertCamera(Camera* cam_to_split, Camera* new_cam, bool vertical=true, double d=0.5);
	void removeCamera(Camera*);
	void removeCamera(int i);
	void moveBorder(int x, int y);
	Camera* operator[](int i)	{ return c[i]; }
	int  size()					{ return c.size(); }
	bool find(int x, int y, Camera*& c);
	void reset()				{ _selected.resetSize(); }

public:
	ActivSet(Camera* cam=0);
	ActivSet(const ActivSet& a);

	~ActivSet();

	ActivSet& operator=(const ActivSet&);
	bool operator<(const ActivSet& a)const { return true; }	// dummy
};

///////////////////////////////////////////////////////////////////////////
// The GM_Window, base class for real windowsystems. The window is a scene.
///////////////////////////////////////////////////////////////////////////

class GM_Window : public Scene
{
	bool			_stereo;

	bool _findWindow(int x, int y, int& index);
	void _moveBorder(int x, int y);

	DisplayObject*	_target;		// NB!!!! take a look at this variable not used proper today.....
	DisplayObject*	_rot_obj;		// Storing Rotating object from pressed down to move, restored in butten up.
	int				_modkey;		// State for mouse/keyboard to see if any modkey are pressed
	int				_viewmode;		// State for Camera moving in plane, the two states acting opposit
	int				_mousebutton;	// Storing mousebutton from pressed down to move, restored in butten up.
	bool			_shaded;		// State for shadeing/wireframe on surfaces
	bool			_running;		// Used to stor the state of simulation while mouse/keboard temporary turn off simulation  
	bool			_isbig;			// State of one window functionality have been used (see _mouseDoubleClick on right knob)
	bool			_border;		// Moving border
	bool			_default_only;	// Only default camera present
	Array<ActivSet>	_activ;			// Active camera set
	
	

protected:
	Array<Camera*>	_camera;
	Array<Light*>	_light;
	Sun*			_sun;
	int				_w, _h;					// GM_Window size in viewport coordinates.
	int				_i, _prevx, _prevy;		// Index of activ camara, and old x and y

	Array<DisplayObject *>  _SelArr;		// BB
	int						_sel_id;		// Id to alt-select type
	double					_move;			// JG

	virtual void _init();
	virtual void _swapBuffers() {}
	virtual bool _toggleRun();

	void _mouseDoubleClick(int button, int x, int y);
	void _mouseButtonDown(int button, int x, int y);
	void _mouseButtonUp(int button, int x, int y);
	void _mouseMotion(int x, int y);
	bool _keyboard (unsigned char key, int x, int y);
	bool _specKeyboard( int speckey, int x, int y);				//JG

	void _setModKey(int mk)			{ _modkey = mk; }			//JG		
	void _display(void);
	void _reshape(int w, int h);

	virtual void _message( const std::string& str) {}
	virtual void _message( DisplayObject* d) {}
	virtual void _message( Point<float,3>& p) {}

	virtual void _keyDef_PLUSS(int x, int y);
	virtual void _keyDef_MINUS(int x, int y);
	virtual void _keyDef_NULL(int x, int y);
	virtual void _keyDef_a(int x, int y);
	virtual void _keyDef_A(int x, int y);
	virtual void _keyDef_b(int x, int y);
	virtual void _keyDef_B(int x, int y);
	virtual void _keyDef_c(int x, int y);
	virtual void _keyDef_C(int x, int y);
	virtual void _keyDef_d(int x, int y);
	virtual void _keyDef_D(int x, int y);
	virtual void _keyDef_e(int x, int y);
	virtual void _keyDef_E(int x, int y);
	virtual void _keyDef_f(int x, int y);
	virtual void _keyDef_F(int x, int y);
	virtual void _keyDef_g(int x, int y);
	virtual void _keyDef_G(int x, int y);
	virtual void _keyDef_h(int x, int y);
	virtual void _keyDef_H(int x, int y);
	virtual void _keyDef_i(int x, int y);
	virtual void _keyDef_I(int x, int y);
	virtual void _keyDef_j(int x, int y);
	virtual void _keyDef_J(int x, int y);
	virtual void _keyDef_k(int x, int y);
	virtual void _keyDef_K(int x, int y);
	virtual void _keyDef_l(int x, int y);
	virtual void _keyDef_L(int x, int y);
	virtual void _keyDef_m(int x, int y);
	virtual void _keyDef_M(int x, int y);
	virtual void _keyDef_n(int x, int y);
	virtual void _keyDef_N(int x, int y);
	virtual void _keyDef_o(int x, int y);
	virtual void _keyDef_O(int x, int y);
	virtual void _keyDef_p(int x, int y);
	virtual void _keyDef_P(int x, int y);
	virtual void _keyDef_q(int x, int y);
	virtual void _keyDef_Q(int x, int y);
	virtual void _keyDef_r(int x, int y);
	virtual void _keyDef_R(int x, int y);
	virtual void _keyDef_s(int x, int y);
	virtual void _keyDef_S(int x, int y);
	virtual void _keyDef_t(int x, int y);
	virtual void _keyDef_T(int x, int y);
	virtual void _keyDef_u(int x, int y);
	virtual void _keyDef_U(int x, int y);
	virtual void _keyDef_v(int x, int y);
	virtual void _keyDef_V(int x, int y);
	virtual void _keyDef_w(int x, int y);
	virtual void _keyDef_W(int x, int y);
	virtual void _keyDef_x(int x, int y);
	virtual void _keyDef_X(int x, int y);
	virtual void _keyDef_y(int x, int y);
	virtual void _keyDef_Y(int x, int y);
	virtual void _keyDef_z(int x, int y);
	virtual void _keyDef_Z(int x, int y);
	virtual void _keyDef_F1(int x, int y);
	virtual void _keyDef_F2(int x, int y);
	virtual void _keyDef_F3(int x, int y);
	virtual void _keyDef_F4(int x, int y);
	virtual void _keyDef_F5(int x, int y);
	virtual void _keyDef_F6(int x, int y);
	virtual void _keyDef_F7(int x, int y);
	virtual void _keyDef_F8(int x, int y);
	virtual void _keyDef_F9(int x, int y);
	virtual void _keyDef_F10(int x, int y);
	virtual void _keyDef_F11(int x, int y);
	virtual void _keyDef_F12(int x, int y);
	virtual void _keyDef_LEFT(int x, int y);
	virtual void _keyDef_UP(int x, int y);
	virtual void _keyDef_RIGHT(int x, int y);
	virtual void _keyDef_DOWN(int x, int y);
	virtual void _keyDef_PAGE_UP(int x, int y);
	virtual void _keyDef_PAGE_DOWN(int x, int y);
	virtual void _keyDef_HOME(int x, int y);
	virtual void _keyDef_END(int x, int y);
	virtual void _keyDef_INSERT(int x, int y);

public:
	GM_Window();
	GM_Window(const GM_Window&);

	virtual
	~GM_Window();

	void reset() { _init(); }
	GM_Window& operator=(const GM_Window&);

	void insertCamera(Camera* cam, bool insert_in_scene = false);
	bool removeCamera(Camera *);

	void insertLight(Light* light, bool insert_in_scene = false);
	bool removeLight(Light* light);

	void setActiv(int new_c, int old_c=1, bool vertical=true, double d=0.5);
	// Camera* getActive() { int i; _findWindow(100,100,i) return _camera[i]; } 
	// Added by Dobrev
	// Needed for manipulation of the default camera
//	Camera* getDefaultCamera() { return _camera[ 0 ]; } 
	void removeActiv(int i);

	void insertSun()				{ _sun = new Sun(); _sun->on(); }
	void removeSun()				{ if(_sun) {delete _sun; _sun = NULL; } }
	void setDirSun(Angle d) { if(_sun) _sun->rotateGlobal(d,Vector3D<float>(1.0,1.0,0.0)); }
	
	void scaleDayLight(double);

	bool stereoToggle();
	bool isStereo()	{ return _stereo; }
};


///////////////////////////////////////////////////////////////////////////////////////////
// Implementation of the GM_Window public inline functions. Great importance on efficiency.
///////////////////////////////////////////////////////////////////////////////////////////

inline
void GM_Window::_display(void)
{
	_camera[_i]->move(_move);
	simulate();
	prepare();
	if(_stereo)
	{
		glDrawBuffer(GL_BACK_LEFT);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		_activ.last().drawCamera();
		_swapBuffers();
		glDrawBuffer(GL_BACK_RIGHT);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		_activ.last().drawCamera(true);
		_swapBuffers();
	}
	else
	{
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		_activ.last().drawCamera();
		_swapBuffers();
	}
}
	

inline
void GM_Window::_reshape(int w, int h)
{
	_w=w; _h=h;
	_activ.last().prepare(_w,_h);
}


} // end namespace GM_lib
