#ifndef		__LOAD3DS_H_
#define		__LOAD3DS_H_

#include <GM_Point3D.h>
#include <GM_LX_Array.h>

#define		MAXNAMESIZE	16+1
using namespace GM_lib;

struct H3dsFace 
{
public:
	Point3D<short>		p;
	short			flags;

	H3dsFace() { flags = 0; }
};

//
// Material structures
//

class H3dsMatList 
{
public:
	GM_String		name;		// Material name
	Array<short>	faces;		// List of faces with this material

	H3dsMatList		() { name = "WG_DEFAULT"; }
};

//
// Object and scene structures
//

class H3dsMeshObj 
{
public:
	GM_String					name;		// object name
	GL_Matrix					mat;		// object matrix
	LX_Array<H3dsFace>			facelist;	// Object Array of H3dsFace objects (face indices)
	LX_Array<H3dsMatList>		bindings;	// Object Array of H3dsMatList objects (to bind faces to materials)
	LX_Array<Point3D<float> >	vertlist;	// Object Array of vertices
	LX_Array<Point2D<float> >	maplist;	// Object Array of texture coordinates
	
	H3dsMeshObj	()
		{}
};


class H3dsScene
{
public:
	Array<MaterialObject>	material;				// Object array of H3dsMat objects
	Array<H3dsMeshObj*>		meshobjlist;			// Object array of H3dsMeshObj objects
	H3dsScene	(){}								// Class constructor
	~H3dsScene	()									// Class destructor
		{ for(int i=0; i<meshobjlist.size(); i++) delete meshobjlist[i]; }
};

// Each 3DS data-chunk starts with a 6 byte header. The first item in the header is a 2 byte (word) id-number.
// After that follows a dword wich gives the size of the data-chunk including the header. The size can be used
// as a relative offset to the next chunk.

enum
{
	CHUNK_RGB1      	= 0x0010,  // 3 floats of RGB
	CHUNK_RGB2      	= 0x0011,  // 3 bytes of RGB
	CHUNK_RGB3          = 0x0012,  // What is this?
	CHUNK_AMOUNT        = 0x0030,
	CHUNK_MAIN      	= 0x4D4D,
	CHUNK_OBJMESH       = 0x3D3D,
	CHUNK_BKGCOLOR      = 0x1200,
	CHUNK_AMBCOLOR  	= 0x2100,
	CHUNK_OBJBLOCK  	= 0x4000,
	CHUNK_TRIMESH   	= 0x4100,
	CHUNK_VERTLIST      = 0x4110,
	CHUNK_FACELIST      = 0x4120,
	CHUNK_FACEMAT       = 0x4130,
	CHUNK_MAPLIST       = 0x4140,
	CHUNK_SMOOLIST      = 0x4150,
	CHUNK_TRMATRIX      = 0x4160,
	CHUNK_LIGHT     	= 0x4600,
	CHUNK_SPOTLIGHT     = 0x4610,
	CHUNK_CAMERA    	= 0x4700,
	CHUNK_MATERIAL  	= 0xAFFF,
	CHUNK_MATNAME   	= 0xA000,
	CHUNK_AMBIENT   	= 0xA010,
	CHUNK_DIFFUSE   	= 0xA020,
	CHUNK_SPECULAR  	= 0xA030,
	CHUNK_SHININESS     = 0xA040,
	CHUNK_SHINE_STRENGTH= 0xA041,
	CHUNK_TRANSPARENCY  = 0xA050,
	CHUNK_DOUBLESIDED   = 0xA081,
	CHUNK_TEXTURE   	= 0xA200,
	CHUNK_BUMPMAP   	= 0xA230,
	CHUNK_MAPFILENAME   = 0xA300,
	CHUNK_MAPOPTIONS    = 0xA351,
	CHUNK_KEYFRAMER 	= 0xB000,
	CHUNK_FRAMES		= 0xB008
};

#endif