#pragma once
#include <GM_Material.h>
#include <GM_Texture.h>
#include <GM_PSurf.h>


namespace GM_lib {

class MaterialCurvature : public Material
{
	Array<Point<float,2> >	plot2D(const Array<float>& s, int k = 20);
	GL_Color				colorComputation(float C, const Array<Point<float,2> >& s, GL_Color mx=Red, GL_Color mn=Blue);
	bool _gauss;   
	
public:
	MaterialCurvature(PSurf<float,3>* ps, int res1=64, int res2 =64, bool gauss=true);
};




inline
MaterialCurvature::MaterialCurvature(PSurf<float,3>* ps, int res1, int res2, bool gauss) : Material(0.8*White,0.8*White,0.8*White,50.0)
{
	_gauss=gauss;
	// Compute the curvature in each pixel
	DMatrix<float> curvature(res1,res2);
	for(int i=0; i<res1; i++)
		for(int j=0; j<res2; j++)
		{
			if(_gauss) curvature[i][j]=ps->gaussCurvature(ps->startPar_u()+i*ps->deltaPar_u()/(float)(res1-1),
			ps->startPar_v()+j*ps->deltaPar_v()/(float)(res2-1));
			else curvature[i][j]=ps->meanCurvature(ps->startPar_u()+i*ps->deltaPar_u()/(float)(res1-1),
			ps->startPar_v()+j*ps->deltaPar_v()/(float)(res2-1));
		}

	// Edit the color mapping
	DVector<float> stmp=curvature.flat();
	Array<float> s(stmp.dim());
	for(int i=0;i<stmp.dim();i++) s[i]=stmp[i]; // missing cast
	
	// Array<float> s=curvature.flat();
	s.sort();								// This take times!!!!!!!!!
	Array<Point<float,2> > su=plot2D(s);	// But sorting is nessesary for editing!!

	// Make the texture
	Texture	texture(res1,res2);
	for(int i=0; i<res1; i++)
		for(int j=0; j<res2; j++)
			texture(i,j) = colorComputation(curvature[i][j], su);
	__texname = texture.LoadGL();
}

inline
GL_Color MaterialCurvature::colorComputation(float C, const Array<Point<float,2> >& s, GL_Color mx, GL_Color mn)
{
	for(int i=1; i<s.size()-1;i++) if(s(i)(1) > C) break;
	double d = s(i-1)(0)+(C-s(i-1)(1))/(s(i)(1)-s(i-1)(1))*(s(i)(0)-s(i-1)(0));
	mx.toHSV();
	mn.toHSV();
	mx = (1-d)*mn+d*mx;
	mx.toRGB();
	return mx;
}

inline
Array<Point<float,2> > MaterialCurvature::plot2D(const Array<float>& s, int k)
{
	int i,j=1;
	float hight = s(s.size()-1)-s(0);
	float delta = hight/(k-1);
	hight /= s.size();

	Array<Point<float,2> > su(k);
	su += Point2D<float>(0,s(0));
	for(i=1; i<s.size(); i++)
		if(s(i) >= (j*delta+s(0)))
		{ 
			su += Point2D<float>(i*hight,s(i));
			j++;
		}
	if(su.last()[1] < s(s.size()-1))
		su += Point2D<float>((s.size()-1)*hight,s(s.size()-1));

	Array<Point<float,2> > w;
	for(i=0; i<4; i++)	w += Point2D<float>(s.size()*(i/(float)3),i/(float)3);
	// Her skal editering av w foreg!!!! M se su ogs!!!	      1-

	Array<Point<float,2> > sus;
	sus.setSize(w.size());
	for(i=0,j=0; i<w.size(); i++)
	{
		sus[i][0]=w[i][1];
		for(; j<s.size()-1; j++) if((float)j >= w[i][0]) break;
		sus[i][1]=s(j);
	}
	return sus;
}

} // end namespace GM_lib
