#ifndef GM_QUADRATURE
#define GM_QUADRATURE
#include <GM_DMatrix.h>
using namespace GM_lib;

template <class T>
class Func		// instead of function pointers
{
public:
	virtual ~Func(){}
	virtual T operator () ( T ) = 0;
};

template <class T>
class NMFunc : public Func<T> // NonMember functions
{
public:
	NMFunc( T (*func)( T ) ){ _func=func; }
	~NMFunc(){}
	T operator () ( T );
private:
	T (*_func)( T );
};
template <class T> 
T NMFunc<T>::operator ()(T d) { return _func( d ); }

template <class C, class T>
class MFunc : public Func<T> // Member functions
{
public:
	MFunc( C &obj, T (C::*func)(T) ){ _obj=obj; _func=func;}
	~MFunc(){}
	T operator ()( T );
private:
	C _obj;
	T (C::*_func)( T);
};
template <class C, class T>
T MFunc<C,T>::operator ()(T d) { return (_obj.*_func)(d); }

template <class T, int stp>
class Quadrature  // integration by quadrature
{
private:	
	T trapez(T a, T b, unsigned int nn)
	{
		unsigned int		k = 1 << nn;// 2^n
		T	h = (b-a)/k;
		T	sum = 0.0;

		for (int i=1; i<k; i++) sum += (*_func)(a+h*i);
		sum *= h;
		sum += ((*_func)(a)+(*_func)(b))*0.5*h;

		return sum;
	}
   Func<T>* _func;
    
public:
	T integrate(T a, T b)
	{
		// unsigned int stp=7;
		static GM_lib::DMatrix<T> In(10,10);
		static GM_lib::Array<T> curve(1025);
		unsigned int k = 1 << stp; // 2^n
		T	h = (b-a)/(T)k;
		for (int i=0; i<=k; i++) curve[i] = (*_func)(a+h*i);
		In[0][0]=trapez(a,b,0);
		//In[0][1]=trapez(a,b,1);
		//In[1][0]=(4.0*In[0][1]-In[0][0])/3.0;

		for(unsigned int v=1;v<=stp;v++) 
		{
			// In[0][v]=trapez(a,b,v);
			// for(unsigned int k=1;k<=v;k++) In[k][v-k]=((1 << 2*k)*In[k-1][v-k+1]-In[k-1][v-k])/((1<<2*k)-1.0);
			unsigned int k1 = 1 << v; // 2^n
			int delta=k/k1;
			h=(b-a)/(T)k1;
			In[0][v]= (curve[0]+curve[k])*0.5; for(int i=1;i<k1;i++) In[0][v]+=curve[i*delta]; 
			In[0][v]*=h;	
			for(int u=1;u<=v;u++) In[u][v-u]=((1 << 2*u)*In[u-1][v-u+1]-In[u-1][v-u])/((1 << 2*u)-1.0);
		}
		return In[stp][0];
	}

	Quadrature(Func<T>& f){  _func=&f;}
	~Quadrature() {}
};

// use:
//
//  X xobj;
//  MFunc<X> f2(xobj,&X::aMemberFunc);
//  Quadrature quad(f2);
//
//   NMFunc mp(phi);                  // double phi(double x); extern function
//   Quadrature quad(mp);
//   double res=quad.integrate(0.0,3.14);
//
#endif