#ifndef __BiOrthoFilters__
#define __BiOrthoFilters__

#include <Filter.h>

namespace Wave_lib {

	template<class T>
	class BiOrthoFilters : public Filters<T> {

		void orthfilt(DVector<T> &filt, DVector<T> &Lo_D, DVector<T> &Hi_D, DVector<T> &Lo_R, DVector<T> &Hi_R, int p = 0);
		virtual void calcFilterCoefs(double dim, FilterCoefs<T> &dbn) = 0;		// must be overloaded by f.eks.	DaubN, CoifN, ....
		void calcFilters(FilterCoefs<T> &f, int p=0);								// calculates BiOrthogonal HighPass and LowPass filters
	public:
		BiOrthoFilters() : Filters<T>() {}
		

	};



	// ************************************************************************
	// **************************   Constructors   ****************************
	// ************************************************************************


	// ************************************************************************
	// **************************    Destructors    ***************************
	// ************************************************************************

	template<class T>
	void BiOrthoFilters<T>::orthfilt(DVector<T> &filt, DVector<T> &Lo_D, DVector<T> &Hi_D, DVector<T> &Lo_R, DVector<T> &Hi_R, int p) {

		// Normalize filter sum.
		Lo_R = filt*((T(1)/ filt.sum())*(T)sqrt(T(2)));
		_qmt(Lo_R, Hi_R, p);
		Hi_D = Hi_R.reverse();
		Lo_D = Lo_R.reverse();
	}

	// ************************************************************************
	// **************************   private func   ****************************
	// ************************************************************************

	template<class T>
	void BiOrthoFilters<T>::calcFilters(FilterCoefs<T> &f, int p) {
		
		int lr = f.values[0].dim();
		int ld = f.values[1].dim();

		int lmax = ((ld>lr)? (ld):(lr));
		if (lmax%2) ++lmax;

		int first1 = (int)floor(T(lmax-lr)/T(2));
		int last1  = (int)ceil(T(lmax-lr)/T(2));
		int first2 = (int)floor(T(lmax-ld)/T(2));
		int last2  = (int)ceil(T(lmax-ld)/T(2));
		int i;

		DVector<T> Rf(first1+lr+last1, T(0));
		for (i=0; i<lr; ++i)
			Rf[i+first1] = f.values[0][i];

		DVector<T> Df(first2+ld+last2, T(0));
		for (i=0; i<ld; ++i)
			Df[i+first2] = f.values[1][i];

		f.Lo_D.setSize(2); f.Hi_D.setSize(2);
		f.Lo_R.setSize(2); f.Hi_R.setSize(2);

		// calculating LowPass & HighPass filters
		orthfilt(Rf, f.Lo_D[0], f.Hi_D[0], f.Lo_R[0], f.Hi_R[0]);
		orthfilt(Df, f.Lo_D[1], f.Hi_D[1], f.Lo_R[1], f.Hi_R[1]);
	}


} // namespace Wave_lib 


#endif // __BiOrthoFilters__