
#ifndef _DL_ALGO_H_
#define _DL_ALGO_H_

#include "GM_DVector.h"
#include <GM_DArrayN.h>
#include <Definitions.h>
#include <Filter.h>

using namespace GM_lib;

DVector<double> lo_r;
DVector<double> hi_r;

static int n=50; //precision of approximation maesured by the number of Daubechies-Lagarias steps

/*******************************************************************
  Evaluation of the scaling function (father wavelet)corresponding 
  to an Orthogonal MRA by Daubechies-Lagarias Algorithm
  ******************************************************************/
template<class T>
T scaleLD(T z, int scale, int shift, Filters<T> *filter, T fdegree)
{
	T val;
	filter->getFilterR(fdegree, lo_r, hi_r);

	int N=lo_r.dim()-1;
	T x= pow(2,scale)*z-shift;
	if(x<=0 || x>=N) val=0;

	else
	{
		DVector<int> size(2);
		size[0]=size[1]=N;
		
		int index=floor(x);
		T dec=x-index;

		DVector<T> dy;
		Array<DArrayN<T,2> > nT;

		dy=dec_to_bin<T>(dec,n);
		nT=get_nT<T>(N);

		DArrayN<T,2> prod;
		prod.setSize(size);
		for(int i=0; i<size[0]; i++)
			for(int j=0; j<size[1]; j++)
				if(j==i) prod[i][j]= T(1);
				else prod[i][j]= T(0);

		DArrayN<T,2> tmp;
		tmp.setSize(size);
		for(int t=0; t<n; t++)
		{
			if (dy[t]==1){
				for(int i=0; i<size[0]; i++)
					for(int j=0; j<size[1]; j++)
					{
						T sum=0;
						for(int k=0; k<size[1]; k++)
							sum+=prod[i][k]*nT[1][k][j];
						tmp[i][j]=sum;
					}
			}
			else {
				for(int i=0; i<size[0]; i++)
					for(int j=0; j<size[1]; j++)
					{
						T sum=0;
						for(int k=0; k<size[1]; k++)
							sum+=prod[i][k]*nT[0][k][j];
						tmp[i][j]=sum;
					}		
			}
			prod=tmp;
		}

		DArrayN<T,2> ret;
		ret.setSize(size);
		for(int j=0; j<size[1]; j++)
			for(int i=0; i<size[0]; i++)
				ret[i][j]=prod[i][j]*pow(2,(scale/2));
			
		DVector<T> u(size[0]);
		for(int i=0; i<size[0]; i++)
		{
			u[i]=0;
			for(int j=0; j<size[1]; j++)
				u[i] += ret[i][j];
			u[i]/=size[0];
		}
		val =  u[index];
	}
   
	return val;
}


/******************************************************************
  Evaluation of the wavelet function (mother wavelet) corresponding
  to an Orthogonal MRA by Daubechies-Lagarias Algorithm
*******************************************************************/
template<class T>
T waveLD(T z, int scale, int shift, Filters<T> *filter, T fdegree)
{
	T val;
	filter->getFilterR(fdegree, lo_r, hi_r);

	int N=lo_r.dim()-1;
	int daun = (N+1)/2;
	T x=pow(2,scale)*z-shift;

	if(x<=daun-N || x>=daun) val=0;
	else
	{
		DVector<int> size(2);
		size[0]=size[1]=N;

		int index=floor(2*x);
		T dec=2*x-index;
		
		DVector<T> dy;
		Array<DArrayN<T,2> > nT;
		
		dy=dec_to_bin<T>(dec,n);
		nT=get_nT<T>(N);

		DArrayN<T,2> prod;
		prod.setSize(size);
		for(int i=0; i<size[0]; i++)
			for(int j=0; j<size[1]; j++)
				if(j==i) prod[i][j]= T(1);
				else prod[i][j]= T(0);

		DArrayN<T,2> tmp;
		tmp.setSize(size);
		for(int t=0; t<n; t++)
		{
			if (dy[t]==1){
				for(int i=0; i<size[0]; i++)
					for(int j=0; j<size[1]; j++)
					{
						T sum=0;
						for(int k=0; k<size[1]; k++)
							sum+=prod[i][k]*nT[1][k][j];
						tmp[i][j]=sum;
					}
			}
			else {
				for(int i=0; i<size[0]; i++)
					for(int j=0; j<size[1]; j++)
					{
						T sum=0;
						for(int k=0; k<size[1]; k++)
							sum+=prod[i][k]*nT[0][k][j];
						tmp[i][j]=sum;
					}		
			}
			prod=tmp;
		}
		
		T fi;
		int ii;
		DVector<T> u(size[0]);
		for(int i=0; i<size[0]; i++)
		{
			ii = i+2 - index;
			if ( (ii > 0) && (ii < N+2) )
				fi= pow(-1,-ii)*lo_r[ii-1];
			else fi=T(0);
			u[i] =fi;
		}

		DVector<T> v(size[0]);
		for(int i=0; i<size[0]; i++)
		{
			v[i]=0;
			for(int j=0; j<size[1]; j++)
				v[i] += prod[i][j];
			v[i]/=size[0];
		}

		T sum=0;
		for(int i=0; i<size[0]; i++)	
			sum+=u[i]*v[i];

		val=pow(2,scale/2)*sum;	
	}	

	return val;
}

template<class T>
DVector<T> dec_to_bin(T dec, int n)
{
	DVector<T> dy;
		for(int i=0; i<n; i++)
		{
			if(dec <= 0.5){ 
				dy.append(1,0);
				dec*=2;
			}
			else {
				dy.append(1,1);
				dec=2*dec-1;
			}
		}
	return dy;
}

template<class T>
Array<DArrayN<T,2> > get_nT(int N)
{
	Array<DArrayN<T,2> > nT;
	nT.setSize(2);

	DVector<int> size(2);
	size[0]=size[1]=N;
	
	for(int i=0; i<nT.size(); i++){
		nT[i].setSize(size);
		nT[i].setElements(0);
	}

	for(int i=0; i<size[0]; i++)
		for(int j=0; j<size[1]; j++)
		{
			if ((2*i-j+1 > 0) && (2*i-j+1 <= N+1)) 
				nT[0][i][j] = sqrt(T(2))*lo_r[2*i-j];

			if ((2*i-j+2 > 0) && (2*i-j+2 <= N+1))
				nT[1][i][j] = sqrt(T(2))*lo_r[2*i-j+1];
		}

	return nT;
}

#endif