
#pragma once
//#ifndef __DaubFilter__
//#define __DaubFilter__

#include <OrthoFilters.h>
#include <assert.h> //For feil rapportering....

#include <iostream>
#include <complex>
using namespace std;

#include <GM_Array.h>
#include <GM_DVector.h>
#include <GM_DMatrix.h>
using namespace GM_lib;

namespace Wave_lib {

	template<class T>
	class DaubFilter : public OrthoFilters<T> {
			
			void _init();							
		
	public:
			DaubFilter();
			~DaubFilter();
			
			void calcFilterCoefs(double dim, FilterCoefs<T> &dbn);			// calculating daubechies filters coefs
	};



	// ************************************************************************
	// **************************   Constructors   ****************************
	// ************************************************************************
	
	template<class T>
	inline DaubFilter<T>::DaubFilter() : OrthoFilters<T>() {
		_init();
	}


	// ************************************************************************
	// **************************    Destructors    ***************************
	// ************************************************************************

	template<class T>
	DaubFilter<T>::~DaubFilter() {}

	// ************************************************************************
	// **************************   private func   ****************************
	// ************************************************************************

	template<class T>
	void DaubFilter<T>::_init() {


		FilterCoefs<T>	 temp;
		DVector<T>		Coefs;

		temp.values.setSize(1);

		temp.dim = 1;
		Coefs.setDim(2);
		Coefs[0] = T(0.50000000000000);
		Coefs[1] = T(0.50000000000000);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 2;
		Coefs.setDim(4);
		Coefs[0] = T(0.34150635094622);
		Coefs[1] = T(0.59150635094587);
		Coefs[2] = T(0.15849364905378);
		Coefs[3] = T(-0.09150635094587);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 3;
		Coefs.setDim(6);
		Coefs[0] = T(0.23523360389270);
		Coefs[1] = T(0.57055845791731);
		Coefs[2] = T(0.32518250026371);
		Coefs[3] = T(-0.09546720778426);
		Coefs[4] = T(-0.06041610415535);
		Coefs[5] = T(0.02490874986589);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);


		temp.dim = 4;
		Coefs.setDim(8);
		Coefs[0] = T(0.16290171402562);
		Coefs[1] = T(0.50547285754565);
		Coefs[2] = T(0.44610006912319);
		Coefs[3] = T(-0.01978751311791);
		Coefs[4] = T(-0.13225358368437);
		Coefs[5] = T(0.02180815023739);
		Coefs[6] = T(0.02325180053556);
		Coefs[7] = T(-0.00749349466513);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);


		temp.dim = 5;
		Coefs.setDim(10);
		Coefs[0] = T(0.11320949129173);
		Coefs[1] = T(0.42697177135271);
		Coefs[2] = T(0.51216347213016);
		Coefs[3] = T(0.09788348067375);
		Coefs[4] = T(-0.17132835769133);
		Coefs[5] = T(-0.02280056594205);
		Coefs[6] = T(0.05485132932108);
		Coefs[7] = T(-0.00441340005433);
		Coefs[8] = T(-0.00889593505093);
		Coefs[9] = T(0.00235871396920);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);


		temp.dim = 6;
		Coefs.setDim(12);
		Coefs[0] = T(0.07887121600143);
		Coefs[1] = T(0.34975190703757);
		Coefs[2] = T(0.53113187994121);
		Coefs[3] = T(0.22291566146505);
		Coefs[4] = T(-0.15999329944587);
		Coefs[5] = T(-0.09175903203003);
		Coefs[6] = T(0.06894404648720);
		Coefs[7] = T(0.01946160485396);
		Coefs[8] = T(-0.02233187416548);
		Coefs[9] = T(0.00039162557603);
		Coefs[10] = T(0.00337803118151);
		Coefs[11] = T(-0.00076176690258);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);


		temp.dim = 7;
		Coefs.setDim(14);
		Coefs[0] = T(0.05504971537285);
		Coefs[1] = T(0.28039564181304);
		Coefs[2] = T(0.51557424581833);
		Coefs[3] = T(0.33218624110566);
		Coefs[4] = T(-0.10175691123173);
		Coefs[5] = T(-0.15841750564054);
		Coefs[6] = T(0.05042323250485);
		Coefs[7] = T(0.05700172257986);
		Coefs[8] = T(-0.02689122629486);
		Coefs[9] = T(-0.01171997078235);
		Coefs[10] = T(0.00887489618962);
		Coefs[11] = T(0.00030375749776);
		Coefs[12] = T(-0.00127395235906);
		Coefs[13] = T(0.00025011342658);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

		 
		temp.dim = 8;
		Coefs.setDim(16);
		Coefs[0] = T(0.03847781105406);
		Coefs[1] = T(0.22123362357624);
		Coefs[2] = T(0.47774307521438);
		Coefs[3] = T(0.41390826621166);
		Coefs[4] = T(-0.01119286766665);
		Coefs[5] = T(-0.20082931639111);
		Coefs[6] = T(0.00033409704628);
		Coefs[7] = T(0.09103817842345);
		Coefs[8] = T(-0.01228195052300);
		Coefs[9] = T(-0.03117510332533);
		Coefs[10] = T(0.00988607964808);
		Coefs[11] = T(0.00618442240954);
		Coefs[12] = T(-0.00344385962813);
		Coefs[13] = T(-0.00027700227421);
		Coefs[14] = T(0.00047761485533);
		Coefs[15] = T(-0.00008306863060);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

	
		temp.dim = 9;
		Coefs.setDim(18);
		Coefs[0] = T(0.02692517479416);
		Coefs[1] = T(0.17241715192471);
		Coefs[2] = T(0.42767453217028);
		Coefs[3] = T(0.46477285717278);
		Coefs[4] = T(0.09418477475112);
		Coefs[5] = T(-0.20737588089628);
		Coefs[6] = T(-0.06847677451090);
		Coefs[7] = T(0.10503417113714);
		Coefs[8] = T(0.02172633772990);
		Coefs[9] = T(-0.04782363205882);
		Coefs[10] = T(0.00017744640673);
		Coefs[11] = T(0.01581208292614);
		Coefs[12] = T(-0.00333981011324);
		Coefs[13] = T(-0.00302748028715);
		Coefs[14] = T(0.00130648364018);
		Coefs[15] = T(0.00016290733601);
		Coefs[16] = T(-0.00017816487955);
		Coefs[17] = T(0.00002782275679);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);


		temp.dim = 10;
		Coefs.setDim(20);
		Coefs[0] = T(0.01885857879640);
		Coefs[1] = T(0.13306109139687);
		Coefs[2] = T(0.37278753574266);
		Coefs[3] = T(0.48681405536610);
		Coefs[4] = T(0.19881887088440);
		Coefs[5] = T(-0.17666810089647);
		Coefs[6] = T(-0.13855493935993);
		Coefs[7] = T(0.09006372426666);
		Coefs[8] = T(0.06580149355070);
		Coefs[9] = T(-0.05048328559801);
		Coefs[10] = T(-0.02082962404385);
		Coefs[11] = T(0.02348490704841);
		Coefs[12] = T(0.00255021848393);
		Coefs[13] = T(-0.00758950116768);
		Coefs[14] = T(0.00098666268244);
		Coefs[15] = T(0.00140884329496);
		Coefs[16] = T(-0.00048497391996);
		Coefs[17] = T(-0.00008235450295);
		Coefs[18] = T(0.00006617718320);
		Coefs[19] = T(-0.00000937920789);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

	}


	template<class T>
	void DaubFilter<T>::calcFilterCoefs(double dim , FilterCoefs<T> &f)
	{	
		int i,j;
		int center = 2*dim-1;
		int nn = 4*dim-1;
		T   n, m;
		DVector<T> P;
		P.setDim(nn);

		f.values.setSize(1);
		f.values[0].setDim(2*dim);

		// P = [a(N) 0 a(N-1) 0 ... 0 a(1) 1 a(1) 0 ... 0 a(N-1) 0 a(N)]
		P[center] = T(1.0);
		for (i=1; i<=dim; ++i) {
			n = m = T(1);
		
			for (j=(-dim+1); j<=dim; ++j) {
				if (j!=i) {
					m *= T(T(1)/T(2)-T(j));			
					n *= T(i-j);
				}
			}	
			m /= n;
			f.values[0][center+i] = m;
		} 

		QFile outfile( "FilterCoefficients" );
		if ( outfile.open( IO_WriteOnly ) ) 
		{
			QTextStream out( &outfile );
			for (i=0; i<f.values[0].dim(); i++)
			{
				out<<(double) f.values[0][i] <<" ";
				out<<"\n\n";
			}
		outfile.close();
		}
 }

} // namespace Wave_lib



// #endif // __DaubFilter__