
//#pragma once
#ifndef HAARFILTER_H
#define HAARFILTER_H

#include <OrthoFilters.h>
#include <assert.h> //For feil rapportering....

#include <iostream>
//#include <complex>
using namespace std;

#include <GM_Array.h>
#include <GM_DVector.h>
#include <GM_DMatrix.h>
using namespace GM_lib;

namespace Wave_lib {

	template<class T>
	class HaarFilter : public OrthoFilters<T> {
			
			void _init();					
			
			void calcFilterCoefs(double dim, FilterCoefs<T> &dbn);			// calculating Haar filters coefs
		public:
			HaarFilter();
			~HaarFilter();
	};



	// ************************************************************************
	// **************************   Constructors   ****************************
	// ************************************************************************
	
	template<class T>
	inline HaarFilter<T>::HaarFilter() : OrthoFilters<T>() 
	{
		_init();
	}


	// ************************************************************************
	// **************************    Destructors    ***************************
	// ************************************************************************

	template<class T>
	HaarFilter<T>::~HaarFilter() 
	{
	}

	// ************************************************************************
	// **************************   private func   ****************************
	// ************************************************************************

	template<class T>
	void HaarFilter<T>::_init() 
	{

		FilterCoefs<T>	temp;
		DVector<T>		Coefs;

		temp.values.setSize(1);

		temp.dim = 1;
		Coefs.setDim(2);
		Coefs[0] = T(0.50000000000000);
		Coefs[1] = T(0.50000000000000);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);
	}

	template<class T>
	void HaarFilter<T>::calcFilterCoefs(double dim, FilterCoefs<T> &f) 
	{
		int i,j;
		int center = 2*dim-1;
		int nn = 4*dim-1;
		T   n, m;
		DVector<T> P;
		P.setDim(nn);

		f.values.setSize(1);
		f.values[0].setDim(2*dim);

		// P = [a(N) 0 a(N-1) 0 ... 0 a(1) 1 a(1) 0 ... 0 a(N-1) 0 a(N)]
		P[center] = T(1.0);
		for (i=1; i<=dim; ++i) {
			n = m = T(1);
		
			for (j=(-dim+1); j<=dim; ++j) {
				if (j!=i) {
					m *= T(T(1)/T(2)-T(j));			
					n *= T(i-j);
				}
			}	
			m /= n;
			f.values[0][center+i] = m;
		} 

		QFile outfile( "FilterCoefficients" );
		if ( outfile.open( IO_WriteOnly ) ) 
		{
			QTextStream out( &outfile );
			for (i=0; i<f.values[0].dim(); i++)
			{
				out<<f.values[0][i] <<" ";
				out<<"\n\n";
			}
		outfile.close();
		}
}

}//end namespace Wave_lib

#endif //HAARFILTER_H