
//#pragma once
#ifndef NEVILLE_H
#define NEVILLE_H

#include <OrthoFilters.h>
#include <assert.h> //For feil rapportering....

#include <iostream>
using namespace std;

#include <GM_Array.h>
#include <GM_DVector.h>
#include <GM_DMatrix.h>
using namespace GM_lib;

namespace Wave_lib {

	template<class T>
	class Neville : public OrthoFilters<T> {
			
			void _init();					
			void calcFilterCoefs(double N, FilterCoefs<T> &dbn);
			T NevilleAlgo ( const T *x, const T *f, const int n, const T xx );

		public:
			Neville();
			~Neville();
	};



	// ************************************************************************
	// **************************   Constructors   ****************************
	// ************************************************************************
	
	template<class T>
	inline Neville<T>::Neville() : OrthoFilters<T>() 
	{
		_init();
	}


	// ************************************************************************
	// **************************    Destructors    ***************************
	// ************************************************************************

	template<class T>
	Neville<T>::~Neville() 
	{
	}

	// ************************************************************************
	// **************************   private func   ****************************
	// ************************************************************************

	template<class T>
	void Neville<T>::_init() {}

	template<class T>
	void Neville<T>::calcFilterCoefs(double N, FilterCoefs<T> &f) 
	{
		DVector<T>	Coefs;
		T xa[40], ya[40], x;
		int _dim, Nrows;

		assert ( N < sizeof xa / sizeof *xa );   //N is greater than dim(xa) 

		// Number of cases for filter calculations 
		Nrows = ( (int) N>>1) + 1;    // N/2 + 1 
		Coefs.setDim(Nrows*N);

		// Generate values of xa 
		xa[0] = (T)0.5*(T)(1 - N);    // -N/2 + 0.5 
		for (int col = 1 ; col < N ; col++)
			xa[col] = xa[col-1] + (T)1;

		// Find filter coefficient values 
		_dim = Nrows*N-1;
		for (int row = 0 ; row < Nrows ; row++) {
			x = (T)row;
			for (col = 0 ; col < N ; col++) {
				for (int cc = 0 ; cc < N ; cc++)
					ya[cc] = (T)0;
				ya[col] = (T)1;
				 Coefs[_dim--]= NevilleAlgo (xa, ya, N, x);
			}
		}

		Coefs.append(Coefs.reverse());
		f.values.setSize(1);
		f.values[0] = Coefs;

		QFile outfile( "FilterCoefficients" );
		if ( outfile.open( IO_WriteOnly ) ) 
		{
			QTextStream out( &outfile );
			for (int i=0; i<f.values[0].dim(); ++i)
			{
				out<<f.values[0][i] <<" ";
				out<<"\n";
			}
		outfile.close();
		}
	}

	template<class T>
	T Neville<T>::NevilleAlgo ( const T *x, const T *f, const int n, const T xx )
	{
		DArrayN<T,1> vy;
		T y;

		DVector<int> size;
        size.setDim(1);
		size[0]=n;
		vy.setSize(size); 

		for (int i=0; i<n; i++ ) {
			vy[i] = f[i];
			for (int j=i-1; j>=0; j--) {
				T den = x[i] - x[j];
				vy[j] = vy[j+1] + (vy[j+1] - vy[j]) * (xx - x[i]) / den;
			}
		}
		y = vy[0];
		return y;
	}


}//end namespace Wave_lib

#endif