
#ifndef POLLENFILTER_H
#define POLLENFILTER_H

#include <OrthoFilters.h>
#include <assert.h> 

#include <iostream>
using namespace std;

#include <GM_Array.h>
#include <GM_DVector.h>
#include <GM_DMatrix.h>
using namespace GM_lib;

namespace Wave_lib {

	template<class T>
	class PollenFilter : public OrthoFilters<T> {
			
			void _init(T phi1, T phi2);		
			void calcFilterCoefs(double dim, FilterCoefs<T> &dbn);			// calculating Haar filters coefs

		public:
			T phi1, phi2;

			PollenFilter(T phi1, T phi2);
			~PollenFilter();
	};



	// ************************************************************************
	// **************************   Constructors   ****************************
	// ************************************************************************
	
	template<class T>
	inline PollenFilter<T>::PollenFilter(T phi1, T phi2) : OrthoFilters<T>() 
	{
		this->phi1=phi1;
		this->phi2=phi2;

		_init(phi1, phi2);
	}


	// ************************************************************************
	// **************************    Destructors    ***************************
	// ************************************************************************

	template<class T>
	PollenFilter<T>::~PollenFilter() 
	{
	}

	// ************************************************************************
	// **************************   private func   ****************************
	// ************************************************************************

	template<class T>
	void PollenFilter<T>::_init(T phi1, T phi2) 
	{

		FilterCoefs<T>	temp;
		DVector<T>		Coefs;

		temp.values.setSize(1);
		
		temp.dim = 1;
		Coefs.setDim(4);
		Coefs[0] = (1 + cos(phi1) - sin(phi1));
		Coefs[1] = (1 + cos(phi1) + sin(phi1));
		Coefs[2] = (1 - cos(phi1) + sin(phi1));
		Coefs[3] = (1 - cos(phi1) - sin(phi1));
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 2;
		Coefs.setDim(6);
		Coefs[0] = ( 1+cos(phi1) - cos( phi2 ) - cos( phi1) * cos( phi2)
				+ sin( phi1 ) - cos( phi2) * sin( phi1) - sin( phi2 ) 
				+ cos( phi1) * sin( phi2) - sin( phi1) * sin( phi2 ) )/2;
		Coefs[1] = ( 1-cos( phi1) + cos( phi2) - cos( phi1) * cos( phi2)
				+ sin( phi1) + cos( phi2) * sin( phi1) - sin( phi2) 
				- cos( phi1) * sin( phi2) - sin( phi1) * sin( phi2) )/2;
		Coefs[2] = ( 1 + cos( phi1) * cos( phi2) + cos( phi2) * sin( phi1)
				- cos( phi1) * sin( phi2 ) + sin( phi1) *sin( phi2 ) );
		Coefs[3] = ( 1 + cos( phi1 ) * cos( phi2 ) - cos( phi2 ) * sin( phi1 ) 
				+ cos( phi1 )* sin( phi2 ) + sin( phi1)* sin( phi2) );
		Coefs[4] = ( 1-cos( phi1) + cos( phi2 )- cos( phi1 )* cos( phi2 ) 
				- sin( phi1 ) - cos( phi2 ) * sin( phi1 ) + sin( phi2 )
				+ cos( phi1 )* sin( phi2 )- sin( phi1 )* sin( phi2) ) /2;
		Coefs[5] = ( 1+cos( phi1 )- cos( phi2 )- cos( phi1 )* cos( phi2 ) 
				- sin( phi1 ) + cos( phi2 ) * sin( phi1 ) + sin( phi2 )
				- cos( phi1 ) * sin( phi2 ) - sin( phi1 )* sin( phi2) )/2;
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

		QFile outfile( "FilterCoefficients" );
		if ( outfile.open( IO_WriteOnly ) ) 
		{
			QTextStream out( &outfile );
			for (int i=0; i<temp.values[0].dim(); i++)
			{
				out<<temp.values[0][i] <<" ";
				out<<"\n\n";
			}
		outfile.close();
		}
	}

	template<class T>
	void PollenFilter<T>::calcFilterCoefs(double dim, FilterCoefs<T> &f) 
	{
	
	}

}//end namespace Wave_lib

#endif 