#include <iostream>
using namespace std;
#include "scalagram.h"

Scalagram::Scalagram(int width,int height,WaveDWT<double> *wavelet)
{
	this->width = width;
	this->height = height;
	windowheight = height;
	windowwidth = width;
	this->wavelet = wavelet;
	init();
}


GraphCurve *Scalagram::getOutCurve()
{
	return outcurve;
}

GraphCurve *Scalagram::getErrorCurve()
{
	return errorcurve;
}
void Scalagram::init()
{

	colorgradient = new ColorGradient();
	outcurve = new GraphCurve("Output signal");
	errorcurve = new GraphCurve("Error");
	scalingfuntion = new GraphCurve("Scaling function");
	
	
	listID1 =  glGenLists(1);

	glNewList(listID1,GL_COMPILE);
		glDisable(GL_LIGHTING);
		glBegin(GL_QUADS);
			glVertex2i(0,0);
			glVertex2i(1,0);
			glVertex2i(1,1);
			glVertex2i(0,1);
		glEnd();
	glEndList();

	float r,g,b;

	colortableid = glGenLists(1);
	glNewList(colortableid,GL_COMPILE);
		glDisable(GL_LIGHTING);
		glBegin(GL_TRIANGLE_STRIP);
			colorgradient->setMinValue(0);
			colorgradient->setMaxValue(height);
			for(int i=0;i<=height;i+=10)
			{
				colorgradient->getValue(r,g,b,i);
				glColor3f(r,g,b);
				glVertex2i(width+width/8 ,0+i);
				glVertex2i(width+width/8 + 10 ,0+i);
			}
		glEnd();
	glEndList();

	colorgradient->setMaxValue(1.0);
	colorgradient->setMinValue(-1.0);

}

Scalagram::~Scalagram(void)
{
}


void Scalagram::localDisplay()
{
	drawBorders();
}

void Scalagram::localSelect()
{

}

void Scalagram::localSimulate(double dt)
{

}
void Scalagram::generateData(DVector<double> &coefs,DVector<int> &longs)
{
	this->coefs = coefs;
	this->longs = longs;
	int pos = 0;
	int start;
	int stop;

	int ant = coefs.dim();

	int diff = ant - points.size() ;
	if (diff>0)
	{
		for (int i=0;i<diff;i++)
			points.alwaysInsert(new ColorPoint(listID1));
	}
//	cout <<"\n Coefs dim "<<coefs.dim()<<"    points.dim "<<points.size();

	int i;

/*	cout <<"\n";
	for (i=0;i<coefs.dim();i++)
		cout <<coefs[i]<<"|";
	cout <<"\nlongs\n";
	for (i=0;i<longs.dim();i++)
		cout <<longs[i]<<"|";*/
	float r,g,b;


	float min,max;
	max = 0;
	min = 0;


	for (i=0;i<coefs.dim();i++)
	{
		if (coefs[i] > max) 
			max = coefs[i];
		if (coefs[i] < min) 
			min = coefs[i];
	}


	colorgradient->setMaxValue(max);
	colorgradient->setMinValue(-max);


	scaling = (max- min) / 100;
	

	max = longs[longs.dim()-2];

	float dh,dw;

	dh = height / (longs.dim()-1);
	dw = (float)width / max ;
	
	_outdata = wavelet->reconstruction(coefs,longs);
	outcurve->setData(_outdata);
	scalingfuntion->setData(coefs.getSubVector(0,longs[0]));
	
	_errordata = (wavelet->getInData()) - _outdata;
	errorcurve->setData(_errordata);
	//scalingfuntion->setScale(width/longs.dim(),10);

	

	for (i=0;i<longs.dim()-1;i++)
	{
		start = pos;
		stop = pos + longs[i];



		min = max / longs[i] ;
		for(int u=start;u<stop;u++)
		{
			points[u]->setIndex(i,u-start,u);
			
			points[u]->translate(Vector3D<float>(((u-start))*min *dw, (longs.dim()-i-2)*dh,0));
			
			
			colorgradient->getValue(r,g,b,coefs[u]);
			points[u]->setValue(coefs[u]);
			points[u]->setWidth(dw*min);
			points[u]->setHeight(dh);
			points[u]->setRGB(r,g,b);
			
			this->insert(points[u]);
		}
		pos = stop;
	}

}


GraphCurve *Scalagram::getScalingFunction()
{
	return scalingfuntion;
}

DVector<double> Scalagram::getCoifisients()
{

	DVector<double> coefs;
	int pos =0;
	int start,stop;
	coefs.setDim(points.size());
	for (int i=longs.dim()-2;i>=0;i--)
	{
		start = pos;
		stop = pos + longs[i];
		
		for(int u=start;u<stop;u++)
		{
			coefs[u] = points[i]->getValue();
		}
		pos = stop;
	}

	return coefs;
}


DVector<int> &Scalagram::getLongs()
{
	return longs;
}


void Scalagram::changeColor(ColorPoint *point,float dv)
{
	float r,g,b;
	point->addValue(scaling*dv);
	colorgradient->getValue(r,g,b,point->getValue());
	point->setRGB(r,g,b);
	coefs[point->getIndex()] = point->getValue();

	//outcurve->setData( wavelet->reconstructon(coefs,longs) );
	
	_outdata = wavelet->reconstruction(coefs,longs);
	outcurve->setData(_outdata);
	//scalingfuntion->setData(coefs.getSubVector(0,longs[0]));
	
	_errordata = (wavelet->getInData()) - _outdata;
	errorcurve->setData(_errordata);

	if (point->getIndex()<longs[0])
	{
		scalingfuntion->setData(point->getIndex(),point->getValue());
		
	}


}


void Scalagram::addMax(float d)
{
	float r,g,b;
//	colorgradient->setMaxValue(max);
	colorgradient->addMaxValue(d);
	for (int i=0;i<points.size();i++)
	{
		colorgradient->getValue(r,g,b,points[i]->getValue());
		points[i]->setRGB(r,g,b);
	}

}
void Scalagram::addMin(float d)
{
	float r,g,b;
//	colorgradient->setMinValue(min);
	colorgradient->addMinValue(d);
	for (int i=0;i<points.size();i++)
	{
		colorgradient->getValue(r,g,b,points[i]->getValue());
		points[i]->setRGB(r,g,b);
	}
}
void Scalagram::drawPoints()
{
	for(int i=0;i<points.size();i++)
	{
		points[i]->display();
	}
}

void Scalagram::drawBorders()
{

	glColor3f(0,0.8,0);
	glLineWidth(3);
	glBegin(GL_LINES);
		glVertex2i(0,-height/10);
		glVertex2i(0,height+height/10);
		glVertex2i(-width/10,0);
		glVertex2i(width+width/10,0);
	glEnd();
	glLineWidth(1);

	glCallList(colortableid);
}

