#ifndef __SplineFilter__
#define __SplineFilter__

//#include "OrthoFilters.h"
#include <BiOrthoFilters.h>
#include <assert.h> //For feil rapportering....

#include <iostream>
#include <complex>
using namespace std;

#include <GM_Array.h>
#include <GM_DVector.h>
#include <GM_DMatrix.h>
using namespace GM_lib;

namespace Wave_lib {

	template<class T>
	class SplineFilter : public BiOrthoFilters<T> {

		void _init();					

		void calcFilterCoefs(double dim, FilterCoefs<T> &dbn);			// calculating daubechies filters coefs
	public:
		SplineFilter();
		~SplineFilter();
	};



	// ************************************************************************
	// **************************   Constructors   ****************************
	// ************************************************************************

	template<class T>
	inline SplineFilter<T>::SplineFilter() : BiOrthoFilters<T>() {
		_init();
	}


	// ************************************************************************
	// **************************    Destructors    ***************************
	// ************************************************************************

	template<class T>
	SplineFilter<T>::~SplineFilter() {



	}

	// ************************************************************************
	// **************************   private func   ****************************
	// ************************************************************************

	template<class T>
	void SplineFilter<T>::_init() {


		FilterCoefs<T>	temp;
		DVector<T>				Coefs;
		DVector<T>				rev;

		temp.values.setSize(2);

		temp.dim = 1.1;
		Coefs.setDim(2);
		Coefs[0] = T(1)/T(2);
		Coefs[1] = T(1)/T(2);
		temp.values[0] = Coefs;
		temp.values[1] = Coefs;
		_filter.alwaysInsert(temp);


		temp.dim = 1.3;
		Coefs.setDim(2);
		Coefs[0] = T(1)/T(2);
		Coefs[1] = T(1)/T(2);
		temp.values[0] = Coefs;
		Coefs.setDim(3);
		Coefs[0] = T(-1)/T(16);
		Coefs[1] = T(1)/T(16);
		Coefs[2] = T(1)/T(2);
		Coefs.append(Coefs.reverse());
		temp.values[1] = Coefs;
		_filter.alwaysInsert(temp);


		temp.dim = 1.5;
		Coefs.setDim(2);
		Coefs[0] = T(1)/T(2);
		Coefs[1] = T(1)/T(2);
		temp.values[0] = Coefs;
		Coefs.setDim(5);
		Coefs[0] = T(3)/T(256);
		Coefs[1] = T(-3)/T(256);
		Coefs[2] = T(-11)/T(128);
		Coefs[3] = T(11)/T(128);
		Coefs[4] = T(1)/T(2);
		Coefs.append(Coefs.reverse());
		temp.values[1] = Coefs;
		_filter.alwaysInsert(temp);


		temp.dim = 2.2;
		Coefs.setDim(3);
		Coefs[0] = T(1)/T(4);
		Coefs[1] = T(1)/T(2);
		Coefs[2] = T(1)/T(4);
		temp.values[0] = Coefs;
		Coefs.setDim(2);
		Coefs[0] = T(-1)/T(8);
		Coefs[1] = T(1)/T(4);
		rev	= Coefs.reverse();
		Coefs.putlast(T(3)/T(4));
		Coefs.append(rev);
		temp.values[1] = Coefs;
		_filter.alwaysInsert(temp);


		temp.dim = 2.4;
		Coefs.setDim(3);
		Coefs[0] = T(1)/T(4);
		Coefs[1] = T(1)/T(2);
		Coefs[2] = T(1)/T(4);
		temp.values[0] = Coefs;
		Coefs.setDim(4);
		Coefs[0] = T(3)/T(128);
		Coefs[1] = T(-3)/T(64);
		Coefs[2] = T(-1)/T(8);
		Coefs[3] = T(19)/T(64);
		rev	= Coefs.reverse();
		Coefs.putlast(T(45)/T(64));
		Coefs.append(rev);
		temp.values[1] = Coefs;
		_filter.alwaysInsert(temp);


		temp.dim = 2.6;
		Coefs.setDim(3);
		Coefs[0] = T(1)/T(4);
		Coefs[1] = T(1)/T(2);
		Coefs[2] = T(1)/T(4);
		temp.values[0] = Coefs;
		Coefs.setDim(6);
		Coefs[0] = T(-5)/T(1024);
		Coefs[1] = T(5)/T(512);
		Coefs[2] = T(17)/T(512);
		Coefs[3] = T(-39)/T(512);
		Coefs[4] = T(-123)/T(1024);
		Coefs[5] = T(81)/T(256);	
		rev	= Coefs.reverse();
		Coefs.putlast(T(175)/T(256));
		Coefs.append(rev);
		temp.values[1] = Coefs;
		_filter.alwaysInsert(temp);


		temp.dim = 2.8;
		Coefs.setDim(3);
		Coefs[0] = T(1)/T(4);
		Coefs[1] = T(1)/T(2);
		Coefs[2] = T(1)/T(4);
		temp.values[0] = Coefs;
		Coefs.setDim(8);
		Coefs[0] = T(35);
		Coefs[1] = T(-70);
		Coefs[2] = T(-300);
		Coefs[3] = T(670);
		Coefs[4] = T(1228);
		Coefs[5] = T(-3126);	
		Coefs[6] = T(-3796);
		Coefs[7] = T(10718);	
		rev	= Coefs.reverse();
		Coefs.putlast(T(22050));
		Coefs.append(rev);
		temp.values[1] = Coefs*(1/powf(2.0f, 15.0f));
		_filter.alwaysInsert(temp);

		temp.dim = 3.1;
		Coefs.setDim(2);
		Coefs[0] = T(1)/T(8);
		Coefs[1] = T(3)/T(8);
		Coefs.append(Coefs.reverse());
		temp.values[0] = Coefs;
		Coefs.setDim(2);
		Coefs[0] = T(-1)/T(4);
		Coefs[1] = T(3)/T(4);
		Coefs.append(Coefs.reverse());
		temp.values[1] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 3.3;
		Coefs.setDim(2);
		Coefs[0] = T(1)/T(8);
		Coefs[1] = T(3)/T(8);
		Coefs.append(Coefs.reverse());
		temp.values[0] = Coefs;
		Coefs.setDim(4);
		Coefs[0] = T(3)/T(64);
		Coefs[1] = T(-9)/T(64);
		Coefs[2] = T(-7)/T(64);
		Coefs[3] = T(45)/T(64);
		Coefs.append(Coefs.reverse());
		temp.values[1] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 3.5;
		Coefs.setDim(2);
		Coefs[0] = T(1)/T(8);
		Coefs[1] = T(3)/T(8);
		Coefs.append(Coefs.reverse());
		temp.values[0] = Coefs;
		Coefs.setDim(6);
		Coefs[0] = T(-5)/T(512);
		Coefs[1] = T(15)/T(512);
		Coefs[2] = T(19)/T(512);
		Coefs[3] = T(-97)/T(512);
		Coefs[4] = T(-26)/T(512);
		Coefs[5] = T(350)/T(512);
		Coefs.append(Coefs.reverse());
		temp.values[1] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 3.7;
		Coefs.setDim(2);
		Coefs[0] = T(1)/T(8);
		Coefs[1] = T(3)/T(8);
		Coefs.append(Coefs.reverse());
		temp.values[0] = Coefs;
		Coefs.setDim(8);
		Coefs[0] = T(35);
		Coefs[1] = T(-105);
		Coefs[2] = T(-195);
		Coefs[3] = T(865);
		Coefs[4] = T(363);
		Coefs[5] = T(-3489);
		Coefs[6] = T(-307);
		Coefs[7] = T(11025);
		Coefs.append(Coefs.reverse());
		temp.values[1] = Coefs*(1/powf(2.0f, 14.0f));
		_filter.alwaysInsert(temp);

		temp.dim = 3.9;
		Coefs.setDim(2);
		Coefs[0] = T(1)/T(8);
		Coefs[1] = T(3)/T(8);
		Coefs.append(Coefs.reverse());
		temp.values[0] = Coefs;
		Coefs.setDim(10);
		Coefs[0] = T(-63);
		Coefs[1] = T(189);
		Coefs[2] = T(469);
		Coefs[3] = T(-1911);
		Coefs[4] = T(-1308);
		Coefs[5] = T(9188);
		Coefs[6] = T(1140);
		Coefs[7] = T(-29676);
		Coefs[8] = T(190);
		Coefs[9] = T(87318);
		Coefs.append(Coefs.reverse());
		temp.values[1] = Coefs*(1/powf(2.0f, 17.0f));
		_filter.alwaysInsert(temp);

		temp.dim = 4.4;
		Coefs.setDim(3);
		Coefs[0] = T(-.045635881557);
		Coefs[1] = T(-.028771763114);
		Coefs[2] = T(.295635881557);
		rev	= Coefs.reverse();
		Coefs.putlast(T(.557543526229));
		Coefs.append(rev);
		temp.values[0] = Coefs;
		Coefs.setDim(4);
		Coefs[0] = T(.026748757411);
		Coefs[1] = T(-.016864118443);
		Coefs[2] = T(-.078223266529);
		Coefs[3] = T(.266864118443);
		rev	= Coefs.reverse();
		Coefs.putlast(T(.602949018236));
		Coefs.append(rev);
		temp.values[1] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 5.5;
		Coefs.setDim(5);
		Coefs[0] = T(.009515330511);
		Coefs[1] = T(-.001905629356);
		Coefs[2] = T(-.096666153049);
		Coefs[3] = T(-.066117805605);
		Coefs[4] = T(.337150822538);
		rev	= Coefs.reverse();
		Coefs.putlast(T(.636046869922));
		Coefs.append(rev);
		temp.values[0] = Coefs;
		Coefs.setDim(4);
		Coefs[0] = T(.028063009296);
		Coefs[1] = T(.005620161515);
		Coefs[2] = T(-.038511714155);
		Coefs[3] = T(.244379838485);
		rev	= Coefs.reverse();
		Coefs.putlast(T(.520897409718));
		Coefs.append(rev);
		temp.values[1] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 6.8;
		Coefs.setDim(5);
		Coefs[0] = T(-0.01020092218704);
		Coefs[1] = T(-0.01023007081937);
		Coefs[2] = T(0.05566486077996);
		Coefs[3] = T(0.02854447171515);
		Coefs[4] = T(-0.29546393859292);
		rev	= Coefs.reverse();
		Coefs.putlast(T(-0.53662880179157));
		Coefs.append(rev);
		temp.values[0] = Coefs;
		Coefs.setDim(8);
		Coefs[0] = T(0.00134974786501);
		Coefs[1] = T(-0.00135360470301);
		Coefs[2] = T(-0.01201419666708);
		Coefs[3] = T(0.00843901203981);
		Coefs[4] = T(0.03516647330654);
		Coefs[5] = T(-0.05463331368252);
		Coefs[6] = T(-0.06650990062484);
		Coefs[7] = T(0.29754790634571);
		rev	= Coefs.reverse();
		Coefs.putlast(T(0.58401575224075));
		Coefs.append(rev);
		temp.values[1] = Coefs;
		_filter.alwaysInsert(temp);
	}


	template<class T>
	void SplineFilter<T>::calcFilterCoefs(double dim, FilterCoefs<T> &f) {

		assert("See _init() function for SplineFilter class\n");
			
	}  




} // namespace Wave_lib



#endif // __SplineFilter__