#ifndef SYMFILTER_H
#define SYMFILTER_H

#include <OrthoFilters.h>
#include <assert.h> //For feil rapportering....

#include <iostream>
//#include <complex>
using namespace std;

#include <GM_Array.h>
#include <GM_DVector.h>
#include <GM_DMatrix.h>
using namespace GM_lib;

namespace Wave_lib {

	template<class T>
	class SymFilter : public OrthoFilters<T> {
			
			void _init();					
			
			void calcFilterCoefs(double dim, FilterCoefs<T> &dbn);			// calculating Sym filters coefs
		public:
			SymFilter();
			~SymFilter();
	};

	// ************************************************************************
	// **************************   Constructors   ****************************
	// ************************************************************************
	
	template<class T>
	inline SymFilter<T>::SymFilter() : OrthoFilters<T>() 
	{
		_init();
	}


	// ************************************************************************
	// **************************    Destructors    ***************************
	// ************************************************************************

	template<class T>
	SymFilter<T>::~SymFilter() 
	{
	}

	// ************************************************************************
	// **************************   private func   ****************************
	// ************************************************************************

	template<class T>
	void SymFilter<T>::_init() 
	{


		FilterCoefs<T>	temp;
		DVector<T>		Coefs;

		temp.values.setSize(1);

		temp.dim = 1;
		Coefs.setDim(2);
		Coefs[0] = T(0.50000000000000);
		Coefs[1] = T(0.50000000000000);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 2;
		Coefs.setDim(4);
		Coefs[0] = T(0.34150635094622);
		Coefs[1] = T(0.59150635094587);
		Coefs[2] = T(0.15849364905378);
		Coefs[3] = T(-0.09150635094587);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 3;
		Coefs.setDim(6);
		Coefs[0] = T(0.23523360389270);
		Coefs[1] = T(0.57055845791731);
		Coefs[2] = T(0.32518250026371);
		Coefs[3] = T(-0.09546720778426);
		Coefs[4] = T(-0.06041610415535);
		Coefs[5] = T(0.02490874986589);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 4;
		Coefs.setDim(8);
		Coefs[0] = T(0.02278517294800);
		Coefs[1] = T(-0.00891235072085);
		Coefs[2] = T(-0.07015881208950);
		Coefs[3] = T(0.21061726710200);
		Coefs[4] = T(0.56832912170500);
		Coefs[5] = T(0.35186953432800);
		Coefs[6] = T(-0.02095548256255);
		Coefs[7] = T(-0.05357445070900);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 5;
		Coefs.setDim(10);
		Coefs[0] = T(0.01381607647893);
		Coefs[1] = T(-0.01492124993438);
		Coefs[2] = T(-0.12397568130675);
		Coefs[3] = T(0.01173946156807);
		Coefs[4] = T(0.44829082419092);
		Coefs[5] = T(0.51152648344605);
		Coefs[6] = T(0.14099534842729);
		Coefs[7] = T(-0.02767209305836);
		Coefs[8] = T(0.02087343221079);
		Coefs[9] = T(0.01932739797744);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 6;
		Coefs.setDim(12);
		Coefs[0] = T(-0.00551593375469);
		Coefs[1] = T(0.00124996104639);
		Coefs[2] = T(0.03162528132994);
		Coefs[3] = T(-0.01489187564922);
		Coefs[4] = T(-0.05136248493090);
		Coefs[5] = T(0.23895218566605);
		Coefs[6] = T(0.55694639196396);
		Coefs[7] = T(0.34722898647835);
		Coefs[8] = T(-0.03416156079324);
		Coefs[9] = T(-0.08343160770584);
		Coefs[10] = T(0.00246830618592);
		Coefs[11] = T(0.01089235016328);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 7;
		Coefs.setDim(14);
		Coefs[0] = T(0.00726069738101);
		Coefs[1] = T(0.00283567134288);
		Coefs[2] = T(-0.07623193594814);
		Coefs[3] = T(-0.09902835340368);
		Coefs[4] = T(0.20409196986287);
		Coefs[5] = T(0.54289135490599);
		Coefs[6] = T(0.37908130098269);
		Coefs[7] = T(0.01233282974432);
		Coefs[8] = T(-0.03503914561106);
		Coefs[9] = T(0.04800738396784);
		Coefs[10] = T(0.02157772629104);
		Coefs[11] = T(-0.00893521582557);
		Coefs[12] = T(-0.00074061295730);
		Coefs[13] = T(0.00189632926710);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

		temp.dim = 8;
		Coefs.setDim(16);
		Coefs[0] = T(0.00133639669640);
		Coefs[1] = T(-0.00021419715012);
		Coefs[2] = T(-0.01057284326418);
		Coefs[3] = T(0.00269319437688);
		Coefs[4] = T(0.03474523295559);
		Coefs[5] = T(-0.01924676063167);
		Coefs[6] = T(-0.03673125438038);
		Coefs[7] = T(0.25769933518654);
		Coefs[8] = T(0.54955331526901);
		Coefs[9] = T(0.34037267359439);
		Coefs[10] = T(-0.04332680770282);
		Coefs[11] = T(-0.10132432764282);
		Coefs[12] = T(0.00537930587524);
		Coefs[13] = T(0.02241181152181);
		Coefs[14] = T(-0.00038334544811);
		Coefs[15] = T(-0.00239172925575);
		temp.values[0] = Coefs;
		_filter.alwaysInsert(temp);

	}

	template<class T>
	void SymFilter<T>::calcFilterCoefs(double dim, FilterCoefs<T> &f) 
	{	
	int i,j;
		int center = 2*dim-1;
		int nn = 4*dim-1;
		T   n, m;
		DVector<T> P;
		P.setDim(nn);

		f.values.setSize(1);
		f.values[0].setDim(2*dim);

		// P = [a(N) 0 a(N-1) 0 ... 0 a(1) 1 a(1) 0 ... 0 a(N-1) 0 a(N)]
		P[center] = T(1.0);
		for (i=1; i<=dim; ++i) {
			n = m = T(1);
		
			for (j=(-dim+1); j<=dim; ++j) {
				if (j!=i) {
					m *= T(T(1)/T(2)-T(j));			
					n *= T(i-j);
				}
			}	
			m /= n;
			f.values[0][center+i] = m;
		} 

		QFile outfile( "FilterCoefficients" );
		if ( outfile.open( IO_WriteOnly ) ) 
		{
			QTextStream out( &outfile );
			for (i=0; i<f.values[0].dim(); i++)
			{
				out<<f.values[0][i] <<" ";
				out<<"\n\n";
			}
		outfile.close();
	}
}

}//end namespace Wave_lib

#endif //SYMFILTER_H
