#include <WaveDWT.h>
#include <GM_Tid.h>
 
using namespace GM_lib;
namespace Wave_lib
{
	template<class T>
	WaveDWT<T>::WaveDWT<T>(Filters<T> *filter,double dim,DWT_MODES _mode)
	{
		
		init();
		this->_filter = filter;
		setFilterIndex(dim);
		this->mode = _mode;
		if (mode == ZPD)
		{
			shift.setDim(1);
			shift[0] = 0;
		}
	}


	template<class T>
	WaveDWT<T>::~WaveDWT<T>()
	{
	}

	template<class T>
	void WaveDWT<T>::init()
	{
		_filter = 0;
		filterindex = 0;
	}

	template<class T>
	void WaveDWT<T>::setFilterIndex(double filterindex)
	{
		this->filterindex = filterindex;
		_filter->getFilterR(filterindex,lo_r,hi_r);
		_filter->getFilterD(filterindex,lo_d,hi_d);
	}

	template<class T>
	void WaveDWT<T>::decomposition(DVector<T> x, const int n,DVector<T> &c, DVector<int> &l)
	{
		_indata = x;
		Tid t;
		t.set();
		DVector<T> a;
		DVector<T> d;

		c.setDim(0);
		l.setDim(0);

		l.append(1, x.dim());
		cout <<"\nSet variabel"<<t.secAndSet();

		for (int i=0; i<n; ++i) {
			t.set();
			dwt(x, a, d);		// decomposition	//FEIL.....
			cout <<"\nQWT"<<t.secAndSet();
			x = a;
			c.putFirst(d);					// store detail
			l.putFirst(d.dim());			// store length
		}
		// Last approximation.
		c.putFirst(x);
		l.putFirst(x.dim());
	}

	template<class T>
	void WaveDWT<T>::dwt(DVector<T> x, DVector<T> &a, DVector<T> &d) 
	{


		Tid t;
		// Compute sizes.
		int lf = lo_d.dim();
		int lx = x.dim();

		int lenEXT;
		int lenKEPT;

		// Extend, Decompose &  Extract coefficients.
		if (mode != PER) {
			lenEXT = lf-1;
			lenKEPT = lx+lf-1;


		} else {
			lenEXT = lf/2;
			lenKEPT = 2*(int)ceil((double)lx/2);
		}

		// wextend(DVector<T> x,int l, int mode=MODE_ZPD, int loc=LOC_BOTH_SIDES)
		t.set();
		DVector<T> y = wextend(x, lenEXT, mode);	
		cout <<"\nWEXTEND "<<t.secAndSet();
		a = convdown(y,lo_d,lenKEPT,shift[0]);
		cout <<"\nCONVDOWN LO "<<t.secAndSet();
		d = convdown(y,hi_d,lenKEPT,shift[0]);
		cout <<"\nCONVDOWN HI "<<t.secAndSet();
	}

	template<class T>
	DVector<T> WaveDWT<T>::convdown(DVector<T> x, DVector<T> f, int lenKEPT, int shift) 
	{
		DVector<T> y;
		Tid t;

		t.set();
		//y =  
		y = convolution(x,f);
		cout <<"\n\tCONVOLUTION "<<t.secAndSet();
		y = wkeep(y,lenKEPT);	//Benyttes p 2 forskjellige mter 
		cout <<"\n\tWKEEP "<<t.secAndSet();
		y = dyaddown(y,shift);	
		cout <<"\n\tDYADOWN "<<t.secAndSet();

		return y;
	}

	//Har fikset delingen..
	//Har tildels feilsjekk......Mangler denne: Y = WKEEP(X,L,FIRST) returns the vector X(FIRST:FIRST+L-1).  .... Hva med center = dele 3 /2 gr ikke....
	template<class T>
	DVector<T> WaveDWT<T>::wkeep(DVector<T> x,int length,int type,int test)
	{
		DVector<T> ret;
		int dim = x.dim();
		if (dim<length) length = dim;

		int mid_dim = (dim/2);
		int start_pos=0;
		int end_pos=0;

		switch(type)
		{
		case LEFT :
			{ 
				start_pos = 0;
				end_pos = length;
				break;
			}
		case CENTER : 
			{

				int l_mid = (length / 2); 
				int d = int(double((length / 2.0)-l_mid)*10.0);
				int s=1;
				if (d==0) 
					s=0;

	//JG			int diff = length - (2*l_mid);
				start_pos = mid_dim - l_mid -((test==1)?s:0);
				end_pos = start_pos + length;      
				break;
			}
		case RIGHT :
			{
				end_pos = dim;
				start_pos = end_pos-length;
				break;
			}
		case FIRST :
			{
				break;
			}
		default:
			assert(!"Returning an empty DVector, from WKEEP (MATLAB_TOOLS.H");
			return ret;
		}

		int p=0;
		ret.setDim(length);
		for(int i=start_pos;i<end_pos;i++)
			ret[p++] = x[i];

		return ret;
	}


	template <class T>
	DVector<T>  WaveDWT<T>::dyaddown(DVector<T> x, int EvenOdd) 
	{
		int row = x.dim();
		int len = (int)floor((double)row/(double)2.0);
		DVector<T> temp(len);	// settes size for return value


		// hvis EvenOdd er Oddetall -> 0,2,4,6,8,... ellers 1,3,5,7,9,....
		int test = 1-(EvenOdd%2); // if odd->par, if par->odd
		for (int i=0;i<len; ++i)
			temp[i]= x[test+2*i];

		return temp;
	}



	template<class T>
	DVector<T> WaveDWT<T>::wextend(DVector<T> x,int l, DWT_MODES mode, LOC loc)
	{
		DVector<T> ret;
		int i;
		int l1 = l * (((loc&LOC_LEFT_SIDE)==LOC_LEFT_SIDE)?1:0);
		int l2 = l * (((loc&LOC_RIGHT_SIDE)==LOC_RIGHT_SIDE)?1:0);
		T *left_side = new T[l1];
		T *right_side = new T[l2];

		switch(mode)
		{
		case ZPD:
			{
				for (i=0;i<l;i++)
				{
					if (l1) left_side[i] = 0.0;
					if (l2) right_side[i] = 0.0;
				}
				break;
			}
		case SP0:
			{
				assert(!"Mode Smooth extension  in wextend is not implemented....");
				break;
			}
		case SP1:
			{
				assert(!"Mode Smooth extension  in wextend is not implemented....");
				break;
			}
		case SYM:
			{
				if (l>x.dim()) assert(!"The function is not finnished, it can't get values bigger than the dimensjon of the vector....");
				for (i=0;i<l;i++)
				{
					if (l1) left_side[l-i-1] = x[i];
					if (l2) right_side[i] = x[x.dim()-i-1];

				}
				break;
			}
		case PPD:
			{
				assert(!"Mode Periodic in wextend is not implemented....");
				break;
			}
		case PER:
			{
				assert(!"Mode Periodic in wextend is not implemented....");
				break;
			}
		default :
			assert(!"A not defined mode in wextend....");
			return ret;
			break;
		}

		int s = x.dim() + l1 + l2;

		ret.setDim(s);
		int p=0;
		for(i=0;i<l1;i++)
			ret[p++] = left_side[i];

		for(i=0;i<x.dim();i++)
			ret[p++] = x[i];

		for(i=0;i<l2;i++)
			ret[p++] = right_side[i];

		delete [] left_side;
		delete [] right_side;
		return ret;
	}



	template<class T>
	DVector<T> WaveDWT<T>::reconstruction(DVector<T> &_coefs,DVector<int> &_longs)
	{
			coefs = _coefs;
			longs = _longs;
			if (!_filter) assert(!"There is no filter constructed. Do not use WaveDWT directly, but use feks waveCoiflets");
			return  appcoef(0);
	}


	

	template<class T>
	DVector<T> WaveDWT<T>::appcoef(int s)
	{
		DVector<T> *d;
		DVector<T> a;
		DVector<int> sf;
		int rmax = longs.dim(); 
		int nmax = rmax-2;	
		int p;
		int imax = rmax + 1;	
		int n = s;

		sf.setDim(1);

		a.setDim(longs[0]);
		
		for (p=0;p<a.dim();p++)	//Kopierer deler av tabellen
			a[p] = coefs[p];
		

		for (p=nmax; p>n;p--)
		{
			sf[0] = p;

			d = detcoef(sf);	//Funker som bare det
			a = idwt(a,d[0],longs[imax-p-1]);
		}
		return a;
	}


	template<class T>
	DVector<T> *WaveDWT<T>::detcoef(DVector<int> levels)
	{
		DVector<T> *tmp;
		DVector<int> f1,l1;
		DVector<int> first;
		DVector<int> last;
	//JG	int nmax = longs.dim()-2;
//JG		int cellFlag = 0;
		int i;
		int u=0;
		int k;
		int nblev; 
		int h,l,z;

		first = cumulative_sum(longs); 
		first+=1;								//Raskere  legge inn tallet 1 slik

		f1.setDim(first.dim()-2);
		for (i=first.dim()-3;i>=0;i--)			//Kopierer tabeller speilvendt..OK
			f1[u++] = first[i];

		l1.setDim(longs.dim()-2);
		u=0;
		for(i=longs.dim()-2;i>=1;i--)			//Kopierer tabeller speilvendt..
			l1[u++] = longs[i];

		last =  f1+l1;
		last+=-1;								//Raskere  legge inn verdier slik

		nblev =  levels.dim();
		tmp = new DVector<T>[nblev];

		for (i=0;i<nblev;i++)
		{
			k=levels[i]-1;		
			z=0;
			h=f1[k];		
			l=last[k];		
			tmp[i].setDim(l-h+1);
			for (int j=h;j<tmp[i].dim()+h;j++) 
				tmp[i][z++]= coefs[j-1];
		}
		return tmp;
	}



	template<class T>
	DVector<T> WaveDWT<T>::idwt(DVector<T>a,DVector<T>d,int l)
	{
		return upsaconv(a,lo_r,l)+upsaconv(d,hi_r,l);
	}


	template<class T>
	DVector<T> WaveDWT<T>::upsaconv(DVector<T> x,DVector<T> f,int s) //Er mer eller mindre ferdig, ikke testet ut, en del parameterverdier som er bare satt og ikke brukt, m se nermere p dette
	{
		DVector<T> sd,y = x;
		//int shift=0;
		//y = 
		y = convolution(dyadic_upsampling(y,0),f);
		y = wkeep(y,s,shift[0],1);
		return y;
	}


	//Skal funke som bare det legger inn 0 ved odde tall eller partall i listen //FUNKER IKKE SLIK SOM PLANLAGT, bytte om odd/even
	template<class T>
	DVector<T>  WaveDWT<T>::dyadic_upsampling(DVector<T> x,int odd) 
	{
		odd = odd%2;
		DVector<T> ret;
		ret.setDim((x.dim()*2)-1+odd); 
		T D;
		for (int i=0;i<x.dim();i++)
		{
			D = x[i];
			ret[i*2+odd] = x[i];
			if (!odd&&i==x.dim()-1) break;
			ret[i*2+1-odd] = 0;
		}
		return ret;
	}



	template<class T>
	DVector<int> WaveDWT<T>::cumulative_sum(DVector<int>x)
	{
		DVector<int> ret;
		ret.setDim(x.dim());
		if (ret.dim()==0) return ret;
		ret[0] = x[0];
		T d;
		for (int i=1;i<x.dim();i++)
		{
			ret[i]=x[i]+ret[i-1];
			d = ret[i];
		}
		return ret;
	}


	template<class T>
	DVector<T> WaveDWT<T>::convolution(DVector<T> x,DVector<T> F)	
	{
		DVector<T> ret;

		T s;
		int i,j,u,p;

		ret.setDim(x.dim()+F.dim()-1);
		
		u=0;
		for (j=-x.dim();j<F.dim()-1;j++)
		{
			ret[u] = 0.0;
				for (i=0;i<x.dim();i++)
			{
				s = 0;
				p = x.dim()-i+j;
				
				if (0<=p &&p<F.dim()) s = F[p];
				if (p<0) break;
				ret[u]+= x[i] * s;
			}
			u++;
		}
		return ret;
	}

} //end namespace Wave_lib
