#include <WaveDWT2D.h>



namespace Wave_lib
{

	template<class T>
	WaveDWT2D<T>::WaveDWT2D<T>(Filters<T> *filter,DWT_MODES mode = ZPD)
	{
		this->mode = mode;
		shift.setDim(2);
		shift[0] = 0;
		shift[1] = 0;
		this->_filter = filter;
		setFilterIndex(_filter->getFilterDim());
	}


	template<class T>
	void WaveDWT2D<T>::setFilterIndex(int filterindex)
	{
		this->filterindex = filterindex;
		_filter->getFilterR(filterindex,lo_r,hi_r);
		_filter->getFilterD(filterindex,lo_d,hi_d);
	}

	template<class T>
	WaveDWT2D<T>::~WaveDWT2D<T>()
	{
	
	}



	template<class T>
	DMatrix<T> WaveDWT2D<T>::reconstructon(DVector<T> &_coefs,DMatrix<int> &_longs)
	{
		this->coefs = _coefs;
		this->longs = _longs;
		DMatrix<T> res = appcoef(coefs,longs);
		return res;
	}


	template<class T>
	void WaveDWT2D<T>::decomposition(DMatrix<T> mat, const int n,DVector<T> &c, DMatrix<int> &s)
	{

		int col = mat.dim1();
		int row = mat.dim2();
		c.setDim(n);

		DMatrix<T> ca;
		DMatrix<T> ch;
		DMatrix<T> cv;
		DMatrix<T> cd;
		int i;

		c.setDim(0);
		s.setdim(0,0);

		s.addRow(mat.dim(), false);

		for (i=0; i<n; ++i) {
			dwt(mat, ca, ch, cv, cd);
			mat = ca;
			c.putfirst(cd.toVector(false));
			c.putfirst(cv.toVector(false));
			c.putfirst(ch.toVector(false));
			
			s.addRow(mat.dim(), false);		// add row as first row
		}
		c.putfirst(mat.toVector(true));
		s.addRow(mat.dim(), false);
	}


	template<class T>
	void WaveDWT2D<T>::dwt(DMatrix<T> mat, DMatrix<T> &ca, DMatrix<T> &ch, DMatrix<T> &cv, DMatrix<T> &cd) 
	{
			/*dwtATTR temp = dwtmode(zpd);

			int shift1 = temp.shift2D[0];
			int shift2 = temp.shift2D[1];
			dwtmodes dwtEXTM = temp.extMode;*/

			int shift1 = shift[0];
			int shift2 = shift[1];

			// Compute sizes.
			int lf = lo_d.dim();

			//sx = size(x);
			DVector<int> sx = mat.dim();
			int sizeEXT;
			DVector<int> sizeKEPT(2);

//			if (dwtEXTM != PER) {
				sizeEXT = lf-1; 
				sizeKEPT = sx+(lf-1);
/*			} else {
				sizeEXT = lf/2; 
				sizeKEPT[0]= 2*(int)ceil((double)sx[0]/2.0);
				sizeKEPT[1]= 2*(int)ceil((double)sx[1]/2.0);
			}*/

			DVector<int> ss(2);
			ss.clear(sizeEXT);
			DMatrix<T> y = wextend(mat, ss[0],LOC_BOTH_SIDES,ZPD);//Funker Funker 
			DMatrix<T> z;
			// convert down y with LowPass filter (Lo_D)
			z = convolution2(y, lo_d,ROW);	//Funker				// z = wconv('row',y,Lo_D);
			ca = convdown(z,lo_d,sizeKEPT,shift1,shift2);	//Funker
			ch = convdown(z, hi_d, sizeKEPT, shift1,shift2); //Funker ikke
			// convert down y with HighPass filter (Hi_D)
			z = convolution2(y, hi_d,ROW);
			cv = convdown(z, lo_d, sizeKEPT, shift1, shift2);
			cd = convdown(z, hi_d, sizeKEPT, shift1, shift2);
		}



	template<class T>
	DMatrix<T> WaveDWT2D<T>::wextend(DMatrix<T> m,int l,int loc,int mode)
	{
		int l1 = l * (((loc&LOC_LEFT_SIDE)==LOC_LEFT_SIDE)?1:0);
		int l2 = l * (((loc&LOC_RIGHT_SIDE)==LOC_RIGHT_SIDE)?1:0);

		DMatrix<T> ret2(m.dim1()+ 2*l1 , m.dim2()+ 2*l2);
		ret2.clear(0);

		switch(mode)
		{
		case ZPD:
			ret2.insert(m,l1,l2);
			break;
		default:
			assert(!"This funciont dose not exist...");
			break;
		}
        return ret2;
	}


	template<class T>
	void write(DMatrix<T> Y,char *text)
	{
		int x,y;
		cout <<"\n\nSkriver ut: "<<text<<" : Size of matrix = "<<Y.size()<<"\n\n";
		for(y=0;y<Y.dim2();y++)
		{
			cout <<"\n";
			for (x=0;x<Y.dim1();x++)
			{
				cout <<Y[x][y]<<"|";
			}
		}
		cout <<"\n\n";
		
	}


	template<class T>//Funker
	DMatrix<T> WaveDWT2D<T>::convdown(DMatrix<T> mat, DVector<T> f, DVector<int> sizeKEPT, int shift1, int shift2)	
	{
			DMatrix<T> y;
			y = convolution2(mat,f,COLUMN); // one dimensional case 
			y = wkeep(y,sizeKEPT[0],sizeKEPT[1]); //LAgt til sizeKEPT[1]
			y = dyaddown(y,COLUMN, shift2);
			y = dyaddown(y,ROW, shift1);
			return y;
	}


	template <class T>
	DMatrix<T> WaveDWT2D<T>::dyaddown(DMatrix<T> mat, int Type, int EvenOdd) {

			DMatrix<T> y;
			DVector<T> temp;
			int i, j;

			int first = 1-EvenOdd%2;
			DVector<int> len = mat.dim();

			switch (Type) {
		case ROW: // removes every second column vector
			for (i=first;i<len[1]; i+=2) {
				temp = mat.getSubVector(true, 0, len[0], i); //Funker //Endre fra false til true
				y.addRow(temp);
			}	
			break;
		case COLUMN: // removes every second row vector
			for (i=first;i<len[0]; i+=2)
				y.addCol(mat[i]);
			break;
		case COLUMN_ROW: // removes every second row & column
			assert(!"Denne stemmer mest ikke, pga endringer i addcol og addrow, de ha byttet plass");
			for (i=first;i<len[0]; i+=2) {
				temp.setDim(0);
				for (j=first;j<len[1]; j+=2)
					temp.append(1, mat[i][j]);
				y.addRow(temp);
			}
			break;
		default:
			assert(!"Unknown type as input to dyadown function");
			break;
			}
			//[r,c] = SIZE(x) -- matlab
			return y;
		}

	//Ikke testet helt ut, usikker om den funker som dne skal
	template<class T>
		DMatrix<T> WaveDWT2D<T>::convolution2(DMatrix<T> mat, DVector<T> F,int type)
	{
		DMatrix<T> ret2;
		int index = 0;
		int diff = 0;
		int i,j,x,y;
		T value;
		if (type==COLUMN) //Funker ikke
		{
			ret2.setdim(mat.dim1(), mat.dim2()+F.dim()-1);
			ret2.clear(0);
			for(x=0;x<ret2.dim1();x++)
			{
				for(y=0;y<ret2.dim2();y++)
				{
					value = 0;
					for (i=0,j=y;i<=y;i++,j--)
						if (i<mat.dim2() && j<F.dim())
							value += mat[x][i] * F[j];

					ret2[x][y] = value;
				}
			}
			return ret2;
		}else //Funker 
		{
			ret2.setdim(mat.dim1()+F.dim()-1, mat.dim2());
			ret2.clear(0);
			for(y=0;y<ret2.dim2();y++)
			{
				for(x=0;x<ret2.dim1();x++)
				{
					value = 0;
					for (i=0,j=x;i<=x;i++,j--)
						if (i<mat.dim1() && j<F.dim())
							value += mat[i][y] * F[j];

					ret2[x][y] = value;
				}
			}
			return ret2;
		}
	}



	template<class T>
	DMatrix<T> WaveDWT2D<T>::appcoef(DVector<T> c,DMatrix<int> l,int s)
	{
		DVector<DMatrix<T> > _detcoef;
		DVector<T> b;
		DMatrix<T> a,c1,c2,c3;
		int rmax = l.dim2(); 
		int nmax = rmax-2;	
		int imax = rmax + 1;	//Tror ikke denne skal vre her
		int nl,nc,p,rm;
		int n = s;
		nl = l[0][0];
		nc = l[0][1];

		a.setdim(nl,nc);
		a.clear();
		a.insert(c,nl*nc);
		rm = rmax;
		for (p=nmax; p>n;p--)
		{
			detcoef(ALL,c,l,p,c1,c2,c3);
			a = idwt(a,c1,c2,c3,l.getRow(rm-p));	///DENNE SKAL FIKSES......
		}
		return a;
	}

	template<class T>
	DMatrix<T> WaveDWT2D<T>::idwt(DMatrix<T>a,DMatrix<T>h,DMatrix<T>v,DMatrix<T>d,DVector<int> s)
	{
		int next = 3;
		int k = next;
		//DWT_Attribute dwtATTR = WaveletManager::getDWTmode();

		return upsaconv(a,lo_r,lo_r,s)+upsaconv(h,hi_r,lo_r,s)+upsaconv(v,lo_r,hi_r,s)+upsaconv(d,hi_r,hi_r,s);
	}


	template<class T>
	DMatrix<T> WaveDWT2D<T>::upsaconv(DMatrix<T>a,DVector<T>f1,DVector<T>f2,DVector<int>s)
	{
		DMatrix<T> ts,r, y = a;
		DVector<int> sx = y.dim();
		int lf = f1.dim();

		y = convolution2(dyadic_upsampling(y,ROW,0),f1,COLUMN); //Feiler her
		y = convolution2(dyadic_upsampling(y,COLUMN,0),f2,ROW);//Feiler her.......
		y = wkeep(y,s[0],s[1],CENTER);	//Mangler en funksjon her ...     [first,last,ok] = GetFirstLast(sx,begInd,varargin{:});  //Tror ikke den siste delen som er lagt til er av betydning
		return y;
	}

	
	//Ikke helt ferdig........Ingen feilsjekk....finnes ikke: Y = WKEEP(X,S,[FIRSTR,FIRSTC]) 
	//Denne fungerer....
	template<class T>
	DMatrix<T> WaveDWT2D<T>::wkeep(DMatrix<T> m,int length_column,int length_row,int type)	
	{
		DMatrix<T> ret4;
		int row_size = m.dim1();
		int column_size = m.dim2();
		int row_mid;
		int column_mid;
		int start_row;
		int start_column;

		switch(type)
		{
		case CENTER:
			row_mid = (int)ceil(row_size / 2.0);
			column_mid = (int)ceil(column_size / 2.0);
			start_row = row_mid - (int)ceil(length_row/2.0);
			start_column = column_mid - (int)ceil(length_column/2.0);
			break;
		default:
			assert(!"Only type=center exists....wkeep(DMatrix<T> m,int length_column,int length_row,int type=CENTER) (MATLAB_TOOLS.H)");
			break;
		}

		ret4.setdim(length_column,length_row);

		for (int j=0;j<length_row;j++)
			for (int i=0;i<length_column;i++)
				ret4[i][j] = m[i+start_row][j+start_column];

		return ret4;
	}


	template<class T>
	DMatrix<T> WaveDWT2D<T>::dyadic_upsampling(DMatrix<T> x,int type,int odd) 
	{
		odd = 1-odd;
		DMatrix<T> ret3;
		int c = 1;//((type&COLUMN)==COLUMN)?2 : 1;
		int r = 1;//((type&ROW)==ROW)?2 : 1;

		int odd1 = 1-odd;
		int odd2 = 1-odd;
		int s1,s2;
		s1 = s2 = 0;

		if ((type&ROW)==ROW)
		{
			c = 2;
			odd1 = 1-odd;
			if (odd) s2 = -1;
			else s2 = 1;
		}
		if ((type&COLUMN)==COLUMN)
		{
			r = 2;
			odd2 = 1-odd;
			if (odd) s1 = -1;
			else s1 = 1;
		}

		ret3.setdim(x.dim1()*r+s1,x.dim2()*c + s2);

		int i,j,t1,t2;


		for (j=0;j<ret3.dim2();j++)
			for (i=0;i<ret3.dim1();i++)
				ret3[i][j] = 0;


		for (j=0;j<x.dim2();j++)
			for (i=0;i<x.dim1();i++)
			{
				t1 = r*i+((r-1)*odd2);
				t2 = c*j+((c-1)*odd1);
				ret3[t1][t2] = x[i][j];
			}
			return ret3;
	}

	template<class T>
	void WaveDWT2D<T>::detcoef(DETCOEF mode,DVector<T> coefs,DMatrix<int> longs,int levels,DMatrix<T> &c1,DMatrix<T> &c2,DMatrix<T> &c3)
	{
		int first,last;
		int nmax = longs.dim2()-2; //-2
		int k = longs.dim2()-levels;
		DVector<int> t1,t2;
		t1 = longs.getSubVector(false,1,k-1,0);
		t2 = longs.getSubVector(false,1,k-1,1);
		first = longs[0][0]*longs[0][1] + ((t1 * t2)*3);

		int add = longs[0][k-1] * longs[1][k-1];  

		switch(mode)
		{
		case ALL:
			{
				last = first+add-1;
				c1 = coefs.vectorToMatrix(first,last,longs[0][k-1],longs[1][k-1]);
				first = first+add;
				last = first+add-1;
				c2 = coefs.vectorToMatrix(first,last,longs[0][k-1],longs[1][k-1]);
				first = first+add;
				last = first+add-1;
				c3 = coefs.vectorToMatrix(first,last,longs[0][k-1],longs[1][k-1]);
				break;
			}
		default:
			assert(!"This mode does not exist....");
			break;
		}
	}

}//namespace Wave_lib