#include "WaveDWTnD.h"
#include <GM_Tid.h>
 
using namespace GM_lib;

namespace Wave_lib
{
	template<class T, int n>
	WaveDWTnD<T,n>::WaveDWTnD<T,n>(Filters<T> *filter,double fdegree,DWT_MODES _mode)
	{
		this->mode = _mode;
		
		this->_filter = filter;
		getFilterCoefs(fdegree);
		
		switch(mode)
		{
		case ZPD:
            shift.setDim(n);
			for(int i=0; i<n; i++)	shift[i] = 0;
			break;
		default:
			assert(!"Wrong DWT_mode or it is not supported yet");
			break;
        }//end switch

		testinit();
	}

	template<class T, int n>
	WaveDWTnD<T,n>::~WaveDWTnD<T,n>(){}

	//////////////////////////////////////////////////////////
	//   Private functions									//
	//////////////////////////////////////////////////////////
	template<class T, int n>
	void WaveDWTnD<T,n>::getFilterCoefs(double fd)
	{
		this->fdegree = fd;
		_filter->getFilterR(fdegree, lo_r, hi_r);
		_filter->getFilterD(fdegree, lo_d, hi_d);	
	}

	/*****************************************************************
	Decompose the insignal into a array of coefs. Handles also 
	multilevels. Returning the array
	******************************************************************/
	template<class T, int n>
	void WaveDWTnD<T,n>::decomposition(DArrayN<T,n> mat, const int level, Array<DArrayN<T,n>* > &c, Array<DVector<int>* > &l)
	{
		DVector<int> msize;
		mat.getSize(msize);
		
		int nrcoefs = (int) pow(2,n); //approx + nr of det.
		int csize = (nrcoefs-1)*level+1;
		c.setSize(csize,NULL);
		
		int lsize = level+2;
		l.setSize(lsize, NULL);
		lsize--;
		if(l[lsize]) delete l[csize];
		l[lsize] = new DVector<int> (msize);

		DArrayN<T,n> tmpmat;
		tmpmat = mat;
		DArrayN<T,n>* pmat;
		pmat =&tmpmat;
		

		for(int t=0; t<level; t++)
		{
			Array<DArrayN<T,n>*> coefs;
			dwt(&tmpmat,coefs);
			tmpmat = *coefs[0];
			
			DVector<int> tmp;
			tmpmat.getSize(tmp);
			lsize--;
			if(l[lsize]) delete l[lsize];
			l[lsize] = new DVector<int> (tmp);
			
			for(int j=nrcoefs-1; j>0; j--)
			{
				csize--;
				if(c[csize]) delete c[csize];
				c[csize] = new DArrayN<T,n> (*coefs[j]);
			}//j
				
		}//for t

		//Last approximation
		if(c[0]) delete c[0];
		c[0] = new DArrayN<T,n> (tmpmat);
		
		DVector<int> tmp;
		tmpmat.getSize(tmp);
		if(l[0]) delete l[0];
		l[0] = new DVector<int> (tmp);

	//	printArrayMatrix(c,"DecArray.txt");
	//	printArrVector(l,"DecLongs1.txt");
		
	}

	/*****************************************************************
	Reconstruct the insignal from a array of coefs. Handles also 
	multilevels. Returning the reconstructed insignal
	******************************************************************/
	template<class T, int n>
	DArrayN<T,n> WaveDWTnD<T,n>::reconstruction(Array<DArrayN<T,n>*> c,Array<DVector<int>* > l)
	{
		DArrayN<T,n> ret;
		int nrdet = (int)pow(2,n)-1;
		int levels = (int)(c.size()-1)/nrdet;

		Array<DArrayN<T,n>*> coefs;
		coefs.setSize(1+nrdet,NULL);
		
        //init 1.approx	
		ret = *c[0];
		
		//printMatrix(ret,"RecRet.txt");

		for(int i=0; i<levels; i++)
		{
			coefs[0] = &ret;
	
			for(int j=0; j<nrdet; j++)
			{
				coefs[j+1] = c[j+1+nrdet*i];
			}
			ret = idwt(coefs,*l[i+2]);
		}

		return ret;
	}
	/***************************************************************
	dwt are doing a single levele discrete wavelet transform, 
	returning a array of pointert to the new coef.
	****************************************************************/
	template<class T, int n>
	void WaveDWTnD<T,n>::dwt(DArrayN<T,n>* mat, Array<DArrayN<T,n>*> &c)
	{
		DArrayN<T,n> mymat;
		mymat = *mat;
		int csize = (int)pow(2,n);
		c.setSize(csize,NULL);

		DVector<int> msize;
		mat->getSize(msize);
		
		for(int i=0; i<csize; i++)
		{
			Array<DVector<T>*> tmpF;
			tmpF.setSize(n,NULL);
			
			DVector<T> tmpvec;

			DVector<int> sizeKEEP;
			sizeKEEP.setDim(n);
			DVector<int> lenEXT;
			lenEXT.setDim(n);
			
			for(int j=0; j<n; j++)
			{
				if(i & (int)pow(2,j)) tmpvec = hi_d;
				else				  tmpvec = lo_d;

				if(tmpF[n-1-j]) delete tmpF[n-1-j];
				tmpF[n-1-j] = new DVector<T> (tmpvec);

				sizeKEEP[n-1-j] = (msize[n-1-j]+tmpvec.dim()-1);
				lenEXT[n-1-j]   = tmpvec.dim()-1;
				
			}
    			 
			DArrayN<T,n> tmpmat;
			tmpmat = wextend(mymat,lenEXT);

			tmpmat = convdown(&tmpmat,tmpF,shift,sizeKEEP);

		//	printDArray(tmpmat, "DWTmat.txt");
			
			if(c[i]) delete c[i];
			c[i] = new DArrayN<T,n> (tmpmat);
		}
	}

	/***************************************************************
	idwt are doing a single levele inverse discrete wavelet transform, 
	returning a DMatrix containing the new coef.
	****************************************************************/
	template<class T, int n>
	DArrayN<T,n> WaveDWTnD<T,n>::idwt(Array<DArrayN<T,n>*> c, DVector<int> sizeKEEP)
	{
		DArrayN<T,n> ret;
		int csize = c.size();
		
		DArrayN<T,n> tmpmat;
		tmpmat = *c[0];
		DVector<int> msize;
		tmpmat.getSize(msize);
		
		for(int i=0; i<csize; i++)
		{
			//generating filter table
			Array<DVector<T>*> tmpF;
			tmpF.setSize(n,NULL);
			
			DVector<T> tmpvec;
			for(int j=0; j<n; j++)
			{
				if(i & (int)pow(2,j)) tmpvec = hi_r;
				else				  tmpvec = lo_r;

				if(tmpF[n-1-j]) delete tmpF[n-1-j];
				tmpF[n-1-j] = new DVector<T> (tmpvec);
			}

			if(i == 0) 
			{
				ret.setSize(sizeKEEP);
				ret.setElements();
			}

			
			DArrayN<T,n> tmpmat;
			tmpmat = upsaconv(c[i],tmpF,sizeKEEP);

			//printMatrix(tmpmat, "TmpMat.txt");

			ret=ret+tmpmat;
		}
	//	printDArray(ret, "idwtMat.txt");

		return ret;
	}
	/*****************************************************************
	convolv, keep and dyaddown in all direction and return the new
	DArrayN.
	*****************************************************************/
	template<class T, int n>
	DArrayN<T,n> WaveDWTnD<T,n>::convdown(DArrayN<T,n>* mat, Array<DVector<T>*> F, DVector<int> shift, DVector<int> sizeKEEP)
	{
		DArrayN<T,n> ret;
		ret = *mat;
	//	printMatrix(ret, "BeforeConvMat.txt");

		if(F.size() == n) 
		{
			DVector<T> F3;
			F3.setDim(2);
			F3[0]=T(0.5); F3[1]=T(0.5); 
			
			//DVector<T> filt;
			for(int i=0; i<n; i++)
			{
				DVector<T> filt = *F[i];
				ret=ret.conv(filt,i+1);
			}	

	//		printMatrix(ret, "ConvMat.txt");

			ret=wkeep(ret,sizeKEEP);

	//		printMatrix(ret, "KeepMat.txt");

			for(int i=0; i<n; i++) 
			{
				ret = ret.dyaddown(i+1,shift); //legg til shift
			}

	//		printMatrix(ret, "DyadMat.txt");

		}
		else {assert("Wrong size of the filter array in convdown");}
	
		return ret;
	}

	/*****************************************************************
	duadup, convolv and keep in all direction and return the new
	DArrayN.
	*****************************************************************/
	template<class T, int n>
	DArrayN<T,1> WaveDWTnD<T,n>::upsaconv1d(DArrayN<T,1>* mat, Array<DVector<T>*> F, DVector<int> sizeKEEP)
	{
		DArrayN<T,1> ret;
		ret = *mat;
		
		if(F.size() == 1) 
		{	
			DVector<T> filt;
			
			filt = *F[0];
			ret = ret.dyadup(1,shift);
			ret=ret.conv(filt,1);
			ret=wkeep1d(ret,sizeKEEP);

		}
		else {assert("Wrong size of the filter array in convdown");}

		return ret;

	}


	/*****************************************************************
	duadup, convolv and keep in all direction and return the new
	DArrayN.
	*****************************************************************/
	template<class T, int n>
	DArrayN<T,n> WaveDWTnD<T,n>::upsaconv(DArrayN<T,n>* mat, Array<DVector<T>*> F,/* DVector<int> shift,*/ DVector<int> sizeKEEP)
	{
		DArrayN<T,n> ret;
		ret = *mat;
		
		if(F.size() == n) 
		{	
			DVector<T> filt;
			for(int i=0; i<n; i++) 
			{	
				filt = *F[i];
				ret = ret.dyadup(i+1,shift);
				ret=ret.conv(filt,i+1);

			//	printMatrix(ret, "UpsaConMat.txt");
			//	int dummy =0;
			}

			//int tmp1 = sizeKEEP[0];
			//int tmp2 = sizeKEEP[1];

			//printMatrix(ret, "UpsaConMat.txt");

			ret=wkeep(ret,sizeKEEP);

			//printMatrix(ret, "UpsaConMat.txt");
		}
		else {assert("Wrong size of the filter array in convdown");}

		return ret;

	}

	/********************************************************************
	Extend the DArrayN m in all direction with the length l, returning
	the new DArrayN
	********************************************************************/
	template<class T, int n>
	DArrayN<T,n> WaveDWTnD<T,n>::wextend(DArrayN<T,n> m,DVector<int> lenEXT,int loc,int mode)
	{
		DArrayN<T,n> retmat;
		DVector<int> msize;
		DVector<int> retsize;
		retsize.setDim(n);
		
		m.getSize(msize);
		for(int i=0; i<n; i++)	retsize[i]=msize[i]+2*lenEXT[i];
		retmat.setSize(retsize);
		retmat.setElements();

	/*	DVector<int> pos;
		for(int i=0; i<n; i++)
		{
			pos.append(1,l);
		}
*/
		switch(mode)
		{
		case ZPD:
			retmat.insertSubArray(m,lenEXT);
			break;
		default:
			assert(!"This mode is not implemented in wextend");
			break;
		}//end switch

		//printMatrix(m,"Amatrix.txt");
		//printMatrix(retmat,"AAmatrix.txt");

		return retmat;
	}

/********************************************************************
	Keep the part of a DMatrix spesified with the length and type 
	(only type=CENTER is implemented) and return the new DArrayN
	********************************************************************/
	template<class T, int n>
	DArrayN<T,1> WaveDWTnD<T,n>::wkeep1d(DArrayN<T,1> m, DVector<int> length, int type)
	{
		DArrayN<T,1> retmat;
		//retmat.setSize(length);
		//retmat.clear();
		DVector<int>  msize;
		m.getSize(msize);
	
		DVector<int> mid;
		DVector<int> first;
		DVector<int> last;

		mid.setDim(1); first.setDim(1); last.setDim(1);
		
	/*	/*DVector<int> start;
		start.append(1,1); start.append(1,1); start.append(1,1);
		DVector<int> end;
		end.append(1,4); end.append(1,4); end.append(1,3);
		*/
		switch(type)
		{
		case CENTER:
			for(int i=0; i<1; i++)
			{
				double d = ((double)(msize[i]-length[i]))/2.0;
				int d1 = (int)floor(d);
				int d2 = msize[i]-(int)ceil(d)-1;
				first[i] = d1;//(int)floor(d);
				last [i] = d2;//msize[i]-(int)ceil(d)-1;

/*				int size = msize[i];
				int tm =(int)ceil(double(size/2));
				mid[i] = tm;
				int lengde = length[i];
				int tz = mid[i]-(int)ceil(double(lengde/2));
				start[i] = tz;
				int ts =start[i]+(length[i]-=1);
				end[i] = ts;
				int dummy=0;
				*/
			}
			break;
		default:
			assert(!"this type is not implemented in wkeep");
			break;
		}
		retmat=m.getSubArray(first,last);
		
		return retmat;
	}



	/********************************************************************
	Keep the part of a DMatrix spesified with the length and type 
	(only type=CENTER is implemented) and return the new DArrayN
	********************************************************************/
	template<class T, int n>
	DArrayN<T,n> WaveDWTnD<T,n>::wkeep(DArrayN<T,n> m, DVector<int> length, int type)
	{
		DArrayN<T,n> retmat;
		//retmat.setSize(length);
		//retmat.clear();
		DVector<int>  msize;
		m.getSize(msize);
	
		DVector<int> mid;
		DVector<int> first;
		DVector<int> last;

		mid.setDim(n); first.setDim(n); last.setDim(n);
		
	/*	/*DVector<int> start;
		start.append(1,1); start.append(1,1); start.append(1,1);
		DVector<int> end;
		end.append(1,4); end.append(1,4); end.append(1,3);
		*/
		switch(type)
		{
		case CENTER:
			for(int i=0; i<n; i++)
			{
				double d = ((double)(msize[i]-length[i]))/2.0;
				int d1 = (int)floor(d);
				int d2 = msize[i]-(int)ceil(d)-1;
				first[i] = d1;//(int)floor(d);
				last [i] = d2;//msize[i]-(int)ceil(d)-1;

/*				int size = msize[i];
				int tm =(int)ceil(double(size/2));
				mid[i] = tm;
				int lengde = length[i];
				int tz = mid[i]-(int)ceil(double(lengde/2));
				start[i] = tz;
				int ts =start[i]+(length[i]-=1);
				end[i] = ts;
				int dummy=0;
				*/
			}
			break;
		default:
			assert(!"this type is not implemented in wkeep");
			break;
		}
		retmat=m.getSubArray(first,last);
		
		return retmat;
	}

	template<class T, int n>
	void WaveDWTnD<T,n>::testinit()
	{
	
	}

	/*****************************************************************
	Skriver ut Matrix3D
	*****************************************************************/
	template<class T, int n>
	void WaveDWTnD<T,n>::printDArray(DArrayN<T,n> arr, QString fname)
	{
		DVector<int> size;
		arr.getSize(size);
		
		QFile outfile( fname );
		if ( outfile.open( IO_WriteOnly ) ) 
		{
//			int newline = 0;
			QTextStream out( &outfile );

			for(int i=0; i<size.dim(); i++)
			{
				out<<size[i]<<" ";
			}
			out<<"\n\n";

			out<<arr;
			
			outfile.close();
		}//end if
	}

	template<class T, int n>
	void WaveDWTnD<T,n>::printDArray(DArrayN<Point3D<T>,n> arr, QString fname)
	{
		DVector<int> size;
		arr.getSize(size);
		
		QFile outfile( fname );
		if ( outfile.open( IO_WriteOnly ) ) 
		{
//			int newline = 0;
			QTextStream out( &outfile );

			for(int i=0; i<size.dim(); i++)
			{
				out<<size[i]<<" ";
			}
			out<<"\n\n";

			out<<arr;
			
			outfile.close();
		}//end if
	}

	template<class T, int n>
	void WaveDWTnD<T,n>::printArrayDArray(Array<DArrayN<T,n>* > arrarr, QString fname)
	{
		QFile outfile( fname );
		if ( outfile.open( IO_WriteOnly ) ) 
		{
			QTextStream out( &outfile );
		
			for(int i=0; i<arrarr.size(); i++)
			{
				out<<"coef"<<i<<"\n\n";
				
				DArrayN<T,n> tt;
				tt = *arrarr[i];
				DVector<int> msize;
				tt.getSize(msize);

				for(int t=0; t<n; t++)	out<<msize[t]<<" ";
				out<<"\n\n";

				out<<tt;
				out<<"\n\n";
			}//i
		
		outfile.close();
		}
	}

	template<class T, int n>
	void WaveDWTnD<T,n>::printArrVector(Array<DVector<int>*> arrvec, QString fname)
	{
		QFile outfile( fname );
		if ( outfile.open( IO_WriteOnly ) ) 
		{
			QTextStream out( &outfile );
		
			for(int i=0; i<arrvec.size(); i++)
			{
				out<<"Longs"<<i<<" : ";
				
				DVector<int> tt;
				tt = *arrvec[i];
				int dim = tt.dim();
				for(int t=0; t<dim; t++)	out<<tt[t]<<" ";
				out<<"\n\n";
			}
		outfile.close();
		}
		
	}

	/******************************************************************************
	********************* VERKTY FUNKSJONER **************************************
	******************************************************************************/

	/*****************************************************************
	********** CASCADE( int coefsnr = 0, int iter = 1) ***************
	** Return the refined phi if coefsnr=0 or psi else. The scaling **
	** /wavelet function are refined iteratively "iter"-times       **  
	*****************************************************************/
	template<class T, int n>
	DArrayN<T,2> WaveDWTnD<T,n>::cascade2d(int type, int iter)
	{
		DArrayN<T,n> ret;

		DArrayN<T,1> scaling;
		DArrayN<T,1> wavelet;

		scaling = cascade1d(0,iter);
		wavelet = cascade1d(1,iter);

		DVector<int> scalsize;
		scaling.getSize(scalsize);
		DVector<int> wavesize;
		wavelet.getSize(wavesize);

		DVector<int> retsize(2);

		switch(type)
		{
		case 0://Scaling
			{
				retsize[0] = scalsize[0];	retsize[1] = scalsize[0];
				ret.setSize(retsize);

				for(int y=0; y<retsize[0]; y++) {
					for(int x=0; x<retsize[1]; x++) {
						ret[y][x] = scaling[y]*scaling[x];
					}
				}
				break;
			}

		case 1://Wavelet1
			{
				retsize[0] = scalsize[0];	retsize[1] = wavesize[0];
				ret.setSize(retsize);

				for(int y=0; y<retsize[0]; y++) {
					for(int x=0; x<retsize[1]; x++) {
						ret[y][x] = wavelet[y]*scaling[x];
					}
				}
				break;
			}
		case 2://Wavelet2
			{
				retsize[0] = wavesize[0];	retsize[1] = scalsize[0];
				ret.setSize(retsize);

				for(int y=0; y<retsize[0]; y++) {
					for(int x=0; x<retsize[1]; x++) {
						ret[y][x] = scaling[y]*wavelet[x];
					}
				}
				break;
			}
			case 3://Wavelet3
			{
				retsize[0] = wavesize[0];	retsize[1] = wavesize[0];
				ret.setSize(retsize);

				for(int y=0; y<retsize[0]; y++) {
					for(int x=0; x<retsize[1]; x++) {
						ret[y][x] = wavelet[y]*wavelet[x];
					}
				}
				break;
			}
		default:
			break;

		}//end switch
		
		
		return ret;
		
	}

	/*****************************************************************
	********** CASCADE( int coefsnr = 0, int iter = 1) ***************
	** Return the refined phi if coefsnr=0 or psi else. The scaling **
	** /wavelet function are refined iteratively "iter"-times       **  
	*****************************************************************/
	template<class T, int n>
	DArrayN<T,1> WaveDWTnD<T,n>::cascade1d(int type, int iter)
	{
		DArrayN<T,1> ret;

		/***************************************************
		Generating the nessecary combination of the filters
		***************************************************/
		Array<DVector<T>*> tmpF;
		tmpF.setSize(1,NULL);
		DVector<T> tmpvec;

		if(type == 1)	tmpvec = hi_r;
		else			tmpvec = lo_r;

		if(tmpF[0]) delete tmpF[0];
		tmpF[0] = new DVector<T> (tmpvec);
		
		/***************************************************
		Generating the starting signal
		***************************************************/
		DVector<int> arrsize(1,1);
		DArrayN<T,1> arr(arrsize,1);

		/***************************************************
		Iterating the phi or psi
		***************************************************/
		DArrayN<T,1> tmpret;
		tmpret = upCoef(&arr,tmpF,iter);

		/***************************************************
		Deciding what part of the signal we want to keep
		***************************************************/
		int _long = lo_r.dim();
		//double pas = pow(2.0,(double)(-iter));
		//int nbpts = (_long-1)/pas+1;
		int nb = 1;
		int lplus = _long-2;
		for(int k=0; k<iter; k++)
		{
			nb = nb*2+lplus;
		}

		double scale = pow(2.0,(double)iter/2.0);
		tmpret = tmpret*scale;

		tmpret = wkeep1d(tmpret,nb);
		DVector<int> retsize(1);
		retsize[0] = nb+2;
		ret.setSize(retsize,T(0));

		DVector<int> start(1);
		start[0] = 1;

		ret.insertSubArray(tmpret,start);
	
		return ret;
	}


	/*****************************************************************
	***** UPCOEF(DArrayN<T,n>* arr, Array<DVector<T>*>F, int iter) ***
	** Return the refined phi if coefsnr=0 or psi else. The scaling **
	** /wavelet function are refined iteratively "iter"-times       **  
	*****************************************************************/
	template<class T, int n>
	DArrayN<T,1> WaveDWTnD<T,n>::upCoef(DArrayN<T,1>* arr, Array<DVector<T>*>F, int iter)
	{
		DArrayN<T,1> ret;
		ret = *arr;

		DVector<int> retsize;
		ret.getSize(retsize);

		DVector<int> sizeKEEP(1);
		sizeKEEP[0] = 2*retsize[0]+F[0]->dim()-2;

		ret = upsaconv1d(&ret,F,sizeKEEP); //1. iteration

		Array<DVector<T>*> tmpF2;
		tmpF2.setSize(1,NULL);
		DVector<T> tmpvec2;

		tmpvec2 = lo_r;

		if(tmpF2[0]) delete tmpF2[0];
		tmpF2[0] = new DVector<T> (tmpvec2);

		for(int k=1; k<iter; k++)
		{
			DVector<int> tmpsize;
			ret.getSize(tmpsize);
			sizeKEEP[0] = 2*tmpsize[0]+tmpF2[0]->dim()-2;
			
			ret = upsaconv1d(&ret,tmpF2,sizeKEEP);
		}

		
		return ret;
	}

}//end namespace Wave_lib