#ifndef _WAVEFLWT_H_
#define _WAVEFLWT_H_

#include "GM_DVector.h"
#include <GM_DArrayN.h>
#include <wave_lib/Definitions.h>
#include <wave_lib/Filter.h>

#include "Lifting.h"
#include "Packets.h"

using namespace GM_lib;
using namespace Wave_lib;

double fdegree;
DVector<double> values;
Filters<double> *_filter;
Array<DArrayN<double,2> > lifting;

template<class T>
void getFilterCoefs(double fd)
{
	fdegree = fd;
	values = _filter->getFilterValues(fdegree);
}

template<class T>
void GetDualWavelet (DVector<int> size, int N, int nTilde, int levels, Filters<T> *filt, double fdegree )
{
	int noGammas, n;
	T maxN;

	// Calculate Filter Coefficients 
	_filter = filt;
	getFilterCoefs<T>(fdegree);

	for(int i=0;i<size.dim();i++)
	{
		// Allocate lifting coefficient matrices 
		maxN = (T)((N > nTilde)? (N):(nTilde)) - (T)1;   // max vanishing moments 
		noGammas = size[i] >> 1;    // number of Gammas in each direction 
		if (noGammas > 0) 
		{
			n = (int)((T)log((T)(size[i]-1)/maxN)/(T)log((T)2) );   // iterations in X 
			n = (n < 0) ? 0 : ((levels < n) ? (levels):(n));   // find lowest level 
		
			DVector<int> lsize(2);
			lsize[0] = n;	lsize[1] = noGammas*nTilde;
			lifting[i].setSize(lsize);

			for ( int t=0 ; t<n ; t++ )
			for ( int j=0 ; j<noGammas*nTilde ; j++ )
				lifting[i][t][j] = NULL;
		}
	}
}


template<class T>
void GetReal (DVector<int> size, int N, int nTilde, int levels )
{
	int n, max_step;
	T  maxN;

	//Calculate number of iterations n. 
	maxN = (T)((N > nTilde)? (N):(nTilde)) - (T)1;   // max vanishing moments 

	DVector<int> nItr(size.dim());
	for(int i=0;i<size.dim();i++)
	{
		nItr[i] = (size[i] == 1) ? 0 : (int)((T)log((T)(size[i]-1)/maxN)/(T)log((T)2) );
		nItr[i] = (nItr[i] < 0) ? 0 : ((levels < nItr[i]) ? (levels):(nItr[i]));   // find lowest level 
	}

	// Minimum number of iterations 
	nItr.getMax(n);   
	max_step = (n==0) ? 0 : 1<<n;  

	// Find lifting coefficients for all levels 
	int level = 0;
	for (int step = 1 ; step < max_step ; step <<= 1 )
	{
		for(int i=0;i<size.dim();i++)
			if (nItr[i]-- > 0) //Get lifting coefficients
				GetLifting <T>(lifting[i][nItr[i]], values, step, N, nTilde, size[i], false);
	}
		level++;   // go to next level
}


/******************************************************************
/*******************  NTERPOLATION CASE, N EVEN *******************
*******************************************************************/
template<class T>
DArrayN<T,3> FLWT3D (DArrayN<T,3> Data, int N, int nTilde, int levels, bool inverse )
{	    
	int max_step, n;  	  
	T maxN;          
 
	DVector<int> size;
	Data.getSize(size);

	// Max number of vanishing moments
	maxN = (T)((N > nTilde)? (N):(nTilde)) - (T)1;   

	// Number of iterations in each direction 
	DVector<int> nItr(size.dim());
	for(int i=0;i<size.dim();i++)
	{
		nItr[i] = (size[i] == 1) ? 0 : (int)((T)log((T)(size[i]-1)/maxN)/(T)log((T)2) );
		nItr[i] = (nItr[i] < 0) ? 0 : ((levels < nItr[i]) ? (levels):(nItr[i]));   // find lowest level 
	}

	// Total number of iterations 
	nItr.getMax(n); 

	DArrayN<T,3> ret;
	ret.setSize(size);
	
	// Forward transform
	if ( !inverse) 
	{ 
		// Calculate wavelet coefficients 
		max_step = (n==0) ? 0 : 1<<n;    // max step
		for (int step=1 ; step<max_step ; step<<=1 )
			for(int t=0; t<size.dim();t++)
			{
				if (nItr[t]-- >0) 
				{
				  for (int i=0 ; i<size[t] ; i++ )
					for (int j=0 ; j<size[t] ; j+=step ) 
					{
						FLWT1D_Predict<T>(Data[i][j], 1, step, N);
						FLWT1D_Update<T> (Data[i][j], 1, step, nTilde, lifting[t][nItr[t]] );
					}
				
					for (int i=0;i<size[t];i++)
						for (int j=0;j<size[t];j++)
							for (int k=0;k<size[t];k++)
								ret[i][j][k] =Data[j][i][k];
					Data=ret;

				  for (int i=0 ; i<size[t] ; i++ ) 
					for (int j=0 ; j<size[t] ; j+=step ) 
					{
						FLWT1D_Predict<T>(Data[i][j], 1, step, N);
						FLWT1D_Update<T> (Data[i][j], 1, step, nTilde, lifting[t][nItr[t]] );
					}

					for (int i=0;i<size[t];i++)
						for (int j=0;j<size[t];j++)
							for (int k=0;k<size[t];k++)
								ret[i][j][k] =Data[i][k][j];
					Data=ret;
				}
			}
	}	
	// Inverse transform 
	else 
	{ 
		// Switch sign of filter & lifting coefficients 
		for ( int i=0 ; i<N*(1+(N>>1)) ; i++ )
			values[i] = -values[i];   // negate filter 

		for(int t=0; t<size.dim();t++)
			for (int y=0 ; y<nItr[t] ; y++ )
				for (int k=0 ; k<(size[t]>>1)*nTilde ; k++ )
					lifting[t][y][k] = -lifting[t][y][k];   // negate lifting 

		// Find original dataset 
		max_step = (n==0) ? 0 : 1<<(n-1);   // initial step 
		for (int step = max_step ; step >= 1 ; step >>= 1 ) 
		{
			for(int t=0; t<size.dim();t++)
				if (n<= nItr[t]) 
				{
				  for (int i=0 ; i<size[t] ; i++ )
					for (int j=0 ; j<size[t] ; j+=step ) 
					{
						FLWT1D_Update<T> (Data[i][j], 1, step, nTilde, lifting[t][nItr[t]-n] );
						FLWT1D_Predict<T>(Data[i][j], 1, step, N);
					}
			
					for (int i=0;i<size[t];i++)
						for (int j=0;j<size[t];j++)
							for (int k=0;k<size[t];k++)
								ret[i][j][k] =Data[j][i][k];
					Data=ret;

				  for (int i=0 ; i<size[t] ; i++ ) 
					for (int j=0 ; j<size[t] ; j+=step ) 
					{
						FLWT1D_Update<T> (Data[i][j], 1, step, nTilde, lifting[t][nItr[t]-n] );
						FLWT1D_Predict<T>(Data[i][j], 1, step, N);
					}

					for (int i=0;i<size[t];i++)
						for (int j=0;j<size[t];j++)
							for (int k=0;k<size[t];k++)
								ret[i][j][k] =Data[i][k][j];
					Data=ret;
				}
			n--;
		} 
	}
	return Data;
}

template<class T>
DArrayN<T,2> FLWT2D (DArrayN<T,2> Data, int N, int nTilde, int levels, bool inverse )
{	    
	int max_step, n;  	  
	T maxN;          
 
	DVector<int> size;
	Data.getSize(size);

	// Max number of vanishing moments
	maxN = (T)((N > nTilde)? (N):(nTilde)) - (T)1;   

	// Number of iterations in each direction 
	DVector<int> nItr(size.dim());
	for(int i=0;i<size.dim();i++)
	{
		nItr[i] = (size[i] == 1) ? 0 : (int)((T)log((T)(size[i]-1)/maxN)/(T)log((T)2) );
		nItr[i] = (nItr[i] < 0) ? 0 : ((levels < nItr[i]) ? (levels):(nItr[i]));   // find lowest level 
	}

	// Total number of iterations 
	nItr.getMax(n);  

	DArrayN<T,2> ret;
	ret.setSize(size);

	// Forward transform 
	if ( !inverse) 
	{ 
		// Calculate wavelet coefficients 
		max_step = (n==0) ? 0 : 1<<n;    // max step
		for (int step=1 ; step<max_step ; step<<=1 )
		{
			for(int t=0; t<size.dim();t++)
			{
				if (nItr[t]-- >0) 
					for (int j=0 ; j<size[t] ; j+=step ) 
					{
						FLWT1D_Predict<T>(Data[j], 1, step, N);
						FLWT1D_Update<T> (Data[j], 1, step, nTilde, lifting[t][nItr[t]] );
					}
					
				for (int i=0;i<size[0];i++)
					for (int j=0;j<size[1];j++)
						ret[i][j] =Data[j][i];
				Data=ret;
			}	
		}
	}
	// Inverse transform 
	else 
	{ 
		// Switch sign of filter & lifting coefficients 
		for ( int i=0 ; i<N*(1+(N>>1)) ; i++ )
			values[i] = -values[i];   // negate filter 

		for(int t=0; t<size.dim();t++)
			for (int y=0 ; y<nItr[t] ; y++ )
				for (int k=0 ; k<(size[t]>>1)*nTilde ; k++ )
					lifting[t][y][k] = -lifting[t][y][k];   // negate lifting 

		// Find original dataset 
		max_step = (n==0) ? 0 : 1<<(n-1);   // initial step 
		for (int step = max_step ; step >= 1 ; step >>= 1 ) 
		{
			for(int t=0; t<size.dim();t++)
			{
				if (n<= nItr[t]) 
					for (int j=0 ; j<size[t] ; j+=step ) 
					{
						FLWT1D_Update<T> (Data[j], 1, step, nTilde, lifting[t][nItr[t]-n]);
						FLWT1D_Predict<T>(Data[j], 1, step, N);
					}
					
				for (int i=0;i<size[0];i++)
					for (int j=0;j<size[1];j++)
						ret[i][j] =Data[j][i];
				Data=ret;	
			}
			n--;
		} 
	}
	return Data;
}


template<class T>
DArrayN<T,1> FLWT1D (DArrayN<T,1> Data, int N,  int nTilde, int levels, bool inverse)
{ 
	int max_step, n; // Number of iterations	               
	T maxN;          // Maximum number of vanishing moments 

	DArrayN<T,1> ret;
	DVector<int> size;
	Data.getSize(size);

	maxN = (T)((N > nTilde)? (N):(nTilde)) - (T)1;   
	n = (size[0] == 1) ? 0 : (int)((T)log((T)(size[0]-1)/maxN)/(T)log((T)2) );
	n = (n < 0) ? 0 : ((levels < n) ? (levels):(n));   // Lowest level 

	if ( !inverse) {   // Forward transform 
		// Calculate wavelet coefficients 
		max_step = (n==0) ? 0 : 1<<n;   // maximum step 
		for (int step=1, s=n-1 ; step<max_step ; step<<=1, s-- ) {
			// Apply forward transform to the vector 
			FLWT1D_Predict<T> ( Data, 1, step, N );
			FLWT1D_Update<T> ( Data,  1, step, nTilde, lifting[0][s]);
		}
	} else 
	{   // Inverse transform 
		// Switch sign of filter & lifting coefficients 
		for (int i=0 ; i<N*(1+(N>>1)) ; i++ )
			values[i] = -values[i];                 // negate filter 
		for (int x=0 ; x<n ; x++ )
			for (int k=0 ; k<(size[0]>>1)*nTilde ; k++ )
				lifting[0][x][k] = -lifting[0][x][k];   // negate lifting  

		// Find original dataset 
		max_step = (n==0) ? 0 : 1<<(n-1);   // initial step 
		for (int step=max_step, s=0 ; step>=1 ; step>>=1, s++ ) 
		{
			// Apply inverse transform to the vector 
			FLWT1D_Update<T> ( Data,  1, step, nTilde, lifting[0][s] );
			FLWT1D_Predict<T> ( Data, 1, step, N );
		}

		// Switch sign of filter & lifting coefficients again 
		for (int i=0 ; i<N*(1+(N>>1)) ; i++ )
			values[i] = -values[i];                 // negate filter 
		for (int x=0 ; x<n ; x++ )
			for (int k=0 ; k<(size[0]>>1)*nTilde ; k++ )
				lifting[0][x][k] = -lifting[0][x][k];   // negate lifting 
	}

	return Data;
}

template<class T>
void FLWT1D_Predict (DArrayN<T,1> &vec, int incr, int step, int N )
{
	int len, j ,     // number of coefficients at current level 
		stepIncr;    // step size between coefficients of the same type 

	long stop1,			// number of cases when L < R 
		stop2,          // number of cases when L = R 
		stop3,soi;      // number of cases when L > R 

	int  i, t, k, fp; 

	DVector<int> size;
	vec.getSize(size);

	len       = (size[0]+ step-1)/step;   // number of coefficients at current level 
	stepIncr  = (step*incr) << 1;   // step size betweeen coefficients 

	j     =  ODD(len);
	stop1 = N >> 1;
	stop3 = stop1 - j;                // L > R 
	stop2 = (len >> 1) - N + 1 + j;   // L = R 
	stop1--;                          // L < R 

	// Cases where nbr. left Lambdas < nbr. right Lambdas 
	k=N; i=(stepIncr >> 1);
	while(stop1--) {
		t=0;
		j = N;
		do {   // Gamma update (Gamma - predicted value) 
			vec[i] -= vec[t]*values[k++];
			t += stepIncr;   // Go to next Lambda coefficient 
		} while(--j);		 // Use all N filter coefficients 
		i += stepIncr;		 // Go to next Gamma coefficient 
	}

	// Cases where nbr. left Lambdas = nbr. right Lambdas 
	soi=0;
	while(stop2--) {
		t=soi;
		j = N;
		fp=k;
		do {   // Gamma update (Gamma - predicted value) 
			vec[i] -= vec[t]*values[fp++];
			t  += stepIncr;   // Go to next Lambda coefficient 
		} while(--j);		  // Use all N filter coefficients 
		soi += stepIncr;	  // Move start point for the Lambdas 
		i += stepIncr;		  // Go to next Gamma coefficient 
	}

	// Cases where nbr. left Lambdas > nbr. right Lambdas
	k=fp;
	while (stop3--) {
		t= soi - stepIncr;
		j = N;
		do {   // Gamma update (Gamma - predicted value)		
			vec[i] -= vec[t]*values[--k];
			t   += stepIncr;   // Go to next Lambda coefficient 
		} while(--j);		   // use all N filter coefficients 
		i += stepIncr;		   // Go to next Gamma coefficient 
	}   
}


template<class T>
void FLWT1D_Update (DArrayN<T,1> &vec, int incr, int step, int nTilde, DArrayN<T,1> lc )
{
	int j, i, t, k=0;        

	int len,               // Number of coefficietns at current level 
		stop1,             // Number of cases when L < R 
		stop2,             // Number of cases when L = R 
		stop3,             // Number of cases when L > R 
		noGammas,          // Number of Gamma coefficients at this level 
		stepIncr;          // Ntep size between coefficients of the same type 
	long	soi;           // Increment for the middle cases 
   
	DVector<int> size;
	vec.getSize(size);

    len      = (size[0]+step-1)/step;   
	stepIncr = (step*incr) << 1;   
	noGammas = len >> 1 ;          

	j	  =  ODD(len) ;
	stop1 = nTilde >> 1;
	stop3 = stop1 - j;                   
	stop2 = noGammas - nTilde + 1 + j;   //L = R 
	stop1--;                           

	// Cases where nbr. left Lambdas < nbr. right Lambdas 
	t =  (stepIncr >> 1);   // Second coefficient is Gamma 
	while(stop1--) {
		i=0;			    // Lambda starts always in first coefficient 
		j = nTilde;
		do {
			vec[i] += vec[t]*lc[k++];   // Lift Lambda (Lambda + lifting value) 
			i    += stepIncr;           // Go to next Lambda coefficient 
		} while(--j);				    // Use all nTilde lifting coefficients 
		t += stepIncr;					// Go to next Gamma coefficient 
		
	}

	// Cases where nbr. left Lambdas = nbr. right Lambdas 
	soi = 0;
	while(stop2--) {
		i = soi;   // First Lambda to be read 
		j = nTilde;
		do {
			vec[i] += vec[t]*lc[k++];   // Lift Lambda (Lambda + lifting value) 
			i    += stepIncr;           // Go to next Lambda coefficient 
		} while(--j);					// Use all nTilde lifting coefficients 
		t  += stepIncr;					// Go to next Gamma coefficient 
		soi += stepIncr;				// Move start point for the Lambdas 
	}
	
	// Cases where nbr. left Lambdas > nbr. right Lambdas 
	int ii= soi - stepIncr;
	while(stop3--) {
		i=ii;
		j = nTilde;
		do {
			vec[i] += vec[t]*lc[k++];   // Lift Lambda (Lambda + lifting value) 
			i += stepIncr;              // Go to next Lambda coefficient 
		} while(--j);					// Use all nTilde lifting coefficients 
		t += stepIncr;					// Go to next Gamma coefficient 
	}
}


/*********************************************************************
******************* AVERAGE INTERPOLATION CASE, N ODD ****************
*********************************************************************/
template<class T>
DArrayN<T,3> FLWT3D (DArrayN<T,3> Data, int levels, bool inverse )
{	    
	int  n,         // Maximum number of levels 
		step,       // Step size of current level 
		max_step;   // Maximum and initial step for the direct and inverse transforms             

	DVector<int> size;
	Data.getSize(size);

	// Number of iterations in each direction 
	DVector<int> nItr(size.dim());
	for(int i=0;i<size.dim();i++)
	{
		nItr[i] = (size[i]==1) ? 0 :(int)ceil(T((T)log((T)size[i])/(T)log((T)2.0))) ;
		nItr[i] = (nItr[i]<0) ? 0 : ((levels < nItr[i]) ? (levels):(nItr[i]));
	}

	nItr.getMax(n);  

	DArrayN<T,3> ret;
	ret.setSize(size);
	
	// Forward transform
	if ( !inverse) 
	{ 
		// Calculate wavelet coefficients 
		max_step = (n==0) ? 0 : 1<<n;   // maximum step 
		for ( step = 1 ; step < max_step ; step <<= 1 ) 
		{
			for(int t=0; t<size.dim();t++)
				if (nItr[t]-- >0) 
				{
				  for (int i=0 ; i<size[t] ; i++ )
					for (int j=0 ; j<size[t] ; j+=step ) 
						FLWT<T>(Data[i][j], 1, step, false);
				
					for (int i=0;i<size[t];i++)
						for (int j=0;j<size[t];j++)
							for (int k=0;k<size[t];k++)
								ret[i][j][k] =Data[j][i][k];
					Data=ret;

				  for (int i=0 ; i<size[t] ; i++ ) 
					for (int j=0 ; j<size[t] ; j+=step ) 
						FLWT<T> (Data[i][j], 1, step, false);

					for (int i=0;i<size[t];i++)
						for (int j=0;j<size[t];j++)
							for (int k=0;k<size[t];k++)
								ret[i][j][k] =Data[i][k][j];
					Data=ret;
				}
			}
	}	
	// Inverse transform 
	else 
	{ 
		// Find original dataset 
		max_step = (n==0) ? 0 : 1<<(n-1);   // initial step 
		for ( step = max_step ; step >= 1 ; step >>= 1 )
		{	
			for(int t=0; t<size.dim();t++)
				if (n<= nItr[t]) 
				{
				  for (int i=0 ; i<size[t] ; i++ )
					for (int j=0 ; j<size[t] ; j+=step ) 
						FLWT<T> (Data[i][j], 1, step, true);
			
					for (int i=0;i<size[t];i++)
						for (int j=0;j<size[t];j++)
							for (int k=0;k<size[t];k++)
								ret[i][j][k] =Data[j][i][k];
					Data=ret;

				  for (int i=0 ; i<size[t] ; i++ ) 
					for (int j=0 ; j<size[t] ; j+=step ) 
						FLWT<T> (Data[i][j], 1, step, true);

					for (int i=0;i<size[t];i++)
						for (int j=0;j<size[t];j++)
							for (int k=0;k<size[t];k++)
								ret[i][j][k] =Data[i][k][j];
					Data=ret;
				}
			n--;
		} 
	}
	return Data;
}

template<class T>
DArrayN<T,2> FLWT2D(DArrayN<T,2> Data, int levels, bool inverse )
{
    int  n,         // Maximum number of levels 
		step,       // Step size of current level 
		max_step;   // Maximum and initial step for the direct and inverse transforms             

	DVector<int> size;
	Data.getSize(size);

	// Number of iterations in each direction 
	DVector<int> nItr(size.dim());
	for(int i=0;i<size.dim();i++)
	{
		nItr[i] = (size[i]==1) ? 0 :(int)ceil(T((T)log((T)size[i])/(T)log((T)2.0))) ;
		nItr[i] = (nItr[i]<0) ? 0 : ((levels < nItr[i]) ? (levels):(nItr[i]));
	}

	nItr.getMax(n);  

	DArrayN<T,2> ret;
	ret.setSize(size);

	// Forward transform 
	if ( !inverse ) 
	{   
		// Calculate wavelet coefficients 
		max_step = (n==0) ? 0 : 1<<n;   // maximum step 
		for ( step = 1 ; step < max_step ; step <<= 1 ) 
		{
			for(int t=0 ; t<size.dim() ; t++)
			{
				if (nItr[t]-- > 0)
					for (int j=0 ; j<size[t] ; j+=step ) 
						FLWT<T> (Data[j], 1, step, false);

				for (int i=0;i<size[0];i++)
						for (int j=0;j<size[1];j++)
							ret[i][j] =Data[j][i];
				Data=ret;	
			}
		}
	} 
	// Inverse transform 
	else 
		{  
		// Find original dataset 
		max_step = (n==0) ? 0 : 1<<(n-1);   // initial step 
		for ( step = max_step ; step >= 1 ; step >>= 1 )
		{	
			for(int t=0 ; t<size.dim() ; t++)
			{
				if (n<= nItr[t])
					for (int j=0 ; j<size[t] ; j+=step ) 
						FLWT<T> (Data[j], 1, step, true);

				for (int i=0;i<size[0];i++)
						for (int j=0;j<size[1];j++)
							ret[i][j] =Data[j][i];
				Data=ret;
			}
			n--;
		}
	}
	return Data;
}

template<class T>
DArrayN<T,1> FLWT1D(DArrayN<T,1> Data, int levels, bool inverse )
{
	int n;  // Number of iterations 

	DVector<int> size;
	Data.getSize(size);

	// Number of iterations 
	n = (size[0]==1) ? 0 :(int)ceil(T((T)log((T)size[0])/(T)log((T)2.0)));
	n = (n<0) ? 0 :((levels < n) ? (levels):(n));

	// Forward transform 
	if (!inverse) 
	{    
		// Calculate wavelet coefficients 
		for (int s = 0 ; s < n ; s++ ) 
			FLWT<T> (Data, 1,1<<s ,false); // Apply forward transform to the vector 
	}
	// Inverse transform
	else {    
		// Find original dataset 
		for (int  s = n-1 ; s >=0 ; s-- ) 
			FLWT<T> ( Data, 1, 1<<s, true ); // Apply inverse transform to the vector 
	}
	return Data;
}

template<class T>
void FLWT(DArrayN<T,1> &vect, int incr, int step, bool inverse )
{
	int nL,			   // Number of Lambda coefficients 
		nG,            // Number of Gamma coefficients 
		stepIncr,      // Step size between coefficients of the same type 
		len;           // Number of coefficients at current level 
	T  lAve,           // Average of Lambdas at current level 
	   lSum = 0;       // Cumulative of Lambdas 

	DVector<int> size;
	vect.getSize(size);

	len      = (size[0] + step -1)/step;   
	stepIncr = (step*incr) << 1; 

	nG = (len >> 1);
	nL = len - nG;

	if (ODD(len)) 	
	{
		int n,s;
		if(!inverse){
			n = len;
			s = stepIncr >> 1;
		}

		T  t = 0;
		int l = 0;
		while (n--){
			t += vect[l];
			l+=s;
		}	
		if(!inverse) 
			lAve = t/(T)len;    
	}

	int j=0;
    while(nG--)
	{
		if(!inverse){
			vect[j+ (stepIncr >> 1)] -= vect[j];        // New Gamma (Haar) coefficient 
			vect[j] += vect[j+ (stepIncr >> 1)]/(T)2;	// New Lambda coefficient 
			lSum += vect[j];
		}
		else{
			vect[j] -= vect[j+ (stepIncr >> 1)]/(T)2;
			lSum += vect[j];
			vect[j+ (stepIncr >> 1)] += vect[j];		//New Gamma coefficient 
			lSum += vect[j+ (stepIncr >> 1)];
		}
		j += stepIncr;
} 

	if (ODD(len)) 
	{
		for(int i=0; i<size[0]; i++){
			vect[i] = lAve*(T)len - lSum;
			lSum += vect[i];
		}
	}
}

/****************************************************************************
 ****************   FLWT with use of filter operators  **********************
 LINEAR: All wavelet coefficients are multiplied by the same factor
         (beta)^j, where j is the current level.
 CIRCULAR: Beta factors depend on the current position of the coefficient
           in a sphere. That is, beta is a function of x^2 + y^2+ z^2 = r^2
 RECTANGULAR: Beta factors depend on the position of the coefficientin a cube
 ****************************************************************************/

template<class T> DArrayN<T,2>
FLWTFilter2D (DArrayN<T,2> Data, int N, int nTilde, T beta, int levels, int _mode )
{
    int step, max_step,  // Step size and max/init step 
		n;				 // Number of iterations 
	T maxN;              // Maximum number of vanishing moments 
 
	DVector<int> size;
	Data.getSize(size);

	// Calculate number of iterations n  
	maxN = (T)((N > nTilde)? (N):(nTilde)) - (T)1;   

	// Number of iterations in each direction 
	DVector<int> nItr(size.dim());
	for(int i=0;i<size.dim();i++)
	{
		nItr[i] = (size[i] == 1) ? 0 : (int)((T)log((T)(size[i]-1)/maxN)/(T)log((T)2) );
		nItr[i] = (nItr[i] < 0) ? 0 : ((levels < nItr[i]) ? (levels):(nItr[i]));   // Find lowest level 
	}

	// Total number of iterations 
	nItr.getMax(n); 

	DArrayN<T,2> ret;
	ret.setSize(size);

	// Calculate the maximum beta factor for each direction 
	DVector<T> nBeta(size.dim());
	nBeta.setElements(1);

	for(int i=0;i<size.dim();i++)
		for (int j=0 ; j<nItr[i] ; j++ )
			nBeta[i] *= beta;
  
    // Apply beta operator 
    max_step = (n==0) ? 0 : 1<<n;   
    for ( step=1 ; step<max_step ; step<<=1 ) 
	{
		for(int t=0; t<size.dim();t++)
			{
				if (nItr[t]-- >0) 
				{	for (int j=0 ; j<size[t] ; j+=step ) 
						if (_mode == CIRCULAR) FLWTFilter1DC(Data[j], 1, step, beta, size.dim()*beta, j, nItr[t]);
						else if (_mode == LINEAR) FLWTFilter1DL(Data[j],1, step, nBeta[t] );

					nBeta[t] /= beta;
				}
					
				for (int i=0;i<size[0];i++)
					for (int j=0;j<size[1];j++)
						ret[i][j] =Data[j][i];
				Data=ret;
			}	
	}

	return Data;
}

template<class T> DArrayN<T,1>
FLWTFilter1D (DArrayN<T,1> Data, int N, int nTilde, T beta, int levels, int _mode )
{
    int  n,        // Number of iterations 
        max_step;  // Maximum step for coefficients in last level 
    T maxN,        // Maximum number of vanishing moments     
	  nBeta;       // Beta factor for the direction 

	DVector<int> size;
	Data.getSize(size);
  
	maxN = (T)((N > nTilde)? (N):(nTilde)) - (T)1;   
	n = (size[0] == 1) ? 0 : (int)((T)log((T)(size[0]-1)/maxN)/(T)log((T)2) );
	n = (n < 0) ? 0 : ((levels < n) ? (levels):(n));   // Find lowest level 

	nBeta = (T)1;
    for (int i=0 ; i<n ; i++)
        nBeta *= beta;

   max_step = (n==0) ? 0 : 1<<n;  
   for (int step=1, s=n-1 ; step<max_step ; step<<=1, s-- )
	{
            for (int j=0 ; j<size[0] ; j+=step ) 
				if (_mode == CIRCULAR) FLWTFilter1DC ( Data, 1, step, beta, size.dim()*beta, j, n);    
				else if (_mode == LINEAR) FLWTFilter1DL ( Data, 1, step, nBeta );
    }
	return Data;
}

template<class T> void
FLWTFilter1DL ( DArrayN<T,1> &Data, int incr, int step, T beta )
{
    int stepIncr, gammaIdx;

	DVector<int> size;
	Data.getSize(size);

    stepIncr = (step*incr) << 1;
    gammaIdx = stepIncr >> 1;

    // Step through Gammas and apply Beta factor 
    while (gammaIdx < size[0]*incr)
	{
        Data[gammaIdx] *= beta;
        gammaIdx += stepIncr;
    }
}

template<class T> void
FLWTFilter1DC (DArrayN<T,1> &Data, int incr, int step, T betaStart, T betaStop, int scale, int levels )
{
	int stepIncr, step2;
    T i, j, s, beta, interval = betaStop - betaStart;

	DVector<int> size;
	Data.getSize(size);

    step2 = step << 1;
    stepIncr = step2*incr;

    //Step through Gammas and apply Beta factor 
    for (int k=0,t=0  ; k<size[0] ; k+=step2 )
	{
        T dist;
		i = k/size[0] - 0.5;
		j = scale/size[0] - 0.5;
			dist = j*j + i*i;
		s = dist*size[0]/size[0];  
		if ( betaStart <= betaStop )
			s = s*interval + betaStart; 
		else 
		{
			s = (T)1 - s;
			s = betaStop - s*interval;
		}
		beta = s;
		for (int k=0; k<levels ; k++ )
			beta *= s;
		Data [t+ (stepIncr >> 1)]*= beta;
		t += stepIncr;
	}
}

/*********************************************************************
******************* CODE NEEDED FOR THE FLWT PACKETS ****************
*********************************************************************/
template<class T> Array<DArrayN<T,2> >
DSFLWT2D ( DArrayN<T,2> &in, int N, int nTilde )
{
	DVector<int> insize;
	in.getSize(insize);

     int x,y,
		 x_2 = ( insize[0] == 1 ) ? 0 : insize[0]>>1,
         y_2 = ( insize[1] == 1 ) ? 0 : insize[1]>>1;
 
	// Ouput regions (4, X/2, Y/2) 
	Array<DArrayN<T,2> > out;
	out.setSize(4);

	DVector<int> size(2);
	size[0]=x_2;
	size[1]=y_2;
	
	for(int i=0; i<out.size(); i++)
		out[i].setSize(size);

  // Apply forward transform 
	DArrayN<T,2> ret;
	ret.setSize(insize);

     if (ODD(N))  in = FLWT2D<T>( in, 1, false );
    else  in = FLWT2D<T>( in, N, nTilde, 1, false );

	// Reorganize coefficients in Mallat format 
	for (int j=0 ; j<insize[0] ; j++ ) 
		in[j] = InplaceChange1D<T>( in[j], 1, false );
					
	for (int i=0;i<insize[0];i++)
		for (int j=0;j<insize[1];j++)
			ret[i][j] =in[j][i];

	for (int j=0 ; j<insize[1] ; j++ ) 
		ret[j] = InplaceChange1D<T>( ret[j], 1, false );
					
	for (int i=0;i<insize[0];i++)
		for (int j=0;j<insize[1];j++)
			in[i][j] =ret[j][i];
	
	// Separate regions 
	for ( y=0 ; y<y_2 ; y++ ) 
		for ( x=0 ; x<x_2 ; x++ ) 
		{
			out[0][y][x] = in[y][x];			// fill region 0 
		    out[1][y][x] = in[y][x+x_2];		// fill region 1 
			out[2][y][x] = in[y+y_2][x];		// fill region 2 
		    out[3][y][x] = in[y+y_2][x+x_2];    // fill region 3 
		}

    return out;
}

template<class T> 
Array<DArrayN<T,1> > DSFLWT1D (DArrayN<T,1> &in, int N, int nTilde )
{
	DVector<int> insize;
	in.getSize(insize);

    int x_2 = ( insize[0] == 1 ) ? 0 : insize[0]>>1;

    // Apply forward transform
	if (ODD(N)) in=FLWT1D<T>( in, 1, false );
    else in = FLWT1D<T>( in, N, nTilde, 1, false);

    // Reorganize coefficients in Mallat format 
    in = InplaceChange1D<T>(in, 1,false);
	
	// Output regions (2, X/2) 
	Array<DArrayN<T,1> > out;
	out.setSize(2);

	DVector<int> size(1);
	size[0]=x_2;
	
	for(int i=0; i<out.size(); i++)
		out[i].setSize(size);

    // Separate regions 
    for (int x=0 ; x<x_2 ; x++ )
	{
        out[0][x] = in[x];		 // fill region 0 
        out[1][x] = in[x+x_2];   // fill region 1 
	}
    return out;
}


template<class T> DArrayN<T,1>
ISFLWT2D (Array<DArrayN<T,2> > in, int width, int height, int N, int nTilde )
{
    int x, y,
		x_2 = width>>1,
        y_2 = height>>1;

    // Output coefficients 
	DArrayN<T,2> out;
	DVector<int>size(2);
	size[0]=width; size[1]=height;
    out.setSize(size);

    for ( y=0 ; y<y_2 ; y++ ) 
		for ( x=0 ; x<x_2 ; x++ ) 
		{
			out[y][x] = in[0][y][x];			// fill region 0 
		    out[y][x+x_2] = in[1][y][x];		// fill region 1 
			out[y+y_2][x] = in[2][y][x];		// fill region 2 
		    out[y+y_2][x+x_2] = in[3][y][x];    // fill region 3 
		}

	DArrayN<T,2> ret;
	ret.setSize(size);

    // Reorganize coefficients in Lifting format 
	for (int j=0 ; j<size[0] ; j++ ) 
		out[j] = InplaceChange1D<T>( out[j], 1, true);
					
	for (int i=0;i<size[0];i++)
		for (int j=0;j<size[1];j++)
			ret[i][j] =out[j][i];

	for (int j=0 ; j<size[1] ; j++ ) 
		ret[j] = InplaceChange1D<T>( ret[j], 1, true );
					
	for (int i=0;i<size[0];i++)
		for (int j=0;j<size[1];j++)
			out[i][j] =ret[j][i];

    // Apply in-place inverse transform 
    if (ODD(N))  out=FLWT2D<T>( out, 1, true );
    else out=FLWT2D<T>( out, N, nTilde, 1, true );

	DArrayN<T,1> tmp;
	DVector<int> tmpsize(1);
	tmpsize[0]=width*height;
    tmp.setSize(tmpsize);

	for (int i=0, t=0;i<width;i++)
		for (int j=0;j<height;j++, t++)
			tmp[t] =out[i][j];

    return tmp;
}

template<class T>
DArrayN<T,1> ISFLWT1D (Array<DArrayN<T,1> > in, int width, int N, int nTilde )
{
    int x_2 = width>>1;
    
	DArrayN<T,1> out;
	DVector<int>size(1);
	size[0]=width;
    out.setSize(size);
   
    // Adjoint regions 
    for (int x=0 ; x<x_2 ; x++ ) 
	{
        out[x] = in[0][x];		 // fill region 0 
        out[x+x_2] = in[1][x];   // fill region 1 
    }

    // Reorganize coefficients in Lifting format 
    out = InplaceChange1D<T>( out, 1, true );

    // Apply in-place inverse transform 
   if (ODD(N)) out=FLWT1D<T>( out, 1, true );
   else out = FLWT1D<T>( out, N, nTilde, 1, true);

    return out;
}

template<class T> DArrayN<T,2>
IFLWTP2D (DArrayN<T,2> in, int N, int nTilde )
{
	DVector<int> insize;
	in.getSize(insize);

	DArrayN<T,2> ret;
	ret.setSize(insize);

    // Reorganize coefficients in Lifting format 
	for (int j=0 ; j<insize[0] ; j++ ) 
		in[j] = InplaceChange1D<T>( in[j], 1, true);
					
	for (int i=0;i<insize[0];i++)
		for (int j=0;j<insize[1];j++)
			ret[i][j] =in[j][i];

	for (int j=0 ; j<insize[1] ; j++ ) 
		ret[j] = InplaceChange1D<T>( ret[j], 1, true );
					
	for (int i=0;i<insize[0];i++)
		for (int j=0;j<insize[1];j++)
			in[i][j] =ret[j][i];

    // Apply in-place inverse transform 
    if (ODD(N))  in = FLWT2D<T>( in, 1, true );
    else  in = FLWT2D<T>( in, N, nTilde, 1, true );

	return in;
}


template<class T> DArrayN<T,1>
IFLWTP1D (DArrayN<T,1> in, int N, int nTilde )
{
	// Reorganize coefficients in Lifting format 
	in =InplaceChange1D<T>( in, 1, true );

	// Apply in-place inverse transform 
	if( ODD(N)) in=FLWT1D<T>(in, 1, true );
	else in =FLWT1D<T>( in, N, nTilde, 1, true );

	return in;
}

template<class T>
DArrayN<T,1> InplaceChange1D (DArrayN<T,1> Data, int offset, bool inverse )
{
	DVector<int> size;
	Data.getSize(size);
	
	DArrayN<T,1> buffer;
	buffer.setSize(size);

    // Number of Gamma coefficients
	int len =size[0];
	int noLambdas, noGammas;
	
    noGammas  = len>>1;
    noLambdas = len - noGammas;
    
    // Check whether to do Lifting format or Mallat format 
    if ( !inverse )
	{
        // Move all lambdas to the left and all gammas to the right 
        for (int i=0,j=0 ; i<len-2 ; i+=2)
		{
			buffer[j] = Data[i*offset];
            buffer[j+(noLambdas)] = Data[(i+1)*offset];
			j++;
        }
        if (ODD(len)) 
            buffer[j] = Data[(len-1)*offset];
        else 
		{
			buffer[j] = Data[(len-2)*offset];
			buffer[j+(noLambdas)] = Data[(len-1)*offset];
        }
    } else 
	{
        // Alternate lambdas and gammas 
        for (int i=0, j=0; i<len-2 ; i+=2 )
		{
			buffer[i]   = Data[j];
			buffer[i+1] = Data[j+(noLambdas)*offset];
			j += offset;
        }

         if (ODD(len))  
			buffer[len-1] = Data[j];
        else 
		{
			buffer[len-2] = Data[j];
            buffer[len-1] = Data[j+(noLambdas)*offset];
        }
    }
    
    // Write results into the original vector 
    for (int i=0,j=0 ; i<len*offset ; i+=offset,j++ ) 
		Data[i] = buffer[j];

	return Data;
}

#endif
