#ifndef _WAVETOOLS_H_
#define _WAVETOOLS_H_

#include <complex>
#include "CoefPoint.h"
#include <GM_DArrayN.h>
using namespace GM_lib;


template<class T>
int mySgn( T val )
{
    int ret;
    if(val>0) ret = 1;
    else if(val<0) ret = -1;
    else ret = 0;
    return ret;
}


/**************************************************************************
************ hardThreshold( ... ) *****************************************
*** Returning a array of coefs after a hard threshold *********************
**************************************************************************/
template<class T>
Array<DArrayN<T,1>*> hardThreshold( Array<DArrayN<T, 1> *> arrmat, double _scalingcoef, int _levels, double &comp )
{
    Array<DArrayN<T,1>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    DArrayN<T,1> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,1> (tmpmat); //copying the first approx.

    double tau = 1;
	int nrofzeros = 0;
	int totnr = 0;
	
    for(int j=0; j<_levels; j++) {
	tau = _scalingcoef;
	for(int l=0; l<1; l++) {
	    tmpmat = *arrmat[j+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);
	    for(int u=0; u<tmpsize[0]; u++) {
		    if(abs(tmpmat[u])>tau) tmpmat[u] =mySgn<T>(tmpmat[u])*tmpmat[u];
			else {
                tmpmat[u]=0;
				nrofzeros++;
			}
			totnr++;
	    }//end u
	    if(ret[j+1+l]) delete ret[j+1+l];
	    ret[j+1+l] = new DArrayN<T,1> (tmpmat);
	}//end l
    }//end j

	comp = ((double)nrofzeros/(double) totnr)*100;
    return ret;
}

template<class T>
Array<DArrayN<T,2>*> hardThreshold( Array<DArrayN<T, 2> *> arrmat, double _scalingcoef, int _levels, double &comp )
{
      Array<DArrayN<T,2>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    DArrayN<T,2> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,2> (tmpmat); //copying the first approx.
	DVector<int> appsize;
	ret[0]->getSize(appsize);
	int appnr = appsize[0]*appsize[1];

    double tau = 1;
	int nrofzeros = 0;
	int totnr = 0;
	
    for(int j=0; j<_levels; j++) {
	tau = _scalingcoef;
	for(int l=0; l<3; l++) {
	    tmpmat = *arrmat[j*3+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);
	    for(int u=0; u<tmpsize[0]; u++) {
			for( int v=0; v<tmpsize[1]; v++) {
			    if(abs(tmpmat[u][v])>tau) tmpmat[u][v] =mySgn<T>(tmpmat[u][v])* abs(tmpmat[u][v]);
			else {
                tmpmat[u][v]=0;
				nrofzeros++;
			}
			totnr++;
			}//end v
	    }//end u
	    if(ret[j*3+1+l]) delete ret[j*3+1+l];
	    ret[j*3+1+l] = new DArrayN<T,2> (tmpmat);
	}//end l
    }//end j

	comp = (double)(nrofzeros/(double) (totnr+appnr))*100.0;
    return ret;
}

template<class T>
Array<DArrayN<T,3>*> hardThreshold( Array<DArrayN<T, 3> *> arrmat, double _scalingcoef, int _levels, double &comp )
{
    static  Array<DArrayN<T,3>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    int detnr =7;
    
    DArrayN<T,3> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,3> (tmpmat); //copying the first approx.
	DVector<int> appsize;
	ret[0]->getSize(appsize);
	int appnr = appsize[0]*appsize[1]*appsize[2];

	double tau = 1;
	int nrofzeros = 0;
	int totnr = 0;

    for(int j=0; j<_levels; j++) {
	tau = _scalingcoef*sqrt((double)((j+1)*_levels)/(double)pow(2,_levels));
	for(int l=0; l<detnr; l++) {
	    tmpmat = *arrmat[j*detnr+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);

	    for(int k=0; k<tmpsize[0]; k++) {
			for(int u=0; u<tmpsize[1]; u++) {
				for( int v=0; v<tmpsize[2]; v++) {
					if(abs(tmpmat[k][u][v])>tau) tmpmat[k][u][v] = mySgn<T>(tmpmat[k][u][v])*abs(tmpmat[k][u][v]);
					else {
						tmpmat[k][u][v]=0;
						nrofzeros++;
					}
					totnr++;
				}//end v
			}//end u
	    }//end k
	    if(ret[j*detnr+1+l]) delete ret[j*detnr+1+l];
	    ret[j*detnr+1+l] = new DArrayN<T,3> (tmpmat);
	}//end l
    }//end j

	comp = (double)(nrofzeros/(double) (totnr+appnr))*100.0;
    return ret;
}

/**************************************************************************
************ softThreshold( ... ) *****************************************
*** Returning a array of coefs after a soft threshold *********************
**************************************************************************/
template<class T>
Array<DArrayN<T,1>*> softThreshold( Array<DArrayN<T, 1> *> arrmat, double _scalingcoef, int _levels, double &comp )
{
    Array<DArrayN<T,1>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    DArrayN<T,1> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,1> (tmpmat); 

    double tau = 1;
	int nrofzeros = 0;
	int totnr = 0;
	
    for(int j=0; j<_levels; j++) {
	tau = _scalingcoef*sqrt((double)(j*_levels)/(double)pow(2,_levels));
	for(int l=0; l<1; l++) {
	    tmpmat = *arrmat[j+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);
	    for(int u=0; u<tmpsize[0]; u++) {
		    tmpmat[u] = mySgn<T>(tmpmat[u])*max(abs(tmpmat[u])-tau,0.0);
			if(tmpmat[u]==0.0) nrofzeros++;
			totnr++;
	    }//end u
	    if(ret[j+1+l]) delete ret[j+1+l];
	    ret[j+1+l] = new DArrayN<T,1> (tmpmat);
	}//end l
    }//end j

	comp = ((double)nrofzeros/(double) totnr)*100;
    return ret;
}

template<class T>
Array<DArrayN<T,2>*> softThreshold( Array<DArrayN<T, 2> *> arrmat, double _scalingcoef, int _levels, double &comp )
{
      Array<DArrayN<T,2>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    DArrayN<T,2> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,2> (tmpmat); 
	DVector<int> appsize;
	ret[0]->getSize(appsize);
	int appnr = appsize[0]*appsize[1];

    double tau = 1;
	int nrofzeros = 0;
	int totnr = 0;
	
    for(int j=0; j<_levels; j++) {
	tau = _scalingcoef*sqrt((double)((j+1)*_levels)/(double)pow(2,_levels));
	for(int l=0; l<3; l++) {
	    tmpmat = *arrmat[j*3+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);
	    for(int u=0; u<tmpsize[0]; u++) {
			for( int v=0; v<tmpsize[1]; v++) {
			    tmpmat[u][v] = mySgn<T>(tmpmat[u][v])*max(abs(tmpmat[u][v])-tau,0.0);
				if(tmpmat[u][v]==0.0) nrofzeros++;
				totnr++;
			}//end v
	    }//end u
	    if(ret[j*3+1+l]) delete ret[j*3+1+l];
	    ret[j*3+1+l] = new DArrayN<T,2> (tmpmat);
	}//end l
    }//end j

	comp = (double)(nrofzeros/(double) (totnr+appnr))*100.0;
    return ret;
}

template<class T>
Array<DArrayN<T,3>*> softThreshold( Array<DArrayN<T, 3> *> arrmat, double _scalingcoef, int _levels, double &comp )
{
    static  Array<DArrayN<T,3>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    int detnr =7;
    
    DArrayN<T,3> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,3> (tmpmat); 
	DVector<int> appsize;
	ret[0]->getSize(appsize);
	int appnr = appsize[0]*appsize[1]*appsize[2];

	double tau = 1;
	int nrofzeros = 0;
	int totnr = 0;

    for(int j=0; j<_levels; j++) {
	tau = _scalingcoef*sqrt((double)((j+1)*_levels)/(double)pow(2,_levels));
	for(int l=0; l<detnr; l++) {
	    tmpmat = *arrmat[j*detnr+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);

	    for(int k=0; k<tmpsize[0]; k++) {
			for(int u=0; u<tmpsize[1]; u++) {
				for( int v=0; v<tmpsize[2]; v++) {
					tmpmat[k][u][v] = mySgn<T>(tmpmat[k][u][v])*max(abs(tmpmat[k][u][v])-tau,0.0);
					if(tmpmat[k][u][v]==0.0) nrofzeros++;
					totnr++;
				}//end v
			}//end u
	    }//end k
	    if(ret[j*detnr+1+l]) delete ret[j*detnr+1+l];
	    ret[j*detnr+1+l] = new DArrayN<T,3> (tmpmat);
	}//end l
    }//end j

	comp = (double)(nrofzeros/(double) (totnr+appnr))*100.0;
    return ret;
}

/**********************************************************************
******** lorentzThreshold( .... ) *************************************
***** returning an array of coefs after the have been threshold *******
** by a Lorentz type of thresholding. First all the wavelet coef are **
** scaled by a calculated number, then we remove a given % of the    **
** smallest coefs and then rescale the coef and put it back into the **
** returning array. In this way we have controll over the % comp.    **
**********************************************************************/
template<class T>
Array<DArrayN<T,1>*> lorentzThreshold( Array<DArrayN<T, 1> *> arrmat, double _tau, double _remove, int _levels, double &comp  )
{
	Array<DArrayN<T,1>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

	/**********************************************************
	*** Copying the approx coef to the returning array.     ***
	*** Copying also the wavelet coef to get the right size ***
	*** of the returning DArrayN							***
	**********************************************************/
	for(int t=0; t<retsize; t++)
	{
        DArrayN<T,1> tmpmat1;
		tmpmat1 = *arrmat[t];
		if(ret[t]) delete ret[t];
		ret[t] = new DArrayN<T,1> (tmpmat1);
	}
	
	Array<CoefPoint<T> > coefpt;
	coefpt.setMax(100000);

	double scale=1;
	double tmpval = 0;
	DVector<int> tmppos(1);
	int tmpmatnr = 1;
	
    for(int j=0; j<_levels; j++) {
		scale = pow((double)2,(double)(j*(_tau+0.5))); //(_tau + d/2)
	
			tmpmatnr = j+1;//+l;
			DArrayN<T,1> tmpmat;
			tmpmat = *arrmat[tmpmatnr];
			
			DVector<int> tmpsize;
			tmpmat.getSize(tmpsize);
	    
			for(int u=0; u<tmpsize[0]; u++) {
				tmppos[0]=u; //x
				tmpval = scale*abs(tmpmat[u]);
				CoefPoint<T> tmppt(tmpmat[u],tmpval,tmpmatnr,tmppos);
				coefpt+=tmppt;
			}//end u
		//}//end l
    }//end j

	/*************************************************************
	** Sorting the coef by size, the smallest coefs comes first **
	** and the next by increasing size of the coef. Remove (set **
	** to zero) then the given % (_remove) of the smallest		**
	** coefs and put the coefs back in the returning array.		**
	*************************************************************/
	coefpt.sort();
	int removenr = (int) coefpt.size()*_remove;
	for(int i=0; i<removenr; i++) coefpt[i].setVal(0.0);

	for(i=0; i<coefpt.size(); i++)
	{
		ret[coefpt[i].getMatnr()]->setElement(coefpt[i].getVal(), coefpt[i].getPos());
	}

	/*************************************************************
	** Calculate the total compression							**
	*************************************************************/
	DVector<int> appsize;
	ret[0]->getSize(appsize);
	comp = ((double)(removenr)/(double)(coefpt.size()+appsize[0]))*100.0;

    return ret;
}


/*************** 2D *******************/
template<class T>
Array<DArrayN<T,2>*> lorentzThreshold( Array<DArrayN<T, 2> *> arrmat, double _tau, double _remove, int _levels, double &comp  )
{
	Array<DArrayN<T,2>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

	/**********************************************************
	*** Copying the approx coef to the returning array.     ***
	*** Copying also the wavelet coef to get the right size ***
	*** of the returning DArrayN							***
	**********************************************************/
	for(int t=0; t<retsize; t++)
	{
        DArrayN<T,2> tmpmat1;
		tmpmat1 = *arrmat[t];
		if(ret[t]) delete ret[t];
		ret[t] = new DArrayN<T,2> (tmpmat1);
	}
	
	Array<CoefPoint<T> > coefpt;
	coefpt.setMax(100000);

	double scale=1;
	double tmpval = 0;
	DVector<int> tmppos(2);
	int tmpmatnr = 0;
	
    for(int j=0; j<_levels; j++) {
		scale = pow((double)2,(double)(j*(_tau+1))); //(_tau + d/2)
	
		for(int l=0; l<3; l++) {
			tmpmatnr = j*3+1+l;
			DArrayN<T,2> tmpmat;
			tmpmat = *arrmat[tmpmatnr];
			
			DVector<int> tmpsize;
			tmpmat.getSize(tmpsize);
	    
			for(int u=0; u<tmpsize[0]; u++) {
				for( int v=0; v<tmpsize[1]; v++) {
					tmppos[0]=u; //y
					tmppos[1]=v; //x

					tmpval = scale*abs(tmpmat[u][v]);
			
					CoefPoint<T> tmppt(tmpmat[u][v],tmpval,tmpmatnr,tmppos);
					coefpt+=tmppt;
				}//end v
			}//end u
		}//end l
    }//end j

	/*************************************************************
	** Sorting the coef by size, the smallest coefs comes first **
	** and the next by increasing size of the coef. Remove (set **
	** to zero) then the given % (_remove) of the smallest		**
	** coefs and put the coefs back in the returning array.		**
	*************************************************************/
	coefpt.sort();
	int removenr = (int) coefpt.size()*_remove;
	for(int i=0; i<removenr; i++) coefpt[i].setVal(0.0);

	for(i=0; i<coefpt.size(); i++)
	{
		ret[coefpt[i].getMatnr()]->setElement(coefpt[i].getVal(), coefpt[i].getPos());
	}

	/*************************************************************
	** Calculate the total compression							**
	*************************************************************/
	DVector<int> appsize;
	ret[0]->getSize(appsize);
	comp = ((double)(removenr)/(double)(coefpt.size()+(appsize[0]*appsize[1])))*100.0;

    return ret;
}

/*************** 3D *******************/
template<class T>
Array<DArrayN<T,3>*> lorentzThreshold( Array<DArrayN<T, 3> *> arrmat, double _tau, double _remove, int _levels, double &comp  )
{
	Array<DArrayN<T,3>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

	/**********************************************************
	*** Copying the approx coef to the returning array.     ***
	*** Copying also the wavelet coef to get the right size ***
	*** of the returning DArrayN							***
	**********************************************************/
	for(int t=0; t<retsize; t++)
	{
        DArrayN<T,3> tmpmat1;
		tmpmat1 = *arrmat[t];
		if(ret[t]) delete ret[t];
		ret[t] = new DArrayN<T,3> (tmpmat1);
	}
	
	Array<CoefPoint<T> > coefpt;
	coefpt.setMax(200000);

	double scale=1;
	double tmpval = 0;
	DVector<int> tmppos(3);
	int tmpmatnr = 0;
	
    for(int j=0; j<_levels; j++) {
		scale = pow((double)2,(double)(j*(_tau+1.5))); //(_tau + d/2)
	
		for(int l=0; l<7; l++) {
			tmpmatnr = j*7+1+l;
			DArrayN<T,3> tmpmat;
			tmpmat = *arrmat[tmpmatnr];
			
			DVector<int> tmpsize;
			tmpmat.getSize(tmpsize);
	    
			for(int z=0; z<tmpsize[0]; z++) {
				for( int y=0; y<tmpsize[1]; y++) {
					for( int x=0; x<tmpsize[2]; x++) {
						tmppos[0]=z; //z
						tmppos[1]=y; //y
						tmppos[2]=x; //x

						tmpval = scale*abs(tmpmat[z][y][x]);
			
						CoefPoint<T> tmppt(tmpmat[z][y][x],tmpval,tmpmatnr,tmppos);
						coefpt+=tmppt;
					}//end x
				}//end y
			}//end z
		}//end l
    }//end j

	/*************************************************************
	** Sorting the coef by size, the smallest coefs comes first **
	** and the next by increasing size of the coef. Remove (set **
	** to zero) then the given % (_remove) of the smallest		**
	** coefs and put the coefs back in the returning array.		**
	*************************************************************/
	coefpt.sort();
	int removenr = (int) coefpt.size()*_remove;
	for(int i=0; i<removenr; i++) coefpt[i].setVal(0.0);

	for(i=0; i<coefpt.size(); i++)
	{
		ret[coefpt[i].getMatnr()]->setElement(coefpt[i].getVal(), coefpt[i].getPos());
	}

	/*************************************************************
	** Calculate the total compression							**
	*************************************************************/
	DVector<int> appsize;
	ret[0]->getSize(appsize);
	comp = ((double)(removenr)/(double)(coefpt.size()+(appsize[0]*appsize[1]*appsize[2])))*100.0;

    return ret;
}


/**************************************************************************
************  firmThreshold( ... ) *****************************************
*** Returning a array of coefs after a firm threshold *********************
**************************************************************************/
template<class T>
Array<DArrayN<T,1>*> firmThreshold( Array<DArrayN<T, 1> *> arrmat, double _scalingcoef1,double _scalingcoef2, int _levels, double &comp )
{
	//make restriction on scaling coefficients in QT
    Array<DArrayN<T,1>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    DArrayN<T,1> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,1> (tmpmat); 

    double tau1;
	double tau2;
	int nrofzeros = 0;
	int totnr = 0;
	
    for(int j=0; j<_levels; j++) {
	tau1 = _scalingcoef1;
	tau2 = _scalingcoef2;
	for(int l=0; l<1; l++) {
	    tmpmat = *arrmat[j+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);
	    for(int u=0; u<tmpsize[0]; u++) {
			if(abs(tmpmat[u])>tau1 && abs(tmpmat[u])<=tau2)
				tmpmat[u] = mySgn<T>(tmpmat[u])* (tau2*(abs(tmpmat[u])-tau1)/(tau2-tau1));
			else if(abs(tmpmat[u])>tau2) tmpmat[u] = tmpmat[u];
			else if(abs(tmpmat[u])<=tau1)
			{
				tmpmat[u]=0.0;
				nrofzeros++;
			}
			totnr++;
	    }//end u
	    if(ret[j+1+l]) delete ret[j+1+l];
	    ret[j+1+l] = new DArrayN<T,1> (tmpmat);
	}//end l
    }//end j

	comp = ((double)nrofzeros/(double) totnr)*100;
    return ret;
}

template<class T>
Array<DArrayN<T,2>*> firmThreshold( Array<DArrayN<T, 2> *> arrmat, double _scalingcoef1, double _scalingcoef2, int _levels, double &comp )
{
    Array<DArrayN<T,2>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    DArrayN<T,2> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,2> (tmpmat); 
	DVector<int> appsize;
	ret[0]->getSize(appsize);
	int appnr = appsize[0]*appsize[1];

    double tau1;
	double tau2;
	int nrofzeros = 0;
	int totnr = 0;
	
    for(int j=0; j<_levels; j++) {
	tau1 = _scalingcoef1;
	tau2 = _scalingcoef2;
	for(int l=0; l<3; l++) {
	    tmpmat = *arrmat[j*3+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);
	    for(int u=0; u<tmpsize[0]; u++) {
			for( int v=0; v<tmpsize[1]; v++) {
			    if(abs(tmpmat[u][v])>tau1 && abs(tmpmat[u][v])<=tau2)
					tmpmat[u][v] = mySgn<T>(tmpmat[u][v])* (tau2*(abs(tmpmat[u][v])-tau1)/(tau2-tau1));
				else if(abs(tmpmat[u][v])>tau2) tmpmat[u][v] =tmpmat[u][v];
				else if(abs(tmpmat[u][v])<=tau1)
				{
					tmpmat[u][v]=0.0;
					nrofzeros++;
				}
				totnr++;
			}//end v
	    }//end u
	    if(ret[j*3+1+l]) delete ret[j*3+1+l];
	    ret[j*3+1+l] = new DArrayN<T,2> (tmpmat);
	}//end l
    }//end j

	comp = (double)(nrofzeros/(double) (totnr+appnr))*100.0;
    return ret;
}

template<class T>
Array<DArrayN<T,3>*> firmThreshold( Array<DArrayN<T, 3> *> arrmat, double _scalingcoef1, double _scalingcoef2,int _levels, double &comp )
{
    static  Array<DArrayN<T,3>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    int detnr =7;
    DArrayN<T,3> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,3> (tmpmat); 
	DVector<int> appsize;
	ret[0]->getSize(appsize);
	int appnr = appsize[0]*appsize[1]*appsize[2];

	double tau1;
	double tau2;
	int nrofzeros = 0;
	int totnr = 0;

    for(int j=0; j<_levels; j++) {
	tau1 = _scalingcoef1*sqrt((double)((j+1)*_levels)/(double)pow(2,_levels));
	tau2 = _scalingcoef2*sqrt((double)((j+1)*_levels)/(double)pow(2,_levels));
	for(int l=0; l<detnr; l++) {
	    tmpmat = *arrmat[j*detnr+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);

	    for(int k=0; k<tmpsize[0]; k++) {
			for(int u=0; u<tmpsize[1]; u++) {
				for( int v=0; v<tmpsize[2]; v++) {
					if(abs(tmpmat[k][u][v])>tau1 && abs(tmpmat[k][u][v])<=tau2)
					tmpmat[k][u][v] = mySgn<T>(tmpmat[k][u][v])* (tau2*(abs(tmpmat[k][u][v])-tau1)/(tau2-tau1));
					else if(abs(tmpmat[k][u][v])>tau2) tmpmat[k][u][v] = mySgn<T>(tmpmat[k][u][v])*tmpmat[k][u][v];
					else if(abs(tmpmat[k][u][v])<=tau1)
					{
						tmpmat[k][u][v]=0.0;
						nrofzeros++;
					}
					totnr++;
				}//end v
			}//end u
	    }//end k
	    if(ret[j*detnr+1+l]) delete ret[j*detnr+1+l];
	    ret[j*detnr+1+l] = new DArrayN<T,3> (tmpmat);
	}//end l
    }//end j

	comp = (double)(nrofzeros/(double) (totnr+appnr))*100.0;
    return ret;
}

/**************************************************************************
************ garroteThreshold( ... ) *****************************************
*** Returning a array of coefs after a garrote threshold *********************
**************************************************************************/
template<class T>
Array<DArrayN<T,1>*> garroteThreshold( Array<DArrayN<T, 1> *> arrmat, double _scalingcoef, int _levels, double &comp )
{
    Array<DArrayN<T,1>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    DArrayN<T,1> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,1> (tmpmat); //copying the first approx.

    double tau = 1;
	int nrofzeros = 0;
	int totnr = 0;
	
    for(int j=0; j<_levels; j++) {
	tau = _scalingcoef;
	for(int l=0; l<1; l++) {
	    tmpmat = *arrmat[j+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);
	    for(int u=0; u<tmpsize[0]; u++) {
		    if(abs(tmpmat[u])>tau) tmpmat[u] = (tmpmat[u]-(tau*tau)/tmpmat[u]);
			else if(abs(tmpmat[u])<=tau)
			{
				tmpmat[u]=0.0; 
				nrofzeros++;
			}
	        totnr++;
	    }//end u

	    if(ret[j+1+l]) delete ret[j+1+l];
	    ret[j+1+l] = new DArrayN<T,1> (tmpmat);
	}//end l
    }//end j

	comp = ((double)nrofzeros/(double) totnr)*100;
    return ret;
}

template<class T>
Array<DArrayN<T,2>*> garroteThreshold( Array<DArrayN<T, 2> *> arrmat, double _scalingcoef, int _levels, double &comp )
{
      Array<DArrayN<T,2>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    DArrayN<T,2> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,2> (tmpmat); //copying the first approx.
	DVector<int> appsize;
	ret[0]->getSize(appsize);
	int appnr = appsize[0]*appsize[1];

    double tau = 1;
	int nrofzeros = 0;
	int totnr = 0;
	
    for(int j=0; j<_levels; j++) {
	tau = _scalingcoef;
	for(int l=0; l<3; l++) {
	    tmpmat = *arrmat[j*3+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);
	    for(int u=0; u<tmpsize[0]; u++) {
			for( int v=0; v<tmpsize[1]; v++) {
			    if(abs(tmpmat[u][v])>tau) tmpmat[u][v] = (tmpmat[u][v]-pow(tau,2)/tmpmat[u][v]);
				else if(abs(tmpmat[u][v])<=tau) 
				{
					tmpmat[u][v]=0.0; 
					nrofzeros++;
				}
				totnr++;
			}//end v
	    }//end u
	    if(ret[j*3+1+l]) delete ret[j*3+1+l];
	    ret[j*3+1+l] = new DArrayN<T,2> (tmpmat);
	}//end l
    }//end j

	comp = (double)(nrofzeros/(double) (totnr+appnr))*100.0;
    return ret;
}

template<class T>
Array<DArrayN<T,3>*> garroteThreshold( Array<DArrayN<T, 3> *> arrmat, double _scalingcoef, int _levels, double &comp )
{
    static  Array<DArrayN<T,3>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    int detnr =7;
    
    DArrayN<T,3> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,3> (tmpmat); //copying the first approx.
	DVector<int> appsize;
	ret[0]->getSize(appsize);
	int appnr = appsize[0]*appsize[1]*appsize[2];

	double tau = 1;
	int nrofzeros = 0;
	int totnr = 0;

    for(int j=0; j<_levels; j++) {
	tau = _scalingcoef;
	for(int l=0; l<detnr; l++) {
	    tmpmat = *arrmat[j*detnr+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);

	    for(int k=0; k<tmpsize[0]; k++) {
			for(int u=0; u<tmpsize[1]; u++) {
				for( int v=0; v<tmpsize[2]; v++) {
					if(abs(tmpmat[k][u][v])>tau) tmpmat[k][u][v] = mySgn<T>(tmpmat[k][u][v])*
						                                    (tmpmat[k][u][v]-pow(tau,2)/tmpmat[k][u][v]);
					else if(abs(tmpmat[k][u][v])<=tau)
					{
						tmpmat[k][u][v]=0.0;
						nrofzeros++;
					}
					totnr++;
				}//end v
			}//end u
	    }//end k
	    if(ret[j*detnr+1+l]) delete ret[j*detnr+1+l];
	    ret[j*detnr+1+l] = new DArrayN<T,3> (tmpmat);
	}//end l
    }//end j

	comp = (double)(nrofzeros/(double) (totnr+appnr))*100.0;
    return ret;
}

/**************************************************************************
************ besovsThreshold( ... ) *****************************************
*** Returning a array of coefs after a besovs threshold *********************
**************************************************************************/
template<class T>
Array<DArrayN<T,1>*> besovsThreshold( Array<DArrayN<T, 1> *> arrmat, 
									 double s, double s1, double p, double p1, double etha, double vv, double t,
									 int _levels, double &comp)
{
	Array<DArrayN<T,1>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    DArrayN<T,1> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,1> (tmpmat); 

	int nrofzeros = 0;
	int totnr = 0;
	double eps;
	
    for(int j=0; j<_levels; j++) {
        eps = 1;//2*p*s - 2*p1*s1 + p - p1;

		for(int l=0; l<1; l++) {
			tmpmat = *arrmat[j+1+l];
			DVector<int> tmpsize;
			tmpmat.getSize(tmpsize);

			double offset, A,B,Q, a,b,q, root;
			for(int u=0; u<tmpsize[0]; u++) 
			{
				/*if(p=p1 && 1/p<etha)
					root=1/( 1+pow( t*pow((double)2,j*(s-s1)),p/(p-1/etha) ) );
				
				else if((p1-1/etha)/(p-1/etha)==2 && 2*p>=p1)
				{
					a=pow( pow(p1,1/etha)*pow(abs(tmpmat[u]),p1-p),1/(p1-1/etha) );
					b=pow( vv*pow((double)2,j*etha/2)*pow(p,1/etha),1/(p1-1/etha) );
					root=2*a+b+sqrt(pow(2*a+b,2)-4*pow(a,2))/2*a;
				}

				else if((p1-1/etha)/(p-1/etha)==3 && 3*p>=p1)
				{
					a=pow( pow(p1,1/etha)*pow(abs(tmpmat[u]),p1-p),1/(p1-1/etha) );
					b=pow( vv*pow((double)2,j*etha/2)*pow(p,1/etha),1/(p1-1/etha) );
					
					q=2+b/a; p=b/a;
					Q=pow(p/3,(double)3)+pow(q/2,(double)2);
					A=pow(-q/2+sqrt(Q),double(1/3));
					B=pow(-q/2-sqrt(Q),double(1/3));

					root=A+B+1;
				}
				
				else if((p1-1/etha)/(p-1/etha)==4 && 4*p>=p1)
				{
					etha=1;
					p1=4;
					p=1;
					complex<T> pi,ti, rt2,rt3;
					double  f,g,r;
					
					a=pow( pow(p1,1/etha)*pow(abs(tmpmat[u]),p1-p),1/(p1-1/etha) );
					b=pow( vv*pow((double)2,j*etha/2)*pow(p,1/etha),1/(p1-1/etha) );
					
					//x^3 + (f/2)*x^2 + ((f^2 -4*h)/16)*x -g^2/64 = 0
					f = 0; g=-b/a; 
					q=2*pow(f/6,(double)3)-((f/2)*((f*f -4*g)/16))/3-g*g/64;
					p=pow((f/2),(double)2)+((f*f -4*g)/16);
					Q=pow(p/3,(double)3)+pow(q/2,(double)2);
					A=pow(-q/2+sqrt(Q),double(1/3));
					B=pow(-q/2-sqrt(Q),double(1/3));

					rt2=complex<T>(-(A+B)/2,sqrt((double)3)*(A-B)/2);
					rt3=complex<T>(-(A+B)/2,-sqrt((double)3)*(A-B)/2);

					pi=sqrt(rt2); ti=sqrt(rt3); r= -g/(8*real(pi)*real(ti)); 
					root= -real(pi)-real(ti)+r+1;
				}*/

				//else 
				//{  
				a=0; b=1;
					while( b-a > 1E-9)//precision
					{
						root = (a+b)/2;
						offset = pow(p1,1/etha)*pow(1-root,p1-1/etha)*pow(abs(tmpmat[u]),p1-p) -
										vv*pow((double)2,j*eps/2)*pow(p,1/etha)*pow(root,p-1/etha);
						if ( offset > 0) a = root;
						else b = root;
					} 
					root = (a+b)/2;
				//}
				if(root<0 && root>1) root=0.5;

				//          
				tmpmat[u]= root*tmpmat[u];

				if(tmpmat[u]==0.0) nrofzeros++;
				totnr++;
			}//end u
			if(ret[j+1+l]) delete ret[j+1+l];
			ret[j+1+l] = new DArrayN<T,1> (tmpmat);
		}//end l
    }//end j

	comp = ((double)nrofzeros/(double) totnr)*100;
    return ret;
}

template<class T>
Array<DArrayN<T,2>*> besovsThreshold( Array<DArrayN<T, 2> *> arrmat, 
									 double s, double s1, double p, double p1, double etha, double vv,double t, 
									 int _levels, double &comp)
{
    Array<DArrayN<T,2>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    DArrayN<T,2> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,2> (tmpmat); //copying the first approx.
	DVector<int> appsize;
	ret[0]->getSize(appsize);
	int appnr = appsize[0]*appsize[1];
	int totnr = 0;
	int nrofzeros = 0;
	double eps,c;
	
    for(int j=0; j<_levels; j++) {
	//c = sqrt((double)((j+1)*_levels)/(double)pow(2,_levels));
	eps  = 2*p*s - 2*p1*s1 + 2*(p - p1);

	for(int l=0; l<3; l++) {
	    tmpmat = *arrmat[j*3+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);

		double offset, a,b, root;
	    for(int u=0; u<tmpsize[0]; u++) {
			for( int v=0; v<tmpsize[1]; v++)
			{

				/*if(p=p1 && 1/p<etha)
					root=1/( 1+pow( t*pow((double)2,j*(s-s1)),p/(p-1/etha) ) );
				
				else if((p1-1/etha)/(p-1/etha)==2 && 2*p>=p1)
				{
					a=pow( pow(p1,1/etha)*pow(abs(tmpmat[u][v]),p1-p),1/(p1-1/etha) );
					b=pow( vv*pow((double)2,j*etha/2)*pow(p,1/etha),1/(p1-1/etha) );
					root=2*a+b+sqrt(pow(2*a+b,2)-4*pow(a,2))/2*a;
				}
				else
				{*/
					a=0; b=1;
					while( b-a > 1E-6)//precision
					{
						root = (a+b)/2;
						offset = pow(p1,1/etha)*pow(1-root,p1-1/etha)*pow(abs(tmpmat[u][v]),p1-p) -
										vv*pow((double)2,(j*eps/2))*pow(p,1/etha)*pow(root,p-1/etha);
						if ( offset > 0) a = root;
						else b = root;
					}	
					root = (a+b)/2;
				//}
			if(root<0 && root>1) root=0.5;

				//            
				tmpmat[u][v]*= root;
				if(tmpmat[u][v]==0.0) nrofzeros++;
				totnr++;
			}//end v
	    }//end u
	    if(ret[j*3+1+l]) delete ret[j*3+1+l];
	    ret[j*3+1+l] = new DArrayN<T,2> (tmpmat);
	}//end l
    }//end j

	comp = (double)(nrofzeros/(double) (totnr+appnr))*100.0;
    return ret;
}

template<class T>
Array<DArrayN<T,3>*> besovsThreshold( Array<DArrayN<T, 3> *> arrmat, 
									  double s, double s1, double p, double p1, double etha, double vv, 
									 int _levels, double &comp)
{
    static  Array<DArrayN<T,3>*> ret;
    int retsize = arrmat.size();
    ret.setSize(retsize,NULL);

    int detnr =7;
	double eps,c;
    
    DArrayN<T,3> tmpmat;
    tmpmat = *arrmat[0];
    if(ret[0]) delete ret[0];
    ret[0] = new DArrayN<T,3> (tmpmat); //copying the first approx.
	DVector<int> appsize;
	ret[0]->getSize(appsize);
	int appnr = appsize[0]*appsize[1]*appsize[2];
	int totnr = 0;
	int nrofzeros = 0;

    for(int j=0; j<_levels; j++) {
	c = sqrt((double)((j+1)*_levels)/(double)pow(2,_levels));
	eps = 2*p*s - 2*p1*s1 + 3*(p - p1);

	for(int l=0; l<detnr; l++) {
	    tmpmat = *arrmat[j*detnr+1+l];
	    DVector<int> tmpsize;
	    tmpmat.getSize(tmpsize);

		double offset, a,b, root;
	    for(int k=0; k<tmpsize[0]; k++) {
			for(int u=0; u<tmpsize[1]; u++) {
				for( int v=0; v<tmpsize[2]; v++)
				{
					a=0; b=1;
					while( b-a > 1E-6)//precision
					{
						root = (a+b)/2;
						offset = pow(p1,1/etha)*pow(1-root,p1-1/etha)*pow(abs(tmpmat[k][u][v]),p1-p) -
					                 vv*pow((double)2,j*eps/2)*pow(p,1/etha)*pow(root,p-1/etha);
						if ( offset > 0) a = root;
						else b = root;
					}	
					root = (a+b)/2;

					//            
					tmpmat[k][u][v]*= root;
					if(tmpmat[k][u][v]==0.0) nrofzeros++;
                    totnr++;
				}//end v
			}//end u
	    }//end k
	    if(ret[j*detnr+1+l]) delete ret[j*detnr+1+l];
	    ret[j*detnr+1+l] = new DArrayN<T,3> (tmpmat);
	}//end l
    }//end j

    comp = (double)(nrofzeros/(double) (totnr+appnr))*100.0;
    return ret;
}

#endif//_WAVETOOLS_H_