#include <math.h>
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <time.h>

#define  CHECKSUCC(X)  if(!(X)){return false;};
#define  DELTEXTURES(n,t)  if(glIsTexture((*t))){glDeleteTextures(n,t);};

#include "gpuDWT.h"

//Decomposition Filters  
const float decomp_filter[18]={
	0.026749,-0.016864,-0.078223,0.266864,0.602949,0.266864,-0.078223,-0.016864,0.026749 //DF
		,0,-0.045636,-0.028772,0.29564,0.55754,0.29564,-0.028772,-0.045636,0 //RF
};


// Reconstruction filter
const float recon_filter[18]={
	0,-0.045636,-0.028772,0.29564,0.55754,0.29564,-0.028772,-0.045636,0 //RF
	,0.026749,-0.016864,-0.078223,0.266864,0.602949,0.266864,-0.078223,-0.016864,0.026749 //DF
};


/* Global cgcontext  */
CGcontext   cgContext;


/**********************************************************************/
/*******     Cg program string					   			 **********/
/**********************************************************************/


/*************  Simple Pass Through  **************************/

char fillsource[]="	fragout_float main(vf30 IN,"
"             uniform samplerRECT texture) \n"
"{"
"  fragout_float OUT;\n"

"  OUT.col =f4texRECT(texture, IN.TEX0.xy); \n"
"  OUT.col.a=1.0; \n"
"  return OUT;	\n"
"} \n";


/********************** dwt horizontal decomposition ************************/
char dwtsource[]="	fragout_float main(vf30 IN,"
"             uniform samplerRECT dwttexture,	"
"			 uniform samplerRECT filtertexture, "
"			 uniform samplerRECT lookuptexture, "
"			 uniform float		 level)			\n"
"{ \n"

"	float level_center=level+0.5;	\n"

"	float3 lookup=f3texRECT(lookuptexture,float2(IN.TEX0.x+4.0, level_center)); \n"
"	float2 base=float2(lookup.z,IN.TEX0.y);	 \n" 
	
"	float2 neighbour=base; \n"

"	float3 weightedsum=float3(0.0,0.0,0.0); \n"

"	/*		offset for selecting between lowpass or high pass filters  */ \n"
"	float offset=lookup.y*9.0; \n"	

"	/*		fetching the neightbours, do boundary extension, then do filtering  */ \n"
"	for (int i=0; i<9; i++)	\n"
"	{"
"		neighbour.x=f3texRECT( lookuptexture,float2(base.x+i,level_center)).x;	\n"
"		weightedsum+=f3texRECT(filtertexture,float2(i+offset+0.5,0.5)).x*f3texRECT(dwttexture,neighbour); \n"
"	}	\n"


"	fragout_float OUT; \n"

"	OUT.col = float4(weightedsum, 1.0); \n"
  
"	return OUT; \n"
"}" ;


/********************** dwt vertical decomposition ************************/
char dwtcolsource[]=" fragout_float main(vf30 IN,"
"             uniform samplerRECT dwttexture,"
"			 uniform samplerRECT filtertexture,"
"			 uniform samplerRECT lookuptexture,"
"			 uniform float		 level) \n"
"{"
"	float  level_center=level+0.5; \n"
"	float3 lookup=f3texRECT(lookuptexture,float2(IN.TEX0.y+4.0, level_center)); \n"
"	float2 base=float2(IN.TEX0.x,lookup.z); \n"

"	float2 neighbour=base; \n"

"	float3 weightedsum=float3(0.0,0.0,0.0); \n"

"	/*		offset for selecting between lowpass or highpass filters  */ \n"
"	float offset=lookup.y*9.0;"

"	/*		fetching the neightbours, do boundary extension, then do filtering  */ \n"
"	for (int i=0; i<9; i++)	\n"
"	{"
"		neighbour.y=f3texRECT(lookuptexture,float2(base.y+i,level_center)).x;"
"		weightedsum+=f3texRECT(filtertexture,float2(i+offset+0.5,0.5)).x*f3texRECT(dwttexture,neighbour); \n"
"	}"


"	fragout_float OUT; \n"

"	OUT.col = float4(weightedsum, 1.0); \n"
  
"	return OUT; \n"
"} ";



/********************** idwt horizontal reconstruction ************************/
char invdwtsource[]=" fragout_float main(vf30 IN,"
"             uniform samplerRECT dwttexture,"
"			 uniform samplerRECT filtertexture,"
"			 uniform samplerRECT lookuptexture[3],"
"			 uniform float		 level) \n"
"{"
"	float3 weightedsum = float3(0,0,0);  \n"
"	float  level_center=level+0.5; \n"
"   float4  lookup[3];	\n"

"	/*		fetch all the indices for addressing filtering neighbours  */ \n"
"	lookup[0] =	f4texRECT(lookuptexture[0],float2(IN.TEX0.x,level_center)); \n"
"	lookup[1] =	f4texRECT(lookuptexture[1],float2(IN.TEX0.x,level_center)); \n"
"	lookup[2] =	f4texRECT(lookuptexture[2],float2(IN.TEX0.x,level_center)); \n"


"	float2 st=float2(0.0,IN.TEX0.y);		\n"

"	/*	for selecting proper filter kernel  */ \n"
"	float2 filter_st=float2(0.0+lookup[2].y*9.0+0.5,0.5); \n"


"	/*	do the filtering  */ \n"
"	st.x=lookup[0].r; \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.x=lookup[0].g; \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.x=lookup[0].b;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.x=lookup[0].a;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.x=lookup[1].r;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.x=lookup[1].g;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.x=lookup[1].b;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.x=lookup[1].a;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.x=lookup[2].r;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"



	"fragout_float OUT;	\n"
	"OUT.col = float4(weightedsum, 1.0);	\n"

	"return OUT;	\n"

"}";

/********************** idwt vertical reconstruction ************************/
char invdwtcolsource[]=" fragout_float main(vf30 IN,"
"             uniform samplerRECT dwttexture,"
"			 uniform samplerRECT filtertexture,"
"			 uniform samplerRECT lookuptexture[3],"
"			 uniform float		texwidth,"
"			 uniform float		texheight,"
"			 uniform float		 level) \n"
"{"

"	float3 weightedsum = float3(0,0,0);  \n"
"	float  level_center=level+0.5; \n"
"   float4  lookup[3];	\n"

"	/*		fetch all the indices for addressing filtering neighbours  */ \n"
"	lookup[0] =	f4texRECT(lookuptexture[0],float2(texwidth-IN.TEX0.y, texheight-level_center)); \n"
"	lookup[1] =	f4texRECT(lookuptexture[1],float2(texwidth-IN.TEX0.y, texheight-level_center)); \n"
"	lookup[2] =	f4texRECT(lookuptexture[2],float2(texwidth-IN.TEX0.y, texheight-level_center)); \n"



"	float2 st=float2(IN.TEX0.x,0.0);		\n"

"	/*	for selecting proper filter kernel  */ \n"
"	float2 filter_st=float2(0.0+lookup[2].y*9.0+0.5,0.5); \n"



"	/*	do the filtering  */ \n"
"	st.y=lookup[0].r;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.y=lookup[0].g;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.y=lookup[0].b;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.y=lookup[0].a;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.y=lookup[1].r;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.y=lookup[1].g;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.y=lookup[1].b;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.y=lookup[1].a;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"
"	filter_st.x+=1.0;	\n"

"	st.y=lookup[2].r;  \n"
"	weightedsum+=f3texRECT(filtertexture,filter_st).x*f3texRECT(dwttexture,st);		\n"



	"fragout_float OUT;	\n"
	"OUT.col = float4(weightedsum, 1.0);	\n"

	"return OUT;	\n"


"}";

// Boundary extension function
int gpuDWT::ext(int index, int datalen, extmode mode)
{
	int id=index;

	int eo=datalen%2;

	//symmetric padding
	switch(mode) {
	case symper:
		
		if(id<0)
			id=-id;

		if(id>=datalen)
			id=datalen-1-(id-datalen+1);
	
		return id;

		break;


	//periodic padding
	
	case per:

		id=(id+datalen+eo)%(datalen+eo);
		//id=(id+datalen)%(datalen);
		
		if(eo==0)
			return id;
	
		else
		{
			if(id!=datalen)
				return id;
			else
				return datalen-1;
		}
		
		
		break;

	default:
		id=(id+datalen)%datalen;
		return id;
		
		break;


	}
}

int gpuDWT::calLevels(int startind, int endind)
{
	int level,lstart,lend;
	
	lstart=startind;
	lend=endind;

	level=1;

	while(lend-lstart>1)
	{
		lstart=ceil(lstart/2.0f);
		lend=ceil(lend/2.0f);
		level++;
	}

	return level;

}

// Calculate the downsampling length and sampling offset for lowpassed part
void gpuDWT::calLength(int startind, int endind, int level, int * llength, int * loffset)
{
	
	int hstart,lstart,hend,lend,i;
	
	lstart=startind;
	lend=endind;
	hstart=startind;
	hend=endind;



	for(i=0; i<level;i++)
	{
		hstart=floor(lstart/2.0f);
		hend=floor(lend/2.0f);
		lstart=ceil(lstart/2.0f);
		lend=ceil(lend/2.0f);
	}



	// low-passed signal length
	*llength=lend-lstart;
	

	// controls the downsampling, i.e, the even/oddness of the filtered samples to be taken for lowpassed-part
	if(hstart<lstart)	
	{
		*loffset=1;
	}
	else
	{
		*loffset=0;
	}
}


bool gpuDWT::createIDATexture(extmode mode,float ** itex1, float ** itex2, float ** itex3, int width, int height, int * texwidth, int * texheight, int startx, int starty, int endx,int endy)
{
	if(width<1 || height<1 )
		return false;

	int i,j,k;

	
	int L=width>height?width:height;


	int xlevels=calLevels(startx,endx)+1; 
	int ylevels=calLevels(starty,endy)+1;

	//maximum number of decomposition levels for this image
	int maxlevels=xlevels>ylevels?xlevels:ylevels;

	*texwidth=L+1;
	*texheight=maxlevels;

	
	// 3 textures holding 9 neighbour indices for indirect addressing
	float (* tex1)[4]=new float[maxlevels*(L+1)][4];
	float (* tex2)[4]=new float[maxlevels*(L+1)][4];
	float (* tex3)[3]=new float[maxlevels*(L+1)][3];


	//create the indirect addressing texture

	for (i=0; i<xlevels; i++)
	{
		int levellength, halflength, loffset;
		
		calLength(startx,endx,i,&levellength,&loffset);
		calLength(startx,endx,i+1,&halflength,&loffset);


		
							
		for (j=0; j<levellength; j++)
		{
			int eo=j%2;
			float neighbours[9];

			// put filter selector to tex3
			if(loffset==1)
			{
				tex3[i*(L+1)+j][1]=1-eo;
			}
			else
			{
				tex3[i*(L+1)+j][1]=eo;
			}
			
			tex3[i*(L+1)+j][2]=0;



			// calculate 9 neighbours
			for(k=-4; k<=4; k+=2)
			{
				if(loffset==1)
				{
					neighbours[k+4]=0.5+halflength*(1.0-eo)+(ext(j+k,levellength,mode)-1.0*eo)/2.0;
				}
				else
				{
					neighbours[k+4]=0.5+halflength*eo+(ext(j+k,levellength,mode)-1.0*eo)/2.0;
				}
				
			}

			eo=1-eo;

			for(k=-3; k<=3; k+=2)
			{
				if(loffset==1)
				{
					neighbours[k+4]=0.5+halflength*(1.0-eo)+(ext(j+k,levellength,mode)-1.0*eo)/2.0;
				}
				else
				{
					neighbours[k+4]=0.5+halflength*eo+(ext(j+k,levellength,mode)-1.0*eo)/2.0;
				}
				

			}

			//put neighbours to 3 textures
			for(k=0; k<4; k++)
				tex1[i*(L+1)+j][k]=neighbours[k];

			for(k=0; k<4; k++)
				tex2[i*(L+1)+j][k]=neighbours[k+4];

			tex3[i*(L+1)+j][0]=neighbours[8];

		}

	}


	for (i=0; i<ylevels; i++)
	{
		int levellength, halflength, loffset;
		
		calLength(starty,endy,i,&levellength,&loffset);
		calLength(starty,endy,i+1,&halflength,&loffset);


							
		for (j=0; j<levellength; j++)
		{
			int eo=j%2;
			float neighbours[9];

			// put filter selector to tex3
			if(loffset==1)
			{
				tex3[(maxlevels-i-1)*(L+1)+L-j][1]=1-eo;
			}
			else
			{
				tex3[(maxlevels-i-1)*(L+1)+L-j][1]=eo;
			}
			tex3[(maxlevels-i-1)*(L+1)+L-j][2]=0;



			// calculate 9 neighbours

			/*  these 5 neighbour share same parity (even/oddness) */
			for(k=-4; k<=4; k+=2)
			{
				if(loffset==1)
				{
					neighbours[k+4]=0.5+halflength*(1.0-eo)+(ext(j+k,levellength,mode)-1.0*eo)/2.0;
				}
				else
				{
					neighbours[k+4]=0.5+halflength*eo+(ext(j+k,levellength,mode)-1.0*eo)/2.0;
				}
				
			}

			eo=1-eo;

			/*  these 4 neighbour share same parity (even/oddness) */
			for(k=-3; k<=3; k+=2)
			{
				if(loffset==1)
				{
					neighbours[k+4]=0.5+halflength*(1.0-eo)+(ext(j+k,levellength,mode)-1.0*eo)/2.0;
				}
				else
				{
					neighbours[k+4]=0.5+halflength*eo+(ext(j+k,levellength,mode)-1.0*eo)/2.0;
				}
				
			}

			//put neighbours to 3 textures
			for(k=0; k<4; k++)
				tex1[(maxlevels-i-1)*(L+1)+L-j][k]=neighbours[k];

			for(k=0; k<4; k++)
				tex2[(maxlevels-i-1)*(L+1)+L-j][k]=neighbours[k+4];

			tex3[(maxlevels-i-1)*(L+1)+L-j][0]=neighbours[8];

		}

	}

	*itex1=(float *)tex1;
	*itex2=(float *)tex2;
	*itex3=(float *)tex3;


return true;
}


gpuDWT::gpuDWT(Filters<double> *filter,double fdegree)
{
	this->_filter = filter;
	getFilterCoefs(fdegree);

	m_imageWidth=0;
	m_imageHeight=0;
	m_iReadBuffer=0;


	m_rt[0]=NULL;
	m_rt[1]=NULL;

	//	decomp_filter[i]= lo_d[i];
}


void gpuDWT::getFilterCoefs(double fd)
{
	this->fdegree = fd;	
}

gpuDWT::~gpuDWT()
{
	finalcleanup();
}

// Cg Error Callback
void cgErrorCallback()
{
  CGerror lastError = cgGetError();
  
  if(lastError)
  {
    exit(0);
  }
} 


bool gpuDWT::createLookupTex(extmode mode, int startx, int starty, int endx, int endy)
{
	//uninitialized imagedimension
	if(m_imageWidth<=0 || m_imageHeight<=0)
		return false;

	
	
	int maxrowlevels=calLevels(startx,endx)+3;
	int maxcollevels=calLevels(starty,endy)+3;
	
	float (* rowlookup)[3];
	float (* collookup)[3];


	// Designed for filter kernel with maximum length 9, so adding 4 at each side for boundary extension, total 8 more entries 
	rowlookup=new float[(m_imageWidth+8)*maxrowlevels][3];
	collookup=new float[(m_imageHeight+8)*maxcollevels][3];



	//wrapping
	/**********************************************************************************/
	/*    -4 -3 -2 -1 0 1 2.........m_imagewidth-2, m_imagewidth-1, m_imagewidth-2....*/
	
	/*********************************************************************************/

	int i;

	//fill up the  lookup table for horizontal decomposition
	for (i=0; i<(m_imageWidth+8)*maxrowlevels; i++)
	{
	
		//determine current row--current level
		int level=i/(m_imageWidth+8);

		int pos=i%(m_imageWidth+8);



		int fullrow,halfrow,loffset;

		calLength(startx,endx,level,&fullrow,&loffset);
		calLength(startx,endx,level+1,&halfrow,&loffset);

		
		
		//The 3 components: boundary extension / High|low / Base Pos
		
		//boundary extension
		rowlookup[i][0]=pos-4;


		switch(mode)	
		{
			case symper:
			if(rowlookup[i][0]<0)
				rowlookup[i][0]=-rowlookup[i][0];

			if(rowlookup[i][0]>fullrow-1)
				rowlookup[i][0]=fullrow-1-(rowlookup[i][0]-fullrow+1);
			break;


			case per:
				rowlookup[i][0]=fmod((rowlookup[i][0]+fullrow),fullrow);
			break;
		
		}

		if(rowlookup[i][0]>=halfrow)
		{
			rowlookup[i][1]=1;	//for Highpass
		}
		else
		{
			rowlookup[i][1]=0;	//for Lowpass
		}


		//Base Position
		// loffset determine even or odd samples to choose for dowsampling

		if(loffset==1)
		{
			rowlookup[i][2]=(rowlookup[i][0]-halfrow*rowlookup[i][1])*2.0+0.5+1.0-rowlookup[i][1];
		}
		else
		{
			rowlookup[i][2]=(rowlookup[i][0]-halfrow*rowlookup[i][1])*2.0+0.5+rowlookup[i][1];
		}
		
		
		rowlookup[i][0]+=0.5;

	}

	//fill up the lookup table for vertical decomposition
	for (i=0; i<(m_imageHeight+8)*maxcollevels; i++)
	{
	
		//determine current row--current level
		int level=i/(m_imageHeight+8);
		
		int pos=i%(m_imageHeight+8);



		int fullcol,halfcol,loffset;
		
		calLength(starty,endy,level,&fullcol,&loffset);
		calLength(starty,endy,level+1,&halfcol,&loffset);

		
		
		//The 3 component:  boundary extension/ High|low / Base Pos
		
		//boundary extension
		collookup[i][0]=pos-4;


		switch(mode)
		{
			case symper:
			if(collookup[i][0]<0)
				collookup[i][0]=-collookup[i][0];

			if(collookup[i][0]>fullcol-1)
				collookup[i][0]=fullcol-1-(collookup[i][0]-fullcol+1);
			break;


			case per:
				collookup[i][0]=fmod((collookup[i][0]+fullcol),fullcol);
			break;
		
		}
		

		if(collookup[i][0]>=halfcol)
		{
			collookup[i][1]=1;	//for Highpass
		}
		else
		{
			collookup[i][1]=0;	//for Lowpass
		}


		//Base Position
		// loffset determine even or odd samples to choose for dowsampling
		
		if(loffset==1)
		{
			collookup[i][2]=(collookup[i][0]-halfcol*collookup[i][1])*2.0+0.5+1.0-collookup[i][1];

		}
		else
		{
			collookup[i][2]=(collookup[i][0]-halfcol*collookup[i][1])*2.0+0.5+collookup[i][1];
		}
		collookup[i][0]+=0.5;

	}


	DELTEXTURES(1,&m_rowLookupTexID);
	DELTEXTURES(1,&m_colLookupTexID);
	
	glGenTextures(1, &m_rowLookupTexID);
	glGenTextures(1, &m_colLookupTexID);
	
	//Create the Horizontal Lookup Texture
	glBindTexture(GL_TEXTURE_RECTANGLE_NV,m_rowLookupTexID);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE); 
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexImage2D( GL_TEXTURE_RECTANGLE_NV, 0, GL_FLOAT_RGB32_NV, m_imageWidth+8, maxrowlevels, 0, GL_RGB, GL_FLOAT, rowlookup);

	//Create the Vertical Lookup Texture
	glBindTexture(GL_TEXTURE_RECTANGLE_NV,m_colLookupTexID);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE); 
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexImage2D( GL_TEXTURE_RECTANGLE_NV, 0, GL_FLOAT_RGB32_NV, m_imageHeight+8, maxcollevels, 0, GL_RGB, GL_FLOAT, collookup);

	delete []rowlookup;
	delete []collookup;

	return true;
	
}

//Render input image data to texture
bool gpuDWT::createImgTex(float *imgbuffer)
{
	//uninitialized image dimensions
	if(m_imageWidth<=0 || m_imageHeight<=0)
		return false;

	glGenTextures(1, &m_imageTexID);
	
	glBindTexture(GL_TEXTURE_RECTANGLE_NV, m_imageTexID);
		
	glTexParameteri(GL_TEXTURE_RECTANGLE_NV,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_RECTANGLE_NV,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE); 
	
	glTexImage2D(GL_TEXTURE_RECTANGLE_NV, 0, GL_FLOAT_RGB32_NV, m_imageWidth, m_imageHeight,
				0, GL_RGB,  GL_FLOAT, imgbuffer);


	// fill the pbuffer with the image texture
	
		m_rt[1]->BeginCapture();
	 
			glMatrixMode(GL_MODELVIEW);
			glPushMatrix();
		

		    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glColor3f(0,0,0);
      
 			cgGLBindProgram(m_fillProg);
		    cgGLEnableProfile(CG_PROFILE_FP30);

			cgGLSetTextureParameter(m_fill_Tex_Param, m_imageTexID);
			cgGLEnableTextureParameter(m_fill_Tex_Param);
		


		    glBegin(GL_QUADS);
		    glTexCoord2f(0, 0); glVertex2f(0, 0);
			glTexCoord2f(m_imageWidth, 0); glVertex2f(m_imageWidth, 0);
		    glTexCoord2f(m_imageWidth, m_imageHeight); glVertex2f(m_imageWidth, m_imageHeight);
		    glTexCoord2f(0, m_imageHeight); glVertex2f(0, m_imageHeight);
			glEnd();
      
	  	    cgGLDisableTextureParameter(m_fill_Tex_Param);
			cgGLDisableProfile(CG_PROFILE_FP30);
	  
   
			glPopMatrix();

		m_rt[1]->EndCapture();
			

	DELTEXTURES(1,&m_imageTexID);

	return true;

}

bool gpuDWT::setImageDimensions(int width, int height)
{
	if(width<=0 || height<=0)
		return false;

	this->m_imageWidth=width;
	this->m_imageHeight=height;

	return true;

}

bool gpuDWT::initglew()
{
	 int err = glewInit();
     if (GLEW_OK != err)
       return false;

	 if (!GLEW_NV_texture_rectangle)
		 return false;
	 
	 return true;

}


bool gpuDWT::initRenderTexture(int BPP)
{
  bool succ;

  //only support 16-bit or 32-bit
  if(BPP!=16 && BPP!=32)
	  return false;
  
  m_rt[0] = new RenderTexture(this->m_imageWidth,this->m_imageHeight);
  m_rt[1] = new RenderTexture(this->m_imageWidth,this->m_imageHeight);

  succ=m_rt[0]->Initialize(true, false, false, false, false, BPP, BPP, BPP, 0, RenderTexture::RT_RENDER_TO_TEXTURE);  
  succ=succ && (m_rt[1]->Initialize(true, false, false, false, false, BPP, BPP, BPP, 0, RenderTexture::RT_RENDER_TO_TEXTURE));  

  if(!succ)
  {
	  exit(-1);
  }
  

  // setup the rendering context for the RenderTexture
  for (int i=0; i<2; i++)
  {
	  m_rt[i]->BeginCapture();
	  glMatrixMode(GL_MODELVIEW);
	  glLoadIdentity();
	  glMatrixMode(GL_PROJECTION);
	  glLoadIdentity();
	  gluOrtho2D(0.0f,m_imageWidth,0.0f,m_imageHeight);
	  glDisable(GL_LIGHTING);
	  glDisable(GL_COLOR_MATERIAL);
	  glDisable(GL_DEPTH_TEST);
	  glDisable(GL_BLEND);
	  glClearColor(0.2, 0.2, 0.2, 1);
	  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	  glPixelStorei(GL_PACK_ALIGNMENT, 1);
	  m_rt[i]->EndCapture();
  }

  return true;
  

}

bool gpuDWT::Init_Cg(dwtmode mode)
{
    cgSetErrorCallback(cgErrorCallback);

    // Create cgContext.
    if(!cgIsContext(cgContext))
	{
		cgContext = cgCreateContext();
	}

	m_fillProg = cgCreateProgram(cgContext, 
			                                  CG_SOURCE,
				                              fillsource, 
					                           GLEW_ARB_fragment_program ? CG_PROFILE_ARBFP1 : CG_PROFILE_FP30, 
						                       NULL,
							                   NULL);

	    if(m_fillProg != NULL)
		{
		  cgGLLoadProgram(m_fillProg);
	
	      m_fill_Tex_Param = cgGetNamedParameter(m_fillProg, "texture");
	      assert(m_fill_Tex_Param != NULL);

	    }

	
	/*******************************************************
	****            forward dwt programs               ****
	********************************************************/
	

	if(mode==forward || mode==both)
	{
		m_dwtrowProg = cgCreateProgram(cgContext, 
			                                  CG_SOURCE,
				                              dwtsource, 
					                           GLEW_ARB_fragment_program ? CG_PROFILE_ARBFP1 : CG_PROFILE_FP30, 
						                       NULL,
							                   NULL);

	    if(m_dwtrowProg != NULL)
		{
		  cgGLLoadProgram(m_dwtrowProg);
	
	      m_dwtrow_imageTex_Param = cgGetNamedParameter(m_dwtrowProg, "dwttexture");
	      assert(m_dwtrow_imageTex_Param != NULL);
		  m_dwtrow_filterTex_Param = cgGetNamedParameter(m_dwtrowProg, "filtertexture");
	      assert(m_dwtrow_filterTex_Param != NULL);

	      m_dwtrow_lookup_Param = cgGetNamedParameter(m_dwtrowProg, "lookuptexture");
	      assert(m_dwtrow_lookup_Param != NULL);
		  m_dwtrow_level_Param = cgGetNamedParameter(m_dwtrowProg, "level");
	      assert(m_dwtrow_level_Param != NULL);

	    }


		m_dwtcolProg = cgCreateProgram(cgContext, 
                                          CG_SOURCE,
                                          dwtcolsource, 
                                           GLEW_ARB_fragment_program ? CG_PROFILE_ARBFP1 : CG_PROFILE_FP30, 
                                           NULL,
                                           NULL);

	    if(m_dwtcolProg != NULL)
		{
		  cgGLLoadProgram(m_dwtcolProg);
	
	      m_dwtcol_imageTex_Param = cgGetNamedParameter(m_dwtcolProg, "dwttexture");
	      assert(m_dwtcol_imageTex_Param != NULL);
		  m_dwtcol_filterTex_Param = cgGetNamedParameter(m_dwtcolProg, "filtertexture");
	      assert(m_dwtcol_filterTex_Param != NULL);

	      m_dwtcol_lookup_Param = cgGetNamedParameter(m_dwtcolProg, "lookuptexture");
	      assert(m_dwtcol_lookup_Param != NULL);
		  m_dwtcol_level_Param = cgGetNamedParameter(m_dwtcolProg, "level");
	      assert(m_dwtcol_level_Param != NULL);

	    }


	}
	
	
	/*******************************************************
	****            inverse dwt programs                ****
	********************************************************/
	
	if(mode==inverse || mode==both)
	{

		m_invdwtrowProg = cgCreateProgram(cgContext, 
		                                  CG_SOURCE,
			                              invdwtsource, 
				                           GLEW_ARB_fragment_program ? CG_PROFILE_ARBFP1 : CG_PROFILE_FP30, 
					                       NULL,
						                   NULL);

	    if(m_invdwtrowProg != NULL)
		{
		  cgGLLoadProgram(m_invdwtrowProg);
	
	      m_invdwtrow_imageTex_Param = cgGetNamedParameter(m_invdwtrowProg, "dwttexture");
	      assert(m_invdwtrow_imageTex_Param != NULL);
		  m_invdwtrow_filterTex_Param = cgGetNamedParameter(m_invdwtrowProg, "filtertexture");
	      assert(m_invdwtrow_filterTex_Param != NULL);


		  m_invdwtrow_lookup_Param[0] = cgGetNamedParameter(m_invdwtrowProg, "lookuptexture[0]");
	  	  assert(m_invdwtrow_lookup_Param[0] != NULL);

		  m_invdwtrow_lookup_Param[1] = cgGetNamedParameter(m_invdwtrowProg, "lookuptexture[1]");
	  	  assert(m_invdwtrow_lookup_Param[1] != NULL);

 		  m_invdwtrow_lookup_Param[2] = cgGetNamedParameter(m_invdwtrowProg, "lookuptexture[2]");
	  	  assert(m_invdwtrow_lookup_Param[2] != NULL);

		  m_invdwtrow_level_Param = cgGetNamedParameter(m_invdwtrowProg, "level");
	      assert(m_invdwtrow_level_Param != NULL);



	    }


		m_invdwtcolProg = cgCreateProgram(cgContext, 
                                          CG_SOURCE,
                                          invdwtcolsource, 
                                           GLEW_ARB_fragment_program ? CG_PROFILE_ARBFP1 : CG_PROFILE_FP30, 
                                           NULL,
                                           NULL);

	    if(m_invdwtcolProg != NULL)
		{
		  cgGLLoadProgram(m_invdwtcolProg);
	
	      m_invdwtcol_imageTex_Param = cgGetNamedParameter(m_invdwtcolProg, "dwttexture");
	      assert(m_invdwtcol_imageTex_Param != NULL);
		  m_invdwtcol_filterTex_Param = cgGetNamedParameter(m_invdwtcolProg, "filtertexture");
	      assert(m_invdwtcol_filterTex_Param != NULL);

	      m_invdwtcol_lookup_Param[0] = cgGetNamedParameter(m_invdwtcolProg, "lookuptexture[0]");
	      assert(m_invdwtcol_lookup_Param[0] != NULL);

  	      m_invdwtcol_lookup_Param[1] = cgGetNamedParameter(m_invdwtcolProg, "lookuptexture[1]");
	      assert(m_invdwtcol_lookup_Param[1] != NULL);

  	      m_invdwtcol_lookup_Param[2] = cgGetNamedParameter(m_invdwtcolProg, "lookuptexture[2]");
	      assert(m_invdwtcol_lookup_Param[2] != NULL);


		  m_invdwtcol_level_Param = cgGetNamedParameter(m_invdwtcolProg, "level");
	      assert(m_invdwtcol_level_Param != NULL);


		  m_invdwtcol_texwidth_Param = cgGetNamedParameter(m_invdwtcolProg, "texwidth");
	      assert(m_invdwtcol_texwidth_Param != NULL);

  		  m_invdwtcol_texheight_Param = cgGetNamedParameter(m_invdwtcolProg, "texheight");
	      assert(m_invdwtcol_texheight_Param != NULL);


		}

	}


	return true;
}



// Retrieve the transformed buffer
void gpuDWT::getbuffer(float *myfloatbuffer)
{

		//select the last operated buffer and readback the pixel values		
		m_rt[1-m_iReadBuffer]->BeginCapture();
		glReadPixels(0,0,m_imageWidth,m_imageHeight,GL_RGB,GL_FLOAT,myfloatbuffer);
		
		m_rt[1-m_iReadBuffer]->EndCapture(); 

}


// Create the high/low pass filter texture
bool gpuDWT::createFilterTex(dwtmode mode)
{

	DELTEXTURES(1,&m_decomFilterTexID);
	DELTEXTURES(1,&m_reconFilterTexID);

	glGenTextures(1,&m_decomFilterTexID);
	glGenTextures(1,&m_reconFilterTexID);
	
	//create the filter textures
	if(mode==forward || mode== both)
	{
	
		glBindTexture( GL_TEXTURE_RECTANGLE_NV, m_decomFilterTexID);
		glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE); 
		glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexImage2D( GL_TEXTURE_RECTANGLE_NV, 0, GL_FLOAT_R32_NV, 18, 1, 0, GL_RED, GL_FLOAT, decomp_filter);
	}
	
	if(mode==inverse || mode == both)
	{
		glBindTexture( GL_TEXTURE_RECTANGLE_NV, m_reconFilterTexID);
		glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE); 
		glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexImage2D( GL_TEXTURE_RECTANGLE_NV, 0, GL_FLOAT_R32_NV, 18, 1, 0, GL_RED, GL_FLOAT, recon_filter);
	}

	return true;

}

//  gpuDWT::forward_dwt
bool gpuDWT::forwarddwt(int level,int startx, int starty, int endx,int endy)
{

	int quadWidth,quadHeight,tmp;

	calLength(startx,endx,level,&quadWidth,&tmp);
	calLength(starty,endy,level,&quadHeight,&tmp);

	//Vertical Decomposition
	if(quadHeight>1)
	{
	
	m_rt[m_iReadBuffer]->BeginCapture();
	
	
	cgGLBindProgram(m_dwtcolProg);
	cgGLEnableProfile(GLEW_ARB_fragment_program ? CG_PROFILE_ARBFP1 : CG_PROFILE_FP30);
	
	
	cgGLSetParameter1f(m_dwtcol_level_Param,level);
		
	cgGLSetTextureParameter(m_dwtcol_imageTex_Param, m_rt[1-m_iReadBuffer]->GetTextureID());
  	cgGLEnableTextureParameter(m_dwtcol_imageTex_Param);

	cgGLSetTextureParameter(m_dwtcol_filterTex_Param, m_decomFilterTexID);
	cgGLEnableTextureParameter(m_dwtcol_filterTex_Param);

	cgGLSetTextureParameter(m_dwtcol_lookup_Param, m_colLookupTexID);
	cgGLEnableTextureParameter(m_dwtcol_lookup_Param);

	
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2f(0.0f, 0.0f);

	glTexCoord2f((float)quadWidth, 0.0f);
	glVertex2f((float)quadWidth, 0.0f);

	glTexCoord2f((float)quadWidth, (float)quadHeight);
	glVertex2f((float)quadWidth, (float)quadHeight);

	glTexCoord2f(0.0f, (float)quadHeight);
	glVertex2f(0.0f, (float)quadHeight);		
	glEnd();


	cgGLDisableTextureParameter(m_dwtcol_imageTex_Param);
	cgGLDisableTextureParameter(m_dwtcol_filterTex_Param);
	cgGLDisableTextureParameter(m_dwtcol_lookup_Param);
	cgGLDisableProfile(GLEW_ARB_fragment_program ? CG_PROFILE_ARBFP1 : CG_PROFILE_FP30);

	m_rt[m_iReadBuffer]->EndCapture();

	m_iReadBuffer=(m_iReadBuffer+1)%2;

	}else
	{
		//pass through if not enough length

			m_rt[m_iReadBuffer]->BeginCapture();
      
 			cgGLBindProgram(m_fillProg);
		    cgGLEnableProfile(CG_PROFILE_FP30);

			cgGLSetTextureParameter(m_fill_Tex_Param, m_rt[1-m_iReadBuffer]->GetTextureID());
			cgGLEnableTextureParameter(m_fill_Tex_Param);
		


			glBegin(GL_QUADS);
			glTexCoord2f(0.0f, 0.0f);
			glVertex2f(0.0f, 0.0f);

			glTexCoord2f((float)quadWidth, 0.0f);
			glVertex2f((float)quadWidth, 0.0f);

			glTexCoord2f((float)quadWidth, (float)quadHeight);
			glVertex2f((float)quadWidth, (float)quadHeight);

			glTexCoord2f(0.0f, (float)quadHeight);
			glVertex2f(0.0f, (float)quadHeight);		
			glEnd();      
			
	  	    cgGLDisableTextureParameter(m_fill_Tex_Param);
			cgGLDisableProfile(CG_PROFILE_FP30);
	  
 
			m_rt[m_iReadBuffer]->EndCapture();
			m_iReadBuffer=(m_iReadBuffer+1)%2;
	}


	//Horizontal  Decomposition
	if(quadWidth>1)
	{

	m_rt[m_iReadBuffer]->BeginCapture();
	
	
	cgGLBindProgram(m_dwtrowProg);
	cgGLEnableProfile(GLEW_ARB_fragment_program ? CG_PROFILE_ARBFP1 : CG_PROFILE_FP30);
	
	
	cgGLSetParameter1f(m_dwtrow_level_Param,(float)level);
		
	cgGLSetTextureParameter(m_dwtrow_imageTex_Param, m_rt[1-m_iReadBuffer]->GetTextureID());
  	cgGLEnableTextureParameter(m_dwtrow_imageTex_Param);

	cgGLSetTextureParameter(m_dwtrow_filterTex_Param, m_decomFilterTexID);
	cgGLEnableTextureParameter(m_dwtrow_filterTex_Param);

	cgGLSetTextureParameter(m_dwtrow_lookup_Param, m_rowLookupTexID);
	cgGLEnableTextureParameter(m_dwtrow_lookup_Param);

	
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2f(0.0f, 0.0f);

	glTexCoord2f((float)quadWidth, 0.0f);
	glVertex2f((float)quadWidth, 0.0f);

	glTexCoord2f((float)quadWidth, (float)quadHeight);
	glVertex2f((float)quadWidth, (float)quadHeight);

	glTexCoord2f(0.0f, (float) quadHeight);
	glVertex2f(0.0f, (float) quadHeight);
		
	glEnd();


	cgGLDisableTextureParameter(m_dwtrow_imageTex_Param);
	cgGLDisableTextureParameter(m_dwtrow_filterTex_Param);
	cgGLDisableTextureParameter(m_dwtrow_lookup_Param);
	cgGLDisableProfile(GLEW_ARB_fragment_program ? CG_PROFILE_ARBFP1 : CG_PROFILE_FP30);

	m_rt[m_iReadBuffer]->EndCapture();

	m_iReadBuffer=(m_iReadBuffer+1)%2;
	
	}
	else
	{
			//pass through if not enough length

			m_rt[m_iReadBuffer]->BeginCapture();
			
    
 			cgGLBindProgram(m_fillProg);
		    cgGLEnableProfile(CG_PROFILE_FP30);

			cgGLSetTextureParameter(m_fill_Tex_Param, m_rt[1-m_iReadBuffer]->GetTextureID());
			cgGLEnableTextureParameter(m_fill_Tex_Param);
		


			glBegin(GL_QUADS);
			glTexCoord2f(0.0f, 0.0f);
			glVertex2f(0.0f, 0.0f);

			glTexCoord2f((float)quadWidth, 0.0f);
			glVertex2f((float)quadWidth, 0.0f);

			glTexCoord2f((float)quadWidth, (float)quadHeight);
			glVertex2f((float)quadWidth, (float)quadHeight);

			glTexCoord2f(0.0f, (float)quadHeight);
			glVertex2f(0.0f, (float)quadHeight);		
			glEnd();      
			
	  	    cgGLDisableTextureParameter(m_fill_Tex_Param);
			cgGLDisableProfile(CG_PROFILE_FP30);
	  
   
			m_rt[m_iReadBuffer]->EndCapture();

			m_iReadBuffer=(m_iReadBuffer+1)%2;

	}

	return true;

}

//  one-stop solution for dwt  
bool gpuDWT::dwtallinone(extmode mode, int imagewidth, int imageheight, float*  imagebuffer, int beg_level, int end_level, bool isforward, bool initGL, bool initGLEW, int startx, int starty,int endx,int endy)
{
	int i;

	if(initGLEW)
	{
		CHECKSUCC(initglew());
	}
	
	CHECKSUCC(setImageDimensions(imagewidth,imageheight));

	CHECKSUCC(Init_Cg(forward));
	CHECKSUCC(initRenderTexture(32));
	CHECKSUCC(createImgTex(imagebuffer));
	CHECKSUCC(createFilterTex(forward));


	// dwt or idwt starts and ends at particular levels you specify
	if(forward)
	{
		CHECKSUCC(createLookupTex(mode,startx,starty,endx,endy));
		for(i=beg_level ; i<=end_level; i++)
		{
			CHECKSUCC(forwarddwt(i,startx,starty,endx,endy));
		}
	}
	else
	{
		CHECKSUCC(createInvLookupTex(mode,startx,starty,endx,endy));
		for(i=beg_level ; i>=end_level; i--)
		{
			CHECKSUCC(inversedwt(i,startx,starty,endx,endy));
		}
	}
		
	getbuffer(imagebuffer);

	return true;

}

bool gpuDWT::dwtallinone(extmode mode, int imagewidth, int imageheight, float* imagebuffer, int level, bool isforward, bool initGL, bool initGLEW, int startx, int starty, int endx,int endy)
{
	int i;

	if(initGLEW)
	{
		CHECKSUCC(initglew());
	}
	
	CHECKSUCC(setImageDimensions(imagewidth,imageheight));


	CHECKSUCC(Init_Cg(isforward?forward:inverse));

	CHECKSUCC(initRenderTexture(32));
	CHECKSUCC(createImgTex(imagebuffer));
	CHECKSUCC(createFilterTex(isforward?forward:inverse));

	

	
	if(isforward)
	{
		CHECKSUCC(createLookupTex(mode,startx,starty,endx,endy));
		for(i=0 ; i<level; i++)
		{
			CHECKSUCC(forwarddwt(i,startx,starty, endx,endy));
		}
	}
	else
	{
		CHECKSUCC(createInvLookupTex(mode,startx,starty,endx,endy));
		for(i=level-1; i>=0; i--)
		{
			CHECKSUCC(inversedwt(i,startx,starty,endx,endy));
		}
	}

	getbuffer(imagebuffer);
		
	return true;

}


// Delete the textures
bool gpuDWT::finalcleanup()
{
	//delete textures and rendertextures
	
	DELTEXTURES(1,&m_colLookupTexID);
	DELTEXTURES(1,&m_rowLookupTexID);
	DELTEXTURES(1,&m_imageTexID);
	DELTEXTURES(1,&m_decomFilterTexID);
	DELTEXTURES(1,&m_reconFilterTexID);
	DELTEXTURES(3,m_LookupTexID);

	if(m_rt[0]!=NULL)
		delete m_rt[0];
	
	if(m_rt[1]!=NULL)
		delete m_rt[1];

	
	//delete resident cg programs
	CGprogram program=cgGetFirstProgram(cgContext);
	CGprogram tmpprog=program;
	
	while (tmpprog!=0)
	{
		tmpprog=cgGetNextProgram(program);
		cgDestroyProgram(program);
		program=tmpprog;
	}


	//destroy cg context
	if(cgIsContext(cgContext))
	{
		cgDestroyContext(cgContext);
	}

	return true;	

}


// Create the Indirect Addressing Texture for inverse dwt
bool gpuDWT::createInvLookupTex(extmode mode,int startx, int starty, int endx, int endy)
{
	float * tex1=NULL;
	float * tex2=NULL;
	float * tex3=NULL;

	int texwidth, texheight;
	
	createIDATexture(mode,&tex1,&tex2,&tex3,m_imageWidth,m_imageHeight,&texwidth,&texheight,startx,starty,endx,endy);
	

	m_ida_texwidth=texwidth;
	m_ida_texheight=texheight;

	DELTEXTURES(3,m_LookupTexID);
	
	glGenTextures(3, m_LookupTexID);


	//Create the Lookup Texture for IDWT
	glBindTexture(GL_TEXTURE_RECTANGLE_NV,m_LookupTexID[0]);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE); 
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexImage2D( GL_TEXTURE_RECTANGLE_NV, 0, GL_FLOAT_RGBA32_NV, texwidth, texheight, 0, GL_RGBA, GL_FLOAT, tex1);


	glBindTexture(GL_TEXTURE_RECTANGLE_NV,m_LookupTexID[1]);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE); 
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexImage2D( GL_TEXTURE_RECTANGLE_NV, 0, GL_FLOAT_RGBA32_NV, texwidth, texheight, 0, GL_RGBA, GL_FLOAT, tex2);


	glBindTexture(GL_TEXTURE_RECTANGLE_NV,m_LookupTexID[2]);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE); 
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri( GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexImage2D( GL_TEXTURE_RECTANGLE_NV, 0, GL_FLOAT_RGB32_NV, texwidth, texheight, 0, GL_RGB, GL_FLOAT, tex3);


	delete[] tex1;
	delete[] tex2;
	delete[] tex3;

	return true;

}

// Do inverse dwt
bool gpuDWT::inversedwt(int level,int startx, int starty, int endx,int endy)
{

	int quadWidth,quadHeight,tmp;

	calLength(startx,endx,level,&quadWidth,&tmp);
	calLength(starty,endy,level,&quadHeight,&tmp);


	// Horizontal Reconstruction
	if(quadWidth>1)
	{
	
	
	m_rt[m_iReadBuffer]->BeginCapture();

	
	cgGLBindProgram(m_invdwtrowProg);
	cgGLEnableProfile(GLEW_ARB_fragment_program ? CG_PROFILE_ARBFP1 : CG_PROFILE_FP30);
	
	
	cgGLSetParameter1f(m_invdwtrow_level_Param,(float)level);

		
	cgGLSetTextureParameter(m_invdwtrow_imageTex_Param, m_rt[1-m_iReadBuffer]->GetTextureID());
  	cgGLEnableTextureParameter(m_invdwtrow_imageTex_Param);

	cgGLSetTextureParameter(m_invdwtrow_filterTex_Param, m_reconFilterTexID);
	cgGLEnableTextureParameter(m_invdwtrow_filterTex_Param);

	cgGLSetTextureParameter(m_invdwtrow_lookup_Param[0], m_LookupTexID[0]);
	cgGLEnableTextureParameter(m_invdwtrow_lookup_Param[0]);

	cgGLSetTextureParameter(m_invdwtrow_lookup_Param[1], m_LookupTexID[1]);
	cgGLEnableTextureParameter(m_invdwtrow_lookup_Param[1]);

	cgGLSetTextureParameter(m_invdwtrow_lookup_Param[2], m_LookupTexID[2]);
	cgGLEnableTextureParameter(m_invdwtrow_lookup_Param[2]);


	
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2f(0.0f, 0.0f);

	glTexCoord2f((float)quadWidth, 0.0f);
	glVertex2f((float)quadWidth, 0.0f);

	glTexCoord2f((float)quadWidth, (float)quadHeight);
	glVertex2f((float)quadWidth, (float)quadHeight);

	glTexCoord2f(0.0f, (float)quadHeight);
	glVertex2f(0.0f, (float)quadHeight);
		
	glEnd();


	cgGLDisableTextureParameter(m_invdwtrow_imageTex_Param);
	cgGLDisableTextureParameter(m_invdwtrow_filterTex_Param);
	cgGLDisableTextureParameter(m_invdwtrow_lookup_Param[0]);
	cgGLDisableTextureParameter(m_invdwtrow_lookup_Param[1]);
	cgGLDisableTextureParameter(m_invdwtrow_lookup_Param[2]);
	cgGLDisableProfile(GLEW_ARB_fragment_program ? CG_PROFILE_ARBFP1 : CG_PROFILE_FP30);

	m_rt[m_iReadBuffer]->EndCapture();

	m_iReadBuffer=(m_iReadBuffer+1)%2;

	}
	else
	{
			//pass through if not enough length

			m_rt[m_iReadBuffer]->BeginCapture();
      
 			cgGLBindProgram(m_fillProg);
		    cgGLEnableProfile(CG_PROFILE_FP30);

			cgGLSetTextureParameter(m_fill_Tex_Param, m_rt[1-m_iReadBuffer]->GetTextureID());
			cgGLEnableTextureParameter(m_fill_Tex_Param);
		


			glBegin(GL_QUADS);
			glTexCoord2f(0.0f, 0.0f);
			glVertex2f(0.0f, 0.0f);

			glTexCoord2f((float)quadWidth, 0.0f);
			glVertex2f((float)quadWidth, 0.0f);

			glTexCoord2f((float)quadWidth, (float)quadHeight);
			glVertex2f((float)quadWidth, (float)quadHeight);

			glTexCoord2f(0.0f, (float)quadHeight);
			glVertex2f(0.0f, (float)quadHeight);		
			glEnd();      
			
	  	    cgGLDisableTextureParameter(m_fill_Tex_Param);
			cgGLDisableProfile(CG_PROFILE_FP30);
	  
   

			m_rt[m_iReadBuffer]->EndCapture();

			m_iReadBuffer=(m_iReadBuffer+1)%2;
	
	}

		
	
	//Vertical reconstruction
	if(quadHeight>1)
	{
	

	m_rt[m_iReadBuffer]->BeginCapture();
	
	
	cgGLBindProgram(m_invdwtcolProg);
	cgGLEnableProfile(GLEW_ARB_fragment_program ? CG_PROFILE_ARBFP1 : CG_PROFILE_FP30);
	
	
	cgGLSetParameter1f(m_invdwtcol_level_Param,level);
	
	cgGLSetParameter1f(m_invdwtcol_texwidth_Param, m_ida_texwidth);
	cgGLSetParameter1f(m_invdwtcol_texheight_Param, m_ida_texheight);

		
	cgGLSetTextureParameter(m_invdwtcol_imageTex_Param, m_rt[1-m_iReadBuffer]->GetTextureID());
  	cgGLEnableTextureParameter(m_invdwtcol_imageTex_Param);

	cgGLSetTextureParameter(m_invdwtcol_filterTex_Param, m_reconFilterTexID);
	cgGLEnableTextureParameter(m_invdwtcol_filterTex_Param);

	cgGLSetTextureParameter(m_invdwtcol_lookup_Param[0], m_LookupTexID[0]);
	cgGLEnableTextureParameter(m_invdwtcol_lookup_Param[0]);

	cgGLSetTextureParameter(m_invdwtcol_lookup_Param[1], m_LookupTexID[1]);
	cgGLEnableTextureParameter(m_invdwtcol_lookup_Param[1]);

	cgGLSetTextureParameter(m_invdwtcol_lookup_Param[2], m_LookupTexID[2]);
	cgGLEnableTextureParameter(m_invdwtcol_lookup_Param[2]);

	
	
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2f(0.0f, 0.0f);

	glTexCoord2f(quadWidth, 0.0f);
	glVertex2f(quadWidth, 0.0f);

	glTexCoord2f(quadWidth, quadHeight);
	glVertex2f(quadWidth, quadHeight);

	glTexCoord2f(0.0f, quadHeight);
	glVertex2f(0.0f, quadHeight);		
	glEnd();


	cgGLDisableTextureParameter(m_invdwtcol_imageTex_Param);
	cgGLDisableTextureParameter(m_invdwtcol_filterTex_Param);
	cgGLDisableTextureParameter(m_invdwtcol_lookup_Param[0]);
	cgGLDisableTextureParameter(m_invdwtcol_lookup_Param[1]);
	cgGLDisableTextureParameter(m_invdwtcol_lookup_Param[2]);
	cgGLDisableProfile(GLEW_ARB_fragment_program ? CG_PROFILE_ARBFP1 : CG_PROFILE_FP30);

	m_rt[m_iReadBuffer]->EndCapture();

	m_iReadBuffer=(m_iReadBuffer+1)%2;
	
	}
	else
	{

			//pass through if not enough length

			m_rt[m_iReadBuffer]->BeginCapture();
      
 			cgGLBindProgram(m_fillProg);
		    cgGLEnableProfile(CG_PROFILE_FP30);

			cgGLSetTextureParameter(m_fill_Tex_Param, m_rt[1-m_iReadBuffer]->GetTextureID());
			cgGLEnableTextureParameter(m_fill_Tex_Param);
		

			glBegin(GL_QUADS);
			glTexCoord2f(0.0f, 0.0f);
			glVertex2f(0.0f, 0.0f);

			glTexCoord2f((float)quadWidth, 0.0f);
			glVertex2f((float)quadWidth, 0.0f);

			glTexCoord2f((float)quadWidth, (float)quadHeight);
			glVertex2f((float)quadWidth, (float)quadHeight);

			glTexCoord2f(0.0f, (float)quadHeight);
			glVertex2f(0.0f, (float)quadHeight);		
			glEnd();      
			
	  	    cgGLDisableTextureParameter(m_fill_Tex_Param);
			cgGLDisableProfile(CG_PROFILE_FP30);
	  
   
			m_rt[m_iReadBuffer]->EndCapture();

			m_iReadBuffer=(m_iReadBuffer+1)%2;
		
	}
	
	return true;

}

bool gpuDWT::initialize(extmode mode, int imagewidth, int imageheight, float*  imagebuffer,  bool initGL, bool initGLEW,int startx,int starty, int endx,int endy)
{

	if(initGLEW) CHECKSUCC(initglew());
	CHECKSUCC(setImageDimensions(imagewidth,imageheight));
	CHECKSUCC(Init_Cg(both));
	CHECKSUCC(initRenderTexture(32));
	CHECKSUCC(createImgTex(imagebuffer));
	CHECKSUCC(createFilterTex(both));

	CHECKSUCC(createLookupTex(mode,startx,starty,endx,endy));
	CHECKSUCC(createInvLookupTex(mode,startx,starty,endx,endy));
	return true;
}


/********** The overloaded functions for general usage **********/
// one-stop dwt transform, suitable for tasks that only do forward or inverse dwt once
bool gpuDWT::dwtallinone(extmode mode, int imagewidth, int imageheight, float*  imagebuffer, int level, bool isforward, bool initGL, bool initGLEW)
{
	return dwtallinone(mode,imagewidth,imageheight,imagebuffer,level,isforward,initGL,initGLEW,0,0,imagewidth,imageheight);
}
	

bool gpuDWT::dwtallinone(extmode mode, int imagewidth, int imageheight, float*  imagebuffer, int beg_level, int end_level, bool isforward, bool initGL, bool initGLEW)
{
	return dwtallinone( mode,  imagewidth,  imageheight, imagebuffer, beg_level,  end_level, isforward, initGL, initGLEW,0,0,imagewidth,imageheight );

}

// initialize and feed in the buffer, both forward and inverse dwt functions are available for call after return
bool gpuDWT::initialize(extmode mode, int imagewidth, int imageheight, float*  imagebuffer, bool initGL, bool initGLEW)
{
	return initialize( mode,  imagewidth,  imageheight, imagebuffer, initGL, initGLEW, 0,0,imagewidth,imageheight );

}


GLubyte *LoadBmp(const char *filename, BITMAPINFO **bmpinfo)
{
	FILE* file;
	
	if((file=fopen(filename,"rb"))==NULL) exit(1);
	BITMAPFILEHEADER bmpheader;
	fread(&bmpheader,sizeof(BITMAPFILEHEADER),1,file);
	
	if(bmpheader.bfType!='MB') exit(1);
	*bmpinfo=(BITMAPINFO*)malloc(sizeof(BITMAPINFO));
	assert(*bmpinfo!=NULL);

	int readsize=fread((*bmpinfo), 1, bmpheader.bfOffBits-sizeof(BITMAPFILEHEADER) , file);
	int bufsize=((*bmpinfo)->bmiHeader.biWidth)*((((*bmpinfo)->bmiHeader.biBitCount)+7)/8)*((*bmpinfo)->bmiHeader.biHeight);	
	GLubyte * imgbits=(GLubyte*) malloc(bufsize);

	assert(imgbits!=NULL);
	fread(imgbits,1,bufsize,file); 

	GLubyte * pbits=imgbits;
	GLubyte temp;
	for (int j=0; j<(*bmpinfo)->bmiHeader.biHeight; j++)
	{
		for(int i=0; i<(*bmpinfo)->bmiHeader.biWidth; i++)
		{
			temp=pbits[0];
			pbits[0]=pbits[2];
			pbits[2]=temp;

			pbits+=3;
		}
	}

	fclose(file);
	return imgbits;
	
}


