#ifndef GPUDWT_H
#define GPUDWT_H

#include "RenderTexture.h"
#include <GL/glut.h>
#include <Cg/cg.h>
#include <Cg/cgGL.h>

#include <Filter.h>

using namespace Wave_lib;

extern  GLubyte *LoadBmp(const char *filename, BITMAPINFO **bmpinfo);

enum extmode {per,symper};//boundary extension modes
enum dwtmode {forward,inverse,both};//dwt modes	

	class gpuDWT  {
	public:

	gpuDWT(Filters<double> *filter, double fdegree);
	virtual ~gpuDWT();	

	/********** The overloaded functions for general usage **********/

	// one-stop dwt transform, suitable for tasks that only do forward or inverse dwt once
	// detail information about the parameters please see inside the cpp file
	bool dwtallinone(extmode mode, int imagewidth, int imageheight, float*  imagebuffer, int level, bool isforward, bool initGL, bool initGLEW);
	bool dwtallinone(extmode mode, int imagewidth, int imageheight, float*  imagebuffer, int beg_level, int end_level, bool isforward, bool initGL, bool initGLEW);

	// initialize and feed in the buffer, both forward and inverse dwt functions are available for call after return
	bool initialize(extmode mode, int imagewidth, int imageheight, float*  imagebuffer, bool initGL, bool initGLEW);



	// one-stop dwt transform, suitable for tasks that only do forward or inverse dwt once
	// detail information about the parameters please see inside the cpp file

	bool dwtallinone(extmode mode, int imagewidth, int imageheight, float*  imagebuffer, int level, bool isforward, bool initGL, bool initGLEW, int startx, int starty,int endx,int endy);
	bool dwtallinone(extmode mode, int imagewidth, int imageheight, float*  imagebuffer, int beg_level, int end_level, bool isforward, bool initGL, bool initGLEW, int startx, int starty,int endx,int endy);

	// initialize and feed in the buffer, both forward and inverse dwt functions are available for call after return
	bool initialize(extmode mode, int imagewidth, int imageheight, float*  imagebuffer, bool initGL, bool initGLEW, int startx, int starty, int endx,int endy);

	// the forward and inverse dwt function
	bool forwarddwt(int level,int startx, int starty, int endx,int endy);
	bool inversedwt(int level,int startx, int starty, int endx,int endy);

	
	
	// retrieve the transformed buffer back
	void getbuffer(float *myfloatbuffer);

	Filters<double> *_filter;
	double fdegree;

	DVector<double> lo_d;
	DVector<double> hi_d;

	DVector<double> lo_r;
	DVector<double> hi_r;

	void getFilterCoefs(double fd);

	
private:



	// Creat the  Ext,HL,BasePos Lookup Texture 
	bool createLookupTex(extmode mode,int startx, int starty, int endx, int endy);
	
	//Creat the  Ext,HL,BasePos Lookup Texture for inverse dwt
	bool createInvLookupTex(extmode mode,int startx, int starty, int endx, int endy);

	//create the high/low pass filter texture
	bool createFilterTex(dwtmode mode);

	//initialize Cg and Cg Shaders
	bool Init_Cg(dwtmode mode);

	//initialize the renderTexture objects
	bool initRenderTexture(int BPP);
	
	//set image dimension variables
	bool setImageDimensions(int width, int height);
	
	//render input image data to texture
	bool createImgTex( float *imgbuffer);	

	//initilize glew
	bool initglew();

	//delete textures
	bool finalcleanup();

	// boundary extension function
	int ext(int index, int datalen, extmode mode);

	//calculate total levels using the JPEG2000 start and end indices
	int calLevels(int startind, int endind);

	// calculate the downsampling length and sampling offset for lowpassed part
	void calLength(int startind, int endind, int level, int * llength, int * loffset);

	//create the indirect addressing table for the inverse DWT
	bool createIDATexture(extmode mode,float ** itex1, float ** itex2, float ** itex3, int width, int height, int * texwidth, int * texheight, int startx, int starty, int endx,int endy);
	


protected:

	//Cg Programs
	CGprogram m_invdwtcolProg;
	CGprogram m_invdwtrowProg;
	CGprogram m_dwtcolProg;
	CGprogram m_dwtrowProg;
	CGprogram m_fillProg;

	//Cg Parameters
	CGparameter m_dwtrow_lookup_Param, m_dwtrow_imageTex_Param,m_dwtrow_filterTex_Param,m_dwtrow_level_Param;
	CGparameter m_dwtcol_lookup_Param, m_dwtcol_imageTex_Param,m_dwtcol_filterTex_Param,m_dwtcol_level_Param;

	CGparameter m_invdwtrow_lookup_Param[3], m_invdwtrow_imageTex_Param,m_invdwtrow_filterTex_Param,m_invdwtrow_level_Param;
	CGparameter m_invdwtcol_lookup_Param[3], m_invdwtcol_imageTex_Param,m_invdwtcol_filterTex_Param,m_invdwtcol_level_Param,m_invdwtcol_texwidth_Param,m_invdwtcol_texheight_Param;


	CGparameter m_fill_Tex_Param;


	//Texture IDs
	GLuint m_colLookupTexID;
	GLuint m_rowLookupTexID;
	
	GLuint m_LookupTexID[3];
	GLuint m_imageTexID;

	
	GLuint m_reconFilterTexID;
	GLuint m_decomFilterTexID;


	//lookuptex dimension
	int m_ida_texwidth;
	int m_ida_texheight;

	//Image Dimensions
	int m_imageHeight;
	int m_imageWidth;

	
	// the current operation buffer
	int m_iReadBuffer;

	//RenderTexture Objects
	RenderTexture * m_rt[2];

	};

#endif 
